!***************************************************************************
!
!	SCIATRAN - Version 4.6 - Last modification: 10 Jun 2022
!
!	Copyright (C) - Institute of Remote Sensing,
!			University of Bremen, Germany
!
!***************************************************************************
SUBROUTINE AERONET_SETUP(START_WAV, END_WAV, *)
  ! Called by "rtm_init.f"
                              
!  USE AER_MIE,   ONLY: DENS_PRF
  USE AEROSOLS,  ONLY: AER_SHIFT, PATH_TO_SCM, OUT_A_SCM, LAY_A, WEL_A, NSCA_A, &
       GE_A, APH_VS, AER_WAVS, NAW => N_AER_WAVS, Greek_Aer, AER_COEFF, Scal_aer, &
       SCI => GT_SCAT_IDX, NG_aer, ASY => GT_ASYM_IDX, Rtau_aer, L_scal, ZS1, &
       EXI => GT_EXTC_IDX, Rwl_aer, ZS2, Plus_mi, NWLS, NWLE, &
       AER_SCALING, Nga_db, LA_N, LA_S, LA_E, L_TOP, IDX_AER, LA_Ndb, AOT_out_VAL, &
       AER_FACT, V_PAP, IDX_AER_db, SHARP_EDGES, ZA_SE
  USE CLOUDS,    ONLY: DO_CLOUD  
  USE CLOUDS_SS, ONLY: DO_SSRCA
  USE DELTA_M,   ONLY: FDM_AER, AER_DM_FLG, D_MF_AER, AER_TRUNC_IDX
  USE GRIDS,     ONLY: NLAYERS, ZHOLD => ZZZ, DZ
  USE GT_POLYN,  ONLY: NLEGEN
  USE INFO,      ONLY: AER_INFO
  USE I_DIMS,    ONLY: Nm_vs
  USE LOG_PARS,  ONLY: Work_Dir, LOG_LEVEL, LUNIT, EU, DIR_OUT, N_SSO, N_ESO
  USE RTM_PARS,  ONLY: DATA_ITER
  USE ST_VARS,   ONLY: RAD_TO_DEG, Vec_Sca, RTM_TYPE, GROUND_HEIGHT
  USE ST_INPUT,  ONLY: APH_TYPE, ADB => PATH_AER_DB
  USE TOOLS,     ONLY: READ_INPUT, CHK_STAT, SET_PTR, CUT_STR, SPLIT_STRING, &
                       Read_File
  USE TYPES,     ONLY: KND, DYN_CHAR
  
  REAL (KND), INTENT(IN) :: START_WAV, END_WAV
  ! Local parameters
  LOGICAL            ERROR
  CHARACTER (LEN = 13), PARAMETER :: R_NAME = 'AERONET_SETUP'
  !  (a) Input variables
  !      AER_DM_FLG        : delta-M approximation yes/no 
  !                          <---- From Module DELTA_M
  !      AER_TRUNC_IDX     : truncation index for delta-M approximation 
  !                          <---- From Module DELTA_M
  !      ZHOLD             : forward model altitude grid
  !      NLAYERS           : number of layers in altitude grid
  !      NLEGEN            : number of Legendre moments
  !      LUNIT             : LOG file unit number
  
  !  (b) Output variables
  !      AER_COEFF         : scattering / extinction coefficients and 
  !                          asymmetry parameters 
  !      AER_PHASMOMS      : Legendre moments
  !      NAW               : number of wavelengths of chosen aerosol 
  !                          parameterization 
  !      AER_WAVS          : wavelengths of aerosol data base (depends on
  ! parameterization chosen)
  INTEGER  NA_LEV
  REAL (KND)  DUM, Scoe, DD1, DD2, Col_Nd, Scal_PRF
  INTEGER, ALLOCATABLE :: IDX_W(:)
  REAL (KND), ALLOCATABLE :: USER_MOM(:,:,:,:), DENS_PRF(:), TAU(:), &
       CEXT(:,:), CSCA(:,:), ASYM_PAR(:,:), AER_LEV(:), AER_PRF(:), &
       E_OD(:), A_OD(:)
  CHARACTER (LEN=200), ALLOCATABLE :: F_name(:)
  CHARACTER (LEN=:), ALLOCATABLE :: STR_INP, DIR_name
  INTEGER, ALLOCATABLE :: LAS(:), LAE(:), IDA(:), N_MOMS(:)
  REAL (KND), POINTER :: AER_PHASMOMS(:,:,:)
  REAL (KND), ALLOCATABLE :: DATA_R(:,:)
  Real (Knd), Allocatable :: Arr(:,:), Ang(:), Phf(:,:)
  INTEGER  I, I1, J, K, L, DUNIT, K1, Itr, LAN, N_COLS, NM_MAX, MAX_MOM, &
       N_ang
  TYPE (DYN_CHAR), ALLOCATABLE :: READ_ARR(:)
  TYPE (DYN_CHAR), ALLOCATABLE :: STR_ARR(:), STR_ARR_1(:) 
  ! Variable for Delta-M
  ! ====================
  REAL (KND)  F_DM, AER_ABS_COEFF
  !=========================================================================
  CALL ST_TRACK (R_NAME, 1, 'I')
      
  CALL DEALLOCATE_ARRAYS ! All arrays allocated below must be 
                         ! conditionally deallocated here
  ALLOCATE( DENS_PRF(NLAYERS) )
  ERROR = .FALSE.

  IF( DO_CLOUD.AND.SHARP_EDGES ) GOTO 138

  STR_INP = 'Directory name of input AERONET parameters'
  CALL Read_Input (STR_INP, READ_ARR, *880)
  DIR_name = CUT_STR(READ_ARR(1)%S)

  WRITE(*,*) ' ---> DIR_name  ', DIR_name 
  
! ... Number of wavelength in AERONET data
  NAW = 4
  STR_INP = 'Filenames of AEROBET optical parameters'
  CALL Read_Input (STR_INP, READ_ARR, *880)
  CALL SPLIT_STRING(READ_ARR(1)%S, STR_ARR)
  ALLOCATE( F_name(3) )
  DO I = 1, 2
     F_name(I) = DIR_name//CUT_STR(STR_ARR(I)%S)
!     WRITE(*,'(A)')  TRIM( F_name(I) )
  ENDDO
  
  NWLS = 1 ; NWLE = NAW
  NM_MAX = 1000

  ALLOCATE( AER_WAVS(NAW), FDM_AER(NLAYERS,NAW), IDA(NLAYERS),            & !A
       AER_COEFF(3,NLAYERS,NAW), TAU(NAW), N_MOMS(NAW), ASYM_PAR(1,NAW),  & !A
       CEXT(1,NAW), CSCA(1,NAW), USER_MOM(NAW,NM_MAX,1,Nm_vs), E_OD(NAW), & !A
       A_OD(NAW) )                                                          ! A

  FDM_AER = 0.d0 ; AER_COEFF = 0.d0 ; USER_MOM   = 0.d0 

  N_COLS = 3
  Call Read_File (N_COLS, F_name(1), Arr, *880)
  AER_WAVS(:) = Arr(:,1)
  E_OD(:)   = Arr(:,2)  ! extinction optical thickness
  A_OD(:)   = Arr(:,3)  ! absorption optical thickness
! .................................................
  DO I = 1, NAW
     WRITE(*,'(2x,I2,2x,F6.1,3(2x,F9.6))') &
          I, AER_WAVS(I), E_OD(I), A_OD(I)
  ENDDO
! .................................................
  DEALLOCATE( Arr )  

  N_COLS = 5
  Call Read_File (N_COLS, F_name(2), Arr, *880)
  N_ang = SIZE( Arr(:,1) )
  ALLOCATE( Ang(N_ang), Phf(N_ang,NAW) )
  Ang(:) = Arr(:,1)
  FORALL( I=1:NAW ) Phf(:,I) = Arr(:,I+1)
! .................................................
  DO I = 1, N_ang
     WRITE(*,'(2x,F7.2,*(2x,F10.5))') &
          Arr(I,1), (Phf(I,K), K = 1,NAW)
  ENDDO
  ! .................................................

  STR_INP  = 'Aerosol particle number density (AERONET)'
  CALL Read_Input (STR_INP, READ_ARR, *880)
  CALL SPLIT_STRING(READ_ARR(1)%S, STR_ARR)
  F_name(3) = DIR_name//CUT_STR(STR_ARR(1)%S)
  READ(STR_ARR(2)%S,*,ERR=101) Scal_PRF
     WRITE(*,'(A,2x,f4.2)')  TRIM( F_name(3) ), Scal_PRF
!     PAUSE
  N_COLS = 2
  Call Read_File (N_COLS, F_name(3), Arr, *880)
  NA_LEV = SIZE( Arr(:,1) )
  IF(ALLOCATED(AER_LEV)) DEALLOCATE(AER_LEV, AER_PRF)
  ALLOCATE( AER_LEV(NA_LEV), AER_PRF(Na_LEV) ) ! A
  AER_LEV(:) = Arr(:,1)
  AER_PRF(:) = Arr(:,2)
! ... Interpolate Number density shape at the standard altitude grid
!  write(*,*) '   AER_LEV ', AER_LEV
!  write(*,*) '   ZHOLD ', ZHOLD
  CALL LIN_INT(Na_LEV, AER_LEV, AER_PRF, NLAYERS, ZHOLD, DENS_PRF, 'Z')
! .................................................
  DO I = 1, NLAYERS
     IF( DENS_PRF(I) == 0.d0 ) CYCLE
     WRITE(*,'(2x,F8.2,2x,E12.4)') ZHOLD(I), DENS_PRF(I) 
  ENDDO
! .................................................
  Col_Nd = SUM( DENS_PRF(:)*DZ(:) )/2.0d0
  DO K = 1, NAW
     CEXT(1,K) = E_OD(K)/Col_Nd
     CSCA(1,K) = (E_OD(K) - A_OD(K))/Col_Nd
  ENDDO
  
  CALL PH_SCAT_TO_COEFF(*880)

  IF( Nm_vs > 1 ) THEN
     USER_MOM(:,:,:,5) = -USER_MOM(:,:,:,5)
     USER_MOM(:,:,:,6) = -USER_MOM(:,:,:,6)
  ENDIF

  !=========================================================
  !   Read user-defined aerosol scenario input parameters
  !=========================================================
  LAN = 1
  ALLOCATE( LAS(LAN), LAE(LAN), L_TOP(LAN), & !A
       IDX_W(LAN), APH_VS( NLEGEN,LAN,Nm_vs,NAW ) )    !A
  APH_VS = 0.0d0  
  AER_PHASMOMS => APH_VS( :,:,1,: )
! ... Top altitude of aerosol layer
  L_TOP(1) = ZHOLD(1)
  DO I = NLAYERS,1,-1
     IF( DENS_PRF(I) == 0.d0 ) THEN
        L_TOP(1) = ZHOLD(I)  ;   EXIT
     ENDIF
  ENDDO
  WRITE(*,*) '   L_TOP(1) = ', L_TOP(1)
  
  IF( AER_SHIFT ) L_TOP(:) = L_TOP(:) + GROUND_HEIGHT
  
  IF( LAN > 1 ) THEN
     IF( L_Top(LAN-1) > ZHOLD(1) ) GOTO 131
  ELSE
     IF( L_Top(LAN) > ZHOLD(1) ) GOTO 133
  ENDIF
  IF( L_Top(1) <= ZHOLD(NLAYERS) ) GOTO 132

  AER_PRF(:) = AER_SCALING*AER_PRF(:)
  IF( AER_SHIFT ) AER_LEV(:) = AER_LEV(:) + GROUND_HEIGHT
  IF( SHARP_EDGES ) AER_LEV(:) = ZA_SE(:)
  
  CALL AER_LAYERING

  ! Scaling of aerosol number density profile if needed
  DENS_PRF(:) = Scal_PRF*DENS_PRF(:)

  DO J = 1,LAN
     DO K1 = LAS(J), LAE(J)
        I1 = 1
        DO K = 1, NAW
           AER_COEFF(EXI,K1,I1) = CEXT(J,K)*DENS_PRF(K1)
           AER_COEFF(SCI,K1,I1) = CSCA(J,K)*DENS_PRF(K1)
!           AER_COEFF(SCI,K1,I1) = AER_COEFF(EXI,K1,I1)* 0.88
           I1 = I1 + 1
        ENDDO
     ENDDO
  ENDDO

  ! Scaling of the scattering coefficient between ZS1 - ZS2
  ! in order to change the single scattering albedo as follows:
  !    SSA' = 1 - V_PAP*(1 - SSA) 
  ! Extinction coefficient remains the same
  IF( L_scal ) THEN
     DO I = 1, NAW
        WHERE( ZHOLD(:) >= ZS1.AND.ZHOLD(:) <= ZS2 ) &
             AER_COEFF(SCI,:,I) = AER_COEFF(SCI,:,I) - (V_PAP - 1.d0) &
             *(AER_COEFF(EXI,:,I ) - AER_COEFF(SCI,:,I ))
     ENDDO
  ENDIF

  MAX_MOM = MAXVAL( N_MOMS(:) )
!  IF (LOG_LEVEL(1) >= N_ESO) THEN
     WRITE(LUNIT,*) '   Max number of expansion coeff. :', MAX_MOM
     WRITE(LUNIT,*) '  ----------------------------------'
!  ENDIF

  ! Single scattering correction algorithm 
  ! --------------------------------------
  IF( DO_SSRCA .OR. TRIM(RTM_TYPE) == 'spher_scat' ) THEN
     Nga_db = MAX_MOM 
     ALLOCATE( Greek_aer( Nga_db,LAN,Nm_vs,NAW ) )  ;  Greek_aer = 0.d0
     DO I = 1,NAW 
        DO K = 1,Nm_vs
           DO L = 1, LAN
              Greek_aer(:,L,K,I) = USER_MOM(I,1:Nga_db,L,K)
           ENDDO
        ENDDO
     ENDDO
     IF( .NOT.Plus_mi ) NG_aer = Nga_db
  ENDIF

  IF( OUT_A_SCM ) THEN 
     IF( LAY_A > LAN ) GOTO 135
     IF( WEL_A > NAW )  GOTO 136
     CALL COEF_TO_SCATMATR(MAX_MOM, Nm_vs, &
          USER_MOM(WEL_A,1:MAX_MOM,LAY_A,1:Nm_vs), NSCA_A, GE_A, PATH_TO_SCM)
  ENDIF

  DO I = 1,LAN 
     DO J = LAS(I),LAE(I)
        AER_COEFF(ASY,J,:) = ASYM_PAR(I,:)
     ENDDO
  ENDDO

  ! Aerosol optical thickness according to input N(z)
  DO K = 1,NAW
     TAU(K) = SUM( AER_COEFF(EXI,:,K)*DZ(:) )/2.d0
  ENDDO

  ! Scaling input N(z) according to required AOD
  IF( Scal_aer ) THEN
     CALL LIN_SEARCH(Rwl_aer, DUM, K, NAW, AER_WAVS, ERROR)
     IF(ERROR) GOTO 129
     DD1 = (1.d0 - DUM)*TAU(K) + DUM*TAU(K+1)
     IF( DD1 > 0.d0 ) THEN
        Scoe = Rtau_aer/DD1
     ELSE
        Scoe = 0.d0
     ENDIF
     AER_COEFF(EXI,:,:) = Scoe*AER_COEFF(EXI,:,:)
     AER_COEFF(SCI,:,:) = Scoe*AER_COEFF(SCI,:,:)
     TAU(:) = Scoe*TAU(:)
  ENDIF  ! Scal_aer 
  AOT_out_VAL = TAU(1)

  WRITE(*,*) '   =====> Inp AOT at reference wl '
  DO K = 1,NAW
     WRITE(*,'(2x,I2,2x,F8.3,2x,E12.4)') K, AER_WAVS(K),&
          SUM( AER_COEFF(EXI,:,K)*DZ(:) )/2.d0
  ENDDO
  
  !  Write aerosol scenario output file
  CALL WRITE_SCEN_AERONET(*880)

  !Allocate array for MODULE DELTA_M
  !=================================
  IF ( AER_DM_FLG ) THEN ;  Itr = AER_TRUNC_IDX
     SELECT CASE( Vec_Sca )
     CASE('vect_rte')  ;  CALL AER_DM_VEC
     CASE('scal_rte')  ;  CALL AER_DM
     END SELECT
     !"Renormalize" scattering and extinction coefficients
     !====================================================
     DO I = 1, NLAYERS
        DO K = 1,NAW
           IF( AER_COEFF(EXI,I,K) == 0.d0 ) FDM_AER(I,K) = 0.d0
           F_DM = FDM_AER(I,K)
           AER_ABS_COEFF = AER_COEFF(EXI,I,K ) - AER_COEFF(SCI,I,K)
           AER_COEFF(SCI,I,K) = AER_COEFF(SCI,I,K)*(1.d0-F_DM)
           AER_COEFF(EXI,I,K) = AER_COEFF(SCI,I,K) + AER_ABS_COEFF 
        ENDDO
     ENDDO
  ELSE
     DO I1 = 1,LAN
        DO K = 1, NAW
           APH_VS(:,I1,:,K) = USER_MOM(K,1:NLEGEN,I1,:)
        ENDDO
     ENDDO
     ! DO I1 = 1,LAN
     !    DO K1 = LAS(I1), LAE(I1)
     !       DO K = 1, NAW
     !          APH_VS(:,K1,:,K) = USER_MOM(K,1:NLEGEN,I1,:)
     !       ENDDO
     !    ENDDO
     ! ENDDO
  ENDIF

  LA_Ndb = LAN
  IF( .NOT.Plus_mi ) THEN
     LA_N = LAN
     ALLOCATE( LA_S(LAN), LA_E(LAN), IDX_AER(NLAYERS) )
     LA_S = LAS ;  LA_E = LAE  ; IDX_AER = IDA
  ELSE
     ALLOCATE( IDX_AER_db(NLAYERS) )
     IDX_AER_db = IDA
  ENDIF

!  Stop '   Stop in "aeronet_setup.f90" '  

  CALL ST_TRACK (R_NAME, 1, 'O')
  RETURN

101 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Reading error')
  WRITE(EU,*) '    Parameter to read: ', STR_INP 
  RETURN 1

126 CALL WRITE_MESSAGE (R_NAME, 'ERROR', 'Error in LIN_SEARCH')
  RETURN 1
  
127 CALL WRITE_MESSAGE (R_NAME,'ERROR', 'Aerosol models name not found')
  RETURN 1

129 CALL WRITE_MESSAGE (R_NAME,'ERROR', 'Reference wavelength for AOT is out of range')
  WRITE(EU,'(A,F7.2)') '    Reference wavelength: ', Rwl_aer
  WRITE(EU,'(A,2(2x,F7.2))') '    Database wavelengths: ', AER_WAVS(1), AER_WAVS(NAW)
  WRITE(EU,'(A)') '    Set reference wavelength between database wavelengths.'
  RETURN 1

125 CALL WRITE_MESSAGE (R_NAME,'ERROR', 'Number of expansion coefficient in file')
  WRITE(EU,'(A,I4)') '    Number of coefficient: ', N_MOMS(I)
  WRITE(EU,'(A,I4)') '    Maximal dimension: ', NM_MAX
  WRITE(EU,'(A)') '    Increase NM_MAX in the file "aeronet_setup.f90".'
  RETURN 1

130 CALL WRITE_MESSAGE(R_NAME, 'ERROR', &
         'Required wavelength range is outside of the data base.')
  WRITE(EU,'(A,2x,F9.2)') '    Last wavelength in data base:', AER_WAVS(NAW)
  WRITE(EU,'(A,2x,F9.2)') '    Last required wavelength    :', END_WAV
  WRITE(EU,*) '    Aerosol data base does not exist '
  WRITE(EU,*) '    for wavelengths > 40 micron.'
  RETURN 1

131 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Upper aerosol layer outside atmosphere')
  WRITE(EU,'(A,2x,F9.2,A,F9.2)') & 
       '    Upper aerosol layer:', L_TOP(LAN-1),' - ',L_TOP(LAN)
  WRITE(EU,'(A,2x,F9.2,A)') '      Top of atmosphere:', ZHOLD(1),' km'
  WRITE(EU,*) '    Check "pao_aer.inp" and altitude grid '
  WRITE(EU,*) '    as defined in the file "control.inp" '
  RETURN 1

132 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Lower aerosol layer outside the atmosphere')
  WRITE(EU,'(A,2x,F6.2,A)') '       Top of aerosol layer:', L_TOP(1),' km'
  WRITE(EU,'(A,2x,F6.2,A)') '       Surface elevation   :', ZHOLD(NLAYERS),' km'
  WRITE(EU,*) '    Check "pao_aer.inp" and "Height above '
  WRITE(EU,*) '    sea level" in the file "control.inp" '
  RETURN 1

133 CALL WRITE_MESSAGE(R_NAME, 'ERROR', ' Aerosol layer outside the atmosphere')
  WRITE(EU,'(A,2x,F9.2,A,F9.2)') '    Aerosol layer:', ZHOLD(NLAYERS),' - ',L_TOP(LAN)
  WRITE(EU,'(A,2x,F9.2,A)') '      Top of atmosphere:', ZHOLD(1),' km'
  WRITE(EU,*) '    Check "pao_aer.inp" and altitude grid '
  WRITE(EU,*) '    as defined in the file "control.inp" '
  RETURN 1

135 CALL WRITE_MESSAGE (R_NAME,'ERROR', &
         'Required number of aerosol layer is out of range')
  WRITE(EU,'(A,I3)') '    Number of aerosol layers: ', LAN
  WRITE(EU,'(A,I3)') '    Required layer number   : ', LAY_A
  RETURN 1

136 CALL WRITE_MESSAGE (R_NAME,'ERROR',&
         'Required number of wavelength is out of range')
  WRITE(EU,'(A,I3)') '    Number of wavelengths for aerosol: ',NAW
  WRITE(EU,'(A,I3)') '    Required wavelength number       : ',WEL_A
  RETURN 1

138 CALL WRITE_MESSAGE (R_NAME, 'ERROR', ' ')
  WRITE(EU,'(A)') '    Option "PAO aerosol as a cloud layer?"'
  WRITE(EU,'(A)') '    can be used in the cloud-free case only.'
  WRITE(EU,'(A)') '    Check input file "pao_aer.inp".'
  RETURN 1
  
880 WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)
  RETURN 1
  
CONTAINS
  
! ************************************************************************
  SUBROUTINE PH_SCAT_TO_COEFF(*)
    INTEGER  Na_st, I, K, L, Lay_out, WL_out
    INTEGER, PARAMETER :: MUM = 500, N_gau = MUM/2
    LOGICAL  Error
    REAL (KND)  Norm, Poly(MUM,N_gau), DUM_READ(MUM,2), TMP1
    REAL (KND), DIMENSION (N_gau) :: X_gau, AG_gau, X_rad,  Phs_gau, Phx, Del
    REAL (KND), ALLOCATABLE :: Sa_inp(:), Phs_inp(:), Sa_rad(:), Ph_out(:)
!     ------------------------------------------------------------------

!     ... Gauss grid and weights for integration (-1,1)
      CALL GAULEG ( -1.D0, 1.D0, X_gau, AG_gau, N_gau )
!     ... Legendre polynomials for all Gauss angles
      CALL WIGNER_PLG( X_gau, N_gau, 0, MUM, Poly )
      X_rad(:) = DACOS( X_gau(N_gau:1:-1) )
      Na_st = N_ang
      ALLOCATE( Sa_inp(Na_st),Sa_rad(Na_st),Phs_inp(Na_st) ) ! A
      Sa_inp(:)  = Ang(:)
!     ... Convert [deg] to [rad] for interpolation of phase functions
      Sa_rad(:) = Sa_inp(:)/RAD_TO_DEG
!     ...  
      DO K = 1,NAW
!        ...  Original scattering function 
         Phs_inp(:) = Phf(:,K)
!        ... Interpolate scattering function on the Gauss grid
         Phs_inp(:) = DLOG( Phs_inp(:) )
         CALL AKIMA_INTERP (Na_st, Sa_rad, Phs_inp, N_gau, X_rad, Phs_gau, Error)
         IF( Error ) THEN
            Call WRITE_MESSAGE(R_NAME,'ERROR', 'Interpolation error')
            RETURN 1
         ENDIF
!        ... Invert to start from -1
         Phs_gau(:) = DEXP( Phs_gau(N_gau:1:-1) )
!        ... Norm of the interpolated phase function
         Norm = SUM( Phs_gau(:)*AG_gau(:) )*0.5d0
!         WRITE(LUNIT,*) '   Norm of the phase function: ', Norm
!        ... Expansion coefficients of the intrpolated phase function
         Phx(:) = 0.d0
         DO L = 1,MUM
            TMP1 = (DBLE(L) - 0.5d0)  &
                 *SUM( Phs_gau(:)*Poly(L,:)*AG_gau(:) )/Norm
            USER_MOM(K,L,1,1) = TMP1
            DO J = 1,N_gau
               Phx(J) = Phx(J) + Poly(L,J)*TMP1
            ENDDO  ! over Gauss angles (J)
            Del(:) = DABS( Phs_gau(:) - Phx(:)*Norm )/Phs_gau(:)*100.
            IF( MAXVAL( Del(:) ) <= 0.1d0 ) EXIT 
         ENDDO  ! over expansion coefficients (L)
         N_moms(K) = L
         ASYM_PAR(1,K) = USER_MOM(K,2,1,1)/3.d0
!         WRITE(LUNIT,*) '   Number of expansion coeff: ', N_moms(K) 
!         WRITE(LUNIT,*) '   Asymmetry parametr: ', ASYM_PAR(1,K) 
      ENDDO  ! over wavelengths (K) 

    END SUBROUTINE PH_SCAT_TO_COEFF
!***********************************************************************

  SUBROUTINE AER_LAYERING
    DO I = 1,LAN
       ! Ordered from the bottom to the top 
       LAS(I) = MINLOC( DABS(L_Top(I)-ZHOLD(:)),1 )
    ENDDO
    LAE(1) = NLAYERS
    DO I = 2,LAN ; LAE(I) = LAS(I-1)-1  ; ENDDO
       ! Indexes of altitude grid levels which are within aerosol layers
       IDA(:) = LAN
       DO I = 1,LAN - 1
          DD1 = ZHOLD(LAS(I)) ;  DD2 = ZHOLD(LAE(I))
          DO J = 1,NLAYERS
             IF( ZHOLD(J) <= DD1.AND.ZHOLD(J) >= DD2 )  IDA(J) = I 
          ENDDO
       ENDDO
       DD1 = ZHOLD(MAXVAL(LAE))
       ! Indexes of altitude grid levels which are below of lowest aerosol layer
       DO J = 1,NLAYERS
          IF( ZHOLD(J).LE.DD1 ) IDA(J) = 1
       ENDDO
!       IF (LOG_LEVEL(1) >= N_ESO) THEN
          WRITE(LUNIT,*) ' ------ Test of aerosol layering (AERONET) --- '
          DO I = 1,LAN
             WRITE(LUNIT,'(2x,I3,2(1x,f7.3))') &
                  I, ZHOLD(LAS(I)), ZHOLD(LAE(I))
          ENDDO
!       ENDIF
  END SUBROUTINE AER_LAYERING
  ! **************************************************************************
  
  SUBROUTINE AER_DM
    DO I1 = 1,LAN
       DO K = 1, NAW
          F_DM = USER_MOM(K,Itr+1,I1,1)/DBLE(2*Itr + 1)
          FDM_AER(LAS(I1):LAE(I1),K) = F_DM
          DO L = 1, Itr
             AER_PHASMOMS(L,I1,K) = ( USER_MOM(K,L,I1,1) - &
                  F_DM*DBLE(2*L-1) )/(1.d0 - F_DM)
          ENDDO
       ENDDO
    ENDDO
    AER_PHASMOMS(Itr+1:NLEGEN,:,:) = 0.0D0
  END SUBROUTINE AER_DM
  
!***********************************************************************
  SUBROUTINE AER_DM_VEC
    DO I1 = 1,LAN
       DO K = 1, NAW
          F_DM = USER_MOM(K,Itr+1,I1,1)/DBLE(2*Itr + 1)
          FDM_AER(LAS(I1):LAE(I1),K) = F_DM
          DO L = 1, Itr
             APH_VS(L,I1,1:4,K) = ( USER_MOM(K,L,I1,1:4) - &
                  F_DM*DBLE(2*L-1) )/(1.0D0 - F_DM)
             APH_VS(L,I1,5:6,K) = USER_MOM(K,L,I1,5:6)/(1.0D0 - F_DM)
          ENDDO
       ENDDO
    ENDDO
    APH_VS(Itr+1:NLEGEN,:,:,:) = 0.0D0
    
  END SUBROUTINE AER_DM_VEC

!***********************************************************************
  SUBROUTINE  WRITE_SCEN_AERONET(*)
    USE  AEROSOLS, ONLY: OWL_AER, NOWL_AER, Alph, C_a
    USE TOOLS,     ONLY: OpenOut, WriteOut, CloseOut 
 
    INTEGER             K1, K2, J, Unt
    CHARACTER(LEN=3)    PHT
    Character (Len=500)  OutStr
    REAL (KND)  SS_A, Ext, S1
    REAL (KND), ALLOCATABLE :: T_OUT(:), E_OUT(:), S_OUT(:), T_BL(:), T_UP(:),&
         P_EXT(:,:)
!***********************************************************************
    
    IF (LOG_LEVEL(1) >= N_SSO) WRITE (lunit,*) ' *** writing: AEROSOL.OUT'

    ! Optical thickness at user defined wavelength grid
    IF( NOWL_AER > 0 ) THEN 
       K1 = LAS(1) ; K2 = LAE(1)
       ALLOCATE( T_OUT(NOWL_AER),E_OUT(NOWL_AER),S_OUT(NOWL_AER),  &
            T_BL(NOWL_AER), T_UP(NOWL_AER), P_EXT(NLAYERS,2))
       DO I = 1, NOWL_AER
          CALL LIN_SEARCH(OWL_AER(I), DUM, K, NAW, AER_WAVS, ERROR)
          IF (ERROR) GOTO 134
          T_OUT(I) = (1.d0 - DUM)*TAU(K) + DUM*TAU(K+1)          
          ! Single scattering albedo at user-defined wavelengths
          S_OUT(I) = (1.d0 - DUM)*AER_COEFF(SCI,NLAYERS-1,K) &
               + DUM*AER_COEFF(SCI,NLAYERS-1,K+1)
          ! Extinction coefficient at user-defined wavelengths
          P_EXT(:,1:2) = AER_COEFF(EXI,:,K:K+1)
          E_OUT(I) = (1.d0 - DUM)*P_EXT(NLAYERS-1,1) + DUM*P_EXT(NLAYERS-1,2)
          ! Optical thickness of the boundary layer
          S1 = ( (1.d0-DUM)*P_EXT(K1,1) + DUM*P_EXT(K1,2) ) &
               *(ZHOLD(K1) - ZHOLD(K1+1)) + ( (1.d0-DUM)*P_EXT(K2,1) &
               + DUM*P_EXT(K2,2) )*(ZHOLD(K2-1) - ZHOLD(K2))
          DO J = K1+1,K2-1
             S1 = S1 + ( (1.d0-DUM)*P_EXT(J,1) + DUM*P_EXT(J,2) ) &
                  *(ZHOLD(J-1) - ZHOLD(J+1))
          ENDDO
          T_BL(I) = S1/2.d0

          ! Optical thickness of the atmosphere above boundary layer
!          S1 = ( (1.d0-DUM)*P_EXT(1,1) + DUM*P_EXT(1,2) )*(ZHOLD(1) - ZHOLD(2)) &
!               + ( (1.d0-DUM)*P_EXT(K1,1) + DUM*P_EXT(K1,2) ) &
!               *(ZHOLD(K1-1) - ZHOLD(K1))
!          DO J = 2,K1-1
!             S1 = S1 + ( (1.d0-DUM)*P_EXT(J,1) + DUM*P_EXT(J,2) ) &
!                  *(ZHOLD(J-1) - ZHOLD(J+1))
!          ENDDO
!          T_UP(I) = S1/2.d0
       ENDDO

       OPEN(NewUnit=DUNIT,FILE = DIR_OUT//'AOT.OUT')            
       WRITE(DUNIT,*) '# AOT and single scattering albedo (boundary layer) '
       WRITE(DUNIT,*) '#        at user-defined wavelengths [nm]:'
       DO I = 1, NOWL_AER
          WRITE(DUNIT,'(2x,F8.3,4(2x,E15.7))') OWL_AER(I),&
               T_BL(I), T_UP(I), T_OUT(I), S_OUT(I)/E_OUT(I)
       ENDDO
       CLOSE(DUNIT)
       
       ! Angstrom parameters for this aerosol model
       Alph = LOG( TAU(1)/TAU(2) )/LOG( AER_WAVS(2)/AER_WAVS(1) )
       T_OUT(2) = TAU(1)*( AER_WAVS(1)/OWL_AER(2) )**Alph
!       T_OUT(2) = TAU(2)*( AER_WAVS(2)/OWL_AER(2) )**Alph
!        C_a = T_OUT(1)/OWL_AER(1)**Alph
!      Alph = LOG( T_BL(1)/T_BL(NOWL_AER) )/LOG( OWL_AER(1)/OWL_AER(NOWL_AER) )
!      C_a = T_BL(1)/OWL_AER(1)**Alph
!         WRITE(*,*) '  Angstrom parameters '
!         WRITE(*,*) '  C_a  = ', C_a
!         WRITE(*,*) '  Alph = ', Alph
    ENDIF  ! NOWL_AER > 0

    Call OpenOut ('SCE_AEROSOL.OUT', Unt)
    WRITE(OutStr,*) '          SETTINGS FOR AERONET INPUT  '
    Call WriteOut (Unt, OutStr)
    WRITE(OutStr,*) '     ==================================='
    Call WriteOut (Unt, OutStr)
    WRITE(OutStr,*) ' '
    Call WriteOut (Unt, OutStr)
  
    IF( Scal_aer ) THEN
       WRITE(OutStr,'(A,F7.3,A,F9.2,A)') &
       '   !!! ATTENTION !!! AOT is set to ',Rtau_aer,' at', Rwl_aer,' nm wavelength.'
       Call WriteOut (Unt, OutStr)
       WRITE(OutStr,'(19x,A)') '  See setting in "control_aer.inp" '
       Call WriteOut (Unt, OutStr)
       WRITE(OutStr,'(6x,A)')  '----------------------------------------------------'
       Call WriteOut (Unt, OutStr)
       WRITE(OutStr,*) 
       Call WriteOut (Unt, OutStr)
    ENDIF

    IF( NOWL_AER > 0 ) THEN
       WRITE ( OutStr, * ) ' AOT at user-defined wavelengths [nm]:'
       Call WriteOut (Unt, OutStr)
       DO I = 1, NOWL_AER
          WRITE(OutStr,'(12x,F7.2,4x,F8.4)') OWL_AER(I),T_OUT(I)
          Call WriteOut (Unt, OutStr)
       ENDDO
    ENDIF
    Call WriteOut (Unt, ' ')

    WRITE(OutStr,'(A27,2X,I2)') '  Number of aerosol layers:', LAN
    Call WriteOut (Unt, OutStr)
    Call WriteOut (Unt, '  ')
    DO I = 1, LAN
       WRITE(OutStr,'(A27,2X,I2)') '  Layer :                 :', I
       Call WriteOut (Unt, OutStr)
       WRITE(OutStr,'(A27,2X,F6.2)') '  Upper boundary [km]     :', L_TOP(I)
       Call WriteOut (Unt, OutStr)
       
       Call WriteOut (Unt, '')
    ENDDO
          
    WRITE(OutStr,'(A47)') 'Profile type: particle number density [1/cm^3].'
    Call WriteOut (Unt, OutStr)
    
    Call WriteOut (Unt, ' ')
    Call WriteOut (Unt,' Vertical profiles of aerosol')
    Call WriteOut (Unt,'         extinction coefficient (Ext coeff) and')
    Call WriteOut (Unt,'         single scattering albedo (Ss albedo):')
    IF(APH_TYPE == 'mie') THEN
       PHT = 'Mie'
    ELSE
       PHT = 'H-G'
    ENDIF

    DO K = 1,NAW
       Call WriteOut (Unt,'   ')
       WRITE(OutStr,'(1x,a11,F9.2,a6,a6,F8.4)') &
            'Wavelength ',AER_WAVS(K),' [nm];',' AOT =',TAU(K)
       Call WriteOut (Unt, OutStr)
       Call WriteOut (Unt,' ----------------------------------------------')
       Call WriteOut (Unt, '     Number, Alt [km], Ext coeff [km-1], Ss albedo')
       DO I = 1, NLAYERS
          Ext = AER_COEFF(EXI,I,K)
          IF( Ext == 0.0d0 ) CYCLE
          SS_A = AER_COEFF(SCI,I,K)/Ext
          WRITE(OutStr,'(6x,I3,4x,F6.2,5x,ES9.2,6x,E12.4)') I, ZHOLD(I), Ext, SS_A
          Call WriteOut (Unt, OutStr)
       END DO
       Call WriteOut (Unt, ' ')
       Call WriteOut (Unt,'    Asymmetry factor used in the aerosol layers :')
       Call WriteOut (Unt,'    ------------------------------------------- :')
       Call WriteOut (Unt,'    Layer  Top [km]  Bottom [km]   Asymmetry    Phase')
       Call WriteOut (Unt,'                                    factor     function')
       DO I = 1,LAN
          WRITE(OutStr,'(6x,I3,3x,F6.2,4x,F6.2,9x,F6.3,7x,a3)') &
               I, ZHOLD(LAS(I)) ,ZHOLD(LAE(I)), Asym_Par(I,K), PHT
          Call WriteOut (Unt, OutStr)
       ENDDO
    ENDDO                   ! Over wavelengths
          
    Call CloseOut (Unt)
    RETURN

134 CALL WRITE_MESSAGE (R_NAME,'ERROR', 'Output wavelength for AOT is out of range')
    WRITE(EU,'(A,F7.2)') '    Output wavelength: ', OWL_AER(I)
    WRITE(EU,'(A,2(2x,F7.2))') '    Database wavelengths: ', AER_WAVS(1), AER_WAVS(NAW)
    WRITE(EU,'(A)') '    Set output wavelength between database wavelengths'
    WRITE(EU,'(A)') '    in the "control_aer.inp" file.'
    RETURN 1

  END SUBROUTINE  WRITE_SCEN_AERONET

!***********************************************************************
  SUBROUTINE DEALLOCATE_ARRAYS
    
    IF(ALLOCATED(DENS_PRF)) DEALLOCATE(DENS_PRF)
    IF(ALLOCATED( AER_WAVS)) DEALLOCATE(AER_WAVS, AER_COEFF, FDM_AER)
    IF(ALLOCATED(L_TOP)) DEALLOCATE(L_TOP, APH_VS)
    IF(ALLOCATED(Greek_aer)) DEALLOCATE(Greek_aer)
    IF(ALLOCATED(LA_S)) DEALLOCATE(LA_S, LA_E, IDX_AER)
    IF(ALLOCATED(IDX_AER_db)) DEALLOCATE(IDX_AER_db)
    
    RETURN
  END SUBROUTINE DEALLOCATE_ARRAYS
!***********************************************************************
END SUBROUTINE AERONET_SETUP
!***********************************************************************

