!*********************************************************************
!
!	SCIATRAN - Version 3.5 - Last modification: 13 Sep 2015
!
!	Copyright (C) - Institute of Remote Sensing,
!			University of Bremen, Germany
!
!*********************************************************************
SUBROUTINE ATOMS_SPI(WL_S, WL_E)
  ! Called by "gt_iface.f" line 686

  USE EMISSION,   ONLY: N_AT
  USE INFO,       ONLY: TG_INFO
  USE TYPE_VARS,  ONLY: KND
! ----------------------------------------------------------------------

  REAL (KND), INTENT(In):: WL_S, WL_E

  CHARACTER (LEN = *), PARAMETER :: R_NAME = 'ATOMS_SPI'
  !     ... Universal constants
  REAL (KND), PARAMETER ::  pi = 3.1415926536d0, &           
       eps_0 = 8.85418781762039d-12, & !  dielectric constant [F/m]
       k_Bolz = 1.3806488d-23,       & ! Boltzmann constant [m^2*kg/s/K]
       S_light = 2.99792458d+08,     & !  speed of light [m/s]
       e_cr = 2.8179403267d-15         !   classical electron radius [m]       
  INTEGER    I, II, JJ, NLin, UNIT
  REAL (KND) Na_Ws, Na_We, Mgi_Ws, Mgi_We, Mga_Ws, Mga_We

  CALL ST_TRACK (R_NAME, 1, 'I')

  ! ... Spectral range relevant to Na emission
  Na_Ws = 588.5d0  ;  Na_We = 590.0d0
  ! ... Spectral range relevant to Mg+ emission
  Mgi_Ws = 277.d0  ;  Mgi_We = 282.d0
  ! ... Spectral range relevant to Mg emission
  Mga_Ws = 284.d0  ;  Mga_We = 287.d0

  N_AT = 0
  DO JJ = 1, SIZE(TG_INFO%NAME)
     IF( .NOT.TG_INFO(JJ)%FM_FLG ) CYCLE
     SELECT CASE (TG_INFO(JJ)%NAME)
     CASE('na')
        IF( WL_S > Na_We.OR.WL_E < Na_Ws ) THEN
           TG_INFO(JJ)%FM_FLG = .FALSE.
           TG_INFO(JJ)%WF_FLG = .FALSE.
        ELSE
!           WRITE(*,*) '   Select "NA" '
           ! ... Spectroscopic information for Na
           CALL SPI_NA (JJ)
           N_AT = N_AT + 1
        ENDIF
     CASE('mgi')
        IF( WL_S > Mgi_We.OR.WL_E < Mgi_Ws ) THEN
           TG_INFO(JJ)%FM_FLG = .FALSE.
           TG_INFO(JJ)%WF_FLG = .FALSE.
        ELSE
!           WRITE(*,*) '   Select "MgI" '
           ! ... Spectroscopic information for Mg+
           CALL SPI_MGI (JJ)
           N_AT = N_AT + 1
        ENDIF
     CASE('mga')
        IF( WL_S > Mga_We.OR.WL_E < Mga_Ws ) THEN
           TG_INFO(JJ)%FM_FLG = .FALSE.
           TG_INFO(JJ)%WF_FLG = .FALSE.
        ELSE
!           WRITE(*,*) '   Select "MgA" '
           ! ... Spectroscopic information for Mg
           CALL SPI_MGA (JJ)
           N_AT = N_AT + 1
        ENDIF
     CASE DEFAULT
     END SELECT
  ENDDO

  CALL ST_TRACK (R_NAME, 1, 'O')
  RETURN

CONTAINS
!***********************************************************************
!     ****** Spectroscopic information for Na atom ******
  SUBROUTINE SPI_NA(II)

    INTEGER, INTENT(IN):: II
    REAL (KND), PARAMETER :: m_Na = 22.99d0*1.66053892d-27 ! Na atom mass [kg]
    REAL (KND) A_dop, Temp, Del_D1_D2, VCD, E1, E2, E3
    REAL (KND), ALLOCATABLE:: Na_LP(:), Na_OS(:), Na_int(:)

    ! ... Distance between "centrum" of D1 and D2 lines [cm-1]
    Del_D1_D2 = 17.196d0 

    ! ... Number of Na lines
    NLin = 4
    TG_INFO(II)%ALP_NLIN = NLin

    IF( ALLOCATED(TG_INFO(II)%ALP_LI) ) THEN
       DEALLOCATE( TG_INFO(II)%ALP_LI, TG_INFO(II)%ALP_LP,  &
            TG_INFO(II)%ALP_HW, TG_INFO(II)%ALP_EP )
    ENDIF
    ALLOCATE( TG_INFO(II)%ALP_LI(NLin), TG_INFO(II)%ALP_LP(NLin), &
         TG_INFO(II)%ALP_HW(NLin), Na_OS(NLin), Na_LP(NLin), Na_int(NLin), &
         TG_INFO(II)%ALP_EP(2,3) )

    ! ... Oscillator strengths (NIST):
    !               ----  D1 ----      ---  D2  ---
    Na_OS = [0.320d0, 0.320d0, 0.641d0, 0.641d0]
    ! ... Including weight for s-resolved lines
    Na_OS(1) = Na_OS(1)*10.d0/16.d0
    Na_OS(2) = Na_OS(2)* 6.d0/16.d0
    Na_OS(3) = Na_OS(3)*20.d0/32.d0
    Na_OS(4) = Na_OS(4)*12.d0/32.d0

    ! ... Position of s-resolved D1 and D2 emission lines [cm-1 vacuum] 

    !        J' --> J    DelJ           E2                              E3
    !  D1 | 1/2 --> 1/2 |  0 | 3(2j^2 + 2j + 1)/(10j(j + 1))  |  1/(2j(j+1))   |
    !  D2 | 3/2 --> 1/2 |  1 |  3j(6j + 7)/(10(j+1)(2j+1))    | (j+2)/(2(j+1)) |
    !
    !            DelJ = -1 > | 3(6j^2 + 5j - 1)/(10j(2j + 1)) |  (J-1)/(2J)    |
    !
    !         E1 = 1 - E2
    !         (McNutt and Mack, 1963)
    !        .... s-resolved  D1 ......  .... s-resolved  D2 ......
    Na_LP = [16956.1476d0, 16956.2088d0, 16973.3448d0, 16973.4018d0]

    ! ... Separation wavenumber for D1 and D2 sodium lines [cm-1]
    TG_INFO(II)%S_NU = ( (Na_LP(1) + Na_LP(2))/2.d0 + Del_D1_D2/2.d0 )

    ! ... Phase function weights for D1 sodium line
    !                          E1      E2      E3
    E1 = 0.d0 ; E2 = 1.d0 ; E3 = 2.d0/3.d0
    TG_INFO(II)%ALP_EP(1,:) = [1.d0, E3, E1]

    !     ... Phase function weights for D2 sodium line
    E1 = 0.5d0 ; E2 = 0.5d0 ; E3 = 5.d0/6.d0
    TG_INFO(II)%ALP_EP(2,:) = [1.d0, E3, E1]

    TG_INFO(II)%ALP_LI(:) = pi*(e_cr*1.d9) /Na_LP(:)**2*Na_OS(:) ! cm^2 * nm
    TG_INFO(II)%ALP_LP(:) = Na_LP(:)

    Temp = 200.d0
    A_dop = Na_LP(1)/S_light*DSQRT( 2.d0*k_Bolz*Temp/m_Na ) 
!    WRITE(*,'(A,2x,e12.4)') '   Doppler line with cm-1 ', A_dop
    A_dop = A_dop/Na_LP(1)**2*1.d7
!    WRITE(*,'(A,2x,e12.4)') '                       nm ', A_dop
      
    VCD = 5.0d9  ! [1/cm^2]
!    WRITE(*,*) '  Optical depth in line centrum ', TG_INFO(II)%ALP_LI(1)/A_dop* VCD

    ! ... Doppler half-width [nm] without SQRT(Temp), which will be included
    !           in routine "st_abs_coeff.f" 
    TG_INFO(II)%ALP_HW(:) = Na_LP(:)/S_light*DSQRT( 2.d0*k_Bolz/m_Na ) &
         ! [cm-1] --> [nm]
         /Na_LP(:)**2*1.d7

  END SUBROUTINE SPI_NA
!***********************************************************************

  SUBROUTINE SPI_MGI(II)

    INTEGER, INTENT(IN):: II
    REAL (KND), PARAMETER :: m_Mg = 24.305d0*1.66053892d-27 ! Mg atom mass [kg]
    REAL (KND) A_dop, Temp, Dmg_D1_D2, VCD, E1, E2, E3
    REAL (KND), ALLOCATABLE:: HD(:), Mgi_LP(:), Mgi_OS(:)

    !     ************ Spectroscopic information for Mg+ ion ***************
    !     ... Distance between "centrum" of D1 and D2 lines [cm-1]
    Dmg_D1_D2 = 91.557d0

    !     ... Number of Mg+ lines
      NLin = 6
      TG_INFO(II)%ALP_NLIN = NLin
      IF( ALLOCATED(TG_INFO(II)%ALP_LI) ) THEN
         DEALLOCATE( TG_INFO(II)%ALP_LI, TG_INFO(II)%ALP_LP,  &
              TG_INFO(II)%ALP_HW, TG_INFO(II)%ALP_EP )
      ENDIF
      ALLOCATE( TG_INFO(II)%ALP_LI(NLin), TG_INFO(II)%ALP_LP(NLin), &
           TG_INFO(II)%ALP_HW(NLin), TG_INFO(II)%ALP_EP(2,3), &
           MGI_OS(NLin), MGI_LP(NLin) )

      !     ... Oscillator strengths (NIST) the same for all isotopes:
      !                   ----  D1 ----               ---  D2  ---
      Mgi_OS = [0.303d0, 0.303d0, 0.303d0, 0.608d0, 0.608d0, 0.608d0]
      !     ... Including weight accounting for isotopic amount
      Mgi_OS(1) = Mgi_OS(1) *0.7899d0
      Mgi_OS(2) = Mgi_OS(2) *0.1000d0 
      Mgi_OS(3) = Mgi_OS(3) *0.1101d0
      Mgi_OS(4) = Mgi_OS(4) *0.7899d0
      Mgi_OS(5) = Mgi_OS(5) *0.1000d0
      Mgi_OS(6) = Mgi_OS(6) *0.1101d0

      !     ... Position of D1 and D2 lines of Mg+ [nm air] 
      !            24Mg               25Mg                    26Mg
      Mgi_LP = [280.2704d0, 280.2704d0-0.00042472d0, 280.2704d0-0.00080878d0, &
                279.5528d0, 279.5528d0-0.00042281d0, 279.5528d0-0.00080534d0 ]
      ALLOCATE( HD(NLin) )
      !     ... convert wavelength to vacuum
      HD(:) = 1.d0/Mgi_LP(:)/Mgi_LP(:)
      HD(:) = 6.4328d-05 + 2.94981d-08/(1.46d-04 - HD(:)) + &
           2.554d-10/(4.1d-05 - HD(:)) + 1.d0

!      write(*,*) '   Lines centra in vacuum [nm]: '
!      write(*,*) Mgi_LP(:)*HD(:)

      !     ... convert to wavenumber (cm-1)
      Mgi_LP(:) = 1.d7/(Mgi_LP(:)*HD(:))

      ! ... Separation wavenumber for D1 and D2 Mg+ lines [cm-1]
      TG_INFO(II)%S_NU = ( (Mgi_LP(1) + Mgi_LP(4))/2.d0 + Dmg_D1_D2/2.d0 )

      !     ... Phase function weights for D1 Mg+ line
      !                           E1      E2      E3
      E1 = 0.d0 ; E2 = 1.d0 ; E3 = 2.d0/3.d0
      TG_INFO(II)%ALP_EP(1,:) = [1.d0, E3, E1]
      !     ... Phase function weights for D2 Mg+ line
      E1 = 0.5d0 ; E2 = 0.5d0 ; E3 = 5.d0/6.d0
      TG_INFO(II)%ALP_EP(2,:) = [1.d0, E3, E1]

      TG_INFO(II)%ALP_LI(:) = pi*(e_cr*1.d9) /Mgi_LP(:)**2*Mgi_OS(:) ! cm^2 * nm
      TG_INFO(II)%ALP_LP(:) = Mgi_LP(:)

      Temp = 200.d0
      A_dop = Mgi_LP(1)/S_light*DSQRT( 2.d0*k_Bolz*Temp/m_Mg ) 
!      WRITE(*,'(A,2x,e12.4)') '   Doppler line with cm-1 ', A_dop
      A_dop = A_dop/Mgi_LP(1)**2*1.d7
!      WRITE(*,'(A,2x,e12.4)') '                       nm ', A_dop

      ! ... Doppler half-width [nm] without SQRT(Temp), which will be included
      !           in routine "st_abs_coeff.f" 
      TG_INFO(II)%ALP_HW(:) = Mgi_LP(:)/S_light*DSQRT( 2.d0*k_Bolz/m_Mg ) &
           ! [cm-1] --> [nm]
           /Mgi_LP(:)**2*1.d7

    END SUBROUTINE SPI_MGI
!***********************************************************************

  SUBROUTINE SPI_MGA(II)

    INTEGER, INTENT(IN):: II
    REAL (KND), PARAMETER :: m_Mg = 24.305d0*1.66053892d-27 ! Mg atom mass [kg]
    REAL (KND) A_dop, Temp, VCD, WLC, E1, E2, E3
    REAL (KND), ALLOCATABLE:: HD(:), Mga_LP(:), Mga_OS(:)

    !     ************ Spectroscopic information for Mg atom ***************

    ! ... Number of Mg lines including isotopic shift
      NLin = 3
      TG_INFO(II)%ALP_NLIN = NLin
      IF( ALLOCATED(TG_INFO(II)%ALP_LI) ) THEN
         DEALLOCATE( TG_INFO(II)%ALP_LI, TG_INFO(II)%ALP_LP,  &
              TG_INFO(II)%ALP_HW, TG_INFO(II)%ALP_EP )
      ENDIF
      ALLOCATE( TG_INFO(II)%ALP_LI(NLin), TG_INFO(II)%ALP_LP(NLin), &
           TG_INFO(II)%ALP_HW(NLin), TG_INFO(II)%ALP_EP(2,3), &
           MGa_OS(NLin), MGa_LP(NLin) )

      ! ... Oscillator strengths (NIST) the same for all isotopes:
      Mga_OS = [1.8d0, 1.8d0, 1.8d0]
      !     ... Including weight accounting for isotopic amount
      Mga_OS(1) = Mga_OS(1) *0.7899d0
      Mga_OS(2) = Mga_OS(2) *0.1000d0 
      Mga_OS(3) = Mga_OS(3) *0.1101d0

      WLC = 285.2127d0
      ! ... Position of Mg lines [nm air] 
      !            24Mg               25Mg                    26Mg
      Mga_LP = [WLC, WLC - 0.00019535d0, WLC - 0.00037713d0]
      ALLOCATE( HD(NLin) )
      ! ... convert wavelength to vacuum
      HD(:) = 1.d0/Mga_LP(:)/Mga_LP(:)
      HD(:) = 6.4328d-05 + 2.94981d-08/(1.46d-04 - HD(:)) + &
           2.554d-10/(4.1d-05 - HD(:)) + 1.d0
!                   WRITE(*,*) '    Line centrum in vacuum [nm] '
!                   WRITE(*,*) Mga_LP(:)*HD(:)
      ! ... convert to wavenumber (cm-1)
      Mga_LP(:) = 1.d7/(Mga_LP(:)*HD(:))

      ! ... There is no separation wavelength in this case
      TG_INFO(II)%S_NU = 0.d0 

      ! ... Phase function weights for Mg line (1^S_0 --> 1^P_1)
      !                           E1      E2      E3
      E1 = 1.d0 ; E2 = 0.d0 ; E3 = 1.d0
      TG_INFO(II)%ALP_EP(1,:) = [1.d0, E3, E1]
      TG_INFO(II)%ALP_EP(2,:) = [1.d0, E3, E1]

      TG_INFO(II)%ALP_LI(:) = pi*(e_cr*1.d9) /Mga_LP(:)**2*Mga_OS(:) ! cm^2 * nm
      TG_INFO(II)%ALP_LP(:) = Mga_LP(:)

      Temp = 200.d0
      A_dop = Mga_LP(1)/S_light*DSQRT( 2.d0*k_Bolz*Temp/m_Mg ) 
!      WRITE(*,'(A,2x,e12.4)') '   Doppler line with cm-1 ', A_dop
      A_dop = A_dop/Mga_LP(1)**2*1.d7
!      WRITE(*,'(A,2x,e12.4)') '                       nm ', A_dop

      ! ... Doppler half-width [nm] without SQRT(Temp), which will be included
      !           in routine "st_abs_coeff.f" 
      TG_INFO(II)%ALP_HW(:) = Mga_LP(:)/S_light*DSQRT( 2.d0*k_Bolz/m_Mg ) &
           ! [cm-1] --> [nm]
           /Mga_LP(:)**2*1.d7

    END SUBROUTINE SPI_MGA
!***********************************************************************

END SUBROUTINE ATOMS_SPI
