!***************************************************************************
!
!	SCIATRAN - Version 4.0 - Last modification: 14 Nov 2018
!
!	Copyright (C) - Institute of Remote Sensing,
!			University of Bremen, Germany
!
!***************************************************************************
SUBROUTINE BIAKIMA_INIT (Z_INV, ALT_EXT, *, X_INV, SZA_EXT)
  
  USE GRIDS,      ONLY: NL => NLAYERS, NS => NSOL
  USE PHOTOCHEM,  ONLY: NSP => N_SOL_PC, PCY_LOC, XS_PC
  USE ST_VARS,    ONLY: DO_PC
  USE TYPES,      ONLY: KND

  REAL (KND) Z_INV(NL), X_INV(NS)
  REAL (KND) ALT_EXT(NL + 4), SZA_EXT(NS + 4)
  Optional :: X_INV, SZA_EXT
  ! Local parameters
  CHARACTER (LEN = *), PARAMETER :: R_NAME = 'BIAKIMA_INIT'
  INTEGER    I
  !==================================================================
  CALL ST_TRACK (R_NAME, 1, 'I')

  ! Consistency check
  DO I = 1, NL - 1
     IF(Z_INV(I + 1).LE.Z_INV(I)) THEN
        CALL WRITE_MESSAGE(R_NAME, 'E', 'Array of height levels decreases')
        CALL ST_TRACK (R_NAME, 1, 'O')
        RETURN 1
     END IF
  END DO
  If(Present(X_INV)) Then
     DO I = 1, NS - 1
        IF(X_INV(I + 1).LE.X_INV(I)) THEN
           CALL WRITE_MESSAGE(R_NAME, 'E', &
                'Array of solar zenith angles decreases')
           CALL ST_TRACK (R_NAME, 1, 'O')
           RETURN 1
        END IF
     END DO
  End If

  ! Extrapolation of X-axis:
  ! Auxilliary points are inverted and moved 2 indices up
  ! *****************************************************
  ALT_EXT(3:NL + 2) = Z_INV(1:NL)

  ! Calculation of extrapolated axis-values left and right
  ! ******************************************************
  ALT_EXT(2) = ALT_EXT(3) + ALT_EXT(4) - ALT_EXT(5)
  ALT_EXT(1) = ALT_EXT(2) + ALT_EXT(3) - ALT_EXT(4)
  ALT_EXT(NL + 3) = ALT_EXT(NL + 2) + ALT_EXT(NL + 1) - ALT_EXT(NL)
  ALT_EXT(NL + 4) = ALT_EXT(NL + 3) + ALT_EXT(NL + 2) - ALT_EXT(NL + 1)

  If(Present(X_INV)) Then
     ! Extrapolation of Y-axis:
     ! Auxilliary points are inverted and moved 2 indices up
     ! *****************************************************
     SZA_EXT(3:NS + 2) = X_INV(1:NS)
     
     ! Calculation of extrapolated axis-values left and right
     ! ******************************************************
     SZA_EXT(2) = SZA_EXT(3) + SZA_EXT(4) - SZA_EXT(5)
     SZA_EXT(1) = SZA_EXT(2) + SZA_EXT(3) - SZA_EXT(4)
     SZA_EXT(NS + 3) = SZA_EXT(NS + 2) + SZA_EXT(NS + 1) - SZA_EXT(NS)
     SZA_EXT(NS + 4) = SZA_EXT(NS + 3) + SZA_EXT(NS + 2) - SZA_EXT(NS + 1)
  End If
  
  IF(.NOT.DO_PC) THEN
     CALL ST_TRACK (R_NAME, 1, 'O')
     RETURN
  END IF
  
  ! Consistency check for photochemistry
  ALLOCATE(PCY_LOC(NSP + 4)) ! A

  DO I = 1, NSP - 1
     IF(XS_PC(I + 1).GE.XS_PC(I)) THEN
        CALL WRITE_MESSAGE(R_NAME, 'E', 'Photochemistry: ' &
             //'Array of solar zenith angles increases')
        CALL ST_TRACK (R_NAME, 1, 'O')
        RETURN 1
     END IF
  END DO

  ! Extrapolation of photochemical Y-axis:
  ! Auxilliary points are moved 2 indices up
  ! ****************************************
  PCY_LOC(3:NSP + 2) = XS_PC(NSP:1:-1)


  ! Calculation of extrapolated axis-values left and right
  ! ******************************************************
  PCY_LOC(2) = PCY_LOC(3) + PCY_LOC(4) - PCY_LOC(5)
  PCY_LOC(1) = PCY_LOC(2) + PCY_LOC(3) - PCY_LOC(4)
  PCY_LOC(NSP + 3) = PCY_LOC(NSP + 2) + PCY_LOC(NSP + 1) - PCY_LOC(NSP)
  PCY_LOC(NSP + 4) = PCY_LOC(NSP + 3) + PCY_LOC(NSP + 2) - PCY_LOC(NSP + 1)

  CALL ST_TRACK (R_NAME, 1, 'O')
  RETURN
!***************************************************************************
END SUBROUTINE BIAKIMA_INIT

!***************************************************************************
SUBROUTINE BIAKIMA_X (N_X, X_LOC, N_Y, X_RAD, RAD)

  USE TYPES,      ONLY: KND
  
  INTEGER     N_X, N_Y
  REAL (KND)  X_LOC(N_X + 4), X_RAD(N_X, N_Y, 4), RAD(N_X, N_Y)
  ! Local parameters
  CHARACTER (LEN = *), PARAMETER :: R_NAME = 'BIAKIMA_X'
  INTEGER     I, IDX_Y
  REAL (KND)  Hf
  REAL (KND), DIMENSION (N_X + 4) :: FUNC_X, ST, T
  !==================================================================
!  CALL ST_TRACK (R_NAME, 1, 'I')

  DO IDX_Y = 1, N_Y
     ! Extrapolation of function:
     ! Auxilliary points are inverted and moved 2 indices up
     ! *****************************************************
     FUNC_X(3:N_X + 2) = RAD(N_X:1:-1,N_Y - IDX_Y + 1)
     
     ! Caculation of slopes of curves
     ! ******************************
     DO I = 3, N_X + 1
        ST(I) = (FUNC_X(I + 1) - FUNC_X(I))/(X_LOC(I + 1) - X_LOC(I))
     END DO
            
     ST(2) = 2.0D0*ST(3) - ST(4)
     ST(1) = 2.0D0*ST(2) - ST(3)
     ST(N_X + 2) = 2.0D0*ST(N_X + 1) - ST(N_X)
     ST(N_X + 3) = 2.0D0*ST(N_X + 2) - ST(N_X + 1)
            
     ! Calculation of all polynomial slopes
     ! ************************************
     DO I = 3, N_X + 2
        IF(ST(I - 2).EQ.ST(I - 1).AND.ST(I).EQ.ST(I + 1)) THEN
           T(I) = 0.5D0*(ST(I + 1) + ST(I))
        ELSE
           T(I) = (ABS(ST(I + 1) - ST(I))*ST(I - 1) + &
                ABS(ST(I - 1) - ST(I - 2))*ST(I))/ &
                (ABS(ST(I + 1) - ST(I)) + ABS(ST(I - 1) - ST(I - 2)))
        END IF
     END DO
            
     ! Calculation of polynomial coefficients
     ! **************************************            
     DO I = 1, N_X - 1
        Hf = X_LOC(I+3) - X_LOC(I+2)
        X_RAD(I,IDX_Y,1) = FUNC_X(I + 2)
        X_RAD(I,IDX_Y,2) = T(I + 2)
        X_RAD(I,IDX_Y,3) = (3.0D0*ST(I+2) - 2.0D0*T(I+2) -T(I+3))/Hf
        X_RAD(I,IDX_Y,4) = (T(I+2) + T(I+3) - 2.0D0*ST(I+2))/Hf/Hf
     END DO

     X_RAD(N_X,IDX_Y,1) = FUNC_X(N_X + 2) ! v1.2.4
     X_RAD(N_X,IDX_Y,2:4) = 0.0D0 ! v1.2.9

  END DO                    ! IDX_Y
  
!  CALL ST_TRACK (R_NAME, 1, 'O')
  RETURN
!***************************************************************************
END SUBROUTINE BIAKIMA_X
      
!***************************************************************************
SUBROUTINE BIAKIMA_Y (X_AX, Y_LOC, X_NEW, X_RAD, Y_ALL)

  USE GRIDS,       ONLY: NSOL
  USE TYPES,       ONLY: KND

  ! 1st index- SZA, 2nd - alts, 3rd - coeffs
  REAL (KND)  X_RAD(:,:,:), Y_ALL(:,:,:) 
  REAL (KND)  Y_LOC(NSOL + 4)
  REAL (KND)  X_NEW, X_AX
  ! Local parameters
  CHARACTER (LEN=*), PARAMETER :: R_NAME = 'BIAKIMA_Y'
  INTEGER            I_N, I, IDX_Y
  REAL (KND), DIMENSION (NSOL + 4) :: FUNC_Y, ST, T
  REAL (KND)  DELTA, Hf, DST_1, DST_2
  !==================================================================
!  CALL ST_TRACK (R_NAME, 1, 'I')

  DELTA = X_NEW - X_AX 
  DO I_N = 1, SIZE(X_RAD,2)
     DO IDX_Y = 1, NSOL
        FUNC_Y(IDX_Y + 2) = &
             X_RAD(IDX_Y,I_N,1) + DELTA*(X_RAD(IDX_Y,I_N,2) + &
             DELTA*(X_RAD(IDX_Y,I_N,3) + X_RAD(IDX_Y,I_N,4)*DELTA))
     END DO
     
     IF(NSOL == 1) THEN
        Y_ALL(1,I_N,1) = FUNC_Y(3) ;  Y_ALL(1,I_N,2:4) = 0.0D0 
        CYCLE
     END IF
          
     ! Caculation of slopes of curves
     ! ******************************
     DO I = 3, NSOL + 1
        ST(I) = (FUNC_Y(I + 1) - FUNC_Y(I))/(Y_LOC(I + 1) - Y_LOC(I))
     END DO
            
     ST(2) = 2.0D0*ST(3) - ST(4)
     ST(1) = 2.0D0*ST(2) - ST(3)
     ST(NSOL + 2) = 2.0D0*ST(NSOL + 1) - ST(NSOL)
     ST(NSOL + 3) = 2.0D0*ST(NSOL + 2) - ST(NSOL + 1)

     ! Calculation of all polynomial slopes
     ! ************************************
     DO I = 3, NSOL + 2
        DST_1 = ABS(ST(I + 1) - ST(I))
        DST_2 = ABS(ST(I - 1) - ST(I - 2))
        IF((DST_1 + DST_2).EQ.0.0D0) THEN
           T(I) = 0.5D0*(ST(I + 1) + ST(I))
        ELSE
           T(I) = (DST_1*ST(I - 1) + DST_2*ST(I))/(DST_1 + DST_2)
        END IF
     END DO

     ! Calculation of polynomial coefficients
     ! **************************************
     DO I = 1, NSOL - 1
        Hf = Y_LOC(I + 3) - Y_LOC(I + 2)
        Y_ALL(I,I_N,1) = FUNC_Y(I + 2)
        Y_ALL(I,I_N,2) = T(I + 2)
        Y_ALL(I,I_N,3) = (3.0D0*ST(I + 2) - 2.0D0*T(I + 2) - T(I + 3))/Hf
        Y_ALL(I,I_N,4) = (T(I + 2) + T(I + 3) - 2.0D0*ST(I + 2))/Hf/Hf
     END DO

     Y_ALL(NSOL,I_N,1) = FUNC_Y(NSOL + 2)
     Y_ALL(NSOL,I_N,2:4) = 0.0D0
  END DO  ! I_N
  
  
!  CALL ST_TRACK (R_NAME, 1, 'O')
  RETURN 
!***************************************************************************
END SUBROUTINE BIAKIMA_Y
!***************************************************************************
SUBROUTINE BIAKIMA_Y_V2 (X_AX, Y_LOC, X_NEW, X_RAD, Y_ALL)

  USE GRIDS,   ONLY: NSOL
  USE ST_VARS, ONLY: MSF_INT
  USE TYPES,   ONLY: KND
  USE IEEE_ARITHMETIC
  
  REAL (KND)  X_RAD(:,:,:,:), Y_ALL(:,:,:,:) 
  REAL (KND)  Y_LOC(NSOL + 4)
  REAL (KND)  X_NEW, X_AX
  ! Local parameters
  CHARACTER (LEN=*), PARAMETER :: R_NAME = 'BIAKIMA_Y'
  INTEGER            I_N, I, IDX_Y, Ks
  REAL (KND), DIMENSION (NSOL + 4) :: FUNC_Y, ST, T
  REAL (KND)  DELTA, Hf, DST_1, DST_2
  !==================================================================
!  CALL ST_TRACK (R_NAME, 1, 'I')

  DELTA = X_NEW - X_AX 
  DO I_N = 1, SIZE(X_RAD,2)
     DO Ks = 1, SIZE(X_RAD,4)
        DO IDX_Y = 1, NSOL
           FUNC_Y(IDX_Y + 2) = &
                X_RAD(IDX_Y,I_N,1,Ks) + DELTA*(X_RAD(IDX_Y,I_N,2,Ks) + &
                DELTA*(X_RAD(IDX_Y,I_N,3,Ks) + X_RAD(IDX_Y,I_N,4,Ks)*DELTA))
        END DO

        If (Ks == 1 .AND. MSF_INT == "LogLin") &
             FUNC_Y(3:NSOL+2) = Log(MAX(FUNC_Y(3:NSOL+2),Tiny(1.0D0)))
           
        IF(NSOL == 1) THEN
           Y_ALL(Ks,I_N,1,1) = FUNC_Y(3) ;  Y_ALL(Ks,I_N,2:4,1) = 0.0D0 
           CYCLE
        END IF
        
        ! Caculation of slopes of curves
        ! ******************************
        DO I = 3, NSOL + 1
           ST(I) = (FUNC_Y(I + 1) - FUNC_Y(I))/(Y_LOC(I + 1) - Y_LOC(I))
        END DO
        
        Select Case (MSF_INT)
        Case ("Akima")
           ST(2) = 2.0D0*ST(3) - ST(4)
           ST(1) = 2.0D0*ST(2) - ST(3)
           ST(NSOL + 2) = 2.0D0*ST(NSOL + 1) - ST(NSOL)
           ST(NSOL + 3) = 2.0D0*ST(NSOL + 2) - ST(NSOL + 1)
           
           ! Calculation of all polynomial slopes
           ! ************************************
           DO I = 3, NSOL + 2
              DST_1 = ABS(ST(I + 1) - ST(I))
              DST_2 = ABS(ST(I - 1) - ST(I - 2))
              IF((DST_1 + DST_2).EQ.0.0D0) THEN
                 T(I) = 0.5D0*(ST(I + 1) + ST(I))
              ELSE
                 T(I) = (DST_1*ST(I - 1) + DST_2*ST(I))/(DST_1 + DST_2)
              END IF
           END DO
           
           ! Calculation of polynomial coefficients
           ! **************************************
           DO I = 1, NSOL - 1
              Hf = Y_LOC(I + 3) - Y_LOC(I + 2)
              Y_ALL(Ks,I_N,1,I) = FUNC_Y(I + 2)
              Y_ALL(Ks,I_N,2,I) = T(I + 2)
              Y_ALL(Ks,I_N,3,I) = (3.0D0*ST(I + 2) - 2.0D0*T(I + 2) - T(I + 3))/Hf
              Y_ALL(Ks,I_N,4,I) = (T(I + 2) + T(I + 3) - 2.0D0*ST(I + 2))/Hf/Hf
           END DO
           
           Y_ALL(Ks,I_N,1,NSOL) = FUNC_Y(NSOL + 2)
           Y_ALL(Ks,I_N,2:4,NSOL) = 0.0D0
        Case ("LogLin")
           ST(NSOL + 2) = 0.d0 
           FORALL (I = 1:NSOL)
              Y_ALL(Ks,I_N,1,I) = FUNC_Y(I + 2)
              Y_ALL(Ks,I_N,2,I) = ST(I + 2)
           END FORALL
           Y_ALL(Ks,I_N,3:4,:) = 0.0D0
        Case ("Spline")
           ForAll(I = 1:NSOL) Y_ALL(Ks,I_N,1,I) = FUNC_Y(I + 2)
           Call spline (NSOL, Y_LOC(3:NSOL - 2), Y_ALL(Ks,I_N,1,:) , Y_ALL(Ks,I_N,2,:),&
                Y_ALL(Ks,I_N,3,:), Y_ALL(Ks,I_N,4,:))
        End Select
     END DO  ! Ks
  END DO  ! I_N
  
  !  CALL ST_TRACK (R_NAME, 1, 'O')
  RETURN 
!***************************************************************************
END SUBROUTINE BIAKIMA_Y_V2
!***************************************************************************
