module cs_table
  use ABSORBERS

  implicit none

  type table_t
    character(256)            :: name

    character(256)            :: absorber
    integer                   :: absorber_id
    integer                   :: np, nt, nw
    character(256)            :: file_desc
    character(256)            :: file_pres
    character(256)            :: file_temp
    character(256)            :: file_wave
    character(256)            :: file_cros
    double precision, pointer :: pres(:)
    double precision, pointer :: temp(:, :)
    double precision, pointer :: wave(:)
    double precision, pointer :: cros(:, :, :)
    double precision          :: line_wing_cut_off

    type(table_t), pointer    :: next
  end type

  integer                                   :: table_input_cnt  = 0
  character(256), dimension(:), allocatable :: table_input_desc
  logical                                   :: table_use_table  = .false.
  logical                                   :: table_gen_table  = .false.

  type table_list
    type(table_t), pointer :: table => null()
  end type
  type(table_list), allocatable :: table_lists(:)

  contains

    subroutine table_init(table, file_desc)
      implicit none

      type(table_t), pointer :: table
      character(*)      :: file_desc

      character(256)    :: name
      character(256)    :: absorber
      integer           :: absorber_id
      integer           :: np, nt, nw
      character(256)    :: file_pres
      character(256)    :: file_temp
      character(256)    :: file_wave
      character(256)    :: file_cros
      double precision  :: line_wing_cut_off

      integer :: idx

      namelist / table_desc / name,               &
                              absorber_id,        &
                              absorber,           &
                              np,                 &
                              nt,                 &
                              nw,                 &
                              file_pres,          &
                              file_temp,          &
                              file_wave,          &
                              file_cros,          &
                              line_wing_cut_off

      allocate(table)

      table%file_desc = file_desc

      open(unit=111, file=trim(table%file_desc))
      read(111, nml=table_desc)
      close(unit=111)

      table%name              = name
      table%absorber          = absorber
      table%absorber_id       = absorber_id
      table%np                = np
      table%nt                = nt
      table%nw                = nw

      idx = index(file_desc, "/", .true.)
      table%file_pres         = file_desc(:idx) // file_pres
      table%file_temp         = file_desc(:idx) // file_temp
      table%file_wave         = file_desc(:idx) // file_wave
      table%file_cros         = file_desc(:idx) // file_cros
      table%line_wing_cut_off = line_wing_cut_off
    end subroutine table_init


    subroutine table_free(table)
      implicit none

      type(table_t), pointer :: table

      deallocate(table)
    end subroutine table_free


    subroutine table_read(table)
      implicit none

      type(table_t) :: table

      allocate(table%pres(table%np))
      allocate(table%temp(table%nt, table%np))
      allocate(table%wave(table%nw))
      allocate(table%cros(table%nw, table%nt, table%np))

      open(unit=111, file=trim(table%file_pres), access="stream")
      read(111) table%pres
      close(unit=111)

      open(unit=111, file=trim(table%file_temp), access="stream")
      read(111) table%temp
      close(unit=111)

      open(unit=111, file=trim(table%file_wave), access="stream")
      read(111) table%wave
      close(unit=111)

      open(unit=111, file=trim(table%file_cros), access="stream")
      read(111) table%cros
      close(unit=111)
    end subroutine table_read


    integer function bin_search(data, value)
      implicit none

      double precision :: data(:)
      double precision :: value

      integer :: center
      integer :: left
      integer :: right

      left  = 1
      right = size(data)

      do while(right - left .gt. 1)
        center = (left + right) / 2
        if(data(center) .le. value) then
          left = center
        else
          right = center
        end if
      end do

      bin_search = left
    end function bin_search


    double precision function interpolate(data_x, data_y, data_z, x, y, e)
      implicit none

      double precision  :: data_x(9)
      double precision  :: data_y(9)
      double precision  :: data_z(9)
      double precision  :: x
      double precision  :: y
      integer           :: e

      integer           :: info
      integer           :: ipiv(9)
      double precision  :: matr(9, 9)
      double precision  :: vect(9)

      matr(:, 1) = 1.0d0
      matr(:, 2) =                               data_x(:)
      matr(:, 3) =                     data_x(:)*data_x(:)
      matr(:, 4) = data_y(:)
      matr(:, 5) = data_y(:)                    *data_x(:)
      matr(:, 6) = data_y(:)          *data_x(:)*data_x(:)
      matr(:, 7) = data_y(:)*data_y(:)
      matr(:, 8) = data_y(:)*data_y(:)*data_x(:)
      matr(:, 9) = data_y(:)*data_y(:)*data_x(:)*data_x(:)

      vect(:) = data_z(:)

      call dgesv(9, 1, matr, 9, ipiv, vect, 9, info)
      e = info
      if(info .ne. 0) then
        write(0, *) "interpolate dgesv error: info=", info
      end if

      interpolate = vect(1)     + vect(2)    *x + vect(3)    *x*x &
                  + vect(4)  *y + vect(5)  *y*x + vect(6)*y  *x*x &
                  + vect(7)*y*y + vect(8)*y*y*x + vect(9)*y*y*x*x
    end function interpolate


    subroutine table_interpolate(table_list, p, t, w, c, e)
      implicit none

      type(table_t), pointer  :: table_list
      double precision        :: p, t, w, c
      integer                 :: e

      type(table_t), pointer  :: table
      integer                 :: iw, ip, it1, it2, it3, iww
      double precision        :: pbuf(9), tbuf(9), cbuf(9)

      e = 0
      c = 0.0d0

      table => table_list
      do while(associated(table))
        if((w .ge. table%wave(1)) .and. (w .le. table%wave(table%nw))) then
          exit
        end if
        table => table%next
      end do
      if(.not. associated(table)) then
        e = -100
        return
      end if

      if((p .lt. table%pres(1)) .or. (p .gt. table%pres(table%np))) then
        e = -200
        return
      end if

      iw = bin_search(table%wave, w)
      if((w - table%wave(iw)) .gt. (table%wave(iw+1) - w)) then
        iw = iw + 1
      end if

      ip = bin_search(table%pres, p) - 1
      ip = max(1, min(ip, table%np-2))
      if((t .lt. table%temp(1, ip+0)) .or. (t .gt. table%temp(table%nt, ip+0))) then
        e = -300
        return
      end if
      if((t .lt. table%temp(1, ip+1)) .or. (t .gt. table%temp(table%nt, ip+1))) then
        e = -301
        return
      end if
      if((t .lt. table%temp(1, ip+2)) .or. (t .gt. table%temp(table%nt, ip+2))) then
        e = -302
        return
      end if
      it1 = bin_search(table%temp(:, ip+0), t) - 1
      it2 = bin_search(table%temp(:, ip+1), t) - 1
      it3 = bin_search(table%temp(:, ip+2), t) - 1
      it1 = max(1, min(it1, table%nt-2))
      it2 = max(1, min(it2, table%nt-2))
      it3 = max(1, min(it3, table%nt-2))
      pbuf(1:3) = table%pres(ip:ip+2)
      pbuf(4:6) = table%pres(ip:ip+2)
      pbuf(7:9) = table%pres(ip:ip+2)
      tbuf(1) = log(table%temp(it1+0, ip+0))
      tbuf(2) = log(table%temp(it2+0, ip+1))
      tbuf(3) = log(table%temp(it3+0, ip+2))
      tbuf(4) = log(table%temp(it1+1, ip+0))
      tbuf(5) = log(table%temp(it2+1, ip+1))
      tbuf(6) = log(table%temp(it3+1, ip+2))
      tbuf(7) = log(table%temp(it1+2, ip+0))
      tbuf(8) = log(table%temp(it2+2, ip+1))
      tbuf(9) = log(table%temp(it3+2, ip+2))
      cbuf(1) = log(table%cros(iw, it1+0, ip+0) + 1.0d-50)
      cbuf(2) = log(table%cros(iw, it2+0, ip+1) + 1.0d-50)
      cbuf(3) = log(table%cros(iw, it3+0, ip+2) + 1.0d-50)
      cbuf(4) = log(table%cros(iw, it1+1, ip+0) + 1.0d-50)
      cbuf(5) = log(table%cros(iw, it2+1, ip+1) + 1.0d-50)
      cbuf(6) = log(table%cros(iw, it3+1, ip+2) + 1.0d-50)
      cbuf(7) = log(table%cros(iw, it1+2, ip+0) + 1.0d-50)
      cbuf(8) = log(table%cros(iw, it2+2, ip+1) + 1.0d-50)
      cbuf(9) = log(table%cros(iw, it3+2, ip+2) + 1.0d-50)

      c = exp(interpolate(pbuf, tbuf, cbuf, p, log(t), e)) - 1.0d-50
      if(c .lt. 0.0d0) then
        c = 0.0d0
      end if

      if(c .ne. c) then
        write(*, *) w, p, t, c
        write(*, *) iw, ip, it1, it2, it3
        write(*, *) pbuf
        write(*, *) tbuf
        write(*, *) cbuf
        write(*, *) table%cros(iw, it1+0, ip+0)
        write(*, *) table%cros(iw, it2+0, ip+1)
        write(*, *) table%cros(iw, it3+0, ip+2)
        write(*, *) table%cros(iw, it1+1, ip+0)
        write(*, *) table%cros(iw, it2+1, ip+1)
        write(*, *) table%cros(iw, it3+1, ip+2)
        write(*, *) table%cros(iw, it1+2, ip+0)
        write(*, *) table%cros(iw, it2+2, ip+1)
        write(*, *) table%cros(iw, it3+2, ip+2)
        stop
      end if
    end subroutine table_interpolate


    subroutine table_generate(table)
      USE ABSORBERS,      ONLY: N_LBL_MOL, N_ST_MOL,   &
                                MIX_ABS_START, ST_GAS_IDX
      USE INFO,           ONLY: TG_INFO
      USE LOG_PARS,       ONLY: C_UNIT, EU, LUNIT, LOG_LEVEL,           &
                                N_SSO
      USE LBL,            ONLY: C, MAX_ISO,     &
                                LBL_WING_CUTOFF,  &
                                SEG_WL, NSEGS_LBL,          &
                                N_ISO,            &
                                N_TEMP_SETS, PT_SCALE
      USE TYPE_VARS,      ONLY: KND
      USE GRIDS,          ONLY: NLAYERS, zzz__prof => ZZZ
      USE PROFILES,       ONLY: temp_prof => TEMP

      implicit none

      type(table_t), pointer        :: table

      double precision, allocatable :: pres(:)
      double precision, allocatable :: temp(:, :)
      double precision, allocatable :: wave(:)
      character(len=*), parameter   :: hitran_dir = "/home/mhilker/Documents/sandkasten/DATA_BASES/HITRAN_2008/"
      integer                       :: i, j, p, t ,w, istart, iend
      logical                       :: lines_found
      integer                       :: counter

      namelist / table_pres / pres
      namelist / table_temp / temp
      namelist / table_wave / wave

      allocate(table%pres(table%np))
      allocate(table%temp(table%nt, table%np))
      allocate(table%wave(table%nw))
      allocate(table%cros(table%nw, table%nt, table%np))

      allocate(pres(table%np))
      allocate(temp(table%nt, table%np))
      allocate(wave(table%nw))

      open(unit=111, file=trim(table%file_desc))
      read(111, nml=table_pres)
      read(111, nml=table_temp)
      read(111, nml=table_wave)
      close(unit=111)

      table%pres(1:table%np)             = pres(1:table%np)
      table%temp(1:table%nt, 1:table%np) = temp(1:table%nt, 1:table%np)
      table%wave(1:table%nw)             = wave(1:table%nw)

      deallocate(pres)
      deallocate(temp)
      deallocate(wave)

      table%cros(1:table%nw, 1:table%nt, 1:table%np) = 0.0d0

      log_level(:)                  = N_SSO
      c_unit                        = 22
      lunit                         = 6
      eu                            = 10

      nsegs_lbl                     = 1
      allocate(seg_wl(2,nsegs_lbl))
      seg_wl(1, 1)                  = 1.0e7/table%wave(table%nw)
      seg_wl(2, 1)                  = 1.0e7/table%wave(1)

      lbl_wing_cutoff               = table%line_wing_cut_off
      TG_INFO(:)%FM_FLG             = .false.
      TG_INFO(table%absorber_id)%FM_FLG = .true.

      nlayers                       = table%np * table%nt
      allocate(zzz__prof(nlayers))
      allocate(temp_prof(nlayers))
      do i=1, nlayers
        zzz__prof(i)  = i
      end do
      do i=1, table%np
        do j=1, table%nt
          temp_prof((i-1)*table%nt+j) = table%temp(j, i)
        end do
      end do

      call Read_HITRAN_Pars(hitran_dir, *110)
      call st_part_sums(*111)

!      if(ngas_lbl .ne. 1) then
!        write(*, *) "warning: ngas_lbl=", ngas_lbl
!      end if

      counter = 0
      write(*,'(A,T20,I3,A)') trim(TG_INFO(table%absorber_id)%Name), 0, "%"
!$omp parallel                                                         &
!$omp private(w, p, t, lines_found, istart, iend)                      &
!$omp shared(table, counter)

!$omp do schedule(dynamic)
      do w=1, table%nw
        call line_search(1, table%wave(w), lines_found, istart, iend)
        if(lines_found) then
          do p=1, table%np
            do t=1, table%nt
              call voigt_shape(                             &
                    istart,                                 &
                    iend,                                   &
                    table%absorber_id,                      &
                    table%wave(w),                          &
                    (p-1)*table%nt+t, 1,                    & 
                  !  Q_RATIO(1, (p-1)*table%nt+t, 1, :),     &
                    table%temp(t, p),                       &
                    table%pres(p)/100.0e0,                  &
                    table%cros(w, t, p),                    &
                    *112                                    &
              )
112           continue
            end do
          end do
        end if

!$omp critical
        counter = counter + 1
        write(*,*) achar(27)//"[2A"
        write(*,'(A,T20,I3,A)') trim(TG_INFO(table%absorber_id)%Name), int(real(counter)/table%nw*100.0), "%"
!$omp end critical

      end do
!$omp end do
!$omp end parallel

      goto 999

110   write(*, *) "error Read_HITRAN_Pars"
      goto 999

111   write(*, *) "error st_part_sums"
      goto 999

999   continue

!      deallocate(do_temp_wf)
      deallocate(zzz__prof)
      deallocate(temp_prof)
      deallocate(seg_wl)

    end subroutine table_generate


    subroutine table_test(table, cnt)
      USE ABSORBERS,      ONLY: N_LBL_MOL, N_ST_MOL,   &
                                MIX_ABS_START,    &
                                ST_GAS_IDX
      USE INFO,           ONLY: TG_INFO
      USE LOG_PARS,       ONLY: C_UNIT, EU, LUNIT, LOG_LEVEL,           &
                                N_SSO
      USE LBL,            ONLY: C, MAX_ISO,      &
                                LBL_WING_CUTOFF,   &
                                SEG_WL, NSEGS_LBL,          &
                                N_ISO,             &
                                N_TEMP_SETS, PT_SCALE
      USE TYPE_VARS,      ONLY: KND
      USE GRIDS,          ONLY: NLAYERS, zzz__prof => ZZZ
      USE PROFILES,       ONLY: temp_prof => TEMP, pres_prof => PRESS

      implicit none

      type(table_t), pointer        :: table
      integer                       :: cnt

      character(len=*), parameter   :: hitran_dir = "/home/mhilker/Documents/sandkasten/DATA_BASES/HITRAN_2008/"
      integer                       :: i, p, w, istart, iend
      logical                       :: lines_found
      double precision, allocatable :: buffer1(:, :), buffer2(:, :)
      integer                       :: error, seed_s, clck1, clck2, clck3
      integer, allocatable          :: seed_v(:)
      double precision              :: pmin, pmax, tmin, tmax, wmin, wmax, r
      integer                       :: counter

      log_level(:)                  = N_SSO
      c_unit                        = 22
      lunit                         = 6
      eu                            = 10

      nsegs_lbl                     = 1
      allocate(seg_wl(2,nsegs_lbl))
      seg_wl(1, 1)                  = 1.0e7/table%wave(table%nw)
      seg_wl(2, 1)                  = 1.0e7/table%wave(1)

      lbl_wing_cutoff               = table%line_wing_cut_off
      TG_INFO(:)%FM_FLG             = .false.
      TG_INFO(table%absorber_id)%FM_FLG = .true.

      nlayers                       = cnt
      allocate(zzz__prof(nlayers))
      allocate(temp_prof(nlayers))
      allocate(pres_prof(nlayers))
      do i=1, nlayers
        zzz__prof(i) = i
      end do

      wmin = table%wave(1)
      wmax = table%wave(table%nw)
      pmin = table%pres(1)
      pmax = table%pres(table%np)
      tmin = minval(table%temp(1, :))
      tmax = maxval(table%temp(table%nt, :))

      write(*, *) "wmin=", wmin
      write(*, *) "wmax=", wmax
      write(*, *) "pmin=", pmin
      write(*, *) "pmax=", pmax
      write(*, *) "tmin=", tmin
      write(*, *) "tmax=", tmax

      call random_seed(size=seed_s)
      allocate(seed_v(seed_s))
      call random_seed(get=seed_v)
      call system_clock(clck1, clck2, clck3)
      do i=1, seed_s
        seed_v(i) = seed_v(i) + clck1
      end do
      call random_seed(put=seed_v)
      deallocate(seed_v)

      do i=1, nlayers
        call random_number(r)
        pres_prof(i) = pmin + r * (pmax - pmin)
        call random_number(r)
        temp_prof(i) = tmin + r * (tmax - tmin)
        p = bin_search(table%pres, pres_prof(i)) - 1
        p = max(1, min(p, table%np-2))
        do while(      (temp_prof(i) .lt. table%temp(1, p+0)) .or. (temp_prof(i) .gt. table%temp(table%nt, p+0)) &
                  .or. (temp_prof(i) .lt. table%temp(1, p+1)) .or. (temp_prof(i) .gt. table%temp(table%nt, p+1)) &
                  .or. (temp_prof(i) .lt. table%temp(1, p+2)) .or. (temp_prof(i) .gt. table%temp(table%nt, p+2)) &
        )
          call random_number(r)
          temp_prof(i) = tmin + r * (tmax - tmin)
        end do
      end do

      call Read_HITRAN_Pars(hitran_dir, *110)
!      if(ngas_lbl .ne. 1) then
!        write(*, *) "warning: ngas_lbl=", ngas_lbl
!        goto 110
!      end if

      call st_part_sums(*111)

      allocate(buffer1(table%nw, nlayers))
      allocate(buffer2(table%nw, nlayers))

      counter = 0
      write(*,'(A,T20,I3,A)') trim(TG_INFO(table%absorber_id)%Name), 0, "%"
!$omp parallel                                                                &
!$omp private(i, w, lines_found, istart, iend, error)                         &
!$omp shared(table, buffer1, buffer2, nlayers, temp_prof, pres_prof, counter)

!$omp do schedule(dynamic)
      do i=1, nlayers
        do w=1, table%nw
          buffer1(w, i) = 0.00d0
          buffer2(w, i) = 0.00d0
          call line_search(1, table%wave(w), lines_found, istart, iend)
          if(lines_found) then
            call voigt_shape(                             &
                  istart,                                 &
                  iend,                                   &
                  table%absorber_id,                      &
                  table%wave(w),                          &
                  i, 1,                                   &
                  !Q_RATIO(1, i, 1, :),                    &
                  temp_prof(i),                           &
                  pres_prof(i)/100.0e0,                   &
                  buffer1(w, i),                          &
                  *112                                    &
            )
112         continue
          end if
          call table_interpolate(table, pres_prof(i), temp_prof(i), table%wave(w), buffer2(w, i), error)
          if(error .ne. 0) then
            if(error .gt. 0) then
              buffer2(w, i) = -1000 - error
            else
              buffer2(w, i) = error
            end if
          end if
        end do

!$omp critical
        counter = counter + 1
        write(*,*) achar(27)//"[2A"
        write(*,'(A,T20,I3,A)') trim(TG_INFO(table%absorber_id)%Name), int(real(counter)/nlayers*100.0), "%"
!$omp end critical

      end do
!$omp  end do
!$omp  end parallel


      open(unit=111, file="table_test_wave.out", access="stream")
      write(111) table%wave
      close(unit=111)
      open(unit=111, file="table_test_pres.out", access="stream")
      write(111) pres_prof
      close(unit=111)
      open(unit=111, file="table_test_temp.out", access="stream")
      write(111) temp_prof
      close(unit=111)
      open(unit=111, file="table_test_cros.hit", access="stream")
      write(111) buffer1
      close(unit=111)
      open(unit=111, file="table_test_cros.tab", access="stream")
      write(111) buffer2
      close(unit=111)

      goto 999

110   write(*, *) "error Read_HITRAN_Pars"
      goto 999

111   write(*, *) "error st_part_sums"
      goto 999

999   continue

      deallocate(buffer1)
      deallocate(buffer2)
      deallocate(zzz__prof)
      deallocate(temp_prof)
      deallocate(pres_prof)
      deallocate(seg_wl)

    end subroutine table_test


    subroutine table_write(table)
      implicit none

      type(table_t) :: table

      open(unit=111, file=trim(table%file_pres), access="stream")
      write(111) table%pres
      close(unit=111)

      open(unit=111, file=trim(table%file_temp), access="stream")
      write(111) table%temp
      close(unit=111)

      open(unit=111, file=trim(table%file_wave), access="stream")
      write(111) table%wave
      close(unit=111)

      open(unit=111, file=trim(table%file_cros), access="stream")
      write(111) table%cros
      close(unit=111)
    end subroutine table_write


    subroutine table_list_free(root)
      implicit none

      type(table_t), pointer :: root

      type(table_t), pointer :: tmp1, tmp2

      tmp1 => root
      do while(associated(tmp1))
        tmp2 => tmp1
        tmp1 => tmp1%next
        call table_free(tmp2)
      end do
    end subroutine table_list_free


    subroutine table_list_add(root, tab)
      implicit none

      type(table_t), pointer :: root
      type(table_t), pointer :: tab

      type(table_t), pointer :: tmp

      tab%next => null()

      if(.not. associated(root)) then
        root => tab
      else
        tmp => root
        do while(associated(tmp%next))
          tmp => tmp%next
        end do
        tmp%next => tab
      end if

    end subroutine table_list_add


    subroutine table_list_print(root)
      implicit none

      type(table_t), pointer :: root

      type(table_t), pointer :: tmp

      tmp => root
      do while(associated(tmp))
        write(*, *) tmp%name
        tmp => tmp%next
      end do

    end subroutine table_list_print

end module cs_table
