!********************************************************************
!
!       SCIATRAN - Version 4.7 - Last modification: 16 Aug 2023
!       Copyright (C) - Institute of Remote Sensing,
!                       University of Bremen, Germany
!
!*********************************************************************
       
!      Subroutine to calculate the Stokes vector of radiation field
!  in the single scattering approximation in the plane-parallel geometry.
!      It is used also in the framework of the single scattering
!  correction algorithm if RAD_FIELD = .TRUE.
!   !!! It is assumed that incident radiation is unpolarized !!!
!      --------------------------------------------------------
   SUBROUTINE GPP_SS_RF(JPOS, JJESFT, POS, ALBEDO, E_W, OSM, &
        LOA, SSB, TE, DELs, FLU_int, *) 
!     ... Called by "st_dom_tau.f"  if SSC_RF == .TRUE.

     USE AEROSOLS,    ONLY: LA_N, LA_S, LA_E, Dwr_mi, N_dwr, Mie_layer
     USE CLOUDS,      ONLY: DO_CLOUD, N_CLD_LAYERS, J_TOP, J_BOT, &
                            Prof_Cpar, SCAT_ANG
     USE CLD_INP,     ONLY: Phm_const, N_ref_db, Ref_db, N_ref_ice, Ref_ice,  &
                            Deriv_Phc, Nphr, Use_Ydb_SCM, Ydb_ang, MLCS
     USE EMISSION,    ONLY: N_AT, Emi_MOMS
     USE GRIDS,       ONLY: NL => NLAYERS, DZ, DZR, ZZZ, NSOL
     USE GT_POLYN,    ONLY: Plm_ss, NLEGEN
     USE INTERN_DIMS, ONLY: Nvs, Nm_vs, Leg_aer => NLG_ALL_A, NL_AER, &
                            NL_Cld, Nm_ice, Leg_cld => NLG_ALL_C, &
                            Nlg_Ray, NL_pla, Nf_pla, NL_soot
     USE LOG_PARS,    ONLY: ERROR_TXT, N_ESO, N_DSO, LOG_LEVEL, EU
     USE NODES,       ONLY: N_NODE, MJU_NODE, MJU_MINUS, MJU_PLUS, N_PLUS,  &
                            N_MINUS
     USE PS_INTERN,   ONLY: COS_SCAT 
     USE RTM_OUTPUT,  ONLY: N_OP
     USE ST_VARS,     ONLY: DO_AEROSOL, PIE, DO_PLANCK, &
                            VST, Emi_FLG
     USE TOOLS,       ONLY: FIND_IDX
     USE TYPE_VARS,   ONLY: KND, DCMP
     USE TYPES,       ONLY: LOA_TYPE, TE_TYPE, SSB_TYPE, OSM_TYPE
     USE VECT_VARS,   ONLY: Rot_l
!     ------------------------------------------------------------------

     CHARACTER (LEN=*), PARAMETER :: R_NAME = 'GPP_SS_RF'
!     Subroutine arguments:
!     ---------------------
     INTEGER,    INTENT(IN) :: JPOS, JJESFT
     REAL (KND), INTENT(IN) :: POS, E_W, ALBEDO, FLU_int
     REAL (KND)  DELs(NL, NSOL, N_NODE, Nvs)
!     ------------------------------------------------------
     TYPE (LOA_type) :: LOA    ! Local optical parameters of atmosphere
     TYPE (OSM_type) :: OSM    ! Original scattering matrices
     TYPE (TE_type)  :: TE     ! Thermal emission
     TYPE (SSB_type) :: SSB    ! Spherical solar beam
!     ------------------------------------------------------
!     ... Local variables & arrays
!     ----------------------------
     INTEGER  I, J, L, K, Im, Jr, Js, Jt, Jb, Jv, Ja, Ka, Npo, &
          Npl, L1, L2,  N_db, NLO, Jmc, KK
     REAL (KND)  T1, Aef, Rad_ss(NL,NSOL,N_NODE,Nvs),  &
          Phm(N_NODE*NSOL,NL,Nvs), Phm_x(N_NODE*NSOL,NL,Nvs), HEL(NL), &
          Plm_as( MAX(Nlg_Ray,Leg_aer,Leg_cld),N_NODE*NSOL), T_AU(NL)
     REAL (KND), ALLOCATABLE :: Rez(:), DUM(:), Phm_mc(:,:,:,:)
     REAL (KND), POINTER :: Rdb(:), P_MOM(:)
!     ==================================================================
       
      CALL ST_TRACK (R_NAME, 1, 'I')
      NLO = N_NODE*NSOL
      IF( Use_Ydb_SCM ) ALLOCATE( DUM(NLO) ) 

      ERROR_TXT = 'Error in '//R_NAME
!      write(*,*) '   gpp_ss_rf.f  '
      T_AU(1) = 0.d0
      DO  J = 1, NL - 1
         T_AU(J+1) = T_AU(J) + (LOA%AL(J) + LOA%AL(J+1))*0.5D0*DZR(J)
      ENDDO

      Npo = MAX(Nlg_Ray,Leg_aer,Leg_cld)
!     ... Rayleigh contribution
!     ******************************************************************
      Phm_x = 0.d0
      Npl = NLg_Ray  ;  P_MOM => LOA%RAY_MOMS(1:Npl,1)      
      DO J = 1, N_NODE*NSOL
         Phm_x(J,:,1) = SUM( Plm_ss(1:Npl,J)*P_MOM(:) )
      ENDDO
      IF( Nm_vs > 1 ) THEN
         CALL WIGNER_PLG(COS_SCAT, N_NODE*NSOL, 2, Npo, Plm_as )
         P_MOM => LOA%RAY_MOMS(1:Npl,5) 
         DO J = 1, N_NODE*NSOL
            T1 = SUM( Plm_as(1:Npl,J)*P_MOM(:) )
            FORALL( K = 2:3 ) Phm_x(J,:,K) = T1*Rot_l(J,K,2)
         ENDDO
      ENDIF
      FORALL( J=1:N_NODE*NSOL, K=1:Nvs ) Phm(J,:,K) = LOA%SCR(:)*Phm_x(J,:,K)
!     ... End of Rayleigh contribution
!     >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

!     ----------- Atomic emission contribution -----------
      IF( Emi_FLG ) THEN
         DO Ka = 1, N_AT  ;  Phm_x = 0.d0 
            DO J = 1,N_NODE*NSOL
               Phm_x(J,:,1) = SUM( Plm_ss(1:Npl,J)*LOA%EPF(Ka,:,1) )
            ENDDO
            IF( Nm_vs > 1 ) THEN
               DO J = 1, N_NODE*NSOL
                  T1 = SUM( Plm_as(1:Npl,J)*LOA%EPF(Ka,:,5) )
                  FORALL( K = 2:3 ) Phm_x(J,:,K) = T1*Rot_l(J,K,2)
               ENDDO
            ENDIF
            FORALL( J=1:N_NODE*NSOL, K=1:Nvs ) Phm(J,:,K) = Phm(J,:,K)  &
                 + LOA%EMC(Ka,:)*Phm_x(J,:,K)
         ENDDO ! over absorber (Ka)
      ENDIF
!     ------- End of atomic emission contribution --------

!     ... Aerosol contribution
!     ******************************************************************
      IF ( DO_AEROSOL ) THEN
         Npl = Leg_aer  ; Phm_x = 0.d0
         WHERE( (1.d0-LOA%DM_A(:)) == 0.d0 )
            HEL(:) = LOA%SCA(:)
         ELSEWHERE
            HEL(:) = LOA%SCA(:)/(1.d0 - LOA%DM_A(:))
         END WHERE
         DO L = 1,LA_N  ;  L1 = LA_S(L)  ;  L2 = LA_E(L)
            P_MOM => OSM%AER_ALL(1:Npl,L,1)
!            WRITE(*,*) '  SUM(P_MOM) ', L, SUM(P_MOM)
            DO J = 1, N_NODE*NSOL
               Phm_x(J,L1:L2,1) = SUM( Plm_ss(1:Npl,J)*P_MOM(:) )
!               IF( J == N_PLUS*NSOL + 1 ) &
!                    WRITE(*,*)  SUM( Plm_ss(1:Npl,J)), Phm_x(J,L1,1)
            ENDDO
            IF( Nm_vs == 1 ) CYCLE 
            P_MOM => OSM%AER_ALL(1:Npl,L,5)
            DO J = 1, N_NODE*NSOL
               T1 = SUM( Plm_as(1:Npl,J)*P_MOM(:) )
               FORALL( K = 2:3 ) Phm_x(J,L1:L2,K) = Rot_l(J,K,2)*T1
            ENDDO
         ENDDO  ! over aerosol layers (L)
         FORALL( J=1:N_NODE*NSOL, K=1:Nvs ) Phm(J,:,K) = Phm(J,:,K) &
              + HEL(:)*Phm_x(J,:,K)
      ENDIF  ! DO_AEROSOL
!     ------- End of aerosol contribution --------

!     ... Cloud contribution
!     ******************************************************************
      IF( DO_CLOUD ) THEN 
         ALLOCATE( Rez(Leg_cld), Phm_mc(N_NODE*NSOL,NL,Nm_ice,Nvs))
         Npl = Leg_cld ; Phm_mc = 0.d0
!         IF( Phm_const ) THEN
!        ... Phase function constant within cloud layer
!        ----------------------------------------------
         DO L = 1,N_CLD_LAYERS
            Jt = J_TOP(L)+1 ; Jb = J_BOT(L)-1 
            IF( MLCS(L)%Phm_co ) THEN
!           ... Phase function constant within this cloud layer
!               -----------------------------------------------
               DO Jmc = 1, MLCS(L)%N_pha
                  IF( MLCS(L)%Npf(Jmc)%Y_Scm ) THEN
!                    ... Scattering matrix for Yang DB:
                     P_MOM => OSM%CLD_SCM(:,L,1,Jmc,1)
                     CALL AL_INTERP (Ydb_ang, SCAT_ANG, P_MOM, &
                          NLO, COS_SCAT, DUM, .TRUE., *880)
                     FORALL( J=1:NLO ) Phm_mc(J,Jt:Jb,Jmc,1) = DUM(J)
                     IF( Nm_vs == 1 ) CYCLE 
                     P_MOM => OSM%CLD_SCM(:,L,1,Jmc,5)            
                     CALL AL_INTERP (Ydb_ang, SCAT_ANG, P_MOM, &
                          NLO, COS_SCAT, DUM, .TRUE., *880)
                     FORALL( J=1:NLO,K=2:3 ) Phm_mc(J,Jt:Jb,Jmc,K) =  &
                          DUM(J)*Rot_l(J,K,2)
                     CYCLE  ! to next thermodynamic phase (Jmc)
                  ENDIF
!                 ... Expansion coefficients:
                  P_MOM => OSM%CLD_ALL(1:Npl,L,1,Jmc,1) 
                  FORALL( J=1:N_NODE*NSOL ) Phm_mc(J,Jt:Jb,Jmc,1)  &
                       = SUM( Plm_ss(1:Npl,J)*P_MOM(:) )
                  IF( Nm_vs == 1 ) CYCLE 
                  P_MOM => OSM%CLD_ALL(1:Npl,L,1,Jmc,5)            
                  FORALL( J=1:N_NODE*NSOL, K=2:3 ) Phm_mc(J,Jt:Jb,Jmc,K) =  &
                       SUM( Plm_as(1:Npl,J)*P_MOM(:) )*Rot_l(J,K,2)
               ENDDO  ! over water/ice phase (Jmc)
               CYCLE  ! to next cloud layer (L)
            ENDIF  ! MLCS(L)%Phm_co 
!        ... Phase function depends on effective radius 
!           -------------------------------------------------------------
            KK = 2
            DO Jmc = 1, MLCS(L)%N_pha
               N_db = MLCS(L)%Npf(Jmc)%Ndb_ref
               Rdb => MLCS(L)%Npf(Jmc)%Pdb_ref(:)
               IF( MLCS(L)%Npf(Jmc)%Y_Scm ) THEN
                  CALL DERIV_CLD_SCM( OSM%CLD_SCM, Phm_mc, *880 )
                  CYCLE
               ENDIF
               CALL DERIV_CLD( OSM%CLD_ALL, Phm_mc )
               KK = KK + 2
            ENDDO  ! over water/ice phase (Jmc)
         ENDDO  ! over cloud layers (L)
!        -------------------------------------------------------------
!         ELSE
!!        ... Phase function depends on effective radius 
!!        -------------------------------------------------------------
!            DO L = 1,N_CLD_LAYERS  
!               Jt = J_TOP(L)+1 ; Jb = J_BOT(L)-1 ; KK = 2
!               DO Jmc = 1, MLCS(L)%N_pha
!                  N_db = MLCS(L)%Npf(Jmc)%Ndb_ref
!                  Rdb => MLCS(L)%Npf(Jmc)%Pdb_ref(:)
!                  IF( MLCS(L)%Npf(Jmc)%Y_Scm ) THEN
!                     CALL DERIV_CLD_SCM( OSM%CLD_SCM, Phm_mc, *880 )
!                     CYCLE
!                  ENDIF
!                  CALL DERIV_CLD( OSM%CLD_ALL, Phm_mc )
!                  KK = KK + 2
!               ENDDO  ! over water/ice phase (Jmc)
!            ENDDO  ! over cloud layers (L)            
!!        -------------------------------------------------------------
!         ENDIF  ! Phm_const

         FORALL( J=1:NLO, K=1:Nvs ) Phm(J,:,K) = Phm(J,:,K) &
              + LOA%SCC_NOSCALE(:,1)*Phm_mc(J,:,1,K)
         IF( Nm_ice == 2 ) THEN
            FORALL( J=1:NLO, K=1:Nvs ) Phm(J,:,K) = Phm(J,:,K) &
                 + LOA%SCC_NOSCALE(:,2)*Phm_mc(J,:,2,K)
         ENDIF
      ENDIF  ! Do_cloud

!     ... Calculate reflected radiation
      CALL REFL_SS_RF(JPOS, JJESFT, POS, ALBEDO, LOA, SSB, TE, &
           Phm, Rad_ss, E_W, Flu_int, T_AU)
!      write(*,*) '  Reflected SS '
!      Js = 3  ;  J = 1
!      DO I = 1, NL
!         WRITE(*,'(2x,I3,*(1x,ES12.4))') I, Rad_ss(I,Js,J,1)
!      ENDDO
!     ... VST=2 (vector) VST=1 (scalar) will be accounted for
!         in the subroutine "sphere_field_s"
      DELs(:,:,1:N_PLUS,:) = DELs(:,:,1:N_PLUS,:) &
            + Rad_ss(:,:,1:N_PLUS,:)*VST
      
!     ... Calculated transmitted radiation
      CALL TRANS_SS_RF(JPOS, JJESFT, POS, LOA, SSB, TE,  &
           Phm, Rad_ss, E_W, T_AU)
      DELs(:,:,1+N_PLUS:N_NODE,:) = DELs(:,:,1+N_PLUS:N_NODE,:)  &
           + Rad_ss(:,:,1+N_PLUS:N_NODE,:)*VST
!      write(*,*) '  Transmitted SS '
!      Js = 3
!      DO I = 1, NL
!         WRITE(*,'(2x,I3,*(1x,ES12.4))') &
!              I, Rad_ss(I,Js,1+N_PLUS,1)
!      ENDDO
      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN

880   WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)
      RETURN 1

    CONTAINS
!    *******************************************************************
      SUBROUTINE DERIV_CLD( S_MAT, Out_x )

        USE WF_NUM,      ONLY: NUM_CLD, PERTURB
!      -----------------------------------------------------------------
        REAL(KND) S_MAT(Leg_cld,NL_Cld,Nphr,Nm_ice,Nm_vs), &
             Out_x(NLO,NL,Nm_ice,Nvs)
!     ... Local variables
        INTEGER   I1, I2
        REAL (KND) DL
!     ----------------------------------------------------
        DO Jr = Jt,Jb
!          ... Effective radius at a given altitude 
           Aef = Prof_Cpar(Jr,KK) 
!            IF( NUM_CLD ) Aef = Aef/PERTURB(Jr)
           CALL LIN_SEARCH_NEW(Aef, DL, I1, I2, N_db, Rdb)
!            IF( NUM_CLD ) Aef = Aef*PERTURB(Jr)
!          ... Interpolate on a given effective radius
           Rez(:) = S_MAT(:,L,I2,Jmc,1)*DL   &
                + S_MAT(:,L,I1,Jmc,1)*(1.d0 - DL)
           DO J = 1, NLO
              Out_x(J,Jr,Jmc,1) = SUM(Plm_ss(1:Leg_cld,J)*Rez(:))
           ENDDO
           IF( Nm_vs == 1 ) CYCLE  ! if scalar case 
!           ========================================
!          ... Interpolate on a given effective radius
           Rez(:) = S_MAT(:,L,I2,Jmc,5)*DL  &
                + S_MAT(:,L,I1,Jmc,5)*(1.d0 - DL)
           DO J = 1, NLO
              T1 = SUM( Plm_as(1:Leg_cld,J)*Rez(:) ) 
              FORALL( K = 2:3 ) Out_x(J,Jr,Jmc,K) = T1*Rot_l(J,K,2)
           ENDDO  ! over LoS (J)
        ENDDO  ! over sub-layers within a cloud (Jr)
        RETURN
      END SUBROUTINE DERIV_CLD
! **********************************************************************      

      SUBROUTINE DERIV_CLD_SCM( S_MAT, Out_x, * )

        CHARACTER (LEN=13), PARAMETER :: R_NAME = 'DERIV_CLD_SCM'
        REAL(KND) S_MAT(Ydb_ang,NL_Cld,Nphr,Nm_ice,Nm_vs), &
             Out_x(NLO,NL,Nm_ice,Nvs)
!     ... Local variables
        INTEGER   I1, I2
        REAL (KND) DL, Rez(Ydb_ang)
!     ----------------------------------------------------
        CALL ST_TRACK (R_NAME, 1, 'I')
        DO Jr = Jt,Jb
!          ... Effective radius at a given altitude 
           Aef = Prof_Cpar(Jr,KK) 
           CALL LIN_SEARCH_NEW(Aef, DL, I1, I2, N_db, Rdb)
!          ... Interpolate on a given effective radius for all scat. angles
           Rez(:) = S_MAT(:,L,I2,Jmc,1)*DL  &
                + S_MAT(:,L,I1,Jmc,1)*(1.d0 - DL)
!          ... Interpolate on scattering angles COS_SCAT
           CALL AL_INTERP (Ydb_ang, SCAT_ANG, Rez, &
                NLO, COS_SCAT, DUM, .TRUE., *880)
           Out_x(:,Jr,Jmc,1) = DUM(:)            
           IF( Nm_vs == 1 ) CYCLE ! if scalar case 
!          ... Interpolate on a given effective radius
           Rez(:) = S_MAT(:,L,I2,Jmc,5)*DL  &
                + S_MAT(:,L,I1,Jmc,5)*(1.d0 - DL)
!          ... Interpolate on scattering angles COS_SCAT
           CALL AL_INTERP (Ydb_ang, SCAT_ANG, Rez,  &
                NLO, COS_SCAT, DUM, .TRUE., *880)
           DO J = 1,NLO 
              FORALL( K=2:3 ) Out_x(J,Jr,Jmc,K) = DUM(J)*Rot_l(J,K,2)
           ENDDO
        ENDDO  ! over sub-layers within a cloud (Jr)

        CALL ST_TRACK (R_NAME, 1, 'O')
        RETURN
        
880     WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)
        RETURN 1

      END SUBROUTINE DERIV_CLD_SCM

    END SUBROUTINE GPP_SS_RF
!***********************************************************************
!***********************************************************************
    SUBROUTINE REFL_SS_RF(JPOS, JJESFT, POS, ALBEDO, LOA, SSB,  &
         TE, Phm, Rad_ss, E_W, Flu_int, T_AU)
!     ... Called by "gpp_ss_rf.f" 

      USE FLU_LAND,    ONLY: FLU_L
      USE GRIDS,       ONLY: NL => NLAYERS, DZR, DZ, ZZZ, NSOL
      USE INTERN_DIMS, ONLY: Nvs
      USE NODES,       ONLY: N_NODE, MJU_NODE, MJU_MINUS, PHI_NODE, &
                             MJU_PLUS, N_PLUS, N_MINUS
      USE ST_VARS,     ONLY: Pie, BRDF_FLAG, DO_PLANCK
      USE TYPE_VARS,   ONLY: KND, DCMP
      USE TYPES,       ONLY: LOA_type, TE_TYPE, SSB_TYPE
      USE WATER,       ONLY: SS_PDF
!     ------------------------------------------------------------------

      CHARACTER (LEN=*), PARAMETER :: R_NAME = 'REFL_SS_RF'
!     Subroutine arguments:
!     ---------------------
      INTEGER,    INTENT(IN) ::  JPOS, JJESFT
      REAL (KND), INTENT(IN) ::  POS, E_W, ALBEDO, Flu_int, &
           Phm(N_NODE*NSOL,NL,Nvs), T_AU(NL)
      REAL (KND), INTENT(OUT)::  Rad_ss(NL,NSOL,N_NODE,Nvs)
!     ------------------------------------------------------
      TYPE (SSB_type) :: SSB    ! Spherical solar beam
      TYPE (LOA_type) :: LOA    ! Local optical parameters of atmosphere
      TYPE (TE_type)  :: TE     ! Thermal emission
!     ------------------------------------------------------
      INTEGER   I, J, Js, Jv, Ja, Jc, KK, Zn
      REAL (KND)  DTA, CV, CS, ECV, SOU, Ck, T1, SMd, Cks, ALW, EXS, &
           Ewei, Mus, Iaj(NL-1,NSOL,N_PLUS,Nvs),  &
           SC_Md(NL-1,NSOL,N_PLUS), Tls(NSOL,N_PLUS,Nvs)
      REAL (KND), POINTER ::  Mue(:,:), MuV(:), Phi(:)
      REAL (KND), ALLOCATABLE ::  Wa2(:,:,:), Wa1(:,:,:)
      COMPLEX (kind(0.0d0)) :: Muc
!     ==================================================================

      CALL ST_TRACK (R_NAME, 1, 'I')
      Mue => SSB%Mue  ;  MuV => Mju_PLUS ; Phi => PHI_NODE(1:N_PLUS)

      !        ... Array SC_Md(I,Js,Jv) contains results of the analytical 
!            integration of the solar exponent or its approximation 
!            (in the pseudo-spherical atmosphere) over each layer along
!            line-of-sight direction. 
!      WRITE(*,*) '  NSOL = ', NSOL
!      WRITE(*,*) '  SHAPE(Mue(Js,I)) ', SHAPE( Mue )
!      WRITE(*,*) '  SHAPE(SSB%SIN_COEF) ', SHAPE( SSB%SIN_COEF )
!      WRITE(*,*) '  SHAPE(SSB%N_sph) ', SHAPE( SSB%N_sph )
      DO I = 1, NL-1    ; DTA = T_AU(I+1) - T_AU(I)
         DO Js = 1,NSOL ; CS = Mue(Js,I) ; SOU = DEXP( -DTA/CS )
            DO Jv = 1, N_PLUS
               CV = MuV(Jv) ; ECV = DEXP( -DTA/CV ) 
               SMd = Cs*Cv/(Cs + Cv)*( 1.d0 - SOU*ECV )
               Zn = -1
               DO Jc = 1,SSB%N_sph(Js,I)-1
                  Ck = DTA/DBLE(Jc)/Pie ; Muc = CS*Ck/DCMP(Ck,-Cs)
                  SMd = SMd + AIMAG( Muc*Cv/(Muc + Cv) )  &
                       *SSB%SIN_COEF(Js,I,Jc)*(1.d0 - Zn*SOU*ECV)
                  Zn = -Zn
               ENDDO  ! over expansion coeff. of solar exp. (Jc)
               SC_Md(I,Js,Jv) = SMd     !/SSB%EXS(I,Js)/CV
            ENDDO  ! over viewing angles (Jv)
         ENDDO  !  over solar angles (Js)
      ENDDO  ! over altitude layers (I)
      
      Iaj = 0.d0
!     ... Thermal emission contribution in Iaj
      IF( DO_PLANCK ) CALL ADD_REF_PLA

!     ... Array Iaj(I,:,:) contains results of the numerical
!         integration of the scattering coefficient and analytical
!         of the solar source function (SC_Md) over each layer 
!         along the line-of-sight
!      WRITE(*,*) '   EXS, Cv, Phm  '
      DO I = 1, NL-1          ; ALW = LOA%AL(I) + LOA%AL(I+1) 
         DO Js = 1, NSOL      ; EXS = SSB%EXS(I,Js)/ALW/4.d0
            J = (Js-1)*N_NODE
            DO Jv = 1, N_PLUS 
               Iaj(I,Js,Jv,:) = Iaj(I,Js,Jv,:) + SC_Md(I,Js,Jv)*EXS/MuV(Jv) &
                    *( Phm(J+Jv,I,:) + Phm(J+Jv,I+1,:) )
!               IF( Js==1.AND.Jv==1 ) &
!                    WRITE(*,'(2x,I3,*(1x,E12.4))') I,  &
!                    EXS, MuV(Jv) , Phm(J,I,1) + Phm(J,I+1,1)
            ENDDO  ! over line-of-sight angles (Jv)
         ENDDO  ! over solar zenith angles (Js)
      ENDDO  ! over all altitude layers (I)

      Tls = 0.d0 
!     ... Thermal emission contribution
      IF( DO_PLANCK ) FORALL( Js=1:NSOL ) Tls(Js,:,1) &
           = Tls(J,:,1) + TE%PLANCK_SURF
!     ... Surface fluorescence contribution
      IF( FLU_L ) FORALL( Js=1:NSOL ) Tls(Js,:,1)  &
           = Tls(Js,:,1) + Flu_int

!     ... Calculation of reflected radiation at all levels using
!         recurrence relation
!      WRITE(*,*) '    Tls :'
      Rad_ss(NL,:,1:N_PLUS,:) = Tls(:,:,:)
      DO I = NL-1,1,-1  ; DTA = T_AU(I+1) - T_AU(I)
         DO Js = 1,NSOL
            DO Jv = 1, N_PLUS  ;  ECV = DEXP(-DTA/Muv(Jv))
               Tls(Js,Jv,:) = Tls(Js,Jv,:)*ECV + Iaj(I,Js,Jv,:)
            ENDDO
         ENDDO
         Rad_ss(I,:,1:N_PLUS,:) = Tls(:,:,:) 
      ENDDO

      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN
    CONTAINS
! **********************************************************************

   SUBROUTINE ADD_REF_PLA
     REAL (KND) SC_pla(NL-1,N_PLUS)

     DO I = 1,NL - 1 ; DTA = T_AU(I+1) - T_AU(I)
        CS = -TE%Mup(I) ; SOU = DEXP( TE%Mup(I)*DTA )
        DO Jv = 1, N_PLUS
           CV = MuV(Jv) ; ECV = DEXP( -DTA/CV ) 
           SMd = Cv/(1.d0 + CS*Cv)*(1.d0 - SOU*ECV)
           Zn = -1
           DO Jc = 1,TE%N_pla(I)-1
              Ck = DTA/DBLE(Jc)/Pie ; Cks = CS*Ck
              Muc = Ck/DCMP( Cks,-1.d0)
              SMd = SMd + AIMAG( Muc*Cv/(Muc + Cv) )  &
                   *TE%PL_COEF(I,Jc)*(1.d0 - Zn*SOU*ECV)
              Zn = -Zn
           ENDDO  ! over Plank function exp. coeff. (Jc)
           SC_pla(I,Jv) = SMd
        ENDDO  ! over upward nodes (Jv)
     ENDDO  ! over altitude layers (I)
      
     DO I = 1,NL-1  ;  T1 = TE%NPLANCK(I)
        DO Jv = 1, N_PLUS  
           Iaj(I,:,Jv,1) = SC_pla(I,Jv)*T1/ MuV(Jv)
        ENDDO  ! over line-of-sight zenith angles (Jv)
     ENDDO  ! over altitude layers (I)
     
   END SUBROUTINE ADD_REF_PLA

 END SUBROUTINE REFL_SS_RF ! REFL_SS_RF as main routine
!********************************************************************C
!********************************************************************C

 SUBROUTINE TRANS_SS_RF(JPOS, JJESFT, POS, LOA, SSB, TE, Phm, Rad_ss, E_W, &
      T_AU )
! ... Called by "gpp_ss_rf.f" 

   USE GRIDS,       ONLY: NL => NLAYERS, DZ, DZR, NSOL
   USE INTERN_DIMS, ONLY: Nvs 
   USE NODES,       ONLY: N_NODE, MJU_NODE, MJU_MINUS, &
                          MJU_PLUS, N_PLUS, N_MINUS
   USE ST_VARS,     ONLY: Pie, DO_PLANCK
   USE TYPE_VARS,   ONLY: KND, DCMP
   USE TYPES,       ONLY: LOA_type, TE_TYPE, SSB_TYPE
! ------------------------------------------------------------------

   CHARACTER (LEN=*), PARAMETER :: R_NAME = 'TRANS_SS_RF'
!     Subroutine arguments:
!     ---------------------
   INTEGER,    INTENT(IN) :: JPOS, JJESFT
   REAL (KND), INTENT(IN) :: POS, E_W, Phm(N_NODE*NSOL,NL,Nvs), T_AU(NL)
   REAL (KND), INTENT(OUT):: Rad_ss(NL,NSOL,N_NODE,Nvs)
!     ------------------------------------------------------
   TYPE (SSB_type) :: SSB    ! Spherical solar beam
   TYPE (LOA_type) :: LOA    ! Local optical parameters of atmosphere
   TYPE (TE_type)  :: TE     ! Thermal emission
!     ------------------------------------------------------ 
   INTEGER   I, J, Js, Jv, Ja, Jc, Zn
   REAL (KND)  ALW, CV, CS, Ck, Cks, DTA, ECV, EXS, Ewei, SOU, SMd, &
        T1, Iaj(NL-1,NSOL,N_MINUS,Nvs),  & 
        Tls(NSOL,N_MINUS,Nvs), SC_Md(NL-1,NSOL,N_MINUS)
   REAL (KND), POINTER :: MuV(:)
   COMPLEX (kind(0.0d0)) :: Muc
!     ==================================================================
     
   CALL ST_TRACK (R_NAME, 1, 'I')

   MuV => MJU_MINUS(:)
!        ... Array SC_Md(I,Js,Jv) contains results of the analytical 
!            integration of the solar exponent or its approximation 
!            (in the pseudo-spherical atmosphere) over each layer along
!            line-of-sight direction.
   DO I = 1, NL-1     ; DTA = T_AU(I+1) - T_AU(I)
      DO Js = 1, NSOL ; CS = SSB%Mue(Js,I) ; SOU = DEXP( -DTA/CS )
         DO Jv = 1, N_MINUS
            Cv = ABS(MuV(Jv)) ; ECV = DEXP( -DTA/Cv ) 
            IF( REAL(Cs) /= REAL(Cv) ) THEN
               SMd = Cs*Cv*(SOU - Ecv)/(Cs - Cv)
            ELSE
               SMd = Ecv*DTA
            ENDIF
            Zn = -1
            DO Jc = 1,SSB%N_sph(Js,I)-1
               Ck = DTA/DBLE(Jc)/Pie ; Muc = CS*Ck/DCMP( Ck,-Cs )
               SMd = SMd + AIMAG( Muc*Cv/(Muc-Cv) )    &
                    *SSB%SIN_COEF(Js,I,Jc)*(Zn*SOU - Ecv)
               Zn = -Zn
            ENDDO
            SC_Md(I,Js,Jv) = SMd
         ENDDO
      ENDDO
   ENDDO

!   WRITE(*,*) '   SC_Md(I,Js=1,Jv=1)  '
!   Js = 1 ; Jv = 1
!   DO I = 1,NL - 1
!      WRITE(*,'(2x,I3,*(1x,E12.4))') I, SC_Md(I,Js,Jv)
!   ENDDO

   Iaj = 0.d0
!     ... Thermal emission contribution 
   IF( DO_PLANCK ) CALL ADD_TRA_PLA

!     ... Array Iaj(I,:,:) contains results of the numerical
!         integration of the scattering coefficient and analytical
!         of the solar source function (SC_Md) over each layer 
!         along the line-of-sight
   DO I = 1,NL - 1     ; ALW = LOA%AL(I) + LOA%AL(I+1) 
      DO Js = 1, NSOL  ; EXS = SSB%EXS(I,Js)/ALW/4.d0
         J = (Js-1)*N_NODE + N_PLUS
         DO Jv = 1, N_MINUS
            Iaj(I,Js,Jv,:) = Iaj(I,Js,Jv,:) + SC_Md(I,Js,Jv) &
                 *EXS/ABS(MuV(Jv))*( Phm(J+Jv,I,:) + Phm(J+Jv,I+1,:) )
!            IF( Js==1.AND.Jv==1 ) &
!                 WRITE(*,'(2x,I3,*(1x,E12.4))') I,  &
!                 EXS, ABS(MuV(Jv)) , Phm(J,I,1) + Phm(J,I+1,1)
         ENDDO  ! over line-of-sight zenith angles (Jv)
      ENDDO  ! over solar zenith angles (Js)
   ENDDO  ! over altitude layers (I)

!   WRITE(*,*) '   Iaj(I,Js=1,Jv=1,1)  '
!   Js = 1 ; Jv = 1
!   DO I = 1,NL - 1
!      WRITE(*,'(2x,I3,*(1x,E12.4))') I, Iaj(I,Js,Jv,1)
!   ENDDO
   Tls = 0.d0 
!     ... Calculation of transmitted radiation at all levels using
!         recurrence relation
   Rad_ss(1,:,N_PLUS+1:N_NODE,:) = Tls(:,:,:)
   DO I = 1, NL-1            ;   DTA = T_AU(I+1) - T_AU(I)
      DO Js = 1, NSOL
         DO Jv = 1, N_MINUS  ; ECV = DEXP( -DTA/ABS(Muv(Jv)) )
            Tls(Js,Jv,:) = Tls(Js,Jv,:)*ECV + Iaj(I,Js,Jv,:)
         ENDDO
      ENDDO
      Rad_ss(I+1,:,N_PLUS+1:N_NODE,:) = Tls(:,:,:) 
   ENDDO

   CALL ST_TRACK (R_NAME, 1, 'O')      
   RETURN

 CONTAINS  ! for suroutine "trans_ss_rf"
      
! **********************************************************************
   SUBROUTINE ADD_TRA_PLA
     REAL (KND)  SC_pla(NL-1,N_MINUS)
!     ==================================================================
     DO I = 1,NL - 1
        DTA = T_AU(I+1) - T_AU(I)
        CS = -TE%Mup(I) ; SOU = DEXP( TE%Mup(I)*DTA )
        DO Jv = 1,N_MINUS
           CV = ABS(MuV(Jv)) ; ECV = DEXP( -DTA/CV ) 
           IF( REAL(CS*CV) /= REAL(1.d0) ) THEN
              SMd = CV*(SOU - Ecv)/(1.d0 - CS*CV)
           ELSE
              SMd = Ecv*DTA
           ENDIF
           Zn = -1
           DO Jc = 1,TE%N_pla(I)-1
              Ck = DTA/DBLE(Jc)/Pie ; Cks = Cs*Ck
              Muc = Ck/DCMP( Cks,-1.d0)
              SMd = SMd + AIMAG( Muc*Cv/(Muc-Cv) )  &
                   *TE%PL_COEF(I,Jc)*(Zn*SOU - Ecv)
              Zn = -Zn
           ENDDO
           SC_pla(I,Jv) = SMd
        ENDDO
     ENDDO
     
     DO I = 1,NL-1  ;  T1 = TE%NPLANCK(I)            
        DO Jv = 1, N_MINUS 
           Iaj(I,:,Jv,1) = SC_pla(I,Jv)*T1/ABS(MuV(Jv))
        ENDDO  ! over line-of-sight zenith angles (Jv)
     ENDDO  ! over altitude layers (I)

   END SUBROUTINE ADD_TRA_PLA
! **********************************************************************

 END SUBROUTINE TRANS_SS_RF
! **********************************************************************
! **********************************************************************
