C********************************************************************
C
C       SCIATRAN - Version 4.7 - Last modification: 16 Aug 2023
C       Copyright (C) - Institute of Remote Sensing,
C                       University of Bremen, Germany
C
C*********************************************************************
       
C           Subroutine to calculate the Stokes vector in the 
c    single scattering approximation in the plane-parallel geometry.
C      It is used also in the framework of the single scattering
C                         correction algorithm.
C      It is assumed that incident radiation is unpolarized !!!
C      --------------------------------------------------------
      SUBROUTINE GPP_SS_STOKES(JPOS, JJESFT, Iot, POS, ALBEDO, E_W, OSM,
     $     GSL, LOA, LWF, PWF, SSB, TE, SRR, SRR_Jact, RADOUT, TAC,  
     $     Abs_soot, FLU_int, *) 
c     ... Call in "dom_spherical.f" if SCATT_MODE = 'ss'
c              in "st_dom_tau.f"    if SCATT_MODE = 'ms'
c                                      WF_FLAG    = .false.
c                                      SSRCA      = .true.
c                                      Mh = 1 (first harmonic only)
c              in "st_adjoint_a.f"  if SCATT_MODE = 'ms' 
c                                      WF_FLAG    = .true.
c                                      SSRCA      = .true.
c                                      Mh = 1 (first harmonic only)
      USE ABSORBERS,   ONLY: N_GAS
      USE AEROSOLS,    ONLY: LA_N, LA_S, LA_E, Dwr_mi, N_dwr, Mie_layer,
     $                       IDX_AER
      USE CLOUDS,      ONLY: DO_CLOUD, N_CLD_LAYERS, J_TOP, J_BOT, 
     $                       Prof_Cpar, SCAT_ANG
      USE CLD_INP,     ONLY: Phm_const, N_ref_db, Ref_db, TDS, MLCS,
     $                       N_ref_ice, Ref_ice, Deriv_Phc, Nphr,
     $                       Ydb_ang
      USE EMISSION,    ONLY: N_AT, Emi_MOMS
      USE GRIDS,       ONLY: NL => NLAYERS, DZ, DZR, ZZZ
      USE GT_POLYN,    ONLY: Plm_ss, NLEGEN
      USE INFO,        ONLY: TG_INFO, WF_DB
      USE INTERN_DIMS, ONLY: Nvs, Nm_vs, Leg_aer => NLG_ALL_A, NL_AER,
     $                       NL_Cld, Nm_ice, Leg_cld => NLG_ALL_C, 
     $                       Nlg_Ray, NL_pla, Nf_pla, NL_soot
      USE LOG_PARS,    ONLY: ERROR_TXT, N_ESO, N_DSO, LOG_LEVEL, EU
      USE PS_INTERN,   ONLY: COS_SCAT 
      USE RTM_OUTPUT,  ONLY: N_OP, STOK_OUT
      USE ST_VARS,     ONLY: DO_AEROSOL, PIE, DO_SATELLITE, SCATT_MODE,
     $                       WF_FLAG, DO_PLANCK, NS_wf, Nc_der, VST, 
     $                       Emi_FLG, Ray_to_WF
      USE SPHER_PARS,  ONLY: NSZA => N_SOL_ANG, N_OBS_ALT, 
     $                       NLO => N_LOS_OUT,  
     $                       JV_S, JV_E, N_AZ_VAR, NLSA => N_LOS_ANG, 
     $                       ANG_OUT, N_AZ_ANG, ANG_SEL_FLAG
      USE TOOLS,       ONLY: FIND_IDX, INQ_WF
      USE TYPE_VARS,   ONLY: KND, DCMP
      USE TYPES,       ONLY: LOA_TYPE, TE_TYPE, SSB_TYPE, RRS_TYPE, 
     $                       OSM_TYPE, LWF_TYPE, PWF_TYPE, GSL_TYPE
      USE VECT_VARS,   ONLY: Rot_l
c     ------------------------------------------------------------------

      CHARACTER (LEN = *), PARAMETER :: R_NAME = 'GPP_SS_STOKES'
c     Subroutine arguments:
c     ---------------------
      INTEGER,    INTENT(IN) :: JPOS, JJESFT, Iot, SRR_Jact
      REAL (KND), INTENT(IN) :: POS, E_W, ALBEDO, TAC(NL,N_GAS),
     $     Abs_soot(NL_soot), FLU_int
      REAL (KND)  RADOUT(N_OP,NLO,Nvs)
c     ------------------------------------------------------
      TYPE (GSL_type) :: GSL    ! Angles 
      TYPE (LOA_type) :: LOA    ! Local optical parameters of atmosphere
      TYPE (LWF_type) :: LWF    ! Local weighting functions
      TYPE (OSM_type) :: OSM    ! Original scattering matrices
      TYPE (PWF_type) :: PWF    ! General weighting function 
      TYPE (TE_type)  :: TE     ! Thermal emission
      TYPE (RRS_type) :: SRR    ! RRS related parameters
      TYPE (SSB_type) :: SSB    ! Spherical solar beam
c     ------------------------------------------------------
      REAL (KND), POINTER :: P_MOM(:), Php(:)
c     ... Local variables & arrays
c     ----------------------------
      INTEGER  I, J, L, K, Im, Jr, Js, Jt, Jb, Jv, Ja, Jw, Ka, Npo, 
     $      Npl, L1, L2, Idr, N_db, Jmc, KK
      REAL (KND)  T1, Phm(NLO,NL,Nvs),  Rad_ss(NL,NLO,Nvs), 
     $     Phm_x(NLO,NL,Nvs), Aef, DUM(NLO),
     $     Plm_as( MAX(Nlg_Ray,Leg_aer,Leg_cld),NLO), HEL(NL)
      REAL (KND), ALLOCATABLE :: Rez(:), Phm_r(:,:,:), Phm_a(:,:,:),
     $     Phm_c(:,:,:), Dhm_x(:,:,:,:), Chm_x(:,:,:,:),
     $     Phm_mc(:,:,:,:), S_Om(:), S_Ab(:),   WF_DUM(:,:,:), 
     $     WF_AVER(:,:,:)
      REAL (KND), POINTER :: Rdb(:)
c     ==================================================================

      CALL ST_TRACK (R_NAME, 1, 'I')

c      WRITE(*,*) ' OSM%DER_ALL on input  ',  Leg_aer,
c     $     SUM( OSM%DER_ALL(1:Leg_aer,47,1,1) )
      ERROR_TXT = 'Error in '//R_NAME
c     ... These arrays contain weighting functions calculated within
c         this routine. They allocated in "dom_spherical.f".
      PWF%WF_esp = 0.d0 ; PWF%WF_ssp = 0.d0
      Npo = MAX(Nlg_Ray,Leg_aer,Leg_cld)

      IF( TE%NO_SUN ) GOTO 123 ! without solar light contribution

c     ... Rayleigh contribution
c     ******************************************************************
      Phm_x = 0.d0
      Npl = NLg_Ray  ;  P_MOM => LOA%RAY_MOMS(1:Npl,1)
      DO J = 1,NLO
         Phm_x(J,:,1) = SUM( Plm_ss(1:Npl,J)*P_MOM(:) )
      ENDDO
      
cc     ------------------------------------------------------------------
c      CALL COEF_TO_SCATMATR
c     $     (Npl, 6, LOA%RAY_MOMS, 181, 'E', './DATA_OUT/R_phm.dat')
cc     ------------------------------------------------------------------
      IF( Nm_vs > 1 ) THEN
         CALL WIGNER_PLG(COS_SCAT, NLO, 2, Npo, Plm_as )
         P_MOM => LOA%RAY_MOMS(1:Npl,5) 
         DO J = 1,NLO
            T1 = SUM( Plm_as(1:Npl,J)*P_MOM(:) )
            FORALL( K = 2:3 ) Phm_x(J,:,K) = T1*Rot_l(J,K,2)
         ENDDO
      ENDIF
      FORALL( J=1:NLO, K=1:Nvs ) Phm(J,:,K) = LOA%SCR(:)*Phm_x(J,:,K)
      IF( Ray_to_WF ) THEN
         ALLOCATE( Phm_r(NLO,NL,Nvs) ) ;  Phm_r(:,:,:) = Phm_x(:,:,:)
      ENDIF

c     ------- End of Rayleigh contribution --------

c     ----------- Atomic emission contribution -----------
      IF( Emi_FLG ) THEN
         DO Ka = 1, N_AT  ;  Phm_x = 0.d0 
            DO J = 1,NLO
               Phm_x(J,:,1) = SUM( Plm_ss(1:Npl,J)*LOA%EPF(Ka,:,1) )
            ENDDO
            IF( Nm_vs > 1 ) THEN
               DO J = 1,NLO
                  T1 = SUM( Plm_as(1:Npl,J)*LOA%EPF(Ka,:,5) )
                  FORALL( K = 2:3 ) Phm_x(J,:,K) = T1*Rot_l(J,K,2)
               ENDDO
            ENDIF
            FORALL( J=1:NLO, K=1:Nvs ) Phm(J,:,K) = Phm(J,:,K) 
     $           + LOA%EMC(Ka,:)*Phm_x(J,:,K)
         ENDDO ! over absorber (Ka)
      ENDIF
c     ------- End of atomic emission contribution --------

c     ... Aerosol contribution
c     ******************************************************************
      IF ( DO_AEROSOL ) THEN
         Npl = Leg_aer  ; Phm_x = 0.d0
         WHERE( (1.d0-LOA%DM_A(:)) == 0.d0 )
            HEL(:) = LOA%SCA(:)
         ELSEWHERE
            HEL(:) = LOA%SCA(:)/(1.d0 - LOA%DM_A(:))
         END WHERE

!         DO J = 1, NL
!            WRITE(*,'(2x,F8.4*(2x,e12.4))') ZZZ(J), HEL(J),LOA%DM_A(J) 
!         ENDDO
!         WRITE(*,*) '   Npl  ', Npl
         
         DO L = 1,LA_N  ;  L1 = LA_S(L)  ;  L2 = LA_E(L)
            P_MOM => OSM%AER_ALL(1:Npl,L,1)
            DO J = 1,NLO
               Phm_x(J,L1:L2,1) = SUM( Plm_ss(1:Npl,J)*P_MOM(:) )
            ENDDO
            IF( Nm_vs == 1 ) CYCLE 
            P_MOM => OSM%AER_ALL(1:Npl,L,5)
            DO J = 1,NLO
               T1 = SUM( Plm_as(1:Npl,J)*P_MOM(:) )
               FORALL( K = 2:3 ) Phm_x(J,L1:L2,K) = Rot_l(J,K,2)*T1
            ENDDO
         ENDDO  ! over aerosol layers (L)

         FORALL( J=1:NLO, K=1:Nvs ) Phm(J,:,K) = Phm(J,:,K)
     $        + HEL(:)*Phm_x(J,:,K)
c ======================================================================
!         WRITE(*,'(A,F8.4,A,F8.4, 2x,e12.4)')
!     $        '   gpp_ss_stokes: Aerosol '
!         DO J = 1, NL
!            WRITE(*,'(2x,I3, 2x,F8.4, *(2x,e12.4))') IDX_AER(J),
!        $           ZZZ(J), HEL(J), Phm_x(1,J,1)
!         ENDDO
c         STOP
c ======================================================================
         IF( INQ_WF('aer', 'WFT') ) THEN
            ALLOCATE( Phm_a(NLO,NL,Nvs) ) 
            FORALL( J=1:NLO,K=1:Nvs ) Phm_a(J,:,K) = Phm_x(J,:,K)
         ENDIF
      ENDIF
c     ------- End of aerosol contribution --------

c     ... Cloud contribution
c     ******************************************************************
      IF( DO_CLOUD ) THEN 
         ALLOCATE( Phm_mc(NLO,NL,Nm_ice,Nvs), Rez(Leg_cld) )
         Npl = Leg_cld ; Phm_x = 0.d0
         Phm_mc = 0.d0
c         WRITE(*,*) '   MLCS(:)%Phm_co = ', MLCS(:)%Phm_co
c         IF( Phm_const ) THEN
c         write(*,*) '     ====> gpp_ss_stokes '
         DO L = 1,N_CLD_LAYERS
            Jt = J_TOP(L)+1 ; Jb = J_BOT(L)-1
c            WRITE(*,*) '     Layer number ', L
            IF( MLCS(L)%Phm_co ) THEN
c           ... Constant phase function within this cloud layer
c               -----------------------------------------------
               Ka = 1
c               WRITE(*,*) '   MLCS(L)%N_pha ', MLCS(L)%N_pha               
               DO Jmc = 1, MLCS(L)%N_pha
                  IF( MLCS(L)%Npf(Jmc)%Y_Scm ) THEN
c                 ... Use scattering matrix:
                     P_MOM => OSM%CLD_SCM(:,L,1,Jmc,1)
                     CALL AL_INTERP (Ydb_ang, SCAT_ANG, P_MOM,
     $                    NLO, COS_SCAT, DUM, .TRUE., *880)
                     FORALL( J=1:NLO ) Phm_mc(J,Jt:Jb,Jmc,1) = DUM(J)
                     IF( Nm_vs == 1 ) CYCLE 
                     P_MOM => OSM%CLD_SCM(:,L,1,Jmc,5)            
                     CALL AL_INTERP (Ydb_ang, SCAT_ANG, P_MOM,
     $                    NLO, COS_SCAT, DUM, .TRUE., *880)
                     FORALL( J=1:NLO,K=2:3 ) Phm_mc(J,Jt:Jb,Jmc,K) = 
     $                    DUM(J)*Rot_l(J,K,2)
                     CYCLE
                  ENDIF
c                 ... Use expansion coefficients:
c                  write(*,*) '   Npl =  ', Npl, L, Jmc
c                  write(*,*) '  SHAPE(OSM%CLD_ALL) ', SHAPE(OSM%CLD_ALL)
                  P_MOM => OSM%CLD_ALL(1:Npl,L,1,Jmc,1)
                  FORALL( J=1:NLO ) Phm_mc(J,Jt:Jb,Jmc,1) = 
     $                 SUM( Plm_ss(1:Npl,J)*P_MOM(:) )
                  IF( Nm_vs == 1 ) CYCLE 
                  P_MOM => OSM%CLD_ALL(1:Npl,L,1,Jmc,5)            
                  FORALL( J=1:NLO, K=2:3 ) Phm_mc(J,Jt:Jb,Jmc,K) = 
     $                 SUM( Plm_as(1:Npl,J)*P_MOM(:) )*Rot_l(J,K,2)
               ENDDO            ! over water/ice phase (Jmc)
               CYCLE
            ENDIF  !  MLCS(L)%Phm_co
c           ... Phase function depends on effective radius 
c               -------------------------------------------
            KK = 2
            DO Jmc = 1, MLCS(L)%N_pha
               N_db = MLCS(L)%Npf(Jmc)%Ndb_ref
               Rdb => MLCS(L)%Npf(Jmc)%Pdb_ref(:)
c               WRITE(*,*) '    L, Jmc, N_db ', L, Jmc, N_db
c               write(*,*) '             selected Rdb  '
c               write(*,'(*(1x,e10.2))') Rdb(:)
               IF( MLCS(L)%Npf(Jmc)%Y_Scm ) THEN
c                           WRITE(*,*) '   ===>  Yang SCM: '
                  CALL DERIV_CLD_SCM( OSM%CLD_SCM, Phm_mc, *880 )
                  CYCLE
               ENDIF
c               write(*,*) '             interpolate on '
c               write(*,'(*(1x,e10.2))') Prof_Cpar(Jt:Jb,KK)
               CALL DERIV_CLD( OSM%CLD_ALL, Phm_mc )
               KK = KK + 2
            ENDDO  ! over water/ice phase (Jmc)
c            WRITE(*,*) '        ------------------------  '
         ENDDO  ! over cloud layers (L)            

         FORALL( J=1:NLO, K=1:Nvs ) Phm(J,:,K) = Phm(J,:,K)
     $        + LOA%SCC_NOSCALE(:,1)*Phm_mc(J,:,1,K)
         IF( Nm_ice == 2 ) FORALL( J=1:NLO, K=1:Nvs ) 
     $        Phm(J,:,K) = Phm(J,:,K)
     $        + LOA%SCC_NOSCALE(:,2)*Phm_mc(J,:,2,K)
c         WRITE(*,*) '     Alt            Water            Ice'
c         DO I = 1, Nl
c            WRITE(*,'(2x,E12.4,4x,4(2x,e12.4))') ZZZ(I),
c     $           (LOA%SCC_NOSCALE(I,Ka), Phm_mc(1,I,Ka,1), 
c     $           Ka = 1, Nm_ice)
c         ENDDO
c         PAUSE
         IF( INQ_WF('cld', 'WFT') ) THEN
            ALLOCATE( Phm_c(NLO,NL,Nvs) )
c          ... Select for WF calculation water or ice in the case of
c              mixed cloud
            Phm_c(:,:,:) = Phm_mc(:,:,1,:)
            ALLOCATE( WF_DUM(NL,NLO,NS_wf), WF_AVER(NL,NLO,NS_wf) )
            WF_AVER = 0.d0
c           ... Array LWF%DT1 is not allocated in the case 'ss' mode
            IF( TRIM(SCATT_MODE)=='ss' ) THEN
               ALLOCATE( LWF%DT1(NL) )  ;  LWF%DT1(:) = 0.d0  
            ENDIF
         ENDIF
      ENDIF  ! Do_cloud
c     ------- End of cloud contribution --------

 123  CONTINUE
      
      IF( DO_SATELLITE ) THEN
c         TEST-1
c            DO I = 1,NL  ;  J = 0  
c               T1 = DZ(I)/2.d0
c               DO Js = 1,NSZA
c                  DO Jv = JV_S(Js),JV_E(Js)
c                     DO Ja = 1,N_AZ_VAR  ;  J = J + 1
c                        PWF%WF_ssp(I,J,:,1) = T1*GSL%EXL(I,Jv)
c     $                       *SSB%EXS(I,Js)/4.d0/GSL%MuV(Jv)
c                     ENDDO
c                  ENDDO
c               ENDDO
c            ENDDO
c            FORALL( I=1:NL ) PWF%WF_ssp(I,:,1,1) = 
c     $           PWF%WF_ssp(I,:,1,1)*Phm_c(:,I,1)  
c        ... Calculate reflected radiation
         CALL REFL_SS(JPOS, JJESFT, POS, ALBEDO, GSL, LOA, PWF, SSB, TE,
     $        SRR, SRR_Jact, Phm, Rad_ss, E_W, Flu_int, LWF)
      ELSE
c        ... Calculated transmitted radiation
         CALL TRANS_SS(JPOS, JJESFT, POS, GSL, LOA, PWF, SSB, TE, 
     $        SRR, SRR_Jact, Phm, Rad_ss, E_W, LWF)
      ENDIF

c     ... Return for an excitation wavelength (reflection and transmission)
      IF( SRR%Exi_WL(JPOS) ) THEN 
         CALL ST_TRACK (R_NAME, 1, 'O')
         RETURN
      ENDIF

      SELECT CASE( TRIM(SCATT_MODE) )
c     ... In the case of single scattering approximation results are
c     stored direct in the output pointer-array STOK_OUT(:,:,:,:)
      CASE('ss')
         DO I = 1,N_OP
            STOK_OUT(SRR_Jact,Iot,I,:,:) = STOK_OUT(SRR_Jact,Iot,I,:,:) 
     $           + Rad_ss(N_OBS_ALT(I),:,:)*E_W
         ENDDO
c     ... Results are stored in temporary array RADOUT(:,:,:)
      CASE DEFAULT
c         WRITE(*,*) '   +++++++++++  Only SS radiation  ++++++++++ '
c         DO I = 1,N_OP
c            RADOUT(I,:,:) = Rad_ss(N_OBS_ALT(I),:,:)*E_W
c         ENDDO
         DO I = 1,N_OP
            RADOUT(I,:,:) = RADOUT(I,:,:) + Rad_ss(N_OBS_ALT(I),:,:)*E_W
         ENDDO
      END SELECT

c      DO I = 1, NL
c         WRITE(*,'(2x,I3,*(1x,ES12.4))') I, Rad_ss(I,1,1)
c      ENDDO

c      IF(LOG_LEVEL(1) >= N_ESO) THEN
!         WRITE(*,'(a)') 'Single scattering contribution :'
!         DO I = 1, NLO
!            WRITE(*,'(1x,I3,*(1x,e15.7))')
!     $           I, Rad_ss(N_OBS_ALT(N_OP),I,:)*E_W            
!       ENDDO
c      ENDIF
      
c     ==================================================================
c      Output of weighting functions calculated in the single scattering
c         approximation or single scattering correction mode
c     ==================================================================
      IF( WF_FLAG ) THEN
         IF( TRIM(SCATT_MODE)/='ss' ) THEN
            LWF%WF_ext = LWF%WF_ext + VST*PWF%WF_esp
         ENDIF
c        ---------------------------------------------------------------
c        ... Gaseous absorber: weighting functions
c        ---------------------------------------------------------------
         IF( INQ_WF('tg', 'WFT') ) THEN
            IF( TRIM(SCATT_MODE)=='ss' ) THEN
               LWF%WF_ext => PWF%WF_esp
               CALL SET_OUTPUT_CWF(JPOS, LWF, DO_PLANCK, TAC, E_W)
            ENDIF
         ENDIF
c        ---------------------------------------------------------------
c        ... General contribution into Weighting functions
c            for Rayleigh, aerosol, and clouds
c        ---------------------------------------------------------------
c         IF( INQ_WF('ray','WFN') .OR. INQ_WF('aer', 'WFT') 
         IF( Ray_to_WF .OR. INQ_WF('aer', 'WFT') 
     $        .OR. INQ_WF('cld','WFT') ) THEN
            DO I = 1,NL  ;  J = 0  
               T1 = DZ(I)/2.d0
               DO Js = 1,NSZA
                  DO Jv = JV_S(Js),JV_E(Js)
                     DO Ja = 1,N_AZ_VAR  ;  J = J + 1
                        PWF%WF_ssp(I,J,:,1) = T1*GSL%EXL(I,Jv)
     $                       *SSB%EXS(I,Js)/4.d0/GSL%MuV(Jv)
                     ENDDO
                  ENDDO
               ENDDO
c               write(*,'(1x,e15.7,*(2x,E12.4))') ZZZ(i),
c     $              GSL%EXL(I,1)*SSB%EXS(I,1)/4.d0/GSL%MuV(1), 
c     $              T1, PWF%WF_ssp(I,1,1,1)
            ENDDO  ! over altitude levels (I)
            L1 = N_OBS_ALT(1)  ;  L2 = L1
            IF( .NOT.DO_SATELLITE ) L2 = L2 - 1
            PWF%WF_ssp(L1,:,:,1) = PWF%WF_ssp(L1,:,:,1)/DZ(L1)*DZR(L2)
         ENDIF
c        ---------------------------------------------------------------
c        ... Rayleigh: weighting functions
c        ---------------------------------------------------------------
         IF( Ray_to_WF) THEN 
            IF( TRIM(SCATT_MODE)=='ss' ) THEN
c              ... Rayleigh SS
               ALLOCATE( LWF%WF_scr(NL,NLO,NS_wf) ) 
               LWF%WF_ext => PWF%WF_esp
               FORALL( I=1:NL,Jw=1:NS_wf ) LWF%WF_scr(I,:,Jw) = 
     $              PWF%WF_ssp(I,:,Jw,1)*Phm_r(:,I,Nc_der(Jw))
               IF(INQ_WF('ray', 'WFN')) 
     $              CALL SET_OUTPUT_RAY(JPOS, LWF, LOA%SCR, E_W)
            ELSE
c              ... Rayleigh MS
               FORALL( I=1:NL,Jw=1:NS_wf ) LWF%WF_scr(I,:,Jw) = 
     $              LWF%WF_scr(I,:,Jw) 
     $              + VST*PWF%WF_ssp(I,:,Jw,1)*Phm_r(:,I,Nc_der(Jw))
            ENDIF
         ENDIF
c        ---------------------------------------------------------------
c        ... Aerosol: weighting functions
c        ---------------------------------------------------------------
         IF(INQ_WF('aer', 'WFT') ) THEN 
            IF( Dwr_mi ) THEN
               ALLOCATE( Dhm_x(NLO,NL,Nvs,N_dwr) ) ; Dhm_x = 0.d0
               DO Idr = 1,N_dwr  
                  DO L = 1,LA_N  ;  L1 = LA_S(L)  ;  L2 = LA_E(L)
                     IF( .NOT.Mie_layer(L1) ) CYCLE
!                            WRITE(*,*) '   SS used for layers: ', L1, L2
                     P_MOM => OSM%DER_ALL(1:Leg_aer,L,1,Idr)
                     DO J = 1,NLO
                        Dhm_x(J,L1:L2,1,Idr) = 
     $                       SUM( Plm_ss(1:Leg_aer,J)*P_MOM(:) )
                     ENDDO
                     IF( Nm_vs == 1 ) CYCLE 
                     P_MOM => OSM%DER_ALL(1:Leg_aer,L,5,Idr)
                     DO J = 1,NLO
                        T1 = SUM( Plm_as(1:Leg_aer,J)*P_MOM(:) )
                        FORALL( K = 2:3 ) Dhm_x(J,L1:L2,K,Idr) 
     $                       = Rot_l(J,K,2)*T1
                     ENDDO
                  ENDDO  ! over aerosol layers (L)                  
               ENDDO  ! over PSD derivatives (Idr)
            ENDIF  ! Dwr_mi
            
            IF( TRIM(SCATT_MODE)=='ss' ) THEN
c              ... Aerosol SS
               ALLOCATE( LWF%WF_sca(NL,NLO,NS_wf,1+N_dwr) ) 
               LWF%WF_ext => PWF%WF_esp
               FORALL( I=1:NL,Jw=1:NS_wf ) LWF%WF_sca(I,:,Jw,1) = 
     $              PWF%WF_ssp(I,:,Jw,1)*Phm_a(:,I,Nc_der(Jw))
c              ... Derivatives of expansion coefficients of scattering
c                  matrix w.r.t. PSD parameters
               IF( Dwr_mi ) THEN
                  DO Idr = 1,N_dwr  
                     FORALL( I=1:NL,Jw=1:NS_wf ) 
                        LWF%WF_sca(I,:,Jw,1+Idr) = PWF%WF_ssp(I,:,Jw,1)
     $                    *Dhm_x(:,I,Nc_der(Jw),Idr)*HEL(I)
                     END FORALL
                  ENDDO  ! over PSD derivatives (Idr)
               ENDIF  ! Dwr_mi aerosol
               CALL SET_OUTPUT_AER
     $              (JPOS, LWF, LOA%SCA, LOA%EXA, DO_PLANCK, E_W, POS)
            ELSE
c              ... Aerosol MS
               FORALL( I=1:NL,Jw=1:NS_wf )  
               LWF%WF_sca(I,:,Jw,1) = LWF%WF_sca(I,:,Jw,1) 
     $              + VST*PWF%WF_ssp(I,:,Jw,1)*Phm_a(:,I,Nc_der(Jw))
     $              *HEL(I)/LOA%SCA(I)
               END FORALL

               IF( Dwr_mi ) THEN
                  DO Idr = 1,N_dwr 
                     FORALL( I=1:NL,Jw=1:NS_wf ) 
                     LWF%WF_sca(I,:,Jw,1+Idr) = LWF%WF_sca(I,:,Jw,1+Idr)
     $                    + VST*PWF%WF_ssp(I,:,Jw,1)
     $                    *Dhm_x(:,I,Nc_der(Jw),Idr)*LOA%SCA(I)
                     END FORALL
                  ENDDO  ! over PSD derivatives (Idr)
               ENDIF  ! Dwr_mi
            ENDIF  ! 'ss' mode

         ENDIF  ! INQ_WF('aer', 'WFT')
c        ---------------------------------------------------------------
c        ... Clouds: weighting functions
c        ---------------------------------------------------------------
         IF( INQ_WF('cld', 'WFT') ) THEN

            IF( Deriv_phc ) THEN
c           ... The dependence of phase function on effective  
c               radius is accounted for in WFs calculations 
c           ---------------------------------------------------------
               ALLOCATE( Chm_x(NLO,NL,Nm_ice,Nvs) ) ; Chm_x = 0.d0
               DO L = 1,N_CLD_LAYERS  
                  Jt = J_TOP(L)+1 ; Jb = J_BOT(L)-1 ; KK = 2
                  DO Jmc = 1, MLCS(L)%N_pha
                     N_db = MLCS(L)%Npf(Jmc)%Ndb_ref
                     Rdb => MLCS(L)%Npf(Jmc)%Pdb_ref(:)
                     IF( MLCS(L)%Npf(Jmc)%Y_Scm ) THEN
                        CALL DERIV_CLD_SCM(OSM%Der_CLD_SCM, Chm_x, *880)
                        CYCLE
                     ENDIF
                     CALL DERIV_CLD( OSM%Der_CLD_ALL, Chm_x )
                     KK = KK + 2
                  ENDDO  ! over water/ice phase (Jmc)
               ENDDO  ! over cloud layers (L)
            ENDIF  ! Deriv_phc

            IF( TRIM(SCATT_MODE)=='ss' ) THEN
c           ... Cloud SS
c           ---------------------------------------------------------
               ALLOCATE( LWF%WF_scc(NL,NLO,NS_wf), ! A
     $              LWF%Dph_Dre(NL,NLO,NS_wf), LWF%DT2(NL)) ! A
c              ... It is used in "st_output_cld.f" for calculation
c                  WFs for CTH, CBH, and TAU. Multiplier 2.d0/DZ(:)
c                  removes quadrature coefficients introduced above
c                  in the "General contribution into WFs ..." section
               LWF%DT2(:) = PWF%WF_ssp(:,1,1,1)*Phm_c(1,:,1)*2.d0/DZ(:)

               LWF%WF_ext => PWF%WF_esp
               DO I = 1, NL ; T1 = DZ(I)/2.d0
                  FORALL( J=1:NLO,Jw=1:NS_wf )  WF_DUM(I,J,Jw) =
     $                 PWF%WF_ssp(I,J,Jw,1)*Phm_c(J,I,1) /T1
               ENDDO
c              ... WF at altitudes below the observation point
               DO I = 1,NL-1 ;  T1 = DZ(I)/2.d0
                  FORALL( J=1:NLO ) LWF%WF_scc(I,J,1) = 
     $                 WF_DUM(I,J,1)*T1
               ENDDO

               IF( Deriv_phc ) THEN
                  LWF%Dph_Dre = 0.d0
                  FORALL( I=1:NL ) LWF%Dph_Dre(I,:,1) =  
     $                 PWF%WF_ssp(I,:,1,1)*Chm_x(:,I,1,1)
               ENDIF  ! Deriv_phc

c              ... Redefine the quadrature coefficient at the top of cloud layer 
               DO L = 1,N_CLD_LAYERS  ;  I = J_TOP(L)+1
                  T1 = DZR(I)/DZ(I)
                  LWF%WF_ext(I,:,:) = LWF%WF_ext(I,:,:)*T1
                  LWF%WF_scc(I,:,1) = LWF%WF_scc(I,:,1)*T1
                  IF( .NOT.Deriv_phc ) CYCLE
                  LWF%Dph_Dre(I,:,1) = LWF%Dph_Dre(I,:,1)*T1
               ENDDO  ! over cloud layers (L)

               CALL SET_OUTPUT_CLD(JPOS, LWF, LOA%Cil, LOA%Cir,  
     $              LOA%DP_CLD, LOA%EXC, LOA%SCC, LOA%DM_CLD, Abs_soot, 
     $              TE%ADD_PLANCK, E_W, LOA%SCC_NOSCALE(:,1))

               DEALLOCATE( LWF%WF_scc, LWF%Dph_Dre, LWF%DT1, LWF%DT2) ! D
            ELSE
c           ... Cloud MS
c            ---------------------------------------------------------
c               Arrays LWF%DT1(NL) and LWF%DT2(NL) are allocated in 
c                  this case in the subroutine "st_adjoint_a.f"
c           ---------------------------------------------------------
               WHERE( LOA%SCC(:) /= 0.d0 )
                  HEL(:) = LOA%SCC_NOSCALE(:,1)/LOA%SCC(:)
               ELSEWHERE
                  HEL(:) = 0.d0
               END WHERE
c               DO I = 1, NL
c                  WRITE(*,'(1x,I4,*(2x,e12.4))') I, HEL(I),
c     $                 SUM( Phm_c(:,I,1) ), SUM( Phm_mc(:,I,1,1) )
c              ENDDO
cc              ... Array LWF%WF_scc was allocated in this case
cc                  in the subroutine "st_adjoint_a.f"
c               FORALL( I=1:NL ) LWF%WF_scc(I,:,1) = LWF%WF_scc(I,:,1)
c     $              + VST*PWF%WF_ssp(I,:,1,1)*Phm_c(:,I,1)*HEL(I)
c -------------------------------------------------------------------------
               DO I = 1, NL
                  T1 = DZ(I)/2.d0
                  FORALL( J=1:NLO,Jw=1:NS_wf )  WF_DUM(I,J,Jw) =
     $                VST*PWF%WF_ssp(I,J,Jw,1) /T1*Phm_c(J,I,1)*HEL(I)
               ENDDO
c              ... WF at altitudes below the observation point
               DO I = 1,NL-1
                  FORALL( J=1:NLO ) WF_AVER(I,J,1) = 
     $                 WF_DUM(I,J,1)*DZ(I)/2.d0
               ENDDO
               FORALL( I=1:NL ) LWF%WF_scc(I,:,1) = LWF%WF_scc(I,:,1)
     $              + WF_AVER(I,:,1)
c              ... Add single scattering contribution to the MS one
c                  saved in subroutines "st_pp_wf.f" or "st_ps_wf.f"
               LWF%DT2(:) = LWF%DT2(:) + VST
     $              *PWF%WF_ssp(:,1,1,1)*Phm_c(1,:,1)*HEL(:)*2.d0/DZ(:)
               IF( Deriv_phc ) THEN
                  FORALL( I=1:NL ) 
     $              LWF%Dph_Dre(I,:,1) = LWF%Dph_Dre(I,:,1) 
     $                 + VST*PWF%WF_ssp(I,:,1,1)*Chm_x(:,I,1,1)*HEL(I)
               ENDIF  ! Deriv_phc
               
c              ... Redefine the quadrature coefficient at the top of cloud layer 
               DO L = 1,N_CLD_LAYERS  ;  I = J_TOP(L)+1 
                  T1 = DZR(I)/DZ(I)
                  LWF%WF_ext(I,:,:) = LWF%WF_ext(I,:,:)*T1
                  LWF%WF_scc(I,:,1) = LWF%WF_scc(I,:,1)*T1
                  IF( .NOT.Deriv_phc ) CYCLE
                  LWF%Dph_Dre(I,:,1) = LWF%Dph_Dre(I,:,1)*T1
               ENDDO  ! over cloud layers (L)

            ENDIF  ! Cloud MS
         ENDIF  ! INQ_WF('cld', 'WFT')
c        ---------------------------------------------------------------
      ENDIF  ! WF_FLAG
c     ========== End block for all WFs =================================

      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN

 880  WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)
      RETURN 1

      CONTAINS
c    *******************************************************************
      
c     Under this CONTAINS belonging to "gpp_ss_stokes.f" 
c           are following subroutines: "deriv_cld"
      SUBROUTINE DERIV_CLD( S_MAT, Out_x )

      REAL(KND) S_MAT(Leg_cld,NL_Cld,Nphr,Nm_ice,Nm_vs), 
     $     Out_x(NLO,NL,Nm_ice,Nvs)
c     ... Local variables
      INTEGER   I1, I2
      REAL (KND) DL
c     ----------------------------------------------------
      DO Jr = Jt,Jb
c       ... Effective radius at a given altitude 
         Aef = Prof_Cpar(Jr,KK) 
         CALL LIN_SEARCH_NEW(Aef, DL, I1, I2, N_db, Rdb)
c         write(*,*) '   I1, I2, DL   ' , I1, I2, DL
c        ... Interpolate on a given effective radius
         Rez(:) = S_MAT(:,L,I2,Jmc,1)*DL 
     $        + S_MAT(:,L,I1,Jmc,1)*(1.d0 - DL)
         DO J = 1,NLO
            Out_x(J,Jr,Jmc,1) = SUM(Plm_ss(1:Leg_cld,J)*Rez(:))
         ENDDO      
         IF( Nm_vs == 1 ) CYCLE ! if scalar case 
c        ========================================
c        ... Interpolate on a given effective radius
         Rez(:) = S_MAT(:,L,I2,Jmc,5)*DL 
     $        + S_MAT(:,L,I1,Jmc,5)*(1.d0 - DL)
         DO J = 1,NLO               
            T1 = SUM( Plm_as(1:Leg_cld,J)*Rez(:) ) 
            FORALL( K = 2:3 ) Out_x(J,Jr,Jmc,K) = T1*Rot_l(J,K,2)
         ENDDO  ! over L-of-S (J)
      ENDDO  ! over sub-layers within a cloud (Jr)

      RETURN
      END SUBROUTINE DERIV_CLD
c ********************************************************************** 

      SUBROUTINE DERIV_CLD_SCM( S_MAT, Out_x, * )

      CHARACTER (LEN=13), PARAMETER :: R_NAME = 'DERIV_CLD_SCM'
      REAL(KND) S_MAT(Ydb_ang,NL_Cld,Nphr,Nm_ice,Nm_vs), 
     $     Out_x(NLO,NL,Nm_ice,Nvs)
c     ... Local variables
      INTEGER   I1, I2
      REAL (KND) DL, Rez(Ydb_ang)
c     ----------------------------------------------------
      CALL ST_TRACK (R_NAME, 1, 'I')
      DO Jr = Jt,Jb
c        ... Effective radius at a given altitude 
         Aef = Prof_Cpar(Jr,KK) 
         CALL LIN_SEARCH_NEW(Aef, DL, I1, I2, N_db, Rdb)
c               WRITE(*,*) '  Aef = ', Jr, Aef
c               WRITE(*,*) S_MAT(2,L,I1,Jmc,1),  S_MAT(2,L,I2,Jmc,1)
c        ... Interpolate on a given R_eff for all scat. angles
         Rez(:) = S_MAT(:,L,I2,Jmc,1)*DL 
     $        + S_MAT(:,L,I1,Jmc,1)*(1.d0 - DL)
c        ... Interpolate on scattering angles COS_SCAT
         CALL AL_INTERP (Ydb_ang, SCAT_ANG, Rez,
     $        NLO, COS_SCAT, DUM, .TRUE., *880)
         Out_x(:,Jr,Jmc,1) = DUM(:)
         IF( Nm_vs == 1 ) CYCLE ! if scalar case 
c        ... Interpolate on a given effective radius
         Rez(:) = S_MAT(:,L,I2,Jmc,5)*DL 
     $        + S_MAT(:,L,I1,Jmc,5)*(1.d0 - DL)
c        ... Interpolate on scattering angles COS_SCAT
         CALL AL_INTERP (Ydb_ang, SCAT_ANG, Rez,
     $        NLO, COS_SCAT, DUM, .TRUE., *880)
         DO J = 1,NLO 
            FORALL(K=2:3) Out_x(J,Jr,Jmc,K) = DUM(J)*Rot_l(J,K,2)
         ENDDO  ! over L-of-S (J)
      ENDDO  ! over sub-layers within a cloud (Jr)

      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN

 880  WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)
      RETURN 1

      END SUBROUTINE DERIV_CLD_SCM

      END
C***********************************************************************
C***********************************************************************
      SUBROUTINE REFL_SS(JPOS, JJESFT, POS, ALBEDO, GSL, LOA, PWF, SSB, 
     $     TE, SRR, SRR_Jact, Phm, Rad_ss, E_W, Flu_int, LWF)
c     ... Call in "gpp_ss_stokes.f" 

      USE AEROSOLS,    ONLY: IDX_AER
      USE BRDF,        ONLY: BRDF_PARAM, DO_OC_COMP, Wgr_svk, Nwl_veg, 
     $                       WL_veg, Alb_svk, N_svk
      USE BRDF_SUBRS,  ONLY: BRDF_ACT_DOM

      USE CLOUDS,      ONLY: DO_CLOUD, N_CLD_LAYERS, J_TOP, J_BOT

      USE FLU_LAND,    ONLY: FLU_L
      USE GRIDS,       ONLY: NL => NLAYERS, DZR, DZ, ZZZ, SOL_GRD
      USE INTERN_DIMS, ONLY: Nvs, Nm_vs, NL_pla
      USE LOG_PARS,    ONLY: LOG_LEVEL, N_DSO
      USE PS_INTERN,   ONLY: COS_SCAT 
      USE ST_VARS,     ONLY: Pie, BRDF_FLAG, DO_RAMAN, SURF_TYPE, 
     $                       DO_PLANCK, WF_FLAG, Nc_der, NS_wf
      USE SPHER_PARS,  ONLY: NSZA => N_SOL_ANG, N_OBS_ALT, ANG_OUT,
     $                       NLO => N_LOS_OUT, N_AZ_ANG, ANG_SEL_FLAG, 
     $                       JV_S, JV_E, N_AZ_VAR, NLSA => N_LOS_ANG
      USE TOOLS,       ONLY: INQ_WF
      USE TYPE_VARS,   ONLY: KND, DCMP
      USE TYPES,       ONLY: LOA_type, TE_TYPE, SSB_TYPE, RRS_TYPE, 
     $                       PWF_TYPE, GSL_TYPE, LWF_TYPE
      USE WATER,       ONLY: SS_PDF
c     ------------------------------------------------------------------

      CHARACTER (LEN=*), PARAMETER :: R_NAME = 'REFL_SS'
c     Subroutine arguments:
c     ---------------------
      INTEGER,    INTENT(IN) ::  JPOS, JJESFT, SRR_Jact
      REAL (KND), INTENT(IN) ::  POS, E_W, ALBEDO, Phm(NLO,NL,Nvs), 
     $     Flu_int
      REAL (KND), INTENT(OUT)::  Rad_ss(NL,NLO,Nvs)
c     ------------------------------------------------------
      TYPE (GSL_type) :: GSL    ! Angles 
      TYPE (SSB_type) :: SSB    ! Spherical solar beam
      TYPE (LOA_type) :: LOA    ! Local optical parameters of atmosphere
      TYPE (LWF_type) :: LWF    ! Local weighting functions
      TYPE (PWF_type) :: PWF    ! Weighting functions
      TYPE (TE_type)  :: TE     ! Thermal emission
      TYPE (RRS_type) :: SRR    ! RRS related parameters
c     ------------------------------------------------------
      INTEGER   I, J, Js, Jv, Ja, Jc, Jw, Jw1, L, N_1, Zn
      REAL (KND)  DTA, CV, CS, ECV, SOU, Ck, T1, T2, SMd, Cks, ALW, EXS,
     $     Ewei, Mus, T_AU(NL), Iaj(NL-1,NLO,Nvs), Taj(NL-1,NLO,Nvs),
     $     Phi(N_AZ_ANG), SC_Md(NL-1,NSZA,NLSA), Tls(NLO,Nvs),
     $     I0_sol(NLO,Nvs), I0_te(NLO,Nvs),
     $     D_DS(2,NL), SSSF(NL,Nvs), EX_L(NL), S1(Nvs), D2
      REAL (KND), POINTER ::  Mue(:,:), PXS(:,:), MuV(:)
      REAL (KND), ALLOCATABLE ::  Wa2(:,:,:), Wa1(:,:,:), R_wf(:,:)
      COMPLEX (kind(0.0d0)) :: Muc
c     ==================================================================

      CALL ST_TRACK (R_NAME, 1, 'I')

      ALLOCATE( R_wf(NLO,Nvs) ) ;  R_wf = 0.d0
      I0_te = 0.d0
      
      Mue => SSB%Mue  ;  MuV => GSL%MuV
      Phi(:) = ANG_OUT(3,1:N_AZ_ANG)

      T_AU(1) = 0.d0
      DO  J = 1, NL - 1
         T_AU(J+1) = T_AU(J) + (LOA%AL(J) + LOA%AL(J+1))*0.5D0*DZR(J)
      ENDDO

      IF( DO_RAMAN ) THEN
c        ... Upward single-scattered rotational Raman (The contribution
c            of inelastic scattering will be calculated only and stored
c            in the pointer-array DEL_RRS(:,:,:)).
c            Call sequence:
c            refl_ss.f --> ref_rrs.f --> sum_ram_ref.f
         CALL REF_RRS
c        ... Return for an excitation wavelength (reflection mode)
         IF( SRR%Exi_WL(JPOS) ) RETURN 
      ELSE
c        ... Array SC_Md(I,Js,Jv) contains results of the analytical 
c            integration of the solar exponent or its approximation 
c            (in the pseudo-spherical atmosphere) over each layer along
c            line-of-sight direction. If DO_RAMAN=.TRUE. this array 
c            is calculated in routine "sum_ram_ref.f".
         IF( .NOT.TE%NO_SUN ) THEN
            DO I = 1,NL - 1
               DTA = T_AU(I+1) - T_AU(I)
               DO Js = 1,NSZA
                  CS = Mue(Js,I) ; SOU = DEXP( -DTA/CS )
                  DO Jv = JV_S(Js),JV_E(Js) 
                     CV = MuV(Jv) ; ECV = DEXP( -DTA/CV ) 
                     SMd = Cs*Cv/(Cs + Cv)*( 1.d0 - SOU*ECV )
                     Zn = -1
                     DO Jc = 1,SSB%N_sph(Js,I)-1
                        Ck = DTA/DBLE(Jc)/Pie ; Muc = CS*Ck/DCMP(Ck,-Cs)
                        SMd = SMd + AIMAG( Muc*Cv/(Muc + Cv) )
     $                       *SSB%SIN_COEF(Js,I,Jc)*(1.d0 - Zn*SOU*ECV)
                        Zn = -Zn
                     ENDDO  ! over expansion coeff. of solar exp. (Jc)
                     SC_Md(I,Js,Jv) = SMd !/SSB%EXS(I,Js)/CV
                  ENDDO  ! over viewing angles (Jv)
               ENDDO  ! over solar angles (Js)
            ENDDO  ! over altitude layers (I)
         ENDIF  ! Including sun light
      ENDIF  ! DO_RAMAN
      
      Iaj = 0.d0  ;  Taj = 0.d0
      I_aj: Block
        IF( DO_PLANCK ) CALL ADD_REF_PLA ! ---> Taj
        IF( TE%NO_SUN ) EXIT I_aj
c       ... Array Iaj(I,:,:) contains results of the numerical integration 
c           of the scattering coefficient and analytical of the solar source 
c           function (SC_Md) over each layer along the line-of-sight
        DO I = 1,NL-1
           ALW = LOA%AL(I)+LOA%AL(I+1)  ;  J = 0
           DO Js = 1,NSZA               ;  EXS = SSB%EXS(I,Js)/ALW/4.d0
              DO Jv = JV_S(Js),JV_E(Js) ; CV = MuV(Jv)
                 DO Ja = 1,N_AZ_VAR     ;  J = J + 1
c                    Iaj(I,J,:) = Iaj(I,J,:) + SC_Md(I,Js,Jv)*EXS/CV*
                    Iaj(I,J,:) = SC_Md(I,Js,Jv)*EXS/CV*
     $                   (Phm(J,I,:) + Phm(J,I+1,:))
c     $                   Phm(J,I,:)*2.d0
c     $                   Phm(J,I+1,:)*2.d0
                 ENDDO  ! over azimuthal angles (Ja)
              ENDDO  ! over line-of-sight angles (Jv)
           ENDDO  ! over solar zenith angles (Js)
        ENDDO  ! over all altitude layers (I)
      END Block I_aj

      Tls = 0.d0 
c     ... Calculate reflection of direct solar beam from surface
      IF( BRDF_FLAG ) THEN
c     ... BRDF reflection from surface: Tls as output
         CALL REFLECTED_DIR  ! ---> Tls
      ELSE
c     ... Lambertian reflection from surface
         IF( .NOT.TE%NO_SUN ) THEN
c        ... Solar radiation is switch on           
            J = 0
            DO Js = 1,NSZA  ;  EXS = SSB%EXS(NL,Js)
               DO Jv = JV_S(Js),JV_E(Js) 
                  DO Ja = 1,N_AZ_VAR  ;  J = J + 1
c                     Tls(J,1) = ALBEDO*Mue(Js,NL-1)*EXS
                     Tls(J,1) = ALBEDO*Sol_grd(Js)*EXS
                  ENDDO
               ENDDO
            ENDDO
         ENDIF  ! Include solar light reflection
      ENDIF  ! BRDF_FLAG
c     ... Save contribution of reflected by surface solar light
      I0_sol(:,:) = Tls(:,:) 
c     ... Thermal emission contribution
      IF( DO_PLANCK ) THEN
         Tls(:,1) = Tls(:,1) + TE%PLANCK_SURF
         I0_te(:,1) = TE%PLANCK_SURF
      ENDIF
c     ... Surface fluorescence contribution
      IF( FLU_L )  Tls(:,1) = Tls(:,1) + Flu_int
c     ... Calculation of reflected radiation at all levels using
c         recurrence relation
      Rad_ss(NL,:,:) = Tls(:,:)
      DO I = NL-1,1,-1  ;  J = 0  ;  DTA = T_AU(I+1) - T_AU(I)
         DO Js = 1,NSZA
            DO Jv = JV_S(Js),JV_E(Js)  ;  ECV = DEXP(-DTA/Muv(Jv))
               DO Ja = 1,N_AZ_VAR  ;  J = J + 1
                  Tls(J,:) = Tls(J,:)*ECV + Iaj(I,J,:) + Taj(I,J,:)
               ENDDO
            ENDDO
         ENDDO
         Rad_ss(I,:,:) = Tls(:,:) 
      ENDDO
c ---------------------------------------------------------------------
!      WRITE(*,*) '    Check Rad_ss '
!      WRITE(*,*) Rad_ss(1,:,1)
c     -------------   Another integration shema ------------       
      D_DS(1,1) = 0.d0
      DO I = 2, NL
         D_DS(1,I) = (ZZZ(I-1) - ZZZ(I))/2.d0
      ENDDO
      D_DS(2,NL) = 0.d0
      DO I = 1, NL-1
         D_DS(2,I) = (ZZZ(I) - ZZZ(I+1))/2.d0
      ENDDO
c     SSB%EXS(I,Js)/4.d0/GSL%MuV(Jv)
c     ALW = LOA%AL(I)+LOA%AL(I+1)
c      GSL%EXL(I,1)
      D2 = 0.d0  ;  EX_L(1) = 1.0d0
      DO I = 2, NL
         D2 = D2 +
     $        LOA%AL(I-1)*D_DS(2,I-1) + LOA%AL(I)*D_DS(1,I) 
         EX_L(I) = DEXP(- D2)
      END DO
      S1(:) = 0.0d0
      DO I = 1, NL
!         WRITE(*,'(1x,F8.4,*(2x,ES12.4))') ZZZ(I),Phm(1,I,:)
c         WRITE(*,'(2x,F8.4,*(2x,ES15.7))')
c     $        ZZZ(I),  Phm(1,I,1), SSB%EXS(I,1)
         SSSF(I,:) = Phm(1,I,:) *SSB%EXS(I,1)/4.d0 
c         S1(:)= S1(:) + SSSF(I,:)*GSL%EXL(I,1)*(D_DS(1,I) + D_DS(2,I))
         S1(:)= S1(:) + SSSF(I,:)*EX_L(I)*(D_DS(1,I) + D_DS(2,I))
      END DO      
!          WRITE(*,*) '  LOA%Al ',   LOA%AL(NL-1:NL)
!      WRITE(*,*) '    Check S1 '
!      WRITE(*,'(*(2x,ES12.4))') S1(:)
c      DO J = 1, NL
c         WRITE(*,'(2x,I3, 2x,F8.4,2(2x,e12.4),2x,E15.7)')
c     $        IDX_AER(J), 
cc     $        ZZZ(J), D_DS(1,J), D_DS(2,J),GSL%EXL(J,1) 
c     $        ZZZ(J), D_DS(1,J), D_DS(2,J), EX_L(J) 
c      ENDDO
c ---------------------------------------------------------------------
      
      IF( .NOT.WF_FLAG ) THEN
         CALL ST_TRACK (R_NAME, 1, 'O')  ;  RETURN
      ENDIF
     
c     ... Weighting function for single scattered reflected radiation
      ALLOCATE( Wa2(NL,NLO,NS_wf), Wa1(NL,NLO,NS_wf) )
      Wa2 = 0.0d0  ; Wa1 = 0.0d0    ; N_1 = N_OBS_ALT(1)  ;  J = 0
      DO Js = 1,NSZA                ; Cs = Mue(Js,NL-1) 
         DO Jv = JV_S(Js),JV_E(Js)  ; Cv = MuV(Jv) 
            Mus = (1.d0/Cs + 1.d0/Cv)
            DO Ja = 1,N_AZ_VAR   ;  J = J + 1
               DO Jw = 1, NS_wf  ;  Jw1 = Nc_der(Jw)
c                 ... Contribution from bottom reflection:
                  T1 = I0_sol(J,Jw1)*GSL%EXL(NL,Jv)
                  T2 = I0_te(J,Jw1)*GSL%EXL(NL,Jv)
c   ... WF at altitudes below instrument (STB Eq.(WF-ss-ext) 1-3 terms)
                  Wa1(N_1:NL,J,Jw) = -T1*Mus - T2/Cv
c     $                 + TE%OPLANCK(N_1:NL)*GSL%EXL(N_1:NL,Jv)/Cv
                  IF( DO_PLANCK ) Wa1(N_1:NL,J,Jw) =  Wa1(N_1:NL,J,Jw)
     $                 + TE%OPLANCK(N_1:NL)*GSL%EXL(N_1:NL,Jv)/Cv
                  DO I = N_1, NL-1
                     Iaj(I,J,Jw1) = Iaj(I,J,Jw1)*GSL%EXL(I,Jv)*Mus
                     Taj(I,J,Jw1) = Taj(I,J,Jw1)*GSL%EXL(I,Jv)/Cv
                  ENDDO  ! over altitudes below instrument (I)
c   ... WF at altitudes below instrument (STB Eq.(WF-ss-ext) 4 and 5 terms)
                 DO I = N_1, NL-1
                     Wa2(I,J,Jw) = -SUM( Iaj(I:NL-1,J,Jw1) )
     $                    - SUM(Taj(I:NL-1,J,Jw1))
                  ENDDO  ! over altitudes below instrument (I)
c                 ... WF at altitudes above instrument
                  T2 = -SUM( Iaj(N_1:NL-1,J,Jw1) )/Mus/Cs
                  DO I = 1, N_1-1
                     Wa2(I,J,Jw) = T2  ;  Wa1(I,J,Jw) = -T1/Cs
                  ENDDO ! over altitudes above instrument (I)
               ENDDO ! over number of Stokes (Jw)
            ENDDO  ! over azimuthal angles (Ja)
         ENDDO  ! over zenith angles (Jv)
      ENDDO  ! over solar angles (Js)

c        ... Without quadrature coefficients for WFs of CTH, CBH, TAU 
c            !!! Single LoS and without polarization !!!
      IF( INQ_WF('cld', 'WFT') ) THEN
         DO L = 1, NL
            IF( LOA%SCC(L) > 0.d0 ) CYCLE
c           ... Set to zero outside of cloud layers:
            Wa1(I,:,:) = 0.d0 ; Wa2(I,:,:) = 0.d0
         ENDDO  ! over layers (L)
         LWF%DT1(:) = LWF%DT1(:) + ( Wa1(:,1,1) + Wa2(:,1,1) )   
      ENDIF  ! 'CLD'
c     ... WF at altitudes above the observation point

c      WRITE(*,*) '   Check N_1 =  ', N_1
      DO I = 1, N_1-1
         FORALL( J=1:NLO,Jw=1:NS_wf ) PWF%WF_esp(I,J,Jw) = 
     $        ( Wa1(I,J,Jw) + Wa2(I,J,Jw) )*DZ(I)/2.d0
      ENDDO
      
c     ... WF at the observation point
      I = N_1
      IF( I > 1 ) THEN          ! correction 23.08.13
         DO J = 1,NLO
            PWF%WF_esp(I,J,:) = 
     $           (Wa1(I-1,J,:) + Wa2(I-1,J,:))*DZR(I-1)/2.d0 
     $           + (Wa1(I,J,:) + Wa2(I,J,:))*DZR(I)/2.d0
         ENDDO
      ELSE
         DO J = 1,NLO
            PWF%WF_esp(I,J,:) = (Wa1(I,J,:) + Wa2(I,J,:))*DZR(I)/2.d0
         ENDDO
      ENDIF
      
c        ... WF at altitudes below the observation point
      DO I = N_1+1,NL-1
         FORALL( J=1:NLO ) PWF%WF_esp(I,J,:) = 
     $        ( Wa1(I,J,:) + Wa2(I,J,:) )*DZ(I)/2.d0
      ENDDO
c      DO I = N_1+1,NL-1
c         FORALL( J=1:NLO ) PWF%WF_esp(I,J,:) = 
c     $        ( (Wa1(I-1,J,:)+Wa1(I+1,J,:))/2.d0 +
c     $        (Wa2(I-1,J,:)+Wa2(I+1,J,:))/2.d0 )*DZ(I)/2.d0
c      ENDDO
      PWF%WF_esp(NL,:,:) =
     $     ( Wa2(NL-1,:,:) + Wa2(NL,:,:) )/2.d0*DZ(NL)/2.d0
     $     + ( Wa1(NL-1,:,:) + Wa1(NL,:,:) )/2.d0*DZ(NL)/2.d0

      IF( DO_PLANCK ) THEN
         ALLOCATE( LWF%WF_TE(NL_pla,NLO) )
         LWF%WF_TE = 0.d0
      ENDIF
         
      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN
      CONTAINS
c     Under this CONTAINS belonging to "refl_ss.f" 
c           are following subroutines:
c     ref_rrs.f
c     sum_ram_ref.f
c     add_ref_pla.f
c     reflected_dir.f
c
c **********************************************************************
c     Implementation of the polarization for rotational Raman
c                                 Last modification: 04 Apr 2012
c **********************************************************************
      SUBROUTINE REF_RRS
c     ... Call in "refl_ss.f"

      USE GT_POLYN,    ONLY: Plm_ss 
      USE INTERN_DIMS, ONLY: Nlg_Ray, Nm_vs
      USE IRRAD,       ONLY: SOL_FLUX 
      USE RAMAN,       ONLY: WL_FACT, LINE_GAS, Sigma_RR, Phot_to_Wat
      USE ST_VARS,     ONLY: WL_GRID, TSM
      USE VECT_VARS,   ONLY: Rot_l
c     ------------------------------------------------------------------

      INTEGER  K, Iae, Iac, Jex, J_act, Nsf, To_act, R_lin
      REAL (KND), POINTER :: P_MOM(:)
      REAL (KND)  IRR, IR0, COF(NL), Rhm(NLO,Nvs), R_j(NL,SRR%M_ESF), 
     $     EXCO(NL,SRR%M_ESF), Wes(SRR%M_ESF), Plm_as(Nlg_Ray,NLO)
c     ==================================================================

      DO J = 1,NLO
         Rhm(J,1) = SUM( Plm_ss(1:NLg_Ray,J)*SRR%RAM_MOMS(1:NLg_Ray,1) )
      ENDDO
c     ... Rotational Raman phase matrix for a given scattering angles
      IF( Nm_vs > 1 ) THEN
         Rhm(:,4) = 0.d0
         CALL WIGNER_PLG(COS_SCAT, NLO, 2, Nlg_Ray, Plm_as )
         P_MOM => SRR%RAM_MOMS(1:NLg_Ray,5) 
         DO J = 1,NLO
            T1 = SUM( Plm_as(1:NLg_Ray,J)*P_MOM(:) )
            FORALL( K = 2:3 ) Rhm(J,K) = T1*Rot_l(J,K,2)
         ENDDO
      ENDIF

      Ewei = E_W
c     ... Energy loss at actual wavelengths due to rotational 
c         Raman scattering 
      IF( SRR%ACT_WL(JPOS) ) THEN
c        ... Set extraterrestrial solar flux at actual WL corresponding
c            to its position in the extended grid
         IRR = -SOL_FLUX(JPOS)/Pie  
         To_act = SRR_Jact
c         IRR = 0.d0

c        ... Save extinction coefficient for all actual wavelengths
         SRR%EXT_ACT(:,To_act,JJESFT) = LOA%AL(:)

         IF( TRIM(TSM)=='speed' ) THEN
c           ... Save solar exponent
            SRR%EXS_ACT(:,To_act,:) = SRR%EXS_ACT(:,To_act,:) 
     $           + SSB%EXS(:,:)*E_W
         ENDIF

c        ... Set total Raman scattering coefficient 
         COF(:) = SRR%R_TOT*LOA%SCR(:)
c                 WRITE(*,*) ' To_act  ', To_act
c                 WRITE(*,*) '    IRR  ', IRR
c                 WRITE(*,*) '    COF  ', SUM(COF(:))
c                 WRITE(*,*) 'LOA%AL(:)', SUM(LOA%AL(:))
c                 WRITE(*,*) ' Sol. exp:  ', Nl, SSB%EXS(Nl,1)
         Nsf = 1
         EXCO(:,1) = LOA%AL(:) ; Wes(1) = 1.d0
         R_j(:,1) = 1.d0
         PXS => SSB%EXS(:,:)

         CALL SUM_RAM_REF(To_act, Nsf, IRR, COF, EXCO, Wes, R_j, Rhm)

         IF( To_act==SRR%N_ACT.AND.TRIM(TSM)=='speed'.AND. 
     $       JJESFT==SRR%NESFT(SRR%N_ACT) ) THEN
c           ... Over all actual wavelengths which here considered
c               as excitation wavelengths
            Ewei = 1.d0
            DO Iac = 1, SRR%N_ACT 
               Iae = SRR%A2E(Iac)
               IR0 = SOL_FLUX(Iae)/Pie
               DO J_act = 1,SRR%NUM_IN(Iae)
                  To_act = SRR%ACT_IN(Iae,J_act)
                  IRR = IR0
                  IF( Phot_to_Wat ) IRR = IR0*WL_GRID(SRR%A2E(To_act))
     $                 /WL_GRID(Iae)
                  R_lin  = SRR%RIN_IN(Iae,J_act)
                  COF(:) = WL_FACT*Sigma_RR(R_lin,:)
     $                 /WL_GRID(SRR%A2E(To_act))**4
     $                 *SRR%GAM_WL(LINE_GAS(R_lin),Iac)
                  PXS  => SRR%EXS_ACT(:,Iac,:)

                  Nsf = SRR%NESFT(To_act)
c                 ... Extinction coefficient at actual wavelength
                  EXCO(:,1:Nsf) = SRR%EXT_ACT(:,To_act,1:Nsf)
                  Wes(1:Nsf) = SRR%WEI(To_act,1:Nsf)
c                 ... Ratio of extinction coefficients at excitation
c                     to the actual wavelengths (see Eq.(SS-RRS) in STB)
                  DO J = 1,Nsf
                     R_j(:,J) = SRR%EXT_ACT(:,Iac,J)/EXCO(:,J)
                  ENDDO
                  CALL SUM_RAM_REF
     $                 (To_act, Nsf, IRR, COF, EXCO, Wes, R_j, Rhm)
               ENDDO
            ENDDO
         END IF
c     ... Energy gain at actual wavelengths due to rotational 
c         Raman scattering at excitation wavelengths
      ELSE
c        ... It is performed for wavelengths of the extended grid
c            which marked as excitation wavelengths. JPOS defines 
c            the wavelength number in the extended grid.

c        ... Set extraterrestrial solar flux at excitation wavelength

         IR0 = SOL_FLUX(JPOS)/Pie
         PXS => SSB%EXS(:,:)
c        ... Over all Raman lines within corresponding excitation bin 
         DO Jex = 1,SRR%NUM_IN(JPOS)
c           ... The actual wavelength number which becomes
c               contribution from Jex-th Raman line
            To_act = SRR%ACT_IN(JPOS,Jex) 
c           ... Raman line number corresponding to Jex-th excitation wavelength
            R_lin  = SRR%RIN_IN(JPOS,Jex)
c           ... The position of this actual wavelength in extended grid
            Iae = SRR%A2E(To_act)
            IRR = IR0
            IF( Phot_to_Wat ) IRR = IR0*WL_GRID(Iae)/WL_GRID(JPOS)
c           ... Array WL_GRID contains the extended wavelength grid
            COF(:) = WL_FACT*Sigma_RR(R_lin,:)/WL_GRID(Iae)**4
     $           *SRR%GAM(LINE_GAS(R_lin))
c           ... Number of ESFT coefficients at actual wavelength
            Nsf = SRR%NESFT(To_act)
c           ... Extract extinction coefficient at actual wavelength !!!
            EXCO(:,1:Nsf) = SRR%EXT_ACT(:,To_act,1:Nsf)
            Wes(1:Nsf) = SRR%WEI(To_act,1:Nsf)
            DO J = 1,Nsf
               R_j(:,J) = LOA%AL(:)/EXCO(:,J)
            ENDDO
            CALL SUM_RAM_REF(To_act, Nsf, IRR, COF, EXCO, Wes, R_j, Rhm)
         ENDDO                  ! over Raman lines within excitation bin 
      ENDIF
      END SUBROUTINE REF_RRS
c **********************************************************************

      SUBROUTINE SUM_RAM_REF(To_act, Nsf, IRR, COF, EXCO, Wes, R_j, Rhm)
c     ... Call in "ref_rrs.f"

      USE GRIDS,       ONLY: NL => NLAYERS, DZR 
      USE RTM_OUTPUT,  ONLY: DEL_RRS
c     ------------------------------------------------------------------
      INTEGER    To_act, Nsf
      REAL (KND) IRR, COF(NL), Rhm(NLO,Nvs), EXCO(NL,SRR%M_ESF),  
     $     R_j(NL,SRR%M_ESF), Wes(SRR%M_ESF)
c     ... Local variables
      INTEGER    Isf, Jo
      REAL (KND) Tau(NL), Del_ss(NL,NLO,Nvs)
c     ==================================================================

      Del_ss(:,:,:) = 0.d0

c     ... Do-loop over ESFT coefficients (if present)
      DO Isf = 1,Nsf
c        ... Optical thickness of jth layer is calculated for actual WL
         Tau(:) = 0.d0
         DO  J = 1,NL-1
            Tau(J+1) = Tau(J) + (EXCO(J,Isf) + EXCO(J+1,Isf))
     $           *0.5d0*DZR(J)
         ENDDO

c        ... Array SC_Md(I,Js,Jv) contains results of the analytical 
c            integration of the solar exponent or its approximation 
c            (in the pseudo-spherical atmosphere) over each layer
c            (see Eq.(SS-RRS) in STB)
         DO I = 1,NL - 1
            DTA = Tau(I+1) - Tau(I)
            DO Js = 1,NSZA
               CS = Mue(Js,I)/R_j(I,Isf) ; SOU = DEXP( -DTA/CS )
               DO Jv = JV_S(Js),JV_E(Js) 
                  CV = MuV(Jv) ; ECV = DEXP( -DTA/CV ) 
                  SMd = Cs*Cv/(Cs + Cv)*( 1.d0 - SOU*ECV )
                  Zn = -1
                  DO Jc = 1,SSB%N_sph(Js,I)-1 ! Proverit' R_j v cicle
                     Ck = DTA/DBLE(Jc)/Pie ; Muc = CS*Ck/DCMP( Ck,-Cs )
                     SMd = SMd + AIMAG( Muc*Cv/(Muc + Cv) )
     $                    *SSB%SIN_COEF(Js,I,Jc)*(1.d0 - Zn*SOU*ECV)
                     Zn = -Zn
                  ENDDO
                  SC_Md(I,Js,Jv) = SMd !/PXS(I,Js)/CV
               ENDDO
            ENDDO
         ENDDO

         DO I = 1,NL - 1
            ALW = EXCO(I,Isf) + EXCO(I+1,Isf)
            J = 0
            DO Js = 1,NSZA
               EXS = PXS(I,Js)/ALW/4.d0
               DO Jv = JV_S(Js),JV_E(Js) 
                  CV = MuV(Jv)
                  DO Ja = 1,N_AZ_VAR  ;  J = J + 1
                     Iaj(I,J,:) = SC_Md(I,Js,Jv)*EXS/CV*Rhm(J,:)
     $                    *(COF(I) + COF(I+1))
                  ENDDO         ! over azimuthal angles (Ja)
               ENDDO            ! -,,- zenith angles (Jv)
            ENDDO               ! -,,- solar zenith angles (Js)
         ENDDO                  ! -,,- all altitude layers (I)

         Tls(:,:) = 0.d0  
         DO I = NL-1,1,-1  ;  J = 0  ;  DTA = Tau(I+1) - Tau(I)
            DO Js = 1,NSZA
               DO Jv = JV_S(Js),JV_E(Js)  ;  ECV = DEXP(-DTA/Muv(Jv))
                  DO Ja = 1,N_AZ_VAR  ;  J = J + 1
                     Tls(J,:) = Tls(J,:)*ECV + Iaj(I,J,:)
                  ENDDO         ! over azimuthal angles (Ja)
               ENDDO            ! over zenith angles (Jv)
            ENDDO               ! over solar angles (Js)
            Del_ss(I,:,:) = Del_ss(I,:,:) + Tls(:,:)*Wes(Isf)
         ENDDO                  ! over layers (I)
      ENDDO                     ! over esft coefficients (Isf)

      Jo = N_OBS_ALT(1)

c      ... TEST TEST ... no ss Raman contribution
c      WRITE(*,*) '   !!! No Raman contribution in SS mode !!!   '
      DEL_RRS(To_act,:,:) = DEL_RRS(To_act,:,:) 
     $     + IRR*Del_ss(Jo,:,:)*Ewei

      END SUBROUTINE SUM_RAM_REF

c **********************************************************************
      SUBROUTINE ADD_REF_PLA
      REAL (KND) SC_pla(NL-1,NLSA)

      DO I = 1,NL - 1
         DTA = T_AU(I+1) - T_AU(I)
         CS = -TE%Mup(I) ; SOU = DEXP( TE%Mup(I)*DTA )
         DO Jv = 1,NLSA
            CV = MuV(Jv) ; ECV = DEXP( -DTA/CV ) 
            SMd = Cv/(1.d0 + CS*Cv)*(1.d0 - SOU*ECV)
            Zn = -1
            DO Jc = 1,TE%N_pla(I)-1
               Ck = DTA/DBLE(Jc)/Pie ; Cks = CS*Ck
               Muc = Ck/DCMP( Cks,-1.d0)
               SMd = SMd + AIMAG( Muc*Cv/(Muc + Cv) )
     $              *TE%PL_COEF(I,Jc)*(1.d0 - Zn*SOU*ECV)
               Zn = -Zn
            ENDDO
            SC_pla(I,Jv) = SMd
         ENDDO
      ENDDO
      
      DO I = 1,NL-1  ;  J = 0 ; T1 = TE%NPLANCK(I)
         DO Js = 1,NSZA
            DO Jv = JV_S(Js),JV_E(Js)  ; CV = MuV(Jv)
               DO Ja = 1,N_AZ_VAR  ;  J = J + 1
                  Taj(I,J,1) = SC_pla(I,Jv)*T1/CV
               ENDDO  ! over azimuthal angles (Ja)
            ENDDO  ! over line-of-sight zenith angles (Jv)
         ENDDO  ! over solar zenith angles (Js)
      ENDDO   ! over altitude layers (I)
      
      END SUBROUTINE ADD_REF_PLA
c **********************************************************************

      SUBROUTINE REFLECTED_DIR
      
      USE BRDF,     ONLY: Ker_cof => IVG_cof, L_SOL, WL_BRF, Nwl_BRF
      USE GT_POLYN, ONLY: XM
      USE ST_VARS,  ONLY: NN => NSTRMS
      USE PRO_SAIL, ONLY: NW_E, NW_S
c     ------------------------------------------------------------------

      CHARACTER (LEN=20) TXT
      INTEGER     K, Ws, Ws1, IN1, N4
      REAL (KND)  WL1(1), Area, DC, Uc, Aveg,
     $     BRF_OUT(Nvs*NLSA,Nvs*NSZA,N_AZ_ANG,1)
      REAL (KND), ALLOCATABLE:: R1_OUT(:,:,:,:), R_dir(:,:,:,:), 
     $     R_dif(:,:,:,:)
c     ==================================================================
      INTERFACE
         SUBROUTINE PROSAIL( N_inc, X_inc, N_ref, X_ref, N_Phi, X1, 
     $        R_dir, R_dif, Nder, dR_dir, dR_dif )
         USE PRO_SAIL,  ONLY: NW_S, NW_E
         USE TYPE_VARS, ONLY: KND
         INTEGER,   INTENT(IN) :: N_inc, N_ref, N_phi, Nder
         REAL(KND), INTENT(IN) :: X_inc(N_inc), X_ref(N_ref), X1(N_Phi)
         REAL(KND)  R_dir(N_ref, N_inc, N_phi, NW_E-NW_S+1),
     $        R_dif (N_ref, N_inc, N_Phi, NW_E-NW_S+1)
         REAL(KND), ALLOCATABLE :: dR_dir(:,:,:,:,:), dR_dif(:,:,:,:,:)
         OPTIONAL :: Nder, dR_dir, dR_dif
         END SUBROUTINE
      END INTERFACE

      WL1(1) = POS
      Area = 0.d0 ; BRF_OUT = 0.d0
      N4 = 1
      SELECT CASE( TRIM(SURF_TYPE) )
         CASE('Ross-Li')                 ; N4 = N4 + 1
         CASE('Ross-Li-snow','snow_fasmar')  ; N4 = N4 + 2
      END SELECT

      ALLOCATE( R1_OUT(Nvs*NLSA,Nvs*NSZA,N_AZ_ANG,N4) )

c     There is no reflection from ocean surface
c     -----------------------------------------
      IF( .NOT.ANY(DO_OC_COMP) ) THEN

         SELECT CASE( TRIM(SURF_TYPE) )
         CASE('Ross-Li','Ross-Li-snow','snow_fasmar')
c           ... Calculation should be performed for two kernel
            CALL BRDF_ACT_DOM( BRDF_PARAM, NLSA, MuV, N_AZ_ANG, Phi,
     $           NSZA, Sol_grd, 1, WL1, R1_OUT, 'none' )
c     $           NSZA, Mue(:,NL-1), 1, WL1, R1_OUT, 'none' )
            CALL LIN_SEARCH_NEW(POS, DC, Ws, Ws1, Nwl_veg, WL_veg)
            Uc = (1.d0 - Dc)
            BRF_OUT(1:Nvs*NLSA:Nvs,1:Nvs*NSZA:Nvs,:,1) = 
     $           (Uc*Ker_cof(1,Ws) + Dc*Ker_cof(1,Ws1))
            DO J = 1,Nwl_BRF 
               BRF_OUT(:,:,:,1) = BRF_OUT(:,:,:,1) +
     $              (Uc*Ker_cof(J+1,Ws) + Dc*Ker_cof(J+1,Ws1))
     $              *R1_OUT(:,:,:,J)
            ENDDO
         CASE('svk')
            IF( POS >= Wgr_svk ) THEN
               TXT = 'second' ; IN1 = 2
            ELSE
               TXT = 'first'  ; IN1 = 1
            ENDIF
            CALL BRDF_ACT_DOM( BRDF_PARAM, NLSA, MuV, N_AZ_ANG, Phi,
     $           NSZA, Sol_grd, 1, WL1, R1_OUT, TXT )
c     $           NSZA, Mue(:,NL-1), 1, WL1, R1_OUT, TXT )
            CALL LIN_SEARCH_NEW(POS, DC, Ws, Ws1, Nwl_veg, WL_veg)
            Aveg = (1.d0 - DC)*Alb_svk(Ws) + DC*Alb_svk(Ws1)
            BRF_OUT = Aveg*N_svk(IN1)*R1_OUT 

         CASE('mRPV_facet')
            CALL BRDF_ACT_DOM( BRDF_PARAM, NLSA, MuV, N_AZ_ANG, Phi,
     $           NSZA, Sol_grd, 1, WL1, R1_OUT, 'Gauss' )
c     $           NSZA, Mue(:,NL-1), 1, WL1, R1_OUT, 'Gauss' )
            BRF_OUT = R1_OUT 

         CASE('snow_artm','white_si')
            CALL BRDF_ACT_DOM( BRDF_PARAM, NLSA, MuV, N_AZ_ANG, Phi,
     $           NSZA, Sol_grd, 1, WL1, R1_OUT, 'Gauss' )
c     $           NSZA, Mue(:,NL-1), 1, WL1, R1_OUT, 'Gauss' )
            BRF_OUT = R1_OUT 

         CASE('melt_pond')
c            CALL POND_BRF( NLSA, MuV, N_AZ_ANG, Phi, NSZA, Mue(:,NL-1),
            CALL POND_BRF( NLSA, MuV, N_AZ_ANG, Phi, NSZA, Sol_grd,
     $           1, WL1, R1_OUT )
            BRF_OUT = R1_OUT

c        ... Additional option in the pp and sp cases
         CASE('rff')
c            CALL RFF_BRDF_INT(NLSA, MuV, NSZA, Mue(:,NL-1), N_AZ_ANG, 
            CALL RFF_BRDF_INT(NLSA, MuV, NSZA, Sol_grd, N_AZ_ANG, 
     $           Phi, POS, R1_OUT)
            BRF_OUT = R1_OUT 

         CASE('prosail')
            ALLOCATE( R_dir(NLSA, NSZA, N_AZ_ANG, NW_E-NW_S+1),
     $           R_dif(NLSA, NSZA, N_AZ_ANG, NW_E-NW_S+1) )
c            CALL PROSAIL( NSZA, Mue(:,NL-1), NLSA, MuV, N_AZ_ANG, Phi, 
            CALL PROSAIL( NSZA, Sol_grd, NLSA, MuV, N_AZ_ANG, Phi, 
     $           R_dir, R_dif )
            CALL LIN_SEARCH_NEW(POS, DC, Ws, Ws1, Nwl_BRF, WL_BRF)
            Uc = (1.d0 - Dc)
            BRF_OUT(:,:,:,1) = Uc*R_dir(:,:,:,Ws) + Dc*R_dir(:,:,:,Ws1)

         CASE DEFAULT
            CALL BRDF_SETUP
            CALL BRDF_ACT_DOM( BRDF_PARAM, NLSA, MuV, N_AZ_ANG, Phi,
     $           NSZA, Sol_grd, 1, WL1, R1_OUT, 'Gauss' )
c     $           NSZA, Mue(:,NL-1), 1, WL1, R1_OUT, 'Gauss' )
            BRF_OUT = R1_OUT 
         END SELECT
      ENDIF

c     There is reflection from ocean surface
c     --------------------------------------
c     ... Whitecaps
      IF( DO_OC_COMP(1) ) THEN 
         IF(LOG_LEVEL(1) >= N_DSO) 
     $        WRITE(*,*) '******* CALL "USER_WHITECAP" in SS_STOKES'
         CALL USER_WHITECAP( Nvs, NLSA, MuV, N_AZ_ANG, Phi, 
     $        NSZA, Sol_grd, 1, WL1, R1_OUT, Area )
c     $        NSZA, Mue(:,NL-1), 1, WL1, R1_OUT, Area )
         BRF_OUT = BRF_OUT + R1_OUT
      ENDIF
c     ... Glint
      IF( DO_OC_COMP(2) ) THEN 
c         WRITE(*,*) ' -------> "gpp_ss_stokes.f"  '
c         DO I = 1, NSZA
c            WRITE(*,*) I, DACOS(Sol_grd(I))/Pie*180.
c         ENDDO            
         CALL BRDF_SETUP
         IF(LOG_LEVEL(1) >= N_DSO) 
     $        WRITE(*,*) ' ====> Call BRDF_ACT_DOM in SS_STOKES'
         
         CALL BRDF_ACT_DOM( BRDF_PARAM, NLSA, MuV, N_AZ_ANG, Phi,
     $        NSZA, Sol_grd, 1, WL1, R1_OUT, SS_PDF )
c     $        NSZA, Mue(:,NL-1), 1, WL1, R1_OUT, SS_PDF )
         BRF_OUT = BRF_OUT + (1.d0 - Area)*R1_OUT 
      ENDIF
c     ... Water leaving
      IF( DO_OC_COMP(3) ) THEN 
         IF(LOG_LEVEL(1) >= N_DSO) 
     $        WRITE(*,*) '******* CALL "USER_WATER" in SS_STOKES'
         CALL USER_WATER( Nvs, NLSA, MuV, N_AZ_ANG, Phi, 
     $        NSZA, Sol_grd, 1, WL1, R1_OUT )
c     $        NSZA, Mue(:,NL-1), 1, WL1, R1_OUT )
         BRF_OUT = BRF_OUT + (1.d0 - Area)*R1_OUT
      ENDIF

c     ... Set to zero surface reflectance if the sun below horizon
c         (more precisely: solar zenith angle larger than 89.5 deg)
      DO K = 1, NSZA
         IF( L_SOL(K) ) BRF_OUT(:,(K-1)*Nvs+1:K*Nvs,:,:) = 0.d0
      ENDDO

      SELECT CASE( ANG_SEL_FLAG )
      CASE( 'one' )
         DO Js = 1,NSZA
            EXS = SSB%EXS(NL,Js)
            DO I = 1,Nvs
               Tls(Js,I) = Sol_grd(Js)*EXS*
c               Tls(Js,I) = Mue(Js,NL-1)*EXS*
     $              BRF_OUT((Js-1)*Nvs+I,(Js-1)*Nvs+1,Js,1)
            ENDDO
         ENDDO
      CASE( 'all' )
         J = 0
         DO Js = 1,NSZA
            EXS = SSB%EXS(NL,Js)
            DO Jv = JV_S(Js),JV_E(Js) 
               DO Ja = 1,N_AZ_VAR  ;  J = J + 1
                  DO I = 1,Nvs
c                     Tls(J,I) = Mue(Js,NL-1)*EXS*
                     Tls(J,I) = Sol_grd(Js)*EXS*
     $                    BRF_OUT((Jv-1)*Nvs+I,(Js-1)*Nvs+1,Ja,1)
                  ENDDO
               ENDDO
            ENDDO
         ENDDO
      END SELECT
      
      END SUBROUTINE REFLECTED_DIR

      END                       ! REFL_SS as main routine
C********************************************************************C
C********************************************************************C

      SUBROUTINE TRANS_SS(JPOS, JJESFT, POS, GSL, LOA, PWF, SSB, TE, 
     $     SRR, SRR_Jact, Phm, Rad_ss, E_W, LWF)
c     ... Call in "gpp_ss_stokes.f" 

      USE GRIDS,       ONLY: NL => NLAYERS, DZ, DZR 
      USE INTERN_DIMS, ONLY: Nvs 
      USE ST_VARS,     ONLY: Pie, DO_RAMAN, DO_PLANCK, WF_FLAG, NS_wf,
     $                       Nc_der
      USE SPHER_PARS,  ONLY: NSZA => N_SOL_ANG, N_OBS_ALT, JV_S, JV_E,
     $                       NLO => N_LOS_OUT, N_AZ_VAR, ANG_OUT, 
     $                       NLSA => N_LOS_ANG, N_AZ_ANG
      USE TOOLS,       ONLY: INQ_WF
      USE TYPE_VARS,   ONLY: KND, DCMP
      USE TYPES,       ONLY: LOA_type, TE_TYPE, SSB_TYPE, RRS_TYPE, 
     $                       GSL_TYPE, PWF_TYPE, LWF_TYPE
c     ------------------------------------------------------------------

      CHARACTER (LEN=8), PARAMETER :: R_NAME = 'TRANS_SS'
c     Subroutine arguments:
c     ---------------------
      INTEGER,    INTENT(IN) :: JPOS, JJESFT, SRR_Jact
      REAL (KND), INTENT(IN) :: POS, E_W, Phm(NLO,NL,Nvs)
      REAL (KND), INTENT(OUT):: Rad_ss(NL,NLO,Nvs)
c     ------------------------------------------------------
      TYPE (GSL_type) :: GSL    ! Angles 
      TYPE (SSB_type) :: SSB    ! Spherical solar beam
      TYPE (LOA_type) :: LOA    ! Local optical parameters of atmosphere
      TYPE (LWF_type) :: LWF    ! Local weighting functions
      TYPE (PWF_type) :: PWF    ! General weighting function 
      TYPE (TE_type)  :: TE     ! Thermal emission
      TYPE (RRS_type) :: SRR    ! RRS related parameters
c     ------------------------------------------------------ 
      INTEGER   I, J, Js, Jv, Ja, Jc, Jw, Jw1, N_1, Zn
      REAL (KND)  ALW, CV, CS, Ck, Cks, DTA, ECV, EXS, Ewei, SOU, SMd, 
     $     T1, T_AU(NL), Iaj(NL-1,NLO,Nvs), Tls(NLO,Nvs),
     $     SC_Md(NL-1,NSZA,NLSA)
      REAL (KND), POINTER :: PXS(:,:), MuV(:)
      REAL (KND), ALLOCATABLE ::  Ta2(:,:,:), Ta1(:,:,:)
      COMPLEX (kind(0.0d0)) :: Muc
c     ==================================================================
     
      CALL ST_TRACK (R_NAME, 1, 'I')

      MuV => GSL%MuV

      T_AU(1) = 0.D0
      DO  J = 1, NL - 1
         T_AU(J+1) = T_AU(J) + (LOA%AL(J) + LOA%AL(J+1))*0.5D0*DZR(J)
      ENDDO

      IF( DO_RAMAN ) THEN
c        ... Downward single-scattered rotational Raman (The contribution
c            of inelastic scattering will be calculated only and stored
c            in the pointer-array DEL_RRS(:,:,:)).
c            Call sequence:
c            trans_ss.f --> tra_rrs.f --> sum_ram_tra.f
         CALL TRA_RRS
c        ... Return for an excitation wavelength (transmission mode)
         IF( SRR%Exi_WL(JPOS) ) RETURN 
      ELSE
c        ... Array SC_Md(I,Js,Jv) contains results of the analytical 
c            integration of the solar exponent or its approximation 
c            (in the pseudo-spherical atmosphere) over each layer along
c            line-of-sight direction. If DO_RAMAN=.TRUE. this array 
c            is calculated in routine "sum_ram_tra.f"
         IF( .NOT.TE%NO_SUN) THEN
            DO I = 1,NL - 1
               DTA = T_AU(I+1) - T_AU(I)
               DO Js = 1,NSZA
                  CS = SSB%Mue(Js,I) ; SOU = DEXP( -DTA/CS )
                  DO Jv = JV_S(Js),JV_E(Js) 
                     CV = MuV(Jv) ; ECV = DEXP( -DTA/CV ) 
                     IF( REAL(CS) /= REAL(CV) ) THEN
                        SMd = CS*CV*(SOU - Ecv)/(CS - CV)
                     ELSE
                        SMd = Ecv*DTA
                     ENDIF
                     Zn = -1
                     DO Jc = 1,SSB%N_sph(Js,I)-1
                        Ck = DTA/DBLE(Jc)/Pie ; Muc = CS*Ck/DCMP(Ck,-Cs)
                        SMd = SMd + AIMAG( Muc*Cv/(Muc-Cv) )
     $                       *SSB%SIN_COEF(Js,I,Jc)*(Zn*SOU - Ecv)
                        Zn = -Zn
                     ENDDO
                     SC_Md(I,Js,Jv) = SMd
                  ENDDO  ! over viewing angles (Jv)
               ENDDO  ! over solar angles (Js)
            ENDDO  ! over altitude layers (I)
         ENDIF  ! Include solar light contribution
      ENDIF  ! DO_RAMAN

      Iaj = 0.d0
      I_aj: Block
        IF( DO_PLANCK ) CALL ADD_TRA_PLA ! Thermal emission contribution 
c       ... Array Iaj(I,:,:) contains results of the numerical integration 
c           of the scattering coefficient and analytical of the solar source 
c           function (SC_Md) over each layer along the line-of-sight
        DO I = 1,NL - 1
           ALW = LOA%AL(I) + LOA%AL(I+1)
           J = 0
           DO Js = 1,NSZA
              EXS = SSB%EXS(I,Js)/ALW/4.d0
              DO Jv = JV_S(Js),JV_E(Js) 
                 CV = MuV(Jv)
                 DO Ja = 1,N_AZ_VAR  ;  J = J + 1
                    Iaj(I,J,:) = Iaj(I,J,:) + SC_Md(I,Js,Jv)*EXS/CV*
     $                   (Phm(J,I,:) + Phm(J,I+1,:))
                 ENDDO  ! over azimuthal angles (Ja)
              ENDDO  ! over line-of-sight angles (Jv)
           ENDDO  ! over solar zenith angles (Js)
        ENDDO  ! over altitude layers (I)
      END Block I_aj

      Tls = 0.d0 
c     ... Calculation of transmitted radiation at all levels using
c         recurrence relation
      Rad_ss(1,:,:) = Tls(:,:)
      DO I = 1,NL-1  ;  J = 0
         DTA = T_AU(I+1) - T_AU(I)
         DO Js = 1,NSZA
            DO Jv = JV_S(Js),JV_E(Js) 
               ECV = DEXP(-DTA/Muv(Jv))
               DO Ja = 1,N_AZ_VAR  ;  J = J + 1
                  Tls(J,:) = Tls(J,:)*ECV + Iaj(I,J,:)
               ENDDO
            ENDDO
         ENDDO
         Rad_ss(I+1,:,:) = Tls(:,:) 
      ENDDO

      IF( .NOT.WF_FLAG ) THEN
         CALL ST_TRACK (R_NAME, 1, 'O')  ;  RETURN
      ENDIF

c     ... Weighting functions for single scattered transmitted radiation
      ALLOCATE( Ta2(NL,NLO,NS_wf), Ta1(NL,NLO,NS_wf) )
      Ta1 = 0.d0  ;  Ta2 = 0.d0  ;  N_1 = N_OBS_ALT(1)
      J = 0
      DO Js = 1,NSZA                 ;  Cs = SSB%Mue(Js,NL-1) 
         DO Jv = JV_S(Js),JV_E(Js)   ;  Cv = MuV(Jv) 
            DO Ja = 1,N_AZ_VAR       ;   J = J + 1
               DO Jw = 1, NS_wf      ; Jw1 = Nc_der(Jw)
                  DO I = 1, N_1-1
                     Iaj(I,J,Jw1) = Iaj(I,J,Jw1)*GSL%EXL(I+1,Jv)
                  ENDDO
                  Ta1(1,J,Jw) = 0.d0
                  DO I = 1, N_1-1
                     Ta1(I+1,J,Jw) = Ta1(I,J,Jw) - Iaj(I,J,Jw1)/Cv
                  ENDDO
                  Ta2(N_1,J,Jw) = 0.d0
                  DO I = N_1-1, 1, -1
                     Ta2(I,J,Jw) = Ta2(I+1,J,Jw) - Iaj(I,J,Jw1)/Cs
                  ENDDO
               ENDDO ! over number of Stokes (Jw)
            ENDDO  ! Ja
         ENDDO  ! Jv
      ENDDO  ! Js

c     ... Without quadrature coefficients for WFs of CTH, CBH, TAU 
c         !!! Single LoS and without polarization !!!
      IF(INQ_WF('cld', 'WFT')) LWF%DT1(:) = LWF%DT1(:) 
     $     + (Ta1(:,1,1) + Ta2(:,1,1))
      DO I = 1, N_1-1
         FORALL( J=1:NLO,Jw=1:NS_wf ) PWF%WF_esp(I,J,Jw) = 
     $        ( Ta1(I,J,Jw) + Ta2(I,J,Jw) )*DZ(I)/2.d0
      ENDDO
      FORALL( J=1:NLO,Jw=1:NS_wf ) PWF%WF_esp(N_1,J,Jw) = 
     $     ( Ta1(N_1,J,Jw) + Ta2(N_1,J,Jw) )*DZR(N_1-1)/2.d0

      CALL ST_TRACK (R_NAME, 1, 'O')      
      RETURN
      CONTAINS
c     Under this CONTAINS belonging to "trans_ss,f" 
c           are following subroutines:
c     tra_rrs.f
c     sum_ram_tra.f
c     add_tra_pla.f
c **********************************************************************
      SUBROUTINE TRA_RRS
c     ... Call in "refl_ss.f"

      USE GT_POLYN,    ONLY: Plm_ss 
      USE INTERN_DIMS, ONLY: Nlg_Ray, Nm_vs
      USE IRRAD,       ONLY: SOL_FLUX 
      USE PS_INTERN,   ONLY: COS_SCAT 
      USE RAMAN,       ONLY: WL_FACT, LINE_GAS, Sigma_RR, Phot_to_Wat
      USE ST_VARS,     ONLY: WL_GRID, TSM
      USE VECT_VARS,   ONLY: Rot_l
c     ------------------------------------------------------------------

      INTEGER  K, Iae, Iac, Jex, Nsf, To_act, R_lin
      REAL (KND), POINTER :: P_MOM(:)
      REAL (KND)  IRR, IR0, COF(NL), Rhm(NLO,Nvs), R_j(NL,SRR%M_ESF), 
     $     EXCO(NL,SRR%M_ESF), Wes(SRR%M_ESF), Plm_as(Nlg_Ray,NLO)
c     ==================================================================

      DO J = 1,NLO
         Rhm(J,1) = SUM( Plm_ss(1:NLg_Ray,J)*SRR%RAM_MOMS(1:NLg_Ray,1) )
      ENDDO
      IF( Nm_vs > 1 ) THEN
         Rhm(:,4) = 0.d0
         CALL WIGNER_PLG(COS_SCAT, NLO, 2, Nlg_Ray, Plm_as )
         P_MOM => SRR%RAM_MOMS(1:NLg_Ray,5) 
         DO J = 1,NLO
            T1 = SUM( Plm_as(1:NLg_Ray,J)*P_MOM(:) )
            FORALL( K = 2:3 ) Rhm(J,K) = T1*Rot_l(J,K,2)
         ENDDO
      ENDIF

      Ewei = E_W

c     ... Energy loss at actual wavelengths due to rotational 
c         Raman scattering 
      IF( SRR%ACT_WL(JPOS) ) THEN
c        ... Set extraterrestrial solar flux at actual WL corresponding
c            to its position in the array WL_ext
         IRR = -SOL_FLUX(JPOS)/Pie  ;  To_act = SRR_Jact

c        ... Save extinction coefficient for all actual wavelengths
         SRR%EXT_ACT(:,To_act,JJESFT) = LOA%AL(:)

         IF( TRIM(TSM)=='speed' ) SRR%EXS_ACT(:,To_act,:) 
     $        = SRR%EXS_ACT(:,To_act,:) + SSB%EXS(:,:)*E_W

c        ... Set scattering coefficient as Raman total
         COF(:) = SRR%R_TOT*LOA%SCR(:)
         Nsf = 1 ; Wes(1) = 1.d0 ; EXCO(:,1) = LOA%AL(:)
         R_j(:,1) = 1.d0
         PXS => SSB%EXS(:,:)
         CALL SUM_RAM_TRA(To_act, Nsf, IRR, COF, EXCO, Wes, R_j, Rhm)

         IF( To_act==SRR%N_ACT .AND. TRIM(TSM)=='speed'.AND. 
     $       JJESFT==SRR%NESFT(SRR%N_ACT) ) THEN
c           ... Over all actual wavelengths which here considered
c               as excitation wavelengths
!            WRITE(*,*) '     Option "speed" SRR%N_ACT = ',SRR%N_ACT
!            WRITE(*,*) SRR%NESFT(SRR%N_ACT), jjesft
            Ewei = 1.d0
            DO Iac = 1, SRR%N_ACT 
               Iae = SRR%A2E(Iac)
               IR0 = SOL_FLUX(Iae)/Pie
               DO Jex = 1,SRR%NUM_IN(Iae)
                  To_act = SRR%ACT_IN(Iae,Jex)
                  IRR = IR0
                  IF( Phot_to_Wat ) IRR = IR0*WL_GRID(SRR%A2E(To_act))
     $                 /WL_GRID(Iae)
                  R_lin  = SRR%RIN_IN(Iae,Jex)
                  COF(:) = WL_FACT*Sigma_RR(R_lin,:)
     $                 /WL_GRID(SRR%A2E(To_act))**4
     $                 *SRR%GAM_WL(LINE_GAS(R_lin),Iac)
                  PXS  => SRR%EXS_ACT(:,Iac,:)

                  Nsf = SRR%NESFT(To_act)
                  EXCO(:,1:Nsf) = SRR%EXT_ACT(:,To_act,1:Nsf)
                  Wes(1:Nsf) = SRR%WEI(To_act,1:Nsf)
                  DO J = 1,Nsf
                     R_j(:,J) = SRR%EXT_ACT(:,Iac,J)/EXCO(:,J)
                  ENDDO
                  CALL SUM_RAM_TRA
     $                 (To_act, Nsf, IRR, COF, EXCO, Wes, R_j, Rhm)
               ENDDO
            ENDDO
         END IF
c     ... Energy gain at actual wavelengths due to rotational 
c         Raman scattering at excitation wavelengths
      ELSE
c        ... Set extraterrestrial solar flux at excitation wavelength
         IR0 = SOL_FLUX(JPOS)/Pie
         PXS => SSB%EXS(:,:)
c        ... Over all excitation wavelengths within corresponding bin 
         DO Jex = 1,SRR%NUM_IN(JPOS)
c           ... See explanations for settings in "REF_RRS.f"
            To_act = SRR%ACT_IN(JPOS,Jex) 
            R_lin  = SRR%RIN_IN(JPOS,Jex)
            Iae    = SRR%A2E(To_act)
            IRR = IR0
            IF( Phot_to_Wat ) IRR = IR0*WL_GRID(Iae)/WL_GRID(JPOS)
            COF(:) = WL_FACT*Sigma_RR(R_lin,:)/WL_GRID(Iae)**4
     $           *SRR%GAM(LINE_GAS(R_lin))
            Nsf = SRR%NESFT(To_act)
            EXCO(:,1:Nsf) = SRR%EXT_ACT(:,To_act,1:Nsf)
            Wes(1:Nsf) = SRR%WEI(To_act,1:Nsf)
            DO J = 1,Nsf
               R_j(:,J) = LOA%AL(:)/EXCO(:,J)
            ENDDO
            CALL SUM_RAM_TRA(To_act, Nsf, IRR, COF, EXCO, Wes, R_j, Rhm)
         ENDDO
      ENDIF

      END SUBROUTINE TRA_RRS
c **********************************************************************
      SUBROUTINE SUM_RAM_TRA(To_act, Nsf, IRR, COF, EXCO, Wes, R_j, Rhm)
c     ... Call in "tra_rrs.f"

      USE GRIDS,       ONLY: NL => NLAYERS, DZR 
      USE RTM_OUTPUT,  ONLY: DEL_RRS
c     ------------------------------------------------------------------
      INTEGER    To_act, Nsf
      REAL (KND) IRR, COF(NL), Rhm(NLO,Nvs), EXCO(NL,SRR%M_ESF),  
     $     R_j(NL,SRR%M_ESF), Wes(SRR%M_ESF)
c     ... Local variables
      INTEGER    Isf, Jo
      REAL (KND) Tau(NL), Del_ss(NL,NLO,Nvs)
c     ==================================================================

      Del_ss(:,:,:) = 0.d0

      DO Isf = 1,Nsf
c        ... Optical thickness of jth layer is calculated for actual WL
         Tau(:) = 0.d0
         DO  J = 1,NL-1
            Tau(J+1) = Tau(J) + (EXCO(J,Isf) + EXCO(J+1,Isf))
     $           *0.5d0*DZR(J)
         ENDDO

c        ... Array SC_Md(I,Js,Jv) contains results of the analytical 
c            integration of the solar exponent or its approximation 
c            (in the pseudo-spherical atmosphere) over each layer.
         DO I = 1,NL - 1
            DTA = Tau(I+1) - Tau(I)
            DO Js = 1,NSZA
               CS = SSB%Mue(Js,I)/R_j(I,Isf) ; SOU = DEXP( -DTA/CS )
               DO Jv = JV_S(Js),JV_E(Js) 
                  CV = MuV(Jv) ; ECV = DEXP( -DTA/CV ) 
                  IF( REAL(CS) /= REAL(CV) ) THEN
                     SMd = CS*CV*(SOU - Ecv)/(CS - CV)
                  ELSE
                     SMd = Ecv*DTA
                  ENDIF
                  Zn = -1
                  DO Jc = 1,SSB%N_sph(Js,I)-1
                     Ck = DTA/DBLE(Jc)/Pie ; Muc = CS*Ck/DCMP( Ck,-Cs )
                     SMd = SMd + AIMAG( Muc*Cv/(Muc-Cv) )
     $                    *SSB%SIN_COEF(Js,I,Jc)*(Zn*SOU - Ecv)
                     Zn = -Zn 
                  ENDDO
                  SC_Md(I,Js,Jv) = SMd
               ENDDO
            ENDDO
         ENDDO

         DO I = 1,NL - 1
            ALW = EXCO(I,Isf) + EXCO(I+1,Isf)
            J = 0
            DO Js = 1,NSZA
               EXS = PXS(I,Js)/ALW/4.d0
               DO Jv = JV_S(Js),JV_E(Js) 
                  CV = MuV(Jv)
                  DO Ja = 1,N_AZ_VAR  ;  J = J + 1
                     Iaj(I,J,:) = SC_Md(I,Js,Jv)*EXS/CV*Rhm(J,:)
     $                    *(COF(I) + COF(I+1))
                  ENDDO         ! Over azimuthal angles
               ENDDO            ! -,,- line-of-sight zenith angles
            ENDDO               ! -,,- solar zenith angles
         ENDDO                  ! -,,- all altitude layers

         Tls = 0.d0 
c        ... Calculation of transmitted radiation at all levels using
c            recurrence relation
         Del_ss(1,:,:) = Tls(:,:)
         DO I = 1,NL-1  ;  J = 0
            DTA = Tau(I+1) - Tau(I)
            DO Js = 1,NSZA
               DO Jv = JV_S(Js),JV_E(Js) 
                  ECV = DEXP(-DTA/Muv(Jv))
                  DO Ja = 1,N_AZ_VAR  ;  J = J + 1
                     Tls(J,:) = Tls(J,:)*ECV + Iaj(I,J,:)
                  ENDDO
               ENDDO
            ENDDO
            Del_ss(I+1,:,:) = Del_ss(I+1,:,:) + Tls(:,:)*Wes(Isf)
         ENDDO
      ENDDO                     ! Over esft 

      Jo = N_OBS_ALT(1)
      DEL_RRS(To_act,:,:) = DEL_RRS(To_act,:,:) 
     $     + IRR*Del_ss(Jo,:,:)*Ewei

      END SUBROUTINE SUM_RAM_TRA
c **********************************************************************

      SUBROUTINE ADD_TRA_PLA
      REAL (KND)  SC_pla(NL-1,NLSA)
c     ==================================================================
      DO I = 1,NL - 1
         DTA = T_AU(I+1) - T_AU(I)
         CS = -TE%Mup(I) ; SOU = DEXP( TE%Mup(I)*DTA )
         DO Jv = 1,NLSA
            CV = MuV(Jv) ; ECV = DEXP( -DTA/CV ) 
            IF( REAL(CS*CV) /= REAL(1.d0) ) THEN
               SMd = CV*(SOU - Ecv)/(1.d0 - CS*CV)
            ELSE
               SMd = Ecv*DTA
            ENDIF
            Zn = -1
            DO Jc = 1,TE%N_pla(I)-1
               Ck = DTA/DBLE(Jc)/Pie ; Cks = Cs*Ck
               Muc = Ck/DCMP( Cks,-1.d0)
               SMd = SMd + AIMAG( Muc*Cv/(Muc-Cv) )
     $              *TE%PL_COEF(I,Jc)*(Zn*SOU - Ecv)
               Zn = -Zn
            ENDDO
            SC_pla(I,Jv) = SMd
         ENDDO
      ENDDO

      DO I = 1,NL-1  ;  J = 0  ;  T1 = TE%NPLANCK(I)            
         DO Js = 1,NSZA
            DO Jv = JV_S(Js),JV_E(Js)  ;  CV = MuV(Jv)
               DO Ja = 1,N_AZ_VAR  ;  J = J + 1
                  Iaj(I,J,1) = SC_pla(I,Jv)*T1/CV 
               ENDDO  ! over azimuthal angles (Ja)
            ENDDO  ! over line-of-sight zenith angles (Jv)
         ENDDO  ! over solar zenith angles (Js)
      ENDDO  ! over all altitude layers (I)

      END SUBROUTINE ADD_TRA_PLA
c **********************************************************************

      END                       ! TRANS_SS
c **********************************************************************
