************************************************************************
C
C	SCIATRAN - Version 4.7 - Last modification: 26 Oct 2023
C	Institute of Remote Sensing, University of Bremen, Germany
C
C***********************************************************************
c      CLD_LAYER_COEFF can be calculated within this routine. 
c      The calculation of extinction and 
c      scattering coefficients for ice and water particles are
c      performed using approximate expressions given by 
c      Kokhanovsky A.A. in "Local optical characteristics of mixed 
c      clouds: simple parameterizations", 
c      Atmos. Res., 2007, 84, 42-48.
c     ********************************************************
      SUBROUTINE GT_CLOUD_PARAMETER_SETUP (*)
c     ... Called by "rtm_init.f" 
c            and by "dom_spherical.f" if NUM_CLD = .TRUE.
      USE ALT_INPUT,  ONLY: IPRT_ALIN
      USE CLD_RET,    ONLY: Ref_wi, Aef_fra, Edg_fra
      USE CLD_INP,    ONLY: D_MF, HOW_CLD_COEF, Plus_ice, 
     $                       N_ref_db, Ref_db, Ext_DB, TDS,  Use_Dim,
     $                       N_ref_Hex, Ref_Hex, Ext_Hex, PHASE_NAME,
     $                       N_ref_Fra, Ref_Fra, Ext_Fra, ADD_ABS, 
     $                       N_ref_Bdb, Ref_Bdb, Ext_Bdb, Use_Bdb,
     $                       N_ref_Ydb, Ref_Ydb, Ext_Ydb, Vol_Ydb, 
     $                       Use_Ydb, Dim_Ydb, CRST,
     $                       Phm_const, N_Rdb, TotA_Bdb, N_ref_ice,
     $                       Ref_ice, MLCS, Use_Ydb_SCM
      USE CLOUDS,      ONLY: NCW => N_CLD_NETWAVS, CLD_NETWAVS, PR_SOOT,
     $                       CLD_LAYER_COEFF, CLD_SAVE_COEFF, PROF_VARS, 
     $                       VT => Var_type, CLA => CLD_LAYER_ARR, 
     $                       CLDPROFILE, CLD_LAYER_NUM, F_Dfit, EPS_CLD, 
     $                       N_CLDPROF, J_TOP, J_BOT, IWP, LWP, 
     $                       CLD_LAYER_DERIV, TAU_CLD_INP, N_CLD_LAYERS, 
     $                       CLD_PHAS_MOMS, CLD_DM, SCC_LAYER_NOSCALE, 
     $                       SCI_LAYER_NOSCALE, 
     $                       REF_IMAG_W, REF_IMAG_I, CLD_F_DM, 
     $                       Prof_Cpar, Prof_Ref, SCALE_PROF,
     $                       CLD_LAYER_HEIGHTS
      USE DATA_RET,    ONLY: RET_FLAG
      USE GRIDS,       ONLY: NLAYERS, ZZZ
      USE GT_POLYN,    ONLY: NLEGEN
      USE INFO,        ONLY: CLD_INFO
      USE INTERN_DIMS, ONLY: Nm_vs, NL_soot! , Nm_ice
      USE LOG_PARS,    ONLY: LOG_LEVEL, N_ESO, LUNIT, EU, N_XSO, DIR_OUT
      USE RTM_PARS,    ONLY: DATA_ITER
      USE RETR_ITER,   ONLY: ITERATION
      USE ST_VARS,     ONLY: WF_FLAG, RTM_CORE, SCATT_MODE, RET_MODE
      USE TOOLS,       ONLY: CHK_STAT, SET_PTR
      USE TYPE_VARS,   ONLY: KND
      USE WF_NUM,      ONLY: NUM_CLD, PERTURB, SCA_or_ABS, Wat_or_Ice
c     ------------------------------------------------------------------

      CHARACTER (LEN=24), PARAMETER :: R_NAME='GT_CLOUD_PARAMETER_SETUP'
      CHARACTER (LEN=90)  TITLE
      CHARACTER (LEN=80)  SUB_TITLE
      Character(Len=:), Allocatable :: Str
      LOGICAL  W_ref, W_lwc, I_ref, I_lwc, W_soot, W_pnc, I_pnc,
     $     INQ_RET, INQ_SOOT, INQ_CTH
      INTEGER  Ir, KK, CDM, INP_LYR, NCL, CLN, JT, JB, N_1, N_2, K, I, 
     $     N_a, N_e, Max_sub_layer, N_der, INX, Max_SL, IVT, N_col, Idb,
     $     Nxr, NMC, Nrb, NZS, II
      REAL (KND)   BASE, TOP, Eff_Ext, Norm, TCI,
     $     Norm_ice, Tau_ice, Tau_Loc, Lim
      REAL (KND), DIMENSION(NLAYERS) :: Z_inp, Pr_inp
      REAL (KND), ALLOCATABLE :: Sh_wat(:), A_ef(:), Sh_ice(:),
     $     R_ice(:), L_Hex(:), Z_act(:), HH(:), A_ef_ice(:),
     $     Dcof_Dr(:,:,:), F_DMI(:), F_DM(:)
      TYPE (DATA_ITER), POINTER :: RTM_PTR
      REAL (KND), POINTER :: Ext_p(:), SCC_p(:), SCC_ns_p(:),
     $     Phm_p(:,:), SCI_p(:), Sec_p(:), P_Rdb(:), P_aef(:)
      TARGET :: A_ef, A_ef_ice
c     ==================================================================

      CALL ST_TRACK (R_NAME, 1, 'I')

      IF( IPRT_ALIN ) THEN
         CALL ST_TRACK (R_NAME, 1, 'O')
         RETURN
      ENDIF
      Max_SL = INT( MAXVAL( CLD_LAYER_NUM,1 ) ) + 3

      W_ref = .FALSE.  ;  W_lwc = .FALSE.  ;  W_pnc = .FALSE.
      I_ref = .FALSE.  ;  I_lwc = .FALSE.  ;  I_pnc = .FALSE.
      W_soot = .FALSE.
      IF( NUM_CLD ) THEN
         SELECT CASE( TRIM(SCA_or_ABS ) )
         CASE('soot__conc') ; W_soot = .TRUE.
c        -------------------------------------
         CASE('lw_content')
            SELECT CASE( TRIM(Wat_or_Ice) )
            CASE('water') ; W_lwc = .TRUE.
            CASE('ice')   ; I_lwc = .TRUE.
            CASE('nomix')
               W_lwc = .TRUE. ; I_lwc = .TRUE.
            END SELECT
c        -------------------------------------
         CASE('par_number')
            SELECT CASE( TRIM(Wat_or_Ice) )
            CASE('water') ; W_pnc = .TRUE.
            CASE('ice')   ; I_pnc = .TRUE.
            CASE('nomix')
               W_pnc = .TRUE. ; I_pnc = .TRUE.
            END SELECT
c        -------------------------------------
         CASE('eff_radius')
            SELECT CASE( TRIM(Wat_or_Ice) )
            CASE('water') ; W_ref = .TRUE.
            CASE('ice')   ; I_ref = .TRUE.
            CASE('nomix')
               W_ref = .TRUE. ; I_ref = .TRUE.
            END SELECT
            ALLOCATE( Dcof_Dr(Max_SL,NCW,2) )
c        -------------------------------------
         END SELECT
c              !!! Check N_Rdb for mixed cloud !!!
         CLD_LAYER_COEFF(:,:,:,1:N_Rdb) = CLD_SAVE_COEFF(:,:,:,:)
      ENDIF  ! NUM_CLD
      
      IF( ITERATION > 1 ) THEN 
         CLD_LAYER_COEFF(:,:,:,1:N_Rdb) 
     $        = CLD_SAVE_COEFF(:,:,:,:)
      ENDIF
      
      IF( LOG_LEVEL(1) <= N_XSO ) THEN
         WRITE(LUNIT,*) '   --- after input ----'
         WRITE(LUNIT,*) '   cloud layer: 1'
         WRITE(LUNIT,*) '    scattering: ',
     $        SUM( CLD_LAYER_COEFF(1,:,2,1:N_Rdb) )
         WRITE(LUNIT,*) '    extinction: ',
     $        SUM( CLD_LAYER_COEFF(1,:,1,1:N_Rdb) )
      ENDIF

      IF(ALLOCATED(PROF_VARS)) DEALLOCATE(PROF_VARS, PR_SOOT)
      ALLOCATE ( PROF_VARS(NLAYERS), PR_SOOT(NL_soot), A_ef_ice(Max_SL),
     $     Sh_wat(Max_SL), A_ef(Max_SL), HH(Max_SL), Sh_ice(Max_SL), ! A
     $     R_ice(Max_SL), L_Hex(Max_SL), Z_act(Max_SL) ) ! A

      PR_SOOT(:) = 0.d0
      Lim = EPS_CLD 

c     Maximal number of sub-layers in clouds
c     --------------------------------------
      Max_sub_layer = INT( MAXVAL( CLD_LAYER_NUM,1 ) ) + 3

c     Derivatives of the shape profile with respect to the CTH and CBH
c     ----------------------------------------------------------------
c     Plus_ice(:) is .true. if:
c        a) there is a mixed cloud
c        b) one cloud layer is water cloud and another one is ice cloud
      IF( ANY(Plus_ice) ) THEN
         N_1 = 3 ; N_2 = 4 ; N_der = 2 
      ELSE
         N_1 = 1 ; N_2 = 2 ; N_der = 1
      ENDIF
      Prof_cpar = 0.d0

      IF( RET_FLAG.AND.ITERATION == 1 ) THEN
c     ... Set parameters for retrieval process
         INQ_RET=CHK_STAT('cld-ref_nomix',CLD_INFO%NAME,CLD_INFO%WF_FLG) 
         INQ_SOOT=CHK_STAT('cld-soot',CLD_INFO%NAME,CLD_INFO%WF_FLG)
         IF( INQ_RET.OR.INQ_SOOT ) THEN
            NCL = 1 ; JT = J_TOP(NCL) + 1 ; JB = J_BOT(NCL) - 1
            INP_LYR = N_CLDPROF(NCL) ; CLN = JB - JT + 1   
            Z_act(1:CLN) = ZZZ(JT:JB)
            TOP = Z_act(1) ; BASE = Z_act( CLN )
         ENDIF
         IF( INQ_SOOT ) THEN
c          ... Transform dimensionless height variable into [km]
            KK = MLCS(NCL)%NZ_SOOT
            Z_inp(1:KK) = TOP - MLCS(NCL)%Z_SOOT(:)*(TOP - BASE)
            Z_inp(KK) = BASE
            CALL SET_PTR( RTM_PTR, 'cld-soot' )
            Pr_inp(1:KK) = MLCS(NCL)%C_SOOT(:)
            CALL AL_INTERP (KK, Z_inp(1:KK), Pr_inp(1:KK),
     $           CLN, Z_act(1:CLN), Sh_wat(1:CLN), .FALSE., *880)  
            Pr_inp = 0.d0  ;  Pr_inp(JT:JB) = Sh_wat(1:CLN)
            RTM_PTR%APR(:) = Pr_inp(:)
            RTM_PTR%CRN(:) = Pr_inp(:)
!            write(*,*) '   A priori profile of soot concentration:'
!            WRITE(*,*) Pr_inp(Jt:Jt+2)
         ENDIF
c        ---------------------------------------------------------------
         IF( INQ_RET ) THEN
            Z_inp(1:INP_LYR) = TOP - 
     $           CLDPROFILE(NCL,1:INP_LYR,1) *(TOP - BASE) 
            Z_inp(INP_LYR) = BASE
            CALL SET_PTR( RTM_PTR, 'cld-ref_nomix' )
            IF( TDS(1)%S == 'ice' )   THEN
               KK = 7 ; Ref_wi(2) = .TRUE.
            ENDIF
            IF( TDS(1)%S == 'water' ) THEN
               KK = 3 ; Ref_wi(1) = .TRUE.
            ENDIF
            Pr_inp(1:INP_LYR) = CLDPROFILE(NCL,1:INP_LYR,KK)
            CALL AL_INTERP (INP_LYR, Z_inp, Pr_inp,
     $           CLN, Z_act, Sh_wat(1:CLN), .FALSE., *880)  
            Pr_inp = 0.d0  ;  Pr_inp(JT:JB) = Sh_wat(1:CLN)
            RTM_PTR%APR(:) = Pr_inp(:)
            RTM_PTR%CRN(:) = Pr_inp(:)
!            write(*,*) '   A priori profile of edge length:'
!            WRITE(*,*) Pr_inp(Jt:Jt+2)
         ENDIF
c        ---------------------------------------------------------------         
         INQ_RET=CHK_STAT('cld-tau_nomix',CLD_INFO%NAME,CLD_INFO%WF_FLG) 
         IF( INQ_RET) THEN
            CALL SET_PTR( RTM_PTR, 'cld-tau_nomix' )
            KK = 1
            RTM_PTR%APR(:) = TAU_CLD_INP(1,KK)
            RTM_PTR%CRN(:) = TAU_CLD_INP(1,KK)  
!            write(*,*) '  Tau in CRN ', RTM_PTR%CRN(:)
         ENDIF           
c        ---------------------------------------------------------------         
         INQ_CTH=CHK_STAT('cld-cth',CLD_INFO%NAME,CLD_INFO%WF_FLG) 
         IF( INQ_CTH) THEN
            CALL SET_PTR( RTM_PTR, 'cld-cth' )
            RTM_PTR%APR(:) = CLD_LAYER_HEIGHTS(1,2)
            RTM_PTR%CRN(:) = CLD_LAYER_HEIGHTS(1,2)  
!            write(*,*) '  Top height: ', RTM_PTR%CRN(:)
         ENDIF           
      ENDIF  ! RET_FLAG.AND.ITERATION == 1

c     ... Set retrieved value of optical thickness
      IF( RET_FLAG.AND.ITERATION > 1.AND.RET_MODE=='std' ) THEN
         INQ_RET=CHK_STAT('cld-tau_nomix',CLD_INFO%NAME,CLD_INFO%WF_FLG) 
         IF( INQ_RET) THEN
            CALL SET_PTR( RTM_PTR, 'cld-tau_nomix' )
            TAU_CLD_INP(1,1) = RTM_PTR%CRN(1) 
!                write(*,*) '  Tau from CRN ', RTM_PTR%CRN(:)
!            pause
         ENDIF
      ENDIF
      
c *****************************************************************************
      DO NCL = 1, N_CLD_LAYERS ! Do-loop over all cloud layers
c         WRITE(*,*) '   '
c         WRITE(*,*) '   --------------->> Cloud layer  ', NCL
         CDM  = CLD_DM(NCL)
c          Top and bottom without infinite sub-layers
c        ---------------------------------------------
         JT = J_TOP(NCL) + 1 ; JB = J_BOT(NCL) - 1
c        Number of levels for shape profile in cloud as input
c        ----------------------------------------------------
         INP_LYR = N_CLDPROF(NCL) 
c        Number of layers in cloud
c        -------------------------
         CLN = JB - JT + 1 ; Z_act(1:CLN) = ZZZ(JT:JB)
         TOP   = Z_act(1)    ; BASE = Z_act( CLN )
                 !WRITE(*,*) '   Top and bottom of cloud and ADD_ABS:'
                 !WRITE(*,*) TOP, BASE, ADD_ABS(NCL) 
         IF( ADD_ABS(NCL) ) THEN
            INQ_SOOT=CHK_STAT('cld-soot',CLD_INFO%NAME,CLD_INFO%WF_FLG)
            IF( RET_FLAG.AND.ITERATION > 1.AND.INQ_SOOT ) THEN
               CALL SET_PTR( RTM_PTR, 'cld-soot' )
               PR_SOOT(JT:JB) = RTM_PTR%CRN(Jt:Jb)
!               write(*,*) '   Retr profile of soot: Iter = ', ITERATION
!               WRITE(*,*) PR_SOOT(Jt:Jt+2)
            ELSE
               NZS = MLCS(NCL)%NZ_SOOT
c              ... Transform dimensionless height variable into [km]
               Z_inp(1:NZS) = TOP - MLCS(NCL)%Z_SOOT(:)*(TOP - BASE)
               Z_inp(NZS) = BASE
c              ... Copy "soot" concentration into temporary array Pr_inp
               Pr_inp(1:NZS) = MLCS(NCL)%C_SOOT(:)
c              ... Interpolate concentration on the actually altitude grid
               CALL AL_INTERP (NZS, Z_inp, Pr_inp,
     $              CLN, Z_act, PR_SOOT(JT:JB), .TRUE., *880)  
               IF( W_soot )
     $              PR_SOOT(Jt:Jb) = PR_SOOT(Jt:Jb)*PERTURB(Jt:Jb)
            ENDIF
         ENDIF  ! ADD_ABS(NCL)
c         write(*,*) '     After ADD_ABS(NCL) '
c        Transform dimensionless height variable into [km]
c        -----------------------------------------------
         Z_inp(1:INP_LYR) = TOP - 
     $        CLDPROFILE(NCL,1:INP_LYR,1) *( TOP - BASE ) 
         Z_inp(INP_LYR) = BASE

c ********** Input parameters for water droplets ***********************
c        ... Copy water "shape" profile into temporary array Pr_inp
         Pr_inp(1:INP_LYR) = CLDPROFILE(NCL,1:INP_LYR,2)
c        ... Interpolate shape-profile (water) on the actually 
c            altitude grid and save result into array Sh_wat
c         WRITE(*,*) Z_inp(1:INP_LYR)
c         WRITE(*,*) '          -----------------     '
c         WRITE(*,*) Pr_inp(1:INP_LYR)
         CALL AL_INTERP (INP_LYR, Z_inp, Pr_inp,
     $        CLN, Z_act, Sh_wat(1:CLN), .TRUE., *880)
c         WRITE(*,*) '           --- 1 '
         IF( W_lwc.OR.W_pnc ) 
     $        Sh_wat(1:CLN) = Sh_wat(1:CLN)*PERTURB(Jt:Jb)
c        ... Integral of shape-function(water) over cloud layer 
         Norm = SUM( ( z_act(1:CLN-1) - z_act(2:CLN) )*
     $        ( Sh_wat(1:CLN-1) + Sh_wat(2:CLN) ) )/2.0d0
     $        + (Sh_wat(1) + Sh_wat(CLN))/2.0d0*Lim
         IF( RET_FLAG.AND.ITERATION > 1.AND.Ref_wi(1) ) THEN
            CALL SET_PTR( RTM_PTR, 'cld-ref_nomix' )
            WRITE(*,*) '    Iteration ', ITERATION
            DO I = Jt,Jb
               WRITE(*,*) ZZZ(I), RTM_PTR%CRN(I)
            ENDDO
            A_ef(1:CLN) = RTM_PTR%CRN(Jt:Jb)
         ELSE
c        ... Copy eff. water radius into temporary array Pr_inp
            Pr_inp(1:INP_LYR) = CLDPROFILE(NCL,1:INP_LYR,3)
c        ... Interpolate of eff. radius on the actually altitude grid
c            and save result into array A_ef
            CALL AL_INTERP (INP_LYR, Z_inp, Pr_inp,
c     $        CLN, Z_act, A_ef(1:CLN), .TRUE., *880)  
     $           CLN, Z_act, A_ef(1:CLN), .FALSE., *880)  
c            WRITE(*,*) '           --- 2 '
         ENDIF
c        ... Numerical perturbation of effective radius
         IF( W_ref ) A_ef(1:CLN) = A_ef(1:CLN)*PERTURB(Jt:Jb)
c ----------------------------------------------------------------------

c ********** Input parameters for ice crystals *************************
c        ... Copy ice shape into temporary array Pr_inp
         Pr_inp(1:INP_LYR) = CLDPROFILE(NCL,1:INP_LYR,4)
c        ... Interpolate on the actually altitude grid and save 
c            result into array Sh_ice
c         WRITE(*,'(A,*(2x,e12.4))') '  ---  Pr_inp   ',Pr_inp(1:INP_LYR)
         CALL AL_INTERP (INP_LYR, Z_inp, Pr_inp,
     $        CLN, Z_act, Sh_ice(1:CLN), .FALSE., *880)  
c         WRITE(*,*) '           --- 3 '
c         WRITE(*,'(A,*(2x,e12.4))') '  ---  Sh_ice(1:3)   ', Sh_ice(1:3)
         IF( I_lwc.OR.I_pnc ) 
     $        Sh_ice(1:CLN) = Sh_ice(1:CLN)*PERTURB(Jt:Jb)
         
c        ... Integral of shape-function (ice) over cloud layer
         Norm_ice = SUM( ( z_act(1:CLN-1) - z_act(2:CLN) )*
     $        ( Sh_ice(1:CLN-1) + Sh_ice(2:CLN) ) )/2.
     $        + (Sh_ice(1) + Sh_ice(CLN))/2.0d0*Lim
c            WRITE(*,*) '  ===> ICE_FORM(NCL,1)%S   '
c            WRITE(*,*) '            TDS(NCL)%S eto  ', TDS(NCL)%S
c            WRITE(*,*) '  CRST(NCL)%SHP(1)%ICE_FORM ',
c     $           CRST(NCL)%SHP(1)%ICE_FORM
         If(TDS(NCL)%S /= "water") Then
            SELECT CASE( CRST(NCL)%SHP(1)%ICE_FORM )
            CASE('hex')
c           ... Copy prism side length into temporary array Pr_inp
               Pr_inp(1:INP_LYR) = CLDPROFILE(NCL,1:INP_LYR,5)
c              ... Interpolate on the actually altitude grid and save 
c                  result into array R_ice
               CALL AL_INTERP (INP_LYR, Z_inp, Pr_inp,
     $              CLN, Z_act, R_ice, .FALSE., *880)
c              ... Copy prism height into temporary array Pr_inp
               Pr_inp(1:INP_LYR) = CLDPROFILE(NCL,1:INP_LYR,6)
c             ... Interpolate on the actually altitude grid and save 
c                 result into array L_Hex
               CALL AL_INTERP (INP_LYR, Z_inp, Pr_inp,
     $              CLN, Z_act, L_Hex, .FALSE., *880)

            CASE('fra','agg','col','dro','smo',
     $           'hoc','hbr','pla','p10','p5e','sbr','soc')
               IF( RET_FLAG.AND.ITERATION > 1.AND.Ref_wi(2) ) THEN
                  CALL SET_PTR( RTM_PTR, 'cld-ref_nomix' )
                  R_ice(1:CLN) = RTM_PTR%CRN(Jt:Jb)
               ELSE
c              ... Copy into temporary array Pr_inp
c                  - edge length       (for fractal),
c                  - effective radius  (for Baum DB),
c                  - maximal dimension (for Young DB)                    
c                  - mean chord        (for stochastic model)
                  Pr_inp(1:INP_LYR) = CLDPROFILE(NCL,1:INP_LYR,7)
c                 ... Interpolate on the actually altitude grid and save 
c                     result into array R_ice
!                 WRITE(*,*) '  R_ice before all_interp '
!                 DO I = 1, INP_LYR
!                                 WRITE(*,*) Z_inp(I), Pr_inp(I)
!                 ENDDO
                  CALL AL_INTERP (INP_LYR, Z_inp, Pr_inp,
     $                 CLN, Z_act, R_ice(1:CLN), .FALSE., *880)
!                 WRITE(*,*) '  R_ice after all_interp '
!                 DO I = 1, CLN
!                    WRITE(*,*) Z_act(I), R_ice(I)
!                 ENDDO
               ENDIF
c               WRITE(*,*) '           --- 4 '
            END SELECT  ! CRST(NCL)%SHP(1)%ICE_FORM
         End If !TDS(NCL) /= "water"
c ----------------------------------------------------------------------

         N_a = 2 ; N_e = CLN + 1  ; INX = 1
         IF( ANY(Plus_ice) ) THEN
            Idb = 1
            CALL LWP_TO_EXT
            INX = 2
            SELECT CASE ( TDS(NCL)%S )
            CASE('ice-sm','mixed-sm') ;  CALL SM_EXT
            CASE('ice','mixed')       ;  CALL IWP_TO_EXT(*880)
            END SELECT
         ELSE  ! no mixed clouds
            SELECT CASE ( TDS(NCL)%S )
            CASE('alay')
               CALL AER_TO_EXT
            CASE('ice-sm')
               CALL SM_EXT
            CASE('water')
               CALL LWP_TO_EXT
               P_Rdb => Ref_db(:) ; P_aef => A_ef(:)
               Idb = 1
            CASE('ice')
               IF( ANY(Plus_ice) ) INX = 2
               CALL IWP_TO_EXT(*880)
               IF( Use_Ydb ) THEN
                  IF( Use_Dim ) THEN
                     P_Rdb => Dim_Ydb(Ncl,:)
                  ELSE
                     P_Rdb => Ref_Ydb(Ncl,:)
                  ENDIF
               ENDIF
               IF( Use_Bdb ) P_Rdb => Ref_Bdb(:)
               P_aef => A_ef_ice(:)
               Idb = 2
            END SELECT
         END IF
c        Numerical derivative w.r.t. cloud position
c        ------------------------------------------
         IF(TRIM(RTM_CORE) == 'ASYMP') CALL DERIV_NUMERIC (*92)

         IF( LOG_LEVEL(1) <= N_XSO ) THEN
            WRITE(LUNIT,*) '   --- before D-M scaling ----'
            WRITE(LUNIT,*) '   cloud layer: ',NCL
            WRITE(LUNIT,*) '    scattering: ',
c     $           SUM( CLD_LAYER_COEFF(NCL,:,4,: ) )
     $           CLD_LAYER_COEFF(NCL,1,4,2:3 )
c            WRITE(LUNIT,*) '    extinction: ',
c     $           SUM( CLD_LAYER_COEFF(NCL,:,3,: ) )
         ENDIF

c         Save shape of cloud parameter in the CLD_LAYER_ARR(:,1) 
c         -------------------------------------------------------
         CLA(JT:JB,1) = Sh_wat(1:CLN)

c        Delta-m approximation
c        ---------------------
         IF ( CDM > 0.AND.TRIM(SCATT_MODE)/='ss') THEN
c                 WRITE(*,*) '            TDS  ', TDS(NCL)%S
c                 WRITE(*,*) '   Plus_ice(NCL) ', Plus_ice(NCL)
            SELECT CASE ( TDS(NCL)%S )
            CASE('mixed','mixed-sm','water')
               P_Rdb => Ref_db(:) ; P_aef => A_ef(:)
            CASE('ice')
               IF( Use_Ydb ) THEN
                  IF( Use_Dim ) THEN
                     P_Rdb => Dim_Ydb(Ncl,:)
                  ELSE
                     P_Rdb => Ref_Ydb(Ncl,:)
                  ENDIF
               ENDIF
               IF( Use_Bdb ) P_Rdb => Ref_Bdb(:)
               P_aef => A_ef_ice(:)
            CASE('ice-sm')
c              ... %Pdb_ref is calculated in "gt_cloud_input.f90"
c                  in subroutine "SET_PDB_REF"
               P_Rdb => MLCS(Ncl)%Npf(1)%Pdb_ref
               P_aef => A_ef_ice(:)
            END SELECT
            Nxr = MLCS(Ncl)%Npf(1)%Ndb_ref  ;  ALLOCATE( F_DM(Nxr) )
c            WRITE(*,*) '  1 Ndb_ref  (Nxr) ===> ',
c     $           MLCS(Ncl)%Npf(1)%Ndb_ref
            DO I = 1, NCW
c               WRITE(*,*) '        Wavelength ', I
               IF( ITERATION == 1 ) THEN 
c                 ... Over R_eff in DB if Phm_const was set to .FALSE.
c                     i.e., phase function depends on the altitude 
                  DO Ir = 1, Nxr
                     Phm_p => CLD_PHAS_MOMS(I,:,NCL,Ir,1,1:Nm_vs) ! vector
                     SELECT CASE ( D_MF )
                     CASE ( 'D-M' ) ! Delta-M
c                       ... Delta-M factor is calculated for all basis
c                           phase functions
                        F_DM(Ir) = Phm_p(CDM+1,1)/DBLE(2*CDM + 1)
c                       ... Redefine moments of truncated phase function
c                           for all basis phase functions
                        FORALL( K = 1:CDM ) Phm_p(K,1) = 
     $                       ( Phm_p(K,1) - F_DM(Ir)*DBLE(2*K-1) )
     $                       /(1.d0 - F_DM(Ir))
                        IF(Nm_vs > 1) THEN
                           FORALL( K = 1:CDM ) 
                           Phm_p(K,2:4) = (Phm_p(K,2:4) -
     $                          F_DM(Ir)*DBLE(2*K-1))/(1.d0 - F_DM(Ir))
                           Phm_p(K,5:6) = Phm_p(K,5:6)/(1.d0 - F_DM(Ir))
                           END FORALL
                        ENDIF
                        Phm_p( CDM+1:NLEGEN,: ) = 0.d0
                     CASE ( 'D-F' ) ! Delta-fit: scalar case only
                        F_DM(Ir) = F_Dfit(NCL,1)
                        Phm_p(1:CDM,1) = Phm_p(1:CDM,1)/(1.d0-F_DM(Ir))
                        Phm_p( CDM+1:NLEGEN,1 ) = 0.d0
                     CASE ( 'D-S' ) ! Delta-S: scalar case only
                        F_DM(Ir)  = F_Dfit(NCL,1)
                     END SELECT
                     CLD_F_DM(I,Ncl,Ir,1) = F_DM(Ir)
                  ENDDO  ! over all R_eff in water data base (Ir)
               ELSE
c              ... ITERATION > 1 use a priori delta-m factors 
                  F_DM(:)  = CLD_F_DM(I,Ncl,:,1)                     
               ENDIF  ! Iteration=1
c              ... Extinction and scattering coefficients are defined  
c                  here at the altitude levels within each cloud layer, 
c                  i.e., for actual values of R_eff
               Ext_p    => CLD_LAYER_COEFF  ( NCL,I,1,: )
               SCC_p    => CLD_LAYER_COEFF  ( NCL,I,2,: )
               SCC_ns_p => SCC_LAYER_NOSCALE( NCL,I,: )
               IF( MLCS(Ncl)%Phm_co.OR.Nxr==1 ) THEN
                  HH(:) = F_DM(1)
c                  write(*,*) '             selected F_DM ', F_DM
               ELSE
                  IF( W_ref.OR.I_ref ) THEN
c                    ... Linear perturbation of D-M factor caused by Ref
                     CALL DER_DREF_PERT(Nxr, P_Rdb, CLN,
     $                   P_Aef(1:CLN), F_DM, Jt, Jb, HH(1:CLN) )
                  ELSE
c                     write(*,*) '             selected Rdb and F_DM '
c                     write(*,'(*(1x,e10.2))') P_Rdb(:)
c                     write(*,'(*(1x,e10.2))') F_DM(:)
                     CALL LIN_INT(Nxr, P_Rdb, F_DM, 
     $                    CLN, P_aef(1:CLN), HH(1:CLN), 'L')
c                     write(*,*) '             interpolate on and result'
c                     write(*,'(*(1x,e10.2))') P_aef(1:CLN)
c                     write(*,'(*(1x,e10.2))') HH(1:CLN)
                  ENDIF
               ENDIF  ! .not.MLCS(Ncl)%Phm_co
c              ... Save non-normalized scattering coefficient
               SCC_ns_p(:) = SCC_p(:)
c              ... Normalized extinction coefficient
               Ext_p(N_a:N_e) = 
     $              Ext_p(N_a:N_e) - SCC_p(N_a:N_e)*HH(1:CLN)
c                  Test only ! Check for iteration
c              ... Normalized scattering coefficient
               SCC_p(N_a:N_e) = SCC_ns_p(N_a:N_e)*(1.d0 - HH(1:CLN))
            END DO  ! over WL in database (I)
            DEALLOCATE( F_DM )
c            WRITE(*,*) '   First phase NCL ', NCL
c            KK = 1
c            DO K = N_a, N_e 
c               WRITE(*,*)  KK, SCC_p(K), SCC_ns_p(K) 
c               KK = KK + 1
c            ENDDO
c            PAUSE
            IF( Plus_ice(NCL) ) THEN
c           =========================================================
c                 Below for mixed cloud (second - ice)
c           =========================================================
               IF( Use_Ydb ) THEN
                  IF( Use_Dim ) THEN
                     P_Rdb => Dim_Ydb(Ncl,:)
                  ELSE
                     P_Rdb => Ref_Ydb(Ncl,:)
                  ENDIF
               ENDIF
               IF( Use_Bdb ) P_Rdb => Ref_Bdb(:)
               IF( TDS(Ncl)%S=='ice-sm'.OR.TDS(Ncl)%S=='mixed-sm' ) 
     $              P_Rdb => MLCS(Ncl)%Npf(2)%Pdb_ref(:)
               P_aef => A_ef_ice(:)
               Nxr = MLCS(Ncl)%Npf(2)%Ndb_ref  ;  ALLOCATE( F_DMI(Nxr) )
c               WRITE(*,*) '  2 Ndb_ref (Nxr) ===> ',
c     $              MLCS(Ncl)%Npf(2)%Ndb_ref
               DO I = 1, NCW
                  IF( ITERATION == 1 ) THEN
                     DO Ir = 1, Nxr ! for Ice DB 
                        Phm_p => CLD_PHAS_MOMS(I,:,NCL,Ir,2,1:Nm_vs) ! vector
                        SELECT CASE ( D_MF )
                         CASE ( 'D-M' ) ! Delta-M
                           F_DMI(Ir) = Phm_p(CDM+1,1)/DBLE(2*CDM + 1)
                           FORALL( K = 1:CDM ) Phm_p(K,1) = 
     $                          ( Phm_p(K,1) - F_DMI(Ir)*DBLE(2*K-1) )
     $                          /(1.d0 - F_DMI(Ir))
                           IF(Nm_vs > 1) THEN
                              FORALL( K = 1:CDM ) 
                              Phm_p(K,2:4) = (Phm_p(K,2:4) - F_DMI(Ir)
     $                             *DBLE(2*K-1))/(1.d0 - F_DMI(Ir))
                              Phm_p(K,5:6) =
     $                             Phm_p(K,5:6)/(1.d0 - F_DMI(Ir))
                              END FORALL
                           ENDIF
                           Phm_p( CDM+1:NLEGEN,: ) = 0.d0
                        CASE ( 'D-F' ) ! Delta-fit scalar case only
                           F_DMI(Ir) = F_Dfit(NCL,2)
                           Phm_p(1:CDM,1) = Phm_p(1:CDM,1)
     $                          /(1.d0-F_DMI(Ir))
                           Phm_p( CDM+1:NLEGEN,1 ) = 0.d0
                        END SELECT
                        CLD_F_DM(I,Ncl,Ir,2) = F_DMI(Ir)
                     ENDDO      ! over all R_eff in ice data base (Ir)
                  ELSE
c                 ... ITERATION > 1 use a priori delta-m factors 
                     F_DMI(1:Nxr) = CLD_F_DM(I,Ncl,1:Nxr,2)
                  ENDIF         ! Iteration=1 (mixed cloud)
                  Ext_p => CLD_LAYER_COEFF( NCL,I,3,: )
                  SCI_p => CLD_LAYER_COEFF( NCL,I,4,: )
                  SCC_ns_p => SCI_LAYER_NOSCALE( NCL,I,: )
                  IF( MLCS(Ncl)%Phm_co.OR.Nxr==1 ) THEN
                     HH(:) = F_DMI(1)
c                     write(*,*) '             selected F_DMI ', F_DMI
                  ELSE
c                     write(*,*) '             selected Rdb and F_DMI '
c                     write(*,'(*(1x,e10.2))') P_Rdb(:)
c                     write(*,'(*(1x,e10.2))') F_DMI(:)
                    CALL LIN_INT(Nxr, P_Rdb, F_DMI, 
     $                    CLN, P_aef(1:CLN), HH(1:CLN), 'L')
c                    write(*,*) '             interpolate on and result'
c                    write(*,'(*(1x,e10.2))') P_aef(1:CLN)
c                    write(*,'(*(1x,e10.2))') HH(1:CLN)
c                    PAUSE
                 ENDIF
c                ... Save non-normalized ice scattering coefficient
                  SCC_ns_p(:) = SCI_p(:)
c                 ... Normalized ice extinction coefficient
                  Ext_p(N_a:N_e) =
     $                 Ext_p(N_a:N_e) - SCI_p(N_a:N_e)*HH(1:CLN)
c                 ... Normalized ice scattering coefficient (ice)
                  SCI_p(N_a:N_e) = SCI_p(N_a:N_e)*(1.d0 - HH(1:CLN))
               END DO   ! over WL in database (I)
c               WRITE(*,*) '   Second phase NCL ', NCL
c               WRITE(*,*) '   Layer eto ', Ncl
c               WRITE(*,*) '   N_1 and N_2 eto ', N_1, N_2
c               DO I = 1, CLN
c                  WRITE(*,*) I, HH(I), CLD_LAYER_COEFF( NCL,1,N_1,I ),
c     $                 CLD_LAYER_COEFF( NCL,1,N_2,I )
c               ENDDO
c               PAUSE
               DEALLOCATE( F_DMI )
            ENDIF  ! mixed cloud
         ELSE
c        ... Delta-M is not applied
            WRITE(*,*) '   No D-M for cloud ' 
            FORALL( K=1:NCW )
              SCC_LAYER_NOSCALE(NCL,K,:) = 
     $           CLD_LAYER_COEFF(NCL,K,2,:)
              CLD_F_DM(K,Ncl,:,:) = 0.d0
            END FORALL
         END IF
c        -------------------------------------------------------
         IF( LOG_LEVEL(1) <= N_XSO ) THEN
            WRITE(LUNIT,*) '   --- before output ----'
            WRITE(LUNIT,*) '   cloud layer: ',NCL
            WRITE(LUNIT,*) '    scattering: ',
c     $           SUM( CLD_LAYER_COEFF(NCL,:,4,: ) )
     $           CLD_LAYER_COEFF(NCL,1,4,2:3 )
c            WRITE(LUNIT,*) '    extinction: ',
c     $           SUM( CLD_LAYER_COEFF(NCL,:,3,: ) )
         ENDIF
c        -------------------------------------------------------
      END DO  ! over cloud layers (NCL)
c *****************************************************************************

c     ... Write out information about vertical profile of cloud
c         or aerosol layer
      OPEN(46,file=DIR_OUT//'/prof_cld.dat')
      DO NCL = 1, N_CLD_LAYERS
c         write(*,*) '  NCL = ', NCL, TDS(NCL)%S, Plus_ice(NCL)
         JT = J_TOP(NCL) + 1 ; JB = J_BOT(NCL) - 1
         IVT = 1             ; N_col = N_2
         SELECT CASE( TDS(NCL)%S )
c        ... aerosol layer
         CASE('alay')
            N_col = 1
            SUB_TITLE = '#   Aerosol layer scenario. Layer: '
            TITLE=
     $              '#  Z[km]     Conc [1/cm^3]   '
c        ... water cloud-layer
         CASE('water')
            SELECT CASE ( VT(NCL,IVT)%S )
            CASE( 'PNC' )
               SUB_TITLE = '#   Water cloud scenario. Layer: '
               TITLE=
     $              '#  Z[km]     Conc [1/cm^3]   R_ef water [mkm]'
            CASE( 'LWP','LWC','TAU' )
               SUB_TITLE = '#   Water cloud scenario. Layer: '
               TITLE=
     $              '#  Z[km]      LWC [g/m^3]   R_ef water [mkm]'
            END SELECT
c        ... ice cloud-layer
         CASE('ice')
            IF( ANY(Plus_ice)  ) IVT = 2
c                write(*,*) '  IVT eto ', IVT
c                write(*,*) '  VT(NCL,IVT)%S ', VT(NCL,IVT)%S
            SELECT CASE ( VT(NCL,IVT)%S )
            CASE( 'CNC' )
               SUB_TITLE = '#   Ice cloud scenario. Layer: '
               TITLE=
     $              '#  Z[km]     Conc [1/cm^3]   R_ef ice [mkm]'
            CASE( 'IWP','IWC','TAU' )
               SUB_TITLE = '#   Ice cloud scenario. Layer: '
c                       write(*,*) '    Use_Ydb eto ', Use_Ydb
c                       write(*,*) '    Use_Dim eto ', Use_Dim
               IF( Use_Ydb.AND.Use_Dim ) THEN
                  TITLE=
     $                 '#  Z[km]      IWC [g/m^3]   D_max ice [mkm]'
               ELSE
                  TITLE=
     $                 '#  Z[km]      IWC [g/m^3]   R_ef ice [mkm]'
               ENDIF
            END SELECT
         CASE('ice-sm')
            IF( ANY(Plus_ice)  ) IVT = 2
c            write(*,*) '  VT(NCL,IVT)%S ', VT(NCL,IVT)%S
            SELECT CASE ( VT(NCL,IVT)%S )
            CASE( 'CNC' )
               SUB_TITLE = '#   Stochastic model scenario. Layer: '
               TITLE=
     $              '#  Z[km]     Conc [1/cm^3]   R_ef ice [mkm]'
            CASE( 'IWP','IWC','TAU' )
               SUB_TITLE = '#   Stochastic model  scenario. Layer: '
               TITLE=
     $              '#  Z[km]      IWC [g/m^3]   R_ef ice [mkm]'
            END SELECT
c        ... mixed cloud-layer
         CASE('mixed','mixed-sm')
            SUB_TITLE = '#   Mixed cloud scenario. Layer: '
            IF( Use_Dim ) THEN
               TITLE='#  Z[km]      LWC [g/m^3]   R_ef water [mkm]'//
     $              '    IWC [g/m^3]   D_max ice [mkm]'
            ELSE
               TITLE='#  Z[km]      LWC [g/m^3]   R_ef water [mkm]'//
     $              '    IWC [g/m^3]   R_ef ice [mkm]'
            ENDIF
         END SELECT
         WRITE( 46,'(A,1x,I3,1x,F9.5,A,1x,F9.5,A)' ) 
     $        TRIM(SUB_TITLE), NCL, ZZZ(JB),' - ',ZZZ(JT),' km'
         WRITE( 46,'(A)' ) TRIM(TITLE)
c         WRITE( *,'(A)' ) '# '
c         WRITE( *,'(A,1x,I3,1x,F9.5,A,1x,F9.5,A)' ) 
c     $        TRIM(SUB_TITLE), NCL, ZZZ(JB),' - ',ZZZ(JT),' km'
c         WRITE( *,'(A)' ) TRIM(TITLE)
c         WRITE( *,'(A)' ) '# '
      ENDDO                     ! over cloud layers (NCL)
c
      DO NCL = 1, N_CLD_LAYERS
         DO I = J_TOP(NCL)+1, J_BOT(NCL)-1
            WRITE(46,'(2x,F15.11,4(2x,e15.7) )') 
     $           ZZZ(i), (Prof_Cpar(I,K), K = 1,N_col)
         ENDDO
      ENDDO
      CLOSE(46)
      
      IF( NUM_CLD ) ITERATION = 2

      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN

 92   Call WRITE_MESSAGE (R_NAME, 'ERROR', 'Interpolation error')
      RETURN 1                                           

 880  WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)
      RETURN 1
 
      CONTAINS
c***********************************************************************
      SUBROUTINE SM_EXT
      USE CLD_INP, ONLY:  ADD_ABS, MLCS, CLD_INHOM
      USE CLOUDS,  ONLY:  Ref_ip, RefI_r, RefI_i
c     -------------------------------------------------------------         
      INTEGER    I, J, KK
      REAL (KND) Rr_05, Ri_05, WL_05, Ro_ice, C_iwp
c     ... Real and imaginary part of ice refractive index according
c         to Waren and Brandt, 2008
      PARAMETER  ( WL_05=500.0d0, Rr_05=1.3130d0,Ri_05=5.889d-10 ) 
c     ... Density of ice, 0.917 g/cm^3"  917 kg/m3
      PARAMETER ( Ro_ice = 0.917d0 )
      REAL (KND) Omega(Ncw,CLN), Alph(Ncw,CLN), C_Nc
      REAL (KND), DIMENSION(CLN):: Iwc_a, Sec_int, V_fra, Yel_ac, Ex_cof
c     ==================================================================
c       ... Optical thickness as integral input parameter is given at 
c           the reference wavelength (see input file "cf_cloud.inp"). 
c     --------------------------------------------------------------
      Yel_ac = 0.d0
      IF( ADD_ABS(Ncl) ) 
     $     Yel_ac(:) = Pr_SOOT(Jt:Jb)*1.d-9 ! m^-1 ==> nm^-1
      A_ef_ice(1:CLN) = R_ice(1:CLN)
c     ... R_ice below is mean chord of irregular ice crystal
c         used to calculate single scattering albedo
      CALL OMEGA_SM( NCW, CLD_NETWAVS, RefI_r, RefI_i, NCL, CLN,
     $     R_ice(1:CLN), Omega, Alph, Yel_ac )
c     ... In the approximation used, the extinction coefficient does
c         not depend on the wavelength
      Ex_cof(:) =  1.d0/R_ice(1:CLN)*1.d9 ! convert [mkm-1] to [km-1]

      WRITE(*,*) '    ***** In "sm_ext" Layer ***** ', Ncl
c      WRITE(*,*) '    VT(NCL,INX)%S  ', VT(NCL,INX)%S
      SELECT CASE ( VT(NCL,INX)%S )
      CASE('TAU')
         write(*,*) '     Tau as input ', TAU_CLD_INP(NCL,INX)
c        ... Sh_ice is a shape-profile of volume fraction of ice.
         Sec_int(:) = Ex_cof(:)*Sh_ice(1:CLN) 
c        ... COT at reference wavelength and Sec_int as shape-profile
         Tau_loc = SUM( ( z_act(1:CLN-1) - z_act(2:CLN) )*
     $        (Sec_int(1:CLN-1) + Sec_int(2:CLN) ) )/2.d0
     $        + (Sec_int(1) + Sec_int(CLN))/2.d0*Lim
c        ... Scaling factor for volume fraction
         C_Nc = TAU_CLD_INP(NCL,INX)/Tau_loc         
c     ... Corresponding volume fraction profile
         V_fra(1:CLN) = C_Nc*Sh_ice(1:CLN)
         Iwc_a(:) = V_fra(1:CLN)*(Ro_ice*1.d09) ! ice water content [g/m^2/km]
      CASE('IWC')
c     ... IWC profile in absolute units is used as input [Sh_ice] = g/m^3
         C_iwp = 1.d03                          ! 1/m --> 1/km
         Iwc_a(:) = C_iwp*Sh_ice(1:CLN)         ! g/m^2/km
         V_fra(:) = Sh_ice(1:CLN)/(Ro_ice*1.d06) ! volume fraction of ice 
      CASE('IWP')
         write(*,*) '    IWP as input ', IWP(NCL)
         write(*,*) '    Norm_ice     ', Norm_ice
c     ... Scaling of the shape-profile to match given IWP [g/m^2]
c         Norm_ice = [km] integral of volume fraction shape-profile
         C_iwp = IWP(NCL)/(Norm_ice*Ro_ice*1.d09) ! [g/m^2/(g/m^2)
         V_fra(:) =  C_iwp*Sh_ice(1:CLN)! volume fraction of ice
         Iwc_a(:)  = V_fra(:)*Ro_ice*1.d09 ! [IWC(:)] = g/m^2/km
      END SELECT

      Sec_int(:) = Ex_cof(:)*V_fra(1:CLN) 
c    ... Used TAU
      Tau_CLD_inp(NCL,INX) = SUM( ( z_act(1:CLN-1) - z_act(2:CLN) )*
     $     ( Sec_int(1:CLN-1) + Sec_int(2:CLN) ) )/2.d0  !*1000.d0
     $     + (Sec_int(1) + Sec_int(CLN))/2.d0*Lim        !*1000.d0
c    ... Used IWP [g/m^2]
      IWP(NCL) = SUM( ( z_act(1:CLN-1) - z_act(2:CLN) )*
     $     ( Iwc_a(1:CLN-1) + Iwc_a(2:CLN) ) )/2.d0  !*1000.d0
     $     + (Iwc_a(1) + Iwc_a(CLN))/2.d0*Lim        !*1000.d0

      WRITE(*,*) '    ice-sm ', NCL, VT(NCL,INX)%S
      WRITE(*,*) '   Tau_CLD_inp(NCL,INX) and IWP(NCL) [g/m^2]',
     $      Tau_CLD_inp(NCL,INX), IWP(NCL)
c     ... Calculate extinction and scattering coefficients 
      DO J = 1,NCW
         CLD_LAYER_COEFF(NCL,J,N_1,2:CLN+1) = Sec_int(1:CLN)
         CLD_LAYER_COEFF(NCL,J,N_2,2:CLN+1) =
     $        CLD_LAYER_COEFF(NCL,J,N_1,2:CLN+1)*Omega(J,1:CLN)
      ENDDO
      
c     ... Extinction and scattering coefficients of snow layer 
c         are set to zero at the top and bottom of infinitesimal 
c         sub-layers
      CLD_LAYER_COEFF( NCL,1:NCW,N_1:N_2,1 )     = 0.d0 
      CLD_LAYER_COEFF( NCL,1:NCW,N_1:N_2,CLN+2 ) = 0.d0 
      
      DO J = 1,NCW
         Ext_p => CLD_LAYER_COEFF(NCL,J,N_1,2:CLN+1)
         Tau_loc = SUM( ( z_act(1:CLN-1) - z_act(2:CLN) )*
     $        (Ext_p(1:CLN-1) + Ext_p(2:CLN) ) )/2.d0
     $        + (Ext_p(1) + Ext_p(CLN))/2.d0*Lim
      ENDDO

c       ... Save IWC(z) for an actual altitude grid [g/m^3]
        Prof_Cpar(Jt:Jb,N_1) = Iwc_a(1:CLN)/1000.0d0
cc       ... Effective radius of stochastic model
c        Prof_Cpar(Jt:Jb,N_2) = 3.d0/4.d0*R_ice(1:CLN)
c       ... Mean chord of irregular ice crystal
        Prof_Cpar(Jt:Jb,N_2) = R_ice(1:CLN)
c       ... A layer is vertically homogeneous
        IF( WF_FLAG ) THEN
           CLD_LAYER_DERIV(NCL,:,N_1,1:CLN+2) = 0.d0
           CLD_LAYER_DERIV(NCL,:,N_2,1:CLN+2) = 0.d0
        ENDIF

      RETURN
      END SUBROUTINE SM_EXT
c *******************************************************************

      SUBROUTINE AER_TO_EXT
      USE CLD_INP, ONLY:  Aer_ref
      
      INTEGER    I, J, KK
      REAL (KND) C_Nc
      REAL (KND), DIMENSION(CLN):: Sec_int, N_part
c     ==================================================================

c       ... Optical thickness as integral input parameter is given at 
c           the reference wavelength (see input file "aer_ref_ecs.dat"). 
c           Array Sh_wat is considered as a shape-profile of 
c           the aerosol particle number density
c       --------------------------------------------------------------
c     ... extinction coefficient at reference wavelength and shape-profile
      Sec_int(:) = Aer_ref*Sh_wat(1:CLN) 
c     ... COT at reference wavelength and N(z) as shape-profile
      Tau_loc = SUM( ( z_act(1:CLN-1) - z_act(2:CLN) )*
     $     (Sec_int(1:CLN-1) + Sec_int(2:CLN) ) )/2.d0
     $     + (Sec_int(1) + Sec_int(CLN))/2.d0*Lim
c     ... Scaling factor for particles number density 
      C_Nc = TAU_CLD_INP(NCL,INX)/Tau_loc
c     ... Corresponding N(z) profile
      N_part(1:CLN) = C_Nc*Sh_wat(1:CLN)
c     --------------------------------------------------------------
        
c     ... Calculate extinction and scattering coefficients 
      DO J = 1,NCW
         CLD_LAYER_COEFF(NCL,J,1,2:CLN+1) = 
     $        CLD_LAYER_COEFF(NCL,J,1,1)*N_part(1:CLN)
         CLD_LAYER_COEFF(NCL,J,2,2:CLN+1) = 
     $        CLD_LAYER_COEFF(NCL,J,2,1)*N_part(1:CLN)
      ENDDO
c     ... Extinction and scattering coefficients of aerosol layer 
c         are set to zero at the top and bottom of infinitesimal 
c         sub-layers
      CLD_LAYER_COEFF( NCL,1:NCW,1:2,1 )     = 0.d0 
      CLD_LAYER_COEFF( NCL,1:NCW,1:2,CLN+2 ) = 0.d0 

      DO J = 1,NCW
         Ext_p => CLD_LAYER_COEFF(NCL,J,1,2:CLN+1)
         Tau_loc = SUM( ( z_act(1:CLN-1) - z_act(2:CLN) )*
     $        (Ext_p(1:CLN-1) + Ext_p(2:CLN) ) )/2.d0
     $        + (Ext_p(1) + Ext_p(CLN))/2.d0*Lim
c              WRITE(*,'(2x,I2,2x,E12.4)') J, Tau_loc
      ENDDO

c       ... Save N(z) [1/cm^3] for the actual altitude grid
        Prof_Cpar(Jt:Jb,1) = Sh_wat(1:CLN)
c       ... Does not use in the case of aerosol layer
        Prof_Cpar(Jt:Jb,2) = 0.d0
c       ... Aerosol layer vertically homogeneous
        IF( WF_FLAG ) THEN
           CLD_LAYER_DERIV(NCL,:,N_1,1:CLN+2) = 0.d0
           CLD_LAYER_DERIV(NCL,:,N_2,1:CLN+2) = 0.d0
        ENDIF

      RETURN
      END SUBROUTINE AER_TO_EXT
c *********************************************************************

      SUBROUTINE LWP_TO_EXT
c      Weast, Robert C., Handbook of Chemistry and Physics. 61st Edition. 
c      Florida: CRC, 1981: F-1. "Density of Various Solids, 
c      Ice, 0.917 g/cm 3"  917 kg/m3
c      Water 1. g/cm^3
c-----------------------------------------------------------------------
      INTEGER  I, J
      REAL (KND) Bk, Ck, Sk, Pi, Pok, C_lwp, Gmu, LWP_loc, C_ec, P_term,
     $     D_D, Ro_w
      REAL (KND), DIMENSION(CLN):: Lwc_a, CSe, Elwc, Sec_int, A_Vol,  
     $     N_part, TM1, TM2, TM3, De_Dr, Hx, E_cof
      REAL (KND), DIMENSION(NCW):: Pkr, Wl, Pd, Hf, Ome, Ke, Ka
      LOGICAL CHECK_DER
      PARAMETER  ( Bk = 1.1d0, Ck = 1.7d-06, Sk = 56.3d0 )
      PARAMETER  ( Pi = 3.1415927d0, Pok = 2.0d0/3.0d0, Ro_w = 1.d0 )
c     ... Gmu is mu-parameter of Gamma-distribution used by Mie 
c         calculations of cloud data base
      PARAMETER  ( Gmu = 6.0d0 )
c     ==================================================================

c      WRITE(*,*) '   LWP_TO_EXT =================>  '

c     ... Transform wavelength in [mkm]
      WL(:)   = CLD_NETWAVS(:)*1.0d-03
c     ... Average volume of droplets (see Eq.(Avol-W) in STB)
c         having Gamma distribution of particle size
      A_Vol(:) = 4.d0/3.d0*Pi*A_ef(1:CLN)**3
     $     *(Gmu + 2.d0)*(Gmu + 1.d0)/(Gmu + 3.d0)**2  ! [mkm^3]
      IF( W_ref ) THEN
c        ... Linear perturbation of cross-section caused by Ref
         CALL DER_DREF_PERT(N_Ref_db, Ref_db, CLN,
     $        A_ef(1:CLN), Ext_DB, Jt, Jb, CSe)
      ELSE
c        ... Extinction cross-sections at reference wavelength (500 nm)
c            are obtained for input effective radii A_ef and 
c            stored after interpolation in array CSe [mkm^2]
         CALL LIN_INT(N_Ref_db, Ref_db, Ext_DB, 
     $        CLN, A_ef(1:CLN), CSe, 'L')
      ENDIF
c     ... [Elwc] = mkm^2/1.d-12*g = m^2/g
      Elwc(:) = CSe(:)/A_Vol(1:CLN)/Ro_w

c       Calculation of the water extinction and single-scattering albedo
c            LWP(NCL) is input value of the liquide water path [g/m^2]
c       ----------------------------------------------------------------
c      write(*,*) '  Check  VT(NCL,INX)%S  ', NCL, INX
      SELECT CASE ( VT(NCL,INX)%S )
      CASE('TAU') 
c       ----------------------------------------------------------------
c       ... Optical thickness as an integral input parameter is given at 
c           the wavelength equal to 500 nm.
c       ... extinction coefficient at reference wavelength and shape-profile
         SELECT CASE( TRIM(SCALE_PROF) )
         CASE('Scale-NC')
c           ... Array Sh_wat is considered as N(z) [1/cm^3], [CSe] = mkm^2
            E_cof(:) = CSe(:)*Sh_wat(1:CLN) *1.d-3 ! [1/km]
c            [V*Ro] = mkm^3*g/cm^3 = g*1.d-12
c            [N(z)] = 1/cm^3 = 1./1.e-4*m^2/1.e-5*km = 1.e+9/m^2/km
            TM1(:) = A_Vol(:)*Ro_w*1.d-3
c           ... 1/cm^3 = 1/1.d+8*mkm^2/1.d-5*km = 1.d-3*1/mkm^2/km
            TM2(:) = 1.d-3
         CASE('Scale-WC')
c          ... Array SH_wat is consideredas a LWC = g/m^3 = 1.d3*g/m^2/km
            E_cof(:) = Elwc(:)*Sh_wat(:)*1.d+3 ! [1/km]
            TM1(:) = 1.d+3 ! g/m^3 = 1.d3*g/m^2/km
c           ... [LWC/(V*Ro)] = g/(m^3*1e-12*g) = 1/(m^2*1e-12*1.e-3*km)
c               = 1.e+3*1/mkm^2/km)
            TM2(:) = 1.d0/A_Vol(:)/Ro_w*1.d+3
         END SELECT
c          ... COT at reference wavelength for a shape-profile
         D_D = SUM( ( z_act(1:CLN-1) - z_act(2:CLN) )*
     $        (E_cof(1:CLN-1) + E_cof(2:CLN) ) )/2.d0
     $        + (E_cof(1) + E_cof(CLN))/2.d0*Lim
c          ... Scaling factor for extinction coefficient
         C_ec = TAU_CLD_INP(NCL,INX)/D_D 
         Prof_Ref(Jt:Jb,1) = E_cof(:)/D_D 
c          ... Scaling LWC and N(z)
         Lwc_a(:)  = C_ec*Sh_wat(1:CLN)*TM1(:) ! [g/m^2/km]
         N_part(:) = C_ec*Sh_wat(1:CLN)*TM2(:) ! [1/mkm^2/km] 
      CASE('LWC')
c       ----------------------------------------------------------------
c          ... LWC profile is used according to the input file without scaling
c              i.e., {Sh_wat] = g/m^3
         C_lwp = 1.D03          ! to have LWC(:) = C_lwp*Sh_wat(:) in g/m^2/km
         Lwc_a(:)  = C_lwp*Sh_wat(1:CLN)
         N_part(:) = Lwc_a(:)/A_Vol(:)/Ro_w ! t-1
      CASE('LWP')
c       ----------------------------------------------------------------
c         ... Scaling of the shape-profile to match given LWP 
         C_lwp = LWP(NCL)/Norm  ! [C_lwp] = g/m^2/km
         Lwc_a(:) = C_lwp*Sh_wat(1:CLN)
         N_part(:) = Lwc_a(:)/A_Vol(:)/Ro_w ! t-1
      CASE('PNC')
c       ----------------------------------------------------------------
c          ... Calculation of the liquid water content profile corresponding 
c              to the given profile of number concentration of particles,
c              [Sh_wat(I)]=1/cm^3, and the effective radius, [A_ef(I)]=micron.
c          ... [Lwc_a] = (1/cm^3)*micron^3*(g/cm^3) = g/m^2/km*1.D-3
         Lwc_a(:) = 1.D-03*Sh_wat(1:CLN)*A_Vol(:)*Ro_w
         N_part(:) = 1.D-03*Sh_wat(1:CLN) ! t-1
c       ----------------------------------------------------------------
      END SELECT

c       ... Save all extinction and scattering cross-section from available 
c           water database (needed for calculations derivatives) 
c           because array CLD_LAYER_COEF will be recalculated for actual 
c           profile of effective radius
      IF( WF_FLAG ) CLD_LAYER_DERIV(NCL,:,1:2,1:N_Ref_db) 
     $     = CLD_LAYER_COEFF(NCL,:,1:2,1:N_Ref_db)

c     ... Liquid water path of the cloud layer
      LWP_loc = SUM( ( z_act(1:CLN-1) - z_act(2:CLN) )*
     $     (Lwc_a(1:CLN-1) + Lwc_a(2:CLN) ) )/2.d0
     $     + (Lwc_a(1) + Lwc_a(CLN))/2.d0*Lim        
        
c       ... Check derivatives
      CHECK_DER = .FALSE.
      IF( CHECK_DER ) CALL DERIV_SCA_ABS_WRT_REF
c
c  ************ Extinction and scattering coefficients **********************
c                  at database discrete wavelengths
c      WRITE(*,*) '    HOW_CLD_COEF ', TRIM(HOW_CLD_COEF)
      SELECT CASE( TRIM(HOW_CLD_COEF) )
      CASE('analytically')
c       ... Water scattering and extinction coefficients are calculated
c           using analytical formulae
         DO I = 1, CLN
            Ext_p => CLD_LAYER_COEFF( NCL,1:NCW,1,I+1 ) 
            SCC_p => CLD_LAYER_COEFF( NCL,1:NCW,2,I+1 ) 
            Pd(:)  = 2.d0*Pi*A_EF(I)/WL(:)
            Pkr(:) = 1.d0/Pd(:)**Pok
c             ... Approximation of extinction coefficient 
c                 (see Eq.(Awd_ext) in STB)
            Ke(:)  = 1.d0 + Bk*Pkr(:) + Ck*( DEXP(Sk*Pkr(:)) - 1.d0 )
c             ... Overwrite 'mie_data' or 'user' value [Ext_p] = 1/km
            Ext_p(:) = 1.5d0/A_ef(I)*Ke(:)*Lwc_a(I)
c             ... Approximation of scattering coefficient 
c                 (see Eq.(Awd_sca) in STB)
            Ka(:) = 1.23d0*(1.d0 - 2.6d0*REF_IMAG_W(:)*Pd(:))
     $           *( 1.d0 + 0.34d0*(1.d0 - DEXP(-8.d0*wl(:)/A_ef(I))) )
c             ... Overwrite 'mie_data' or 'user' value [Scc_p] = 1/km
            SCC_p(:) = Ext_p(:)
     $           - 4.d0*Pi*REF_IMAG_W(:)/WL(:)*Ka(:)*Lwc_a(I)
         ENDDO

      CASE('from data base')
c       ... Water scattering and extinction coefficients are used
c           from database
         IF( PHASE_NAME(Ncl)%S == 'user') THEN
            DO J = 1,NCW
               CLD_LAYER_COEFF(NCL,J,1,2:CLN+1) = 
     $              CLD_LAYER_COEFF(NCL,J,1,1)*N_part(1:CLN)
               CLD_LAYER_COEFF(NCL,J,2,2:CLN+1) = 
     $              CLD_LAYER_COEFF(NCL,J,2,1)*N_part(1:CLN)
            ENDDO
         ELSE
            DO J = 1,NCW
               Sec_p => CLD_LAYER_COEFF(NCL,J,1,1:N_Ref_db) 
c                *******************************************************
               IF( W_ref ) THEN
c                   ... Linear perturbation of cross-section caused by Ref
                  CALL DER_DREF_PERT(N_Ref_db, Ref_db, CLN,
     $                 A_ef, Sec_p, Jt, Jb, Sec_int)
               ELSE
c                   ... Interpolate water extinction cross-section [micron^2]  
c                       on the effective radius profile
                  CALL LIN_INT(N_Ref_db, Ref_db, Sec_p, 
     $                 CLN, A_ef(1:CLN), Sec_int, 'L')
               ENDIF
c                ... [ext.coef] = cm^3/m^2/micron/km = 1/km
               CLD_LAYER_COEFF(NCL,J,1,2:CLN+1) 
     $              = Sec_int(1:CLN)*N_part(1:CLN)

               Sec_p => CLD_LAYER_COEFF(NCL,J,2,1:N_Ref_db)
c                *******************************************************
               IF( W_ref ) THEN
c                   ... Linear perturbation of cross-section caused by Ref
                  CALL DER_DREF_PERT(N_Ref_db, Ref_db, CLN,
     $                 A_ef, Sec_p, Jt, Jb, Sec_int)
               ELSE
c                   ... Interpolate water scattering cross-section [micron^2]  
c                       on the effective radius profile
                  CALL LIN_INT(N_Ref_db, Ref_db, Sec_p, 
     $                 CLN, A_ef(1:CLN), Sec_int, 'L')
               ENDIF                 
c                ... [sca.coef] = cm^3/m^2/micron/km = 1/km
               CLD_LAYER_COEFF(NCL,J,2,2:CLN+1) 
     $              = Sec_int(1:CLN)*N_part(1:CLN)
            ENDDO               ! over basis CLD WL grid (J)
         ENDIF
      END SELECT
c       ... Extinction and scattering coefficients of a cloud are set 
c           to zero at the top and bottom of infinitesimal layers
      CLD_LAYER_COEFF( NCL,1:NCW,1:2,1 )     = 0.d0 
      CLD_LAYER_COEFF( NCL,1:NCW,1:2,CLN+2 ) = 0.d0
      
c             Renormalization   !!!    Preliminary   !!!
      IF( PHASE_NAME(Ncl)%S == 'user') THEN
c           WRITE(*,*) '***** Test Tau in "user" cloud mode *****'
         DO J = 1,NCW
            Ext_p => CLD_LAYER_COEFF(NCL,J,1,2:CLN+1)
            SCC_p => CLD_LAYER_COEFF(NCL,J,2,2:CLN+1)
            Tau_loc = SUM( ( z_act(1:CLN-1) - z_act(2:CLN) )*
     $           (Ext_p(1:CLN-1) + Ext_p(2:CLN) ) )/2.d0
     $           + (Ext_p(1) + Ext_p(CLN))/2.d0*Lim
            Ext_p(:) = Ext_p(:)*TAU_CLD_INP(NCL,INX)/Tau_loc
            SCC_p(:) = SCC_p(:)*TAU_CLD_INP(NCL,INX)/Tau_loc
         ENDDO  ! over reference WL (J)
      ENDIF  ! 'user'

c       ... Optical thickness for given LWP at 500 nm
      Hx(:) = CSe(:)*N_part(:)
      Tau_loc = SUM( (z_act(1:CLN-1) - z_act(2:CLN))*
     $     (Hx(1:CLN-1)+Hx(2:CLN) ) )/2.d0 + (Hx(1)+Hx(CLN))/2.d0*Lim

c      WRITE(*,*) '   VT(NCL,INX)%S ', VT(NCL,INX)%S 
c      WRITE(*,*) '  Tau_loc water: ', Tau_loc  
      
      Eff_Ext = Tau_CLD_inp(NCL,INX)/(TOP - BASE)

c       ... Save R_ef(z) for the actual altitude grid
      Prof_Cpar(Jt:Jb,2) = A_ef(1:CLN)
c      write(*,*) Prof_cpar( Jt:Jb,2 )

      SELECT CASE ( VT(NCL,INX)%S )
      CASE( 'TAU' )
c       ----------------------------------------------------------------
         LWP(NCL) = LWP_loc  
         Prof_Cpar(Jt:Jb,1) = Lwc_a(1:CLN)/1000.0d0 
         SELECT CASE( SCALE_PROF )
         CASE('Scale-NC') ; P_term =  0.d0
         CASE('Scale-WC') ; P_term = -3.d0
         END SELECT
      CASE( 'PNC' )  ! Particle number concentration
c       ----------------------------------------------------------------
         Tau_CLD_inp(NCL,INX) = Tau_loc ; P_term = -3.d0
c        ... Save N(z) [1/cm^3] for the actual altitude grid
         LWP(NCL) = LWP_loc ; Prof_Cpar(Jt:Jb,1) = Sh_wat(1:CLN)
      CASE( 'LWP' )
c       ----------------------------------------------------------------
         Tau_CLD_inp(NCL,INX) = Tau_loc  ; P_term = -3.d0
c          ... Save LWC(z) for the actual altitude grid [g/m^3]
         Prof_Cpar(Jt:Jb,1) = Lwc_a(1:CLN)/1000.0d0 
      CASE( 'LWC' )
c       ----------------------------------------------------------------
         Tau_CLD_inp(NCL,INX) = Tau_loc ; P_term = -3.d0
         LWP(NCL) = SUM( ( z_act(1:CLN-1) - z_act(2:CLN) )*
     $        ( Sh_wat(1:CLN-1) + Sh_wat(2:CLN) ) )/2.0d0*1000.0d0
     $        + (Sh_wat(1) + Sh_wat(CLN) )/2.0d0*Lim*1000.0d0 ! modif
c          ... Save LWC(z) for the actual altitude grid [g/m^3]
         Prof_Cpar(Jt:Jb,1) = Sh_wat(1:CLN) !*CLA(JT:JB,1) ! modif
      END SELECT  ! 

c       ... Derivatives of the absorption and scattering coefficient
c           with respect to the effective radius of water droplets
      TCI = Tau_CLD_inp(NCL,INX)

c        IF( WF_FLAG .OR. NUM_CLD ) THEN
      IF( WF_FLAG ) THEN
         SELECT CASE( TRIM(HOW_CLD_COEF) )
         CASE('from data base')
c        ------ Derivatives 'numerically' for water --------------------
            CALL DCOF_DREF(N_Ref_db, Ref_db, Ext_DB, A_ef(1:CLN), 
     $           1, 2, N_part, CSe, P_term)
         CASE('analytically')
c        ------ Derivatives 'analytically' for water ---------------------
            DO I = 1, CLN  
               Ext_p => CLD_LAYER_DERIV( NCL,1:NCW,1,I+1 ) 
               SCC_p => CLD_LAYER_DERIV( NCL,1:NCW,2,I+1 ) 
               Hf(:) = 1.34d0 - 0.34d0*DEXP( -8.d0*wl(:)/A_ef(I) )
               Pd(:) = 2.d0*Pi*A_ef(I)/WL(:)
               Ka(:) = 1.23d0*(1.d0 - 2.6d0*REF_IMAG_W(:)*Pd(:))*Hf(:)
c                ... Abs_coef w.r.t. R_ef : R_ef/Abs*D_Abs/D_Ref
               Ext_p(:) = 8.d0*wl(:)/A_ef(I)*(1.d0 - 1.34d0/Hf(:)) +
     $              1.d0 - 1.23d0*Hf(:)/Ka(:)    

               Pkr(:) = 1.d0/Pd(:)**Pok
               Ke(:)  = 1.d0 + Bk*Pkr(:) + Ck*( DEXP(Sk*Pkr(:))-1.d0 )
               SCC_P(:) = 1.d0 + 
     $              ( Bk + Ck*Sk*DEXP(Sk*Pkr(:)) )*Pkr(:)/Ke(:)*Pok
               ome(:) =1.d0-Ka(:)/Ke(:)/1.5d0*REF_IMAG_W(:)*2.d0*Pd(:)
c                ... Sca_coef w.r.t. R_ef : R_ef/Sca*D_Sca/D_Ref
               SCC_P(:) = -SCC_P(:)/ome(:)
     $              - Ext_p(:)*(1.d0/ome(:) - 1.d0)
c                ... If N(z) selected for analytical representation of coeff.
               IF(VT(NCL,INX)%S=='TAU' .AND. 
     $              SCALE_PROF=='Scale-NC') THEN
                  Ext_p(:) = Ext_p(:)  + 3.d0
                  SCC_P(:) = SCC_P(:)  + 3.d0
               ENDIF
            ENDDO  ! over cloud sub-layers (I)

c           ... The following will be used to correct WF for effective 
c               radius calculated under condition of constant optical
c               thickness at the reference wavelength and analytical
c               expression for extinction coefficient
            IF( VT(NCL,INX)%S=='TAU' ) THEN
c                                       This | is reference WL in micron
               TM1(:) = 2.d0*Pi*A_ef(1:CLN)/0.5d0
               TM2(:) = 1.d0/TM1(:)**POK
               TM3(:) = 1.5d0*Lwc_a(:)/A_ef(1:CLN)
     $              *( 1.d0 + BK*TM2(:) + CK*(DEXP(SK*TM2(:))-1.d0) )
c             ... Analytically derivative of the extinction coefficient
c                 at the reference wavelength (0.5 micron) w.r.t. 
c                 effective radius, i.e., Dext/Dref
               De_Dr(:) = -TM3(:)*
     $              ( 1.d0/A_ef(1:CLN) + POK/TM1(:)**(5.d0/3.d0)
     $              *(BK + CK*SK*POK*DEXP(SK*TM2(:)))*4.d0*Pi )
c             ... See second term in Eq.(De_Dref) in STB
               Prof_Ref(Jt:Jb,2) = De_Dr(:)*A_ef(1:CLN)/TCI
c              ... If N(z) selected for analytical representation of coeff.
               IF( SCALE_PROF=='Scale-NC' ) THEN
c                 ... Extinction coefficient for reference wavelength (500 nm)
c                     using analytical approximation
                  TM3(:) = 1.d0 + Bk*TM2(:) + Ck*(DEXP(Sk*TM2(:))-1.d0)
                  Hx(:) = 1.5d0/A_ef(1:CLN)*TM3(:)*N_part(:)*A_Vol(:)
c                ... See second term in Eq.(De_Dref_NR) in STB
                  Prof_Ref(Jt:Jb,2) = Prof_Ref(Jt:Jb,2) 
     $                 + 3.d0*Hx(1:CLN)/TCI
            ENDIF  ! SCALE_PROF=='Scale-NC'
         ENDIF  ! VT(NCL,INX)%S=='TAU'
c        ------ End case derivatives 'analytically' for water -------------

         END SELECT
         CLD_LAYER_DERIV( NCL,1:NCW,1:2,1 )     = 0.d0
         CLD_LAYER_DERIV( NCL,1:NCW,1:2,CLN+2 ) = 0.d0 
      ENDIF  ! WF_FLAG

      RETURN
      END SUBROUTINE LWP_TO_EXT
c *********************************************************************

      SUBROUTINE IWP_TO_EXT(*)
      USE CLD_INP,  ONLY: MLCS
c     ----------------------------------------------------------------
c     Weast, Robert C., Handbook of Chemistry and Physics. 61st Edition. 
c      Florida: CRC, 1981: F-1. "Density of Various Solids, 
c      Ice, 0.917 g/cm^3"  917 kg/m3
      INTEGER     I, J, N_cr, I_db, II
      REAL (KND)  Pi, Ro_ice, C_iwp, C_ec, IWP_loc, P_term, D_D,
     $     Len_hex, Cv, Car, Car1, Scl, Pa_03, P12
      REAL (KND), DIMENSION(CLN) :: Iwc_a, Vol, CSe, Eiwc, Sec_int, 
     $     N_part, C_conc, S_sur, Ex_cof, TM1, TM2, Tel, Area
      REAL (KND), DIMENSION(NCW) :: Wl, Pd, LC 
      REAL (KND)  ome_ice(NCW,CLN)
      PARAMETER ( Pi = 3.1415927d0, Ro_ice = 0.917d0 )
      REAL (KND), ALLOCATABLE:: Ef_Rad(:), CS_DB(:)
      TARGET :: Ef_Rad
      REAL (KND), POINTER :: R_cr(:), Sec_p(:), D_Po(:)
c    ===================================================================

      Ex_cof(:) = 1.d0
      INQ_RET = CHK_STAT('cld-ref_nomix',CLD_INFO%NAME,CLD_INFO%WF_FLG)  

c      WRITE(*,*) '   IWP_TO_EXT =================>  '
c      WRITE(*,*) '  ICE_FORM(NCL,1)%S = ', CRST(NCL)%SHP(1)%ICE_FORM
      
c     ... Volume of ice particles depends on its shape
c     --------------------------------------------------
      SELECT CASE( CRST(NCL)%SHP(1)%ICE_FORM )
      CASE('hex')
c     ------------------------------------------------------------------
         Vol(:) = 1.5D0*DSQRT(3.D0)*R_ice(1:CLN)**2*L_Hex(1:CLN)
c        !!! At this step arrays Ref_Hex and R_ice contain
c            side length of crystal but not effective radius !!!
         CALL LIN_INT(N_Ref_Hex, Ref_Hex, Ext_Hex, 
     $        CLN, R_ice(1:CLN), CSe, 'L') ! [CSe] = mkm^2
c        ... Recalculate cross-section in ext. coef. over iwc [m^2/g]
         Eiwc(:) = CSe(:)/Vol(:)/Ro_ice
c         ... Convert side length to the effective radius         
         ALLOCATE( Ef_Rad(N_Ref_Hex), CS_DB(N_Ref_Hex) )
         CS_DB(:) = Ext_Hex(:)
         Len_hex = 100.d0  ! 100 mkm only is implemented
         Ef_Rad(:) = 3.d0
     $        *1.5D0*DSQRT(3.D0)*Ref_Hex(:)**2*Len_hex ! Volume
     $        /( 3.D0*DSQRT(3.D0)*Ref_Hex(:)**2 + 
     $        6.D0*Len_hex*Ref_Hex(:) )                 ! Surface
c        ... Particles effective radii in DB
         N_cr = N_Ref_Hex ; R_cr => Ef_Rad(:)
c        ... Particles total area for selected vertical profile of
c            side length (R_ice)
         S_sur(:) = 3.D0*DSQRT(3.D0)*R_ice(1:CLN)**2 + 
     $        6.D0*L_Hex(1:CLN)*R_ice(1:CLN)
c        ... Selected vertical profile of effective radius
         A_ef_ice(1:CLN) = 3.D0*Vol(:)/S_sur(:)
c        ... Derivative of the fractal effective radius w.r.t. to the
c            edge length 'a'. It is not implemented yet !!
         IF( I_ref ) GOTO 201
         IF( LOG_LEVEL(1) >= N_ESO ) WRITE(LUNIT,*) 
     $        "   Effective radius of 'Hexagon' [mkm]"

      CASE('fra')
c     ------------------------------------------------------------------
         Cv = DSQRT(2.d0)/12.d0*(1.d0 + 1.d0/2.d0 + 3.d0/16.d0)
         Car = 3.d0*DSQRT(3.d0)/2.d0
         N_cr = N_Ref_Fra 

         IF( RET_FLAG.AND.ITERATION >1.AND.INQ_RET ) THEN
            CALL SET_PTR( RTM_PTR, 'cld-ref_nomix' )
            A_ef_ice(1:CLN) = RTM_PTR%CRN(Jt:Jb) ! update previous eff. rad.
!            write(*,*) '  Retrieved R_eff Iter = ', ITERATION

            II = ITERATION - 1
            CALL SET_PTR( RTM_PTR, 'cld-ref_nomix', ITR = II )
            Scl = RTM_PTR%NXT(Jt+1)/RTM_PTR%CRN(Jt+1)

!            write(*,*) '    Scaling factor = ', Scl
!            write(*,*) '  Retrieved Aef and scaled Aef'
            Aef_fra =  Aef_fra*Scl
            Edg_fra = Edg_fra*Scl
!            WRITE(*,*) A_ef_ice(1), Aef_fra

c           ... New estimation of tetrahedron edge length
            Tel(:) = A_ef_ice(1:CLN)* Car/3.d0/Cv ! [mkm^2]
!            write(*,*) '  Retrieved edge length and scaled Edg'
!            WRITE(*,*) Tel(1), Edg_fra

c          ... New volume
            Vol(:) = Cv*Tel(:)**3
c           ... Approximation of extinction cross-section
            CSe(:) = 1.5d0*Vol(:)/A_ef_ice(1:CLN)  ! ! [mkm^2]
c           ... Recalculate cross-section in ext. coef. over iwc [m^2/g]
            Eiwc(:) = CSe(:)/Vol(:)/Ro_ice
         ELSE
c           ... The volume of fractal of the second generation based on 
c               the regular tetrahedron with the edge length 'a':
c               V = V(a) + 4*V(a/2) + 4*3*V(a/4), where 
c               V(a) = sqrt(2)/12*a^3 is the tetrahedron volume.
            Vol(:) = Cv*R_ice(1:CLN)**3
c           !!! At this step arrays Ref_Fra and R_ice contain 
c               edge length of fractal but not effective radius !!!
            IF( TRIM(HOW_CLD_COEF)=='analytically' ) THEN
               Area(:) = Car*R_ice(1:CLN)**2  ! 
               CSe(:) = Area(:)/2.d0
            ELSE
               CALL LIN_INT(N_Ref_Fra, Ref_Fra, Ext_Fra, 
     $              CLN, R_ice(1:CLN), CSe, 'L') ! [CSe] = mkm^2
c              ... Convert edge length to the effective radius in DB
               ALLOCATE( Ef_Rad(N_Ref_Fra), CS_DB(N_Ref_Fra) )
c              ... Save cross-section at DB edge lengths
               CS_DB(:) = Ext_Fra(:)
c              ... Eff. rad. according to Eq.(Eff_r-3) in STB
               Ef_Rad(:) = 3.d0*Cv*Ref_Fra(:)**3/(Ext_Fra(:)*2.d0)
               R_cr => Ef_Rad(:)
            ENDIF
c           ... Particles total area for selected vertical profile of
c               edge length (R_ice). We assume here that the extinction 
c               efficiency Q_eff = 2, and use theorem 4*APA = Total_area, 
c               APA - average projected area. Additionally
c               the definition Q_eff = ext_cross-section/APA
            S_sur(:) = CSe(:)*2.d0
c           ... Recalculate cross-section in ext. coef. over iwc [m^2/g]
            Eiwc(:) = CSe(:)/Vol(:)/Ro_ice
c           ... Effective radius of fractal particle
            A_ef_ice(1:CLN) = 3.d0*Vol(:)/S_sur(:)
            Edg_fra = R_ice(1) ;  Aef_fra = A_ef_ice(1)
         ENDIF ! RET_FLAG.AND.ITERATION >1.AND.INQ_RET
         IF( LOG_LEVEL(1) >= N_ESO ) WRITE(LUNIT,*) 
     $        '   Effective radius of fractal particle [mkm]'
c       ... Derivative of the fractal effective radius w.r.t. to the
c            edge length 'a'. It is not implemented yet !!
        IF( I_ref ) GOTO 202

       CASE('agg')  ! Baum DB
c     ------------------------------------------------------------------
         N_ref_ice(Ncl) = N_Ref_Bdb 
         IF( .NOT.MLCS(Ncl)%Phm_co ) Ref_ice(Ncl,:) = Ref_Bdb(:) 
         IF( I_ref ) THEN 
            R_ice(1:CLN) = R_ice(1:CLN)*PERTURB(Jt:Jb)
c           ... Linear perturbation of cross-section caused by Ref
            CALL DER_DREF_PERT(N_Ref_Bdb, Ref_Bdb, CLN,
     $           R_ice(1:CLN), Ext_BDB, Jt, Jb, Eiwc)
         ELSE
c           ... Extinction coeff. over iwc at 500 nm for a given
c               profile of effective radius R_ice
            CALL LIN_INT(N_Ref_Bdb, Ref_Bdb, Ext_Bdb, 
     $           CLN, R_ice(1:CLN), Eiwc, 'L') ! [Eiwc] = m^2/g
         ENDIF
         ALLOCATE( CS_DB(N_Ref_Bdb) )
c        ... Vol_DB = Ref_Bdb*TotA_Bdb/3.d0 [mkm^3]
c            [V*Ro] = mkm^3*g/cm^3 = g*1.d-12
         CS_DB(:) = Ext_Bdb(:)*Ref_Bdb(:)*TotA_Bdb(:)/3.d0*Ro_ice ! mkm^2
c        ... Effective radii of crystals in DB
         N_cr = N_Ref_Bdb ; R_cr => Ref_Bdb(:)
c        ... Selected profile of ice particle effective radius
         A_ef_ice(1:CLN) = R_ice(1:CLN) 
c        ... Total area of selected effective radius profile
         CALL LIN_INT(N_Ref_Bdb, Ref_Bdb, TotA_Bdb, 
     $        CLN, R_ice(1:CLN), S_sur, 'L') ! mkm^2
c        ... Averaged volume of ice crystal population [mkm^3]
         Vol(:) = A_ef_ice(1:CLN)*S_sur(:)/3.d0  ! mkm^3
c            [V*Ro] = mkm^3*g/cm^3 = g*1.d-12
         CSe(:) = Eiwc(:)*Vol(:)*Ro_ice ! mkm^2
         IF( LOG_LEVEL(1) >= N_ESO ) WRITE(LUNIT,*) 
     $        "   Effective radius of 'AggregateSolidColumns' [mkm]"

      CASE('col','dro','hoc','hbr','pla','p10','p5e','sbr','soc')
c     ... In 3-d the average projected area of a convex solid is 
c         1/4 the surface area, as Cauchy showed in the 19th century. 
c     ------------------------------------------------------------------
         N_ref_ice(Ncl) = MLCS(Ncl)%Na_Ydb
         IF( .NOT.MLCS(Ncl)%Phm_co ) THEN
            IF( Use_Dim ) THEN
               Ref_ice(Ncl,:) = Dim_Ydb(Ncl,:)
            ELSE
               Ref_ice(Ncl,:) = Ref_Ydb(Ncl,:)
            ENDIF
         ENDIF

c         WRITE(*,*) ' ******   gt_cloud_parameter_setup  ******'
c         WRITE(*,*) '     Ncl = ', Ncl
c         WRITE(*,*) ' MLCS(Ncl)%Na_Ydb ', MLCS(Ncl)%Na_Ydb
c         pause
         
         N_Ref_Ydb = MLCS(Ncl)%Na_Ydb
c        ... Particles effective radii in DB for cloud layer Ncl
         N_cr = N_Ref_Ydb
         IF( Use_Dim ) THEN
            R_cr => Dim_Ydb(Ncl,1:N_cr)
         ELSE
            R_cr => Ref_Ydb(Ncl,1:N_cr)
         ENDIF
         D_Po => Dim_Ydb(Ncl,1:N_cr)
c         WRITE(*,*) '  Dim_Ydb  and Ref_Ydb from DB'
c         DO I = 1, N_cr
c            WRITE(*,'(2x,I3,*(2x,E12.4))')
c     $           I, Dim_Ydb(Ncl,I), Ref_Ydb(Ncl,I)
c         ENDDO
c         pause
         
         ALLOCATE( CS_DB(N_cr) )  ;  CS_DB(:) = Ext_Ydb(Ncl,1:N_cr)

         IF( RET_FLAG.AND.ITERATION >1.AND.INQ_RET ) THEN
            CALL SET_PTR( RTM_PTR, 'cld-ref_nomix' )
            A_ef_ice(1:CLN) = RTM_PTR%CRN(Jt:Jb) ! update previous eff. rad.
            write(*,*) '  Retrieved R_eff Iter = ', ITERATION
            WRITE(*,'(A,*(2x,e12.4))') '  Min, max From DB    ',
     $           MINVAL(R_cr), MAXVAL(R_cr)
            WRITE(*,'(A,*(2x,e12.4))') '  Min max After Retr  ',
     $           MINVAL(A_ef_ice(1:CLN)), MAXVAL(A_ef_ice(1:CLN))
c            PAUSE
            CALL LIN_INT(N_cr, R_cr, Ext_Ydb(Ncl,1:N_cr), 
     $           CLN, A_ef_ice(1:CLN), CSe, 'L') ! [CSe] = mkm^2
c           ... Retrieved volume profile of ice particle
            CALL LIN_INT(N_cr, R_cr, Vol_Ydb(Ncl,1:N_cr), 
     $           CLN, A_ef_ice(1:CLN), Vol(1:CLN), 'L') 
c           ... Recalculate cross-section in ext. coef. over iwc [m^2/g]
            Eiwc(:) = CSe(:)/Vol(:)/Ro_ice            
         ELSE
            IF( I_ref ) THEN 
               R_ice(1:CLN) = R_ice(1:CLN)*PERTURB(Jt:Jb)
               CALL DER_DREF_PERT(N_cr, D_Po, CLN, R_ice(1:CLN),
     $              Ext_Ydb(Ncl,1:N_cr), Jt, Jb, CSe)
               CALL DER_DREF_PERT(N_cr, D_Po, CLN, R_ice(1:CLN), 
     $              R_cr, Jt, Jb, A_ef_ice(1:CLN) )
               CALL DER_DREF_PERT(N_cr, D_Po, CLN, R_ice(1:CLN),
     $              Vol_Ydb(Ncl,1:N_cr), Jt, Jb, Vol(1:CLN) )
               Eiwc(:) = CSe(:)/Vol(:)/Ro_ice
            ELSE
c               WRITE(*,*) '           --- 5 '
c               ... D_Po is particles dimension (not effective radius)
c                   CSe [mkm^2] - extinction cross-section at reference WL 
c                   interpolated on selected particles dimension within cloud
               !WRITE(*,*) '  *******  N_cr = ', N_cr
c               WRITE(*,*) '   D_Po and Ext_Ydb '
c               DO I = 1, N_cr
c                  WRITE(*,'(2x,I3,*(2x,E12.4))')
c     $                 I, D_Po(I), Ext_Ydb(Ncl,I)
c               ENDDO
               CALL LIN_INT(N_cr, D_Po, Ext_Ydb(Ncl,1:N_cr), 
     $              CLN, R_ice(1:CLN), CSe, 'L') 
c               WRITE(*,*) '           --- 6 '
c               WRITE(*,*) '   D_Po and R_cr from DB'
c               DO I = 1, N_cr
c                  WRITE(*,'(2x,I3,*(2x,E12.4))')
c     $                 I, D_Po(I), R_cr(I)
c               ENDDO
c              ... Selected profile of R_eff/D_max of ice particle
               IF( Use_Dim ) THEN
                  A_ef_ice(1:CLN) = R_ice(1:CLN)
                  !WRITE(*,*) '  R_ice =====> '
                  !WRITE(*,*) R_ice(1:CLN) 
               ELSE
                  CALL LIN_INT(N_cr, D_Po, R_cr, 
     $                 CLN, R_ice(1:CLN), A_ef_ice(1:CLN), 'L')
               ENDIF
c              ... Selected volume profile of ice particle
               CALL LIN_INT(N_cr, D_Po, Vol_Ydb(Ncl,1:N_cr), 
     $               CLN, R_ice(1:CLN), Vol(1:CLN), 'L') 
c              ... Recalculate cross-section in ext. coef. over iwc [m^2/g]
               Eiwc(:) = CSe(:)/Vol(:)/Ro_ice
c               WRITE(*,*) '   Inp. dimension, A_ef_ice, and Vol'
c               DO I = 1, CLN
c                  WRITE(*,'(2x,I3,*(2x,E12.4))')
c     $                 I, R_ice(I), A_ef_ice(I), Vol(I)
c               ENDDO
c               WRITE(*,*) '  ----------------------------------- '
            ENDIF
         ENDIF
      END SELECT  ! over ice crystal shape

      IF( RET_FLAG.AND.ITERATION==1.AND.INQ_RET ) THEN
c     ... Start of WF calculation or retrieval process.
         Pr_inp = 0.d0  ;  Pr_inp(Jt:Jb) = A_ef_ice(1:CLN) ! Dimension (R_ice)
         CALL SET_PTR( RTM_PTR, 'cld-ref_nomix' )
         RTM_PTR%APR(:) = Pr_inp(:)
         RTM_PTR%CRN(:) = Pr_inp(:)
         write(*,*) '  A priori R_eff(1:3) Iter = ', ITERATION
         WRITE(*,*) Pr_inp(Jt:Jt+2) !A_ef_ice(1:3)
      ENDIF
      
      IF( LOG_LEVEL(1) >= N_ESO ) WRITE(LUNIT,'(1x,*(1x,F7.3))')
     $     A_ef_ice(1:CLN) 

c     ... Transform wavelength [nm] in [mkm]
      WL(:) = CLD_NETWAVS(:)*1.0d-03

c     ... IWC/N(z) profiles according to selected integral parameter
c     ==============================================================
c      WRITE(*,*) '   TRIM(SCALE_PROF) ', TRIM(SCALE_PROF)
      SELECT CASE ( VT(NCL,INX)%S )
      CASE('TAU')
c     ------------------------------------------------------------------
         SELECT CASE( TRIM(SCALE_PROF) )
         CASE('Scale-NC')
c          ... Array Sh_ice is considered as N(z) [1/cm^3], [CSe] = mkm^2
            Ex_cof(:) = CSe(:)*Sh_ice(1:CLN) *1.d-3 ! [1/km]
c            [V*Ro] = mkm^3*g/cm^3 = g*1.d-12
c            [N(z)] = 1/cm^3 = 1./1.e-4*m^2/1.e-5*km = 1.e+9/m^2/km
            TM1(:) = Vol(:)*Ro_ice*1.d-3
c           ... 1/cm^3 = 1/1.d+8*mkm^2/1.d-5*km = 1.d-3*1/mkm^2/km
            TM2(:) = 1.d-3 
         CASE('Scale-WC')
c        ...  Sh_ice is considered here as [IWC] = g/m^3 = 1.d3*g/m^2/km
c             [Eiwc] = m^2/g for all crystals form
            Ex_cof(:) = Eiwc(:)*Sh_ice(1:CLN)*1.d+3 ! [1/km]
            TM1(:) = 1.d+3  ! g/m^3 = 1.d3*g/m^2/km
c           ... [IWC/(V*Ro)] = g/(m^3*1e-12*g) = 1/(m^2*1e-12*1.e-3*km)
c               = 1.e+3*1/mkm^2/km)
            TM2(:) = 1.d0/Vol(:)/Ro_ice*1.d+3
         END SELECT
c        ... COT at reference wavelength for a shape-profile
         D_D = SUM( ( z_act(1:CLN-1) - z_act(2:CLN) )*
     $        (Ex_cof(1:CLN-1) + Ex_cof(2:CLN) ) )/2.d0
     $        + (Ex_cof(1) + Ex_cof(CLN))/2.d0*Lim
c         WRITE(*,'(A,*(2x,e12.4))') '  --- D_D           ', D_D
c        ... Scaling factor for extinction coefficient
         C_ec = TAU_CLD_INP(NCL,INX)/D_D
c-------------------------------------------------------------------   
         Prof_Ref(Jt:Jb,N_1) = Ex_cof(:)/D_D
c-------------------------------------------------------------------   
c        ... Scaling IWC [g/m^2/km] and N(z) profiles
         Iwc_a(:)  = C_ec*Sh_ice(1:CLN)*TM1(:) ! [g/m^2/km]
c         N_part(:) = C_ec*Sh_wat(1:CLN)*TM2(:) ! [1/mkm^2/km] corrected 05.28.19
         N_part(:) = C_ec*Sh_ice(1:CLN)*TM2(:) ! [1/mkm^2/km]
c         WRITE(*,'(A,*(2x,e12.4))') '  --- C_ec           ', C_ec
c         WRITE(*,'(A,*(2x,e12.4))') '  --- Sh_ice(1:3)    ', Sh_ice(1:3)
c         WRITE(*,'(A,*(2x,e12.4))') '  --- TM2(1:3)       ', TM2(1:3)
c         WRITE(*,'(A,*(2x,e12.4))') '  --- N_part(1:3)    ', N_part(1:3)
c         WRITE(*,'(A,*(2x,e12.4))') ' N_part ', N_part(1:3)
      CASE('IWC')
c     ------------------------------------------------------------------
c         ... IWC profile is used in absolute units as input, 
c              i.e., [Sh_ice] = g/m^3
         C_iwp = 1.d03                      ! 1/m --> 1/km
         Iwc_a(:) = C_iwp*Sh_ice(1:CLN)     ! g/m^2/km
         N_part(:) = Iwc_a(:)/Vol(:)/Ro_ice ! 1/mkm^2/km
      CASE('IWP')
c     ------------------------------------------------------------------
c        ... Scaling of the shape-profile to match given IWP [g/m^2]
c            Norm_ice=[(g/m^3)*km] integral of IWC shape-profile
         C_iwp = IWP(NCL)/Norm_ice           ! [m/km]
         Iwc_a(:)  = C_iwp*Sh_ice(1:CLN)     ! [IWC(:)] = g/m^2/km
         N_part(:) = Iwc_a(:)/Vol(:)/Ro_ice  ! 1/mkm^2/km
      CASE('CNC')
c     ------------------------------------------------------------------
c       ... Calculation of the ice water content profile corresponding 
c           to the given profile of number concentration of crystals,
c           [Sh_ice(I)]=1/cm^3, and the form of crystal.
c       ------------------------------------------------------------------
c          ... [Iwc_a] = (1/cm^3)*micron^3*(g/cm^3) = g/m^2/km*1.d-3
         Iwc_a(:)  = 1.d-03*Sh_ice(1:CLN)*Vol(:)*Ro_ice
         N_part(:) = 1.d-03*Sh_ice(1:CLN) ! 1/mkm^2/km
      END SELECT

c     ... Number of crystals per unit volume [1/mkm^2/km]
      C_conc(1:CLN) = N_part(:)

      IF( Use_Bdb ) C_conc(1:CLN) = Iwc_a(1:CLN) ! [g/m^2/km]

c     ... Save all extinction and scattering cross-section from available 
c         ice database (needed for calculations derivatives)
c         because array CLD_LAYER_COEF will be recalculated for 
c         actual profile of effective radius
      IF( WF_FLAG ) CLD_LAYER_DERIV(NCL,:,N_1:N_2,1:N_cr) 
     $     = CLD_LAYER_COEFF(NCL,:,N_1:N_2,1:N_cr)
c     ==================================================================

c     ... Ice water path [g/m^2] corresponding to given IWC
      IWP_loc = SUM( ( z_act(1:CLN-1) - z_act(2:CLN) )*
     $     (Iwc_a(1:CLN-1) + Iwc_a(2:CLN) ) )/2.d0
     $     + (Iwc_a(1) + Iwc_a(CLN))/2.d0*Lim 

c     ... COT corresponding to given ice water content
      TM1(:) = Eiwc(:)*Iwc_a(:)
      Tau_loc = SUM( ( z_act(1:CLN-1) - z_act(2:CLN) )*
     $     (TM1(1:CLN-1) + TM1(2:CLN)  ) )/2.d0
     $     + (TM1(1)  + TM1(CLN) )/2.d0*Lim
      
c          WRITE(LUNIT,*) '  ----> IWP and TAU for mode: ',VT(NCL,INX)%S
c          WRITE(LUNIT,'(2x,*(2x,es15.7))') IWP_loc, Tau_loc
c          WRITE(LUNIT,*) '  TRIM(HOW_CLD_COEF) ', TRIM(HOW_CLD_COEF)
          
c          IF( Use_Dim ) THEN
c             WRITE(*,'(7x,A)') '   Z[cm]'//
c     $    '  D_max,    N_part [1/cm^3],   Iwc_a [g/m^3] (at layer top)'
c          ELSE
c             WRITE(*,'(7x,A)')'   Z[cm]'//
c     $     '  R_eff,    N_part [1/cm^3],   Iwc_a [g/m^3] (at layer top)'
c          ENDIF
cc         ... Altitude in [cm]
c          DO I = 1, CLN
c             WRITE(*,'(2x,*(2x,es12.4))') z_act(I)*1.E5, A_ef_ice(I),
c     $            N_part(I)*1.0d3, Iwc_a(I)*1.0d-3
c          ENDDO
c     
c     ----- Calculation or interpolation optical parameters -----------
c     ==================================================================
      SELECT CASE( TRIM(HOW_CLD_COEF) )
      CASE('analytically')
c     ------------------------------------------------------------------
c        ... Calculation of ice extinction coefficient [1/km]
c            and single-scattering albedo 
         ome_ice(:,:) = 1.D0
         DO I = 1, CLN
            Ext_p => CLD_LAYER_COEFF( NCL,1:NCW,N_1,I+1 ) 
            Scc_p => CLD_LAYER_COEFF( NCL,1:NCW,N_2,I+1 ) 
            Ext_p(:) = 1.5d0*Iwc_a(I)/A_ef_ice(I)/Ro_ice
            WHERE( WL(:) > 0.3d0 )
               Pd(:) = 8.d0*Pi*A_ef_ice(I)/WL(:)*REF_IMAG_I(:)
               ome_ice(:,I) = 1.d0 - 0.47d0*(1.d0 - DEXP(-1.8d0*Pd(:)))
            END WHERE
            Scc_p(:) = ome_ice(:,I)*Ext_p( : )

            IF( I==1.OR.I==2 ) THEN
               WRITE(*,*) '  Analytic ext., scat., abs '
               WRITE(*,'(2x,*(2x,es12.4))') A_ef_ice(I),
     $              Ext_p(1), Scc_p(1), Ext_p(1)*(1.d0-ome_ice(1,I))
            ENDIF

         ENDDO                  ! over levels within cloud (I)
c         WRITE(*,*) '===>> Ome ice : ', ome_ice(:,1)
         IF( LOG_LEVEL(1) >= N_ESO ) THEN                       !
            Ext_p => CLD_LAYER_COEFF( NCL,2,N_1,2:CLN+1 )       !
            Tau_ice = SUM( ( z_act(1:CLN-1) - z_act(2:CLN) )*   !
     $           (Ext_p(1:CLN-1) + Ext_p(2:CLN) ) )/2.d0        !
     $           + (Ext_p(1) + Ext_p(CLN))/2.d0*Lim             !
            WRITE(*,*) '===>> Tau_ice from analytic: ', Tau_ice !
         ENDIF

      CASE('from data base')
c     ------------------------------------------------------------------
c     ... ice extinction coefficient [1/km] and single-scattering
c         albedo from database
         IF( PHASE_NAME(Ncl)%S == 'user') THEN
c             ... User-defined cross-section should be [mkm^2]
            DO J = 1,NCW
               CLD_LAYER_COEFF(NCL,J,N_1,2:CLN+1) = 
     $              CLD_LAYER_COEFF(NCL,J,N_1,1)*N_part(1:CLN)
               CLD_LAYER_COEFF(NCL,J,N_2,2:CLN+1) = 
     $              CLD_LAYER_COEFF(NCL,J,N_2,1)*N_part(1:CLN)
            ENDDO
         ELSE
            DO J = 1,NCW
c              ... Select extinction coefficient:
               Sec_p => CLD_LAYER_COEFF(NCL,J,N_1,1:N_cr)
               IF( I_ref ) THEN
c                 ... Linear perturbation of bulk extinction caused by Ref
                  CALL DER_DREF_PERT(N_cr, R_cr, CLN,
     $                 A_ef_ice, Sec_p, Jt, Jb, Sec_int)
               ELSE
c                 ... Interpolate extinction cross-section [micron^2]  
c                     or bulk extinction [m^2/g] for Baum DB
c                     on the given values of effective radius
                  CALL LIN_INT(N_cr, R_cr, Sec_p, 
     $                 CLN, A_ef_ice(1:CLN), Sec_int, 'L')
               ENDIF
c                C_conc - [g/m^2/km]   for Baum DB
c                       - [1/mkm^2/km] for other
               CLD_LAYER_COEFF(NCL,J,N_1,2:CLN+1) = 
     $              Sec_int(1:CLN)*C_conc(1:CLN) !  1/km

c              ... Select scattering coefficient:
               Sec_p => CLD_LAYER_COEFF(NCL,J,N_2,1:N_cr)
               IF( I_ref ) THEN
c                 ... Linear perturbation of bulk scattering caused by Ref
                  CALL DER_DREF_PERT(N_cr, R_cr, CLN,
     $                 A_ef_ice, Sec_p, Jt, Jb, Sec_int)
               ELSE
c                 ... Interpolate scattering cross-section [micron^2]  
c                     or bulk scattering  [m^2/g] for Baum DB
c                     on the given values of effective radius
                  CALL LIN_INT(N_cr, R_cr, Sec_p, CLN,
     $                 A_ef_ice(1:CLN), Sec_int, 'L')
               ENDIF
               CLD_LAYER_COEFF(NCL,J,N_2,2:CLN+1) = 
     $              Sec_int(1:CLN)*C_conc(1:CLN) !  1/km

            ENDDO  ! over wavelengths (J)
         ENDIF
c     ---- end case 'from data base' ---
c         WRITE(*,*) '     Ncl, N_2 ', Ncl, N_2
c         WRITE(*,*) '  CLD_LAYER_COEFF(NCL,1,N_2,2:3) ',
c     $        CLD_LAYER_COEFF(NCL,1,N_2,2:3)
c         WRITE(*,*) '  CLD_LAYER_COEFF(NCL,2,N_2,2:3) ',
c     $        CLD_LAYER_COEFF(NCL,2,N_2,2:3)
      END SELECT

c     ... Extinction and scattering coefficients of a cloud are set 
c         to zero at the top and bottom of infinitesimal layers
      CLD_LAYER_COEFF( NCL,1:NCW,N_1:N_2,1 )     = 0.d0 
      CLD_LAYER_COEFF( NCL,1:NCW,N_1:N_2,CLN+2 ) = 0.d0 

c       ... Optical thickness for given IWC at 500 nm
      Sec_int(:) = Eiwc(:)*Iwc_a(:)
      Tau_ice = SUM( ( z_act(1:CLN-1) - z_act(2:CLN) )*
     $     (Sec_int(1:CLN-1) + Sec_int(2:CLN) ) )/2.d0
     $     + (Sec_int(1) + Sec_int(CLN))/2.d0*Lim
c      WRITE(*,*) '===>> Tau_ice at 500 nm: ', Tau_ice

c       Save R_ef(z) for the actual altitude grid
c     ------------------------------------------
c      WRITE(*,*) '   A_ef_ice  ',  A_ef_ice(1:CLN)
      Prof_Cpar(Jt:Jb,N_2) = A_ef_ice(1:CLN) ! ___> Dimension
        
      SELECT CASE ( VT(NCL,INX)%S )
      CASE( 'TAU' )
c     ------------------------------------------------------------------
         IWP(NCL) = IWP_loc
c        ... Save IWC(z) for an actual altitude grid [g/m^3] 
         Prof_Cpar(Jt:Jb,N_1) = Iwc_a(1:CLN)/1000.0d0 
         SELECT CASE( SCALE_PROF )
         CASE('Scale-NC') ; P_term =  0.d0 ; Pa_03 = 3.d0
         CASE('Scale-WC') ; P_term = -3.d0 ; Pa_03 = 0.d0
         END SELECT
      CASE( 'CNC' )
c     ------------------------------------------------------------------
         Tau_CLD_inp(NCL,INX) = Tau_ice ; P_term = -3.d0 ; Pa_03 = 3.d0
c          ... Save N(z) [1/cm^3] for the actual altitude grid
         IWP(NCL) = IWP_loc  ;  Prof_Cpar(Jt:Jb,N_1) = Sh_ice(1:CLN)
      CASE( 'IWP' )
c     ------------------------------------------------------------------
         Tau_CLD_inp(NCL,INX) = Tau_ice ; P_term = -3.d0 ; Pa_03 = 0.d0
c        ... Save IWC(z) for the actual altitude grid [g/m^3]
         Prof_Cpar(Jt:Jb,N_1) = Iwc_a(1:CLN)/1000.0d0
      CASE( 'IWC' )
c     ------------------------------------------------------------------
         Tau_CLD_inp(NCL,INX) = Tau_ice ; P_term = -3.d0 ; Pa_03 = 0.d0
         IWP(NCL) = SUM( ( z_act(1:CLN-1) - z_act(2:CLN) )*
     $        ( Sh_ice(1:CLN-1) + Sh_ice(2:CLN) ) )/2.d0*1000.d0
     $        + (Sh_ice(1) + Sh_ice(CLN))/2.d0*Lim*1000.d0
c        ... Save IWC(z) for the actual altitude grid [g/m^3]
         Prof_Cpar(Jt:Jb,N_1) = Sh_ice(1:CLN) 
      END SELECT
c
c ***************  Derivative for ice particles *****************
c      IF( WF_FLAG .OR. NUM_CLD ) THEN
      IF( WF_FLAG ) THEN
         TCI = Tau_CLD_inp(NCL,INX)
         SELECT CASE( TRIM(HOW_CLD_COEF) )
         CASE('from data base')
c        ------ Derivatives 'numerically' for ice --------------------
            CALL DCOF_DREF(N_cr, R_cr, CS_DB, A_ef_ice(1:CLN), N_1, N_2,
     $           N_part, CSe, P_term)
         CASE('analytically')
c        ------ Derivatives 'analytically' for ice --------------------
c        ... This option is allowed for 'Fractal' and 'Hexagon' particles
            ome_ice(:,:) = 1.D0
            DO I = 1, CLN 
               Ext_p => CLD_LAYER_DERIV( NCL,1:NCW,N_1,I+1 )
               Scc_p => CLD_LAYER_DERIV( NCL,1:NCW,N_2,I+1 )
               WHERE( WL(:) > 0.3d0 )
                  Pd(:) = 8.d0*Pi*A_ef_ice(I)/WL(:)*REF_IMAG_I(:)
                  ome_ice(:,I) = 1.d0 - 0.47d0
     $                 *(1.d0 - DEXP(-1.8d0*Pd(:)))  ! ssa
                  LC(:) = 0.47d0*1.8d0*Pd(:)*DEXP(-1.8d0*Pd(:)) ! 
c                 ... Fa w.r.t. R2: R2/Fa*dFa/dR2 (Eq.(dFa/dr2) in STB)
                  Ext_p(:) = (-1.0d0+LC(:)/(1.d0-ome_ice(:,I)) + Pa_03)
c                 ... Fs w.r.t. R2: R2/Fs*dFs/dR2 (Eq.(dFs/dr2) in STB)
                  SCC_P(:) = ( -1.0d0 - LC(:)/ome_ice(:,I) + Pa_03 )
               ELSEWHERE
                  Ext_p(:) =  0.0d0 ; SCC_P(:) = -1.0d0
               END WHERE

            ENDDO  ! over cloud sub-layers (I)
c           ... Analytically derivative of the extinction coefficient
c               at the reference wavelength (0.5 micron) w.r.t. 
c               effective radius, i.e., Dext/Dref *Ref /Tau_c
c           ... See second term in Eq.(De_Dref) in STB
            IF( VT(NCL,INX)%S=='TAU' ) THEN
               SELECT CASE( SCALE_PROF )
               CASE('Scale-NC') ; P12 =  2.d0
               CASE('Scale-WC') ; P12 = -1.d0
               END SELECT
c-------------------------------------------------------------------   
               Prof_Ref(Jt:Jb,N_2) = Ex_cof(:)/TAU_CLD_INP(NCL,INX)*P12
c-------------------------------------------------------------------   
            ENDIF
c        ------ End case derivatives 'analytically' for ice -------------
         END SELECT

         CLD_LAYER_DERIV( NCL,1:NCW,N_1:N_2,1 )     = 0.d0
         CLD_LAYER_DERIV( NCL,1:NCW,N_1:N_2,CLN+2 ) = 0.d0

      ENDIF  ! WF_FLAG .OR. NUM_CLD 

      RETURN

 201  WRITE(*,*) '  Numerical perturbation w.r.t effective radius '//
     $     'is not allowed for hexagonal cloud particles'
      RETURN 1      
 202  WRITE(EU,*) '  Numerical perturbation w.r.t effective radius '//
     $     'is not allowed for fractal cloud particles'
      RETURN 1      
 203  WRITE(EU,*)'  Option "DENS_PROFILE" is not allowed '//
     $     'for Baum ice database'
      RETURN 1      

      END SUBROUTINE IWP_TO_EXT

c *********************************************************************
      SUBROUTINE DER_DREF_PERT(N_Ref, Ref, CLN, Rper, Sec_p, Jt, Jb, 
     $     Sec_i)

      USE WF_NUM,  ONLY: PERTURB
c     ------------------------------------------------------------------
      INTEGER, INTENT(IN)::  N_Ref, CLN, Jt, Jb
      REAL (KND), INTENT(IN):: Ref(N_Ref), Rper(CLN), Sec_p(N_Ref)
      REAL (KND)  Sec_i(CLN)
c     ... Local variables
      INTEGER I, J, K, KK, Ws, Ws1
      REAL (KND) DD1, Dco_Dr(N_ref), D_int(CLN), Rbas(CLN)

      Rbas(:) = Rper(:)/PERTURB(Jt:Jb)
      Dco_Dr(1) = (Sec_p(2) - Sec_p(1))/(Ref(2) - Ref(1))
      DO K = 2,N_Ref-1
         Dco_Dr(K) = (Sec_p(K+1) - Sec_p(K))/(Ref(K+1) - Ref(K))
      ENDDO
      Dco_Dr(N_Ref) = Dco_Dr(N_Ref-1)
      DO KK = 1,CLN
         CALL LIN_SEARCH_NEW(Rper(KK), DD1, Ws, Ws1, N_Ref, Ref)
         D_int(KK) = (1.d0 - DD1)*Dco_Dr(Ws) + DD1*Dco_Dr(Ws1)
      ENDDO
c     ... Interpolation of non-perturbed cross-section 
      CALL LIN_INT(N_Ref, Ref, Sec_p, CLN, Rbas, Sec_i, 'L')
      Sec_i(:) = Sec_i(:) + D_int(:)*(Rper(:) - Rbas(:))

      RETURN
      END SUBROUTINE DER_DREF_PERT

c *********************************************************************
c     Numerical derivatives of absorption and scattering coefficient 
c   with respect to effective radius of water droplets or ice crystals
c **********************************************************************
      SUBROUTINE DCOF_DREF(N_Ref, Ref, Edb, A_in, N1, N2, 
     $     N_Part, Scr, P_term)

      INTEGER,   INTENT(IN):: N_Ref, N1, N2
      REAL(KND), INTENT(IN):: Ref(N_Ref), Edb(N_Ref), A_in(CLN), P_term,
     $     N_Part(CLN), Scr(CLN)
c     ------------------------------------------------------------------
      INTEGER  J, KK, Ws, Ws1
      REAL(KND)  Pco(N_Ref), DDr(N_Ref), DUM
      REAL (KND), DIMENSION(CLN):: Cf_int, DDr_int
c     ------------------------------------------------------------------

      DO J = 1,NCW
c        ---------------------------------------------------------------
c        ... Calculate absorption cross-section
         Pco(:) = CLD_LAYER_DERIV( NCL,J,N1,1:N_Ref ) 
     $          - CLD_LAYER_DERIV( NCL,J,N2,1:N_Ref )
c        ... Derivative of absorption cross-section w.r.t. R_eff
c            for all values of effective radius in database
c         WRITE(*,*) '   Number of R_eff ', N_Ref
         
         DDr(1) = (Pco(2) - Pco(1))/(Ref(2) - Ref(1))
         DO K = 2,N_Ref-1
            DDr(K) = (Pco(K+1) - Pco(K))/(Ref(K+1) - Ref(K))
         ENDDO
         DDr(N_Ref) = DDr(N_Ref-1)
c        ... Interpolate derivative on the input effective radius
         DO KK = 1,CLN
            CALL LIN_SEARCH_NEW(A_in(KK), DUM, Ws, Ws1, N_Ref, Ref)
            DDr_int(KK) = (1.d0 - Dum)*DDr(Ws) + Dum*DDr(Ws1)
         ENDDO
c         ... Interpolate absorption cross-section on the actual R_ef profile
         CALL LIN_INT(N_Ref, Ref, Pco, CLN, A_in, Cf_int, 'L')
c        ... Save derivative R_ef/Abs*DAbs/DR_ef - 3
         WHERE( Cf_int(:) > 0.d0 )
            CLD_LAYER_DERIV(NCL,J,N1,2:CLN+1) = 
     $           A_in(:)/Cf_int(:) * DDr_int(:) + P_term
         ELSEWHERE 
            CLD_LAYER_DERIV(NCL,J,N1,2:CLN+1) = 0.d0 
         END WHERE

c        ---------------------------------------------------------------
c        ... Extract scattering cross-section
         Pco(:) = CLD_LAYER_DERIV(NCL,J,N2,1:N_Ref)
c        ... Derivative of scattering cross-section  w.r.t. R_eff
c            for all values of effective radius in database
         DDr(1) = (Pco(2) - Pco(1))/(Ref(2) - Ref(1))
         DO K = 2,N_Ref-1
            DDr(K) = (Pco(K+1) - Pco(K))/(Ref(K+1) - Ref(K))
         ENDDO
         DDr(N_Ref) = DDr(N_Ref-1)
         DO KK = 1,CLN
            CALL LIN_SEARCH_NEW(A_in(KK), DUM, Ws, Ws1, N_Ref, Ref)
            DDr_int(KK) = (1.d0 - Dum)*DDr(Ws) + Dum*DDr(Ws1)
         ENDDO
c        ... Interpolate scattering cross-section on actual R_ef
         CALL LIN_INT(N_Ref, Ref, Pco, CLN, A_in, Cf_int, 'L')
c        ... Save derivative R_ef/Sca*DSca/DR_ef. Derivative calculated
c            for the case of constant scattering matrix within cloud
         CLD_LAYER_DERIV(NCL,J,N2,2:CLN+1) = 
     $        A_in(1:CLN)/Cf_int(:) * DDr_int(:) + P_term
      ENDDO  ! over WL in cloud DB (J)

c     ... The following will be used to calculate WF for effective 
c         radius in the case of constant optical thickness at 
c         the reference wavelength and cross-section from database.
      IF( VT(NCL,INX)%S=='TAU' ) THEN
c        ... Derivative of DB extinction coefficient w.r.t. R_eff         
         DDr(1) = (Edb(2) - Edb(1))/(Ref(2) - Ref(1))
         DO K = 2,N_Ref-1
            DDr(K) = (Edb(K+1) - Edb(K))/(Ref(K+1) - Ref(K))
         ENDDO
         DDr(N_Ref) = DDr(N_Ref-1)
         WRITE(*,*) '   A_IN(1:5): ', A_in(1:5)
         DO KK = 1,CLN
            CALL LIN_SEARCH_NEW(A_in(KK), DUM, Ws, Ws1, N_Ref, Ref)
            DDr_int(KK) = (1.d0 - Dum)*DDr(Ws) + Dum*DDr(Ws1)
         ENDDO
         WRITE(*,*) '   TCI =  ', TCI
c     ... See second term in Eq.(WFref-tc-N) of STB - r_e/tau_0 De/Dr_e
         Prof_Ref(Jt:Jb,N2) = DDr_int(:)*N_Part(:)*A_in(:)/TCI 
c        ... See second term in Eq.(WFref-tc-L) of STB
c            P_term = -3   if 'Scale-WC' 
c            P_term =  0   if 'Scale-NC' 
         Prof_Ref(Jt:Jb,N2) = Prof_Ref(Jt:Jb,N2)
     $        + P_term*Scr(:)*N_Part(:)/TCI 
      ENDIF

      RETURN
      END SUBROUTINE DCOF_DREF
c **********************************************************************

      SUBROUTINE DERIV_NUMERIC (*)
      INTEGER     N_STEP
      REAL (KND)  DDL, H_STEP, Scal, Z_pert(NLAYERS)

        DDL = 0.01
c       New height-grid for shifted cloud bottom
c       ----------------------------------------
        Z_pert(1) = ZZZ(Jt-1) ! Set top of cloud + LIM
        Z_pert(2) = ZZZ(Jt)
        N_STEP  = CLD_LAYER_NUM(NCL)
        H_STEP  = (ZZZ(Jt) - ZZZ(Jb) - DDL)/DBLE(N_STEP)
        DO I = 3,N_STEP+2
           Z_pert(I) = Z_pert(I-1) - H_STEP 
        ENDDO
        Z_pert(N_STEP+3) = Z_pert(N_STEP+2) - LIM
        Z_inp(1:INP_LYR) = TOP - 
     $       CLDPROFILE(NCL,1:INP_LYR,1)*( TOP - BASE - DDL) 

        Z_pert(1:CLN) = Z_pert(2:N_STEP+2)

        Z_pert(CLN) = Z_pert(CLN) + lim
        CALL AL_INTERP ( INP_LYR, Z_inp, Pr_inp, 
     $       CLN, Z_pert, PROF_VARS, .TRUE., *300 ) 
        Z_pert(CLN) = Z_pert(CLN) - lim

        PROF_VARS(1:CLN) = CLA(JT:JB,1)*PROF_VARS(1:CLN)        
        Scal = SUM( ( Z_pert(1:CLN-1) - Z_pert(2:CLN) )*
     $       ( PROF_VARS(1:CLN-1) + PROF_VARS(2:CLN) ) )/2.

c       Effective const. ext. coeff. for perturbed cloud bottom
c       -------------------------------------------------------
        Scal = Tau_Cld_Inp( ncl,1 )/Scal
        
c       Numerical derivative of vert. prof. of extinction coeff.
c       w.r.t. altitude of cloud bottom
c       -------------------------------------------------------
c        WRITE(*,*) '  Scal    = ', Scal
c        WRITE(*,*) '  Eff_Ext = ', Eff_Ext
c        K = J_TOP(1)+1
c        DO I = 1,CLN
c           WRITE(*,'(2x,I2,2(2x,e15.7),2(3x,e12.4))')
c     $          I, ZZZ(K), PROF_VARS(I), Sh_wat(I), CLA(K,1)
c           K = K + 1
c        ENDDO
        PROF_VARS(2:CLN+1) = 
     $       (PROF_VARS(1:CLN)*Scal - Sh_wat(1:CLN)*Eff_Ext)/ddl
        PROF_VARS(1)       = 0.0d0
        PROF_VARS(CLN+2) = 0.0d0
c        WRITE(*,*) ' ------------------------- '
c        K = J_TOP(1)
c        DO I = 1,CLN+2
c           WRITE(*,'(2x,I2,2(2x,e15.7),2(3x,e12.4))')
c     $          I, ZZZ(K), PROF_VARS(I)
c           K = K + 1
c        ENDDO
c        stop
      RETURN

 300  RETURN 1

      END SUBROUTINE DERIV_NUMERIC
c **************************************************************************

c *********************************************************************
c              Test of numerical derivatives from 
c               analytical parameterization (AP)
c    and defined in data base (DB) for scattering and extinction 
c     coefficients w.r.t. effective radius of water droplets 
c **********************************************************************
      SUBROUTINE DERIV_SCA_ABS_WRT_REF

      INTEGER    I, K
      REAL (KND) X_rad, D_eR, Vol
      REAL (KND) Bk, Ck, Sk, Pi, Pok, Gmu, Dco_Dr(N_Ref_db)
      REAL (KND), DIMENSION(1):: Pkr, Wl, Pd, Hf, Ome, Ke, Ka
      REAL (KND), DIMENSION(N_Ref_db):: Sca_db, Ext_db, Abs_db,
     $     S_ext, S_sca, S_abs, S_abs_d, Dsca_Dr_APa, Dabs_Dr_APa, 
     $     Dsca_Dr_APn1, S_ext_d, S_sca_d, Dsca_Dr_APn2, 
     $     Dabs_Dr_APn1, Dabs_Dr_APn2, Dsca_Dr_DBn2, Dabs_Dr_DBn2
      PARAMETER  ( Bk = 1.1d0, Ck = 1.7d-06, Sk = 56.3d0 )
      PARAMETER  ( Pi = 3.1415927d0, Pok = 2.0d0/3.0d0 )
      PARAMETER  ( Gmu = 6.0d0 )
c     ==================================================================

      WL(1) = CLD_NETWAVS(1)*1.0d-03

      WRITE(*,*) 
      WRITE(*,'(a,F5.2,a)') 
     $     '    Test performed For wavelength :', WL(1),' mkm'

c       **************** Analytical approximation *********************
c
c       ... Scattering and absorption coefficients according to 
c           analytical parameterization (AP) for Lwc_a(1)=1 and Ref_db(I)
      DO I = 1, N_Ref_db
         X_rad  = Ref_db(I) ; Pd(1) = 2.d0*Pi*X_rad/WL(1)
         Pkr(1) = 1.d0/Pd(1)**Pok
         Ke(1)  = 1.d0 + Bk*Pkr(1) + Ck*( DEXP(Sk*Pkr(1)) - 1.d0 )
         S_ext(I) = 1.5d0/X_rad*Ke(1)
         Ka(1) = 1.23d0*(1.d0 - 2.6d0*REF_IMAG_W(1)*Pd(1))
     $        *( 1.d0 + 0.34d0*(1.d0 - DEXP(-8.d0*wl(1)/A_ef(I))) )
         S_sca(I) = S_Ext(I) - 4.d0*Pi*REF_IMAG_W(1)/WL(1)*Ka(1)
         S_abs(I) = S_ext(I) - S_sca(I)
      ENDDO
      D_er = 0.5d0
c       ... Scattering and absorption coefficients according to 
c           analytical parameterization (AP) for Lwc_a(1)=1 and 
c           Ref_db(I) + D_er
      DO I = 1, N_Ref_db
         X_rad  = Ref_db(I) + D_er ; Pd(1) = 2.d0*Pi*X_rad/WL(1)
         Pkr(1) = 1.d0/Pd(1)**Pok
         Ke(1)  = 1.d0 + Bk*Pkr(1) + Ck*( DEXP(Sk*Pkr(1)) - 1.d0 )        
         S_ext_d(I) = 1.5d0/X_rad*Ke(1)
         Ka(1) = 1.23d0*(1.d0 - 2.6d0*REF_IMAG_W(1)*Pd(1))
     $        *( 1.d0 + 0.34d0*(1.d0 - DEXP(-8.d0*wl(1)/A_ef(I))) )
         S_sca_d(I) = S_Ext_d(I) - 4.d0*Pi*REF_IMAG_W(1)/WL(1)*Ka(1)
         S_abs_d(I) = S_ext_d(I) - S_sca_d(I)
      ENDDO
c       ... Numerical derivatives (AP) for Delta_Ref = D_er 
      DO I = 1,N_Ref_db
         Dsca_Dr_APn1(I) = (S_sca_d(I) - S_sca(I))/D_er
         Dabs_Dr_APn1(I) = (S_abs_d(I) - S_abs(I))/D_er
      ENDDO

c       ... Numerical derivatives (AP) for Delta_Ref = 2 mkm 
c           corresponding to the step grid in the current DB
      Dco_Dr(1) = (S_sca(2) - S_sca(1))/(Ref_db(2) - Ref_db(1))
      DO K = 2,N_Ref_db-1
         Dco_Dr(K)=(S_sca(K+1)-S_sca(K))/(Ref_db(K+1)-Ref_db(K))
      ENDDO
      Dsca_Dr_APn2(1:N_Ref_db-1) = Dco_Dr(1:N_Ref_db-1)
     $     *Ref_db(1:N_Ref_db-1)/S_sca(1:N_Ref_db-1)
c     ... Absorption coefficient
      Dco_Dr(1) = (S_abs(2) - S_abs(1))/(Ref_db(2) - Ref_db(1))
      DO K = 2,N_Ref_db-1
         Dco_Dr(K)=(S_abs(K+1)-S_abs(K))/(Ref_db(K+1)-Ref_db(K))
      ENDDO
      Dabs_Dr_APn2(1:N_Ref_db-1) = Dco_Dr(1:N_Ref_db-1)
     $     *Ref_db(1:N_Ref_db-1)/S_abs(1:N_Ref_db-1)

c       ... Analytical derivatives of AP w.r.t R_ef
      DO I = 1, N_Ref_db
         Hf(1) = 1.34d0 - 0.34d0*DEXP( -8.d0*wl(1)/Ref_db(I) )
         Pd(1) = 2.d0*Pi*Ref_db(I)/WL(1)
         Ka(1) = 1.23d0*(1.d0 - 2.6d0*REF_IMAG_W(1)*Pd(1))*Hf(1)
c     ... Abs_coef w.r.t. R_ef : R_ef/Abs*D_Abs/D_Ref
         Dabs_Dr_APa(I) = 8.d0*wl(1)/Ref_db(I)*(1.d0 - 1.34d0/Hf(1)) +
     $        1.d0 - 1.23d0*Hf(1)/Ka(1)           
         Pkr(1) = 1.d0/Pd(1)**Pok
         Ke(1)  = 1.d0 + Bk*Pkr(1) + Ck*( DEXP(Sk*Pkr(1))-1.d0 )
         Dsca_Dr_APa(I) = 1.d0 + 
     $        ( Bk + Ck*Sk*DEXP(Sk*Pkr(1)) )*Pkr(1)/Ke(1)*Pok
         ome(1) =1.d0-Ka(1)/Ke(1)/1.5d0*REF_IMAG_W(1)*2.d0*Pd(1)
c     ... Sca_coef w.r.t. R_ef : R_ef/Sca*D_Sca/D_Ref
         Dsca_Dr_APa(I) = -Dsca_Dr_APa(I)/ome(1)
     $        - Dabs_Dr_APa(I)*(1.d0/ome(1) - 1.d0)
      ENDDO
      
c       **************** Numerical derivatives data base (DB) *********

c     ... Extract extinction and scattering cross-section from
c         DB for cloud layer NCL and first actual wavelength
      Ext_db(1:N_Ref_db) = CLD_LAYER_COEFF(NCL,1,1,1:N_Ref_db)
      Sca_db(1:N_Ref_db) = CLD_LAYER_COEFF(NCL,1,2,1:N_Ref_db)
      Abs_db(:) = Ext_db(:) - Sca_db(:)

c     ... Derivative of scattering cross-section  w.r.t. R_eff
      Dsca_Dr_DBn2(1) = (Sca_db(2) - Sca_db(1))/(Ref_db(2) - Ref_db(1))
      DO K = 2,N_Ref_db-1
         Dsca_Dr_DBn2(K)=(Sca_db(K+1)-Sca_db(K))/(Ref_db(K+1)-Ref_db(K))
      ENDDO
c     ... Derivative of absorption cross-section  w.r.t. R_eff
      Dabs_Dr_DBn2(1) = (Abs_db(2) - Abs_db(1))/(Ref_db(2) - Ref_db(1))
      DO K = 2,N_Ref_db-1
         Dabs_Dr_DBn2(K)=(Abs_db(K+1)-Abs_db(K))/(Ref_db(K+1)-Ref_db(K))
      ENDDO
c      WRITE(*,*)  Dabs_Dr_DBn2(:)
      WRITE(*,*) 
      WRITE(*,'(5x,a,5x,a,14x,a)')
     $     'R_ef','Scattering coefficient','Derivatives w.r.t. R_eff'
      WRITE(*,'(17x,a,9x,a,7x,a,7x,a,7x,a,7x,a)')
     $     'DB','Analytic','DB/Num2','AP/Num2','AP/Num1','AP/Ana'
      DO I = 1,N_ref_db-1
         Vol = 4.d0/3.d0*Pi*Ref_db(I)**3
     $        *(Gmu + 2.d0)*(Gmu + 1.d0)/(Gmu + 3.d0)**2
         WRITE(*,'(4x,F5.2,6(2x,e12.4))') 
     $        Ref_db(I), Sca_db(I)/Vol, S_sca(I), 
     $        Dsca_Dr_DBn2(I)*Ref_db(I)/Sca_db(I) - 3.,
     $        Dsca_Dr_APn2(i), 
     $        Dsca_Dr_APn1(I)*Ref_db(I)/S_sca(I),
     $        Dsca_Dr_APa(I)
      ENDDO

      WRITE(*,'(a)') '------------------------------------------------'
      WRITE(*,'(5x,a,5x,a,14x,a)')
     $     'R_ef','Absorption coefficient','Derivatives w.r.t. R_eff'
      WRITE(*,'(17x,a,9x,a,7x,a,7x,a,7x,a,7x,a)')
     $     'DB','Analytic','DB/Num2','AP/Num2','AP/Num1','AP/Ana'
      DO I = 1,N_ref_db-1
         Vol = 4.d0/3.d0*Pi*Ref_db(I)**3
     $        *(Gmu + 2.d0)*(Gmu + 1.d0)/(Gmu + 3.d0)**2
         WRITE(*,'(4x,F5.2,6(2x,e12.4))') 
     $        Ref_db(I), Abs_db(I)/Vol, S_abs(I), 
     $        Dabs_Dr_DBn2(I)*Ref_db(I)/Abs_db(I) - 3.,
     $        Dabs_Dr_APn2(i), 
     $        Dabs_Dr_APn1(I)*Ref_db(I)/S_abs(I),
     $        Dabs_Dr_APa(I)
      ENDDO

      WRITE(*,'(a)') '------------------------------------------------'
      WRITE(*,'(a,F8.4)') 
     $  '  DB/Num2 numerical derivative of data base cross-section :'//
     $  '      R_eff step ', Ref_db(2) - Ref_db(1)
      WRITE(*,'(a,F8.4)') 
     $  '  AP/Num2 numerical derivative of analytical approximation:'//
     $  '      R_eff step ', Ref_db(2) - Ref_db(1)
      WRITE(*,'(a,F8.4)') 
     $  '  AP/Num1 numerical derivative of analytical approximation:'//
     $  '      R_eff step ', D_er
      WRITE(*,'(a,F8.4)') 
     $     '  AP/Ana analytical derivative of analytical approximation:'
      WRITE(*,'(a)') '------------------------------------------------'
      WRITE(*,*) 

      STOP '   Stop in "DERIV_SCA_ABS_WRT_REF" '
c *******************************************************************
      END SUBROUTINE DERIV_SCA_ABS_WRT_REF

c *******************************************************************
      END   ! of the main subroutine "gt_cloud_parameter_setup.f"
C********************************************************************C
      SUBROUTINE OMEGA_SM( Nwl, WL_arr, Ri_r, Ri_i, NCL, CLN, Chord, 
     $     Omeg, Alph, Yel_ac)

      USE CLD_INP,    ONLY: ADD_ABS, MLCS
      USE ST_VARS,    ONLY: PIE
      USE TYPE_VARS,  ONLY: KND
c     --------- Input parameters -------------------------------------
      INTEGER, INTENT(IN):: Nwl, NCL, CLN
      REAL (KND), DIMENSION(Nwl):: WL_arr, Ri_r, Ri_i
      REAL (KND), DIMENSION(CLN):: Chord, Yel_ac, Abs_yel
      REAL (KND)  Omeg(Nwl,CLN), Alph(Nwl,CLN)
c     --------- Local variables --------------------------------------
      INTEGER I, J, K
      REAL(KND), DIMENSION(CLN):: x
      REAL (KND) Rr, U1, D1, T2, T3, Td
c     ================================================================      
c     ... Set to zero in the case of non-absorbing media
      Abs_yel = 0.d0 ! without yellow absorption
      DO I = 1, Nwl
         IF( ADD_ABS(Ncl) ) THEN
            IF( WL_arr(I) <= 500.d0 ) THEN
               Abs_yel(:) = Yel_ac(:)
     $              *DEXP( -1.5d-2*(WL_arr(I) - 390.d0) ) 
            ELSE
               Abs_yel(:) = Yel_ac(:)
     $              *DEXP(-1.5d-2*110.d0 - 1.1d-2*(WL_arr(I)- 500.d0))
            ENDIF
         ENDIF
         Alph(I,:) = (4.d0*Pie*Ri_i(I)/WL_arr(I) + Abs_yel(:))
     $        * Chord(:) *1.d3  ! [micron] in [nm] for mean chord
         Rr = Ri_r(I)           ! real part of refraction index
         U1 = 2.0*(5.0*Rr**6 + 8.0*Rr**5 + 6.0*Rr**4 - 5.0*Rr**3 -Rr-1)
         D1 = 3.0*( Rr**3 +  Rr**2 + Rr + 1.0)*(Rr**4 - 1.0)
         T2 = Rr**2*(Rr**2 - 1.0)**2/( Rr**2 + 1.0)**3
     $        *DLOG( (Rr+1.0)/(Rr-1.0) )
         T3 = 8.0*Rr**4*(Rr**4 + 1.0)/(Rr**4-1.0)**2/(Rr**2 + 1.0)
     $        *DLOG( Rr )
         Td = U1/D1 + T2 - T3
         x(:) = Rr**2* Alph(I,:)
         Omeg(I,:) = 1.0 - x(:)*Td/(x(:) + Td)
      ENDDO  ! over cloud wavelength grid (I)

      RETURN
      END SUBROUTINE OMEGA_SM
C********************************************************************C
