!***************************************************************************
!
!	SCIATRAN - Version 4.6 - Last modification: 15 Feb 2023
!
!	Institute of Remote Sensing, University of Bremen, Germany
!
!***************************************************************************
SUBROUTINE GT_LAYER_SETUP (*)
  ! Called by "rtm_main.f"

  USE AEROSOLS,   ONLY: AER_SHIFT, AER_ADD_LEV, SHARP_EDGES
  USE BRDF,       ONLY: RefW_r, RefW_i, Ref_type
  USE ALT_INPUT,  ONLY: SOUT_FLAG, DIR_SOUT
  USE CLOUDS,     ONLY: N_CLD_LAYERS, DO_CLOUD, J_TOP, J_BOT, CLD_LAYER_NUM, &
                        CLH => CLD_LAYER_HEIGHTS, CLD_LAYER_ARR, EPS_CLD, &
                        Prof_Cpar, Prof_Ref, INF_LAYER, TRANS_LAY, GT_TL
  USE CLD_INP,    ONLY: Plus_ice, SUB_LAYERING
  USE GRIDS,      ONLY: NLAYERS_INP, ZZZ_INP, NLAYERS, ZZZ, DZR, DZ, NDEP_INP,&
                         DEP_INP, GRID_MARK, ALT_MARK, GRID_MARK_UW, DEP_MARK
  USE LOG_PARS,   ONLY: LOG_LEVEL, EU, LUNIT, N_ESO, Work_Dir
  USE RTM_OUTPUT, ONLY: N_OP
  USE ST_VARS,    ONLY: GROUND_HEIGHT, RTM_CORE, DO_AEROSOL, RTM_TYPE, Add_lev, &
                         ARB_OL, RTM_MODE
  USE SPHER_PARS, ONLY: SET_OUT_ALT, ALT_OUT, NOA => N_OBS_ALT
  USE ST_INPUT,   ONLY: DEP_NAME, NET_NAME, AER_PAR_TYPE
  USE TYPES,      ONLY: KND, DYN_CHAR
  USE TOOLS,      ONLY: READ_INPUT
  USE WATER,      ONLY: UWT, OC_DEP, NL_UW => NLAYERS_UW, ZZZ_UW, DZ_UW, &
                        Obs_POSITION, INS_POSI, INS_VIEW
  USE WATER_ICE,  ONLY: IWS, IWT, LA_N_IW
  
  CHARACTER (LEN = *), PARAMETER :: R_NAME = 'GT_LAYER_SETUP'
  LOGICAL, ALLOCATABLE :: MASK(:)
  INTEGER I, J, K,  N_STEP, MAX_LAYERS, NL_LOC, N_S, JC, JE, JS, NC, ADD_LEVEL, N_LEV
  INTEGER, ALLOCATABLE:: Ice_T(:), Ice_B(:)
  CHARACTER (LEN=:), ALLOCATABLE :: STR_INP
  REAL (KND)  LIM, C_BOT, C_TOP, Z_STEP, CLAYER, DIFF
  REAL (KND), PARAMETER :: DELTA = 1.0D-4
  REAL*4, ALLOCATABLE :: WWW(:)
  REAL (KND), ALLOCATABLE :: HEIGHTS(:), NEWHEIGHT(:), TMP(:), NEW_ARR(:,:), &
       XXX(:), XIN(:), DZIN(:)
  TYPE (DYN_CHAR), ALLOCATABLE :: READ_ARR(:)
  !==================================================================
      
  CALL ST_TRACK (R_NAME, 1, 'I')
  CALL DEALLOCATE_ARRAYS ! All arrays allocated below must be 
                         ! conditionally deallocated here
  ! 0. Initializations
  ! ==================
  IF ( DO_CLOUD ) THEN
     MAX_LAYERS = SUM(CLD_LAYER_NUM(1:N_CLD_LAYERS)) &
          + 3*N_CLD_LAYERS + NLAYERS_INP
  ELSE
     MAX_LAYERS = NLAYERS_INP + 1
  END IF

  IF( DO_AEROSOL ) MAX_LAYERS = MAX_LAYERS + 1

  ALLOCATE( HEIGHTS(MAX_LAYERS) )
  NL_LOC  = NLAYERS_INP
  HEIGHTS = 0.d0                  

  ! =============================
  ! * 1. Create layering arrays *
  ! =============================

  ! To establish layering, work initially from ground up.  Our normal
  ! convention is to number layers from the top down (so ZZZ(1) gives
  ! the top, 2 is below that, etc.). But it's more convenient to create
  ! layers from the ground up, so we compute a bottom-up layer array,
  ! HEIGHTS, and convert it to the other convention for ZZZ after we're done.

  ! 1a.  No clouds - clear atmosphere
  ! =================================
  DO J = 1, NLAYERS_INP
     IF(ZZZ_INP(J) > GROUND_HEIGHT + DELTA) EXIT
  END DO
  N_S = J

  ! IF(GROUND_HEIGHT > DELTA.AND.ZZZ_INP(N_S) - GROUND_HEIGHT < 0.2D0) &
  !    ZZZ_INP(N_S) = (ZZZ_INP(N_S + 1) + GROUND_HEIGHT)/2.0D0

  NL_LOC = NLAYERS_INP - N_S + 2
  HEIGHTS(1) = GROUND_HEIGHT
  HEIGHTS(2:NL_LOC) = ZZZ_INP(N_S:NLAYERS_INP)
  
  ! IF(GROUND_HEIGHT > DELTA.AND.ZZZ_INP(N_S) - GROUND_HEIGHT < 0.2D0) &
  !     HEIGHTS(2) = (ZZZ_INP(N_S + 1) + GROUND_HEIGHT)/2.0D0

  ! Boundary aerosol layer or aerosol as "cloud layer"
  IF( DO_AEROSOL ) THEN
     IF(AER_ADD_LEV) CALL ADD_TOP_BL(*880)
     IF(SHARP_EDGES) CALL AER_GRID(*880)
  ENDIF   ! DO_AEROSOL 
! **********************************************************************

  ! 1c. Cloud layers in the atmosphere
  IF( DO_CLOUD ) THEN         
     ALLOCATE(NEWHEIGHT(MAX_LAYERS), NEW_ARR(MAX_LAYERS,2), XIN(MAX_LAYERS),&
          WWW(MAX_LAYERS), DZIN(MAX_LAYERS), XXX(MAX_LAYERS), MASK(MAX_LAYERS))

     ! Initialize Cloud Layer Variables
     ! --------------------------------
     NEW_ARR(:,:) = 0.D0  ; NEWHEIGHT(:) = 0.D0
         
     DO NC = 1, N_CLD_LAYERS
        C_TOP = CLH(NC,2)
        ! WRITE(*,*) NC, CLH(NC,1), C_TOP
        IF( C_TOP > HEIGHTS(NL_LOC) ) GOTO 201
     ENDDO

     SELECT CASE( TRANS_LAY )
     CASE ('Default')
        EPS_CLD = INF_LAYER
     CASE ('Dynamic')
        DIFF = MINVAL( (CLH(:,2)-CLH(:,1)),1 )
        EPS_CLD = DIFF*INF_LAYER
     CASE('User_def')
        EPS_CLD = GT_TL
     CASE DEFAULT
        EPS_CLD = INF_LAYER
     END SELECT
     
     LIM = EPS_CLD 
     ! WRITE(*,*) '    -------->> LIM = ', LIM

     DO NC = 1, N_CLD_LAYERS
        CLAYER = DBLE ( NC )
        C_BOT  = CLH(NC,1)
        ! If required cloud bottom, C_BOT, is at the surface,
        ! the cloud bottom is shifted upwards to C_BOT+LIM.
        ! Therefore, the cloud including "infinitesimal" layer
        ! has bottom at the surface. This allow us consider
        ! this situation using the same algorithms as for 
        ! other cloud layers in the atmosphere. 
        IF( C_BOT == HEIGHTS(1) ) C_BOT = C_BOT + LIM
        C_TOP  = CLH(NC,2)

        ! Number of "finite" sub-layers within cloud
        N_STEP = INT( CLD_LAYER_NUM(NC) )
        Z_STEP = (C_TOP - C_BOT)/ DBLE(N_STEP)

        ! Altitude grid including bottom and top 
        ! altitudes of a cloud
        XIN(1) = C_BOT
        DO J = 2,N_STEP + 1
           XIN(J) = XIN(J-1) + Z_STEP 
        ENDDO

        SELECT CASE ( SUB_LAYERING(NC)%S )
        CASE('equidistant')
        CASE('logarithmic')
           ! write(*,*) '  Top and bot before'
           ! WRITE(*,*) XIN(N_STEP+1), C_TOP 
           ! WRITE(*,*) XIN(1), C_BOT
           Z_STEP = (DLOG(XIN(N_STEP+1)/XIN(1)))/ DBLE(N_STEP)
           XIN(1) = DLOG(C_BOT)
           DO J = 2,N_STEP + 1
              XIN(J) = XIN(J-1) + Z_STEP 
           ENDDO
           DO J = 2,N_STEP+1
              DZIN(J) = DEXP(XIN(J)) - DEXP(XIN(J-1))
           ENDDO
           XIN(N_STEP+1) = C_TOP
           K = 2
           DO J = N_STEP+1,2,-1
              XIN(J-1) = XIN(J) - DZIN(K)
              K = K + 1
           ENDDO
        END SELECT
        ! An "infinitesimal" layer is added at the bottom
        XXX(1)          = C_BOT - LIM
        XXX(2:N_STEP+2) = XIN(1:N_STEP+1)
        ! and at the top of a cloud
        XXX(N_STEP + 3) = XIN(N_STEP+1) + LIM
        ADD_LEVEL       = N_STEP + 3 
        ! Find index Js for which HEIGHTS(Js) < C_BOT
        ! -------------------------------------------
        MASK(:) = .FALSE.
        WWW(:) = SNGL(HEIGHTS(:)) + SNGL(-C_BOT + LIM )
        WHERE(WWW(1:NL_LOC) < 0.0) MASK(1:NL_LOC)= .TRUE.
        Js = COUNT( MASK ) ; N_LEV = Js + ADD_LEVEL
        NEWHEIGHT(1:Js) =  HEIGHTS(1:Js)
        NEWHEIGHT( Js+1:N_LEV ) = XXX(1:ADD_LEVEL)
        ! Find Je : HEIGHTS(Je) >= C_TOP
        ! --------------------------------------
        MASK(:) = .FALSE. ; WWW(:) = SNGL(HEIGHTS(:)) - SNGL(C_TOP + LIM)
        WHERE(WWW(1:NL_LOC) <= 0.0) MASK(1:NL_LOC)= .TRUE.
        Je = COUNT( MASK ) + 1 ; Jc = N_LEV + NL_LOC - Je + 1
        NEWHEIGHT( N_LEV+1:Jc ) = HEIGHTS( Je:NL_LOC )
        NL_LOC = Jc

        NEW_ARR(JS + 2:N_LEV-1,1) = 1.D0
        NEW_ARR(JS + 1:N_LEV,2)   = CLAYER 
        J_TOP(NC) = N_LEV   
        J_BOT(NC) = JS + 1
        HEIGHTS(1:NL_LOC) = NEWHEIGHT(1:NL_LOC)
        ! WRITE(*,*) '  N_step, Z_STEP  = ', N_STEP, Z_STEP
        ! WRITE(*,*) '  Js = ',Js,'  N_lev = ',N_LEV
        ! DO J = 1, Jc
        !    IF( NEW_ARR(J,2) > 0.d0 ) THEN
        !       WRITE(*,'(1x,I3,1x,E12.4,2(1x,F4.1))') &
        !           J, NEWHEIGHT(J), NEW_ARR(J,2), NEW_ARR(J,1)
        !    ELSE
        !       WRITE(*,'(1x,I3,1x,E12.4,2x,a3,1x,F4.1)') &
        !           J, NEWHEIGHT(J), ' - ', NEW_ARR(J,1)
        !    ENDIF
        ! ENDDO
        ! WRITE(*,*) '  J_bot = ', J_BOT(NC), NL_LOC-J_BOT(NC)+1
        ! WRITE(*,*) '  J_top = ', J_TOP(NC), NL_LOC-J_TOP(NC)+1
     END DO
     DO NC = 1, N_CLD_LAYERS
        J_TOP(NC) = NL_LOC - J_TOP(NC)  + 1
        J_BOT(NC) = NL_LOC - J_BOT(NC)  + 1
     ENDDO
  ENDIF  ! DO_CLOUD
  ! **********************************************************************

  ! Invert arrays so that they start at the top and works down,
  ! Following the standard convention in this program.
  ! ---------------------------------------------------------------
  NLAYERS = NL_LOC

  IF( DO_CLOUD ) THEN
     ALLOCATE(ZZZ(NLAYERS), DZR(1:NLAYERS), DZ(1:NLAYERS), & ! A
          CLD_LAYER_ARR(NLAYERS,2)) ! A
     ZZZ(1:NLAYERS) =  HEIGHTS(NLAYERS:1:-1)
     CLD_LAYER_ARR(1:NLAYERS,1:2) = NEW_ARR(NLAYERS:1:-1,1:2)
     DEALLOCATE(HEIGHTS, NEWHEIGHT, NEW_ARR, XXX, WWW, MASK)
     IF( ANY(Plus_ice) ) THEN
        ALLOCATE( Prof_Cpar(NLAYERS,4), Prof_Ref(NLAYERS,4) ) ! A
     ELSE
        ALLOCATE( Prof_Cpar(NLAYERS,2), Prof_Ref(NLAYERS,2) ) ! A
     ENDIF
     Prof_Cpar = 0.d0  ;   Prof_Ref = 0.d0
     ! DO I = 1,NLAYERS
     !   WRITE(*,'(2x,F8.3,2(2x,e12.4))') &
     !       ZZZ(I),CLD_LAYER_ARR(I,1),CLD_LAYER_ARR(I,2)
     ! ENDDO
  ELSE
     ALLOCATE(ZZZ(NLAYERS), DZR(1:NLAYERS), DZ(1:NLAYERS)) ! A
     ZZZ(1:NLAYERS) =  HEIGHTS(NLAYERS:1:-1)
     DEALLOCATE(HEIGHTS)
  END IF

  IF( SOUT_FLAG ) THEN
     OPEN(77,file= TRIM(DIR_SOUT)//'height.dat' )
     DO I = 1,NLAYERS
        WRITE(77,'(2x,e15.7)') ZZZ(I)
     ENDDO
     CLOSE(77)
  ENDIF

  DZR (NLAYERS) = 0.D0
  DZR(1:NLAYERS - 1) = ZZZ(1:NLAYERS - 1) - ZZZ(2:NLAYERS)
  
  DZ(1)           = ZZZ(1) - ZZZ(2)
  DZ(NLAYERS)     = ZZZ(NLAYERS - 1) - ZZZ(NLAYERS)
  DZ(2:NLAYERS-1) = ZZZ(1:NLAYERS - 2) - ZZZ(3:NLAYERS)
  
  ! --------------- Create underwater depth grid ------------------------
  IF( UWT ) THEN
     K = 0
!     WRITE(*,*) '  NDEP_INP and OC_DEP ', NDEP_INP, OC_DEP
     DO I = 1,NDEP_INP
!         WRITE(*,*) I, DEP_INP(I), OC_DEP - DEP_INP(I)
        IF( ABS(OC_DEP - DEP_INP(I)) <= 1.d-10 ) THEN
           K = I ; EXIT
        ENDIF
     ENDDO
!     WRITE(*,*) '   K = ',K
     IF( K == 0 ) THEN
        GOTO 202
     ELSE
        NL_UW = K  ;  ALLOCATE( ZZZ_UW(NL_UW) ) 
        ZZZ_UW(1:K-1) = DEP_INP(1:K-1) ; ZZZ_UW(NL_UW) = OC_DEP
     ENDIF

     ! Sub-grid depth of ice layer in the case of RT through ice and water
     IF( IWT ) THEN
        ALLOCATE( Ice_T(LA_N_IW), Ice_B(LA_N_IW) )
        CALL ICE_GRID(*880) 
        DO I = 1, LA_N_IW
           IWS(1)%IW_TOP(I) = Ice_T(I)+1 ;  IWS(1)%IW_BOT(I) = Ice_B(I)-1
        ENDDO
     ELSE
        IWS(1)%IW_TOP(1) = 1  ;  IWS(1)%IW_BOT(1) = NL_UW
     ENDIF
     IWS(2)%IW_TOP(1) = 1  ;  IWS(2)%IW_BOT(1) = NL_UW

     ALLOCATE( DZ_UW(NL_UW) ) 
     DZ_UW(1)         = ZZZ_UW(2)       - ZZZ_UW(1)
     DZ_UW(NL_UW)     = ZZZ_UW(NL_UW)   - ZZZ_UW(NL_UW-1)
     DZ_UW(2:NL_UW-1) = ZZZ_UW(3:NL_UW) - ZZZ_UW(1:NL_UW - 2)
     IF( LOG_LEVEL(1) >= N_ESO ) THEN
        WRITE(LUNIT,*) ' ---->> Altitude depth-grid in ocean [meter]:'
        DO J = 1,NL_UW
           WRITE(LUNIT,'(2X,I3,2X,F9.4)') J, ZZZ_UW(J) 
        ENDDO
        IF( IWT ) THEN
           DO I = 1,LA_N_IW
              WRITE(*,*) '    Ice layer top and bottom of ice layer ', I
              WRITE(*,'(4x,2(2X,F9.4))') ZZZ_UW(Ice_T(I)+1), ZZZ_UW(Ice_B(I)-1)
           ENDDO
        ENDIF
     ENDIF
!     STOP  '   Stop in gt_layer_setup:  '
  ENDIF  ! UWT
  ! **********************************************************************

  ! Set INDEX FOR OBSERVER ALTITUDE
  ObserverAltitude: Block
    IF( RTM_CORE /= 'DOM' ) Exit ObserverAltitude
    Select Case (RTM_MODE)
    Case ('flux','vod','spher_alb')
       Exit ObserverAltitude
    End Select
    ADD_LEV = .FALSE.
    ! If specific output altitudes are defined
    IF( SET_OUT_ALT ) THEN       
       IF( ARB_OL ) THEN 
          ALLOCATE( GRID_MARK(NLAYERS) ) ; GRID_MARK(:) = .FALSE.
          IF( UWT ) THEN
             ALLOCATE( GRID_MARK_UW(NL_UW) ) 
             GRID_MARK_UW(:) = .FALSE.
          ENDIF
       ENDIF       
       SELECT CASE ( RTM_TYPE )
          ! For spherical case NOA corresponds to the 
          ! bottom level of the layer 
          ! where the observation point is located.
       CASE( 'spher_scat' )   
          N_LEV = MINLOC( DABS(ZZZ(:)-ALT_OUT(1)),1 )
          DIFF = ZZZ(N_LEV) - ALT_OUT(1)
          IF( DIFF > 0.d0 .AND. DABS(DIFF) > DELTA ) THEN
             ! WRITE(*,*) '    C-1', ZZZ(N_LEV), ALT_OUT(1)
              NOA(1) = N_LEV + 1
              ADD_LEV = .TRUE.
           ENDIF
           IF( DIFF < 0.d0 .AND. DABS(DIFF) > DELTA ) THEN
              ! WRITE(*,*) '    C-2', ZZZ(N_LEV+1), ALT_OUT(1)
              NOA(1) = N_LEV
              ADD_LEV = .TRUE.
           ENDIF
           IF( DABS(DIFF) <= DELTA ) THEN
              ! WRITE(*,*) '    C-3', ZZZ(N_LEV), ALT_OUT(1)
              NOA(1) = N_LEV 
              ADD_LEV = .FALSE.
           ENDIF
        CASE DEFAULT 
           ! If output at standard altitude grid levels:
           IF( UWT ) THEN
              SELECT CASE( TRIM(Obs_POSITION(1)) ) ! Preliminary ??
              CASE('Ocean')                  
                 ! ZZZ_UW contains all additional output levels if
                 ! parameter ARB_OL = .TRUE.
                 DO I = 1,N_OP ; NOA(I) = MINLOC( &
                      DABS(ZZZ_UW(:) - DABS(ALT_OUT(I)) ),1)
                 ENDDO
                 IF( ARB_OL ) THEN
                    DO I = 1,N_OP ; IF( DEP_MARK(I) ) &
                         GRID_MARK_UW(NOA(I)) = .TRUE.
                    ENDDO
                 ELSE
                    ALT_OUT(:) = -ZZZ_UW( NOA(:) )
                 ENDIF
              CASE('Atmos')
                 DO I = 1,N_OP
                    NOA(I) = MINLOC( DABS(ZZZ(:)-ALT_OUT(I)),1 )
                 ENDDO
                 IF( ARB_OL ) THEN
                    DO I = 1,N_OP
                       IF( ALT_MARK(I) ) GRID_MARK(NOA(I)) = .TRUE.
                    ENDDO
                 ELSE
                    ALT_OUT(:) = ZZZ( NOA(:) )
                 ENDIF
              END SELECT
              ! RT in atmosphere
           ELSE
              ! ZZZ contains all additional output levels if
              ! parameter ARB_OL = .TRUE.
              DO I = 1,N_OP
                 NOA(I) = MINLOC( DABS(ZZZ(:)-ALT_OUT(I)),1 )
              ENDDO
              ! Additional output levels will be marked as .true.
              ! if they do not coincide with levels of the
              ! standard height grid
              IF( ARB_OL ) THEN
                 DO I = 1,N_OP
                    IF( ALT_MARK(I) ) GRID_MARK(NOA(I)) = .TRUE.
                 ENDDO
              ELSE
                 ALT_OUT(:) = ZZZ( NOA(:) )
              ENDIF
           ENDIF
        END SELECT
        ! If standard output altitudes are defined        
     ELSE
        SELECT CASE( TRIM(INS_POSI) )
        CASE('toa','too')
           NOA(1) = 1
        CASE('boa')
           NOA(1) = NLAYERS
        CASE('boo')
           NOA(1) = NL_UW
        END SELECT
     ENDIF

!   ************************  Screen output ****************************
     IF( LOG_LEVEL(1) >= N_ESO ) THEN
        IF( ARB_OL ) THEN
           IF( UWT ) THEN
              SELECT CASE( TRIM(Obs_POSITION(1)) ) ! Preliminary ??
              CASE('Ocean')                  
                 WRITE(*,*) '  Additional output levels in ocean'
                 WRITE(*,*) "  which do't coincide with input depths"
                 DO I = 1,NL_UW
                    IF( GRID_MARK_UW(I) ) WRITE(*,*) I, GRID_MARK_UW(I), ZZZ_UW(I) 
                 ENDDO
              CASE('Atmos')
                 WRITE(*,*) '  Additional output levels in atmosphere'
                 WRITE(*,*) "  which do't coincide with input heights"
                 DO I = 1,NLAYERS
                    IF( GRID_MARK(I) ) WRITE(*,*) I,GRID_MARK(I),ZZZ(I) 
                 ENDDO
              END SELECT
           ELSE
              WRITE(*,*) '  Additional output levels in atmosphere'
              WRITE(*,*) "  which do't coincide with input heights"
              DO I = 1,NLAYERS
                 IF( GRID_MARK(I) ) WRITE(*,*) I,GRID_MARK(I),ZZZ(I) 
              ENDDO
           ENDIF
           ! ===========================================================
        ENDIF
        DO I = 1,N_OP
           WRITE(LUNIT,'(A,I3,2x,I3)') '    NOA  = ', I, NOA(I)
           WRITE(LUNIT,'(A,I3,2x,A)') &
                '   Obs_POSITION :', I, TRIM(Obs_POSITION(I))
        ENDDO
     ENDIF  ! LOG_LEVEL(1) >= N_ESO 
     ! ********************* End screen output ****************************
     !         PAUSE
   End Block ObserverAltitude
   
  ! WRITE(*,*) '   Main altitude grid  NLAYERS = ', NLAYERS
  ! WRITE(*,*) ZZZ(:)
  ! STOP

  CALL ST_TRACK (R_NAME, 1, 'O')
  RETURN
  
201 CALL WRITE_MESSAGE(R_NAME,'ERROR',' ')
  WRITE(EU,*) '  Cloud top above the top of atmosphere.'
  WRITE(EU,'(A,1X,F7.2)') '       Cloud top height:', C_TOP
  WRITE(EU,'(A,1X,F7.2)') '  Atmosphere top height:',HEIGHTS(NL_LOC)
  WRITE(EU,*) '  Check input files: cloud.inp and ',TRIM(NET_NAME)
  RETURN 1

202 CALL WRITE_MESSAGE(R_NAME,'ERROR',' ')
  WRITE(EU,*) '  Ocean bottom is outer of depth-grid.'
  WRITE(EU,'(A,1x,F7.2)') '    Ocean bottom:', OC_DEP
  WRITE(EU,'(A,1x,F7.2)') '   Maximal depth:',DEP_INP(NDEP_INP)
  WRITE(EU,*) '   Check input file:', TRIM(DEP_NAME)
  RETURN 1

880 WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)
  RETURN 1

CONTAINS
! **********************************************************************
  SUBROUTINE ICE_GRID(*)
    USE WATER_ICE, ONLY: ICE_LAY_NUM, L_TOP_I, L_BOT_I

    CHARACTER (LEN=*), PARAMETER :: R_NAME = 'ICE_GRID'
    INTEGER Ii, LA_N, NL_LOC, MAX_LAYERS, Ic_TOP, Ic_BOT
    LOGICAL TOP_ICE
    LOGICAL, ALLOCATABLE :: MASK(:)
    REAL*4, ALLOCATABLE :: WWW(:)
    REAL (KND), ALLOCATABLE :: HEIGHTS(:), NEWHEIGHT(:), TMP(:), NEW_ARR(:,:), &
         XXX(:), XIN(:), DZIN(:)
    ! ---------------------------------------------------------------------    

    LA_N = SIZE( L_TOP_I )
!    WRITE(*,*) ' Number of ice layers ', LA_N

    DO Ii = 1, LA_N
       MAX_LAYERS =  ICE_LAY_NUM(Ii) + 3 + NL_UW
       ALLOCATE( HEIGHTS(MAX_LAYERS) )
       NL_LOC  = NL_UW  ;   HEIGHTS(1:NL_UW) =  ZZZ_UW(:)
!       WRITE(*,*) '   Ice layer ', Ii
!       WRITE(*,*) '  ZZZ_UW(1) = HEIGHTS(1) = ', HEIGHTS(1)
       ALLOCATE( NEWHEIGHT(MAX_LAYERS), NEW_ARR(MAX_LAYERS,2), &
            WWW(MAX_LAYERS), DZIN(MAX_LAYERS), XXX(MAX_LAYERS), &
            XIN(MAX_LAYERS), MASK(MAX_LAYERS) )
       NEW_ARR(:,:) = 0.d0  ; NEWHEIGHT(:) = 0.d0
       C_TOP = L_BOT_I(Ii) ;  C_BOT = L_TOP_I(Ii)  ;  CLAYER = DBLE(Ii)
       LIM = INF_LAYER
       TOP_ICE = .FALSE.
       IF( C_BOT == HEIGHTS(1) ) TOP_ICE = .TRUE.
!   ... Number of "finite" sub-layers within ice layer
       N_STEP = ICE_LAY_NUM(Ii)
       Z_STEP = (C_TOP - C_BOT)/ DBLE(N_STEP)
!       write(*,*) '  Z_step in ice: ', Z_STEP
!   ... Depth grid including bottom and top depthss of an ice layer
       XIN(1) = C_BOT
       DO J = 2,N_STEP + 1
          XIN(J) = XIN(J-1) + Z_STEP 
       ENDDO

!       write(*,*) '    Check XINV'
!       DO J = 1, N_STEP + 1
!          WRITE(*,*) J, XIN(J)
!       ENDDO
       IF( TOP_ICE ) THEN
          !... No "infinitesimal" layer at the top of ice layer
          XXX(1:N_STEP+1) = XIN(1:N_STEP+1)
          !    and at the bottom of an ice layer
          XXX(N_STEP + 2) = XIN(N_STEP+1) + LIM
          ADD_LEVEL       = N_STEP + 2 
       ELSE
          !... An "infinitesimal" layer is added at the top
          XXX(1)          = C_BOT - LIM
          XXX(2:N_STEP+2) = XIN(1:N_STEP+1)
          !    and at the bottom of an ice layer
          XXX(N_STEP + 3) = XIN(N_STEP+1) + LIM
          ADD_LEVEL       = N_STEP + 3 
       ENDIF
    !   ...  Find index Js for which HEIGHTS(Js) < C_BOT
!   -------------------------------------------
       MASK(:) = .FALSE.
       WWW(:) = SNGL(HEIGHTS(:)) + SNGL(-C_BOT + LIM )
!    IF( TOP_ICE ) WWW(1) = 0.d0
       WHERE(WWW(1:NL_LOC) < 0.0) MASK(1:NL_LOC)= .TRUE.
       Js = COUNT( MASK ) ; N_LEV = Js + ADD_LEVEL
       NEWHEIGHT(1:Js) =  HEIGHTS(1:Js)
       NEWHEIGHT( Js+1:N_LEV ) = XXX(1:ADD_LEVEL)
!   ... Find Je : HEIGHTS(Je) >= C_TOP
!   --------------------------------------
       MASK(:) = .FALSE. ; WWW(:) = SNGL(HEIGHTS(:)) - SNGL(C_TOP + LIM)
       WHERE(WWW(1:NL_LOC) <= 0.0) MASK(1:NL_LOC)= .TRUE.
       Je = COUNT( MASK ) + 1 ; Jc = N_LEV + NL_LOC - Je + 1
       NEWHEIGHT( N_LEV+1:Jc ) = HEIGHTS( Je:NL_LOC )
       NL_LOC = Jc
       NEW_ARR(JS + 2:N_LEV-1,1) = 1.d0
       NEW_ARR(JS + 1:N_LEV,2)   = CLAYER 
       Ic_TOP = N_LEV   
       Ic_BOT = JS + 1
       HEIGHTS(1:NL_LOC) = NEWHEIGHT(1:NL_LOC)

!       WRITE(*,*) '  N_step, Z_STEP  = ', N_STEP, Z_STEP
!       WRITE(*,*) '  Js = ',Js,'  N_lev = ',N_LEV
!       DO J = 1, Jc
!          IF( NEW_ARR(J,2) > 0.d0 ) THEN
!             WRITE(*,'(1x,I3,1x,ES14.6,2(1x,F4.1))') &
!                  J, NEWHEIGHT(J), NEW_ARR(J,2), NEW_ARR(J,1)
!          ELSE
!             WRITE(*,'(1x,I3,1x,ES14.6,2x,a3,1x,F4.1)') &
!                  J, NEWHEIGHT(J), ' - ', NEW_ARR(J,1)
!          ENDIF
!       ENDDO
!       WRITE(*,*) '  Ic_bot = ', Ic_BOT
!       WRITE(*,*) '  Ic_top = ', Ic_TOP
       IF( TOP_ICE )  THEN
          Ice_T(Ii) = 0
       ELSE
          Ice_T(Ii) = Ic_BOT
       ENDIF
       Ice_B(Ii) = Ic_TOP
       !   ... Set joint water + ice depth grid
       NL_UW = NL_LOC
       DEALLOCATE( ZZZ_UW ) ; ALLOCATE( ZZZ_UW(NL_UW) ) 
       ZZZ_UW(:) = HEIGHTS(1:NL_LOC)
       DEALLOCATE(HEIGHTS, NEWHEIGHT, NEW_ARR, XIN, WWW, DZIN, XXX, MASK)
    ENDDO  ! over ice layers (Ii)

!    STOP '   Stop in "gt_layer_setup.f90" '
    
    RETURN
101 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Reading error')
    WRITE(EU,*) '    Parameter to read: ', TRIM(STR_INP) 
    RETURN 1

880 WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)
    RETURN 1

  END SUBROUTINE ICE_GRID
  
! **********************************************************************
  SUBROUTINE AER_GRID(*)
    USE AEROSOLS,  ONLY: ZA_SE
    USE TYPES,     ONLY: DYN_CHAR
    USE TOOLS,     ONLY: CUT_STR, SET_PATH, READ_INPUT
    CHARACTER (LEN = *), PARAMETER :: R_NAME = 'AER_GRID (gt_layer_setup.f)'
    CHARACTER (LEN=:), ALLOCATABLE :: STR_INP
    CHARACTER (LEN=100) Msg(10)
    TYPE (DYN_CHAR), ALLOCATABLE :: READ_ARR(:)
    INTEGER  N_ZE, I1, Ilo, I2, NZ0, J1, J2, Jp, N_res, I, NL_new, N_use
    LOGICAL, ALLOCATABLE :: LZER(:)
    REAL (KND) Z1, Z2
    REAL (KND), ALLOCATABLE ::  ECO(:), Z_E(:), HT1(:), Zx(:), Ex(:)
    ! ------------------------------------------------------------------

    ! Geometrical thickness of sub-layers is the same as for cloud:
    ! see above and "gt_cloud_parameter_setup.f"
    LIM = INF_LAYER
      
    SELECT CASE (TRIM(AER_PAR_TYPE))
    CASE ('mn','mn_mi')
       STR_INP = 'Extinction coefficient'
    CASE('pao')
       STR_INP = 'Aerosol particle number density (PAO)'
    END SELECT
    
    CALL Read_Input (STR_INP, READ_ARR, *880)
    N_ZE = SIZE( READ_ARR )
    ALLOCATE( ECO(N_ZE), Z_E(N_ZE), LZER(N_ZE) )
    DO I = 1, N_ZE
       READ(READ_ARR(I)%S, *,ERR = 101) Z_E(I), ECO(I)
    END DO

    ! Rewrite in ascending order if needed
    IF( Z_E(1) > Z_E(N_ZE) ) THEN
       Z_E(1:N_ZE) = Z_E(N_ZE:1:-1)
       ECO(1:N_ZE) = ECO(N_ZE:1:-1)
    ENDIF
      
    IF( AER_SHIFT ) Z_E(:) = Z_E(:) + GROUND_HEIGHT 
    
    ! Check if bottom of aerosol layer above surface
    IF( Z_E(1) < HEIGHTS(1) ) GOTO 105

    ALLOCATE( Zx(N_ZE), Ex(N_ZE) )
    Zx(:) = Z_E(:) ; Ex(:) = Eco(:)
    
    ! Save main altitude grid into array HT1
123 ALLOCATE( HT1(NL_LOC) )
    HT1(:) = HEIGHTS(1:NL_LOC) 
    
    LZER = .FALSE.
    WHERE( Ex(:) > 0.d0 )
       LZER(:) = .TRUE.
    END WHERE

    IF( COUNT(LZER) == N_ZE ) GOTO 107
      
    ! Check if aerosol layer starts from surface
    IF( LZER(1) ) THEN
       IF( Z_E(1) > HEIGHTS(1) )  GOTO 106
    ENDIF
    ! Find lower altitude Z1 of first aerosol layer
    DO I = 1, N_ZE
       IF( LZER(I) ) THEN
          Z1 = Zx(I) ; I1 = I ; EXIT
       ENDIF
    ENDDO
    ! Find upper altitude Z2 of first aerosol layer
    DO I = I1, N_ZE
       IF( .NOT.LZER(I) ) THEN
          Z2 = Zx(I-1) ; I2 = I-1 ; EXIT
       ENDIF
    ENDDO

    ! Introduce lower and upper "infinitesimal" sub-layers
    IF( I1 > 1 ) THEN
       Zx(I1-1) = Zx(I1) - LIM 
       Ilo = I1 - 1 ; Jp = 1
       N_use = (I2+1 - Ilo + 1) - 2
    ELSE
       Ilo = I1     ;  Jp = 0
       N_use = (I2+1 - Ilo + 1) - 1
    ENDIF
    Z1 = Zx(Ilo)
    Zx(I2+1) = Zx(I2) + LIM 
    Z2 = Zx(I2+1)
    
    NZ0 =  I2+1 - Ilo + 1
    ! Number of levels within aerosol layer
    !    WRITE(*,*) ' NZ0 = ', NZ0 
    !    WRITE(*,*) Z1, Z2
    !    DO I = 1, N_ZE
    !       WRITE(*,'(2x, f9.5, *(2x,E12.4))') Zx(I), Ex(I)
    !    ENDDO
    ! Main grid layer number which below Z1
    DO I = 1, NL_LOC
       IF(  HT1(I) >= Z1 ) THEN
          J1 = MAX(I-1,1) 
          EXIT
       ENDIF
    ENDDO
    ! Main grid layer number which above Z2
    DO I = 1, NL_LOC
       IF(  HT1(I) >= Z2 ) THEN
          J2 = I ; EXIT
       ENDIF
    ENDDO
    ! WRITE(*,*) '  J1 = ',J1 
    ! WRITE(*,*) HT1(J1), Z1
    ! WRITE(*,*) Z2, HT1(J2)
    ! WRITE(*,*) '   Grid before setting -1 '
    ! WRITE(*,*)  HT1(1:NL_LOC)
    HT1(J1+Jp:J2-1) = -1.d0

    N_res = COUNT(HT1(:) >= 0.d0)
    ! WRITE(*,*) '   NL_LOC = ', NL_LOC 
    ! WRITE(*,*) '   N_res = ', N_res
    ! WRITE(*,*)  HT1(1:NL_LOC)
    NL_new = N_res + NZ0
    DEALLOCATE( HEIGHTS )
    ALLOCATE( HEIGHTS(NL_new) )
    !    WRITE(*,*) '   NL_new = ', NL_new
    !    WRITE(*,*) '  MAX_LAYERS  ', MAX_LAYERS 
    ! Create extended altitude grid     
    IF( I1 > 1 ) THEN
       HEIGHTS(1:J1)            = HT1(1:J1)
       HEIGHTS(J1+1:J1+NZ0)     = Zx(Ilo:I2+1)
       HEIGHTS(J1+NZ0+1:NL_new) = HT1(J2:NL_LOC)
    ELSE
       HEIGHTS(1:NZ0)     = Zx(1:I2+1)
       HEIGHTS(NZ0+1:NL_new) = HT1(J2:NL_LOC)
    ENDIF
    
    NL_LOC = NL_new
    ! WRITE(*,*) 
    ! WRITE(*,*)  HEIGHTS(1:NL_LOC)
    IF( COUNT(Ex(:)>0.d0) > N_use ) THEN
       WRITE(*,*) '  ****** Set next aerosol layer ******* '
       ! WRITE(*,*) '  NZ0-2  ', COUNT(Ex(:)>0.d0), N_use
       Ex(I1:I2) = 0.d0
       DEALLOCATE( HT1 )
       GOTO 123
    ENDIF

    ! Save aerosol altitude grid with sharp edges to perform
    ! interpolation in subroutine "man_aer_setup.f"
    ALLOCATE( ZA_SE(N_ZE) )
    ZA_SE(:) = Zx(:)

    RETURN
    
101 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Reading error')
    WRITE(EU,*) '    Parameter to read: ', TRIM(STR_INP) 
    RETURN 1

104 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Too few values in the field')
    WRITE(EU,*) '    Parameter to read: ', TRIM(STR_INP)
    RETURN 1
    
105 Msg(1)='Bottom of aerosol layer below the surface.'
    Msg(2)=' Check the "man_aer.inp" control file.'
    CALL WRITE_MSG(R_NAME, 'ERROR', Msg, 2, *880)
    
106 Msg(1)='Extinction coefficient at lowest aerosol level'
    Msg(2)=' should be zero or lowest altitude'
    Msg(3)=' should be coincide with surface altitude'
    CALL WRITE_MSG(R_NAME, 'ERROR', Msg, 3, *880)
    
107 Msg(1)='The sharp edges of aerosol layer can not be defined'
    Msg(2)='  because no zero values for extinction coefficient'
    Msg(3)='  or aerosol namber density were specified in input data.'
    CALL WRITE_MSG(R_NAME, 'ERROR', Msg, 3, *880)
    
880 WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)
    RETURN 1
    
  END SUBROUTINE AER_GRID
  ! **********************************************************************
  SUBROUTINE ADD_TOP_BL(*)
    
    CHARACTER (LEN=*), PARAMETER :: R_NAME = 'ADD_TOP_BL'
    INTEGER  I_AER
    LOGICAL  ADD_AER
    REAL (KND) TOP_BL, D1, D2
    ! ------------------------------------------------------------------

    SELECT CASE (TRIM(AER_PAR_TYPE))
    CASE DEFAULT
       SELECT CASE (TRIM(AER_PAR_TYPE))
       CASE ('mn','mn_mi')
          STR_INP  = 'Layering of scattering function/matrix'
       CASE ('wm','wm_mi')
          STR_INP  = 'WMO aerosol layers'
       CASE('st','st_mi')
          STR_INP  = 'Aerosol layers (SCIA_AER)'
       CASE('pao')
          STR_INP  = 'PAO aerosol layers'
       END SELECT
       CALL Read_Input (STR_INP, READ_ARR, *880)
       READ(READ_ARR(1)%S,*,ERR = 101) TOP_BL
    CASE('lt','lt_mi')
       Top_BL = 2.d0
    END SELECT
    
    IF( AER_SHIFT ) Top_BL = Top_BL + GROUND_HEIGHT
    
    ADD_AER = .FALSE.
    DO I = 2, NL_LOC
       IF( TOP_BL < HEIGHTS(I) ) THEN
          D1 = HEIGHTS(I) - TOP_BL
          D2 = TOP_BL - HEIGHTS(I-1)
          IF( D1 > DELTA .AND. D2 > DELTA ) THEN
             ! Add new level corresponding to the top 
             ! of boundary aerosol layer
             ADD_AER = .TRUE. ;  I_AER = I - 1
             EXIT
          ENDIF
       ENDIF
    ENDDO
    ! Substitute the exact position of the boundary 
    ! aerosol layer into main altitude grid
    IF( ADD_AER ) THEN
       ALLOCATE(TMP(MAX_LAYERS))
       TMP(1:I_AER) = HEIGHTS(1:I_AER)
       TMP(I_AER+1) = Top_BL 
       TMP(I_AER+2:NL_LOC+1) = HEIGHTS(I_AER+1:NL_LOC)
       NL_LOC = NL_LOC + 1
       HEIGHTS(1:NL_LOC) = TMP(1:NL_LOC)
    ENDIF
    
    RETURN

101 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Reading error')
    WRITE(EU,*) '    Parameter to read: ', TRIM(STR_INP) 
    RETURN 1

880 WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)
    RETURN 1
  END SUBROUTINE ADD_TOP_BL

! ***********************************************************************
  SUBROUTINE DEALLOCATE_ARRAYS
    
    IF(ALLOCATED(ZZZ)) DEALLOCATE(ZZZ, DZR, DZ)
    IF(ALLOCATED(CLD_LAYER_ARR)) DEALLOCATE(CLD_LAYER_ARR, Prof_Cpar, Prof_Ref)
    IF(ALLOCATED(ZZZ_UW)) DEALLOCATE(ZZZ_UW)
    IF(ALLOCATED(GRID_MARK)) DEALLOCATE(GRID_MARK)
    IF(ALLOCATED(GRID_MARK_UW)) DEALLOCATE(GRID_MARK_UW)
    If(Allocated(DZ_UW)) DeAllocate (DZ_UW)
    
    RETURN
    
  END SUBROUTINE DEALLOCATE_ARRAYS
!***********************************************************************
END SUBROUTINE GT_LAYER_SETUP
!***********************************************************************
