C   THIS CODE COMPUTES THE BIDIRECTIONAL REFLECTION FUNCTION
C   FOR A SEMI-INFINITE PARTICULATE SLAB USING THE ARRAY
C   OF PRECALCULATED FOURIER COMPONENTS OF THE REFLECTION
C   FUNCTION SPECIFIED AT THE DIVISION NODES OF A QUADRATURE
C   FORMULA ON THE INTERVAL (0, 1) (FILE 11).

C   INPUT PARAMETERS:
C
C   MU0 - COSINE OF THE ILLUMINATION ZENITH ANGLE 
C   MU - NMU VALUES OF THE COSINE OF THE REFLECTION ZENITH
C        ANGLE EVENLY DISTRIBUTED OVER THE INTERVAL (0, 1]
C        (THE SMALLEST MU VALUE IS 0.01)
C   PHI - NPHI VALUES OF THE AZIMUTH ANGLE EVENLY DISTRIBUTED
C   OVER THE INTERVAL [0, 180] DEGREES

C   OUTPUT PARAMETERS:
C
C   RO - ARRAY OF THE RESPECTIVE BRF VALUES (FILE 14)
  
C   FOR MORE INFORMATION SEE THE DESCRIPTION OF THE CODE COMPUTING
C   THE FOURIER COMPONENTS OF THE REFLECTION FUNCTION.

C   WHILE THE COMPUTER PROGRAM HAS BEEN TESTED FOR A VARIETY OF CASES,
C   IT IS NOT INCONCEIVABLE THAT IT CONTAINS UNDETECTED ERRORS. ALSO,
C   INPUT PARAMETERS CAN BE USED WHICH ARE OUTSIDE THE ENVELOPE OF
C   VALUES FOR WHICH RESULTS ARE COMPUTED ACCURATELY. FOR THIS REASON,
C   THE AUTHORS AND THEIR ORGANIZATION DISCLAIM ALL LIABILITY FOR
C   ANY DAMAGES THAT MAY RESULT FROM THE USE OF THE PROGRAM. 
      
      SUBROUTINE GT_R_INFINITY ( SOL, OBS, AAZ, REFL, ERROR )

      IMPLICIT NONE                        

      DOUBLE PRECISION    ZERO, RAD_TO_DEG
      PARAMETER          ( ZERO = 0.0D0 )
      PARAMETER	         ( RAD_TO_DEG = 57.29578D0 )

c     Input parameters
      DOUBLE PRECISION   SOL, OBS, AAZ

c     Output parameters
      DOUBLE PRECISION   REFL
      LOGICAL            error

c     Local variables
      DOUBLE PRECISION, ALLOCATABLE :: X(:), RI(:), Y(:), R(:,:), RR(:),
     $                                 AL1(:)
      DOUBLE PRECISION   MU0, MU, PHI, ALB, P11
      INTEGER            IFLAG, I, J, M1, NSEP, NG, MMAX1, MMAX2

c     Start of code
      mu   = dcos( obs / RAD_TO_DEG  )
      mu0  = dcos( sol / RAD_TO_DEG )
      PHI  = aaz / RAD_TO_DEG 

      OPEN (11,file = 
c     $     '../data/r_infinit/refl.write_kappa_1e-4',status='old')
     $     '../data/r_infinit/precalculated.dat',status='old')
         READ (11,*) NSEP
C***  READ IN QUADRATURE DIVISION POINTS
         READ (11,'(I5,5D15.8)') NG

         ALLOCATE( R(NG,NG), X(NG), Y(NG), RI(NG) ) ! A
         READ (11,'(6D15.8)') X

         IF (MU  < X(1))  MU  = X(1)
         IF (MU0 < X(1))  MU0 = X(1)
         IF (MU  > X(NG)) MU  = X(NG)
         IF (MU0 > X(NG)) MU0 = X(NG)

C  READ IN Number of EEXPANSION COEFFICIENTS and SINGLE-SCATTERING ALBEDO 
         READ (11,'(I5,5D15.8)') MMAX1, ALB
         ALLOCATE ( RR(MMAX1), AL1(MMAX1) ) ! A

C ***  READ IN  LEGENDR EEXPANSION COEFFICIENTS
         READ (11,'(6D15.8)') ( al1(M1), M1 = 1,MMAX1 )

         DO M1 = 1,MMAX1
            READ (11,*) IFLAG

C***    READ IN (M1-1)th FOURIER COMPONENT OF THE REFLECTION FUNCTION
            DO I = 1,NG
               READ (11,'(6D15.8)') (R(I,J), J = 1,I)
               DO J = 1,I
                  R(J,I) = R(I,J)
               ENDDO
            ENDDO
            MMAX2 = M1
            DO I = 1,NG                                                
               DO J = 1,NG                                             
                  Y(J) = R(I,J)*X(J)*X(I)            
               ENDDO                              
               Call AKIMA_INTERP(NG, X, Y, 1, MU0, RI(I), error)
            ENDDO
            Call AKIMA_INTERP(NG, X, RI, 1, MU, RR(M1), error)
            RR(M1) = RR(M1) / MU
            IF ( IFLAG .EQ. 0 ) Goto 100
         ENDDO                       

 100     CONTINUE
      CLOSE( 11 )

      REFL = RR( 1 )
      DO M1 = 2, MMAX2
         REFL = REFL + 2.D0 * RR( M1 ) * DCOS( DFLOAT(M1-1)*PHI )
      ENDDO
      IF ( NSEP .EQ. 1 ) THEN
         CALL MATR ( MU, MU0, PHI, MMAX1, AL1, P11)
         REFL = REFL + 0.25D0 * ALB * P11 * MU0 / ( MU + MU0 )
      ENDIF
      REFL = REFL / MU0
      
      RETURN 
      END                                                               
      
C********************************************************************   

C    CALCULATION OF THE PHASE FUNCTION FOR GIVEN MU, MU0, AND PHI-PHI0
C    AL1 - LEGENDRE EXPANSION COEFFICIENTS
C    MMAX1 - NUMBER OF COEFFICIENTS

      SUBROUTINE MATR ( MU, MU0, PHI, MMAX1, AL1, P11 )

      IMPLICIT NONE

c     Input parameters
      INTEGER          MMAX1
      DOUBLE PRECISION MU, MU0, PHI
      DOUBLE PRECISION AL1( MMAX1 )

c     Output parameter
      DOUBLE PRECISION P11

c     Local parameters
      INTEGER          LMAX, L1, L
      DOUBLE PRECISION PI, U, F11, P, P1, PP1

c     Start of code
      PI = DACOS( -1.D0 )      
      IF(DABS(PHI) < 1.D-7 .OR. DABS(PHI - PI) < 1.D-7)
     $     PHI = PHI + 1.D-7
      IF (DABS(MU - MU0) < 1.D-10) MU0 = MU0*0.999999999d0
      IF (DABS(MU + MU0) < 1.D-10) MU0 = MU0*0.999999999d0
      U    = -MU*MU0 + DSQRT(1.D0-MU*MU)*DSQRT(1.D0-MU0*MU0)*DCOS(PHI)
      F11  = 0.D0
      P1   = 0.D0
      PP1  = 1.D0
      LMAX = MMAX1 - 1
      DO L1 = 1,MMAX1
         L = L1-1
         F11 = F11 + AL1( L1 ) * PP1
         IF( L .EQ. LMAX ) GO TO 1
         P = ( DFLOAT( 2*L+1 )*U*PP1 - DFLOAT( L ) * P1 ) / DFLOAT( L1 )
         P1  = PP1
         PP1 = P
      ENDDO   
    1 P11 = F11
      RETURN
      END
c **********************************************************************
