!***************************************************************************
!
!	SCIATRAN - Version 4.5 - Last modification: 28 Aug 2021
!
!	Copyright (C) - Institute of Remote Sensing,
!			University of Bremen, Germany
!
!***************************************************************************
SUBROUTINE KOP_HSOL_SETUP(*)
  ! Call in "rtm_init.f"                              
  
  USE CLOUDS_SS,   ONLY: MUM => MAXMOM, DO_SSRCA
  USE DELTA_M,     ONLY: F_DLM_HSOL, DO_HSOL_DM, HSOL_TRUNC_IDX
  USE HYDROSOLS,   ONLY: LA_N => LA_N_HS, IDX_HS, L_BOT_HS, &
                         LA_S => LA_S_HS, LA_E => LA_E_HS, HS_SCALING, &
                         SEL_VSC, Greek_HS, NG_HS
  USE INTERN_DIMS, ONLY: Nm_vs, Nvs
  USE LOG_PARS,    ONLY: LUNIT, EU, DIR_OUT, LOG_LEVEL, N_SSO, N_ESO, C_UNIT, &
                         N_DSO, Work_Dir, DIR_OUT
  USE ST_VARS,     ONLY: RTD => RAD_TO_DEG, Vec_Sca, RTM_TYPE
  USE TOOLS,       ONLY: Read_Input, SET_PATH, Cut_Str, SPLIT_STRING
  USE TYPES    ,   ONLY: KND, DYN_CHAR
  USE WATER,       ONLY: NL_UW => NLAYERS_UW, ZZZ_UW, DZ_UW, NLEG_W
  USE WATER_ICE,   ONLY: IWS, IWT, LA_N_IW
  
  ! Local parameters
  CHARACTER (LEN = *), PARAMETER :: R_NAME = 'KOP_HSOL_SETUP'
  !  (a) Input variables
  !  ===================
  !      DO_HSOL_DM        : delta-M approximation yes/no 
  !                          <---- From Module DELTA_M
  !      HSOL_TRUNC_IDX    : truncation index for delta-M approximation 
  !                          <---- From Module DELTA_M
  !      ZZZ_UW             : forward model altitude grid in ocean
  !      NL_UW             : number of layers in altitude grid in ocean
  !      NLEG_W            : number of Legendre moments in water
  !      LUNIT             : LOG file unit number

  !  (b) Output variables
  !  ====================
  !      HS_COEFF         : scattering and extinction coefficients
  !      HS_PHASMOMS      : Legendre moments
  !      NAW              : number of wavelengths of chosen hydrosol 
  !                         optical parameters
  INTEGER     II, N_Z_EXT, NMP, I_SL(2), NWL0, IiW
  INTEGER,  ALLOCATABLE :: M_Lay_Wav(:,:)
  LOGICAL     TEST_MODE, ALL_HERE, ERROR
  CHARACTER (LEN=:), ALLOCATABLE :: DIR_name, Ph_type
  CHARACTER (LEN=:), ALLOCATABLE :: STR_INP
  TYPE (DYN_CHAR), ALLOCATABLE :: READ_ARR(:), STR_ARR(:), Names(:)
  REAL (KND)  TMP1
  REAL (KND), ALLOCATABLE :: HEL(:,:), Z_EXT(:), TMP2(:), TAU(:), &
       P_CONC(:,:), ASYM_PAR(:,:)
  REAL (KND), ALLOCATABLE, TARGET :: USER_MOM(:,:,:,:)
  REAL (KND), POINTER :: HS_PHASMOMS(:,:,:,:)
  INTEGER   I, I1, I2, J, K, L, IUNIT, OUNIT, Itr, NU_A, MAX_CO, N_GAU, NAW
  PARAMETER ( IUNIT = 33, OUNIT = 55 )
  ! Scattering coefficient [m^-1 ppm^-1] for 
  ! small and large particles at 550 nm
  REAL (KND), PARAMETER :: Sca_S = 1.34d0, Sca_L = 0.312d0
! ... From the structure IWS
  INTEGER N_HPC
  ! Variable for Delta-M
  ! ======================
  REAL (KND)  F_DEL
  !==================================================================
  CALL ST_TRACK (R_NAME, 1, 'I')

  IiW = 2
  N_HPC = IWS(IiW)%N_HPC

  ERROR = .FALSE.
  ! For the sum of small and large hydrosol components described by 
  ! Kopelevich model
  NMP = 2

  ! Optical parameters for this model are defined at 550 nm wavelength.
  ! The wavelength dependence will be used in analytical form
  NWL0 = 1

  ! =============================
  ! Kopelevich PARAMETERIZATION
  ! =============================
  STR_INP = 'Directory name for scattering matrixes (Kopelevich)'
  CALL Read_Input (STR_INP, READ_ARR, *880)
  DIR_name = SET_PATH(READ_ARR(1)%S)

  ! HS_COEFF can be allocated in the routine "man_hsol_setup.f"
  !    if "man_hsol.inp" file is used along with "kop_hsol.inp"
  IF( ALLOCATED(IWS(IiW)%HS_COEFF) ) THEN
     ALL_HERE = .FALSE.
!     WRITE(LUNIT,*) '  _____ HSOL parameters are already allocated _______'
  ELSE
     ALL_HERE = .TRUE.
!     WRITE(LUNIT,*) '  _____ HSOL parameters are not allocated _______'
     NAW = 1
     IWS(IiW)%N_HS_WL = NAW
     ! N_HPC - number of hydrosol particulate component
     ALLOCATE( IWS(IiW)%HS_COEFF(2,NL_UW,NAW,N_HPC),  &
          F_DLM_HSOL(NL_UW,NAW,N_HPC) ) ! A

     F_DLM_HSOL = 0.D0  ; IWS(IiW)%HS_COEFF = 0.D0 
     LA_N = 1
     ALLOCATE( LA_S(LA_N), LA_E(LA_N), L_BOT_HS(LA_N), &
          IWS(IiW)%HSOL_PHAS_VS( NLEG_W,LA_N,Nm_vs,NAW,N_HPC ) ) ! A

     IWS(IiW)%HSOL_PHAS_VS = 0.0d0  
     LA_S(1)     = 1
     LA_E(1)     = NL_UW 
     L_BOT_HS(1) = ZZZ_UW(NL_UW)
  ENDIF

  ALLOCATE( TMP2(Nm_vs) ) 

  STR_INP = 'Hydrosol scattering function representation (Kopelevich)'
  CALL Read_Input (STR_INP, READ_ARR, *880)
  PH_TYPE = CUT_STR(READ_ARR(1)%S)

  ALLOCATE( TAU(NMP), M_Lay_Wav(NMP,NWL0), Names(2), &
       USER_MOM(MUM,NMP,Nm_vs,NWL0) ) ! A

  USER_MOM = 0.0d0
  HS_PHASMOMS => IWS(IiW)%HSOL_PHAS_VS(:,:,1,:,:)

  SELECT CASE (Ph_type)
  CASE('Expansion_coeff')
     STR_INP = 'File names containing expansion coefficients (Kopelevich)'
     CALL Read_Input (STR_INP, READ_ARR, *880)
  CASE('Scattering_matrix')
     STR_INP = 'File names containing scattering function/matrix (Kopelevich)'
     CALL Read_Input (STR_INP, READ_ARR, *880)
  CASE DEFAULT
     GOTO 127
  END SELECT
  ! File names for small and large particles phase functions
  CALL SPLIT_STRING(READ_ARR(1)%S, STR_ARR)
  Names(1)%S = DIR_name//CUT_STR(STR_ARR(1)%S)
  Names(2)%S = DIR_name//CUT_STR(STR_ARR(2)%S)

  ! Reading concentration of particulate matter
  ! ===========================================
  STR_INP = 'Particulate mater concentration (Kopelevich)'
  CALL Read_Input (STR_INP, READ_ARR, *880)
  ! Number of levels
  N_Z_EXT = SIZE(READ_ARR)
  ALLOCATE( P_CONC(N_Z_EXT,NMP), Z_EXT(N_Z_EXT), HEL(NL_UW,NMP) )
  DO I = 1, N_Z_EXT
     ! Depth and concentration of small and large particles
     READ(READ_ARR(I)%S,*,ERR=101) Z_EXT(I), (P_CONC(I,K), K=1,NMP)
  END DO
  DO K = 1, NMP
     CALL LIN_INT (N_Z_EXT, Z_EXT, P_CONC(:,K), NL_UW, ZZZ_UW, HEL(:,K), 'Z')
  ENDDO

  DO I = 1, N_HPC
     SELECT CASE( TRIM(SEL_VSC(I)) )
     CASE('Kos')
        I_SL(1) = I
        IWS(IiW)%HS_COEFF(2,:,NWL0,I) = HEL(:,1)*Sca_S
        IWS(IiW)%HS_COEFF(1,:,NWL0,I) = HEL(:,1)*Sca_S
     CASE('Kol')
        I_SL(2) = I
        IWS(IiW)%HS_COEFF(2,:,NWL0,I) = HEL(:,2)*Sca_L
        IWS(IiW)%HS_COEFF(1,:,NWL0,I) = HEL(:,2)*Sca_L
     END SELECT
  ENDDO

  IF( IWT ) THEN
! ... Set to zero hydrosol scatt. and ext. coefficients within ice layer
     DO Ii = 1, LA_N_IW
        I1 = IWS(1)%IW_TOP(Ii)  ;  I2 = IWS(1)%IW_BOT(Ii)
        DO K = 1, N_HPC
           IWS(IiW)%HS_COEFF(1,I1:I2,NWL0,K) = 0.d0
           IWS(IiW)%HS_COEFF(2,I1:I2,NWL0,K) = 0.d0
        ENDDO  ! over hydrosol components (K)
     ENDDO  ! over ice layers (Ii)
  ENDIF

  !*********************************************************************
  !*              Scattering function/matrix generation                *
  !*********************************************************************
  SELECT CASE (TRIM(Ph_type))
  CASE('Expansion_coeff')
     ! Reading of the expansion coefficients 
     ! ==================================================================
     IF (LOG_LEVEL(1) >= N_ESO) WRITE(LUNIT,*) &
          '    Reading of the expansion coefficients  '
     DO I1 = 1,NMP
        OPEN( IUNIT, FILE=Names(I1)%S, STATUS='OLD' )
        CALL SKIP_HEADER (IUNIT)           
        DO I = 1,MUM
           READ(IUNIT,*,END=28,ERR=29) tmp1, (tmp2(J), J=1,Nm_vs)
           USER_MOM(I,I1,:,NWL0) = tmp2(:) 
           WRITE(*,'(f5.1,6(1x,e12.4))') tmp1,(tmp2(J), J=1,Nm_vs)
           ! The additional condition, I > 2, was added to read Rayleigh 
           ! phase function in the scalar case which is used for test purpose
           IF( SUM(TMP2(:)) == 0.d0 .AND. I > 2 ) GOTO 28
        END DO
29      CALL WRITE_MESSAGE (R_NAME, 'ERROR', 'Error reading data file !')
        WRITE(EU,*) '    File name : ', Names(I1)%S
        RETURN 1             
28      CONTINUE
        CLOSE(IUNIT)
        IF( Nm_vs > 1 ) THEN
           USER_MOM(:,I1,5,NWL0) = -USER_MOM(:,I1,5,NWL0)
           USER_MOM(:,I1,6,NWL0) = -USER_MOM(:,I1,6,NWL0)
        ENDIF
        M_Lay_Wav(I1,NWL0) = I - 1
        NU_A = I - 1
!        WRITE(*,*) '  Number of user moment :', NU_A
!        WRITE(*,*) USER_MOM(NU_A,I1,:,NWL0)
     ENDDO                  ! end for small and large particles
     
     IF( LOG_LEVEL(1) >= N_ESO ) WRITE(LUNIT,'(2x,17(1x,f10.7))') &
          (USER_MOM(L,1,1,NWL0)/(2*(L-1) + 1), L=1,11)

     ! Test output of scattering matrix corresponding to 
     ! the given expansion coefficients
     TEST_MODE = .FALSE.         
     IF( TEST_MODE ) THEN 
        ! Corresponds to Lay_out and Wl_out used below
        NU_A = M_Lay_Wav(1,NWL0) 
        CALL COEF_TO_SCATMATR(NU_A, Nm_vs, USER_MOM(1:NU_A,1,1:Nm_vs,NWL0),&
             19, 'E', DIR_OUT//'kop_hsol.dat')
     ENDIF

     !         NU_A = M_Lay_Wav(1,NWL0)
     !         WRITE(*,*) '   CALL to "COEF_TO_SCATMATR"', M_Lay_Wav(1,NWL0)
     !         CALL COEF_TO_SCATMATR(NU_A, Nm_vs, 
     !     $        USER_MOM(1:NU_A,1,1:Nm_vs,NWL0),
     !     $        19, 'E', DIR_OUT//'test-1.dat')
     !===================================================================
  CASE('Scattering_matrix')
     !  Calculate the expansion coefficients of scattering
     !   function/matrix
     !==================================================================
     MAX_CO = 400 ; N_GAU = MAX_CO
     CALL PH_SCAT_TO_COEFF(MAX_CO, N_GAU, *880)
  END SELECT

  NU_A = M_Lay_Wav(1,NWL0)

  ! Single scattering correction algorithm -> save all expansion coeff.
  ! --------------------------------------    ------------------------
  IF( DO_SSRCA ) THEN
     NG_HS = MAXVAL( M_Lay_Wav )
     IF (LOG_LEVEL(1) >= N_ESO) WRITE(LUNIT,*) '  Allocate Greek_HS :', NG_HS
     IF( ALL_HERE ) ALLOCATE( Greek_HS( NG_HS,LA_N,Nm_vs,NAW,N_HPC ) )
     Greek_HS(:,1,:,NWL0,I_SL(1)) = USER_MOM(1:NG_HS,1,:,NWL0)
     Greek_HS(:,1,:,NWL0,I_SL(2)) = USER_MOM(1:NG_HS,2,:,NWL0)
  ENDIF

  ! Optical thickness at 550 nm for small and large particles 
  ! according to Kopelevich model
  DO K = 1,NMP
     TAU(K) = SUM( IWS(IiW)%HS_COEFF(2,:,NWL0,I_SL(K))*DZ_UW(:) )/2.d0
  ENDDO

  ! ************** Delta-M approach ****************
  ! Allocate array for MODULE DELTA_M
  ! =================================
  IF ( DO_HSOL_DM  ) THEN 
     Itr = HSOL_TRUNC_IDX
     SELECT CASE( Vec_Sca )
     CASE('vect_rte')
        CALL AER_DM_VEC
     CASE('scal_rte')
        CALL AER_DM
     END SELECT

     ! "Renormalize" scattering and extinction coefficients
     ! ====================================================
     DO I = 1, NL_UW        ! Over all levels in ocean
        DO K = 1, 2         ! Over small and large components
           II = I_SL(K)
           IWS(IiW)%HS_COEFF(1,I,NWL0,II) = IWS(IiW)%HS_COEFF(1,I,NWL0,II) &
                *(1.D0 - F_DLM_HSOL(I,NWL0,II))
           IWS(IiW)%HS_COEFF(2,I,NWL0,II) = IWS(IiW)%HS_COEFF(1,I,NWL0,II) 
        ENDDO
     ENDDO
  ELSE
     L = MAXVAL( M_Lay_Wav )
     IF( L > NLEG_W ) THEN
        CALL WRITE_MESSAGE(R_NAME, 'WARNING', &
             'The number of hydrosol expansion moments greater than '&
             //'"Number of Legendre moments"')
        WRITE(EU,'(a46,1x,I4)') &
             '      "Number of Legendre moments"           ', NLEG_W
        WRITE(EU,'(a46,1x,I4)') &
             '      Maximal number of moments after reading', L
     ENDIF
     IWS(IiW)%HSOL_PHAS_VS(:,1,:,NWL0,I_SL(1)) = USER_MOM(1:NLEG_W,1,:,NWL0)
     IWS(IiW)%HSOL_PHAS_VS(:,1,:,NWL0,I_SL(2)) = USER_MOM(1:NLEG_W,2,:,NWL0)
  ENDIF

  CALL WRITE_SCEN_HS 

  CALL ST_TRACK (R_NAME, 1, 'O')
  RETURN

101 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Reading error')
    WRITE(EU,*) '    Parameter to read: ', TRIM(STR_INP)
    GOTO 881

127 CALL WRITE_MESSAGE (R_NAME, 'ERROR', 'Type of hydrosol '//&
         'scattering function parameterization is not recognized')
    GOTO 881
    
880 WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)

881 CONTINUE    
  RETURN 1
CONTAINS
!***************************************************************************
  SUBROUTINE AER_DM
    IF (LOG_LEVEL(1) >= N_ESO) THEN
       WRITE(LUNIT,*) '   *************************  '
       WRITE(LUNIT,*) '   Hydrosol D-M scalar case  '
       WRITE(LUNIT,*) '   *************************  '
    ENDIF
    DO I1 = 1,NMP
       II = I_SL(I1)
       F_DEL = USER_MOM(Itr+1,I1,1,NWL0)/DBLE(2*Itr + 1)
       F_DLM_HSOL(1:NL_UW,NWL0,II) = F_DEL
       !       WRITE(*,*) '   Delta-m HS ', F_DLM_HSOL(1,NWL0,II)
       DO L = 1,Itr
          HS_PHASMOMS(L,1,NWL0,II) = (USER_MOM(L,I1,1,NWL0) - &
               F_DEL*DBLE(2*L-1))/(1.D0 - F_DEL)
       ENDDO                  ! Over moments
       HS_PHASMOMS(Itr+1:NLEG_W,1,:,II ) = 0.D0
    ENDDO                     ! Over small and large particles
  END SUBROUTINE AER_DM
  
!***************************************************************************
  SUBROUTINE AER_DM_VEC
    IF (LOG_LEVEL(1) >= N_ESO) THEN
       WRITE(LUNIT,*) '   ***********************  '
       WRITE(LUNIT,*) '   Hydosol D-M vector case  '
       WRITE(LUNIT,*) '   ***********************  '
    ENDIF
    DO I1 = 1,NMP
       II = I_SL(I1)
       F_DEL = USER_MOM(Itr+1,I1,1,NWL0)/DBLE(2*Itr + 1)
       F_DLM_HSOL(1:NL_UW,NWL0,II) = F_DEL
       DO L = 1, Itr
          IWS(IiW)%HSOL_PHAS_VS(L,1,1:4,NWL0,II) = (USER_MOM(L,I1,1:4,NWL0) - &
               F_DEL*DBLE(2*L-1))/(1.D0 - F_DEL)
          IWS(IiW)%HSOL_PHAS_VS(L,1,5:6,NWL0,II) = &
               USER_MOM(L,I1,5:6,NWL0)/(1.0D0 - F_DEL)
       ENDDO
    ENDDO
    IWS(IiW)%HSOL_PHAS_VS(Itr+1:NLEG_W,1,:,NWL0,:) = 0.0D0
    
  END SUBROUTINE AER_DM_VEC

  !***************************************************************************
  SUBROUTINE WRITE_SCEN_HS
    DOUBLE PRECISION    SS_A, Ext, Sca_coS, Sca_coL, Abs
    IF (LOG_LEVEL(1) >= N_SSO) WRITE (LUNIT,*) ' *** Writing: SCE_KOP-HSOL.OUT'
    
    OPEN (IUNIT, FILE = DIR_OUT//'/SCE_KOP-HSOL.OUT')        
    CALL WRITE_UD(1,IUNIT)
    
    WRITE(IUNIT,'(11x,27A)') ('=', I = 1,27)
    WRITE(IUNIT,'(12x,A)') 'KOPELEVICH HYDROSOL MODEL'
    WRITE(IUNIT,'(11x,27A)') ('=', I = 1,27)
    WRITE(IUNIT,*)
    
    WRITE(IUNIT,*) ' Concentrations of small and large hydrosol particles [ppm],'
    WRITE(IUNIT,'(9x,A)') 'extinction coefficient (Ext coeff) [m-1] and'
    WRITE(IUNIT,'(9x,A)') 'single scattering albedo (Ss albedo):'
    WRITE(IUNIT,'(9x,A)') '(HOT - hydrosol optical thickness)'
    
    K = 1                   ! One reference wavelength only
    WRITE(IUNIT,*) '   '
    WRITE(IUNIT,'(33x,A)') 'Small     Large'
    WRITE(IUNIT,'(4x,A,2(1x,F8.4))') 'Wavelength 550 [nm]  HOT =', TAU(1), TAU(2)
    WRITE(IUNIT,'(4x,50A)') ('-', I = 1,50)
    
    WRITE(IUNIT,'(5x,A, 3x,A, 3x,A, 9x,A, 5x,A)') &
         'Number',' Depth  ','Concentration','Ext coeff','Ss albedo'
    WRITE(IUNIT,'(25x,A,7x,A)') ('Small   Large', I=1,2)
    
    DO I = 1, NL_UW
       Ext = IWS(IiW)%HS_COEFF(2,I,NWL0,I_SL(1)) ; Abs = 0.d0
       Sca_coS = IWS(IiW)%HS_COEFF(1,I,NWL0,I_SL(1))/(1.D0 - F_DLM_HSOL(I,NWL0,I_SL(1)))
       Sca_coL = IWS(IiW)%HS_COEFF(1,I,NWL0,I_SL(2)) !/(1.D0 - F_DLM_HSOL(I,NWL0,1))
       IF( Ext == 0.d0 ) CYCLE
       SS_A = 1.0d0
       
       WRITE(IUNIT,100) I, ZZZ_UW(I), HEL(I,1),HEL(I,2), Sca_coS,Sca_coL, SS_A
    END DO
100 FORMAT(6x,I3, 4x,F7.3, 2x,2(1x,ES9.2), 2x,2(1x,ES9.2), 3x,F4.1)
    
    WRITE(IUNIT,*) ' '
    
    SELECT CASE (TRIM(Ph_type))
    CASE('Expansion_coeff')
       WRITE(IUNIT,*) '    Expansion coefficients used in the hydrosol layers :'
       WRITE(IUNIT,'(5x,46A)') ('-', I=1,46)
       WRITE(IUNIT,*) '    Layer  Top [m]  Bottom [m]   File name'
       DO I = 1,LA_N
          WRITE(IUNIT,'(6x,I3,3x,F6.2,4x,F6.2,6x,a)') &
               I, ZZZ_UW(LA_S(I)) ,ZZZ_UW(LA_E(I)), Names(1)%S
          WRITE(IUNIT,'(34x,A)') Names(2)%S
       ENDDO
   
    CASE('Scattering_matrix')
       WRITE(IUNIT,*) '    Scattering matrix used in the hydrosol layers :'
       WRITE(IUNIT,'(5x,46A)') ('-', I=1,46)
       WRITE(IUNIT,*) '    Layer  Top [m]  Bottom [m]   File name'
       DO I = 1,LA_N
          WRITE(IUNIT,'(6x,I3,3x,F6.2,4x,F6.2,6x,a)') &
               I, ZZZ_UW(LA_S(I)) ,ZZZ_UW(LA_E(I)), Names(1)%S
          WRITE(IUNIT,'(34x,A)') Names(2)%S
       ENDDO
    END SELECT

    WRITE(IUNIT,*) ' '
    WRITE(IUNIT,*) '    Number of expansion coeff. and Delta-M factor :'
    WRITE(IUNIT,'(5x,46A)') ('-', I=1,46)

    WRITE(IUNIT,*) '    Layer  Top [km]  Bottom [km]   N coef   Delta-M'
    DO I = 1,LA_N
       WRITE(IUNIT,101) I, ZZZ_UW(LA_S(I)) ,ZZZ_UW(LA_E(I)), &
            M_Lay_Wav(1,NWL0), F_DLM_HSOL(I,NWL0,I_SL(1)), 'Small'
       WRITE(IUNIT,'(37x, I4, 2x,E10.2, 3x,A)') &
            M_Lay_Wav(2,NWL0), F_DLM_HSOL(I,NWL0,I_SL(2)), 'Large'
    ENDDO
101 FORMAT(6x,I3, 3x,F6.2, 4x,F6.2, 9x,I4, 2x,E10.2, 3x,A)
    
    CALL WRITE_UD(2,IUNIT)
    CLOSE (IUNIT)          
  END SUBROUTINE WRITE_SCEN_HS

  !***************************************************************************
  SUBROUTINE PH_SCAT_TO_COEFF(MAX_CO, N_GAU, *)
    INTEGER  MAX_CO, N_GAU
    LOGICAL  Error, TEST_PH
    INTEGER  Na_st, I, L, N_mom, N_COEFF, KK, INBV
    CHARACTER (LEN=16) OUT_NAME(2)
    REAL (KND), PARAMETER :: Pie = 3.1415926536D0
    REAL (KND)  Norm, S1, Cofp, Cofm, Dum, Clk, Poly(MAX_CO,N_gau), &
         DUM_READ(MAX_CO,2)
    REAL (KND), DIMENSION (N_gau) :: X_gau, AG_gau, X_rad, Phs_gau
    REAL (KND), ALLOCATABLE :: Sa_inp(:), Phs_inp(:), Sa_rad(:), &
         Ph_out(:), Phs_log(:)
    REAL (KND), ALLOCATABLE :: R12(:), R33(:), H11(:), H12(:), H33(:), &
         D_02g(:,:), D_P2g(:,:), D_M2g(:,:), Cf(:), Ph_ret(:,:)
    REAL (KND), ALLOCATABLE ::  WORK(:), B_COEFF(:), S_NODES(:)
! ==========================================================================
    Error = .FALSE.
    ! If test_ph is set to .TRUE., the phase function will be recalculated
    ! using obtained expansion coefficients and written in the given 
    ! output file for the small and large particles
!    TEST_PH = .TRUE.
    TEST_PH = .FALSE.
    IF( TEST_PH ) THEN
       OUT_NAME(1) = 'Ph_kop_small.dat'
       OUT_NAME(2) = 'Ph_kop_large.dat'
    ENDIF

    !... Gauss grid and weights for integration (-1,1)
    CALL GAULEG ( -1.D0, 1.D0, X_gau, AG_gau, N_gau )
    !     ... Legendre polynomials for all Gauss angles
    CALL WIGNER_PLG( X_gau, N_gau, 0, MAX_CO, Poly )
    X_rad(:) = DACOS( X_gau(N_gau:1:-1) )

    !.. Reading of the user scattering function/matrix 
    !   for small and large particles
    DO I1 = 1,NMP
!      =======>  Reading phase functions of Kopelevich model
       OPEN(C_UNIT, FILE=Names(I1)%S, STATUS='OLD') 
       CALL SKIP_HEADER (C_UNIT)           
       DO I = 1,MAX_CO
          READ(C_UNIT,*,END=128,ERR=129) (DUM_READ(I,J), J=1,2)
       END DO
129    CALL WRITE_MESSAGE (R_NAME, 'ERROR', 'Error reading data file !')
       WRITE(EU,*) '    File name : ', Names(I1)%S
       RETURN 1             
128    CONTINUE
       Na_st = I - 1
       CLOSE(C_UNIT)
!      <======= End of reading        
!       WRITE(*,*) '  Number of scattering angles:', Na_st
!       WRITE(*,*) '  First :', DUM_READ(1,1:2)
!       WRITE(*,*) '  Last  :', DUM_READ(Na_st,1:2)
       
       ALLOCATE(Sa_inp(Na_st),Sa_rad(Na_st),Phs_inp(Na_st), &
            Phs_log(Na_st)) ! A

       ALLOCATE( WORK(5*(Na_st+2)),B_COEFF(Na_st+2),S_NODES(Na_st+6) )

       ! Original scattering function 
       Sa_inp(:)  = DUM_READ(1:Na_st,1)
       Phs_inp(:) = DUM_READ(1:Na_st,2)            
       
       ! Convert [deg] to [rad] for interpolation of Ph 
       Sa_rad(:) = Sa_inp(:)/RTD
       
       ! Interpolate scattering function on the Gauss grid
!       WRITE(*,*) '  Sa_rad :', sa_rad(1),' - ',Sa_rad(Na_st)
!       WRITE(*,*) '   X_rad :',  x_rad(1),' - ', x_rad(N_gau)
       Phs_log(:) = DLOG( Phs_inp(:) )

!       CALL AKIMA_INTERP(Na_st, Sa_rad, Phs_log, N_gau, X_rad, Phs_gau, Error)
!       CALL AL_INTERP(Na_st, Sa_rad, Phs_log,      &
!            N_gau, X_rad, Phs_gau, .FALSE., *880)  
       CALL DBINT4 (Sa_rad, Phs_log, Na_st, 2, 2, 0.d0, 0.d0, 1, S_NODES, &
            B_COEFF, N_COEFF, KK, WORK, *880) 
       INBV = 1
       DO I = 1, N_gau
          CALL DBVALU( S_NODES, B_COEFF, N_COEFF, KK, 0,  &
               X_rad(I), INBV, WORK(1:12), Phs_gau(I), *880 )
       END DO
      
       IF( Error ) THEN
          Call WRITE_MESSAGE (R_NAME,'ERROR', 'Interpolation error')
          RETURN 1
       ENDIF
       ! Invert to start from -1 phase function interpolated on Gauss grid
       Phs_gau(:) = EXP( Phs_gau(N_gau:1:-1) )
       
       ! Norm of the interpolated phase function
       Norm = SUM( Phs_gau(:)*AG_gau(:) )*0.5d0
       
       IF (LOG_LEVEL(1) >= N_ESO) THEN
          S1 = 0.d0
          DO I = 1, Na_st - 1
             S1 = S1 + (Phs_inp(I)*DSIN(Sa_rad(I)) &
                  + Phs_inp(I+1)*DSIN(Sa_rad(I+1)))/2.d0 &
                  *(Sa_rad(I+1) - Sa_rad(I))
          ENDDO
          WRITE(LUNIT,*) '   Integral of the phase function '
          WRITE(LUNIT,*) '          Initial scat-angle grid:', S1*2.*Pie
          WRITE(LUNIT,*) '   Integral of the phase function '
          WRITE(LUNIT,*) '         Extended scat-angle grid:', Norm*4.*Pie
          ! Number of expansion coefficients of intrpolated Ph
!          WRITE(*,*) '   MAX_CO = ', MAX_CO
       ENDIF
!      ... Expansion of the interpolated phase function into Legendre series
       DO L = 1,MAX_CO
          TMP1 = (DBLE(L) - 0.5d0)*SUM( Phs_gau(:)*Poly(L,:)*AG_gau(:) )/Norm
          IF( ABS(TMP1) < 1.d-06 ) THEN 
!             IF( TMP1 < 0.d0 ) THEN 
!                WRITE(LUNIT,*) '   Negative expansion coefficient'
!                WRITE(LUNIT,*) '   for L = ', L, TMP1
!                WRITE(LUNIT,*) '   Number of Gauss angles ',N_GAU
!                WRITE(LUNIT,*) '   Continue?'
!                EXIT
!             ENDIF
          ENDIF
          USER_MOM(L,I1,1,NWL0) = TMP1
       ENDDO
       M_Lay_Wav(I1,NWL0) = L-1
!       WRITE(*,*) '  M_Lay_Wav ', I1, M_Lay_Wav(I1,NWL0)

       IF( TEST_PH ) THEN
          ALLOCATE( Ph_out(N_gau) )
          DO J = 1,N_gau
             Ph_out(J) = SUM(Poly(1:L-1,J)*USER_MOM(1:L-1,I1,1,NWL0))
          ENDDO
          OPEN(OUNIT,FILE=DIR_OUT//'/'//OUT_NAME(I1))
          WRITE(OUNIT,'(a)') '#  Interpolated original and '//&
               'recalculated phase functions'
          WRITE(OUNIT,'(a,2x,I3)') '#  Hydrosol layer :', I1
          WRITE(OUNIT,'(a,2x,I3)') '#  Wavelength    :', 1
          WRITE(OUNIT,'(a,2x,E12.4)')  '#  Norm of interpolated Ph', Norm
          WRITE(OUNIT,'(a,2x,I5)') '#  Number of expansion coefficients used',&
               M_Lay_Wav(I1,NWL0)
          DO J = 1,N_gau
             WRITE(OUNIT,'(1x,3(e15.7))') DACOS(X_gau(J))*RTD, &
                  Phs_gau(J), Ph_out(J)*Norm
          ENDDO
          CLOSE(OUNIT)
          DEALLOCATE( Ph_out )
       ENDIF  ! TEST_PH 
       IF (LOG_LEVEL(1) >= N_ESO) &
            WRITE(LUNIT,*) '   Number of expansion coeff: ', L-1
       DEALLOCATE( Sa_inp, Sa_rad, Phs_inp, Phs_log, &
            WORK, B_COEFF, S_NODES)
    ENDDO  ! small and large particles in Kopelevich model (I1=1:2)

!   ... Calculate other elements of scattering matrix (approximation)
    IF( Nvs > 1 ) THEN
       ALLOCATE( Ph_ret(N_gau, Nm_vs) ) 
       ALLOCATE( R12(N_gau), R33(N_gau),  H11(N_gau), H12(N_gau), &
            H33(N_gau), D_02g(Max_co,N_gau), D_P2g(Max_co,N_gau), & 
            D_M2g(Max_co,N_gau), Cf(Nm_vs) )
       CALL WIGNER_PLG     ( X_gau, N_gau, 2, Max_co, D_02g )
       CALL WIGNER_FUNCTION( X_gau, N_gau, 2, Max_co, D_P2g, D_M2g )
       R12(:) = -(1.d0 - X_gau(:)**2)/(1.d0 + X_gau(:)**2)
       R33(:) =  2.d0*X_gau(:)/(1.d0 + X_gau(:)**2)
       DO I1 = 1, NMP
         N_mom = M_Lay_Wav(I1,NWL0)
          DO J = 1, N_gau
             H11(J) = SUM( Poly(1:N_mom,J)*USER_MOM(1:N_mom,I1,1,NWL0) )
          ENDDO
          H12(:) = H11(:)*R12(:) 
          H33(:) = H11(:)*R33(:) 
          DO L = 1, N_mom
             Clk = DBLE(L) - 0.5d0
!             Cf(1) =  Clk*SUM( H11(:)*Poly(L,:)*AG_gau(:) )
             Cf(4) =  SUM( H33(:)*Poly(L,:)*AG_gau(:) )
             Cf(5) = -SUM( H12(:)*D_02g(L,:)*AG_gau(:) )
             Cf(6) = 0.d0
             Cofm = SUM( (H11(:) - H33(:))*D_M2g(L,:)*AG_gau(:) )
             Cofp = SUM( (H11(:) + H33(:))*D_P2g(L,:)*AG_gau(:) )
             Cf(2) = (Cofp + Cofm)/2.d0
             Cf(3) = (Cofp - Cofm)/2.d0
             Cf(2:Nm_vs) = Clk*Cf(2:Nm_vs)
             Cf(5:6) = -Cf(5:6)
             USER_MOM(L,I1,2:Nm_vs,NWL0) = Cf(2:Nm_vs)
          ENDDO  ! over expansion coefficients (L)
!! =======> Check Kopelevich expansion coefficients 
!          Ph_ret = 0.d0
!          DO L = 1, N_mom
!             Cf(:) = USER_MOM(L,I1,:,NWL0)
!             Ph_ret(:,1) = Ph_ret(:,1) + Poly(L,:) *Cf(1)
!             Ph_ret(:,4) = Ph_ret(:,4) + Poly(L,:) *Cf(4)
!             Ph_ret(:,5) = Ph_ret(:,5) - D_02g(L,:) *Cf(5)
!             Ph_ret(:,6) = Ph_ret(:,6) - D_02g(L,:) *Cf(6)
!             Cofp = Cf(2) + Cf(3)  ;  Cofm = Cf(2) - Cf(3)
!             Ph_ret(:,2) = Ph_ret(:,2) &
!                  + (D_P2g(L,:)*Cofp + D_M2g(L,:)*Cofm)/2.d0
!             Ph_ret(:,3) = Ph_ret(:,3) &
!                  + (D_P2g(L,:)*Cofp - D_M2g(L,:)*Cofm)/2.d0
!          ENDDO
!          OPEN( 47,FILE='./DATA_OUT/Orig-'//OUT_NAME(I1) )
!          DO J = 1,N_gau
!             Dum = ACOS( X_gau(J) )/Pie*180.
!             WRITE(47,'(1x,*(e15.7))') Dum, &
!                  H11(J), H11(J), H33(J), H33(J), H12(J)
!          ENDDO
!          CLOSE(OUNIT)
!          OPEN( 47,FILE='./DATA_OUT/Ret-'//OUT_NAME(I1) )
!          DO J = 1,N_gau
!             Dum = ACOS( X_gau(J) )/Pie*180.
!             WRITE(47,'(1x,*(e15.7))') Dum, Ph_ret(J,1), &
!                  Ph_ret(J,2), Ph_ret(J,3), Ph_ret(J,4), Ph_ret(J,5)
!          ENDDO
!          CLOSE(OUNIT)
!! <=======  End check
       ENDDO  ! over large and small Kopelevich phase function
    ENDIF    
!    STOP '  stop in kop_hsol_setup.f '

    RETURN
880 CONTINUE    
    RETURN 1
    
  END SUBROUTINE PH_SCAT_TO_COEFF

  !***************************************************************************
  SUBROUTINE OUT_COEFF
    OPEN(OUNIT,FILE=DIR_OUT//'/H_G_coeff.out')
    WRITE(OUNIT,'(a)') '#  Expansion coefficients of the H-G phase function'
    WRITE(OUNIT,'(a)') '#  in the lower hydrosol layer at first wavelength. '
    WRITE(OUNIT,'(a,2x,F6.3)') '#  Asymmetry parameter :', ASYM_PAR(1,1)
    DO L = 1,NU_A
       WRITE(OUNIT,'(2x,I5,2x,E15.7)') L-1, USER_MOM(L,1,1,NWL0)
    ENDDO
    CLOSE(OUNIT)
  END SUBROUTINE OUT_COEFF

!***************************************************************************
END SUBROUTINE KOP_HSOL_SETUP
!***************************************************************************

