!**************************************************************************
!
!	SCIATRAN - Version 4.7 - Last modification: 08 Aug 2023
!
!	Copyright (C) - Institute of Remote Sensing,
!			University of Bremen, Germany
!
!****************************************************************************
Subroutine VOIGT_SHAPE (ISTART, IEND, IdxMol, POS_WN, IA, ITS, TEMP, PRESS, &
     ABS_COEFF, *, UseLineInp)

  ! CALCULATES ABSORPTION COEFFICIENT ABS_COEFF [CM2/MOLECULE] FOR GIVEN
  ! PRESS(URE) [MBAR] AND TEMP(ERATURE) [K] AND WAVELENGTH [NM]
  ! FROM GIVEN LINE PARAMETERS:
  !
  !  * POSITION [CM-1] /TG_INFO%LinePars%Pos/
  !  * INTENSITY [CM-1/(MOLECULE.CM-2)] @ 296 K /TG_INFO%LinePars%Int/
  !  * AIR BROADENING (LORENTZ) WIDTH [CM-1], 
  !    HWHM @ 296 K AND 1013.25 MBAR /TG_INFO(II)%LinePars%LorW/
  !  * LOWEST STATE ENERGY [CM-1] /TG_INFO%LinePars%LSE/
  !  * COEFF. OF T-DEP. OF AIR BROADENED HALFWIDTH (N) [-] /TG_INFO%LinePars%TExp/
  !  * DOPPLER WIDTH [CM-1], 1/E HALF-WIDTH @ 296 K /TG_INFO%LinePars%DopW/
  !  * LINE SHIFT [CM-1] /TG_INFO%LinePars%Shift/

  Use INFO,    Only: TG_INFO
  Use LBL,     Only: HT_C2, StEm_MaxWn, OptLineSum, LineSumAcc
  Use ST_VARS, Only: PIE
  Use TYPES,   Only: KND

  Character (LEN=*), PARAMETER :: R_NAME = 'VOIGT_SHAPE'
  Integer IdxMol, ISTART, IEND, IA, ITS, II
  Logical StoreAC
  Logical, Target :: UseLineInp(IEND - ISTART + 1)
  Logical, Pointer :: UseLine(:)
  Real (Knd) ABS_COEFF, TEMP, PRESS, POS_WN, XGAU, XLOR, AC, VOIGTA, &
       VOIGTX, XINT, XPOS, SQRT_PI, GI
  Real (Knd), Allocatable :: AC_Arr(:)
  COMPLEX(8), External :: VGT
  Optional :: UseLineInp
  !==================================================================

  ABS_COEFF = 0.0D0
  SQRT_PI = DSQRT(PIE)

  If(Present(UseLineInp)) Then
     UseLine => UseLineInp
  Else
     Allocate(UseLine(IEND - ISTART + 1))
     UseLine = .True.
  End If
  If(OptLineSum .and. All(UseLine)) Then
     Allocate(AC_Arr(IEND - ISTART + 1))
     StoreAC = .True.
  Else
     StoreAC = .False.
  End If
  
  ! Loop over all selected lines
  ! ----------------------------
  Do II = ISTART, IEND
     If(.Not. UseLine(II - ISTART + 1)) Cycle
     Associate(Pars => TG_INFO(IdxMol)%LinePars(II)) 
      
       ! Pressure is to be converted to atm
       XLOR   = Pars%LorW*(Press/1013.25D0)*(296.0D0/Temp)**Pars%TExp
       XGAU   = Pars%DopW*Sqrt(Temp)

       XINT = Pars%Int*TG_INFO(IdxMol)%PartFuncR(Pars%Iso,IA,ITS)* &
            Exp( Pars%LSE*HT_C2*( 1.0D0/296.0D0 - 1.0D0/Temp ) )
       ! Accounting for the stimulated emission at small wavenumbers
       If (Pars%Pos > StEm_MaxWn) &
            XINT = XINT*(1.0D0 - Exp(-HT_C2*Pars%Pos/Temp))
       XPOS = Pars%Pos + Pars%Shift*PRESS/1013.25D0
       ! Rybicki, SAO:
       VOIGTA  = XLOR/XGAU
       VOIGTX  = ABS(POS_WN - XPOS)/XGAU
       ! VOIGT2:
       ! ABS_COEFF = ABS_COEFF + XINT*VOIGT2(VOIGTX,VOIGTA)/XGAU
       ! HUMLICEK:
       AC = XINT*REAL(VGT(VOIGTA,VOIGTX))/XGAU/SQRT_PI
       If(StoreAC) AC_Arr(II - ISTART + 1) = AC
       ABS_COEFF = ABS_COEFF + AC
     End Associate
  End Do
  If(StoreAC.AND.ANY(UseLine)) Then
     Do
        II = MinLoc(AC_Arr,1,UseLine)
        UseLine(II) = .False.
        If(All(.Not.UseLine)) Exit
        If(Abs(Sum(AC_Arr,UseLine)/Sum(AC_Arr) - 1.) > LineSumAcc) Exit
     End Do
     UseLine(II) = .True.
     DeAllocate(AC_Arr)
  End If
        
  Return
!****************************************************************************
End Subroutine VOIGT_SHAPE
!****************************************************************************
Subroutine ST_PART_SUMS (*)

  USE GRIDS,    ONLY: NLAYERS, ZZZ
  USE INFO,     ONLY: TG_INFO, WF_DB
  USE LBL,      ONLY: N_TEMP_SETS, PT_SCALE 
  USE LOG_PARS, ONLY: EU
  USE PROFILES, ONLY: TEMP
  USE TOOLS,    ONLY: INQ_WF
  Use TYPES,    Only: KND

  Character (Len=*), Parameter :: R_NAME = 'ST_PART_SUMS'
  Integer  IdxIso, JJ, KK, II
  Real (Knd)     GI, Q_REF, Q_ACT, TTT
  Real (Knd), Allocatable :: FACTOR(:)
  !==================================================================

  Call ST_TRACK (R_NAME, 1, 'I')

  IF (INQ_WF('temp', 'RIN')) THEN
     N_TEMP_SETS = 2
     ALLOCATE (FACTOR(N_TEMP_SETS))
     FACTOR(:) = (/1.0D0, PT_SCALE/)
  ELSE
     N_TEMP_SETS = 1
     ALLOCATE (FACTOR(N_TEMP_SETS))
     FACTOR(:) = 1
  END IF

  Do II = 1, Size(TG_INFO)
     Associate(TGI => TG_INFO(II))
       If(.Not.Allocated(TGI%LinePars)) Cycle
       If(Allocated(TGI%PartFuncR)) DeAllocate(TGI%PartFuncR)
       Allocate(TGI%PartFuncR(TGI%NIso, NLAYERS, N_TEMP_SETS))
       TGI%PartFuncR = 0.0D0
       Do IdxIso = 1, TGI%NIso
          Call BD_TIPS_2020(TGI%IdxHT, 296.0D0, IdxIso, GI, Q_REF) 
          Do JJ = 1, NLAYERS
             DO KK = 1, N_TEMP_SETS
                TTT = TEMP(JJ)*FACTOR(KK)
                Call BD_TIPS_2020(TGI%IdxHT, TTT, IdxIso, GI, Q_ACT)
                If (Q_ACT.GT.0.0D0) Then
                   TGI%PartFuncR(IdxIso,JJ,KK) = Q_REF/Q_ACT
                Else
                   CALL ERROR_REPORT
                   TGI%PartFuncR(IdxIso,JJ,KK) = 1.0D0
                End If
             End DO
          End Do
       End Do
     End Associate
  End Do
  CALL ST_TRACK (R_NAME, 1, 'O')
  Return
      
Contains
!****************************************************************************
  Subroutine ERROR_REPORT
    CALL WRITE_MESSAGE (R_NAME, 'WARNING', 'Division by zero in partition function ratio!')
    WRITE(EU,*) '    Molecule and isotope number: ', TG_INFO(II)%NameUC, IdxIso
    WRITE(EU,*) '    Layer number and altutude: ', JJ, ZZZ(JJ)
    WRITE(EU,*) '    Q_REF Q_ACT: ', Q_REF, Q_ACT
    WRITE(EU,'(A,2(2X,I0),2X,F7.2)') '     HITRAN_GAS_ID, ISO_ID, TEMP: ',&
         TG_INFO(II)%IdxHT, IdxIso, TEMP(JJ)
    WRITE(EU,*) '    partition function ratio is set 1.0'
   END SUBROUTINE ERROR_REPORT
!****************************************************************************
 END Subroutine ST_PART_SUMS
!****************************************************************************
 Subroutine LINE_SEARCH (IdxMol, POS_WN, LINES_FOUND, ISTART, IEND)

   Use Info,  Only: TG_INFO
   Use LBL,   Only: L_W_C => LBL_WING_CUTOFF
   Use Types, Only: Knd
   Logical  LINES_FOUND
   Integer  IdxMol, ISTART, IEND
   Real (Knd) POS_WN, WN_DELTA
   Real (Knd), POINTER :: WL_LBL(:)
  !==================================================================
   LINES_FOUND = .False.

   Associate(WN => TG_INFO(IdxMol)%LinePars%Pos)
     ! All lines too far "on the left hand side" ?
     IF(WN(Size(WN)) < POS_WN - L_W_C) Return    
     ! All lines too far "on the right hand side" ?
     IF (WN(1) > POS_WN + L_W_C) Return
     ! Determine first and last index of relevant lines
     IEND = COUNT(WN(:) <= POS_WN + L_W_C)
     ISTART = COUNT(WN(1:IEND) < POS_WN - L_W_C) + 1
     LINES_FOUND = .TRUE.
   End Associate
   
   Return
!****************************************************************************
 End Subroutine LINE_SEARCH
!****************************************************************************
  COMPLEX(8) FUNCTION VGT(Y1,X1)
!
! COMPLEX PROBABILITY FUNCTION FOR COMPLEX ARGUMENTS Z=X+IY
! REAL PART: VOIGT FUNCTION K(X,Y)
!
! SOURCE: J. HUMLICEK, JQSRT, 27, 437, 1982
!
! PARAMETERS: 
!             X      VOIGT FREQUENCY PARAMETER
!             Y      VOIGT BROADENING PARAMETER
!             PRBFCT VALUE OF PROBABILITY FUNCTION
!             VOIGT  REAL PART OF PRBFCT
    REAL X,Y,S, ax !mb
    REAL*8 X1,Y1
    COMPLEX T,U,PRBFCT
    COMPLEX APPROX1,APPROX2,APPROX3,APPROX4
    APPROX1(T)=(T*0.5641896)/(0.5+(T*T))
    APPROX2(T,U)=(T*(1.410474+U*0.5641896))/(0.75+(U*(3.+U)))
    APPROX3(T)=(16.4955+T*(20.20933+T*(11.96482+ T*(3.778987+0.5642236*T))))&
         /(16.4955+T*(38.82363+T*(39.27121+T*(21.69274+T*(6.699398+T)))))
    APPROX4(T,U)=(T*(36183.31-U*(3321.99-U*(1540.787-U &
         *(219.031-U*(35.7668-U*(1.320522-U*0.56419)))))) &
         /(32066.6-U*(24322.8-U*(9022.23-U*(2186.18 &
         -U*(364.219-U*(61.5704-U*(1.84144-U))))))))

    X=SNGL(X1)
    Y=SNGL(Y1)
    IF(Y.GT.15.) THEN
       ! REGION I
       T=CMPLX(Y,-X)
       PRBFCT=APPROX1(T)
    ELSEIF(Y.LT.15. .AND. Y.GE.5.5) THEN
       ! REGION I OR II
       T=CMPLX(Y,-X)
       S=ABS(X)+Y
       IF(S.GE.15.)THEN
          PRBFCT=APPROX1(T)
       ELSE
          U=T*T
          PRBFCT=APPROX2(T,U)
       ENDIF
    ELSEIF(Y.LT.5.5 .AND. Y.GT.0.75) THEN
       T=CMPLX(Y,-X)
       S=ABS(X)+Y
       IF(S.GE.15)THEN
          PRBFCT=APPROX1(T)
       ELSEIF(S.LT.5.5)THEN
          PRBFCT=APPROX3(T)
       ELSE
          U=T*T
          PRBFCT=APPROX2(T,U)
       ENDIF
    ELSE
       T=CMPLX(Y,-X)
       AX=ABS(X)
       S=AX+Y
       IF(S.GE.15.)THEN
          PRBFCT=APPROX1(T)
       ELSEIF(S.LT.15. .AND. S.GE.5.5)THEN
          U=T*T
          PRBFCT=APPROX2(T,U)
       ELSEIF(S.LT.5.5 .AND. Y.GE.(0.195*AX-0.176))THEN
          PRBFCT=APPROX3(T)
       ELSE
          U=T*T
          PRBFCT=CEXP(U)-APPROX4(T,U)
       ENDIF
    ENDIF
    VGT=CMPLX(PRBFCT)
    RETURN
!****************************************************************************
  END FUNCTION VGT
!****************************************************************************
  DOUBLE PRECISION FUNCTION VOIGT2 (X, A)

    ! PROGRAM WRITTEN BY G. RYBICKI, SMITHSONIAN ASTROPHYSICAL OBSERVATORY
    ! COMPUTES VOIGT FUNCTION FOR ANY VALUE OF X AND ANY POSITIVE VALUE OF A.
    ! KILROY IS AN INITIALIZATION SWITCH WHOSE EFFECT IS INTENDED TO BE THAT
    ! THE ARRAY C IS CALCULATED ONLY ONCE, NAMELY THE FIRST TIME VOIGT2 IS
    ! CALLED AFTER BEING LOADED INTO MEMORY.
    ! NOTE:  X  =  (SIGMA - SIGMA0) / XG, WHERE XG  =  4.301E-7 * SIGMA0 *
    ! SQRT (T/M); A  =  (LORENTZ HWHM)/XG .  THIS ROUTINE GIVES A VOIGT
    ! PROFILE NORMALIZED TO AREA XG (IN THE DOPPLER LIMIT THE LINE CENTER
    ! IS ALWAYS 1/SQRT(PI)).

    USE LBL, ONLY : C
    ! IMPLICIT REAL*8 (A - H, O - Z)
	
    implicit none
    double precision b1, b2, zr, zi, s, t
    double precision a, x, q1, a2, e, q2, a1
    integer n

    DATA Q1, Q2 / 9.42477796076938D0, .564189583547756D0/

    IF (A .EQ. 0.D0) GO TO 106

    ! CALCULATION FOR THE GENERAL CASE

    A1 = 3.D0 * A
    A2 = A * A
    E = DEXP (-Q1 * A)
    IF (A .LT. 0.1D0) GO TO 101
    ZR = E * DCOS (Q1 * X)
    ZI = E * DSIN (Q1 * X)
    VOIGT2 = 0.D0
    GO TO 102
101 ZR = .5D0 * (E + 1.D0 / E) * DCOS (Q1 * X)
    ZI = .5D0 * (E - 1.D0 / E) * DSIN (Q1 * X)
    ! Modification to avoid underflow problem
    ! Michael Buchwitz, 23 Sept 97
    if ( (A2 - X * X) .lt. -1.0d2 ) then
       voigt2 = 0.0d0
    else
       VOIGT2 = Q2 * DEXP (A2 - X * X) * DCOS (2.D0 * A * X)
    endif
102 B1 = (1.D0 - ZR) * A * 1.5D0
    B2 = -ZI
    S = -8.D0 - 1.5D0 * X
    T = S * S + 2.25D0 * A2
    DO 104 N = 1, 31
       T = T + S + 0.25D0
       S = S + 0.5D0
       B1 = A1 - B1
       B2 = -B2
       IF (T .GT. 2.5D-12) GO TO 103
       VOIGT2 = VOIGT2 - C (N) * A * 29.608813203268D0
       GO TO 104
103    VOIGT2 = VOIGT2 + C (N) * (B1 + B2 * S) / T
104    CONTINUE
105    RETURN
       
       ! CALCULATION FOR A = 0

106    VOIGT2 = Q2 * DEXP (-X * X)
       GO TO 105
       
!****************************************************************************
  End FUNCTION VOIGT2
!****************************************************************************




