!***************************************************************************
!
!	SCIATRAN - Version 3.8 - Last modification: 3 Aug 2017
!
!	Copyright (C) - Institute of Remote Sensing,
!			University of Bremen, Germany
!
!***************************************************************************
SUBROUTINE LT_AER_SETUP( START_WAV, END_WAV,* )
  ! Called by "rtm_init.f"
                              
  USE AER_MIE,     ONLY: DENS_PRF
  USE AEROSOLS,    ONLY: AER_SHIFT, PATH_TO_SCM, OUT_A_SCM, Plus_mi, LAY_A, &
                         WEL_A, NSCA_A, GE_A, APH_VS, Nga_db, NAW => N_AER_WAVS, &
                         NG_aer, Greek_Aer, Scal_aer, Rtau_aer, Rwl_aer, AER_SCALING,& 
                         AER_WAVS, AER_COEFF, NWLS, NWLE, SCI => GT_SCAT_IDX, &
                         ASY => GT_ASYM_IDX, EXI => GT_EXTC_IDX, AER_PARAM, &
                         COMPAT_MODE, LA_N, LA_S, LA_E, L_TOP, IDX_AER, LA_Ndb,&
                         AOT_out_VAL, IDX_AER_db, L_scal, ZS1, ZS2, AER_FACT, V_PAP
  USE CLOUDS_SS,   ONLY: MUM => MAXMOM, DO_SSRCA
  USE DELTA_M,     ONLY: FDM_AER, AER_DM_FLG, AER_TRUNC_IDX
  USE GRIDS,       ONLY: NLAYERS, ZHOLD => ZZZ, DZ
  USE GT_POLYN,    ONLY: NLEGEN
  USE INFO,        ONLY: AER_INFO
  USE INTERN_DIMS, ONLY: Nm_vs
  USE LOG_PARS,    ONLY: LUNIT, EU, DIR_OUT, LOG_LEVEL, N_SSO, N_ESO, N_DSO, Work_Dir
  USE RTM_PARS,    ONLY: DATA_ITER
  USE ST_INPUT,    ONLY: ADB => PATH_AER_DB, APH_TYPE
  USE ST_VARS,     ONLY: RTM_TYPE, GROUND_HEIGHT
  USE TOOLS,       ONLY: CHK_STAT, SET_PTR, READ_INPUT, OpenOut, WriteOut, CloseOut
  USE TYPES,       ONLY: KND, DYN_CHAR

  ! (a) Input variables
  !     ===============
  !     APH_TYPE          : type of phase function type
  !                         'none', 'mie', 'h-g' - Henyey-Greenstein 
  !     AER_DM_FLG        : delta-M approximation yes/no 
  !                         <---- From Module DELTA_M
  !     AER_TRUNC_IDX     : truncation index for delta-M approximation 
  !                         <---- From Module DELTA_M
  !     ZHOLD             : forward model altitude grid
  !     NLAYERS           : number of layers in altitude grid
  !     NLEGEN            : number of Legendre moments
  !     LUNIT             : LOG file unit number

  ! (b) Output variables
  !     ================
  !     AER_COEFF         : scattering / extinction coefficients and 
  !                         asymmetry parameters 
  !     AER_PHASMOMS      : Legendre moments
  !     NAW        : number of wavelengths of chosen aerosol 
  !                         parameterization
  !     AER_WAVS          : wavelengths of aerosol data base (depends on
  !                         parameterization chosen)
  REAL (KND), INTENT(IN) :: START_WAV, END_WAV
  LOGICAL               ERROR
  CHARACTER (LEN=*), PARAMETER :: R_NAME = 'LT_AER_SETUP'
  INTEGER    NWL_DB
  INTEGER, ALLOCATABLE :: M_Lay_Wav(:,:)
  REAL (KND) GG, TMP1, DUM, Scoe, DD1, DD2, SAFI(NLAYERS)
  REAL (KND), ALLOCATABLE :: DB_WAVS(:), TAU(:), Top_LT(:), ASYM_PAR(:,:), &
       USER_MOM(:,:,:,:)
  REAL (KND), POINTER :: AER_PHASMOMS(:,:,:)       

  ! (d) Local variables for LOWTRAN parameterization
  !     ============================================
  !     SEASON            : season index
  !     BL_VIS            : boundary layer visibility index
  !     BL_HUM            : boundary layer humidity index
  !     BL_TYPE           : boundary layer aerosol type index
  !     TR_VIS            : tropospheric visibility index
  !     TR_HUM            : tropospheric humidity index
  !     STR_LOAD          : stratospheric aerosol load index
  !     STR_TYPE          : stratospheric aerosol type index
  !     MES_LOAD          : mesospheric aerosol load index
  !     W1                : reference extinction profile
  !     SAFI              : reference extinction profile interpolated
  !                         to the altitude grid ZHOLD       
  INTEGER   SEASON, BL_VIS, BL_HUM, BL_TYPE, I_L, LA_LT, TR_VIS,&
       TR_HUM, STR_LOAD, STR_TYPE, MES_LOAD, N_SKIP, LAN
  INTEGER   I, I1, K, J, K1, K2, L, Itr, IUNIT, OUNIT, Ke, Ka, Ks
  INTEGER   SH(4), PC(4)
  INTEGER, ALLOCATABLE :: LAS(:), LAE(:), IDA(:)
  LOGICAL   TEST_MODE, REM_LAYER(4)
  CHARACTER (LEN=12)    SNA(4)
  CHARACTER (LEN=80)    TXT(4)
  CHARACTER (LEN=150)   INP_NAME(4)
  CHARACTER (LEN=:), ALLOCATABLE :: STR_INP
  REAL (KND) F_DM, W1(34), HRD(12), LT_Z(34)
  PARAMETER (IUNIT = 33, OUNIT = 55)
  ! Test arrays
  INTEGER   LA_ACT
  REAL (KND) TOP_ACT(4)
  TYPE (DATA_ITER), POINTER :: RTM_PTR
  TYPE (DYN_CHAR), ALLOCATABLE :: STR_ARR(:), READ_ARR(:)
  !=========================================================================
  
  !===================================================
  !             LOWTRAN PARAMETERIZATION
  !===================================================
  CALL ST_TRACK (R_NAME, 1, 'I')  

  CALL DEALLOCATE_ARRAYS ! All arrays allocated below must be 
                         ! conditionally deallocated here

  IF(CHK_STAT('aer-dens',AER_INFO%NAME,AER_INFO%WF_FLG) ) THEN
     ALLOCATE( DENS_PRF(NLAYERS) )
     DENS_PRF(:) = 1.d0
     CALL SET_PTR( RTM_PTR, 'aer-dens' )
     RTM_PTR%APR(:) = DENS_PRF(:)
     RTM_PTR%CRN(:) = DENS_PRF(:)    
  ENDIF

  ! General setting
  ! -----------------
  ERROR   = .FALSE.

  ! Standard number of aerosol layers in LOWTRAN data base
  ! ------------------------------------------------------
  LA_LT = 4
  ALLOCATE( TOP_LT(LA_LT), L_TOP(LA_LT) ) ! A

  ! Standard top altitudes used in LOTRAN parameterization
  ! ------------------------------------------------------
  Top_LT(1) = 100.d0
  Top_LT(2) = 30.d0 ; Top_LT(3) = 10.d0 ; Top_LT(4) = 2.d0

  IF( AER_SHIFT ) Top_LT(:) = Top_LT(:) + GROUND_HEIGHT
  L_TOP(:) = Top_LT(:)

  ! Actual number of aerosol layers
  REM_LAYER(:) = .FALSE.
  LA_ACT = LA_LT ; DD1 = ZHOLD(1) ; I_L = 1
  DO K = 1,3
     IF( Top_LT(K) >= DD1 ) THEN
        IF( Top_LT(K+1) >= DD1 ) THEN
           ! Decrease number of aerosol layers
           LA_ACT = LA_ACT - 1 
           IF(.NOT.COMPAT_MODE) REM_LAYER(K) = .TRUE.               
        ELSE
           Top_ACT(I_L) = DD1 ; I_L = I_L + 1
        ENDIF
     ELSE
        Top_ACT(I_L) = TOP_LT(K) ; I_L = I_L + 1
     ENDIF
  ENDDO
  Top_ACT(LA_ACT) = TOP_LT(4)

  DD1 = ZHOLD(NLAYERS) ; I_L = LA_LT
  DO K = LA_ACT, 2, -1
     IF( Top_ACT(K) <= DD1 .AND..NOT.COMPAT_MODE) THEN
        LA_ACT = LA_ACT - 1  ;  REM_LAYER(I_L) = .TRUE.
        Top_LT(K) = Top_ACT(K-1) ; I_L = I_L - 1
     ELSE
        Top_LT(K) = TOP_ACT(K)
     ENDIF
  ENDDO
  Top_LT(1) = TOP_ACT(1)

  LAN = LA_ACT
  ALLOCATE(LAS(LAN), LAE(LAN)) ! A
  DO K = 1, LAN
     LAS(K) = MINLOC( DABS(Top_LT(K)-ZHOLD(:)),1 )
  ENDDO
  IF( LAN > 1 ) LAE(1:LAN-1) = LAS(2:LAN) - 1
  LAE(LAN) = NLAYERS

  ! Reading wavelength grid for LOWTRAN parameterization
  ! ----------------------------------------------------
  NWL_DB = 18
  ALLOCATE( DB_WAVS(NWL_DB) )
  OPEN(IUNIT,FILE = TRIM(ADB)//'DB_LT_WL.dat',STATUS='OLD')
  DO I = 1,11
     READ(IUNIT,*)
  ENDDO
  DO I = 1,NWL_DB
     READ(IUNIT,*) DB_WAVS(I)
  ENDDO
  CLOSE(IUNIT)
  IF( DB_WAVS(NWL_DB) < END_WAV ) GOTO 128
  
  ! Find position of start- and end-wavelengths of current net in
  ! wavelength array of aerosol data
  ! -------------------------------------------------------------
  IF( START_WAV < DB_WAVS(1) ) THEN
     NWLS = 1 
     CALL WRITE_MESSAGE(R_NAME,'WARNING',' ')
     WRITE(EU,'(A)') '    Required wavelength range is outside of'//&
          ' the aerosol data base.'
     WRITE(EU,'(A,2x,F9.2)') '    First wavelength in data base:', DB_WAVS(1)
     WRITE(EU,'(A,2x,F9.2)') '    First required wavelength    :', START_WAV
     WRITE(EU,*) '  !!! The extrapolation with constant is used !!!'
  ELSE
     CALL LIN_SEARCH( START_WAV, DUM, NWLS, NWL_DB, DB_WAVS, ERROR )
     IF (ERROR) GOTO 126
  ENDIF
  
  CALL LIN_SEARCH( END_WAV,   DUM, NWLE, NWL_DB, DB_WAVS, ERROR )
  IF (ERROR) GOTO 126
  
  NWLE = NWLE + 1
  IF(DB_WAVS(NWLE) == END_WAV) NWLE = NWLE + 1
  NAW = NWLE - NWLS + 1
  
  ALLOCATE( APH_VS(NLEGEN,LAN,Nm_vs,NAW), USER_MOM(MUM,LAN,Nm_vs,NAW), & !A
       M_Lay_Wav(LAN,NAW), ASYM_PAR(LAN,NAW), TAU(NAW), AER_WAVS(NAW), & !A
       AER_COEFF(3,NLAYERS,NAW), IDA(NLAYERS) ) !A

  USER_MOM     = 0.d0
  AER_PHASMOMS => APH_VS( :,:,1,: )              
  AER_PHASMOMS = 0.d0
  AER_COEFF    = 0.d0  
  AER_WAVS(:)  = DB_WAVS (NWLS:NWLE)

  !==================================
  !   Read scenario input parameters
  !==================================
  STR_INP = 'Season'
  CALL Read_Input (STR_INP, READ_ARR, *880)
  READ(READ_ARR(1)%S,*,ERR = 101) SEASON
  
  ! Boundary layer 
  ! --------------
  STR_INP = 'Boundary layer aerosol type'
  CALL Read_Input (STR_INP, READ_ARR, *880)
  READ(READ_ARR(1)%S,*,ERR = 101) BL_TYPE
  STR_INP = 'Boundary layer visibility'
  CALL Read_Input (STR_INP, READ_ARR, *880)
  READ(READ_ARR(1)%S,*,ERR = 101) BL_VIS
  STR_INP = 'Boundary layer humidity'
  CALL Read_Input (STR_INP, READ_ARR, *880)
  READ(READ_ARR(1)%S,*,ERR = 101) BL_HUM

  ! Troposphere
  ! -----------
  STR_INP = 'Tropospheric humidity'
  CALL Read_Input (STR_INP, READ_ARR, *880)
  READ(READ_ARR(1)%S,*,ERR = 101) TR_HUM
  STR_INP = 'Tropospheric visibility'
  CALL Read_Input (STR_INP, READ_ARR, *880)
  READ(READ_ARR(1)%S,*,ERR = 101) TR_VIS
  
  ! Stratosphere
  ! ------------
  STR_INP = 'Stratospheric aerosol loading'
  CALL Read_Input (STR_INP, READ_ARR, *880)
  READ(READ_ARR(1)%S,*,ERR = 101) STR_LOAD
  STR_INP = 'Stratospheric aerosol type'
  CALL Read_Input (STR_INP, READ_ARR, *880)
  READ(READ_ARR(1)%S,*,ERR = 101) STR_TYPE
  
  ! Mesosphere
  ! ----------
  STR_INP = 'Mesospheric aerosol loading'
  CALL Read_Input (STR_INP, READ_ARR, *880)
  READ(READ_ARR(1)%S,*,ERR = 101) MES_LOAD
  
  IF( TRIM(AER_PARAM)=='default' ) THEN
     SEASON = 2
     BL_TYPE  = 3 ; BL_VIS   = 2 ; BL_HUM = 3
     TR_HUM   = 3 ; TR_VIS   = 2 
     STR_LOAD = 1 ; STR_TYPE = 1
     MES_LOAD = 1
  ENDIF

  !=====================================================================
  !  Setup aerosol extinction profile at 550 nm (=reference profile ):
  !=====================================================================
  N_SKIP = 50  ;  K1 = 1
  OPEN(IUNIT, FILE=TRIM(ADB)//'DB_PR.dat', STATUS='OLD')
  DO I = 1,N_SKIP
     READ(IUNIT,*)
  ENDDO
  DO K = 1,3             ! Layer 1
     READ(IUNIT,*) K2, LT_Z(K1), (HRD(J), J=1,5)
     W1(K1) = HRD(BL_VIS)  ;  K1 = K1 + 1
  ENDDO
  L = (SEASON-1)*2 + TR_VIS
  DO K = 1,8             ! Layer 2
     READ(IUNIT,*) K2, LT_Z(K1), (HRD(J), J=1,4)
     W1(K1) = HRD(L)  ;  K1 = K1 + 1
  ENDDO
  L = (SEASON-1)*4 + STR_LOAD
  DO K = 1,16             ! Layer 3
     READ(IUNIT,*) K2, LT_Z(K1), (HRD(J), J=1,8)
     W1(K1) = HRD(L)  ;  K1 = K1 + 1
  ENDDO
  DO K = 1,7             ! Layer 4
     READ(IUNIT,*) K2, LT_Z(K1), (HRD(J), J=1,2)
     W1(K1) = HRD(MES_LOAD)  ;  K1 = K1 + 1
  ENDDO
  CLOSE(IUNIT)

  IF( AER_SHIFT ) LT_Z(:) = LT_Z(:) + GROUND_HEIGHT

  !==================================================================
  !  W1 (1-33) now contains the aerosol extinction profile for 550 nm.
  !  Interpolate from 33 layers to "NLAYERS"  layers:
  !==================================================================
  CALL LIN_INT(33, LT_Z(1:33), W1(1:33), NLAYERS, ZHOLD, SAFI, 'Z')

  SAFI(:) = AER_SCALING*SAFI(:) 
  ! SAFI now contains the aerosol reference profile at the height
  ! levels of the forward model altitude grid ZHOLD.
  ! ==============================================================

  !================================================================
  !  Calculate extinction / absorption coefficients and asymmetry 
  !  parameter:
  !  (This is done by multiplying the reference profile with 
  !  appropriate scaling factors. This data is taken from 
  !  LOWTRAN 7 BLOCK DATA module EXTDTA.)        
  !================================================================

  ! Indexes of altitude grid levels which are within aerosol layers
  IDA(:) = 1
  IF( LAN > 1 ) THEN
     DO I = 1,LAN - 1
        DD1 = ZHOLD(LAS(I)) ;  DD2 = ZHOLD(LAS(I+1))
        DO J = 1,NLAYERS
           IF( ZHOLD(J) .LE. DD1 .AND. ZHOLD(J) > DD2 ) IDA(J)=I
        ENDDO
     ENDDO
  ENDIF
  DD1 = ZHOLD(LAS(LAN))
  DO J = 1,NLAYERS
     IF( ZHOLD(J) .LE. DD1 ) IDA(J) = LAN
     ! WRITE(*,'(2x,F7.3,2x,I2)') ZHOLD(J), IDA(J)
  ENDDO

  IF(LOG_LEVEL(1) >= N_DSO) THEN
     WRITE(LUNIT,*) ' ------ Test of aerosol layering  -------- '
     DO I = 1,LAN
        WRITE(LUNIT,'(2x,I3,2(1x,f7.3))') I, ZHOLD(LAS(I)) ,ZHOLD(LAE(I)) 
     ENDDO
     WRITE(LUNIT,*) ' ----------------------------------------- '
  ENDIF

  ! Assign extinction/absorption/asymmetry coefficients:
  ! ====================================================
  ! In actual wavelength range only !!
  N_SKIP = 11
  SNA(1) = 'rural'
  SNA(2) = 'urban'
  SNA(3) = 'maritime'
  SNA(4) = 'tropospheric'      
  INP_NAME(1) = TRIM(ADB)//'DBO_MS.dat'
  INP_NAME(2) = TRIM(ADB)//'DBO_ST.dat'
  INP_NAME(3) = TRIM(ADB)//'DBO_TR.dat'
  INP_NAME(4) = TRIM(ADB)//'DBO_BL-'//TRIM(SNA(BL_TYPE))//'.dat'
  SH(1) = 1        ; PC(1) = 1
  SH(2) = STR_TYPE ; PC(2) = 3
  SH(3) = TR_HUM   ; PC(3) = 4 
  SH(4) = BL_HUM   ; PC(4) = 4 
  I_L = 1
  
  DO L = 1,4
     IF( REM_LAYER(L) ) CYCLE
     Ke = SH(L) ; Ka = PC(L)+SH(L) ; Ks = 2*PC(L)+SH(L)
     OPEN(OUNIT, FILE=TRIM(INP_NAME(L)), STATUS='OLD')
     DO I = 1,N_SKIP + NWLS - 1 
        READ(OUNIT,*) 
     ENDDO
     DO K = 1,NAW  
        READ(OUNIT,*) (HRD(J), J = 1,3*PC(L))
        DO I = LAS(I_L),LAE(I_L)
           AER_COEFF(EXI,I,K) =  HRD(Ke)*SAFI(I)
           AER_COEFF(SCI,I,K) = (HRD(Ke) - HRD(Ka))*SAFI(I)
           AER_COEFF(ASY,I,K) =  HRD(Ks)
        ENDDO
        ASYM_PAR(I_L,K) = HRD(Ks)
     ENDDO
     CLOSE(OUNIT)
     I_L = I_L + 1
  ENDDO
  
  IF( L_scal ) THEN
     ! Scaling of number density between ZS1 - ZS2
     DO I = 1, NAW
        WHERE( ZHOLD(:) >= ZS1.AND.ZHOLD(:) <= ZS2 ) 
           AER_COEFF(SCI,:,I) = AER_FACT*AER_COEFF(SCI,:,I) 
           AER_COEFF(EXI,:,I) = AER_FACT*AER_COEFF(EXI,:,I) 
        END WHERE
     ENDDO
     ! Scaling of the scattering coefficient between ZS1 - ZS2
     ! in order to change the single scattering albedo as follows:
     !    SSA' = 1 - V_PAP*(1 - SSA) 
     ! Extinction coefficient remains the same
     DO I = 1, NAW
        WHERE( ZHOLD(:) >= ZS1.AND.ZHOLD(:) <= ZS2 ) 
           AER_COEFF(SCI,:,I) = AER_COEFF(SCI,:,I) - (V_PAP - 1.d0) &
                *(AER_COEFF(EXI,:,I ) - AER_COEFF(SCI,:,I ))
        END WHERE
     ENDDO
  ENDIF

  IF(LOG_LEVEL(1) >= N_ESO) THEN
     TXT(1) = '... Mesosphere :'
     TXT(2) = '... Stratosphere :'
     TXT(3) = '... Troposphere :'
     TXT(4) = '... Boundary :'
     K1 = 1 ; I_L = 1
     DO K = 1, LA_LT
        IF( REM_LAYER(K) ) CYCLE 
        WRITE(LUNIT,*) TRIM(TXT(K))
        DO I = LAS(I_L),LAE(I_L)
           WRITE(LUNIT,'(2x,I3,2x,f7.3,3x,f7.3,1x,e12.4)') &
                I, ZHOLD(I), AER_COEFF(ASY,I,K1), AER_COEFF(EXI,I,K1)
        END DO
        I_L = I_L + 1
     ENDDO
  ENDIF

  IF ( APH_TYPE=='mie' ) THEN
     CALL WRITE_MESSAGE(R_NAME,'WARNING',' ')
     WRITE(EU,*) '    The Henyey-Greenstein parameterization'
     WRITE(EU,*) '    is used for LOWTRAN aerosol model.'
  ENDIF

  ! Calculate Legendre expansion coefficients corresponding 
  !         to the given asymmetry factors
  ! ================================================================
  DO I1 = 1,LAN
     DO K = 1,NAW  ;  GG = ASYM_PAR(I1,K)
        DO L = 1,MUM  ;  TMP1 = GG**(L-1)*(2*(L-1) + 1)
           IF( TMP1 < 1.d-06 ) EXIT
           USER_MOM(L,I1,1,K) = TMP1
        ENDDO
        M_Lay_Wav(I1,K) = L-1
     ENDDO
  ENDDO

  IF( OUT_A_SCM ) THEN 
     I1 = M_Lay_Wav(LAY_A,WEL_A)
     CALL COEF_TO_SCATMATR(I1, Nm_vs, USER_MOM(1:I1,LAY_A,1:Nm_vs,WEL_A),&
          NSCA_A, GE_A, PATH_TO_SCM)
  ENDIF

  ! Test output of expansion coefficients corresponding to  
  ! the given asymmetry parameter
  ! ============================================================
  TEST_MODE = .FALSE.
  IF( TEST_MODE ) CALL OUT_COEFF

  ! Single scattering correction algorithm -> save all expansion coeff.
  ! --------------------------------------    ------------------------
  IF( DO_SSRCA .OR. TRIM(RTM_TYPE) == 'spher_scat' ) THEN
     Nga_db = MAXVAL( M_Lay_Wav )
     IF (LOG_LEVEL(1) >= N_ESO) WRITE(LUNIT,*) '  Allocate Greek_aer :', Nga_db
     ALLOCATE( Greek_aer( Nga_db,LAN,Nm_vs,NAW ) )
     Greek_aer = 0.d0
     DO I = 1,NAW 
        Greek_aer(:,:,:,I) = USER_MOM(1:Nga_db,:,:,I)
     ENDDO
     IF( .NOT.Plus_mi ) NG_aer = Nga_db
  ENDIF

  DO K = 1,NAW
     ! TAU(K) = AER_SCALING*SUM( AER_COEFF(EXI,:,K)*DZ(:) )/2.d0
     TAU(K) = SUM( AER_COEFF(EXI,:,K)*DZ(:) )/2.d0
  ENDDO

  IF( Scal_aer ) THEN
     CALL LIN_SEARCH(Rwl_aer, DUM, K, NAW, AER_WAVS, ERROR)
     IF (ERROR) GOTO 129
     Scoe = Rtau_aer/( (1.d0 - DUM)*TAU(K) + DUM*TAU(K+1) )
     AER_COEFF(EXI,:,:) = Scoe*AER_COEFF(EXI,:,:)
     AER_COEFF(SCI,:,:) = Scoe*AER_COEFF(SCI,:,:)
     TAU(:) = Scoe*TAU(:)
  ENDIF
  
  AOT_out_VAL = TAU(1)

  !======================================
  !   Write aerosol scenario output file
  !======================================
  CALL WRITE_SCEN_LA(*880)

  !Allocate array for MODULE DELTA_M
  !=================================
  ALLOCATE( FDM_AER(NLAYERS,NAW) ) ! A
  FDM_AER = 0.0D0

  IF( AER_DM_FLG ) THEN
     Itr = AER_TRUNC_IDX
     CALL AER_DM
  ELSE
     APH_VS(:,:,:,:) = USER_MOM(1:NLEGEN,:,:,:)
  ENDIF

  LA_Ndb = LAN
  IF( .NOT.Plus_mi ) THEN
     LA_N = LAN
     ALLOCATE( LA_S(LAN), LA_E(LAN), IDX_AER(NLAYERS) )
     LA_S = LAS ;  LA_E = LAE  ; IDX_AER = IDA
  ELSE
     ALLOCATE( IDX_AER_db(NLAYERS) )
     IDX_AER_db = IDA
  ENDIF
  
  CALL ST_TRACK (R_NAME, 1, 'O')
  
  RETURN
  
101 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Reading error')
  WRITE(EU,*) '    Parameter to read: ', TRIM(STR_INP) 
  GOTO 880
  
126 CALL WRITE_MESSAGE (R_NAME, 'ERROR', 'Error in LIN_SEARCH')
  RETURN 1
  
128 CALL WRITE_MESSAGE(R_NAME, 'ERROR', &
         'Required wavelength range is outside of the data base.')
  WRITE(EU,'(A,2x,F9.2)') '    Last wavelength in data base:', DB_WAVS(NWL_DB)
  WRITE(EU,'(A,2x,F9.2)') '    Last required wavelength    :', END_WAV
  WRITE(EU,*) '    Use another aerosol data base. '
  WRITE(EU,*) '    Sea the file "control_aer.inp" '
  RETURN 1
  
129 CALL WRITE_MESSAGE (R_NAME,'ERROR',&
         'Reference wavelength for AOT is out of range')
  WRITE(EU,'(A,F7.2)') '    Reference wavelength: ', Rwl_aer
  WRITE(EU,'(A,2(2x,F7.2))') '    Database wavelengths: ', AER_WAVS(1), AER_WAVS(NAW)
  WRITE(EU,'(A)') '    Set reference wavelength between database wavelengths'
  WRITE(EU,'(A)') '    in the "control_aer.inp" file.'
  RETURN 1
  
880 WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)
  RETURN 1
  
CONTAINS
!***********************************************************************
  SUBROUTINE AER_DM
    IF (LOG_LEVEL(1) >= N_ESO) THEN
       WRITE(LUNIT,*) '   ***********************  '
       WRITE(LUNIT,*) '   Aerosol D-M scalar case  '
       WRITE(LUNIT,*) '   ***********************  '
    ENDIF
    DO I1 = 1,LAN
       DO K = 1,NAW
          F_DM = USER_MOM(Itr+1,I1,1,K)/DBLE(2*Itr + 1)
          FDM_AER(LAS(I1):LAE(I1),K) = F_DM
          DO L = 1,Itr
             AER_PHASMOMS(L,I1,K) = (USER_MOM(L,I1,1,K) - &
                  F_DM*DBLE(2*L-1))/(1.D0 - F_DM)
          END DO              ! Over moments
       ENDDO                  ! Over wavelengths
    ENDDO                     ! Over layers
    AER_PHASMOMS(Itr+1:NLEGEN,:,1:NAW ) = 0.0D0

    ! "Renormalize" scattering and extinction coefficients
    ! ====================================================
    DO I = 1,NLAYERS
       DO K = 1,NAW
          F_DM = FDM_AER(I,K)
          DUM = AER_COEFF(EXI,I,K) - AER_COEFF(SCI,I,K)
          AER_COEFF(SCI,I,K) = AER_COEFF(SCI,I,K)*(1.D0 - F_DM)
          AER_COEFF(EXI,I,K) = AER_COEFF(SCI,I,K) + DUM
       ENDDO
    ENDDO
  END SUBROUTINE AER_DM

!***********************************************************************
  SUBROUTINE OUT_COEFF
    INTEGER     NU_A
    NU_A = M_Lay_Wav(1,1)
    OPEN(OUNIT,FILE=DIR_OUT//'/H_G_coeff.out')
    WRITE(OUNIT,'(a)') '#  Expansion coefficients of the H-G phase function'
    WRITE(OUNIT,'(a)') '#  in the lower aerosol layer at first wavelength. '
    WRITE(OUNIT,'(a,2x,F6.3)') '#  Asymmetry parameter :', ASYM_PAR(1,1)
    DO L = 1,NU_A
       WRITE(OUNIT,'(2x,I5,2x,E15.7)') L-1, USER_MOM(L,1,1,1)
    ENDDO
    CLOSE(OUNIT)
  END SUBROUTINE OUT_COEFF

  !***********************************************************************
  SUBROUTINE WRITE_SCEN_LA(*)
    USE  AEROSOLS, ONLY: OWL_AER, NOWL_AER
    
    Integer Unt
    REAL (KND) SS_A, Ext
    CHARACTER (LEN=3)  PHT
    Character (Len=500)  OutStr
    CHARACTER (LEN=60) SEASON_STR(2), BL_TYPE_STR(4), VIS_STR(5), HUM_STR(4), &
         STR_LOAD_STR(4), STR_TYPE_STR(3), MES_LOAD_STR(2)
    REAL (KND), ALLOCATABLE :: T_OUT(:)
    
    IF (LOG_LEVEL(1) >= N_SSO) WRITE (LUNIT,*) ' *** Writing: AEROSOL.OUT'
    PHT = 'H_G'

    SEASON_STR = (/"Fall/Winter  ", "Spring/Summer"/)
    BL_TYPE_STR = (/"rural       ", "urban       ", "maritime    ",  "tropospheric"/)
    VIS_STR = (/"50 km", "23 km", "10 km", " 5 km", " 2 km"/)
    HUM_STR = (/" 0 %", "70 %", "80 %", "90 %"/)
    STR_LOAD_STR = (/"background       ", "moderate volcanic", & 
         "high volcanic    ", "extreme volcanic "/)
    STR_TYPE_STR = (/"background    ", "aged volcanic ", "fresh volcanic"/)
    MES_LOAD_STR(1) = "Normal mesosphere" 
    MES_LOAD_STR(2) = "Transition from volcanic stratosphere to normal mesosphere"
    
    ! Optical thickness at user defined wavelength grid
    IF( NOWL_AER > 0 ) ALLOCATE( T_OUT(NOWL_AER) )
    DO I = 1, NOWL_AER
       CALL LIN_SEARCH(OWL_AER(I), DUM, K, NAW, AER_WAVS, ERROR)
       IF (ERROR) GOTO 130
       T_OUT(I) =  (1.d0 - DUM)*TAU(K) + DUM*TAU(K+1)
    ENDDO

    Call OpenOut ('SCE_AEROSOL.OUT', Unt)
    WRITE ( OutStr, * ) '          ========================='
    Call WriteOut (Unt, OutStr)
    WRITE ( OutStr, * ) '          LOWTRAN AEROSOL SELECTION'
    Call WriteOut (Unt, OutStr)
    WRITE ( OutStr, * ) '          ========================='
    Call WriteOut (Unt, OutStr)
    WRITE ( OutStr, * ) ' '
    Call WriteOut (Unt, OutStr)
     
    IF( Scal_aer ) THEN
       WRITE(OutStr,'(A,F7.3,A,F7.2,A)') &
            '   !!! ATTENTION !!! AOT is set to ',Rtau_aer,' at', Rwl_aer,' nm wavelength.'
       Call WriteOut (Unt, OutStr)
       WRITE(OutStr,'(19x,A)') '  See setting in "control_aer.inp" '
       Call WriteOut (Unt, OutStr)
       WRITE(OutStr,'(6x,A)')  '----------------------------------------------------'
       Call WriteOut (Unt, OutStr)
       WRITE(OutStr,*) 
       Call WriteOut (Unt, OutStr)
    ENDIF

    WRITE(OutStr,*) '   A '//TRIM(SEASON_STR(SEASON))//' season'
    Call WriteOut (Unt, OutStr)
    WRITE ( OutStr, * ) ' '
    Call WriteOut (Unt, OutStr)
   
    WRITE(OutStr,*) '   Boundary layer aerosol type: '//TRIM(BL_TYPE_STR(BL_TYPE))
    Call WriteOut (Unt, OutStr)
    WRITE(OutStr,*) '   Boundary layer visibility  : '//TRIM(VIS_STR(BL_VIS))
    Call WriteOut (Unt, OutStr)
    WRITE(OutStr,*) '   Boundary layer humidity    : '//TRIM(HUM_STR(BL_HUM))
    Call WriteOut (Unt, OutStr)
    WRITE ( OutStr, * ) ' '
    Call WriteOut (Unt, OutStr)
          
    WRITE(OutStr,*) '   Tropospheric visibility: '//TRIM(VIS_STR(TR_VIS))
    Call WriteOut (Unt, OutStr)
    WRITE(OutStr,*) '   Tropospheric humidity  : '//TRIM(HUM_STR(TR_HUM))
    Call WriteOut (Unt, OutStr)
    WRITE ( OutStr, * ) ' '
    Call WriteOut (Unt, OutStr)
    
    WRITE(OutStr,*) '   Stratospheric aerosol loading: '//TRIM(STR_LOAD_STR(STR_LOAD))
    Call WriteOut (Unt, OutStr)
    WRITE(OutStr,*) '   Stratospheric aerosol type   : '//TRIM(STR_TYPE_STR(STR_TYPE))
    Call WriteOut (Unt, OutStr)
    WRITE ( OutStr, * ) ' '
    Call WriteOut (Unt, OutStr)
          
    WRITE(OutStr,*) '   '//TRIM(MES_LOAD_STR(MES_LOAD))
    Call WriteOut (Unt, OutStr)
    WRITE ( OutStr, * ) ' '
    Call WriteOut (Unt, OutStr)
    
    IF( NOWL_AER > 0 ) THEN
       WRITE ( OutStr, * ) ' AOT at user-defined wavelengths [nm]:'
       Call WriteOut (Unt, OutStr)
       DO I = 1, NOWL_AER
          WRITE(OutStr,'(12x,F7.2,4x,F8.4)') OWL_AER(I),T_OUT(I)
          Call WriteOut (Unt, OutStr)
       ENDDO
    ENDIF
    WRITE ( OutStr, * ) ' '
    Call WriteOut (Unt, OutStr)

    WRITE ( OutStr, * ) ' Vertical profiles of aerosol'
    Call WriteOut (Unt, OutStr)
    WRITE ( OutStr, * ) '         extinction coefficient (Ext coeff) and'
    Call WriteOut (Unt, OutStr)
    WRITE ( OutStr, * ) '         single scattering albedo (Ss albedo):'
    Call WriteOut (Unt, OutStr)
    DO K = 1,NAW
       WRITE(OutStr,*) '   '
       Call WriteOut (Unt, OutStr)
       WRITE(OutStr,'(1x,a11,F7.2,a6,a6,F8.4)') &
            'Wavelength ',AER_WAVS(K),' [nm];',' AOT =',TAU(K)
       Call WriteOut (Unt, OutStr)
       Call WriteOut (Unt, ' ----------------------------------------------')
       Call WriteOut (Unt, '     Number, Alt [km], Ext coeff [km-1], Ss albedo')
       DO I = 1, NLAYERS
          Ext = AER_COEFF(EXI,I,K)
          IF( Ext == 0.0d0 ) CYCLE
          SS_A = AER_COEFF(SCI,I,K)/Ext
          WRITE(OutStr,'(6x,I3,4x,F6.2,5x,ES9.2,6x,E12.4)') I, ZHOLD(I), Ext, SS_A
          Call WriteOut (Unt, OutStr)
       END DO
       WRITE(OutStr,*) ' '
       Call WriteOut (Unt, OutStr)
       WRITE(OutStr,*) '    Asymmetry factor used in the aerosol layers :'
       Call WriteOut (Unt, OutStr)
       WRITE(OutStr,*) '    ------------------------------------------- :'
       Call WriteOut (Unt, OutStr)
       WRITE(OutStr,*) '    Layer  Top [km]  Bottom [km]   Asymmetry    Phase'
       Call WriteOut (Unt, OutStr)
       WRITE(OutStr,*) '                                    factor     function'
       Call WriteOut (Unt, OutStr)
       DO I = 1,LAN
          WRITE(OutStr,'(6x,I3,3x,F6.2,4x,F6.2,9x,F6.3,7x,a3)') &
               I, ZHOLD(LAS(I)) ,ZHOLD(LAE(I)), Asym_Par(I,K), PHT
          Call WriteOut (Unt, OutStr)
       ENDDO
    ENDDO                   ! Over wavelengths
    Call CloseOut (Unt)
    
    RETURN
    
130 CALL WRITE_MESSAGE (R_NAME,'ERROR', 'Output wavelength for AOT is out of range')
    WRITE(EU,'(A,F7.2)') '    Output wavelength: ', OWL_AER(I)
    WRITE(EU,'(A,2(2x,F7.2))') '    Database wavelengths: ', AER_WAVS(1), AER_WAVS(NAW)
    WRITE(EU,'(A)') '    Set output wavelength between database wavelengths'
    WRITE(EU,'(A)') '    in the "control_aer.inp" file.'
    RETURN 1
    
  END SUBROUTINE WRITE_SCEN_LA

!***********************************************************************
  SUBROUTINE DEALLOCATE_ARRAYS
    
    IF(ALLOCATED(DENS_PRF)) DEALLOCATE(DENS_PRF)
    IF(ALLOCATED(L_TOP)) DEALLOCATE(L_TOP)
    IF(ALLOCATED(APH_VS)) DEALLOCATE(APH_VS, AER_WAVS, AER_COEFF)
    IF(ALLOCATED(Greek_aer)) DEALLOCATE(Greek_aer)
    IF(ALLOCATED(FDM_AER)) DEALLOCATE(FDM_AER)
    IF(ALLOCATED(LA_S)) DEALLOCATE(LA_S, LA_E, IDX_AER)
    IF(ALLOCATED(IDX_AER_db)) DEALLOCATE(IDX_AER_db)
    
    RETURN
  END SUBROUTINE DEALLOCATE_ARRAYS
!***********************************************************************
END SUBROUTINE LT_AER_SETUP
!***********************************************************************

