!***************************************************************************
!
!       SCIATRAN - Version 4.7 - Last modification: 03 Jan 2024
!
!       Institute of Remote Sensing, University of Bremen, Germany
!
!***************************************************************************
SUBROUTINE MAN_AER_SETUP(START_WAV, END_WAV, *)
  ! Called from "rtm_init.f"                              
  
  USE AER_MIE, ONLY: DENS_PRF, T_550, AOD_RET
  USE AEROSOLS,ONLY: AER_SHIFT, PATH_TO_SCM, OUT_A_SCM, LAY_A, WEL_A,&
                     NSCA_A, GE_A, NG_aer, APH_VS, NAW => N_AER_WAVS, &
                     AER_WAVS, Greek_Aer, SCI => GT_SCAT_IDX, Scal_aer, &
                     AER_COEFF, ASI => GT_ASYM_IDX, Rtau_aer, NWLS, NWLE,&
                     EXI => GT_EXTC_IDX, Rwl_aer, AER_SCALING, LA_N, LA_S, &
                     LA_E, L_TOP, IDX_AER, AOT_out_VAL, L_scal, ZS1, ZS2, &
                     ZA_SE, AER_FACT, V_PAP, R_erw, SHARP_EDGES, Use_AP, &
                     AP_par, AP_wlr, INT_SCM_A, CRIT_SCM_A
  Use AltInp,  Only: ALIN_ACT, NC_AER, ALIN_NWL, ALIN_WL, ALIN_AER_A, &
                     ALIN_AER_S, ALIN_AER_EC
  Use Cld_SS,  ONLY: MUM => MAXMOM, DO_SSRCA
  USE CLOUDS,  ONLY: DO_CLOUD  
  USE DATA_RET,ONLY: RET_FLAG
  USE DELTA_M, ONLY: FDM_AER, AER_DM_FLG, D_MF_AER, AER_TRUNC_IDX
  USE GRIDS,   ONLY: NLAYERS, ZHOLD => ZZZ, DZ
  USE GT_POLYN,ONLY: NLEGEN
  USE INFO,    ONLY: AER_INFO
  USE I_DIMS,  ONLY: Nm_vs, Nvs
  USE LOG_PARS,ONLY: LUNIT, EU, DIR_OUT, N_SSO, N_ESO, C_UNIT, LOG_LEVEL,&
                     Work_Dir
  USE RTM_PARS,ONLY: DATA_ITER
  USE ST_VARS, ONLY: RAD_TO_DEG, Vec_Sca, RTM_TYPE, VHL, GROUND_HEIGHT,&
                     SCATT_MODE, RTM_MODE
  USE TOOLS,   ONLY: CUT_STR, SET_PATH, CHK_STAT, SET_PTR, Read_Input,&
                     SPLIT_STRING, Read_Text, IsNum, UPPER_LOWER_CASE
  USE TYPES,   ONLY: KND, DYN_CHAR, DYN_CHAR_L
  !==================================================================

  Character (Len = *), PARAMETER :: R_NAME = 'MAN_AER_SETUP'
  ! (a) Input variables
  ! ===================
  !     AER_DM_FLG        : delta-M approximation yes/no 
  !                         <---- From Module DELTA_M
  !     AER_TRUNC_IDX     : truncation index for delta-M approximation 
  !                         <---- From Module DELTA_M
  !     ZHOLD             : forward model altitude grid
  !     NLAYERS           : number of layers in altitude grid
  !     NLEGEN            : number of Legendre moments
  !     LUNIT             : LOG file unit number

  ! (b) Output variables
  ! ====================
  !     AER_COEFF         : scattering / extinction coefficients and 
  !                         asymmetry parameters 
  !     AER_PHASMOMS      : Legendre moments
  !     NAW               : number of wavelengths of chosen aerosol 
  !                         parameterization
  !     AER_WAVS          : wavelengths of aerosol data base (depends on
  !                         parameterization chosen)
  REAL (KND), INTENT(IN) :: START_WAV, END_WAV
  INTEGER    NZE, NZA, N_GAU, Nel
  LOGICAL    Invert, TEST_MODE, ERROR, CheckFlg
  REAL (KND) Scoe, TMP1, T0, GG, DUM, D1, D2, Mom_tes
  INTEGER, ALLOCATABLE :: M_Lay_Wav(:,:)
  REAL (KND), ALLOCATABLE :: PHAS_LOC (:), PL_RES(:,:), R_COF(:), Z_EXT(:),&
       Z_ABS(:), TMP2(:), TAU(:), EXT_COEFF(:,:), SSA_TMP(:,:), AER_SSA(:), &
       X_ang(:), ASYM_PAR(:,:), Z_ASYM(:)
  REAL (KND), ALLOCATABLE, TARGET :: USER_MOM(:,:,:,:)
  CHARACTER (LEN=2)  ETYPE
  CHARACTER (LEN=:), ALLOCATABLE :: STR_INP, Ph_type, NAME, DIR_name, TXT
  CHARACTER (LEN=3),  ALLOCATABLE :: ESM(:)
  TYPE (DYN_CHAR), ALLOCATABLE :: READ_ARR(:), STR_ARR(:), Names(:,:)
  TYPE (DYN_CHAR_L), ALLOCATABLE :: ReadArr(:)
  REAL (KND), POINTER :: AER_PHASMOMS(:,:,:)
  INTEGER   I, I1, J, K, K1, Ks, L, NLEG_LOC, OUNIT, Itr, N_Z_ASYM, Nrw, IUNIT
  PARAMETER  ( OUNIT = 55 )
  TYPE (DATA_ITER), POINTER :: RTM_PTR
  ! Variable for Delta-M
  ! ======================
  REAL (KND)     F_DELTAM, AER_ABS_COEFF
  ! Variable for Delta-fit
  ! ======================
  INTEGER             N_ANG, Nleg_df, N_KAT, NU_A
  REAL (KND)    F_Dfit_loc
      
  ! ================================================================== 
  INTERFACE
     SUBROUTINE SCM_TO_EC(MAX_CO, N_GAU, LA_N, NAW, Nel, ESM, Names, INT_SCM, &
          CRIT_SCM, Lay_out, WL_out, OUT_SCM, M_Lay_Wav, USER_MOM, OUT_PATH, *)
       Use Cld_SS,  ONLY: MUM => MAXMOM
       USE I_DIMS,   ONLY: Nm_vs, Nvs
       USE LOG_PARS, ONLY: LUNIT, EU, DIR_OUT, LOG_LEVEL, N_ESO, C_UNIT
       USE ST_VARS,  ONLY: Pie, RTD => RAD_TO_DEG
       USE TYPES,    ONLY: KND, DYN_CHAR
       INTEGER, INTENT(IN) :: MAX_CO, N_GAU, Nel, Lay_out, WL_out, LA_N, NAW
       INTEGER   M_Lay_Wav(LA_N,NAW)
       LOGICAL   OUT_SCM
       REAL (KND) USER_MOM(MUM,LA_N,Nm_vs,NAW), CRIT_SCM
       CHARACTER (LEN=3)  ESM(Nel), INT_SCM
       CHARACTER (LEN=*)   OUT_PATH
       TYPE (DYN_CHAR), ALLOCATABLE :: Names(:,:)      
     END SUBROUTINE SCM_TO_EC
  END INTERFACE

  CALL ST_TRACK (R_NAME, 1, 'I')  

  CALL DEALLOCATE_ARRAYS ! All arrays allocated below must be 
                         ! conditionally deallocated here

  N_ANG = 180
  ALLOCATE( X_ang(N_ANG) )
  ERROR = .FALSE.

  ! =============================
  ! User-defined parameterization
  ! =============================
  IF( VHL ) THEN
     CALL READ_MAN_VHL(*880)
  ELSE
     CALL READ_MAN_AER(*880)
  ENDIF

  ! ******************************************************************
  !       Scattering function/matrix generation             
  ! ******************************************************************
  SELECT CASE (Ph_type)
  CASE('Asymmetry_factor')
     !  Expansion coefficients according to asymmetry factors
     ! ------------------------------------------------------
     DO I1 = 1,LA_N
        DO K = 1,NAW
           GG = ASYM_PAR(I1,K)
           DO L = 1,MUM
              TMP1 = GG**(L-1)*(2*(L-1) + 1)
              IF( TMP1 < 1.d-06 ) EXIT
              USER_MOM(L,I1,1,K) = TMP1
           ENDDO
           M_Lay_Wav(I1,K) = L-1
        ENDDO
     ENDDO
     NU_A = M_Lay_Wav(1,1)
     
     IF (LOG_LEVEL(1) >= N_ESO) THEN
        WRITE(*,*) '   Full namber of EK = ', NU_A
        WRITE(*,*) '   First 17 coefficients  '
        WRITE(LUNIT,'(2x,17(1x,f10.7))') &
             (USER_MOM(L,1,1,1)/(2*(L-1) + 1), L=1,17)
     ENDIF
           
     TEST_MODE = .FALSE.
     ! Test output of expansion coefficients corresponding to  
     ! the given asymmetry parameter
     ! ===============================================================
     IF( TEST_MODE ) CALL OUT_COEFF

     ! Output of scattering matrix corresponding to 
     ! the given asymmetry factor
     ! ===============================================================
     IF( OUT_A_SCM ) THEN 
        ! Corresponds to Lay_out, Wl_out used below
        NU_A = M_Lay_Wav(LAY_A,WEL_A) 
        IF( LAY_A > LA_N ) GOTO 135
        IF( WEL_A > NAW )  GOTO 136
        CALL COEF_TO_SCATMATR(NU_A, Nm_vs, &
             USER_MOM(1:NU_A,LAY_A,1:Nm_vs,WEL_A), NSCA_A, GE_A, PATH_TO_SCM)
     ENDIF

  CASE('Expansion_coeff')
     ! Reading of the expansion coefficients 
     ! ==================================================================
     IF (LOG_LEVEL(1) >= N_ESO) WRITE(LUNIT,*) &
          '    Reading of the expansion coefficients  '
     IF( ALIN_ACT(3) ) THEN
        M_Lay_Wav(1,1:2) = NC_AER
        NU_A = NC_AER
        DO I = 1, Nm_vs
           USER_MOM(1:NC_AER,1,I,1) = ALIN_AER_EC(1:NC_AER,I)
           USER_MOM(1:NC_AER,1,I,2) = ALIN_AER_EC(1:NC_AER,I)
        ENDDO
     ELSE
        DO I1 = 1,LA_N
           DO K = 1,NAW
              OPEN( NewUnit = IUNIT, FILE=Names(I1,K)%S, STATUS='OLD' )
              CALL SKIP_HEADER (IUNIT)           
              DO I = 1,MUM
                 READ(IUNIT,*,END=28,ERR=29)tmp1,(tmp2(J),J=1,Nm_vs)
                 !IF( I < 10 ) THEN
                 !   WRITE(*,'(A,2(1x,I3))') '      ',I1, K
                 !   WRITE(*,'(1x,F4.1,6(1x,E12.4))') & 
                 !     tmp1,(tmp2(J),J=1,Nm_vs)
                 !ENDIF
                 Ks = I - IDINT(tmp1)
                 IF( Ks/=1 .AND. Ks/=0 ) THEN
                    ETYPE = 'E0' ;  GOTO 132
                 ENDIF
                 IF( I==1 ) T0 = tmp1
                 IF( I==2 ) THEN
                    IF( DABS(DABS(tmp1-T0)-1.d0) > 1.d-10) GOTO 132
                 ENDIF
                     
                 !! For S.Korkin Greek symbol matrix (2k+1)
                 ! DO K1 = 1, Nm_vs
                 !    tmp2(K1) = tmp2(K1)*( 2.d0*tmp1 + 1.d0)
                 ! ENDDO

                 USER_MOM(I,I1,:,K) = tmp2(:) 
                 ! IF( I < 100 ) THEN
                 !    WRITE(*,'(f5.1,6(1x,e12.4))') &
                 !         tmp1,(tmp2(J),J=1,Nm_vs)
                 ! ENDIF
                 ! The additional condition, I > 2, was added to read 
                 ! Rayleigh phase function in the scalar case
                 ! which is used for test purpose
                 IF( SUM(TMP2(:)) == 0.d0 .AND. I > 2 ) GOTO 28
              END DO
29            CALL WRITE_MESSAGE (R_NAME, 'ERROR', 'Error reading data file !')
              WRITE(EU,*) '  File name : ', Names(I1,K)%S
              RETURN 1             
28            CONTINUE
              CLOSE(IUNIT)

              ! For S.Korkin Greek symbol matrix not used
              IF( Nm_vs > 1 ) THEN
                 USER_MOM(:,I1,5,K) = -USER_MOM(:,I1,5,K)
                 USER_MOM(:,I1,6,K) = -USER_MOM(:,I1,6,K)
                 ! Leads to the opposite sign by V
                 !USER_MOM(:,I1,6,K) =  USER_MOM(:,I1,6,K)
              ENDIF

              M_Lay_Wav(I1,K) = I - 1
              NU_A = I - 1
              ! WRITE(*,*) &
              !      '  Number of user moment after reading:', NU_A
              ! WRITE(*,*) &
              !      '  Check first two moments of phase function :'
              ! WRITE(*,'(4x,2(1x,f8.5))') &
              !      USER_MOM(1,I1,1,K), USER_MOM(2,I1,1,K)
              ! Test of first expansion coefficients
              Mom_tes = USER_MOM(1,I1,1,K)
              ETYPE = 'E1'
              IF( DABS(Mom_tes - 1.d0) > 1.d-8 ) GOTO 132
              ! Test of second expansion coefficients
              Mom_tes = USER_MOM(2,I1,1,K)/3.
              ETYPE = 'E2'
              IF( Mom_tes > 1.d0 ) GOTO 132
              ETYPE = 'E3'
              IF( Mom_tes < 0.d0 ) GOTO 132
           ENDDO            ! end for all user wavelengths
        ENDDO               ! end for all layers
     ENDIF

     IF( LOG_LEVEL(1) >= N_ESO ) WRITE(LUNIT,'(2x,17(1x,f10.7))') &
          (USER_MOM(L,1,1,1)/(2*(L-1) + 1), L=1,11)

     ! Test output of scattering matrix corresponding to 
     ! the given expansion coefficients
     IF( OUT_A_SCM ) THEN 
        ! Corresponds to Lay_out, Wl_out used below
        NU_A = M_Lay_Wav(LAY_A,WEL_A) 
        IF( LAY_A > LA_N ) GOTO 135
        IF( WEL_A > NAW )  GOTO 136
        CALL COEF_TO_SCATMATR(NU_A, Nm_vs, &
             USER_MOM(1:NU_A,LAY_A,1:Nm_vs,WEL_A), NSCA_A, GE_A, PATH_TO_SCM)
     ENDIF

! ===================================================================
  CASE('Scattering_matrix')
     N_GAU = MUM  
     IF( .NOT.ALLOCATED(ESM) )  ALLOCATE( ESM(Nel) ) 
     CALL SCM_TO_EC(MUM, N_GAU, LA_N, NAW, Nel, ESM, Names, INT_SCM_A, &
          CRIT_SCM_A, Lay_A, WeL_A, OUT_A_SCM, M_Lay_Wav, USER_MOM, &
          PATH_TO_SCM, *880)
!     WRITE(*,*) '   MUM  and N_GAU',  MUM, N_GAU
!     PAUSE '   After scm_to_ec.f90 '
  END SELECT

  ! Preliminary for first layer and first wavelength ???
  NU_A = M_Lay_Wav(1,1)

  ! Single scattering correction algorithm -> save all expansion coeff.
  ! --------------------------------------    ------------------------
  IF( DO_SSRCA.OR.TRIM(RTM_TYPE)=='spher_scat'.OR.SCATT_MODE =='ss' ) THEN
     NG_aer = MAXVAL( M_Lay_Wav )
     IF (LOG_LEVEL(1) >= N_ESO) WRITE(LUNIT,*) &
          '  Allocate Greek_aer :', NG_aer
     ALLOCATE( Greek_aer( NG_aer,LA_N,Nm_vs,NAW ) )
     Greek_aer = 0.d0
     DO I = 1,NAW 
        Greek_aer(:,:,:,I) = USER_MOM(1:NG_aer,:,:,I)
     ENDDO
  ENDIF
  
  IF( L_scal ) THEN
     ! Scaling of number density between ZS1 - ZS2
     DO I = 1, NAW
        WHERE( ZHOLD(:) >= ZS1.AND.ZHOLD(:) <= ZS2 ) 
           AER_COEFF(SCI,:,I) = AER_FACT*AER_COEFF(SCI,:,I) 
           AER_COEFF(EXI,:,I) = AER_FACT*AER_COEFF(EXI,:,I) 
        END WHERE
     ENDDO
     ! Scaling of the scattering coefficient between ZS1 - ZS2
     ! in order to change the single scattering albedo as follows:
     !    SSA' = 1 - V_PAP*(1 - SSA) 
     ! Extinction coefficient remains the same
     DO I = 1, NAW
        WHERE( ZHOLD(:) >= ZS1.AND.ZHOLD(:) <= ZS2 ) 
           AER_COEFF(SCI,:,I) = AER_COEFF(SCI,:,I) - (V_PAP - 1.d0) &
                *(AER_COEFF(EXI,:,I ) - AER_COEFF(SCI,:,I ))
        END WHERE
     ENDDO
  ENDIF   ! scaling in selected altitude range

  DO K = 1,NAW
     TAU(K) = SUM( AER_COEFF(EXI,:,K)*DZ(:) )/2.0d0
  ENDDO

  !WRITE(*,*) '  Tau 1 :', TAU(:)
  !DO I = 1, NLAYERS
  !   WRITE(*,'(2x, f8.4, *(2x,E12.4))') &
  !        ZHOLD(I), (AER_COEFF(EXI,I,K), K = 1,NAW)
  !ENDDO
  
  ! Scaling of number density at all altitudes
  ! write(*,*) '    Before SCAL '
  IF( Scal_aer ) THEN
     CALL LIN_SEARCH(Rwl_aer, DUM, K, NAW, AER_WAVS, ERROR)
     IF (ERROR) GOTO 126
     Scoe = Rtau_aer/( (1.d0 - DUM)*TAU(K) + DUM*TAU(K+1) )
     AER_COEFF(EXI,:,:) = Scoe*AER_COEFF(EXI,:,:)
     AER_COEFF(SCI,:,:) = Scoe*AER_COEFF(SCI,:,:)
     TAU(:) = Scoe*TAU(:)
!     WRITE(*,*) '  New tau:', TAU
  ENDIF
  AOT_out_VAL = TAU(1)
  
  ! Test of the aerosol OD 
  ! OPEN(IUNIT, FILE = DIR_OUT//'/aer_OD.dat') 
  !   DO K = 1,NAW
  !      WRITE(IUNIT,*) AER_WAVS(K), TAU(K)
  !   ENDDO
  ! CLOSE(IUNIT)

  IF(CHK_STAT('aer-erw',AER_INFO%NAME,AER_INFO%WF_FLG) ) THEN
     ALLOCATE( R_erw(NLAYERS,NAW) )
     ! Select position of reference wavelength in the array
     ! AER_WAVS(:) of the input WL grid
     Nrw = 3 
     IF( Nrw > NAW ) Nrw = 2
     ! Ratio of extinction coefficient (EC) at input WL-grid
     ! to EC at reference wavelength
     R_erw = 1.d0
     DO I = 1, NAW
        WHERE( AER_COEFF(EXI,:,Nrw) > 0.d0 ) &
             R_erw(:,I) = AER_COEFF(EXI,:,I)/AER_COEFF(EXI,:,Nrw)
     ENDDO
     CALL SET_PTR( RTM_PTR, 'aer-erw' )
     RTM_PTR%APR(:) = AER_COEFF(EXI,:,Nrw)
     RTM_PTR%CRN(:) = AER_COEFF(EXI,:,Nrw)
     
     OPEN(NewUnit=IUNIT, FILE = DIR_OUT//'/aer_ext-coef.dat') 
     WRITE(IUNIT,'(A)') '#  A priori aerosol extinction coefficient profile'
     WRITE(IUNIT,'(A,f8.3,A)') '#  at the reference wavelength:', &
          AER_WAVS(Nrw),' nm'
     DO K = 1,NLAYERS
        IF( AER_COEFF(EXI,K,Nrw) == 0.d0 ) CYCLE
        WRITE(IUNIT,*) ZHOLD(K), AER_COEFF(EXI,K,Nrw)
     ENDDO
     CLOSE(IUNIT)
  ENDIF
  
  IF(CHK_STAT('aer-dens',AER_INFO%NAME,AER_INFO%WF_FLG) ) THEN
     IF( RTM_MODE =='ret' ) THEN 
        ALLOCATE( DENS_PRF(NLAYERS) )
        DENS_PRF(:) = 1.d0
        CALL SET_PTR( RTM_PTR, 'aer-dens' )
        RTM_PTR%APR(:) = DENS_PRF(:)
        RTM_PTR%CRN(:) = DENS_PRF(:)    
        CALL LIN_SEARCH(550.d0, DUM, K, NAW, AER_WAVS, ERROR)
        IF (ERROR) GOTO 126
        T_550 = (1.d0 - DUM)*TAU(K) + DUM*TAU(K+1)
        IF( RET_FLAG ) AOD_RET(1,1) = T_550
     ENDIF
  ENDIF

  ! ************** Delta-M, Delta-fit approach ****************
  !  Allocate array for MODULE DELTA_M
  ! =================================
  IF ( AER_DM_FLG  ) THEN 
     Itr = AER_TRUNC_IDX
     SELECT CASE( Vec_Sca )
     CASE('vect_rte')
        CALL AER_DM_VEC
     CASE('scal_rte')
        SELECT CASE ( TRIM(D_MF_AER) )
        CASE( 'D-FA' )
           Nleg_df = NU_A
           ALLOCATE(PHAS_LOC( Nleg_df), PL_RES( Nleg_df,N_ANG )) ! A 
           Nleg_Loc = AER_TRUNC_IDX + 1
           ALLOCATE( R_COF( Nleg_Loc ))
           CALL Start_Delta_Fit( N_ANG, Nleg_df )
           DO I1 = 1,LA_N
              !WRITE(*,*) '  I1 = ',I1
              !                                         Not proof
              PHAS_LOC(1:Nleg_df) = USER_MOM(1:Nleg_df,I1,1,1)
              CALL Delta_Fit( N_ANG, Nleg_df, PHAS_LOC )
              DO K1 = LA_S(I1), LA_E(I1)
                 DO K = NWLS, NWLE
                    FDM_AER(K1,K) = F_Dfit_loc
                    AER_PHASMOMS(1:Nleg_loc,K1,K) = R_cof(1:Nleg_loc)/R_cof(1)
                 ENDDO
              ENDDO
           ENDDO
        CASE( 'D-MA' )
           CALL AER_DM
        END SELECT
     END SELECT

     ! "Renormalize" scattering and extinction coefficients
     ! ====================================================
     !  IF( FDM_AER(1,1) /= 
     DO I = 1, NLAYERS
        IF (LOG_LEVEL(1) >= N_ESO) &
             write(LUNIT,*) I, AER_COEFF(SCI,I,1), FDM_AER( I,1 ) 
        DO K = NWLS, NWLE
           F_DELTAM = FDM_AER( I,K )
           AER_ABS_COEFF = AER_COEFF(EXI,I,K) - AER_COEFF(SCI,I,K)
           AER_COEFF(SCI,I,K) = AER_COEFF(SCI,I,K)*(1.0D0 - F_DELTAM)
           AER_COEFF(EXI,I,K) = AER_COEFF(SCI,I,K) + AER_ABS_COEFF 
        ENDDO
     ENDDO
     
     IF( ALIN_ACT(3) ) THEN
        DO I = 1, NLAYERS
           F_DELTAM = FDM_AER(I,1)
           ALIN_AER_S(:,I) = ALIN_AER_S(:,I)*(1.D0 - F_DELTAM)
        ENDDO
     ENDIF
     
  ELSE
     L = MAXVAL( M_Lay_Wav )
     ! WRITE(*,*) '   Max value of moments :', L
     IF( L > NLEGEN ) THEN
        CALL WRITE_MESSAGE(R_NAME, 'WARNING', &
             'The number of aerosol expansion moments greater than '&
             //'"Number of Legendre moments"')
        WRITE(EU,'(a46,1x,I4)') &
             '      "Number of Legendre moments"           ', NLEGEN
        WRITE(EU,'(a46,1x,I4)') &
             '      Maximal number of moments after reading', L
        WRITE(EU,*)'      To enhance accuracy use "D-MA" option'&
             //' in the file "control_aer.inp".'
     ENDIF
     DO K = 1,NAW
        APH_VS(:,:,:,K) = USER_MOM(1:NLEGEN,:,:,K)
     ENDDO
  ENDIF

  CALL WRITE_SCEN_MN(*880)

  ! STOP '   Stop after "WRITE_SCEN_MN"'
      
  CALL ST_TRACK (R_NAME, 1, 'O')  
  RETURN
  
880 WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)
  
  RETURN 1
  
126 CALL WRITE_MESSAGE (R_NAME,'ERROR',&
         'Reference wavelength for AOT is out of range')
  WRITE(EU,'(A,F7.2)') '    Reference wavelength: ', Rwl_aer
  WRITE(EU,'(A,2(2x,F7.2))') &
       '    Database wavelengths: ', AER_WAVS(1), AER_WAVS(NAW)
  WRITE(EU,'(A)') '    Set reference wavelength between database wavelengths.'
  RETURN 1
  
132 CONTINUE
  SELECT CASE(ETYPE)
  CASE('E0')
     IF(Nvs > 1) THEN
        CALL WRITE_MESSAGE (R_NAME, 'ERROR', ' Looks like you try '//&
             'to use phase function instead of phase matrix.')
        WRITE(EU,'(A,2x,A)') &
             '    In the vector case the program needs phase matrix.'
     ELSE
        CALL WRITE_MESSAGE (R_NAME, 'ERROR', ' ')
        WRITE(EU,'(A,2x,A)') '    Structure of input file is incorrect.'
     ENDIF
  CASE('E1')
     CALL WRITE_MESSAGE (R_NAME, 'ERROR', &
          '    First expansion coefficient of the '// &
          'phase function is not equal to 1.')
     WRITE(EU,'(A)') '    Check normalization of the phase function used.'
  CASE('E2')
     CALL WRITE_MESSAGE (R_NAME, 'ERROR', & 
          '    Second expansion coefficient (asymmetry factor) ')
     WRITE(EU,'(A)') '    of phase function greater than 1.'
  CASE('E3')
     CALL WRITE_MESSAGE (R_NAME, 'ERROR',& 
          '    Second expansion coefficient (asymmetry factor) ')
     WRITE(EU,'(A)') '    of phase function is negative.'
  END SELECT

  WRITE(EU,'(A,2x,A)') '    Wrong structure of input file: ',Names(I1,K)%S
  WRITE(EU,'(A,2x,A)') '    or an error during the reading.'
  WRITE(EU,'(A)') '    Check input file "man_aer.inp" and the file'
  WRITE(EU,'(A,2x,A)') '    containing expansion coefficients: ',Names(I1,K)%S
  RETURN 1

135 CALL WRITE_MESSAGE (R_NAME,'ERROR',&
         'Required number of aerosol layer is out of range')
  WRITE(EU,'(A,I3)') '    Number of aerosol layers: ', LA_N
  WRITE(EU,'(A,I3)') '    Required layer number   : ', LAY_A
  RETURN 1
  
136 CALL WRITE_MESSAGE (R_NAME,'ERROR',&
         'Required number of wavelength is out of range')
  WRITE(EU,'(A,I3)') '    Number of wavelengths for aerosol: ',NAW
  WRITE(EU,'(A,I3)') '    Required wavelength number       : ',WEL_A
  RETURN 1
  
CONTAINS
  
  !**************************************************************************
  SUBROUTINE READ_MAN_AER(*)
    
    CHARACTER (LEN = *), PARAMETER :: R_NAME = 'READ_MAN_AER'
    Integer Inf
    
    CALL ST_TRACK (R_NAME, 1, 'I')  
    
    STR_INP = 'Directory name for scattering matrices'
    CALL Read_Input (STR_INP, READ_ARR, *880)
    DIR_name = SET_PATH(READ_ARR(1)%S)
    
    STR_INP = 'Use Angstrom parameterization'
    CALL Read_Input (STR_INP, READ_ARR, *880 )
    CALL SPLIT_STRING( READ_ARR(1)%S, STR_ARR ) 
    READ(STR_ARR(1)%S,*,ERR = 101)  Use_AP 
    IF( Use_AP ) THEN
       READ(STR_ARR(2)%S,*,ERR = 101)  AP_wlr
       READ(STR_ARR(3)%S,*,ERR = 101)  AP_par
    ENDIF
    
    IF( .NOT.Use_AP ) THEN
       STR_INP = 'Wavelength grid (aerosol parameters)'
       CALL Read_Input (STR_INP, READ_ARR, *880, REARRANGE=.TRUE.)
       NAW = SIZE(READ_ARR) 
       NWLS = 1 ; NWLE = NAW
    ELSE
       NAW = 1 ; NWLS = 1 ; NWLE = NAW
    ENDIF
    
    IF( ALIN_ACT(3) ) THEN 
       NAW = 2 ;  NWLS = 1 ; NWLE = NAW
    ENDIF
    ALLOCATE( AER_WAVS(NAW), AER_COEFF(3,NLAYERS,NAW),& ! A
         FDM_AER(NLAYERS,NAW), TMP2(Nm_vs), IDX_AER(NLAYERS) )  ! A
    
    IF( .NOT.Use_AP ) THEN
       DO I = 1, NAW
          READ(READ_ARR(I)%S,*,ERR = 101)  AER_WAVS(I)
       ENDDO
    ELSE
       AER_WAVS(1) = AP_wlr
    ENDIF
    
    IF( ALIN_ACT(3) ) THEN 
       AER_WAVS(1) = START_WAV ; AER_WAVS(2) = END_WAV
    ENDIF
    
    FDM_AER = 0.d0  ;  AER_COEFF = 0.d0 
    
    IF( (AER_WAVS(1)>START_WAV).AND.(.NOT.USE_AP) ) THEN
       IF( NAW== 1 ) THEN
          CALL WRITE_MESSAGE(R_NAME, 'ERROR', &
               'The extrapolation of aerosol parameters is not '&
               //'possible for a single input wavelength:')
          WRITE(EU,'(A,1x,f7.3)') &
               '     single wavelength for aerosol - ',AER_WAVS(1)
          WRITE(EU,'(A,1x,f7.3)') &
               '     start wavelength for radiance - ',START_WAV
          RETURN 1
       ELSE
          CALL WRITE_MESSAGE(R_NAME, 'WARNING', &
               'The extrapolation of aerosol parameters was used because')
          WRITE(EU,'(a34,1x,f7.3,1x,a38,1x,f7.3)') &
          '     first wavelength for aerosol ',AER_WAVS(1),&
          ' > than start wavelength for radiance ',START_WAV
       ENDIF
    ENDIF
    ! ----------------------------------------------------------------------
    IF( (AER_WAVS(NWLE)<END_WAV).AND.(.NOT.USE_AP) ) THEN
       IF( NAW== 1 ) THEN
          CALL WRITE_MESSAGE(R_NAME, 'ERROR',& 
               'The extrapolation of aerosol parameters is not '&
               //'possible for a single input wavelength:')
          WRITE(EU,'(A,1x,f7.3)') &
               '     single wavelength for aerosol - ', AER_WAVS(NWLE)
          WRITE(EU,'(A,1x,f7.3)') &
               '       end wavelength for radiance - ', END_WAV
          RETURN 1
       ELSE
          CALL WRITE_MESSAGE(R_NAME, 'WARNING', &
               'The extrapolation of aerosol parameters was used because')
          WRITE(EU,'(A,1x,f7.3)') &
               '       last wavelength for aerosol - ', AER_WAVS(NWLE)
          WRITE(EU,'(A,1x,f7.3)') &
               '       end wavelength for radiance - ', END_WAV
       ENDIF
    ENDIF

    STR_INP = 'Aerosol scattering function representation'
    CALL Read_Input (STR_INP, READ_ARR, *880)

    Call Split_String(READ_ARR(1)%S, STR_ARR)
    I1 = SIZE(STR_ARR)
    PH_TYPE = CUT_STR(STR_ARR(1)%S)
    IF( I1==1 ) THEN
       INT_SCM_A = 'lin' ; CRIT_SCM_A = 0.1d0 ! Default settings 
    ELSE
       IF( I1==2.OR.I1 > 3 ) GOTO 109
       CALL UPPER_LOWER_CASE (STR_ARR(2)%S,'U2L')
       INT_SCM_A = CUT_STR(STR_ARR(2)%S)
       READ(STR_ARR(3)%S,*,ERR = 101)  CRIT_SCM_A
       SELECT CASE(INT_SCM_A)
       CASE('aki','lin','spl')
       CASE DEFAULT
          CALL NOT_FOUND (R_NAME, STR_INP, INT_SCM_A, *880)    
       END SELECT
    ENDIF
!    WRITE(*,*) ' Man-aerosol settings '
!    WRITE(*,*) '   1 - ', PH_TYPE
!    WRITE(*,*) '   2 - ', INT_SCM_A 
!    WRITE(*,*) '   3 - ', CRIT_SCM_A 
    
    STR_INP = 'Layering of scattering function/matrix'
    CALL Read_Input (STR_INP, READ_ARR, *880, REARRANGE=.TRUE.)
    LA_N = SIZE(READ_ARR) 
    
    IF( ALIN_ACT(3) ) THEN
       PH_TYPE = 'Expansion_coeff' ; LA_N = 1
    ENDIF

    ALLOCATE( LA_S(LA_N), LA_E(LA_N), L_TOP(LA_N), TAU(NAW), &! A
         APH_VS( NLEGEN,LA_N,Nm_vs,NAW ), M_Lay_Wav(LA_N,NAW), & ! A
         Names(LA_N,NAW), USER_MOM(MUM,LA_N,Nm_vs,NAW) )         ! A
    APH_VS = 0.d0  ;  USER_MOM = 0.d0
    AER_PHASMOMS => APH_VS(:,:,1,:) ; Invert = .FALSE.
    ! Read top altitude all aerosol layers
    DO I = 1, LA_N
       READ(READ_ARR(I)%S,*,ERR = 101)  L_TOP(I)
    ENDDO
    IF( ALIN_ACT(3) ) L_TOP(1) = ZHOLD(1)
    IF( AER_SHIFT ) L_TOP(:) = L_TOP(:) + GROUND_HEIGHT
    
    IF( LA_N > 1 ) THEN
       IF( L_TOP(1) < L_TOP(2) )  Invert = .TRUE.
       IF( USE_AP ) GOTO 141
    ENDIF
    
    !WRITE(*,*) '    L_TOP bot to top: Invert = ', Invert
    !WRITE(*,*) L_TOP

    SELECT CASE (Ph_type)
    CASE('Asymmetry_factor')
       ! Reading of the asymmetry factors
       IF(Nvs > 1) GOTO 131
       ALLOCATE( ASYM_PAR(LA_N+1,NAW),Z_ASYM(LA_N+1) )
       STR_INP = 'Asymmetry factor values'
       CALL Read_Input (STR_INP, READ_ARR, *880)
       IF( SIZE( READ_ARR ) /= LA_N ) GOTO 104
       DO I = 1, LA_N
          Call Split_String(READ_ARR(I)%S, STR_ARR)
          If(Size(STR_ARR) > 1) Then
             If(Size(STR_ARR) /= NAW) GoTo 109
             READ(READ_ARR(I)%S,*,ERR=101) (ASYM_PAR(I,K), K=1,NAW)
          Else
             READ(READ_ARR(I)%S,*,ERR=101) ASYM_PAR(I,1)
             If(NAW > 1) ASYM_PAR(I,2:NAW) = ASYM_PAR(I,1)
          End If
       END DO
       ASYM_PAR(LA_N+1,:) = ASYM_PAR(LA_N,:)
       N_Z_ASYM = LA_N + 1 
       IF( Invert ) THEN
          Z_ASYM(1:LA_N) = L_TOP(LA_N:1:-1)
       ELSE
          Z_ASYM(1:LA_N) = L_TOP(:)
       ENDIF
       Z_ASYM(LA_N+1) = ZHOLD(NLAYERS)
       
       DO K = 1,NAW
          CALL LIN_INT (N_Z_ASYM, Z_ASYM, ASYM_PAR(:,K), NLAYERS, &
               ZHOLD, AER_COEFF(ASI,:,K), 'Z') 
       ENDDO

       !WRITE(*,*) ' ---- Asymmetry factors  -----'
       !DO I = 1, NLAYERS
       !   IF( AER_COEFF(ASI,I,1) == 0.d0 ) CYCLE
       !   WRITE(*,*) ZHOLD(I), AER_COEFF(ASI,I,1), AER_COEFF(ASI,I,2)
       !ENDDO

       !------------------------------------------------------------------
    CASE('Expansion_coeff')
       STR_INP = 'File names containing expansion coefficients'
       CALL Read_Input (STR_INP, READ_ARR, *880)
       IF( SIZE( READ_ARR ) /= LA_N ) GOTO 104
       DO I = 1, LA_N
          Call Split_String(READ_ARR(I)%S, STR_ARR)
          If(Size(STR_ARR) > 1) Then
             If(Size(STR_ARR) /= NAW) GoTo 109
             DO K = 1,NAW
                Names(I,K)%S = DIR_name//'/'//CUT_STR(STR_ARR(K)%S)
             ENDDO
          Else
             Names(I,1)%S = DIR_name//'/'//CUT_STR(STR_ARR(1)%S)
             If(NAW == 1) Cycle
             Do K = 2,NAW 
                Names(I,K)%S = Names(I,1)%S
             End Do
          End If
       END DO
       
       IF( Invert ) Then
          Do L = 1, LA_N
             Do K = 1, NAW
                Names(L,K)%S  = Names(LA_N-L+1,K)%S
             End Do
          End Do
       End IF
       !------------------------------------------------------------------
    CASE('Scattering_matrix')
       Nel = 1
       IF(Nvs > 1) THEN           
          STR_INP = 'Sequence of scattering matrix elements'
          CALL Read_Input (STR_INP, READ_ARR, *880)
          CALL SPLIT_STRING(READ_ARR(1)%S, STR_ARR)
          Nel = SIZE(STR_ARR)
          ALLOCATE( ESM(Nel) )
          IF( Nel > 6 ) GOTO 134
          DO I = 1, Nel
             READ(STR_ARR(I)%S,*,ERR=101)  ESM(I)
             SELECT CASE( ESM(I) )
             CASE('F11','F22','F33','F44','F12','F21','F34','F43')
             CASE DEFAULT
                CALL NOT_FOUND (R_NAME, STR_INP, ESM(I), *880)    
             END SELECT
          ENDDO
       ENDIF  ! scattering matrix

       ! Read file names contaning scattering function/matrix
       STR_INP = 'File names containing scattering function/matrix'
       CALL Read_Input (STR_INP, READ_ARR, *880)
       IF( SIZE( READ_ARR ) /= LA_N ) GOTO 104
       DO I = 1, LA_N
          Call Split_String(READ_ARR(I)%S, STR_ARR)
          If(Size(STR_ARR) > 1) Then
             If(Size(STR_ARR) /= NAW) GoTo 109
             DO K = 1,NAW
                Names(I,K)%S = DIR_name//'/'//CUT_STR(STR_ARR(K)%S)
             ENDDO
          Else
             Names(I,1)%S = DIR_name//'/'//CUT_STR(STR_ARR(1)%S)
             If(NAW == 1) Cycle
             Do K = 2,NAW 
                Names(I,K)%S = Names(I,1)%S
             End Do
          End If
       END DO
       IF( Invert ) Then
          Do L = 1, LA_N
             Do K = 1, NAW
                Names(L,K)%S  = Names(LA_N-L+1,K)%S
             End Do
          End Do
       End IF
       !------------------------------------------------------------------
    CASE DEFAULT
       GOTO 127
    END SELECT
    
    ! Top of aerosol layers in the descending order?
    IF( Invert ) THEN 
       L_TOP(:) = L_TOP(LA_N:1:-1)
    ENDIF
    ! WRITE(*,*) '    L_TOP after Invert = ', Invert
    ! WRITE(*,*) L_TOP

    ! Below top of aerosol layers from the top of atmosphere
    ! to the bottom
    IF( LA_N > 1 ) THEN
       IF( L_TOP(2) > ZHOLD(1) ) GOTO 102
    ENDIF
    IF( L_TOP(LA_N) <= ZHOLD(NLAYERS) ) GOTO 103
    
    IF (LOG_LEVEL(1) >= N_ESO)  WRITE(*,'(A)') &
         '  Top of aerosol layers counting from the top of atmosphere: '
    DO I = 1,LA_N
       LA_S(I) = MINLOC( DABS(L_TOP(I)-ZHOLD(:)),1 )
       IF (LOG_LEVEL(1) >= N_ESO) THEN
          WRITE(LUNIT,'(1x,I3,2x,I3,2x,F9.3)') I, LA_S(I), ZHOLD(LA_S(I))
       ENDIF
    ENDDO
    
    ! Indexes of altitude grid levels which are within aerosol layers
    IDX_AER(:) = 1
    IF( LA_N > 1 ) THEN
       DO I = 1,LA_N - 1
          D1 = ZHOLD(LA_S(I)) ;  D2 = ZHOLD(LA_S(I+1))
          DO J = 1,NLAYERS
             IF( ZHOLD(J) .LE. D1 .AND. ZHOLD(J) > D2 ) IDX_AER(J) = I
          ENDDO
       ENDDO
    ENDIF
    !write(*,*) 'LA_N = ', LA_N
    !write(*,*) 'LA_S = ', LA_S(LA_N)
    D1 = ZHOLD(LA_S(LA_N))
    DO J = 1,NLAYERS
       IF( ZHOLD(J) .LE. D1 ) IDX_AER(J) = LA_N
    ENDDO

    DO I = 1,LA_N-1
       LA_E(I) = LA_S(I+1)-1
    ENDDO
    LA_E(LA_N) = NLAYERS
    
    !DO I = 1,LA_N
    !   WRITE(LUNIT,'(1x,I2,2x,2(2x,I3,2x,F8.3))') I, &
    !       LA_S(I), ZHOLD(LA_S(I)), LA_E(I), ZHOLD(LA_E(I))
    !ENDDO

    !DO I = 1,NLAYERS
    !   WRITE(LUNIT,'(1x,I3,2x,F8.3,2x,I2)') I, ZHOLD(I), IDX_AER(I)
    !ENDDO
      
    IF (LOG_LEVEL(1) >= N_ESO) THEN
       WRITE(LUNIT,*) ' ------ Test of aerosol layering man mode --- '
       WRITE(LUNIT,*) ' ----                                     --- '
       DO I = 1,LA_N
          WRITE(LUNIT,'(2x,I3,2(1x,f7.3))') I, ZHOLD(LA_S(I)) ,ZHOLD(LA_E(I)) 
       ENDDO
       WRITE(LUNIT,*) ' ----------------------------------------- '
    ENDIF
    
    IF( ALIN_ACT(3) ) THEN
       AER_COEFF(SCI,:,1) = ALIN_AER_S(1,:)
       AER_COEFF(SCI,:,2) = ALIN_AER_S(ALIN_NWL,:)
       AER_COEFF(EXI,:,1) = ALIN_AER_S(1,:) + ALIN_AER_A(1,:)
       AER_COEFF(EXI,:,2) = ALIN_AER_S(ALIN_NWL,:) + ALIN_AER_A(ALIN_NWL,:)
    ELSE
       
       IF( SHARP_EDGES ) GOTO 138
       
       STR_INP = 'Extinction coefficient'
       CALL Read_Input (STR_INP, READ_ARR, *880)
       TXT = DIR_name//CUT_STR(READ_ARR(1)%S)
       INQUIRE(FILE=TXT, EXIST=CheckFlg)
       IF (ALLOCATED(Z_EXT)) DEALLOCATE(Z_EXT, EXT_COEFF)
       If (CheckFlg) Then
          Call Read_Text (TXT, ReadArr, Inf, SkipHeader=.True.,&
               SkipEmpty=.True.)
          If(Inf /= 0) GoTo 106
          DeAllocate (READ_ARR)
          Allocate (READ_ARR(Size(ReadArr)))
          ForAll(I = 1:Size(ReadArr)) READ_ARR(I)%S = ReadArr(I)%S
       Else
          If(.Not.IsNum(READ_ARR(1)%S)) GoTo 107
       End If
       Call Split_String(READ_ARR(1)%S, STR_ARR)
       If(Size(STR_ARR) /= NAW + 1) GoTo 108
       NZE = SIZE( READ_ARR )
       ALLOCATE( EXT_COEFF(NZE,NAW), Z_EXT(NZE) ) ! A
       DO I = 1, NZE
          READ(READ_ARR(I)%S,*,ERR=101) Z_EXT(I), (EXT_COEFF(I,K), K=1,NAW)
       END DO

       IF( AER_SHIFT ) Z_EXT(:) = Z_EXT(:) + GROUND_HEIGHT
       
       DO K = 1,NAW
          EXT_COEFF(:,K) = AER_SCALING*EXT_COEFF(:,K)
          CALL LIN_INT (NZE, Z_EXT, EXT_COEFF(:,K), NLAYERS,  &
               ZHOLD, AER_COEFF(EXI,:,K), 'Z')
       ENDDO
            
!  DO I = 1, NLAYERS
!!     IF( AER_COEFF(EXI,I,1)==0.d0 ) CYCLE
!!     IF( ZHOLD(I) > 8.d0 ) CYCLE
!     WRITE(*,'(2x, f8.4, *(2x,E12.4))') &
!          ZHOLD(I), (AER_COEFF(EXI,I,K), K = 1,NAW)
!  ENDDO
!  STOP  '    Stop in "man_aer_setup"'

!  Reading single scattering albedo (all layers and wavelengths)
       STR_INP = 'Single scattering albedo'
       CALL Read_Input (STR_INP, READ_ARR, *880)
       TXT = DIR_name//CUT_STR(READ_ARR(1)%S)
       INQUIRE(FILE=TXT, EXIST=CheckFlg)
       IF (ALLOCATED(Z_ABS)) DEALLOCATE(Z_EXT, SSA_TMP, AER_SSA)
       If (CheckFlg) Then
          Call Read_Text (TXT, ReadArr, Inf, SkipHeader=.True.,&
               SkipEmpty=.True.)
          If(Inf /= 0) GoTo 106
          DeAllocate (READ_ARR)
          Allocate (READ_ARR(Size(ReadArr)))
          ForAll(I = 1:Size(ReadArr)) READ_ARR(I)%S = ReadArr(I)%S
       Else
          If(.Not.IsNum(READ_ARR(1)%S)) GoTo 107
       End If
       Call Split_String(READ_ARR(1)%S, STR_ARR)
       If(Size(STR_ARR) /= NAW + 1) GoTo 108
       NZA = SIZE( READ_ARR )
       ALLOCATE( SSA_TMP(NZA,NAW), Z_ABS(NZA), AER_SSA(NLAYERS) ) ! A
       DO I = 1, NZA
          READ(READ_ARR(I)%S,*,ERR=101) Z_ABS(I), (SSA_TMP(I,K), K=1,NAW)
       END DO
       
       IF( AER_SHIFT ) Z_ABS(:) = Z_ABS(:) + GROUND_HEIGHT
       
       IF( Z_EXT(NZE) > Z_ABS(NZA) ) GOTO 105
       
       DO K = 1,NAW
          CALL LIN_INT (NZA, Z_ABS, SSA_TMP(:,K), NLAYERS, ZHOLD, &
               AER_SSA(:), 'Z')
          AER_COEFF(SCI,:,K) = AER_COEFF(EXI,:,K)*AER_SSA(:)
       ENDDO
    ENDIF

    CALL ST_TRACK (R_NAME, 1, 'O')
    RETURN
    
880 WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)
    RETURN 1
    
101 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Reading error')
    WRITE(EU,*) '    Parameter to read: ', TRIM(STR_INP) 
    !      WRITE(EU,*) '    Input line       : ', TRIM(STR_READ) 
    RETURN 1
    
102 CALL WRITE_MESSAGE(R_NAME, 'ERROR', &
         'Upper aerosol layer outside the atmosphere')
    WRITE(EU,'(A,2x,F9.2,A,F9.2)') &
         '    Upper aerosol layer:', L_TOP(2),' - ',L_TOP(1)
    WRITE(EU,'(A,2x,F9.2,A)') &
         '      Top of atmosphere:', ZHOLD(1),' km'
    WRITE(EU,*) '    Check "man_aer.inp" and altitude grid '
    WRITE(EU,*) '    as defined in the file "control.inp" '
    RETURN 1
    
103 CALL WRITE_MESSAGE(R_NAME, 'ERROR', &
         'Lower aerosol layer outside the atmosphere')
    WRITE(EU,'(A,2x,F6.2,A)') &
         '       Top of aerosol layer:',L_TOP(LA_N),' km'
    WRITE(EU,'(A,2x,F6.2,A)') & 
         '       Surface elevation   :', ZHOLD(NLAYERS),' km'
    WRITE(EU,*) '    Check "man_aer.inp" and "Height above '
    WRITE(EU,*) '    sea level" in the file "control.inp" '
    RETURN 1
    
104 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Wrong number of lines in the field')
    WRITE(EU,*) '    Parameter to read: ', TRIM(STR_INP)
    RETURN 1
    
105 CALL WRITE_MESSAGE(R_NAME, 'ERROR',&
         'Incompatible definition of aerosol parameters')
    WRITE(EU,'(A,2x,F8.3)')&
         '    Max altitude for extinction: ', Z_ext(NZE)
    WRITE(EU,'(A,2x,F8.3)')&
         '    Max altitude for SSA:        ', Z_ABS(NZA)
    WRITE(EU,*) '    Check settings in the file "man_aer.inp"'
    RETURN 1

106 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'File is not found: '//TXT)
    WRITE(EU,*) '    Parameter to read: ', TRIM(STR_INP)
    RETURN 1
    
107 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Wrong input or file does not exist')
    WRITE(EU,*) '    Parameter: ', TRIM(STR_INP) 
    RETURN 1
    
108 CALL WRITE_MESSAGE(R_NAME, 'ERROR','The number of wavelengths '//&
         'must be the same as specified by the "Wavelength grid'//&
         '(aerosol parameters)" input field')
    WRITE(EU,*) '    Parameter: ', TRIM(STR_INP) 
    RETURN 1
    
109 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Wrong number of entries')
    WRITE(EU,*) '    Parameter: ', TRIM(STR_INP) 
    RETURN 1
    
127 CALL WRITE_MESSAGE (R_NAME, 'ERROR', 'Type of aerosol '//&
         'scattering function parameterization is not recognized')
    RETURN 1

130 CALL WRITE_MESSAGE (R_NAME, 'ERROR',& 
         '    The scattering function can be used '//&
         'as input in the scalar case only.')
    WRITE(EU,'(A)') '    Check input file "man_aer.inp".'
    RETURN 1
    
134 CALL WRITE_MESSAGE (R_NAME, 'ERROR', 'Number of matrix elements '// &
         'can not be larger than 6.')
  WRITE(EU,'(A,2(2x,F7.2))') &
       '    Check input values in the file "man_aer.inp" '
  RETURN 1
    
138 CALL WRITE_MESSAGE (R_NAME, 'ERROR', ' ')
    WRITE(EU,'(A)') '    The input parameter "Aerosol as a cloud layer?" '//&
         'is selected as "yes".'
    WRITE(EU,'(A)') &
         '    This option can not be used with the manual aerosol setup'
    RETURN 1
    
139 CALL WRITE_MESSAGE (R_NAME, 'ERROR', 'The input parameter'//&
         '"Aerosol as a cloud layer?" is not recognized')
    RETURN 1

140 CALL WRITE_MESSAGE (R_NAME, 'ERROR', 'The input parameter'//&
         '"Aerosol as a cloud layer?" selected as "yes" ')
    WRITE(EU,'(A)') &
         '    requires "Source for aerosol ..." parameter as "list"'
    WRITE(EU,'(A)') '    Check input file "man_aer.inp".'
    RETURN 1
    
141 CALL WRITE_MESSAGE (R_NAME, 'ERROR',' ')
    WRITE(EU,'(A)') ' Only single aerosol layer is allowed if the parameter'
    WRITE(EU,'(A)') '   "Use Angstrom parameterization" is set to "true".'
    WRITE(EU,'(A)') '    Check input file "man_aer.inp".'
    RETURN 1

131 CALL WRITE_MESSAGE (R_NAME, 'ERROR',&
         '    The asymmetry factor can be used '//&
         'as input in the scalar case only.')
    WRITE(EU,'(A)') '    Check input file "man_aer.inp".'
    RETURN 1
    
  END SUBROUTINE READ_MAN_AER

  !**************************************************************************
  SUBROUTINE READ_MAN_VHL(*)
    
    CHARACTER (LEN = 12), PARAMETER :: R_NAME = 'READ_MAN_VHL'
    REAL (KND) TAU_LAY, S_SA
    !==================================================================
    
    CALL ST_TRACK (R_NAME, 1, 'I')  
    
    STR_INP = 'Directory name for input files'
    CALL Read_Input (STR_INP, READ_ARR, *880)
    DIR_name = SET_PATH(READ_ARR(1)%S)
    
    ! Number of wavelength and wavelength grid
    NAW = 2 ;  NWLS = 1 ; NWLE = NAW
    
    ALLOCATE( AER_WAVS(NAW), AER_COEFF(3,NLAYERS,NAW), &
         FDM_AER(NLAYERS,NAW), TMP2(Nm_vs), IDX_AER(NLAYERS) ) ! A
    AER_WAVS(1) = START_WAV - 5.d0 ; AER_WAVS(2) = END_WAV + 5.d0
    
    FDM_AER = 0.d0  ;  AER_COEFF = 0.d0 
    
    STR_INP = 'Scattering function representation (VHL)'
    CALL Read_Input (STR_INP, READ_ARR, *880)
    PH_TYPE = CUT_STR(READ_ARR(1)%S)
    
    ! Layering of scattering function/matrix
    LA_N = 1
    
    ALLOCATE( LA_S(LA_N), LA_E(LA_N), L_TOP(LA_N), TAU(NAW), & 
         APH_VS( NLEGEN,LA_N,Nm_vs,NAW ), M_Lay_Wav(LA_N,NAW), & ! A
         Names(LA_N,NAW), USER_MOM(MUM,LA_N,Nm_vs,NAW) ) ! A
    
    APH_VS = 0.d0  ;  USER_MOM = 0.d0
    AER_PHASMOMS => APH_VS(:,:,1,:)
    Invert = .FALSE.
    
    ! Top altitude of homogeneous layer
    L_TOP(1) = 1.d0
    
    IF( AER_SHIFT ) L_TOP(:) = L_TOP(:) + GROUND_HEIGHT

    SELECT CASE (Ph_type)
    CASE('Asymmetry_factor')
       ! ****** Reading of the asymmetry factors *********
       ! ===================================================
       IF(Nvs > 1) GOTO 131
       IF (LOG_LEVEL(1) >= N_ESO) THEN
          WRITE(LUNIT,*) ' ----- "Asymmetry_factor" mode is used ---------'
          WRITE(LUNIT,*) '    Reading of the asymmetry factors  '
       ENDIF
       ALLOCATE( ASYM_PAR(LA_N,NAW) )
       
       STR_INP = 'Asymmetry factor value (VHL)'
       CALL Read_Input (STR_INP, READ_ARR, *880)
       READ(READ_ARR(1)%S,*,ERR=101) ASYM_PAR(1,1)
       AER_COEFF(ASI,:,1:2) = ASYM_PAR(1,1)
       
    CASE('Expansion_coeff')
       IF (LOG_LEVEL(1) >= N_ESO) WRITE(LUNIT,*) &
            ' ----- "Expansion_coeff" mode is used ---------'

       ! Read the file names contaning the expansion coefficients
       STR_INP = 'File name containing expansion coefficient (VHL)'
       CALL Read_Input (STR_INP, READ_ARR, *880)
       Names(1,1)%S = DIR_name//CUT_STR(READ_ARR(1)%S)
       Names(1,2)%S = Names(1,1)%S
       
    CASE('Scattering_matrix')
       IF (LOG_LEVEL(1) >= N_ESO) WRITE(LUNIT,*)  &       
            ' ----- "Scattering_matrix" mode is used ---------'
       IF(Nvs > 1) GOTO 130            
       ! Read the file name contaning the scattering function/matrix
       STR_INP = 'File name containing scattering function/matrix (VHL)'
       CALL Read_Input (STR_INP, READ_ARR, *880)
       Names(1,1)%S = DIR_name//CUT_STR(READ_ARR(1)%S)
       Names(1,2)%S = Names(1,1)%S
       
    CASE DEFAULT
       GOTO 127
    END SELECT
    
    DO I = 1,LA_N
       LA_S(I) = MINLOC( DABS(L_TOP(I)-ZHOLD(:)),1 )
       IF (LOG_LEVEL(1) >= N_ESO) WRITE(LUNIT,*) I, LA_S(I)
    ENDDO
    
    ! Indexes of altitude grid levels which are within aerosol layers
    IDX_AER(:) = 1
    IF( LA_N > 1 ) THEN
       DO I = 1,LA_N - 1
          D1 = ZHOLD(LA_S(I)) ;  D2 = ZHOLD(LA_S(I+1))
          DO J = 1,NLAYERS
             IF( ZHOLD(J) .LE. D1 .AND. ZHOLD(J) > D2 ) IDX_AER(J) = I
          ENDDO
       ENDDO
    ENDIF
    
    D1 = ZHOLD(LA_S(LA_N))
    DO J = 1,NLAYERS
       IF( ZHOLD(J) .LE. D1 ) IDX_AER(J) = LA_N
    ENDDO
    
    DO I = 1,LA_N-1
       LA_E(I) = LA_S(I+1)-1
    ENDDO
    LA_E(LA_N) = NLAYERS
    
    IF (LOG_LEVEL(1) >= N_ESO) THEN
       WRITE(LUNIT,*) ' ------ Test of layering man mode --- '
       WRITE(LUNIT,*) ' ----     homogeneous layer       --- '
       DO I = 1,LA_N
          WRITE(LUNIT,'(2x,I3,2(1x,f7.3))') I, ZHOLD(LA_S(I)) ,ZHOLD(LA_E(I)) 
       ENDDO
       WRITE(LUNIT,*) ' ----------------------------------------- '
    ENDIF
    
    ! Reading optical thickness of the layer
    STR_INP = 'Optical thickness (VHL)'
    CALL Read_Input (STR_INP, READ_ARR, *880)
    READ (READ_ARR(1)%S, *, ERR = 101) TAU_LAY
    AER_COEFF(EXI,:,:) = TAU_LAY
    
    ! Reading single scattering albedo of layer
    STR_INP = 'Single scattering albedo (VHL)'
    CALL Read_Input (STR_INP, READ_ARR, *880)
    READ (READ_ARR(1)%S, *, ERR = 101) S_SA
    AER_COEFF(SCI,:,:) = AER_COEFF(EXI,:,:)*S_SA
    
    CALL ST_TRACK (R_NAME, 1, 'O')  
    RETURN
    
880 WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)
    RETURN 1
    
101 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Reading error')
    WRITE(EU,*) '    Parameter to read: ', TRIM(STR_INP) 
    !WRITE(EU,*) '    Input line       : ', TRIM(STR_READ) 
    RETURN 1
    
127 CALL WRITE_MESSAGE (R_NAME, 'ERROR', 'Type of '//&
         'scattering function parameterization is not recognized')
    RETURN 1
    
130 CALL WRITE_MESSAGE (R_NAME, 'ERROR', &
         '    The scattering function can be used '//&
         'as input in the scalar case only.')
    WRITE(EU,'(A)') '    Check input file "man_vhl.inp".'
    RETURN 1
    
131 CALL WRITE_MESSAGE (R_NAME, 'ERROR', &
         '    The asymmetry factor can be used '//&
         'as input in the scalar case only.')
    WRITE(EU,'(A)') '    Check input file "man_vhl.inp".'
    RETURN 1
    
  END SUBROUTINE READ_MAN_VHL

  !********************************************************************
  !                  Delta-fit
  !*********************************************************************
  SUBROUTINE Start_Delta_Fit( N_ANG, Nleg_df )
    INTEGER     N_ang, Nleg_df
    !     Local variables
    REAL (KND)  Kat_ang
    REAL (KND), DIMENSION (N_ANG) ::  X_gau, AG_ang
    !------------------------------------------------------------------

    ! Cut-angle ]deg]
    Kat_ang = 2.0d0
    Kat_ang = DCOS( Kat_ang/RAD_TO_DEG )
    
    CALL GAULEG ( -1.D0, 1.D0, X_gau, AG_ang, N_ANG )
    
    ! Invert Gauss grid to start from zero scattering angle 
    X_ang(:) = X_gau(N_ang:1:-1)

    ! Legendre polynomials for all Gauss angles and all 
    ! moments of the phase function Nleg_df
    CALL WIGNER_PLG( X_ang, N_ang, 0, Nleg_df, PL_RES )

    ! Calculate phase function for all Gauss nodes
    DO I = 1,N_ang
       IF( X_ang(I) < Kat_ang ) Then
          N_KAT = I
          EXIT
       ENDIF
    ENDDO
    IF (LOG_LEVEL(1) >= N_ESO) THEN
       WRITE(LUNIT,*) 
       WRITE(LUNIT,*) ' ***** Cut-angle for Delta-fit approximation **** '
       WRITE(LUNIT,'(2x,i3,2x,e12.4)') &
            N_KAT,DACOS(X_ang(N_KAT))*RAD_TO_DEG
       WRITE(LUNIT,*) 
    END IF
    
  END SUBROUTINE Start_Delta_Fit
  !********************************************************************
  
  SUBROUTINE Delta_Fit( N_ANG, Nleg_df, Phas_moms )
    INTEGER     N_ang, Nleg_df
    REAL (KND)  Phas_moms(Nleg_df)
    ! Local variables
    INTEGER     I, K
    REAL (KND), DIMENSION (N_ANG) :: Phm1, Phm1_RD
    REAL (KND)  A_VEC( (Nleg_loc + 1)*(Nleg_loc + 2)/2 ),&
         PL_Nor( Nleg_loc,N_ANG ), PL_Nor1( Nleg_loc,N_ANG ),&
         A_M( Nleg_loc,Nleg_loc )
    !------------------------------------------------------------------

    DO I = 1, N_ANG
       Phm1(I) = SUM( PL_RES(:,I)*Phas_moms(:) )
    ENDDO
    
    PL_NOR1 = 0.0d0
    DO K = 1,N_ANG
       PL_NOR(:,K) = PL_RES(1:Nleg_loc,K)/Phm1(K)
    ENDDO
    PL_NOR1(:,N_KAT:N_ang) = PL_NOR(:,N_KAT:N_ang)
    A_M(:,:) = MATMUL( PL_NOR(:,:),TRANSPOSE(PL_NOR1(:,:)) )
    
    FORALL (I = 1:Nleg_loc) A_VEC(I*(I - 1)/2 + 1:I*(I + 1)/2) = A_M(I,1:I)
    !WRITE(*,*) A_VEC
    !WRITE(*,*) '  CHINV3 ' 
    CALL CHINV3 (Nleg_loc*(Nleg_loc+1)/2, Nleg_loc, A_VEC)
    !WRITE(*,*) '  After CHINV3'
    
    FORALL (I = 1:Nleg_loc) A_M(I,1:I) = A_VEC(I*(I - 1)/2 +1:I*(I + 1)/2)
    FORALL (I = 1:Nleg_loc, J = 1:Nleg_loc, J > I) A_M(I,J) = A_M(J,I)
    !WRITE(*,*) '   ... Calculation R_COF'
    R_COF(:) = MATMUL( A_M,SUM( PL_NOR1(:,:),2 ) )
    F_Dfit_loc   = 1.0d0 - R_COF(1)
    !write( *,* ) ' ---> R_COF(1) = ', R_COF(1)

    DO I = 1, N_ANG
       Phm1_RD(I) = SUM( PL_RES(1:Nleg_loc,I)*R_COF(1:Nleg_loc) )
    ENDDO
    
    OPEN( 44, file = DIR_OUT//'/phase.test' )
    DO I = 1, N_ANG
       WRITE( 44,'(5(2x,e15.7))') &
            X_ANG(I), DACOS(X_ANG(I))/3.14159*180., Phm1(I), Phm1_RD(I)
    ENDDO
    CLOSE(44)
    ! stop '...... End of Delta_Fit .... '

  END SUBROUTINE Delta_Fit

  ! *******************************************************************
  SUBROUTINE AER_DM
    IF (LOG_LEVEL(1) >= N_ESO) THEN
       WRITE(LUNIT,*) '   ***********************  '
       WRITE(LUNIT,*) '   Aerosol D-M scalar case  '
       WRITE(LUNIT,*) '   ***********************  '
    ENDIF
    DO I1 = 1,LA_N
       DO K = 1,NAW
          F_DELTAM = USER_MOM(Itr+1,I1,1,K)/DBLE(2*Itr + 1)
          FDM_AER(LA_S(I1):LA_E(I1),K) = F_DELTAM
          IF (LOG_LEVEL(1) >= N_ESO) &
               WRITE(LUNIT,*) '   Delta-m AEr ', F_DELTAM
          DO L = 1,Itr
             AER_PHASMOMS(L,I1,K) = (USER_MOM(L,I1,1,K) -&
                  F_DELTAM*DBLE(2*L-1))/(1.D0 - F_DELTAM)
          ENDDO               ! Over moments
       ENDDO                  ! Over wavelengths
    ENDDO                     ! Over layers
    AER_PHASMOMS(Itr+1:NLEGEN,:,NWLS:NWLE ) = 0.0D0
    
  END SUBROUTINE AER_DM

  !*******************************************************************
  SUBROUTINE AER_DM_VEC
    IF (LOG_LEVEL(1) >= N_ESO) THEN
       WRITE(LUNIT,*) '   ***********************  '
       WRITE(LUNIT,*) '   Aerosol D-M vector case  '
       WRITE(LUNIT,*) '   ***********************  '
    ENDIF
    DO I1 = 1,LA_N
       ! F_DELTAM = USER_MOM(Itr+1,I1,1)/DBLE(2*Itr + 1)
       DO K = 1,NAW
          F_DELTAM = USER_MOM(Itr+1,I1,1,K)/DBLE(2*Itr + 1)
          FDM_AER(LA_S(I1):LA_E(I1),K) = F_DELTAM
          DO L = 1, Itr
             APH_VS(L,I1,1:4,K) = (USER_MOM(L,I1,1:4,K) -&
                  F_DELTAM*DBLE(2*L-1))/(1.0D0 - F_DELTAM)
             APH_VS(L,I1,5:6,K) = &
                  USER_MOM(L,I1,5:6,K)/(1.0D0 - F_DELTAM)
          ENDDO
       ENDDO
    ENDDO
    APH_VS(Itr+1:NLEGEN,:,:,:) = 0.0D0
    
  END SUBROUTINE AER_DM_VEC

  ! *******************************************************************
  SUBROUTINE WRITE_SCEN_MN(*)
    USE  AEROSOLS, ONLY: OWL_AER, NOWL_AER
    USE TOOLS,     ONLY: OpenOut, WriteOut, CloseOut 
    
    CHARACTER (LEN=:), Allocatable :: OUT_NAME
    Character (Len=500)  OutStr
    INTEGER     O_WL, Unt
    REAL (KND)  SS_A, Ext, Sca_co, Abs
    REAL (KND), ALLOCATABLE :: T_OUT(:)
    ! ------------------------------------------------------------------
    
    IF( VHL ) THEN
       OUT_NAME = 'SCE_VHL.OUT' ; O_WL = 1
    ELSE
       OUT_NAME = 'SCE_AEROSOL.OUT' ; O_WL = NAW
    ENDIF
    
    IF (LOG_LEVEL(1) >= N_SSO) &
         WRITE (LUNIT,'(A,A)') ' *** Writing: ',TRIM(OUT_NAME)

    ! Optical thickness at user defined wavelength grid
    IF( NOWL_AER > 0 ) ALLOCATE( T_OUT(NOWL_AER) )
    DO I = 1, NOWL_AER
       CALL LIN_SEARCH(OWL_AER(I), DUM, K, NAW, AER_WAVS, ERROR)
       IF (ERROR) GOTO 133
       T_OUT(I) =  (1.d0 - DUM)*TAU(K) + DUM*TAU(K+1)
    ENDDO
    
    Call OpenOut (OUT_NAME, Unt)
    CALL WRITE_UD(1, Unt)
    IF( VHL ) THEN
       WRITE(OutStr,*) '           ========================'
       Call WriteOut (Unt, OutStr)
       WRITE(OutStr,*) '            MANUAL LAYER SELECTION'
       Call WriteOut (Unt, OutStr)
       WRITE(OutStr,*) '           ========================'
       Call WriteOut (Unt, OutStr)
    ELSE
       WRITE(OutStr,*) '          =========================='
       Call WriteOut (Unt, OutStr)
       WRITE(OutStr,*) '           MANUAL AEROSOL SELECTION'
       Call WriteOut (Unt, OutStr)
       WRITE(OutStr,*) '          =========================='
       Call WriteOut (Unt, OutStr)
    ENDIF
    IF( ALIN_ACT(3) ) THEN
       WRITE(OutStr,*) '           Alternative reading mode'
       Call WriteOut (Unt, OutStr)
    ENDIF
    
    IF( Scal_aer ) THEN
       WRITE(OutStr,'(A,F7.3,A,F9.2,A)') &
            '   !!! ATTENTION !!! AOT is set to ',Rtau_aer,' at',&
            Rwl_aer,' nm wavelength.'
       Call WriteOut (Unt, OutStr)
       WRITE(OutStr,'(19x,A)') '  See setting in "control_aer.inp" '
       Call WriteOut (Unt, OutStr)
       WRITE(OutStr,'(6x,A)') &
            '----------------------------------------------------'
       Call WriteOut (Unt, OutStr)
       WRITE(OutStr,*) 
       Call WriteOut (Unt, OutStr)
    ENDIF
    
    IF( NOWL_AER > 0 ) THEN
       WRITE ( OutStr, * ) ' AOT at user-defined wavelengths [nm]:'
       Call WriteOut (Unt, OutStr)
       DO I = 1, NOWL_AER
          WRITE(OutStr,'(12x,F7.2,4x,F8.4)') OWL_AER(I),T_OUT(I)
          Call WriteOut (Unt, OutStr)
       ENDDO
    ENDIF
    
    WRITE ( OutStr, * ) ' '
    Call WriteOut (Unt, OutStr)
    WRITE ( OutStr, * ) ' Vertical profiles of:'
    Call WriteOut (Unt, OutStr)
    WRITE ( OutStr, * )'         extinction coefficient (Ext coeff) and'
    Call WriteOut (Unt, OutStr)
    WRITE ( OutStr, * ) '         single scattering albedo (Ss albedo):'
    Call WriteOut (Unt, OutStr)

    DO K = 1, O_WL
       
       !WRITE(LUNIT,'(1x,a11,F9.2,a6,a6,F8.4)') &
       !        'Wavelength ',AER_WAVS(K),' [nm];',' AOT =',TAU(K)
       
       WRITE(OutStr,*) '   '
       Call WriteOut (Unt, OutStr)
       IF( VHL ) THEN
          WRITE(OutStr,'(1x,a,F8.4)') ' Optical thickness ',TAU(K)
          Call WriteOut (Unt, OutStr)
          WRITE(OutStr,*)' --------------------------'
          Call WriteOut (Unt, OutStr)
       ELSE
          WRITE(OutStr,'(1x,a11,F9.2,a6,a6,F8.4)') &
               'Wavelength ',AER_WAVS(K),' [nm];',' AOT =',TAU(K)
          Call WriteOut (Unt, OutStr)
          WRITE(OutStr,*) '++++++++++++++++++++++++++++++++++++++++++++++'
          Call WriteOut (Unt, OutStr)
       ENDIF
       WRITE ( OutStr, * ) '    Number, Alt [km], Ext coeff [km-1], Ss albedo'
       Call WriteOut (Unt, OutStr)
       DO I = 1, NLAYERS
          Ext = AER_COEFF(EXI,I,K)
          Abs = AER_COEFF(EXI,I,K) - AER_COEFF(SCI,I,K)
          Sca_co = AER_COEFF(SCI,I,K)/(1.D0 - FDM_AER(I,K))
          IF( Ext == 0.0d0 ) CYCLE
          SS_A = Sca_co/(Sca_co + Abs)
          WRITE(OutStr,'(6x,I3,4x,F8.4,5x,ES9.2,6x,E12.4)') &
               I, ZHOLD(I), Sca_co+Abs, SS_A
          Call WriteOut (Unt, OutStr)
       END DO
       WRITE(OutStr,*) ' '
       Call WriteOut (Unt, OutStr)
       
       SELECT CASE (Ph_type)
       CASE('Asymmetry_factor')
          WRITE(OutStr,*) '    Asymmetry factor used in the aerosol layers :'
          Call WriteOut (Unt, OutStr)
          WRITE(OutStr,*) '    ------------------------------------------- '
          Call WriteOut (Unt, OutStr)
          WRITE(OutStr,*) '    Layer  Top [km]  Bottom [km]   Asymmetry factor'
          Call WriteOut (Unt, OutStr)
          DO I = 1,LA_N
             WRITE(OutStr,'(6x,I3,3x,F6.2,4x,F6.2,14x,F6.3)') &
                  I, ZHOLD(LA_S(I)) ,ZHOLD(LA_E(I)), Asym_Par(I,K)
             Call WriteOut (Unt, OutStr)
          ENDDO
       CASE('Expansion_coeff')
          WRITE(OutStr,*) &
               '    Expansion coefficients used in the aerosol layers :'
          Call WriteOut (Unt, OutStr)
          WRITE(OutStr,*) &
               '    ------------------------------------------------- '
          Call WriteOut (Unt, OutStr)
          WRITE(OutStr,*) '    Layer  Top [km]  Bottom [km]   File name'
          Call WriteOut (Unt, OutStr)
          DO I = 1,LA_N
             WRITE(OutStr,'(6x,I3,3x,F6.2,4x,F6.2,6x,a)') &
                  I, ZHOLD(LA_S(I)) ,ZHOLD(LA_E(I)), Names(I,K)%S
             Call WriteOut (Unt, OutStr)
          ENDDO
       CASE('Scattering_matrix')
          WRITE(OutStr,*) &
               '    Scattering matrix used in the aerosol layers :'
          Call WriteOut (Unt, OutStr)
          WRITE(OutStr,*) '    -------------------------------------------- '
          Call WriteOut (Unt, OutStr)
          WRITE(OutStr,*) '    Layer  Top [km]  Bottom [km]   File name'
          Call WriteOut (Unt, OutStr)
          DO I = 1,LA_N
             WRITE(OutStr,'(6x,I3,3x,F6.2,4x,F6.2,6x,a)')&
                  I, ZHOLD(LA_S(I)) ,ZHOLD(LA_E(I)), Names(I,K)%S
             Call WriteOut (Unt, OutStr)
          ENDDO
       END SELECT
       WRITE(OutStr,*) ' '
       Call WriteOut (Unt, OutStr)
       WRITE(OutStr,*) &
            '    Number of expansion coeff. and Delta-M factor:'
       Call WriteOut (Unt, OutStr)
       WRITE(OutStr,*) '    --------------------------------------------- '
       Call WriteOut (Unt, OutStr)
       WRITE(OutStr,*) '    Layer  Top [km]  Bottom [km]   N coef   Delta-M'
       Call WriteOut (Unt, OutStr)
       DO I = 1,LA_N
          WRITE(OutStr,'(6x,I3,3x,F6.2,4x,F6.2,9x,I4,2x,E10.2)')&
               I, ZHOLD(LA_S(I)) ,ZHOLD(LA_E(I)), &
               M_Lay_Wav(I,K), FDM_AER(LA_S(I),K)
          Call WriteOut (Unt, OutStr)
       ENDDO
    ENDDO                     ! Over wavelengths of aerosol DB
    CALL WRITE_UD(2,Unt)
    Call CloseOut (Unt)
    
    RETURN
    
133 CALL WRITE_MESSAGE (R_NAME,'ERROR',&
         'Output wavelength for AOT is out of range')
    WRITE(EU,'(A,F7.2)') '    Output wavelength: ', OWL_AER(I)
    WRITE(EU,'(A,2(2x,F7.2))') &
         '    Database wavelengths: ', AER_WAVS(1), AER_WAVS(NAW)
    WRITE(EU,'(A)') &
         '    Set output wavelength between database wavelengths'
    WRITE(EU,'(A)') '    in the "control_aer.inp" file.'
    RETURN 1
    
  END SUBROUTINE WRITE_SCEN_MN

    ! ******************************************************************* 
    SUBROUTINE OUT_COEFF
      OPEN(OUNIT,FILE=DIR_OUT//'/H_G_coeff.out')
      WRITE(OUNIT,'(a)') '#  Expansion coefficients of the H-G phase function'
      WRITE(OUNIT,'(a)') '#  in the lower aerosol layer at first wavelength. '
      WRITE(OUNIT,'(a,2x,F6.3)') '#  Asymmetry parameter :', ASYM_PAR(1,1)
      DO L = 1,NU_A
         WRITE(OUNIT,'(2x,I5,2x,E15.7)') L-1, USER_MOM(L,1,1,1)
      ENDDO
      CLOSE(OUNIT)
    END SUBROUTINE OUT_COEFF

    !***********************************************************************
    SUBROUTINE DEALLOCATE_ARRAYS
      
      IF(ALLOCATED(DENS_PRF)) DEALLOCATE(DENS_PRF)
      IF(ALLOCATED(Greek_aer)) DEALLOCATE(Greek_aer)
      IF(ALLOCATED(FDM_AER)) DEALLOCATE(FDM_AER, AER_WAVS, AER_COEFF, &
           IDX_AER, LA_S, LA_E, L_TOP, APH_VS)

      RETURN
    END SUBROUTINE DEALLOCATE_ARRAYS

!***********************************************************************
  END SUBROUTINE MAN_AER_SETUP
!**************************************************************************
