!***************************************************************************
!
!	SCIATRAN - Version 4.7 - Last modification: 03 Jan 2024
!
!	Copyright (C) - Institute of Remote Sensing,
!			University of Bremen, Germany
!
!***************************************************************************
SUBROUTINE MAN_HSOL_SETUP(START_WAV, END_WAV, *)
  ! Called by "rtm_init.f"
                              
  USE CLOUDS_SS,   ONLY: MUM => MAXMOM, DO_SSRCA
!  USE DELTA_M,     ONLY: F_DLM_HSOL, DO_HSOL_DM, HSOL_TRUNC_IDX
  USE DELTA_M,     ONLY: DO_HSOL_DM, HSOL_TRUNC_IDX, F_DLM_HSOL
  USE HYDROSOLS,   ONLY: NWLS => NWLS_HS, NWLE => NWLE_HS, LA_N => LA_N_HS,& 
                         IDX_HS, LA_S => LA_S_HS, LA_E => LA_E_HS, &
                         HS_SCALING, Greek_HS, NG_HS, Scal_HS, Rtau_HS, Rwl_HS,&
                         L_BOT_HS, OUT_H_SCM, LAY_H, WEL_H, NSCA_H, PATH_H_SCM,&
                         GE_H, INT_SCM_H, CRIT_SCM_H
  USE INTERN_DIMS, ONLY: Nm_vs, Nvs
  USE LOG_PARS,    ONLY: LUNIT, EU, DIR_OUT, LOG_LEVEL, N_SSO, N_ESO, C_UNIT,&
                         Work_Dir
  USE ST_VARS,     ONLY: RTD => RAD_TO_DEG, Vec_Sca, RTM_TYPE
  USE TOOLS,       ONLY: READ_INPUT, SET_PATH, CUT_STR, SPLIT_STRING, IsNum, &
                         UPPER_LOWER_CASE
  USE TYPES,       ONLY: KND, DYN_CHAR
  USE WATER,       ONLY: NL_UW => NLAYERS_UW, ZZZ_UW, DZ_UW, NLEG_W
  USE WATER_ICE,   ONLY: IWS, IWT, LA_N_IW

  REAL (KND), INTENT(IN):: START_WAV, END_WAV
  ! Local parameters
  CHARACTER (LEN = *), PARAMETER :: R_NAME = 'MAN_HSOL_SETUP'

  !  (a) Input variables
  !  ===================
  !      DO_HSOL_DM        : delta-M approximation yes/no 
  !                          <---- From Module DELTA_M
  !      HSOL_TRUNC_IDX    : truncation index for delta-M approximation 
  !                          <---- From Module DELTA_M
  !      ZZZ_UW             : forward model altitude grid in ocean
  !      NL_UW             : number of layers in altitude grid in ocean
  !      NLEG_W            : number of Legendre moments in water
  !      LUNIT             : LOG file unit number

  !  (b) Output variables
  !  ====================
  !      HS_COEFF         : scattering and extinction coefficients
  !      HS_PHASMOMS      : Legendre moments
  !      NAW              : number of wavelengths of chosen hydrosol 
  !                         optical parameters
  !      HS_WAVS          : wavelengths of hydrosol data base (depends on
  !                         parameterization chosen)
  INTEGER     N_Z_EXT, N_Z_ABS, Nel
  LOGICAL     ERROR
  CHARACTER (LEN=:), ALLOCATABLE :: DIR_name, Ph_type
  CHARACTER (LEN=200)  STR_READ, STR_INP, STR_READ_ARR(90)
  Type (Dyn_Char), Allocatable :: Names(:,:), Ex_Nam(:)
  CHARACTER (LEN=3),  ALLOCATABLE :: ESM(:)
  INTEGER   I, Ii, I1, I2, J, K, L, IUNIT, OUNIT, Itr, NU_A, N_GAU, &
       M_cof, M_ang, NAW,  IiW
  INTEGER,  ALLOCATABLE :: M_Lay_Wav(:,:)
  REAL (KND)  Scoe, TMP1, GG, DUM, D1, D2
  REAL (KND), ALLOCATABLE :: Z_EXT(:), Z_ABS(:), TMP2(:), TAU(:), &
       DUMM(:), EXT_COEFF(:,:), SSA_TMP(:,:), HS_SSA(:), ASYM_PAR(:,:)!,   &
!       F_DLM_HSOL(:,:,:)
  REAL (KND), ALLOCATABLE, TARGET :: USER_MOM(:,:,:,:)
  REAL (KND), POINTER :: HS_PHASMOMS(:,:,:), HS_WAVS(:)
  PARAMETER ( IUNIT = 33, OUNIT = 55 )
! ... From the structure IWS
  INTEGER  N_HPC  
  ! Variable for Delta-M
  ! ======================
  REAL (KND)  F_DEL, ABS_COEFF
  TYPE (DYN_CHAR), ALLOCATABLE :: READ_ARR(:), STR_ARR(:)
  !==================================================================
  INTERFACE
     SUBROUTINE SCM_TO_EC(MAX_CO, N_GAU, LA_N, NAW, Nel, ESM, Names, INT_SCM, &
          CRIT_SCM, Lay_out, WL_out, OUT_SCM, M_Lay_Wav, USER_MOM, OUT_PATH, *)
       Use Cld_SS,  ONLY: MUM => MAXMOM
       USE I_DIMS,   ONLY: Nm_vs, Nvs
       USE LOG_PARS, ONLY: LUNIT, EU, DIR_OUT, LOG_LEVEL, N_ESO, C_UNIT
       USE ST_VARS,  ONLY: Pie, RTD => RAD_TO_DEG
       USE TYPES,    ONLY: KND, DYN_CHAR
       INTEGER, INTENT(IN) :: MAX_CO, N_GAU, Nel, Lay_out, WL_out, LA_N, NAW
       INTEGER   M_Lay_Wav(LA_N,NAW)
       LOGICAL   OUT_SCM
       REAL (KND) USER_MOM(MUM,LA_N,Nm_vs,NAW), CRIT_SCM
       CHARACTER (LEN=3)  ESM(Nel), INT_SCM
       CHARACTER (LEN=*)   OUT_PATH
       TYPE (DYN_CHAR), ALLOCATABLE :: Names(:,:)      
     END SUBROUTINE SCM_TO_EC
  END INTERFACE

  CALL ST_TRACK (R_NAME, 1, 'I')  

  ERROR = .FALSE.

  IiW = 2
  N_HPC = IWS(IiW)%N_HPC

  ! =============================
  ! USER-DEFINED PARAMETERIZATION
  ! =============================
  STR_INP = 'Directory name for scattering matrixes (hydrosols)'
  CALL Read_Input (STR_INP, READ_ARR, *880)
  DIR_name = SET_PATH(READ_ARR(1)%S)

  STR_INP = 'Wavelength grid for hydrosols'
  CALL Read_Input (STR_INP, READ_ARR, *880, ReArrange = .TRUE.)
  NAW = SIZE(READ_ARR) ;  NWLS = 1 ;  NWLE = NAW
  IWS(IiW)%N_HS_WL = NAW

  if(allocated(IWS(IiW)%HS_WL)) DeAllocate (IWS(IiW)%HS_WL, IWS(IiW)%HS_COEFF)
  if(allocated(F_DLM_HSOL)) DeAllocate (F_DLM_HSOL)
  if(allocated(TMP2)) DeAllocate (TMP2)
  if(allocated(IDX_HS)) DeAllocate (IDX_HS)
  if(allocated(DUMM)) DeAllocate (DUMM)
  ALLOCATE( IWS(IiW)%HS_WL(NAW), IWS(IiW)%HS_COEFF(2,NL_UW,NAW,N_HPC),  &
       TMP2(Nm_vs), F_DLM_HSOL(NL_UW,NAW,N_HPC), IDX_HS(NL_UW), DUMM(NAW) ) ! A
  F_DLM_HSOL = 0.D0  ;  IWS(IiW)%HS_COEFF = 0.D0 

  HS_WAVS => IWS(IiW)%HS_WL(:)
  ! Reading of wavelength grid for hydrosol optical parameters
  DO I = 1, NAW
     READ(READ_ARR(I)%S,*,ERR=101) HS_WAVS(I)
!     WRITE(*,'(4x,I3,2x,f6.1)') I, HS_WAVS(I)
  END DO

  !  Check the position of start/end wavelength with respect
  !  to the wavelength range required for RT calculations
  IF( HS_WAVS(1)    > START_WAV ) GOTO 104
  IF( HS_WAVS(NWLE) < END_WAV )   GOTO 105

  STR_INP = 'Scattering function representation for hydrosols'
  CALL Read_Input (STR_INP, READ_ARR, *880)
  Call Split_String(READ_ARR(1)%S, STR_ARR)
  I1 = SIZE(STR_ARR)
  PH_TYPE = CUT_STR(STR_ARR(1)%S)
  IF( I1==1 ) THEN
     INT_SCM_H = 'lin' ; CRIT_SCM_H = 0.1d0 ! Default settings 
  ELSE
     IF( I1==2.OR.I1 > 3 ) GOTO 110
     CALL UPPER_LOWER_CASE (STR_ARR(2)%S,'U2L')
     INT_SCM_H = CUT_STR(STR_ARR(2)%S)
     READ(STR_ARR(3)%S,*,ERR = 101)  CRIT_SCM_H
     SELECT CASE(INT_SCM_H)
     CASE('aki','lin','spl')
     CASE DEFAULT
        CALL NOT_FOUND (R_NAME, STR_INP, INT_SCM_H, *880)    
     END SELECT
   ENDIF
!  WRITE(*,*) ' Man-hydrosol settings '
!  WRITE(*,*) '   1 - ', PH_TYPE
!  WRITE(*,*) '   2 - ', INT_SCM_H
!  WRITE(*,*) '   3 - ', CRIT_SCM_H 

  STR_INP = 'Layering of scattering function/matrix (hydrosols)'
  CALL Read_Input (STR_INP, READ_ARR, *880, ReArrange = .TRUE.)
  LA_N = SIZE(READ_ARR)

  if(allocated(LA_S)) DeAllocate(LA_S, LA_E, L_BOT_HS)
  if(allocated(TAU)) DeAllocate(TAU, M_Lay_Wav, Names, USER_MOM)
  ALLOCATE( LA_S(LA_N), LA_E(LA_N), L_BOT_HS(LA_N), TAU(NAW), &!A
       IWS(IiW)%HSOL_PHAS_VS( NLEG_W,LA_N,Nm_vs,NAW,N_HPC ), M_Lay_Wav(LA_N,NAW), &!A
       Names(LA_N,NAW), USER_MOM(MUM,LA_N,Nm_vs,NAW) ) ! A

  IWS(IiW)%HSOL_PHAS_VS = 0.d0  ;  USER_MOM = 0.d0
  HS_PHASMOMS => IWS(IiW)%HSOL_PHAS_VS(:,:,1,:,1)

  ! Read bottom altitude of all hydrosol layers
  ! ===========================================
  DO I = 1, LA_N
     READ(READ_ARR(I)%S,*,ERR=101) L_BOT_HS(I)
  END DO
  IF( LA_N > 1 ) THEN
     IF( L_BOT_HS(2) < L_BOT_HS(1) )  GOTO 106
  ENDIF

  SELECT CASE (Ph_type)
  CASE('Asymmetry_factor')
     ! ****** Reading of the asymmetry factors *********
     IF(Nvs > 1) GOTO 133
     ALLOCATE( ASYM_PAR(LA_N,NAW) )
     STR_INP = 'Asymmetry factor values (hydrosols)'
     CALL Read_Input (STR_INP, READ_ARR, *880)
     IF(SIZE(READ_ARR) /=  LA_N) GOTO 101
     DO I = 1, LA_N
        Call Split_String(READ_ARR(I)%S, STR_ARR)
        If( Size(STR_ARR) > 1 ) Then
           If(Size(STR_ARR) /= NAW) GoTo 109
           READ(READ_ARR(I)%S,*,ERR=101) (ASYM_PAR(I,K), K=1,NAW)
        Else
           READ(READ_ARR(I)%S,*,ERR=101) ASYM_PAR(I,1)
           If(NAW > 1) ASYM_PAR(I,2:NAW) = ASYM_PAR(I,1)
        End If
     END DO  ! over hydrosol layers (I)

  CASE('Expansion_coeff')
     STR_INP = 'File names for expansion coefficients (hydrosols)'
     CALL Read_Input (STR_INP, READ_ARR, *880)
     IF(SIZE(READ_ARR) /= LA_N) GOTO 101
     DO I = 1, LA_N
        Call Split_String(READ_ARR(I)%S, STR_ARR)
        If(Size(STR_ARR) > 1) Then
           If(Size(STR_ARR) /= NAW) GoTo 109
           DO K = 1,NAW
              Names(I,K)%S = DIR_name//'/'//CUT_STR(STR_ARR(K)%S)
           ENDDO
        Else
           Names(I,1)%S = DIR_name//'/'//CUT_STR(STR_ARR(1)%S)
           If(NAW == 1) Cycle
           Do K = 2,NAW 
              Names(I,K)%S = Names(I,1)%S
           End Do
        End If
     End DO  ! over hydrosol layers (I)
  CASE('Scattering_matrix')
     IF (LOG_LEVEL(1) >= N_ESO) WRITE(LUNIT,*) &   
          ' ----- "Scattering_matrix" mode is used ---------'
     ! Read the file names contaning the scattering function/matrix
     Nel = 1
     IF( Nvs > 1 ) THEN
        STR_INP = 'Sequence of scattering matrix elements (hydrosols)'
        CALL Read_Input (STR_INP, READ_ARR, *880)
        CALL SPLIT_STRING(READ_ARR(1)%S, STR_ARR)
        Nel = SIZE(STR_ARR)
        ALLOCATE( ESM(Nel) )
        IF( Nel > 6 ) GOTO 131
        DO I = 1, Nel
           READ(STR_ARR(I)%S,*,ERR=101)  ESM(I)
           SELECT CASE( ESM(I) )
           CASE('F11','F22','F33','F44','F12','F21','F34','F43')
           CASE DEFAULT
              CALL NOT_FOUND (R_NAME, STR_INP, ESM(I), *880)    
           END SELECT
        ENDDO
     ENDIF  ! scattering matrix

     STR_INP = 'File names for scattering function/matrix (hydrosols)'
     CALL Read_Input (STR_INP, READ_ARR, *880)
     IF(SIZE(READ_ARR) < LA_N) GOTO 101
     DO I = 1,LA_N
        CALL SPLIT_STRING(READ_ARR(I)%S, STR_ARR)
        II = SIZE(STR_ARR)
        IF( II > 1.AND.II < NAW ) GOTO 201
        IF( II == 1 ) THEN
           DO K = 1,NAW
              Names(I,K)%S = DIR_name//CUT_STR(STR_ARR(1)%S)
           ENDDO
        ELSE
           DO K = 1,NAW
              Names(I,K)%S = DIR_name//CUT_STR(STR_ARR(K)%S)
           ENDDO
        ENDIF
     ENDDO  ! over hydrosollayers (I)
  CASE DEFAULT
     GOTO 127
  END SELECT

!  DO K = 1,NAW
!     WRITE(*,'(2x,I3,*(3x,A))') K, (TRIM(Names(I,K)), I = 1, LA_N)
!  ENDDO
!  stop '   Stop in man_hsol_setup.f90 '
  
!  WRITE(*,*) '     man_hsol_setup '
!  WRITE(*,*) '  ZZZ_UW(:)   ', ZZZ_UW(:) 
  
  IF( L_BOT_HS(1)    < ZZZ_UW(1) ) GOTO 102
  IF( L_BOT_HS(LA_N) > ZZZ_UW(NL_UW) ) GOTO 103

  ! LA_S(1) = 1
  ! DO I = 2,LA_N
  !    LA_S(I) = MINLOC( DABS(L_BOT_HS(I-1)-ZZZ_UW(:)),1 )
  ! ENDDO
  ! New version VR 08.11.2015
  DO I = 1,LA_N
     LA_E(I) = MINLOC( DABS(L_BOT_HS(I)-ZZZ_UW(:)),1 )
  ENDDO

  IF(LOG_LEVEL(1) >= N_ESO) THEN
     WRITE(LUNIT,*) '  Check bottom depth of hydrosol layers:'
     DO I = 1,LA_N
        WRITE(LUNIT,'(2x,I3,2x,I3,2x,F7.3)') I, LA_E(I), ZZZ_UW(LA_E(I))
     ENDDO
  END IF

  ! IF( LA_N == 1 ) THEN
  !    LA_E(LA_N) = MINLOC( DABS(L_BOT_HS(1)-ZZZ_UW(:)),1 )
  ! ENDIF
  ! DO I = 1,LA_N-1
  !    LA_E(I) = LA_S(I+1)-1
  ! ENDDO
  ! New version VR 08.11.2015
  LA_S(1) = 1
  DO I = 2, LA_N
     LA_S(I) = LA_E(I)-1
  ENDDO

  ! Indexes of altitude grid levels which are within hydrosol layers
  IDX_HS(:) = 1
  IF( LA_N > 1 ) THEN
     DO I = 1,LA_N - 1
        D1 = ZZZ_UW(LA_S(I)) ;  D2 = ZZZ_UW(LA_S(I+1))
        DO J = 1,NL_UW
           IF( ZZZ_UW(J) > D1 .AND. ZZZ_UW(J) <= D2 ) IDX_HS(J) = I
        ENDDO
     ENDDO
  ENDIF
  D1 = ZZZ_UW(LA_S(LA_N))
  DO J = 1,NL_UW
     IF( ZZZ_UW(J) > D1 ) IDX_HS(J) = LA_N
  ENDDO

  ! Reading extinction coefficient for all layers and wavelengths
  ! =================================================================
  STR_INP = 'Extinction coefficient (hydrosols)'
  CALL Read_Input (STR_INP, READ_ARR, *880)
  N_Z_EXT = SIZE(READ_ARR)
  ALLOCATE( EXT_COEFF(N_Z_EXT,NAW), Z_EXT(N_Z_EXT), Ex_Nam(N_Z_EXT) )
  CALL SPLIT_STRING(READ_ARR(1)%S, STR_ARR)
  If( IsNum(STR_ARR(2)%S) ) Then
     DO I = 1, N_Z_EXT
        READ(READ_ARR(I)%S,*,ERR=101)  Z_EXT(I), (EXT_COEFF(I,K), K=1,NAW)
     ENDDO
  ELSE
     DO I = 1, N_Z_EXT
        CALL SPLIT_STRING(READ_ARR(I)%S, STR_ARR)
        READ(STR_ARR(1)%S,*,ERR=101)  Z_EXT(I)
        Ex_Nam(I)%S = DIR_name//CUT_STR(STR_ARR(2)%S)
        OPEN( IUNIT, FILE=Ex_Nam(I)%S, STATUS='OLD' )
        CALL SKIP_HEADER (IUNIT)
        READ( IUNIT,* ) (DUMM(K),K = 1, NAW)
        IF( MAXVAL( DABS(DUMM - HS_WAVS) ) > 1.d-6 ) THEN
           WRITE(LUNIT,*) ' Reference wavelengths and from extinction file:'
           DO K = 1, NAW
              WRITE(LUNIT,*)  HS_WAVS(K), DUMM(K), HS_WAVS(K)-DUMM(K) 
           ENDDO
           GOTO 203
        ENDIF
        READ( IUNIT,* ) (EXT_COEFF(I,K),K = 1, NAW)
        CLOSE(IUNIT)
     ENDDO
  ENDIF
!  DO I = 1, N_Z_EXT
!     WRITE(*,*)  Z_EXT(I), TRIM( Ex_Nam(I) )
!  ENDDO
!  stop     
  DO K = 1,NAW
     CALL LIN_INT (N_Z_EXT, Z_EXT, EXT_COEFF(:,K), NL_UW, ZZZ_UW, &
          IWS(IiW)%HS_COEFF(2,:,K,1), 'Z')
  ENDDO
!  DO K = 1,NAW,10
!     WRITE(*,*)  K, IWS(IiW)%HS_COEFF(2,1:NL_UW,K,1)
!  ENDDO
  
  ! Reading single scattering albedo for all layers and wavelengths
  ! ===================================================================
  STR_INP = 'Single scattering albedo (hydrosols)'
  CALL Read_Input (STR_INP, READ_ARR, *880)
  N_Z_ABS = SIZE(READ_ARR)
  ALLOCATE( SSA_TMP(N_Z_ABS,NAW), Z_ABS(N_Z_ABS), HS_SSA(NL_UW) ) ! A
  CALL SPLIT_STRING(READ_ARR(1)%S, STR_ARR)
  If( IsNum(STR_ARR(2)%S) ) Then
     DO I = 1, N_Z_ABS
        READ(READ_ARR(I)%S,*,ERR=101)  Z_ABS(I), (SSA_TMP(I,K), K=1,NAW)
     END DO
  ELSE
     DO I = 1, N_Z_ABS
        CALL SPLIT_STRING(READ_ARR(I)%S, STR_ARR)
        READ(STR_ARR(1)%S,*,ERR=101)  Z_ABS(I)
        Ex_Nam(I)%S = DIR_name//CUT_STR(STR_ARR(2)%S)
        OPEN( IUNIT, FILE=Ex_Nam(I)%S, STATUS='OLD' )
        CALL SKIP_HEADER (IUNIT)
        READ( IUNIT,* ) (DUMM(K),K = 1, NAW)
        IF( MAXVAL( DABS(DUMM - HS_WAVS) ) > 1.d-6 ) THEN
           WRITE(LUNIT,*) ' Reference wavelengths and from SSA file:'
           DO K = 1, NAW
              WRITE(LUNIT,*)  HS_WAVS(K), DUMM(K), HS_WAVS(K)-DUMM(K) 
           ENDDO
          GOTO 203
        ENDIF
        READ( IUNIT,* ) (SSA_TMP(I,K),K = 1, NAW)
        CLOSE(IUNIT)
     ENDDO
  ENDIF

  DO K = 1,NAW
     IF( ANY(SSA_TMP(:,K)>1.0d0) .OR. ANY(SSA_TMP(:,K)<0.0d0) ) GOTO 130
     CALL LIN_INT (N_Z_ABS, Z_ABS, SSA_TMP(:,K), NL_UW, ZZZ_UW, &
          HS_SSA(:), 'Z')
     IWS(IiW)%HS_COEFF(1,:,K,1) = IWS(IiW)%HS_COEFF(2,:,K,1)*HS_SSA(:)
  ENDDO

  IF( IWT ) THEN
! ... Set to zero hydrosol scatt. and ext. coefficients within ice layer
     DO Ii = 1, LA_N_IW ; I1 = IWS(1)%IW_TOP(Ii) ; I2 = IWS(1)%IW_BOT(Ii)
        IWS(IiW)%HS_COEFF(1,I1:I2,:,1) = 0.d0
        IWS(IiW)%HS_COEFF(2,I1:I2,:,1) = 0.d0
     ENDDO  ! over ice layers (Ii)
  ENDIF

  STR_INP = 'Hydrosol scattering matrix output'
  CALL Read_Input (STR_INP, READ_ARR, *880)
  READ(READ_ARR(1)%S, *, ERR = 101) OUT_H_SCM
  IF( OUT_H_SCM ) THEN
     READ(READ_ARR(2)%S, *, ERR = 101) LAY_H, WEL_H, NSCA_H, GE_H
     PATH_H_SCM = SET_PATH(READ_ARR(3)%S)
  ELSE
     PATH_H_SCM = './'
  ENDIF
  
  IF (LOG_LEVEL(1) >= N_ESO) THEN
     WRITE(LUNIT,*) ' ------ Test of hydrosol layering man mode --- '
     WRITE(LUNIT,*) ' ----                                     --- '
     DO I = 1,LA_N
        WRITE(LUNIT,'(2x,I3,2(1x,f7.3))') I, ZZZ_UW(LA_S(I)) ,ZZZ_UW(LA_E(I)) 
     ENDDO
     WRITE(LUNIT,*) ' ----------------------------------------- '
     DO I = 1, NL_UW
        WRITE(*,'(2x,I3,2x,F8.4,2x,I2,2x,E12.4)') &
             I, ZZZ_UW(I), IDX_HS(I), IWS(IiW)%HS_COEFF(1,I,1,1)
     END DO
     WRITE(LUNIT,*) ' ----------------------------------------- '
  ENDIF
  
  ! *********************************************************************
  ! *              Scattering function/matrix generation                *
  ! *********************************************************************
  SELECT CASE (TRIM(Ph_type))
  CASE('Asymmetry_factor')
     ! Calculate expansion coefficients corresponding asymmetry factors
     ! ===================================================================
     DO I1 = 1,LA_N
        DO K = 1,NAW
           GG = ASYM_PAR(I1,K)
           DO L = 1,MUM
              TMP1 = GG**(L-1)*(2*(L-1) + 1)
              IF( TMP1 < 1.d-06 ) EXIT
              USER_MOM(L,I1,1,K) = TMP1
           ENDDO  ! over expansion coefficients (L)
           M_Lay_Wav(I1,K) = L-1
        ENDDO  ! over wavelength grid (K)
     ENDDO  ! over hydrosol layers (I1)
     NU_A = M_Lay_Wav(1,1)

     IF( LOG_LEVEL(1) >= N_ESO ) WRITE(LUNIT,'(2x,17(1x,f10.7))') &
          (USER_MOM(L,1,1,1)/(2*(L-1) + 1), L=1,17)
     
!    ... Calculate other elements of scattering matrix 
     IF( Nvs > 1 ) THEN
        M_cof = MAXVAL( M_Lay_Wav )
        M_ang = 2*M_cof
        CALL HG_SCATT_MAT( M_ang, M_cof )
     ENDIF
     
     ! Test output of expansion coefficients corresponding to  
     ! the given asymmetry parameter
     ! ===============================================================
     IF( OUT_H_SCM ) CALL OUT_COEFF

     ! Test output of scattering matrix corresponding to 
     ! the given asymmetry parameter
     ! ===============================================================
     IF( OUT_H_SCM ) CALL COEF_TO_SCATMATR(NU_A, Nm_vs, &
          USER_MOM(1:NU_A,1,1:Nm_vs,1), NSCA_H, GE_H, PATH_H_SCM)

  CASE('Expansion_coeff')
     ! Reading of the expansion coefficients 
     IF (LOG_LEVEL(1) >= N_ESO) &
          WRITE(LUNIT,*) '    Reading of the expansion coefficients  '
     DO I1 = 1,LA_N
        DO K = 1,NAW
           OPEN( IUNIT, FILE=Names(I1,K)%S, STATUS='OLD' )
           CALL SKIP_HEADER (IUNIT)           
           DO I = 1,MUM
              READ(IUNIT,*,END=28,ERR=29) tmp1, (tmp2(J), J=1,Nm_vs)
              USER_MOM(I,I1,:,K) = tmp2(:) 
!              WRITE(*,'(f5.1,6(1x,e12.4))') tmp1,(tmp2(J), J=1,Nm_vs)
              ! The additional condition, I > 2, was added to read 
              ! Rayleigh phase function in the scalar case
              ! which is used for test purpose
              IF( SUM(TMP2(:)) == 0.d0 .AND. I > 2 ) GOTO 28
           END DO
29         CALL WRITE_MESSAGE (R_NAME, 'ERROR', 'Error reading data file !')
           WRITE(EU,*) '    File name : ', Names(I1,K)%S
           RETURN 1             
28         CONTINUE
           CLOSE(IUNIT)
           IF( Nm_vs > 1 ) THEN
              USER_MOM(:,I1,5,K) = -USER_MOM(:,I1,5,K)
              USER_MOM(:,I1,6,K) = -USER_MOM(:,I1,6,K)
           ENDIF
           M_Lay_Wav(I1,K) = I - 1
           NU_A = I - 1
!               WRITE(*,*) '  Number of user moment :', NU_A
!               WRITE(*,*) USER_MOM(NU_A,I1,:,K)
        ENDDO  ! over wavelength grid (K)
     ENDDO  ! over hydrosol layers (I1)

     IF( LOG_LEVEL(1) >= N_ESO ) WRITE(LUNIT,'(2x,17(1x,f10.7))') &
          (USER_MOM(L,1,1,1)/(2*(L-1) + 1), L=1,11)

     ! Test output of scattering matrix corresponding to 
     ! the given expansion coefficients
     IF( OUT_H_SCM ) THEN 
        NU_A = M_Lay_Wav(1,4) !Corresponds to Lay_out, Wl_out used below
        CALL COEF_TO_SCATMATR(NU_A, Nm_vs, USER_MOM(1:NU_A,1,1:Nm_vs,4),&
             NSCA_H, GE_H, PATH_H_SCM)
     ENDIF
     !         NU_A = M_Lay_Wav(1,1)
     !         WRITE(*,*) '   CALL to "COEF_TO_SCATMATR"', M_Lay_Wav(1,1)
     !         CALL COEF_TO_SCATMATR(NU_A, Nm_vs, 
     !     $        USER_MOM(1:NU_A,1,1:Nm_vs,1),
     !     $        NSCA_H, GE_H, PATH_H_SCM)

  CASE('Scattering_matrix')
     ! Calculate the expansion coefficients of scattering function/matrix
     N_GAU = MUM
     IF( .NOT.ALLOCATED(ESM) )  ALLOCATE( ESM(Nel) ) 
     CALL SCM_TO_EC(MUM, N_GAU, LA_N, NAW, Nel, ESM, Names, INT_SCM_H,   &
          CRIT_SCM_H, Lay_H, WeL_H, OUT_H_SCM, M_Lay_Wav, USER_MOM, &
          PATH_H_SCM, *880)
!     WRITE(*,*) '   MUM  and N_GAU',  MUM, N_GAU
  END SELECT

  ! Preliminary for first layer and first wavelength ???
  NU_A = M_Lay_Wav(1,1)
  
  ! Single scattering correction algorithm -> save all expansion coeff.
  ! --------------------------------------    ------------------------
  ! write(*,*) '   Before SSRCA '
  IF( DO_SSRCA .OR. TRIM(RTM_TYPE) == 'spher_scat' ) THEN
     NG_HS = MAXVAL( M_Lay_Wav )
     IF (LOG_LEVEL(1) >= N_ESO) &
          WRITE(LUNIT,*) '  Allocate Greek_HS :', NG_HS
     ALLOCATE( Greek_HS( NG_HS,LA_N,Nm_vs,NAW,N_HPC ) )
     Greek_HS = 0.d0
     DO I = 1,NAW 
        Greek_HS(:,:,:,I,1) = USER_MOM(1:NG_HS,:,:,I)
     ENDDO
  ENDIF

  !      write(*,*) '    TAU before scaling', HS_SCALING
  DO K = 1,NAW
     TAU(K) = HS_SCALING*SUM( IWS(IiW)%HS_COEFF(2,:,K,1)*DZ_UW(:) )/2.d0
  ENDDO

  !      write(*,*) '    Before SCAL TAU(1) = ', TAU(1)
  IF( Scal_HS ) THEN
     CALL LIN_SEARCH(Rwl_HS, DUM, K, NAW, HS_WAVS, ERROR)
     IF (ERROR) GOTO 126
     Scoe = Rtau_HS/( (1.D0 - DUM)*TAU(K) + DUM*TAU(K+1) )
     IWS(IiW)%HS_COEFF(2,:,:,1) = Scoe*IWS(IiW)%HS_COEFF(2,:,:,1)
     IWS(IiW)%HS_COEFF(1,:,:,1) = Scoe*IWS(IiW)%HS_COEFF(1,:,:,1)
     WRITE(*,*) '  Old tau:', TAU
     TAU(:) = Scoe*TAU(:)
     WRITE(*,*) '  New tau:', TAU
  ENDIF

  !c        Test of the hydrosol OD 
  !      OPEN(IUNIT, FILE = DIR_OUT//'/aer_OD.dat') 
  !        DO K = 1,NAW
  !           WRITE(IUNIT,*) HS_WAVS(K), TAU(K)
  !        ENDDO
  !      CLOSE(IUNIT)

  ! ************** Delta-M approach ****************
  ! Allocate array for MODULE DELTA_M
  ! =================================
  IF ( DO_HSOL_DM  ) THEN   ! Set to .TRUE. in "read_uwt.f"
     Itr = HSOL_TRUNC_IDX   ! Set to NLEG_W-1 in "gt_settings_check.f"
     SELECT CASE( Vec_Sca )
     CASE('vect_rte')
        CALL AER_DM_VEC
     CASE('scal_rte')
        CALL AER_DM
     END SELECT

     ! "Renormalize" scattering and extinction coefficients
     ! ====================================================
     DO I = 1, NL_UW
        DO K = NWLS, NWLE
           F_DEL = F_DLM_HSOL( I,K,1 )
           ABS_COEFF = IWS(IiW)%HS_COEFF(2,I,K,1) - IWS(IiW)%HS_COEFF(1,I,K,1)
           IWS(IiW)%HS_COEFF(1,I,K,1) = IWS(IiW)%HS_COEFF(1,I,K,1)*(1.0d0 - F_DEL)
           IWS(IiW)%HS_COEFF(2,I,K,1) = IWS(IiW)%HS_COEFF(1,I,K,1) + ABS_COEFF 
        ENDDO  ! over wavelengths (K)
     ENDDO   ! over layers (I1)
  ELSE
     L = MAXVAL( M_Lay_Wav )
     IF( L > NLEG_W ) THEN
        CALL WRITE_MESSAGE(R_NAME, 'WARNING', &
             'The number of hydrosol expansion moments greater than '&
             //'"Number of Legendre moments"')
        WRITE(EU,'(a46,1x,I4)') &
             '      "Number of Legendre moments"           ', NLEG_W
        WRITE(EU,'(a46,1x,I4)') &
             '      Maximal number of moments after reading', L
     ENDIF
     DO K = 1,NAW
        IWS(IiW)%HSOL_PHAS_VS(:,:,:,K,1) = USER_MOM(1:NLEG_W,:,:,K)
     ENDDO
  ENDIF

  CALL WRITE_SCEN_HS 

  CALL ST_TRACK (R_NAME, 1, 'O')
  RETURN

880 WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)
  RETURN 1

101 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Reading error')
  WRITE(EU,*) '    Parameter to read: ', TRIM(STR_INP) 
  WRITE(EU,*) '    Input line       : ', TRIM(STR_READ) 
  RETURN 1

201 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Reading error')
  WRITE(EU,*) '    Parameter to read: ', TRIM(STR_INP) 
  WRITE(EU,*) '    Number of entries does not equal to the number of wavelength' 
  WRITE(EU,*) '    Nunber of wavelengths       : ', NAW
  WRITE(EU,*) '    Nunber of entries           : ', II
  RETURN 1
  
203 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Reading error')
  WRITE(EU,*) '    Number of wavelengths or their values in the file' 
  WRITE(EU,*) '    - ', Ex_Nam(I)%S
  WRITE(EU,*) '    does not equal to the reference wavelengths'
  RETURN 1

102 CALL WRITE_MESSAGE(R_NAME, 'ERROR', &
         'Upper hydrosol layer outside the ocean')
  WRITE(EU,'(A,2x,F9.2,A,F9.2)') &
       '    Upper hydrosol layer:', ZZZ_UW(1),' - ',L_BOT_HS(1)
  WRITE(EU,'(A,2x,F9.2,A)') &
       '      Top of ocean     :', ZZZ_UW(1),' m'
  WRITE(EU,*) '    Check input file "man_hsol.inp"'
  RETURN 1

103 CALL WRITE_MESSAGE(R_NAME, 'ERROR', &
         ' Lower hydrosol layer outside the ocean')
  WRITE(EU,'(A,2x,F6.2,A)') &
       '       Bottom of hydrosol layer:',L_BOT_HS(LA_N),' mm'
  WRITE(EU,'(A,2x,F6.2,A)') &
       '                Ocean bottom   :', ZZZ_UW(NL_UW),' mm'
  WRITE(EU,*) '    Check "man_hsol.inp" and "Ocean depth" '
  WRITE(EU,*) '    in the file "control_uwt.inp" '
  RETURN 1

104 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'The interpolation of '//&
         'hydrosol parameters can not be performed because')
  WRITE(EU,'(a34,1x,f7.3,1x,a38,1x,f7.3)') &
       '     first wavelength for hydrosol ',HS_WAVS(1), &
       ' > than start wavelength for radiance ',START_WAV
  RETURN 1

105 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'The interpolation of hydrosol '//&
         'parameters can not be performed because')
  WRITE(EU,'(a35,1x,f7.3,1x,a36,1x,f7.3)') &
       '       last wavelength for hydrosol ',HS_WAVS(NWLE),&
       ' < than end wavelength for radiance ',END_WAV
  RETURN 1

106 CALL WRITE_MESSAGE(R_NAME, 'ERROR', &
         'The bottom altitudes of hydrosol layers have to be'&
         //' given in the ascending order')
  WRITE(EU,'(A)') '   Check file "man_hsol.inp" after key-sentence'
  WRITE(EU,'(A)') '   "Layering of scattering function/matrix"'
  RETURN 1

109 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Wrong number of entries')
  WRITE(EU,*) '    Parameter: ', TRIM(STR_INP) 
  RETURN 1

110 CALL WRITE_MESSAGE(R_NAME, 'ERROR', ' ')
    WRITE(EU,*) '    Wrong number of entries '
    WRITE(EU,*) '      Parameter: ', TRIM(STR_INP) 
    WRITE(EU,*) '    Check the "man_aer.inp" file. '
    RETURN 1

126 CALL WRITE_MESSAGE (R_NAME,'ERROR',&
         'Reference wavelength for HOT is out of range')
  WRITE(EU,'(A,F7.2)') '    Reference wavelength: ', Rwl_HS
  WRITE(EU,'(A,2(2x,F7.2))') &
       '    Database wavelengths: ', HS_WAVS(1), HS_WAVS(NAW)
  WRITE(EU,'(A)') &
       '    Set reference wavelength between database wavelengths.'
  RETURN 1
  
127 CALL WRITE_MESSAGE (R_NAME, 'ERROR', 'Type of hydrosol '//&
         'scattering function parameterization is not recognized')
  RETURN 1

130 CALL WRITE_MESSAGE (R_NAME, 'ERROR', 'SSA of hydrosol particles'// &
         'can not be larger than 1.')
  WRITE(EU,'(A,2(2x,F7.2))') &
       '    Check input values in the file "man_hsol.inp" '
  RETURN 1
133 CALL WRITE_MESSAGE (R_NAME, 'ERROR',&
         '    The asymmetry factor can be used '//&
         'as input in the scalar case only.')
    WRITE(EU,'(A)') '    Check input file "man_hsol.inp".'
    RETURN 1
131 CALL WRITE_MESSAGE (R_NAME, 'ERROR', 'Number of matrix elements '// &
         'can not be larger than 6.')
  WRITE(EU,'(A,2(2x,F7.2))') &
       '    Check input values in the file "man_hsol.inp" '
  RETURN 1
  
CONTAINS
  
!***************************************************************************
  SUBROUTINE HG_SCATT_MAT( N_gau, M_cof )

    USE ST_VARS, ONLY: Pie
! ------------------------------------------------------------------------
    INTEGER N_gau, M_cof
    INTEGER N_cof, L
    REAL (KND), DIMENSION (N_gau) :: X_gau, AG_gau, R12, R33, &
         H11, H12, H33
    REAL (KND)  Cofm, Cofp, Clk, Dum
    REAL (KND)  Poly(M_cof,N_gau), D_02g(M_cof,N_gau), & 
         D_P2g(M_cof,N_gau), D_M2g(M_cof,N_gau), Cf(Nm_vs)
!    REAL (KND) Ph_ret(N_gau,Nm_vs)
! =========================================================================
    
    ! Gauss grid and weights for interval (-1,1)
    CALL GAULEG ( -1.d0, 1.d0, X_gau, AG_gau, N_gau )
    ! Legendre polynomials and Wigner functions for all Gauss angles
    CALL WIGNER_PLG     ( X_gau, N_gau, 0, M_cof, Poly )
    CALL WIGNER_PLG     ( X_gau, N_gau, 2, M_cof, D_02g )
    CALL WIGNER_FUNCTION( X_gau, N_gau, 2, M_cof, D_P2g, D_M2g )
    
    ! Normalized elements of Rayleigh scattering matrix
    !    R11 = R22, R33 = R44, R34 = 0
    !    R11(:) =  3.d0/4.d0*(1.d0 + X_gau(:)**2)
    !    R12(:) = -3.d0/4.d0*(1.d0 - X_gau(:)**2)
    !    R33(:) =  3.d0/2.d0*X_gau(:)
    R12(:) = -(1.d0 - X_gau(:)**2)/(1.d0 + X_gau(:)**2)
    R33(:) =  2.d0*X_gau(:)/(1.d0 + X_gau(:)**2)
    
    ! Elements of HG scattering matrix
    DO I1 = 1, LA_N
       DO K = 1, NAW
          GG = ASYM_PAR(I1,K)
          H11(:) = (1.d0 - GG**2) &
               /(1.d0 - 2.d0*GG*X_gau(:) + GG**2)**1.5d0
          H12(:) = H11(:)*R12(:) !/R11(:)
          H33(:) = H11(:)*R33(:) !/R11(:)
          DO L = 1, M_Lay_Wav(I1,K) 
             Clk = DBLE(L) - 0.5d0
             Cf(1) =  Clk*SUM( H11(:)*Poly(L,:)*AG_gau(:) )
             Cf(4) =  SUM( H33(:)*Poly(L,:)*AG_gau(:) )
             Cf(5) = -SUM( H12(:)*D_02g(L,:)*AG_gau(:) )
             Cf(6) = 0.d0
             Cofm = SUM( (H11(:) - H33(:))*D_M2g(L,:)*AG_gau(:) )
             Cofp = SUM( (H11(:) + H33(:))*D_P2g(L,:)*AG_gau(:) )
             Cf(2) = (Cofp + Cofm)/2.d0
             Cf(3) = (Cofp - Cofm)/2.d0
             Cf(2:Nm_vs) = Clk*Cf(2:Nm_vs)
             Cf(5:6) = -Cf(5:6)
             USER_MOM(L,I1,:,K) = Cf(:)
          ENDDO  ! over expansion coefficients (L)
       ENDDO  ! over wavelength grid (K)
    ENDDO  ! over hydrosol layers (I1)
    
    ! Check HG expansion coefficients 
!    I1 = 1 ;  K = 1
!    Ph_ret(:,:) = 0.d0
!    DO L = 1,NU_A
!       Cf(:) = USER_MOM(L,I1,:,K)
!       Ph_ret(:,1) = Ph_ret(:,1) + Poly(L,:) *Cf(1)
!       Ph_ret(:,4) = Ph_ret(:,4) + Poly(L,:) *Cf(4)
!       Ph_ret(:,5) = Ph_ret(:,5) - D_02g(L,:) *Cf(5)
!       Ph_ret(:,6) = Ph_ret(:,6) - D_02g(L,:) *Cf(6)
!       Cofp = Cf(2) + Cf(3)
!       Cofm = Cf(2) - Cf(3)
!       Ph_ret(:,2) = Ph_ret(:,2) + &
!            (D_P2g(L,:)*Cofp + D_M2g(L,:)*Cofm)/2.d0
!       Ph_ret(:,3) = Ph_ret(:,3) + &
!            (D_P2g(L,:)*Cofp - D_M2g(L,:)*Cofm)/2.d0
!    ENDDO
!    OPEN( 47,FILE='./DATA_OUT/Phm_orig.dat' )
!    DO J = 1,N_gau
!       Dum = ACOS( X_gau(J) )/Pie*180.
!       WRITE(47,'(1x,*(e15.7))') Dum, &
!            H11(J), H11(J), H33(J), H33(J), H12(J)
!    ENDDO
!    CLOSE(OUNIT)
!    OPEN( 47,FILE='./DATA_OUT/Phm_ret.dat' )
!    DO J = 1,N_gau
!       Dum = ACOS( X_gau(J) )/Pie*180.
!       WRITE(47,'(1x,*(e15.7))') Dum, Ph_ret(J,1), &
!            Ph_ret(J,2), Ph_ret(J,3), Ph_ret(J,4), Ph_ret(J,5)
!    ENDDO
!    CLOSE(OUNIT)

  END SUBROUTINE HG_SCATT_MAT
  
!***************************************************************************
   SUBROUTINE AER_DM
    IF (LOG_LEVEL(1) >= N_ESO) THEN
       WRITE(LUNIT,*) '   *************************  '
       WRITE(LUNIT,*) '   Hydrosol D-M scalar case  '
       WRITE(LUNIT,*) '   *************************  '
       WRITE(LUNIT,*) '   Number of Legendre terms :', NLEG_W
       WRITE(LUNIT,*) '           Truncation index :', Itr
    ENDIF
    DO I1 = 1,LA_N
       DO K = 1,NAW
          F_DEL = USER_MOM(Itr+1,I1,1,K)/DBLE(2*Itr + 1)
          F_DLM_HSOL(LA_S(I1):LA_E(I1),K,1) = F_DEL
          DO L = 1,Itr
             HS_PHASMOMS(L,I1,K) = (USER_MOM(L,I1,1,K) - &
                  F_DEL*DBLE(2*L-1))/(1.D0 - F_DEL)
          ENDDO  ! over moments (L)
       ENDDO  ! over wavelengths (K)
    ENDDO   ! over layers (I1)
    HS_PHASMOMS(Itr+1:NLEG_W,:,NWLS:NWLE ) = 0.D0
    
  END SUBROUTINE AER_DM

!***************************************************************************
  SUBROUTINE AER_DM_VEC
    IF (LOG_LEVEL(1) >= N_ESO) THEN
       WRITE(LUNIT,*) '   ***********************  '
       WRITE(LUNIT,*) '   Hydosol D-M vector case  '
       WRITE(LUNIT,*) '   ***********************  '
    ENDIF
    DO I1 = 1,LA_N
       DO K = 1,NAW
          F_DEL = USER_MOM(Itr+1,I1,1,K)/DBLE(2*Itr + 1)
          F_DLM_HSOL(LA_S(I1):LA_E(I1),K,1) = F_DEL
          DO L = 1, Itr
             IWS(IiW)%HSOL_PHAS_VS(L,I1,1:4,K,1) = (USER_MOM(L,I1,1:4,K) -&
                  F_DEL*DBLE(2*L-1))/(1.0D0 - F_DEL)
             IWS(IiW)%HSOL_PHAS_VS(L,I1,5:6,K,1) = &
                  USER_MOM(L,I1,5:6,K)/(1.0D0 - F_DEL)
          ENDDO
       ENDDO
    ENDDO
    IWS(IiW)%HSOL_PHAS_VS(Itr+1:NLEG_W,:,:,:,:) = 0.d0

  END SUBROUTINE AER_DM_VEC

!***************************************************************************
  SUBROUTINE WRITE_SCEN_HS
    DOUBLE PRECISION    SS_A, Ext, Sca_co, Abs
    IF (LOG_LEVEL(1) >= N_SSO) WRITE (LUNIT,*) ' *** Writing: SCE_MAN-HSOL.OUT'

    OPEN (IUNIT, FILE = DIR_OUT//'/SCE_MAN-HSOL.OUT')        
    CALL WRITE_UD(1,IUNIT)
    WRITE ( IUNIT, * ) '           ==========================='
    WRITE ( IUNIT, * ) '            MANUAL HYDROSOL SELECTION '
    WRITE ( IUNIT, * ) '           ==========================='

    IF( Scal_HS ) THEN
       WRITE(IUNIT,'(A,F7.3,A,F9.2,A)') '   !!! ATTENTION !!! HOT is set to ',&
            Rtau_HS,' at', Rwl_HS,' nm wavelength.'
       WRITE(IUNIT,'(19x,A)') '  See setting in "gt_fileread.f" '
       WRITE(IUNIT,'(6x,A)') &
            '----------------------------------------------------'
       WRITE(IUNIT,*) 
    ENDIF

    WRITE ( IUNIT, * ) ' '
    WRITE ( IUNIT, * ) ' Vertical profiles of hydrosol particles'
    WRITE ( IUNIT, * ) '         extinction coefficient (Ext coeff) and'
    WRITE ( IUNIT, * ) '         single scattering albedo (Ss albedo):'
    WRITE ( IUNIT, * ) '         (HOT - hydrosol optical thickness)'

    DO K = 1,NAW            ! Reference wavelength do-loop
       WRITE(IUNIT,*) '   '
       WRITE(IUNIT,'(1x,A,F9.2,A,A,ES12.4)') &
            '--> Wavelength ',HS_WAVS(K),' [nm];',' HOT =',TAU(K)
       WRITE(IUNIT,'(5x,41A)') ('-', I=1,41)

       WRITE ( IUNIT, * ) '    Number, Depth [m], '&
            //'Ext coeff [m-1], Ss albedo'
       DO I = 1, NL_UW
          Ext = IWS(IiW)%HS_COEFF(2,I,K,1)
          Abs = IWS(IiW)%HS_COEFF(2,I,K,1) - IWS(IiW)%HS_COEFF(1,I,K,1)
          Sca_co = IWS(IiW)%HS_COEFF(1,I,K,1)/(1.D0 - F_DLM_HSOL(I,K,1))
          IF( Ext == 0.0d0 ) CYCLE
          SS_A = Sca_co/(Sca_co + Abs)
          WRITE(IUNIT,'(6x,I3,4x,F7.3,5x,ES9.2,6x,E12.4)') &
               I, ZZZ_UW(I), Sca_co+Abs , SS_A
       END DO
       WRITE(IUNIT,*) ' '

       SELECT CASE (TRIM(Ph_type))
       CASE('Asymmetry_factor')
          WRITE(IUNIT,*) &
               '    Asymmetry factor used in the hydrosol layers :'
          WRITE(IUNIT,'(5x,50A)') ('-', I=1,50)
          WRITE(IUNIT,*) &
               '    Layer  Top [km]  Bottom [km]   Asymmetry factor'
          DO I = 1,LA_N
             WRITE(IUNIT,'(6x,I3,3x,F6.2,4x,F7.2,14x,F6.3)') &
                  I, ZZZ_UW(LA_S(I)) ,ZZZ_UW(LA_E(I)), Asym_Par(I,K)
          ENDDO
       CASE('Expansion_coeff')
          WRITE(IUNIT,*) &
               '    Expansion coefficients used in the hydrosol layers :'
          WRITE(IUNIT,'(5x,50A)') ('-', I=1,50)
          WRITE(IUNIT,*) &
               '    Layer  Top [km]  Bottom [km]   File name'
          DO I = 1,LA_N
             WRITE(IUNIT,'(6x,I3,3x,F6.2,4x,F7.2,6x,a)') &
                  I, ZZZ_UW(LA_S(I)) ,ZZZ_UW(LA_E(I)), Names(I,K)%S
          ENDDO
       CASE('Scattering_matrix')
          WRITE(IUNIT,*) &
               '    Scattering matrix used in the hydrosol layers :'
          WRITE(IUNIT,'(5x,50A)') ('-', I=1,50)
          WRITE(IUNIT,*) &
               '    Layer  Top [km]  Bottom [km]   File name'
          DO I = 1,LA_N
             WRITE(IUNIT,'(6x,I3,3x,F6.2,4x,F7.2,6x,a)') &
                  I, ZZZ_UW(LA_S(I)) ,ZZZ_UW(LA_E(I)), Names(I,K)%S
          ENDDO
       END SELECT

       WRITE(IUNIT,*) ' '
       WRITE(IUNIT,*) '    Number of expansion coeff. and Delta-M factor:'
       WRITE(IUNIT,'(5x,45A)') ('-', I=1,45)
       WRITE(IUNIT,*) '    Layer  Top [km]  Bottom [km]   N coef   Delta-M'
       DO I = 1,LA_N
          WRITE(IUNIT,'(6x,I3,3x,F6.2,4x,F7.2,9x,I4,2x,E10.2)') &
               I, ZZZ_UW(LA_S(I)) ,ZZZ_UW(LA_E(I)), &
               M_Lay_Wav(I,K), F_DLM_HSOL(LA_S(I),K,1)
       ENDDO
    ENDDO                     ! Over all wavelengths 

    CALL WRITE_UD(2,IUNIT)
    
    CLOSE (IUNIT)          
  END SUBROUTINE WRITE_SCEN_HS

  !***************************************************************************
  SUBROUTINE OUT_COEFF
    OPEN(OUNIT,FILE=DIR_OUT//'/H_G_coeff.out')
    WRITE(OUNIT,'(a)') '#  Expansion coefficients of the H-G phase function'
    WRITE(OUNIT,'(a)') '#  in the lower hydrosol layer at first wavelength. '
    WRITE(OUNIT,'(a,2x,F6.3)') '#  Asymmetry parameter :', ASYM_PAR(1,1)
    DO L = 1,NU_A
       WRITE(OUNIT,'(2x,I5,*(1x,E15.7))') L-1, USER_MOM(L,1,:,1)
    ENDDO
    CLOSE(OUNIT)
  END SUBROUTINE OUT_COEFF
  
!***************************************************************************
END SUBROUTINE MAN_HSOL_SETUP
!***************************************************************************

