!***************************************************************************
!
!	SCIATRAN - Version 4.6 - Last modification: 10 Feb 2023
!
!	Copyright (C) - Institute of Remote Sensing,
!			University of Bremen, Germany
!
!***************************************************************************
SUBROUTINE MAN_ICE_SETUP(START_WAV, END_WAV, *)
  ! Called by "rtm_init.f"
                              
  USE CLOUDS_SS,   ONLY: MUM => MAXMOM, DO_SSRCA
  USE DELTA_M,     ONLY: DO_HSOL_DM, HSOL_TRUNC_IDX
  USE HYDROSOLS,   ONLY: NWLS => NWLS_HS, NWLE => NWLE_HS,  &
                         HS_SCALING, Greek_HS, NG_HS, Scal_HS, Rtau_HS, &
                         Rwl_HS
  USE INTERN_DIMS, ONLY: Nm_vs, Nvs
  USE LOG_PARS,    ONLY: LUNIT, EU, DIR_OUT, LOG_LEVEL, N_SSO, N_ESO, C_UNIT,&
                         Work_Dir
  USE ST_VARS,     ONLY: RTD => RAD_TO_DEG, Vec_Sca, RTM_TYPE, C_SYS
  USE TOOLS,       ONLY: READ_INPUT, SET_PATH, CUT_STR, SPLIT_STRING
  USE TYPES,       ONLY: KND, DYN_CHAR
  USE WATER,       ONLY: NL_UW => NLAYERS_UW, ZZZ_UW, DZ_UW, NLEG_W
  USE WATER_ICE,   ONLY: IWS, LA_N => LA_N_IW, LA_S => LA_S_IW, &
                         LA_E => LA_E_IW, L_TOP_I, L_BOT_I, &
                         IWT
  
  REAL (KND), INTENT(IN):: START_WAV, END_WAV
  ! Local parameters
  CHARACTER (LEN = *), PARAMETER :: R_NAME = 'MAN_ICE_SETUP'

  !  (a) Input variables
  !  ===================
  !      DO_HSOL_DM        : delta-M approximation yes/no 
  !                          <---- From Module DELTA_M
  !      HSOL_TRUNC_IDX    : truncation index for delta-M approximation 
  !                          <---- From Module DELTA_M
  !      ZZZ_UW             : forward model altitude grid in ocean
  !      NL_UW             : number of layers in altitude grid in ocean
  !      NLEG_W            : number of Legendre moments in water
  !      LUNIT             : LOG file unit number

  !  (b) Output variables
  !  ====================
  !      HS_COEFF         : scattering and extinction coefficients
  !      HS_PHASMOMS      : Legendre moments
  !      NAW              : number of wavelengths of chosen inclusions 
  !                         optical parameters
  !      HS_WAVS          : wavelengths of ice data base (depends on
  !                         parameterization chosen)
  INTEGER     N_Z_EXT, N_Z_ABS, Nel
  LOGICAL     TEST_MODE, ERROR
  CHARACTER (LEN=:), ALLOCATABLE :: DIR_name, Ph_type
  CHARACTER (LEN=200)  STR_READ, STR_INP, STR_READ_ARR(90)
  CHARACTER (LEN=80), ALLOCATABLE :: Names(:,:), Asym_nam(:,:), X_nam(:,:)
  CHARACTER (LEN=3),  ALLOCATABLE :: ESM(:)
  INTEGER   I, I1, I2, Ii, J, K, L, IUNIT, OUNIT, Itr, NU_A, MAX_CO, N_GAU, &
       M_cof, M_ang, NAW, IiW, JT, JB, ILN, Nc_out
  INTEGER,  ALLOCATABLE :: M_Lay_Wav(:,:,:)
  REAL (KND)  Scoe, TMP1, GG, DUM, D1, D2
  REAL (KND), ALLOCATABLE :: Z_EXT(:), Z_ABS(:), TMP2(:), TAU(:), &
       R_COF(:,:), SSA_TMP(:,:), HS_SSA(:), ASYM_PAR(:,:,:), &
       F_DLM_HSOL(:,:,:), Z_Act(:), Z_inp(:)
  REAL (KND), ALLOCATABLE, TARGET :: USER_MOM(:,:,:,:,:)
  REAL (KND), POINTER :: HS_PHASMOMS(:,:,:,:), HS_WAVS(:)
  PARAMETER ( IUNIT = 33, OUNIT = 55 )
  
!  ... From the structure IWS
  INTEGER N_HPC
  
  ! Variable for Delta-M
  ! ======================
  REAL (KND)  F_DEL, ABS_COEFF

  TYPE (DYN_CHAR), ALLOCATABLE :: READ_ARR(:), STR_ARR(:)
  !==================================================================
  CALL ST_TRACK (R_NAME, 1, 'I')  

  ERROR = .FALSE.
  IiW = 1
  ! =============================
  ! USER-DEFINED PARAMETERIZATION
  ! =============================
  STR_INP = 'Directory name (cai)'
  CALL Read_Input (STR_INP, READ_ARR, *880)
  DIR_name = SET_PATH(READ_ARR(1)%S)

  STR_INP = 'Wavelength grid (cai)'
  CALL Read_Input (STR_INP, READ_ARR, *880, ReArrange = .TRUE.)
  NAW = SIZE(READ_ARR) ;  NWLS = 1 ;  NWLE = NAW
  IWS(IiW)%N_HS_WL = NAW

  N_HPC = IWS(IiW)%N_HPC
  
  ALLOCATE( IWS(IiW)%HS_WL(NAW), IWS(IiW)%HS_COEFF(2,NL_UW,NAW,N_HPC), &
       TMP2(Nm_vs), F_DLM_HSOL(NL_UW,NAW,N_HPC) ) ! A
  F_DLM_HSOL = 0.D0  ;  IWS(IiW)%HS_COEFF = 0.D0 

  HS_WAVS => IWS(IiW)%HS_WL(:)
  ! Reading of wavelength grid for optical parameters of inclusions
  DO I = 1, NAW
     READ(READ_ARR(I)%S,*,ERR=101) HS_WAVS(I)
  END DO
      
  !  Check the position of start/end wavelength with respect
  !  to the wavelength range required for RT calculations
  IF( HS_WAVS(1)    > START_WAV ) GOTO 104
  IF( HS_WAVS(NWLE) < END_WAV )   GOTO 105

  STR_INP = 'Scattering function representation (cai)'
  CALL Read_Input (STR_INP, READ_ARR, *880)
  PH_TYPE = CUT_STR(READ_ARR(1)%S)

  IF( C_SYS=='ice' ) THEN
     LA_N = 1
  ELSE
     STR_INP = 'Layering of scattering function/matrix (cai)'
     CALL Read_Input (STR_INP, READ_ARR, *880)
     LA_N = SIZE(READ_ARR)
  ENDIF
  
  ALLOCATE( LA_S(LA_N), LA_E(LA_N), TAU(NAW), & !A
       IWS(IiW)%HSOL_PHAS_VS( NLEG_W,LA_N,Nm_vs,NAW,N_HPC ),   & !A
       M_Lay_Wav(LA_N,NAW,N_HPC),  & !A
       Names(LA_N,NAW), USER_MOM(MUM,LA_N,Nm_vs,NAW,N_HPC) )     ! A

  IWS(IiW)%HSOL_PHAS_VS = 0.d0  ;  USER_MOM = 0.d0
  HS_PHASMOMS => IWS(IiW)%HSOL_PHAS_VS(:,:,1,:,:)

!  The reading of top and bottom depth of all ice layers
!    is shifted into subroutine "read_uwt.f90" In the case of mode 'ice'
!    L_BOT_I(1) is set to OC_DEP
  IF( .NOT.IWT ) THEN
     IF( L_TOP_I(1) /= 0.d0 ) GOTO 132
  ENDIF
  
  IF( LA_N > 1 ) THEN
     IF( L_BOT_I(2) < L_BOT_I(1) )  GOTO 106
  ENDIF

  SELECT CASE (Ph_type)
  CASE('Asymmetry_factor')
     ! ****** Reading of the asymmetry factors *********
     ! ===================================================
     IF (LOG_LEVEL(1) >= N_ESO) THEN
        WRITE(LUNIT,*) ' ----- "Asymmetry_factor" mode is used ---------'
        WRITE(LUNIT,*) '    Reading of asymmetry factors file names  '
     ENDIF
     ALLOCATE( Asym_nam(LA_N,N_HPC) )
     STR_INP = 'Asymmetry factor (cai)'
     CALL Read_Input (STR_INP, READ_ARR, *880)
     IF(SIZE(READ_ARR) < LA_N) GOTO 101
     DO I = 1,LA_N
        CALL SPLIT_STRING(READ_ARR(I)%S, STR_ARR)
        DO K = 1,N_HPC
           Asym_nam(I,K) = DIR_name//CUT_STR(STR_ARR(K)%S)
        ENDDO
     ENDDO
!    ... Reading of asymmetry factors from file
     ALLOCATE( ASYM_PAR(LA_N,NAW, N_HPC) )
     DO I1 = 1,LA_N
        DO K = 1,N_HPC
           OPEN(IUNIT, FILE=TRIM(Asym_nam(I1,K)), STATUS='OLD' )
           CALL SKIP_HEADER (IUNIT)           
           READ(IUNIT,*) (ASYM_PAR(I1,J,K), J=1,NAW)
        ENDDO  ! over inclusions in ice (K)
     ENDDO
!     DO K = 1,N_HPC
!        WRITE(*,'(A,2x,I2)') '--> Asymmetry parameter of inclusion ',K
!        DO I1 = 1,LA_N
!        WRITE(*,'(A,2x,I2)') '                        in ice layer ',I1
!           WRITE(*,'(*(2x,e12.4))') (ASYM_PAR(I1,J,K), J = 1,NAW)
!        ENDDO  ! over ice layers (I1)
!     ENDDO  ! over inclusions in ice (K)
!     stop '   Stop after reading Asymmetry parameter'
  CASE('Expansion_coeff')
     IF (LOG_LEVEL(1) >= N_ESO) WRITE(LUNIT,*) &
          ' ----- "Expansion_coeff" mode is used ---------'
     ! Read the file names contaning the expansion coefficients
     ! ========================================================
     STR_INP = 'File names for expansion coefficients (cai)'
     CALL Read_Input (STR_INP, READ_ARR, *880)
     IF(SIZE(READ_ARR) < LA_N) GOTO 101
     DO I = 1,LA_N
        CALL SPLIT_STRING(READ_ARR(I)%S, STR_ARR)
        DO K = 1,NAW
           Names(I,K) = DIR_name//CUT_STR(STR_ARR(K)%S)
        ENDDO
     ENDDO

  CASE('Scattering_matrix')
     IF (LOG_LEVEL(1) >= N_ESO) WRITE(LUNIT,*) &   
          ' ----- "Scattering_matrix" mode is used ---------'
     ! Read the file names contaning the scattering function/matrix
     ! ============================================================
     Nel = 1
     IF( Nvs > 1 ) THEN
        STR_INP = 'Sequence of scattering matrix elements (cai)'
        CALL Read_Input (STR_INP, READ_ARR, *880)
        CALL SPLIT_STRING(READ_ARR(1)%S, STR_ARR)
        Nel = SIZE(STR_ARR)
        ALLOCATE( ESM(Nel) )
        IF( Nel > 6 ) GOTO 131
        DO I = 1, Nel
           READ(STR_ARR(I)%S,*,ERR=101)  ESM(I)
           SELECT CASE( ESM(I) )
           CASE('F11','F22','F33','F44','F12','F21','F34','F43')
           CASE DEFAULT
              CALL NOT_FOUND (R_NAME, STR_INP, ESM(I), *880)    
           END SELECT
        ENDDO
     ENDIF  ! scattering matrix

     STR_INP = 'File names for scattering function/matrix (cai)'
     CALL Read_Input (STR_INP, READ_ARR, *880)
     IF(SIZE(READ_ARR) < LA_N) GOTO 101
     DO I = 1,LA_N
        CALL SPLIT_STRING(READ_ARR(I)%S, STR_ARR)
        DO K = 1,NAW
           Names(I,K) = DIR_name//CUT_STR(STR_ARR(K)%S)
        ENDDO
     ENDDO
  CASE DEFAULT
     GOTO 127
  END SELECT

  IF( L_BOT_I(1)    < ZZZ_UW(1) )     GOTO 102
  IF( L_BOT_I(LA_N) > ZZZ_UW(NL_UW) ) GOTO 103

  DO I = 1,LA_N
     LA_S(I) = MINLOC( DABS(L_TOP_I(I) - ZZZ_UW(:)),1 )
     LA_E(I) = MINLOC( DABS(L_BOT_I(I) - ZZZ_UW(:)),1 )
  ENDDO

  IF(LOG_LEVEL(1) >= N_ESO) THEN
     WRITE(LUNIT,*) '  Check bottom depth of ice layers:'
     DO I = 1,LA_N
        WRITE(LUNIT,'(2x,I3,2x,I3,2x,F7.3)') I, LA_E(I), ZZZ_UW(LA_E(I))
     ENDDO
  END IF
  ! Reading extinction coefficient for all layers and wavelengths
  ! =================================================================
  STR_INP = 'Extinction coefficient (cai)'
  CALL Read_Input (STR_INP, READ_ARR, *880)
  IF(SIZE(READ_ARR) < LA_N) GOTO 101
  ALLOCATE( X_nam(LA_N,N_HPC) )
  DO I1 = 1,LA_N
     CALL SPLIT_STRING(READ_ARR(I1)%S, STR_ARR)
!    ... Filenames containing extinction coefficients of inclusions in ice
     DO K = 1,N_HPC
        X_nam(I1,K) = DIR_name//CUT_STR(STR_ARR(K)%S)
     ENDDO  ! over inclusions in ice (K)
  ENDDO  ! over ice layers (I1)

  DO I1 = 1, LA_N
! ... Top and bottom of an ice layer
     JT = IWS(1)%IW_TOP(I1)  ; JB = IWS(1)%IW_BOT(I1)
     ILN = JB - JT + 1
     ALLOCATE( Z_act(ILN) )  ; Z_act(:) = ZZZ_UW(JT:JB)
     DO K = 1,N_HPC
        OPEN(IUNIT, FILE=TRIM(X_nam(I1,K)), STATUS='OLD' )
        CALL SKIP_HEADER (IUNIT)
        READ(IUNIT,*) N_Z_EXT
        ALLOCATE( R_COF(N_Z_EXT,NAW), Z_EXT(N_Z_EXT) )
        DO I = 1, N_Z_EXT
           READ(IUNIT,*) Z_EXT(I), (R_COF(I,J), J = 1,NAW)
        ENDDO
        IF( MINVAL(Z_EXT) /= 0.d0.OR.MAXVAL(Z_EXT) /= 1.d0 ) GOTO 133
        ALLOCATE( Z_inp(N_Z_EXT) )
        Z_inp(1:N_Z_EXT) = Z_act(1) + Z_EXT(:)*( Z_act(ILN) - Z_act(1) )
        Z_inp(N_Z_EXT) = Z_act(ILN)        
        DO J = 1,NAW
           CALL LIN_INT (N_Z_EXT, Z_inp, R_COF(:,J), ILN, Z_act, &
                IWS(IiW)%HS_COEFF(2,JT:JB,J,K), 'Z')
        ENDDO        
        DEALLOCATE( R_COF,Z_EXT,Z_inp )
     ENDDO  ! over inclusions in ice (K)
     DEALLOCATE( Z_act )
  ENDDO  ! over ice layers (I1)

!  DO I = 1, NL_UW
!     WRITE(*,'(2x,F8.4,2x,*(2x,e12.4))') &
!          ZZZ_UW(I), (IWS(IiW)%HS_COEFF(2,I,1,K),K=1,N_HPC)
!  ENDDO
!  WRITE(*,*) '  ------------------------------------------------------- '
!  STOP '   Test in man_ice_setup '
  
  ! Reading single scattering albedo for all layers and wavelengths
  ! ===================================================================
  STR_INP = 'Single scattering albedo (cai)'
  CALL Read_Input (STR_INP, READ_ARR, *880)
  IF(SIZE(READ_ARR) < LA_N) GOTO 101
  DO I1 = 1,LA_N
     CALL SPLIT_STRING(READ_ARR(I1)%S, STR_ARR)
!    ... Filenames containing SSA of inclusions in ice
     DO K = 1,N_HPC
        X_nam(I1,K) = DIR_name//CUT_STR(STR_ARR(K)%S)
     ENDDO ! over inclusions in ice (K)
  ENDDO  ! over ice layers (I1)

  DO I1 = 1, LA_N
     JT = IWS(1)%IW_TOP(I1)  ; JB = IWS(1)%IW_BOT(I1)
     ILN = JB - JT + 1
     ALLOCATE( Z_act(ILN) )  ; Z_act(:) = ZZZ_UW(JT:JB)
     DO K = 1,N_HPC
        OPEN(IUNIT, FILE=TRIM(X_nam(I1,K)), STATUS='OLD' )
        CALL SKIP_HEADER (IUNIT)
        READ(IUNIT,*) N_Z_ABS
        ALLOCATE( SSA_TMP(N_Z_ABS,NAW), Z_ABS(N_Z_ABS), HS_SSA(NL_UW) )
        DO I = 1, N_Z_ABS
           READ(IUNIT,*) Z_ABS(I), (SSA_TMP(I,J), J = 1,NAW)
        ENDDO
        IF( MINVAL(Z_ABS) /= 0.d0.OR.MAXVAL(Z_ABS) /= 1.d0 ) GOTO 133
        ALLOCATE( Z_inp(N_Z_ABS) )
        Z_inp(1:N_Z_ABS) = Z_act(1) + &
             Z_ABS(:)*( Z_act( ILN) - Z_act(1) ) 
        Z_inp(N_Z_ABS) = Z_act(ILN)
        DO J = 1,NAW
           IF( ANY(SSA_TMP(:,J)>1.0d0) .OR. ANY(SSA_TMP(:,J)<0.0d0) ) GOTO 130
           CALL LIN_INT (N_Z_ABS, Z_inp, SSA_TMP(:,J), ILN, Z_act, &
                HS_SSA(JT:JB), 'Z')
           IWS(IiW)%HS_COEFF(1,JT:JB,J,K) = &
                IWS(IiW)%HS_COEFF(2,JT:JB,J,K)*HS_SSA(JT:JB)
        ENDDO
        DEALLOCATE( SSA_TMP,Z_ABS,HS_SSA,Z_inp )
     ENDDO  ! over inclusions in ice (K)
     DEALLOCATE( Z_act )
  ENDDO  ! over ice layers (I1)
 
  IF (LOG_LEVEL(1) >= N_ESO) THEN
     WRITE(*,*) '-------> Extinction coefficient all inclusions at first WL '
     DO K = 1, NL_UW
        WRITE(*,'(2x,f9.4,*(2x,E12.4))') ZZZ_UW(K), IWS(IiW)%HS_COEFF(2,K,1,:)
     ENDDO
     WRITE(*,*) '-------> Scattering coefficient all inclusions at first WL'
     DO K = 1, NL_UW
        WRITE(*,'(2x,f9.4,*(2x,E12.4))') ZZZ_UW(K), IWS(IiW)%HS_COEFF(1,K,1,:)
     ENDDO
!  STOP '   Test in man_ice_setup '  
     WRITE(LUNIT,*) ' ------ Test of ice layering man mode --- '
     WRITE(LUNIT,*) ' ----                                 --- '
     DO I = 1,LA_N
        WRITE(LUNIT,'(2x,I3,2(1x,f7.3))') I, ZZZ_UW(LA_S(I)) ,ZZZ_UW(LA_E(I)) 
     ENDDO
     WRITE(LUNIT,*) ' ----------------------------------------- '
     DO I = 1, NL_UW
        WRITE(*,'(2x,I3,2x,F9.4,*(2x,E12.4))') &
             I, ZZZ_UW(I), IWS(IiW)%HS_COEFF(1,I,1,:)
     END DO
     WRITE(LUNIT,*) ' ----------------------------------------- '
  ENDIF
!  STOP '   After  "Test of ice layering" '
  
! *********************************************************************
! *              Scattering function/matrix generation                *
! *********************************************************************
  SELECT CASE (TRIM(Ph_type))
  CASE('Asymmetry_factor')
     ! Calculate expansion coefficients corresponding asymmetry factors
     ! ===================================================================
     DO I1 = 1,LA_N
        DO K = 1,NAW
           DO Ii = 1, N_HPC
              GG = ASYM_PAR(I1,K,Ii)
              DO L = 1,MUM
                 TMP1 = GG**(L-1)*(2*(L-1) + 1)
                 IF( TMP1 < 1.d-06 ) EXIT
                 USER_MOM(L,I1,1,K,Ii) = TMP1
              ENDDO  ! over expansion coefficients (L)
              M_Lay_Wav(I1,K,Ii) = L-1
           ENDDO  ! over inclusions in ice (Ii)
        ENDDO  ! over wavelength grid (K)
     ENDDO  ! over ice layers (I1)
     NU_A = M_Lay_Wav(1,1,1)
!     WRITE(*,*) '    M_Lay_Wav  ', M_Lay_Wav(1,1,:)
!---------------------------------------------------------------------------------
     IF( LOG_LEVEL(1) >= N_ESO ) THEN
        WRITE(LUNIT,*) '  Check of 5 expansion coefficients: '
        DO I1 = 1, LA_N
           DO Ii = 1, N_HPC
              WRITE(LUNIT,'(2(2x,I2),4x,*(1x,f10.7))') &
                   I1, Ii, (USER_MOM(L,I1,1,1,Ii)/(2*(L-1) + 1), L=1,5)
           ENDDO  ! over inclusions in ice (Ii)
        ENDDO  ! over ice layers (I1)
     ENDIF
!---------------------------------------------------------------------------------

!    ... Calculate other elements of scattering matrix 
     IF( Nvs > 1 ) THEN
        M_cof = MAXVAL( M_Lay_Wav )  ;  M_ang = 2*M_cof
        CALL HG_SCATT_MAT( M_ang, M_cof )
     ENDIF
     
     TEST_MODE = .FALSE.
     ! Test output of expansion coefficients corresponding to  
     ! the given asymmetry parameter
     ! ===============================================================
     IF( TEST_MODE ) CALL OUT_COEFF

     ! Test output of scattering matrix corresponding to 
     ! the given asymmetry parameter
     ! ===============================================================
     Nc_out = 1
     Ii = M_Lay_Wav(1,1,Nc_out)
     IF( TEST_MODE ) CALL COEF_TO_SCATMATR(Ii, Nm_vs, &
          USER_MOM(1:Ii,1,1:Nm_vs,1,Nc_out), 19, 'E', DIR_OUT//'/man_ice.dat')

  CASE('Expansion_coeff')
     ! Reading of the expansion coefficients 
     ! ==================================================================
     IF (LOG_LEVEL(1) >= N_ESO) &
          WRITE(LUNIT,*) '    Reading of the expansion coefficients  '
     DO I1 = 1,LA_N
        DO K = 1,NAW
           OPEN( IUNIT, FILE=TRIM(Names(I1,K)), STATUS='OLD' )
           CALL SKIP_HEADER (IUNIT)           
           DO I = 1,MUM
              READ(IUNIT,*,END=28,ERR=29) tmp1, (tmp2(J), J=1,Nm_vs)
              USER_MOM(I,I1,:,K,1) = tmp2(:) 
!              WRITE(*,'(f5.1,6(1x,e12.4))') tmp1,(tmp2(J), J=1,Nm_vs)
              ! The additional condition, I > 2, was added to read 
              ! Rayleigh phase function in the scalar case
              ! which is used for test purpose
              IF( SUM(TMP2(:)) == 0.d0 .AND. I > 2 ) GOTO 28
           END DO
29         CALL WRITE_MESSAGE (R_NAME, 'ERROR', 'Error reading data file !')
           WRITE(EU,*) '    File name : ', TRIM(Names(I1,K))
           RETURN 1             
28         CONTINUE
           CLOSE(IUNIT)
           IF( Nm_vs > 1 ) THEN
              USER_MOM(:,I1,5,K,1) = -USER_MOM(:,I1,5,K,1)
              USER_MOM(:,I1,6,K,1) = -USER_MOM(:,I1,6,K,1)
           ENDIF
           M_Lay_Wav(I1,K,1) = I - 1
           NU_A = I - 1
!               WRITE(*,*) '  Number of user moment :', NU_A
!               WRITE(*,*) USER_MOM(NU_A,I1,:,K,1)
        ENDDO  ! over wavelength grid (K)
     ENDDO  ! over ice layers (I1)

     IF( LOG_LEVEL(1) >= N_ESO ) WRITE(LUNIT,'(2x,17(1x,f10.7))') &
          (USER_MOM(L,1,1,1,1)/(2*(L-1) + 1), L=1,11)

     ! Test output of scattering matrix corresponding to 
     ! the given expansion coefficients
     TEST_MODE = .FALSE.         
     IF( TEST_MODE ) THEN 
        NU_A = M_Lay_Wav(1,4,1) !Corresponds to Lay_out, Wl_out used below
        CALL COEF_TO_SCATMATR(NU_A, Nm_vs, USER_MOM(1:NU_A,1,1:Nm_vs,4,1),&
             19, 'E', DIR_OUT//'man_hsol.dat')
     ENDIF
     !         NU_A = M_Lay_Wav(1,1,1)
     !         WRITE(*,*) '   CALL to "COEF_TO_SCATMATR"', M_Lay_Wav(1,1,1)
     !         CALL COEF_TO_SCATMATR(NU_A, Nm_vs, 
     !     $        USER_MOM(1:NU_A,1,1:Nm_vs,1,1),
     !     $        19, 'E', DIR_OUT//'test-1.dat')

  CASE('Scattering_matrix')
     ! Calculate the expansion coefficients of scattering function/matrix
     ! ==================================================================
!     MAX_CO = 500 ; N_GAU = 2*MAX_CO
     MAX_CO = 1000 ; N_GAU = MAX_CO
     CALL PH_SCAT_TO_COEFF(MAX_CO, N_GAU, *880)
!     STOP
  END SELECT

  ! Preliminary for first layer and first wavelength ???
  NU_A = MAXVAL( M_Lay_Wav )
  
  ! Single scattering correction algorithm -> save all expansion coeff.
  ! --------------------------------------    ------------------------
  ! write(*,*) '   Before SSRCA '
  IF( DO_SSRCA .OR. TRIM(RTM_TYPE) == 'spher_scat' ) THEN
     NG_HS = MAXVAL( M_Lay_Wav )
     IF (LOG_LEVEL(1) >= N_ESO) &
          WRITE(LUNIT,*) '  Allocate Greek_HS :', NG_HS
     ALLOCATE( Greek_HS( NG_HS,LA_N,Nm_vs,NAW,N_HPC ) )
     Greek_HS = 0.d0
     DO I = 1,NAW 
        Greek_HS(:,:,:,I,:) = USER_MOM(1:NG_HS,:,:,I,:)
     ENDDO
  ENDIF
!        write(*,*) '     Scaling factor', HS_SCALING
!        write(*,*) '     N_HPC ', N_HPC
  DO K = 1,NAW
     TAU(K) = 0.d0
     DO Ii = 1, N_HPC 
        TAU(K) = TAU(K) + SUM( IWS(IiW)%HS_COEFF(2,:,K,Ii)*DZ_UW(:) )/2.d0
     ENDDO
     TAU(K) = HS_SCALING*TAU(K)
  ENDDO

!  write(*,*) '    Before SCAL TAU(1) = ', TAU(1)
!  PAUSE  
  IF( Scal_HS ) THEN
     CALL LIN_SEARCH(Rwl_HS, DUM, K, NAW, HS_WAVS, ERROR)
     IF (ERROR) GOTO 126
     Scoe = Rtau_HS/( (1.D0 - DUM)*TAU(K) + DUM*TAU(K+1) )
     IWS(IiW)%HS_COEFF(2,:,:,:) = Scoe*IWS(IiW)%HS_COEFF(2,:,:,:)
     IWS(IiW)%HS_COEFF(1,:,:,:) = Scoe*IWS(IiW)%HS_COEFF(1,:,:,:)
!     WRITE(*,*) '  Old tau:', TAU
     TAU(:) = Scoe*TAU(:)
!     WRITE(*,*) '  New tau:', TAU
  ENDIF

  ! ************** Delta-M approach ****************
  ! Allocate array for MODULE DELTA_M
  ! =================================
  IF ( DO_HSOL_DM  ) THEN   ! Set to .TRUE. in "read_uwt.f"
     Itr = HSOL_TRUNC_IDX   ! Set to NLEG_W-1 in "gt_settings_check.f"
     SELECT CASE( Vec_Sca )
     CASE('vect_rte')
        CALL AER_DM_VEC
     CASE('scal_rte')
        CALL AER_DM
     END SELECT

     ! "Renormalize" scattering and extinction coefficients
     ! ====================================================
     DO I = 1, NL_UW
        DO K = NWLS, NWLE
           DO Ii = 1, N_HPC
              F_DEL = F_DLM_HSOL( I,K,Ii )
              ABS_COEFF = IWS(IiW)%HS_COEFF(2,I,K,Ii) - IWS(IiW)%HS_COEFF(1,I,K,Ii)
              IWS(IiW)%HS_COEFF(1,I,K,Ii) = IWS(IiW)%HS_COEFF(1,I,K,Ii)*(1.0d0 - F_DEL)
              IWS(IiW)%HS_COEFF(2,I,K,Ii) = IWS(IiW)%HS_COEFF(1,I,K,Ii) + ABS_COEFF
           ENDDO  ! over inclusions in ice (Ii)
        ENDDO  ! over wavelengths (K)
     ENDDO   ! over layers (I1)
  ELSE
     L = MAXVAL( M_Lay_Wav )
     IF( L > NLEG_W ) THEN
        CALL WRITE_MESSAGE(R_NAME, 'WARNING', &
             'The number of inclusions expansion moments greater than '&
             //'"Number of Legendre moments"')
        WRITE(EU,'(a46,1x,I4)') &
             '      "Number of Legendre moments"           ', NLEG_W
        WRITE(EU,'(a46,1x,I4)') &
             '      Maximal number of moments after reading', L
     ENDIF
     DO K = 1,NAW
        IWS(IiW)%HSOL_PHAS_VS(:,:,:,K,:) = USER_MOM(1:NLEG_W,:,:,K,:)
     ENDDO
  ENDIF

  CALL WRITE_SCEN_ICE 

  CALL ST_TRACK (R_NAME, 1, 'O')
  RETURN

880 WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)
  RETURN 1

101 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Reading error')
  WRITE(EU,*) '    Parameter to read: ', TRIM(STR_INP) 
  WRITE(EU,*) '    Input line       : ', TRIM(STR_READ) 
  RETURN 1

102 CALL WRITE_MESSAGE(R_NAME, 'ERROR','Upper ice layer outside the ocean')
  WRITE(EU,'(A,2x,F9.2,A,F9.2)') &
       '    Upper ice layer:', ZZZ_UW(1),' - ',L_BOT_I(1)
  WRITE(EU,'(A,2x,F9.2,A)') &
       '      Top of ocean     :', ZZZ_UW(1),' m'
  WRITE(EU,*) '    Check input file "cf_ice_man.inp"'
  RETURN 1

103 CALL WRITE_MESSAGE(R_NAME, 'ERROR', ' Lower ice layer outside the ocean')
  WRITE(EU,'(A,2x,F6.2,A)') &
       '       Bottom of ice layer:',L_BOT_I(LA_N),' mm'
  WRITE(EU,'(A,2x,F6.2,A)') &
       '                Ocean bottom   :', ZZZ_UW(NL_UW),' mm'
  WRITE(EU,*) '    Check "cf_ice_man.inp" and "Ocean depth" '
  WRITE(EU,*) '    in the file "control_uwt.inp" or "control_uit.inp"'
  RETURN 1

104 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'The interpolation of '//&
         'inclusions parameters can not be performed because')
  WRITE(EU,'(a34,1x,f7.3,1x,a38,1x,f7.3)') &
       '     first wavelength for inclusion ',HS_WAVS(1), &
       ' > than start wavelength for radiance ',START_WAV
  RETURN 1

105 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'The interpolation of inclusion '//&
         'parameters can not be performed because')
  WRITE(EU,'(a35,1x,f7.3,1x,a36,1x,f7.3)') &
       '       last wavelength for inclusion ',HS_WAVS(NWLE),&
       ' < than end wavelength for radiance ',END_WAV
  RETURN 1

106 CALL WRITE_MESSAGE(R_NAME, 'ERROR', &
         'The bottom altitudes of ice layers have to be'&
         //' given in the ascending order')
  WRITE(EU,'(A)') '   Check file "man_hsol.inp" after key-sentence'
  WRITE(EU,'(A)') '   "Layering of scattering function/matrix"'
  RETURN 1

126 CALL WRITE_MESSAGE (R_NAME,'ERROR',&
         'Reference wavelength for IOT is out of range')
  WRITE(EU,'(A,F7.2)') '    Reference wavelength: ', Rwl_HS
  WRITE(EU,'(A,2(2x,F7.2))') &
       '    Database wavelengths: ', HS_WAVS(1), HS_WAVS(NAW)
  WRITE(EU,'(A)') &
       '    Set reference wavelength between database wavelengths.'
  RETURN 1
  
127 CALL WRITE_MESSAGE (R_NAME, 'ERROR', 'Type of ice inclusion '//&
         'scattering function parameterization is not recognized')
  RETURN 1

130 CALL WRITE_MESSAGE (R_NAME, 'ERROR', 'SSA of inclusions in ice'// &
         'should be in the range [0,1].')
  WRITE(EU,'(A,2(2x,F7.2))') &
       '    Check input values in the file  ', TRIM(X_nam(1,K))
  RETURN 1

131 CALL WRITE_MESSAGE (R_NAME, 'ERROR', 'Number of matrix elements '// &
         'can not be larger than 6.')
  WRITE(EU,'(A,2(2x,F7.2))') &
       '    Check input values in the file "man_hsol.inp" '
  RETURN 1

132 CALL WRITE_MESSAGE (R_NAME, 'ERROR', ' ')
  WRITE(EU,'(A)') ' The top altitude of ice layer have to be set to 0  '
  WRITE(EU,'(A)') '   in the case of RT calculations through ice.  '
  WRITE(EU,'(A)') ' Check file "cf_ice_man.inp" after key-sentence'
  WRITE(EU,'(A)') '   "Layering of scattering function/matrix (cai)"'
  RETURN 1

133 CALL WRITE_MESSAGE (R_NAME, 'ERROR', ' ')
  WRITE(EU,'(A)') ' The dimensionless top and bottom depths of an ice layer '
  WRITE(EU,'(A)') '    have to be in the range [0,1] '
  WRITE(EU,'(A,A)') ' Check input file: ', TRIM(X_nam(1,K))
  RETURN 1
  
CONTAINS
  
!***************************************************************************
  SUBROUTINE HG_SCATT_MAT( N_gau, M_cof )

    USE ST_VARS, ONLY: Pie
! ------------------------------------------------------------------------
    INTEGER N_gau, M_cof
    INTEGER N_cof, L
    REAL (KND), DIMENSION (N_gau) :: X_gau, AG_gau, R12, R33, &
         H11, H12, H33
    REAL (KND)  Cofm, Cofp, Clk, Dum
    REAL (KND)  Poly(M_cof,N_gau), D_02g(M_cof,N_gau), & 
         D_P2g(M_cof,N_gau), D_M2g(M_cof,N_gau), Cf(Nm_vs)
!    REAL (KND) Ph_ret(N_gau,Nm_vs)
! =========================================================================
    
    ! Gauss grid and weights for interval (-1,1)
    CALL GAULEG ( -1.d0, 1.d0, X_gau, AG_gau, N_gau )
    ! Legendre polynomials and Wigner functions for all Gauss angles
    CALL WIGNER_PLG     ( X_gau, N_gau, 0, M_cof, Poly )
    CALL WIGNER_PLG     ( X_gau, N_gau, 2, M_cof, D_02g )
    CALL WIGNER_FUNCTION( X_gau, N_gau, 2, M_cof, D_P2g, D_M2g )
    
    ! Normalized elements of Rayleigh scattering matrix
    !    R11 = R22, R33 = R44, R34 = 0
    !    R11(:) =  3.d0/4.d0*(1.d0 + X_gau(:)**2)
    !    R12(:) = -3.d0/4.d0*(1.d0 - X_gau(:)**2)
    !    R33(:) =  3.d0/2.d0*X_gau(:)
    R12(:) = -(1.d0 - X_gau(:)**2)/(1.d0 + X_gau(:)**2)
    R33(:) =  2.d0*X_gau(:)/(1.d0 + X_gau(:)**2)
    
    ! Elements of HG scattering matrix
    DO I1 = 1, LA_N
       DO K = 1, NAW
          DO Ii = 1, N_HPC
             GG = ASYM_PAR(I1,K,Ii)
             H11(:) = (1.d0 - GG**2) &
                  /(1.d0 - 2.d0*GG*X_gau(:) + GG**2)**1.5d0
             H12(:) = H11(:)*R12(:) !/R11(:)
             H33(:) = H11(:)*R33(:) !/R11(:)
             DO L = 1, M_Lay_Wav(I1,K,Ii) 
                Clk = DBLE(L) - 0.5d0
                Cf(1) =  Clk*SUM( H11(:)*Poly(L,:)*AG_gau(:) )
                Cf(4) =  SUM( H33(:)*Poly(L,:)*AG_gau(:) )
                Cf(5) = -SUM( H12(:)*D_02g(L,:)*AG_gau(:) )
                Cf(6) = 0.d0
                Cofm = SUM( (H11(:) - H33(:))*D_M2g(L,:)*AG_gau(:) )
                Cofp = SUM( (H11(:) + H33(:))*D_P2g(L,:)*AG_gau(:) )
                Cf(2) = (Cofp + Cofm)/2.d0
                Cf(3) = (Cofp - Cofm)/2.d0
                Cf(2:Nm_vs) = Clk*Cf(2:Nm_vs)
                Cf(5:6) = -Cf(5:6)
                USER_MOM(L,I1,:,K,Ii) = Cf(:)
             ENDDO  ! over expansion coefficients (L)
          ENDDO  ! over inclusions in ice (Ii)
       ENDDO  ! over wavelength grid (K)
    ENDDO  ! over ice layers (I1)
    
    ! Check HG expansion coefficients 
!    I1 = 1 ;  K = 1
!    Ph_ret(:,:) = 0.d0
!    DO L = 1,NU_A
!       Cf(:) = USER_MOM(L,I1,:,K,1)
!       Ph_ret(:,1) = Ph_ret(:,1) + Poly(L,:) *Cf(1)
!       Ph_ret(:,4) = Ph_ret(:,4) + Poly(L,:) *Cf(4)
!       Ph_ret(:,5) = Ph_ret(:,5) - D_02g(L,:) *Cf(5)
!       Ph_ret(:,6) = Ph_ret(:,6) - D_02g(L,:) *Cf(6)
!       Cofp = Cf(2) + Cf(3)
!       Cofm = Cf(2) - Cf(3)
!       Ph_ret(:,2) = Ph_ret(:,2) + &
!            (D_P2g(L,:)*Cofp + D_M2g(L,:)*Cofm)/2.d0
!       Ph_ret(:,3) = Ph_ret(:,3) + &
!            (D_P2g(L,:)*Cofp - D_M2g(L,:)*Cofm)/2.d0
!    ENDDO
!    OPEN( 47,FILE='./DATA_OUT/Phm_orig.dat' )
!    DO J = 1,N_gau
!       Dum = ACOS( X_gau(J) )/Pie*180.
!       WRITE(47,'(1x,*(e15.7))') Dum, &
!            H11(J), H11(J), H33(J), H33(J), H12(J)
!    ENDDO
!    CLOSE(OUNIT)
!    OPEN( 47,FILE='./DATA_OUT/Phm_ret.dat' )
!    DO J = 1,N_gau
!       Dum = ACOS( X_gau(J) )/Pie*180.
!       WRITE(47,'(1x,*(e15.7))') Dum, Ph_ret(J,1), &
!            Ph_ret(J,2), Ph_ret(J,3), Ph_ret(J,4), Ph_ret(J,5)
!    ENDDO
!    CLOSE(OUNIT)

  END SUBROUTINE HG_SCATT_MAT
  
!***************************************************************************
   SUBROUTINE AER_DM
    IF (LOG_LEVEL(1) >= N_ESO) THEN
       WRITE(LUNIT,*) '   ***************************  '
       WRITE(LUNIT,*) '   Inclusions D-M scalar case  '
       WRITE(LUNIT,*) '   ***************************  '
       WRITE(LUNIT,*) '   Number of Legendre terms :', NLEG_W
       WRITE(LUNIT,*) '           Truncation index :', Itr
    ENDIF
    DO I1 = 1,LA_N
       DO K = 1,NAW
          DO Ii = 1, N_HPC
             F_DEL = USER_MOM(Itr+1,I1,1,K,Ii)/DBLE(2*Itr + 1)
             F_DLM_HSOL(LA_S(I1):LA_E(I1),K,Ii) = F_DEL
             DO L = 1,Itr
                HS_PHASMOMS(L,I1,K,Ii) = (USER_MOM(L,I1,1,K,Ii) - &
                     F_DEL*DBLE(2*L-1))/(1.D0 - F_DEL)
             ENDDO  ! over moments (L)
          ENDDO  ! over inclusions in ice (Ii)
       ENDDO  ! over wavelengths (K)
    ENDDO   ! over layers (I1)
    HS_PHASMOMS(Itr+1:NLEG_W,:,NWLS:NWLE,:) = 0.d0
    
  END SUBROUTINE AER_DM

!***************************************************************************
  SUBROUTINE AER_DM_VEC
    IF (LOG_LEVEL(1) >= N_ESO) THEN
       WRITE(LUNIT,*) '   ***********************  '
       WRITE(LUNIT,*) '   Hydosol D-M vector case  '
       WRITE(LUNIT,*) '   ***********************  '
    ENDIF
    DO I1 = 1,LA_N
       DO K = 1,NAW
          DO Ii = 1, N_HPC
             F_DEL = USER_MOM(Itr+1,I1,1,K,Ii)/DBLE(2*Itr + 1)
             F_DLM_HSOL(LA_S(I1):LA_E(I1),K,Ii) = F_DEL
             DO L = 1, Itr
                IWS(IiW)%HSOL_PHAS_VS(L,I1,1:4,K,Ii) = (USER_MOM(L,I1,1:4,K,Ii) -&
                     F_DEL*DBLE(2*L-1))/(1.0D0 - F_DEL)
                IWS(IiW)%HSOL_PHAS_VS(L,I1,5:6,K,Ii) = &
                     USER_MOM(L,I1,5:6,K,Ii)/(1.0D0 - F_DEL)
             ENDDO
          ENDDO  ! over inclusions in ice (Ii)
       ENDDO
    ENDDO
    IWS(IiW)%HSOL_PHAS_VS(Itr+1:NLEG_W,:,:,:,:) = 0.d0

  END SUBROUTINE AER_DM_VEC

!***************************************************************************
  SUBROUTINE WRITE_SCEN_ICE
    REAL (KND), ALLOCATABLE :: SS_A(:), Ext(:), Sca_co(:), Abs(:)
    CHARACTER (LEN = :), ALLOCATABLE :: FMT_S1, FMT_S2, FMT_A, FMT_B, FMT_C
    Character N_STR*20
!   ========================================================================= 
!    DOUBLE PRECISION    SS_A, Ext, Sca_co, Abs
    IF (LOG_LEVEL(1) >= N_SSO) WRITE (LUNIT,*) ' *** Writing: SCE_MAN-ICE.OUT'
    IF( .NOT.ALLOCATED(SS_A) ) &
         ALLOCATE( SS_A(N_HPC), Ext(N_HPC), Sca_co(N_HPC), Abs(N_HPC) )

    OPEN (IUNIT, FILE = DIR_OUT//'/SCE_MAN-ICE.OUT')
    CALL WRITE_UD(1,IUNIT)
    WRITE ( IUNIT, * ) '           =================================='
    WRITE ( IUNIT, * ) '            MANUAL ICE INCLUSIONS  SELECTION '
    WRITE ( IUNIT, * ) '           =================================='

    IF( Scal_HS ) THEN
       WRITE(IUNIT,'(A,F7.3,A,F9.2,A)') '   !!! ATTENTION !!! IOT is set to ',&
            Rtau_HS,' at', Rwl_HS,' nm wavelength.'
       WRITE(IUNIT,'(19x,A)') '  See setting in "gt_fileread.f" '
       WRITE(IUNIT,'(6x,A)') &
            '----------------------------------------------------'
       WRITE(IUNIT,*) 
    ENDIF

    WRITE ( IUNIT, * ) ' '
    WRITE ( IUNIT, * ) ' Vertical profiles of ice inclusions '
    WRITE ( IUNIT, * ) '         extinction coefficient (Ext coeff) and'
    WRITE ( IUNIT, * ) '         single scattering albedo (Ss albedo):'
    WRITE ( IUNIT, * ) '         (IOT - ice optical thickness)'

    FMT_A = '(6x,I3,4x,F7.3,2x,'
    FMT_B = '(1x,ES9.2)'
    FMT_C = '(1x,E12.4))'
    WRITE(N_STR,'(I0)') N_HPC
    FMT_S1 = TRIM(FMT_A)//TRIM(N_STR)//TRIM(FMT_B)//'2x'//TRIM(N_STR)//TRIM(FMT_C)
    FMT_A = '(6x,I3,3x,F6.2,4x,F7.2,5x,'
    FMT_B = '(1x,I4),'
    FMT_C = '(1x,E10.2))'
    FMT_S2 = TRIM(FMT_A)//TRIM(N_STR)//TRIM(FMT_B)//'2x,'//TRIM(N_STR)//TRIM(FMT_C)

    DO K = 1,NAW            ! Reference wavelength do-loop
       WRITE(IUNIT,*) '   '
       WRITE(IUNIT,'(1x,A,F9.2,A,A,ES12.4)') &
            '--> Wavelength ',HS_WAVS(K),' [nm];',' IOT =',TAU(K)
       WRITE(IUNIT,'(5x,41A)') ('-', I=1,41)

       WRITE ( IUNIT, * ) '    Number, Depth [m], '&
            //'Ext coeff [m-1] =>,    Ss albedo =>'
       DO I = 1, NL_UW
          Ext(:) = IWS(IiW)%HS_COEFF(2,I,K,1)
          Abs(:) = IWS(IiW)%HS_COEFF(2,I,K,1) - IWS(IiW)%HS_COEFF(1,I,K,1)
          Sca_co(:) = IWS(IiW)%HS_COEFF(1,I,K,1)/(1.D0 - F_DLM_HSOL(I,K,1))
          IF( SUM(Ext(:)) == 0.0d0 ) CYCLE
          SS_A(:) = Sca_co(:)/(Sca_co(:) + Abs(:))
          WRITE(IUNIT,FMT = FMT_S1) I, ZZZ_UW(I), Sca_co(:)+Abs(:) , SS_A(:)
       END DO  ! over depth (I)
       WRITE(IUNIT,*) ' '

       SELECT CASE (TRIM(Ph_type))
       CASE('Asymmetry_factor')
          WRITE(IUNIT,*) &
               '    Asymmetry factor used in the ice layers :'
          WRITE(IUNIT,'(5x,50A)') ('-', I=1,50)
          WRITE(IUNIT,*) &
               '    Layer  Top [km]  Bottom [km]   Asymmetry factor'
          DO I = 1,LA_N
             WRITE(IUNIT,'(6x,I3,3x,F6.2,4x,F7.2,8x,*(1x,F6.3))') &
                  I, ZZZ_UW(LA_S(I)), ZZZ_UW(LA_E(I)), Asym_Par(I,K,:)
          ENDDO
       CASE('Expansion_coeff')
          WRITE(IUNIT,*) &
               '    Expansion coefficients used in ice layers :'
          WRITE(IUNIT,'(5x,50A)') ('-', I=1,50)
          WRITE(IUNIT,*) &
               '    Layer  Top [km]  Bottom [km]   File name'
          DO I = 1,LA_N
             WRITE(IUNIT,'(6x,I3,3x,F6.2,4x,F7.2,6x,a)') &
                  I, ZZZ_UW(LA_S(I)) ,ZZZ_UW(LA_E(I)), TRIM(Names(I,K))
          ENDDO
       CASE('Scattering_matrix')
          WRITE(IUNIT,*) '    Scattering matrix used in ice layers :'
          WRITE(IUNIT,'(5x,50A)') ('-', I=1,50)
          WRITE(IUNIT,*) &
               '    Layer  Top [km]  Bottom [km]   File name'
          DO I = 1,LA_N
             WRITE(IUNIT,'(6x,I3,3x,F6.2,4x,F7.2,6x,a)') &
                  I, ZZZ_UW(LA_S(I)) ,ZZZ_UW(LA_E(I)), TRIM(Names(I,K))
          ENDDO
       END SELECT

       WRITE(IUNIT,*) ' '
       WRITE(IUNIT,*) '    Number of expansion coeff. and Delta-M factor:'
       WRITE(IUNIT,'(5x,45A)') ('-', I=1,45)
       WRITE(IUNIT,*) '    Layer  Top [km]  Bottom [km]   N coef       Delta-M'
       DO I = 1,LA_N
          WRITE(IUNIT,FMT = FMT_S2) I, ZZZ_UW(LA_S(I)), ZZZ_UW(LA_E(I)), &
               M_Lay_Wav(I,K,:), F_DLM_HSOL(LA_S(I),K,:)
       ENDDO
    ENDDO  ! over all wavelengths 

    CALL WRITE_UD(2,IUNIT)
    
    CLOSE (IUNIT)          
  END SUBROUTINE WRITE_SCEN_ICE

!***************************************************************************
  SUBROUTINE PH_SCAT_TO_COEFF(MAX_CO, N_GAU, *)
    USE ST_VARS, ONLY: Pie
    INTEGER   MAX_CO, N_GAU
    INTEGER   Na_st, I, K, KK, N_COEFF, L, Lay_out, WL_out, INBV, N_mom
    LOGICAL   Error, TEST_PH, Log_mat(4)
    REAL (KND)  Norm, S1, Clk, Cofp, Cofm
    REAL (KND)  Poly(MAX_CO,N_gau), DUM_READ(Max_co,Nel+1)
    REAL (KND), DIMENSION (N_gau) :: X_gau, AG_gau, X_rad, Phs_gau
    REAL (KND), ALLOCATABLE :: Sa_inp(:), Phs_inp(:), Sa_rad(:), Ph_out(:), &
         Phs_log(:), D_02g(:,:), D_P2g(:,:), D_M2g(:,:), Cf(:), &
         Sca_mat(:,:), Int_mat(:,:), Ph_ret(:,:)
    REAL (KND), ALLOCATABLE ::  WORK(:), B_COEFF(:), S_NODES(:)
  !==================================================================
    Error = .FALSE.
    ! If test_ph is set to .TRUE., the phase function will be recalculated
    ! using obtained expansion coefficients and written in the given 
    ! output file for the ice layer (lay_out) and wavelength (wl_out)
    TEST_PH = .TRUE.
    IF( TEST_PH ) THEN
       Lay_out = 1  ;   WL_out  = 1
    ENDIF

    ! Gauss grid and weights for integration (-1,1)
    CALL GAULEG ( -1.d0, 1.d0, X_gau, AG_gau, N_gau )
    ! Legendre polynomials for all Gauss angles
    CALL WIGNER_PLG( X_gau, N_gau, 0, Max_co, Poly )
    X_rad(:) = DACOS( X_gau(N_gau:1:-1) )
    IF( Nvs > 1 ) THEN
       ALLOCATE( D_02g(Max_co,N_gau), D_P2g(Max_co,N_gau), & 
            D_M2g(Max_co,N_gau), Cf(Nm_vs) )
       CALL WIGNER_PLG     ( X_gau, N_gau, 2, Max_co, D_02g )
       CALL WIGNER_FUNCTION( X_gau, N_gau, 2, Max_co, D_P2g, D_M2g )
    ENDIF  ! Nvs > 1

    ! Reading of the user scattering function/matrix 
    DO I1 = 1,LA_N
       DO K = 1,NAW
          OPEN(C_UNIT, FILE=TRIM(Names(I1,K)), STATUS='OLD') 
          CALL SKIP_HEADER (C_UNIT)           
          DO I = 1,Max_co
             READ(C_UNIT,*,END=128,ERR=129) (DUM_READ(I,J), J=1,Nel+1)
          END DO
129       CALL WRITE_MESSAGE (R_NAME, 'ERROR', 'Error reading data file !')
          WRITE(EU,*) '    File name : ', TRIM(Names(I1,K)) 
          RETURN 1             
128       CONTINUE
          Na_st = I - 1
          CLOSE(C_UNIT)
!          WRITE(*,*) '  Number of scattering angles:', Na_st
!          WRITE(*,*) '  First :', DUM_READ(1,1:2)
!          WRITE(*,*) '  Last  :', DUM_READ(Na_st,1:2)

          ALLOCATE( Sa_inp(Na_st), Sa_rad(Na_st), Phs_inp(Na_st), &
               Phs_log(Na_st) )              ! A          
          ! Original angle grid and scattering function 
          Sa_inp(:)  = DUM_READ(1:Na_st,1)
          Phs_inp(:) = DUM_READ(1:Na_st,2)            

          ! Convert [deg] to [rad] for interpolation of Ph 
          Sa_rad(:) = Sa_inp(:)/RTD
          ! Sa_inp(:) = DCOS( Sa_rad(Na_st:1:-1) )

          ! Interpolate scattering function on the Gauss grid
          ! WRITE(*,*) '  Sa_rad :', sa_rad(1),' - ',Sa_rad(Na_st)
          ! WRITE(*,*) '   X_rad :',  x_rad(1),' - ', x_rad(N_gau)
          Phs_log(:) = DLOG( Phs_inp(:) )
          ! CALL AKIMA_INTERP (Na_st, Sa_rad, Phs_log, N_gau,  &
          ! X_rad, Phs_gau, Error)  
          ! IF( Error ) THEN
          !    Call WRITE_MESSAGE (R_NAME,'ERROR', 'Interpolation error')
          !    RETURN 1
          ! ENDIF
          CALL LIN_INT (Na_st, Sa_rad, Phs_log, N_gau, X_rad, Phs_gau, 'L')  

          ! Invert to start from -1
          Phs_gau(:) = DEXP( Phs_gau(N_gau:1:-1) )

          ! Norm of the interpolated phase function
          Norm = SUM( Phs_gau(:)*AG_gau(:) )*0.5d0

          IF (LOG_LEVEL(1) >= N_ESO) THEN
             S1 = 0.d0
             DO I = 1, Na_st - 1
                S1 = S1 + (Phs_inp(I)*DSIN(Sa_rad(I)) &
                     + Phs_inp(I+1)*DSIN(Sa_rad(I+1)))/2.d0 &
                     *(Sa_rad(I+1) - Sa_rad(I))
             ENDDO
             WRITE(LUNIT,*) '   Integral of the phase function '
             WRITE(LUNIT,*) '     Initial scat-angle grid: ', S1*2.*Pie
             WRITE(LUNIT,*) '     Extended scat-angle grid:', Norm*4.*Pie
             ! Number of expansion coefficients of intrpolated Ph
!             WRITE(*,*) '   Max_co = ', Max_co
          ENDIF
             
          ! Expansion of F11 matrix element          
          DO L = 1,Max_co
             TMP1 = (DBLE(L) - 0.5d0)*SUM( Phs_gau(:)*Poly(L,:)*AG_gau(:) )&
                  /Norm
             IF( ABS(TMP1) < 1.d-06 ) THEN 
                IF( TMP1 < 0.d0 ) THEN 
                   WRITE(LUNIT,*) '   Negative expansion coefficient'
                   WRITE(LUNIT,*) '   for L = ', L
                   WRITE(LUNIT,*) '   Number of Gauss angles ',N_GAU
                   WRITE(LUNIT,*) '   Continue?'
                   EXIT
                ENDIF
             ENDIF
             USER_MOM(L,I1,1,K,1) = TMP1
          ENDDO

          M_Lay_Wav(I1,K,1) = L-1
!          WRITE(*,*) '   Number of coeff = ', M_Lay_Wav(I1,K,1)
          IF( TEST_PH .AND. I1==Lay_out .AND. K==WL_out ) THEN
             ALLOCATE( Ph_out(N_gau) )
             DO J = 1,N_gau
                Ph_out(J) = SUM(Poly(1:L-1,J)*USER_MOM(1:L-1,I1,1,K,1))
             ENDDO
             ! WRITE(*,*) '   Norm of ph after expansion',&
             ! SUM( Ph_out(:)*AG_gau(:) )*0.5d0
             OPEN(OUNIT,FILE=DIR_OUT//'/Ph_scat.out')
             WRITE(OUNIT,'(a)') '#  Interpolated original and '//&
                  'recalculated phase functions'
             WRITE(OUNIT,'(a,2x,I3)') '#  Ice layer :', I1
             WRITE(OUNIT,'(a,2x,I3)') '#  Wavelength    :', K
             WRITE(OUNIT,'(a,2x,E12.4)') '#  Norm of interpolated Ph', Norm
             WRITE(OUNIT,'(a,2x,I5)') &
                  '#  Number of expansion coefficients used', M_Lay_Wav(I1,K,1)
             DO J = 1,N_gau
                WRITE(OUNIT,'(1x,3(e15.7))') DACOS(X_gau(J))*RTD, &
                     Phs_gau(J), Ph_out(J)*Norm
             ENDDO
             CLOSE(OUNIT)
          ENDIF
          IF (LOG_LEVEL(1) >= N_ESO) &
               WRITE(LUNIT,*) '   Number of expansion coeff: ', L-1
          IF( Nvs == 1 ) THEN
             DEALLOCATE( Sa_inp,Sa_rad,Phs_inp,Phs_log )
             CYCLE
          ENDIF
!         *************************************************          
! ***************** Other elements of scattering matrix *************
!         *************************************************          
          ALLOCATE( WORK(5*(Na_st+2)),B_COEFF(Na_st+2),S_NODES(Na_st+6) )
          ALLOCATE( Sca_mat(Na_st,Nm_vs), Int_mat(N_gau,Nm_vs) )
          Log_mat(:) = .FALSE.
          Sca_mat(:,1) = DUM_READ(1:Na_st,2)
          DO I = 2, Nel
             SELECT CASE( ESM(I) )
             CASE('F22')  ;  SCA_MAT(:,2) = DUM_READ(1:Na_st,I+1)
                Log_mat(2) = .TRUE.
             CASE('F33')  ;  SCA_MAT(:,3) = DUM_READ(1:Na_st,I+1)
                Log_mat(3) = .TRUE.
             CASE('F44')  ;  SCA_MAT(:,4) = DUM_READ(1:Na_st,I+1)
                Log_mat(4) = .TRUE.
             CASE('F12','F21') ;  SCA_MAT(:,5) = DUM_READ(1:Na_st,I+1)
             CASE('F34','F43') ;  SCA_MAT(:,6) = DUM_READ(1:Na_st,I+1)
             END SELECT
          ENDDO  ! over scattering matrix elements (I)
          IF( .NOT.Log_mat(2) ) SCA_MAT(:,2) = DUM_READ(1:Na_st,2)
          IF( .NOT.Log_mat(3) ) SCA_MAT(:,3) = SCA_MAT(:,4) 
          IF( .NOT.Log_mat(4) ) SCA_MAT(:,4) = SCA_MAT(:,3) 

          DO J = 1, Nm_vs
             CALL DBINT4 (Sa_rad, Sca_mat(:,J), Na_st, 2, 2, 0.d0, 0.d0, 1,  &
                  S_NODES, B_COEFF, N_COEFF, KK, WORK, *880) 
             INBV = 1
             DO I = 1, N_gau
                CALL DBVALU (S_NODES, B_COEFF, N_COEFF, KK, 0,  &
                     X_rad(I), INBV, WORK(1:12), Int_mat(I,J), *880 )
             END DO
             Int_mat(:,J) = Int_mat(N_gau:1:-1,J)
          ENDDO  ! over 6 matrix elements (J)
          N_mom = M_Lay_Wav(I1,K,1)
          DO L = 1, N_mom
             Clk = (DBLE(L) - 0.5d0)/Norm
!             Cf(1) =  Clk*SUM( Int_mat(:,1)*Poly(L,:)*AG_gau(:) )
             Cf(4) =  SUM( Int_mat(:,4)*Poly(L,:)*AG_gau(:) )
             Cf(5) = -SUM( Int_mat(:,5)*D_02g(L,:)*AG_gau(:) )
             Cf(6) = -SUM( Int_mat(:,6)*D_02g(L,:)*AG_gau(:) )
             Cofm =   SUM( (Int_mat(:,2) - Int_mat(:,3))*D_M2g(L,:)*AG_gau(:) )
             Cofp =   SUM( (Int_mat(:,2) + Int_mat(:,3))*D_P2g(L,:)*AG_gau(:) )
             Cf(2) = (Cofp + Cofm)/2.d0  ;  Cf(3) = (Cofp - Cofm)/2.d0
             Cf(2:Nm_vs) = Clk*Cf(2:Nm_vs)
             Cf(5:6) = -Cf(5:6)
             USER_MOM(L,I1,2:Nm_vs,K,1) = Cf(2:Nm_vs)
          ENDDO

          IF( I1 == 1.and.K == 1) THEN
             IF(.NOT.ALLOCATED(Ph_ret) ) ALLOCATE( Ph_ret(N_gau,Nm_vs) )
             Ph_ret = 0.d0
             DO L = 1, N_mom
                Cf(:) = USER_MOM(L,I1,:,K,1)
                Ph_ret(:,1) = Ph_ret(:,1) + Poly(L,:) *Cf(1)
                Ph_ret(:,4) = Ph_ret(:,4) + Poly(L,:) *Cf(4)
                Ph_ret(:,5) = Ph_ret(:,5) - D_02g(L,:) *Cf(5)
                Ph_ret(:,6) = Ph_ret(:,6) - D_02g(L,:) *Cf(6)
                Cofp = Cf(2) + Cf(3)  ;  Cofm = Cf(2) - Cf(3)
                Ph_ret(:,2) = Ph_ret(:,2) &
                     + (D_P2g(L,:)*Cofp + D_M2g(L,:)*Cofm)/2.d0
                Ph_ret(:,3) = Ph_ret(:,3) &
                     + (D_P2g(L,:)*Cofp - D_M2g(L,:)*Cofm)/2.d0
             ENDDO

             OPEN(OUNIT,FILE=DIR_OUT//'/Ph_scat-ori.out')
             DO J = 1,Na_st
                WRITE(OUNIT,'(1x,*(e15.7))') Sa_rad(J)*RTD, &
                     (Sca_mat(J,I), I = 1,Nm_vs)
             ENDDO
             CLOSE(OUNIT)

             OPEN(OUNIT,FILE=DIR_OUT//'/Ph_scat-int.out')
             DO J = 1,N_gau
                WRITE(OUNIT,'(1x,*(e15.7))') DACOS(X_gau(J))*RTD, &
                     (Int_mat(J,I), I = 1,Nm_vs)
             ENDDO
             CLOSE(OUNIT)

             OPEN(OUNIT,FILE=DIR_OUT//'/Ph_scat-ret.out')
             DO J = 1,N_gau
                WRITE(OUNIT,'(1x,*(e15.7))') DACOS(X_gau(J))*RTD, &
                     (Ph_ret(J,I)*Norm, I = 1,Nm_vs)
             ENDDO
             CLOSE(OUNIT)
          ENDIF

          DEALLOCATE( Sa_inp, Sa_rad, Phs_inp, Phs_log, Sca_mat, Int_mat, &
               WORK, B_COEFF, S_NODES)          
       ENDDO  ! over wavelength grid (K)
    ENDDO  ! over ice layers (I1)

    RETURN
    
880 CONTINUE    
    RETURN 1
    
  END SUBROUTINE PH_SCAT_TO_COEFF

  !***************************************************************************
  SUBROUTINE OUT_COEFF
    OPEN(OUNIT,FILE=DIR_OUT//'/H_G_coeff.out')
    WRITE(OUNIT,'(a)') '#  Expansion coefficients of the H-G phase function'
    WRITE(OUNIT,'(a)') '#  in the lower ice layer at first wavelength. '
    WRITE(OUNIT,'(a,2x,*(F6.3))') '#  Asymmetry parameter :', ASYM_PAR(1,1,:)
    DO L = 1, MAXVAL( M_Lay_Wav(1,1,:),1 )
       WRITE(OUNIT,'(2x,I5,*(1x,E15.7))') L-1, (USER_MOM(L,1,:,1,1),I=1,N_HPC)
    ENDDO
    CLOSE(OUNIT)
  END SUBROUTINE OUT_COEFF
  
!***************************************************************************
END SUBROUTINE MAN_ICE_SETUP
!***************************************************************************

