C**********************************************************************
C
C	SCIATRAN - Version 4.3 - Last modification: 27 Jan 2021
C
C	Copyright (C) - Institute of Remote Sensing,
C			University of Bremen, Germany
c
c    Calculation of the particular solution corresponding to the 
c     rotational Raman scattering source function in atmosphere.
c
c      This routine in contrast to "part_sol_rrs.f" utilizes the
c       simplification that rotational Raman scattering source
c       function can be approximated as a constant within the 
c                      layer [tau_{l-1},tau+l] 
c             (following the assumption of A Kyling et al)
c                    
c *********************************************************************
      SUBROUTINE PART_SOL_RRS_SIMPL(JPOS, Mh, NN, NSZA, NL, DTR, ALW, 
     $     Norm_lay, Psim_lay, Psip_lay, MuS, PART_U, PART_D, SDO, SRR,
     $     SRR_Jact)
c     ... Call in "solv_dom.f"

      USE GRIDS,       ONLY: ZZZ
      USE GT_POLYN,    ONLY: AG
      USE INTERN_DIMS, ONLY: NL_pla, Nf_pla, Nvs, Ng1_brf, Ns_brf, 
     $                       NN_pla, NNv
      USE IRRAD,       ONLY: Fbe, IRRAD_FLAG, SOL_FLUX, WL_IRRAD_DIM
      USE LOG_PARS,    ONLY: LUNIT, LOG_LEVEL, N_DSO, N_RSO, N_ESO
      USE RAMAN,       ONLY: N_WL_C, Ram_Egs, WL_FACT, Sigma_RR, 
     $                       LINE_GAS
      USE ST_VARS,     ONLY: Pie, APPR_RRS, Vec_Sca, Kvs
      USE TYPE_VARS,   ONLY: KND
      USE TYPES,       ONLY: SDO_TYPE, RRS_TYPE
      USE WATER,       ONLY: INCL_INEL, CHL_UW, ABS_COEFF_UW, 
     $                       Fua_c, Hua_c
c     ------------------------------------------------------------------

c     ------------------------------------------
      TYPE (SDO_type) :: SDO    ! DOM auxiliary solution variables
      TYPE (RRS_type) :: SRR    ! Structure for rotational Raman
c     ----------------------
      INTEGER,    INTENT(IN):: JPOS, Mh, NN, NSZA, NL, SRR_Jact
      REAL (KND), INTENT(IN):: MuS(NSZA), DTR(NL), ALW(NL-1),
     $     Norm_lay(NNv,NL-1), Psim_lay(NNv,NNv,NL-1), 
     $     Psip_lay(NNv,NNv,NL-1)
      TARGET :: Psim_lay, Psip_lay
      REAL (KND) PART_U(NL,2*NNv,Kvs*NSZA), PART_D(NL,2*NNv,Kvs*NSZA)
c     -----------------------------------------------------
c     ... Local variables 
c     -------------------
      CHARACTER (LEN=18), PARAMETER :: R_NAME = 'PART_SOL_RRS_SIMPL'
      INTEGER  I, J, K, Ia, II, Iex, Js, K1, K2, LL1, Lr1, Iau,  
     $     Isa, Ni2, Ri, Nr_max, Nr_s
      REAL (KND) Mu0, Nuk, E_Nk, DTA, Cmk, Smk, Rel, M_cof,
     $     Nrk, S_Ram, WL_ACT, S1, S2
      REAL (KND), DIMENSION (NNv) :: E_Nj, Nuj, 
     $     akj, bkj, SBb_kj, SAa_kj, Kpa, Kpb
      REAL (KND), DIMENSION (NNv,NNv) :: Faj, Fbj 
      REAL (KND) Cmj(NNv,NSZA), Smj(NNv,NSZA), Em0(NSZA), UU(NNv,NSZA),
     $     VV(NNv,NSZA), akd(NNv,NSZA), bkd(NNv,NSZA), 
     $     Qp(NNv,NSZA), Qm(NNv,NSZA)
      INTEGER,    ALLOCATABLE :: Nsa(:), Rin_Lin(:,:)
      REAL (KND), ALLOCATABLE :: Phot_Wat(:), R_cof(:,:),
     $     S_Qp(:,:), S_Qm(:,:) 
      TARGET :: Faj, Fbj
      REAL (KND), POINTER :: Psip(:,:), Psim(:,:), Ex_sol(:),  
     $     ajs(:,:), bjs(:,:), Apj(:,:), Bpj(:,:)
c     ==================================================================

      CALL ST_TRACK (R_NAME, 1, 'I')
      IF(LOG_LEVEL(1) >= N_ESO) WRITE(LUNIT,'(12x,A)') '  *** '//R_NAME

c       WRITE(*,*) '   Harmonic Mh       = ',Mh
c       WRITE(*,*) '   Actual wavelength = ',SRR%WL_EXT(JPOS)

       DO I = 1, NL-1
          IF( SDO%NcC(I) > 0 ) THEN
             WRITE(*,*) '   Number of layers ', NL
             WRITE(*,*) '   The complex eigenvalue for '
             WRITE(*,*) I, SDO%NcC(I)
             WRITE(*,*) SDO%NcC(:)
             STOP
          ENDIF
       ENDDO

c     ... It will be extracted the actual wavelength value
      WL_Act = SRR%WL_EXT(JPOS)

c     ... Multiplier in Raman scattering coefficient which is 
c         constant for a given actual wavelength
      M_cof = WL_FACT/WL_ACT**4

      SELECT CASE( TRIM(APPR_RRS) )
      CASE('exact matching')
         Ni2 = N_WL_C
         ALLOCATE( Phot_wat(Ni2), Nsa(Ni2), R_cof(N_WL_C,NL-1) ) 

         DO J = 1,Ni2
            Nsa(J) = J
            DO I = 1,NL-1
c              ... Raman scattering coefficient
               R_cof(J,I) = (Sigma_RR(J,I) + Sigma_RR(J,I+1))/ALW(I)
            ENDDO
            R_cof(J,:) = R_cof(J,:)*SRR%GAM_WL(LINE_GAS(J),J)*M_cof/2.d0
         ENDDO

         Phot_Wat(:) = 1.d0
         IF( IRRAD_FLAG ) THEN
c           ... SRR_Jact is the actual wavelength number
            K1 = (SRR_Jact-1)*(N_WL_C+1)
            Phot_Wat(:) = SOL_FLUX(K1+2:K1+N_WL_C+1)/SOL_FLUX(JPOS)

            IF( TRIM(WL_IRRAD_DIM(2)) == 'phot/s') Phot_Wat(:) = 
     $           Phot_Wat(:)*WL_ACT/SRR%WL_EXT(K1+2:K1+N_WL_C+1)
         ENDIF
         
      CASE('simple binning')

c        ... Nr_max is maximal number of excitation WL within any bin
         Nr_max = MAXVAL(SRR%N_EtoA(:,:))
         ALLOCATE( Nsa(N_WL_C), Rin_Lin(N_WL_C,Nr_max) ) ; Rin_Lin = 0 

c        ... Current actual wavelength number
         Ia = SRR_Jact

c         WRITE(*,*) 'Current actual wavelength number',
c     $        Ia, SUM(SRR%N_EtoA(:,Ia))

         J = 0
c        ... Do-loop over excitation wavelengths of extended WL-grid 
c            where RT calculations have been performed and saved
c            (where SRR%NUM_IN(:) /= 0). For a current emission 
c            wavelength contribute only part of these wavelengths.
c            
         DO I = 1, SRR%N_sav
c           ... Position of this wavelength within full extended WL-grid
            Iex = SRR%Npo_sav(I)

c           ... The discrete-ordinates results can be saved for the 
c               atmospheric part at Iex-th bin even if NUM_IN(Iex)=0. 
c               This takes place if this wavelength is excitation 
c               WL for RRS and VRS simultaneously. This bin should be 
c               skipped because it does not contain excitation Raman lines.
            IF( SRR%NUM_IN(Iex)==0 ) CYCLE

c           ... If N_EtoA(Iex,Ia)==0 then there is no contribution from 
c               Iex-th bin to the current Ia-th actual WL, i.e., 
c               the bin does not contain excitation WLs for this actual WL
            IF( SRR%N_EtoA(Iex,Ia)==0 ) CYCLE            

c           ... J counts bins which contribute to this actual WL and Nsa(J)
c               saves position where D-O results were saved
            J = J + 1  ;  Nsa(J) = I  ;  II = 0
c           ... Over full number of excitation wavelength within this bin
            DO K = 1, SRR%NUM_IN(Iex)
c              ... Select lines contributing to the current actual wl only
               IF( SRR%ACT_IN(Iex,K)==Ia ) THEN
                  II = II + 1
c                 ... Count number of Raman lines
                  Rin_Lin(J,II) = SRR%RIN_IN(Iex,K)
                  IF( Rin_Lin(J,II) == 0 ) THEN
                     WRITE(*,'(10(1x,I5))') I, SRR%Npo_sav(I), J, II 
                     STOP
                  ENDIF
               ENDIF
            ENDDO
         ENDDO
         
         Ni2 = J

c         WRITE(*,*) '         Number of saved wavelengths : ', SRR%N_sav
c         WRITE(*,*) '   and found where N_EtoA(Iex,Ia)/=0 : ', Ni2

         ALLOCATE( Phot_wat(Ni2),R_cof(Ni2,NL-1) ) 
         Phot_Wat(:) = 1.d0
         
c        ... Over sequence of active bins
         DO J = 1,Ni2
c           ... Position of this bin within full extended WL-grid
            Iex = SRR%Npo_sav(Nsa(J))
c           ... Amount of excitation lines within this bin
            Nr_s = SRR%N_EtoA(Iex,SRR_Jact)

            DO I = 1,NL-1
               S1 = 0.d0
c              ... Sum contribution of all Raman lines
               DO II = 1,Nr_s
                  Ri = Rin_Lin(J,II)
                  S1 = S1 + 
     $                 M_cof*(Sigma_RR(Ri,I)+Sigma_RR(Ri,I+1))/2.d0
     $                 *SRR%GAM_WL(LINE_GAS(Ri),Nsa(J))/ALW(I)
               ENDDO
               R_cof(J,I) = S1
            ENDDO
         ENDDO

         IF( IRRAD_FLAG ) THEN
            DO I = 1,Ni2
               K1 = SRR%Npo_sav(Nsa(I))
               Phot_Wat(I) = SOL_FLUX(K1)/SOL_FLUX(JPOS)
            ENDDO
            IF( TRIM(WL_IRRAD_DIM(2)) == 'phot/s') THEN
               DO I = 1,Ni2
                  Iex = SRR%Npo_sav(Nsa(I))
                  Phot_Wat(I) = Phot_Wat(I)*WL_ACT/SRR%WL_EXT(Iex)
               ENDDO
            ENDIF
         ENDIF
         
      END SELECT

      ALLOCATE( S_Qp(NNv,NSZA), S_Qm(NNv,NSZA)) ! A

c     ... Coefficients for the inelastically scattered 
c         direct solar light at an excitation wavelength
      SELECT CASE( Vec_Sca )
      CASE( 'scal_rte' )
         S_Qp(:,:) = Ram_Egs(:,:,1,Mh) 
         S_Qm(:,:) = Ram_Egs(:,:,2,Mh)
      CASE( 'vect_rte' )
c        ... In the vector case F14=1/4 shoud be replaced by 
c            2*F14=1/2, therefore, Ram_Egs(:,:,1:2,Mh) should be 
c            multiplied by 2.
         S_Qp(:,:) = 2.d0*Ram_Egs(:,:,1,Mh) 
         S_Qm(:,:) = 2.d0*Ram_Egs(:,:,2,Mh)
      END SELECT
c
c     --------------------- Main block ---------------------------
c
c      WRITE(*,*) '    Start integration over layer'
      DO I = 1,NL - 1           ! Over all layers in atmosphere
         IF( SDO%MASK_NL(I) ) 
     $        STOP "  RRS can't be calculated for pure scattering "

         Psim => Psim_lay(:,:,I) ; Psip => Psip_lay(:,:,I)
         LL1 = (I-1)*2*NNv + 1 ; LR1 = (I-1)*2*NNv + NNv 
         DTA = DTR(I+1)
         UU = 0.d0  ;  VV = 0.d0

c     ... Summation over excitation wavelengths
         DO Iau = 1,Ni2
            Isa = Nsa(Iau)
            S1 = R_cof(Iau,I)

c           ... Sum contribution of different phase function            
            Qp (:,:) = S1*S_Qp (:,:) ; Qm (:,:) = S1*S_Qm (:,:)

            Faj(:,:) = S1*SRR%GNp(Isa,Mh,I,:,:)
            Fbj(:,:) = S1*SRR%GNm(Isa,Mh,I,:,:)

c           ... Solar exponent at excitation wavelength
            Ex_sol => SRR%EXS_AUX(Isa,I,:)

c           ... Coefficients a'j and b'j of the particular solution
c               of the direct solar exponent (see Eq.(a'j) and (b'j) 
C               in STB) at the excitation WL. They include multipliers
c               EXP(-Tau_(l-1)/Mu0) calculated at these wavelengths.
c               First dimension of arrays ajs and bjs is 1:NNv
            ajs => SRR%Aj(Isa,Mh,:,I,:)
            bjs => SRR%Bj(Isa,Mh,:,I,:)

c           ... Integration constants Aj and Bj for excitation WL
c                              separation const, SZA
            Apj => SRR%MVEC(Isa,Mh,  LL1:LR1    ,:)
            Bpj => SRR%MVEC(Isa,Mh,LR1+1:LR1+NNv,:)

            S_Ram = Phot_Wat(Iau)

c           ... Relation of extinction coefficients actual/excitation
            Rel = ALW(I)/SRR%EXT_AUX(Isa,I)

c           ... Separation constants for a excitation wavelength
            Nuj(:) = SRR%Nuj_EXC(Isa,Mh,:,I)*Rel

            E_Nj(:) = DEXP( -DTA/Nuj(:) )
            Em0(:)  = DEXP( -DTA/( Rel*Mus(:)) )

            DO Js = 1,NSZA
               Mu0 = Rel*Mus(Js) ! SZA in water
               WHERE( REAL(Nuj(:)) /= REAL(Mu0) )
                  Cmj(:,Js) = Nuj(:)*Mu0/(Nuj(:)-Mu0)*(E_Nj(:)-Em0(Js)) 
               ELSEWHERE
                  Cmj(:,Js) = E_Nj(:)*DTA 
               END WHERE            
               Smj(:,Js)=Nuj(:)*Mu0/(Nuj(:)+Mu0)*(1.d0-E_Nj(:)*Em0(Js)) 
c              ... Contribution of the single scattering emission source 
c                  function
               DO K = 1,NNv
                  akd(K,Js) = SUM(Psip(:,K)*Qp(:,Js)+Psim(:,K)*Qm(:,Js))
                  bkd(K,Js) = SUM(Psim(:,K)*Qp(:,Js)+Psip(:,K)*Qm(:,Js))
               ENDDO
               akd(:,Js) = akd(:,Js)*0.5/Norm_lay(:,I) 
               bkd(:,Js) = bkd(:,Js)*0.5/Norm_lay(:,I)
            ENDDO

            DO K = 1,NNv        ! over separation constant of actual Wl
c              ... Separation const for actual WL
               Nuk  = SDO%Nuj_lay(K,I)  ;  E_Nk = DEXP( -DTA/Nuk )
               Nrk = 0.5d0/Norm_lay(K,I)

c              ... These coefficients are calculated according to
c                  Eqs.(Saa_kj) and (Sba_kj) in STB
               DO J = 1,NNv
                  akj(J) = SUM(Psip(:,K)*Faj(:,J) + Psim(:,K)*Fbj(:,J))
                  bkj(J) = SUM(Psim(:,K)*Faj(:,J) + Psip(:,K)*Fbj(:,J))
               ENDDO

               DO Js = 1,NSZA
                  Mu0 = Rel*Mus(Js) ! SZA in water
                  IF( REAL(Mu0)/= REAL(Nuk) ) THEN  
                     Cmk = Mu0*Nuk/(Mu0 - Nuk)*(Em0(Js) - E_Nk)
                  ELSE
                     Cmk = Em0(Js)*DTA
                  ENDIF
                  Smk = Mu0*Nuk/(Mu0 + Nuk)*(1.d0 - Em0(Js)*E_Nk)

c                 ------------------------------------------------------
c                 ... K'aj as mean value at the top and bottom:
                  Kpa(:) = ( Apj(:,Js) + Apj(:,Js)*E_Nj(:)
     $                  + ajs(:,Js)*Cmj(:,Js) /Rel )/2.d0
c                 ... K'bj as mean value at the top and bottom:
                  Kpb(:) = ( Bpj(:,Js) + Bpj(:,Js)*E_Nj(:) 
     $                 + bjs(:,Js)*Smj(:,Js) /Rel )/2.d0

c                               arj*K'aj        bkj*K'bj
                  SAa_kj(:) = akj(:)*Kpa(:) + bkj(:)*Kpb(:)
c                               brj*K'aj        akj*K'bj
                  SBb_kj(:) = bkj(:)*Kpa(:) + akj(:)*Kpb(:)

c                 ... Perform summation over all excitation wavelengths
c                 ... Coefficient at the bottom of the layer
                  UU(K,Js) = UU(K,Js) + S_Ram*SUM( SBb_kj(:) )*Nuk*
     $                 (1.d0 - E_Nk)*Nrk
     $                 + S_Ram*bkd(K,Js)*Smk*Ex_sol(Js)/4.d0
c                 ... Coefficient at the bottom of the layer
                  VV(K,Js) = VV(K,Js) + S_Ram*SUM( SAa_kj(:) )*Nuk*
     $                 (1.d0 - E_Nk)*Nrk
     $                 + S_Ram*akd(K,Js)*Cmk*Ex_sol(Js)/4.d0
               ENDDO  ! over solar angles (Js)
            ENDDO  ! over separation constant of actual Wl (K)
         ENDDO  ! over excitation wavelengths (Iau, Isa)

c        ... Final results for particular solutions
c            The matrix G_NL has the following structure
c             | Phi+ Phi- |  
c             | Phi- Phi+ |
c            Phi+ Phi- vectors are given according to Eqs.(G-NL) in STB
c            and calculated in "homog_prtic.f"
         DO Js = 1,NSZA
            Part_U(I,:,Js) = Part_U(I,:,Js)
     $           + MATMUL( SDO%G_NL(I,:,NNv+1:2*NNv),UU(:,Js) )
            Part_D(I,:,Js) = Part_D(I,:,Js)
     $           + MATMUL( SDO%G_NL(I,:,1:NNv),VV(:,Js) )
         ENDDO

      ENDDO                     ! over altitude layers

      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN

      END 
c **********************************************************************
