!***************************************************************************
!
!	SCIATRAN - Version 4.2 - Last modification: 26 Jun 2020
!
!	Copyright (C) - Institute of Remote Sensing,
!			University of Bremen, Germany
!
!***************************************************************************
SUBROUTINE PHM_ALONG_LS(JPOS, Mh, IMS1, IMS2, XM_1, RAY_MOMS, AER_MOMS, &
     CLD_MOMS, SCRL, EMCL, H36, SCAL, DM_LS, SCCL, Phi, FL, IOB, S_F, SRR, S_RR)
  ! Call in 'sfi_spher.f'

  !!! This routine works in the vector case only !!!
  ! ---------------  Parameters description  ------------------------
  ! Input:
  ! ======
  ! Mh           - Harmonic number
  ! IMS1         - First point along line-of-sight to be considered (LS)
  ! IMS2         - Last point along line-of-sight to be considered (LS)
  ! XM_1(:  )    - Here is the cosine between the direction of radiation 
  !                propagation and tau-axis !!!
  ! RAY_MOMS(Nlg_Ray,Nm_vs) 
  !              - Expansion coefficients of the Rayleigh phase matrix
  ! AER_MOMS(Nlg_Aer,NL_Aer,Nm_vs)
  !              - Expansion coefficients of the aerosol phase matrix
  ! CLD_MOMS(Nlg_Cld,NL_Cld,Nphr,Nm_ice,Nm_vs) 
  !              - Expansion coefficients of the cloud phase matrix
  ! SCRL(:)      - Rayleigh scattering coefficient along LS
  ! SCAL(:)      - Aerosol scattering coefficient along LS
  ! SCCL(:,:)      - Cloud scattering coefficient along LS
  ! Phi(:)       - Azimuthal angle along LS
  ! FL(:,:)      - Interpolated radiation field along LS
  ! Output:
  ! =======
  ! S_F(:,:)     - Multiple-scattering source function along LS
  ! -----------------------------------------------------------------------
  USE AEROSOLS,    ONLY: IDX_AER
  USE CLOUDS,      ONLY: DO_CLOUD, CLA => CLD_LAYER_ARR
  USE CLOUD_INPUT, ONLY: Nphr
  USE EMISSION,    ONLY: N_AT
  USE GT_POLYN,    ONLY: Leg => NLEGEN, AG, AG1, Plm_G, Dp2_G, Dm2_G, SGN
  USE INFO,        ONLY: TG_INFO
  USE INTERN_DIMS, ONLY: Nlg_Ray, Nm_vs, Nvs, NNv, NL_AER, NLG_AER, NL_CLD,&
                         NLG_CLD, Nm_ice, NN_MS => N_NODE_MS
  USE RAMAN,       ONLY: RRS_ADJ
  USE ST_VARS,     ONLY: IDM, LDM, DO_AEROSOL, Pie, Emi_FLG, NN => NSTRMS
  USE TYPES,       ONLY: KND, RRS_TYPE

  INTEGER      JPOS, Mh, IMS1, IMS2, IOB(:)
  REAL (KND), DIMENSION (:) :: XM_1, SCRL, Phi
  REAL (KND)  RAY_MOMS(Nlg_Ray,Nm_vs), AER_MOMS(Nlg_Aer,NL_Aer,Nm_vs),&
       CLD_MOMS(Nlg_Cld,NL_Cld,Nphr,Nm_ice,Nm_vs)
  REAL (KND), ALLOCATABLE :: EMCL(:,:), H36(:,:,:), S_RR(:,:), SCAL(:), &
       DM_LS(:), SCCL(:,:) 
  REAL (KND)  FL(:,:), S_F(:,:)
  TYPE (RRS_type) :: SRR 
  ! Local parameters
  CHARACTER (LEN=*), PARAMETER :: R_NAME = 'PHM_ALONG_LS'
  INTEGER     I, J, K, L, K1, K2, KK, Ka, Jg, SIGN_M, INC, Ks, Is, Is1, &
       Jls, I_OUT, Jex, Nls 
  REAL (KND), DIMENSION (Leg,IMS1:IMS2) :: Plm_L, Dp2_L, Dm2_L 
  REAL (KND)  X_MOMS(Leg,Nm_vs), Vec(4), V1(4), V2(4), V3(4), Sg(NN,4), &
       Sp(4), Ph_W
  !==================================================================
  CALL ST_TRACK (R_NAME, 1, 'I')

  Nls = IMS2 - IMS1 + 1
  ! Calculation of Legendre polynomial along LS
  CALL WIGNER_PLG (XM_1(IMS1:IMS2), Nls, Mh-1, Leg, Plm_L)
  
  ! Calculation of  Wigner functions along LS
  CALL WIGNER_FUNCTION( XM_1(IMS1:IMS2), Nls, Mh-1, Leg, Dp2_L, Dm2_L )
  SIGN_M = 1 - 2*MODULO(Mh-1,2)
  Dp2_L(:,:) = 0.5d0*( Dp2_L(:,:) + Dm2_L(:,:) )
  Dm2_L(:,:) = Dp2_L(:,:) - Dm2_L(:,:)
  Dp2_L(:,:) = SIGN_M*Dp2_L(:,:) ! Rlm(mu)
  Dm2_L(:,:) = SIGN_M*Dm2_L(:,:) ! Tlm(mu)
         
  ! Includes RRS 
  IF( RRS_ADJ ) THEN
     ! Source function is calculated without multiplication
     ! by Raman scattering coefficient
     DO Jls = IMS1, IMS2
        CALL MAKE_SF(Nlg_Ray, SRR%RAM_MOMS, S_RR(Jls,:) )               
     ENDDO  ! over line-of-sight (Jls)
  ENDIF  ! RRS_ADJ

  ! Over all points along L-of-S
  DO Jls = IMS1, IMS2
     X_MOMS = 0.d0
     X_MOMS(1:Nlg_Ray,:) = SCRL(Jls)*RAY_MOMS(1:Nlg_Ray,:)
     IF( Emi_FLG ) THEN
        DO Ka = 1, N_AT
           X_MOMS(1:Nlg_Ray,:) = X_MOMS(1:Nlg_Ray,:) &
                + EMCL(Ka,Jls)*H36(Ka,1:Nlg_Ray,:)
        ENDDO
     ENDIF

!    ... SCAL does not contain delta-m factor
     IF( DO_AEROSOL ) X_MOMS(1:Leg,:) = X_MOMS(1:Leg,:) &
          + SCAL(Jls)*AER_MOMS(1:Leg,IDX_AER(IOB(Jls)),:) &
          * (1.d0 - DM_LS(Jls))

     IF( DO_CLOUD ) THEN
        INC = INT( CLA(IOB(Jls),2) )
        IF( INC > 0 ) THEN
           DO J = 1, Nm_ice
              X_MOMS(1:Leg,:) = X_MOMS(1:Leg,:) &
                   + SCCL(Jls,J)*CLD_MOMS(1:Leg,INC,1,J,:)
           ENDDO
        END IF
     ENDIF
     CALL MAKE_SF(Leg, X_MOMS, S_F(Jls,:) )
  ENDDO  ! over angles along LS (Jls)

  CALL ST_TRACK (R_NAME, 1, 'O')
  RETURN

CONTAINS
!**********************************************************************
!   Calculate source function along line-of-sight
!**********************************************************************
  SUBROUTINE MAKE_SF(N_LEG, MOMS, SF )
    INTEGER  N_LEG
    REAL (KND) MOMS(N_LEG,Nm_vs), SF(Nvs)
    ! Contribution of the downward radiation in the source function.
    ! The elements of array FL(:,L) 
    ! 1:NNv - contain the downward radiation
    DO K1 = 1,NN
       Ks = Nvs*(K1-1)
       Vec(1:4) = FL(Ks+1:Ks+4,Jls) 
       Sg(K1,1:4) = 0.d0
       DO Is = Mh,N_Leg
          ! V1 -> Plm(ga)*I(ga)
          V1(1) = Plm_G(Mh,Is,K1)*Vec(1)
          V1(2) = Dp2_G(Mh,Is,K1)*Vec(2) - Dm2_G(Mh,Is,K1)*Vec(3)
          V1(3) = Dp2_G(Mh,Is,K1)*Vec(3) - Dm2_G(Mh,Is,K1)*Vec(2)
          V1(4) = Plm_G(Mh,Is,K1)*Vec(4)
          ! V2 -> Bl*Plm(ga)*I(ga)
          V2(1) = MOMS(Is,1)*V1(1) + MOMS(Is,5)*V1(2)
          V2(2) = MOMS(Is,5)*V1(1) + MOMS(Is,2)*V1(2)
          V2(3) = MOMS(Is,3)*V1(3) + MOMS(Is,6)*V1(4)
          V2(4) = MOMS(Is,4)*V1(4) - MOMS(Is,6)*V1(3)
          ! V3 -> Plm(ls)*Bl*Plm(ga)*I(ga)
          V3(1) = Plm_L(Is,Jls)*V2(1)
          V3(2) = Dp2_L(Is,Jls)*V2(2) - Dm2_L(Is,Jls)*V2(3)
          V3(3) = Dp2_L(Is,Jls)*V2(3) - Dm2_L(Is,Jls)*V2(2)
          V3(4) = Plm_L(Is,Jls)*V2(4)
          ! Sum over moments
          Sg(K1,1:4) = Sg(K1,1:4) + V3(1:4)
       ENDDO
    ENDDO                  !End over Gauss angles
    ! Sum over Gauss angles (lower hemisphere)
    DO I = 1,4
       Sp(I) = SUM( Sg(1:NN,I)*AG1(1:NN) )
    ENDDO
    ! Contribution of upward radiation in the source function
    ! The elements  of array FL(:,L)
    ! NNv+1:2*NNv - contain the upward radiation
    K1 = 1
    DO K2 = NN+1,2*NN
       Ks = Nvs*(K2-1)
       Vec(1:4) = FL(Ks+1:Ks+4,Jls) 
       Sg(K1,1:4) = 0.d0 ; Is1 = 1
       DO Is = Mh,N_Leg
          ! V1 -> Plm(ga)*I(ga)
          V1(1) = Plm_G(Mh,Is,K1)*Vec(1)
          V1(2) = Dp2_G(Mh,Is,K1)*Vec(2) - Dm2_G(Mh,Is,K1)*Vec(3)
          V1(3) = Dp2_G(Mh,Is,K1)*Vec(3) - Dm2_G(Mh,Is,K1)*Vec(2)
          V1(4) = Plm_G(Mh,Is,K1)*Vec(4)
          ! V1 -> D*Plm(ga)*I(ga) (D = diag[1,1,-1,-1])
          V1(3:4) = -V1(3:4)
          ! V2 -> Bl*D*Plm(ga)*I(ga)
          V2(1) = MOMS(Is,1)*V1(1) + MOMS(Is,5)*V1(2)
          V2(2) = MOMS(Is,5)*V1(1) + MOMS(Is,2)*V1(2)
          V2(3) = MOMS(Is,3)*V1(3) + MOMS(Is,6)*V1(4)
          V2(4) = MOMS(Is,4)*V1(4) - MOMS(Is,6)*V1(3)
          ! V3 -> Plm(ls)*Bl*D*Plm(ga)*I(ga)
          V3(1) = Plm_L(Is,Jls)*V2(1)
          V3(2) = Dp2_L(Is,Jls)*V2(2) - Dm2_L(Is,Jls)*V2(3)
          V3(3) = Dp2_L(Is,Jls)*V2(3) - Dm2_L(Is,Jls)*V2(2)
          V3(4) = Plm_L(Is,Jls)*V2(4)
          ! Sum over moments
          Sg(K1,1:4) = Sg(K1,1:4) + SGN(Is1)*V3(1:4)
          Is1 = Is1 + 1
       ENDDO
       K1 = K1 + 1
    ENDDO
    ! Sum over Gauss angles (lower + upper hemisphere )
    DO I = 1,4
       SF(I) = Sp(I) + SUM( Sg(1:NN,I)*AG1(1:NN) )
    ENDDO
    ! Fourier series coefficient 0.5 will be multiplied in
    ! 'gpp_converge.f' by 1 if Mh = 1 and by 2 if Mh > 1
    SF(1:2) = 0.5d0*SF(1:2)*DCOS( Phi(Jls)*DBLE(Mh-1) ) !*1.7d0
    SF(3:4) = 0.5d0*SF(3:4)*DSIN( Phi(Jls)*DBLE(Mh-1) )

!***************************************************************************
  END SUBROUTINE MAKE_SF
!***************************************************************************
END SUBROUTINE PHM_ALONG_LS
!***************************************************************************
