c ****************************************************************************
c                 This subroutine used as the basis 
c    REAL FUNCTION PLKAVG( WNUMLO, WNUMHI, T ) from standard DISORT package
c          adopted for FORTRAN 90 in the framework of SCIATRAN
c ****************************************************************************
      SUBROUTINE PLKAVG( WNUMLO, WNUMHI, T, RES, * )
c        Computes Planck function integrated between two wavenumbers
c
c  INPUT :  WNUMLO : Lower wavenumber (inv cm) of spectral interval
c
c           WNUMHI : Upper wavenumber
c
c           T      : Temperature (K)
c
c  OUTPUT : RES    : Integrated Planck function ( Watts/sq m )
c                      = Integral (WNUMLO to WNUMHI) of
c                        2h c**2  nu**3 / ( EXP(hc nu/kT) - 1)
c                        (where h=Plancks constant, c=speed of
c                         light, nu=wavenumber, T=temperature,
c                         and k = Boltzmann constant)
c
c  Reference : Specifications of the Physical World: New Value
c                 of the Fundamental Constants, Dimensions/N.B.S.,
c                 Jan. 1974
c
c  Method :  For WNUMLO close to WNUMHI, a Simpson-rule quadrature
c            is done to avoid ill-conditioning; otherwise
c
c            (1)  For WNUMLO or WNUMHI small,
c                 integral(0 to WNUMLO/HI) is calculated by expanding
c                 the integrand in a power series and integrating
c                 term by term;
c
c            (2)  Otherwise, integral(WNUMLO/HI to INFINITY) is
c                 calculated by expanding the denominator of the
c                 integrand in powers of the exponential and
c                 integrating term by term.
c
c  Accuracy :  At least 6 significant digits, assuming the
c              physical constants are infinitely accurate
c
c  ERRORS WHICH ARE NOT TRAPPED:
c
c      * power or exponential series may underflow, giving no
c        significant digits.  This may or may not be of concern,
c        depending on the application.
c
c      * Simpson-rule special case is skipped when denominator of
c        integrand will cause overflow.  In that case the normal
c        procedure is used, which may be inaccurate if the
c        wavenumber limits (WNUMLO, WNUMHI) are close together.
c
c  LOCAL VARIABLES
c
c        A1,2,... :  Power series coefficients
c        C2       :  h * c / k, in units cm*K (h = Plancks constant,
c                      c = speed of light, k = Boltzmann constant)
c        D(I)     :  Exponential series expansion of integral of
c                       Planck function from WNUMLO (i=1) or WNUMHI
c                       (i=2) to infinity
c        EPSIL    :  SMALLEST NUMBER SUCH THAT 1+EPSIL .GT. 1 on
c                       computer
c        EX       :  EXP( - V(I) )
c        EXM      :  EX**M
c        MMAX     :  No. of terms to take in exponential series
c        MV       :  Multiples of V(I)
c        P(I)     :  Power series expansion of integral of
c                       Planck function from zero to WNUMLO (I=1) or
c                       WNUMHI (I=2)
c        PI       :  3.14159...
c        SIGMA    :  Stefan-Boltzmann constant (W/m**2/K**4)
c        SIGDPI   :  SIGMA / PI
c        SMALLV   :  Number of times the power series is used (0,1,2)
c        V(I)     :  C2 * (WNUMLO(I=1) or WNUMHI(I=2)) / temperature
c        VCUT     :  Power-series cutoff point
c        VCP      :  Exponential series cutoff points
c        VMAX     :  Largest allowable argument of EXP function
c
c ----------------------------------------------------------------------
      USE ST_VARS,    ONLY: PI => PIE
      USE THERMAL_VAR,      ONLY: C2 => PL_C2, VCUT, SIGDPI, VCP, 
     $                            CONC => PL_CONC, VMAX, EPSIL

c     .. Parameters ..
      DOUBLE PRECISION    A1, A2, A3, A4, A5, A6
      PARAMETER          ( A1 = 1.0/ 3.0, A2 = -1.0/8.0, A3 = 1.0/60.0,
     $                     A4 = -1.0/5040.0, A5 = 1.0/ 272160.0,
     $                     A6 = -1.0/13305600.0 )
c     .. Routine name ..
      CHARACTER (Len=*), PARAMETER :: ROUTINE_NAME = 'PLKAVG'

c     .. Scalar Arguments ..
      DOUBLE PRECISION  T, WNUMHI, WNUMLO, RES

c     .. Local Scalars ..
      INTEGER           I, K, M, MMAX, N, SMALLV
      DOUBLE PRECISION  DEL, EX, EXM, HH, MV, OLDVAL, VAL, VAL0, VSQ, X

c     .. Local Arrays ..
      DOUBLE PRECISION  D( 2 ), P( 2 ), V( 2 )

c     .. Statement Functions ..
      DOUBLE PRECISION      PLKF

c     .. Statement Function definitions ..
      PLKF( X ) = X**3 / ( DEXP( X ) - 1 )

      IF( T < 0.0 .OR. WNUMHI.LE.WNUMLO .OR. WNUMLO < 0. ) THEN
         CALL WRITE_MESSAGE(ROUTINE_NAME, 'ERROR', 
     $        'Temperature or wavenums. wrong')
         RETURN 1
      ENDIF

      IF( T < 1.D-4 ) THEN
         RES = 0.0 ; RETURN
      END IF

      V( 1 ) = C2*WNUMLO/T ; V( 2 ) = C2*WNUMHI/T

      IF( V( 1 ) > EPSIL .AND. V( 2 ) < VMAX .AND.
     $     ( WNUMHI - WNUMLO ) / WNUMHI < 1.E-2 ) THEN
c        ** Wavenumbers are very close.  Get integral
c        ** by iterating Simpson rule to convergence.
         HH     = V( 2 ) - V( 1 )
         OLDVAL = 0.0
         VAL0   = PLKF( V( 1 ) ) + PLKF( V( 2 ) )
         DO 20 N = 1, 10
            DEL  = HH / ( 2*N )
            VAL  = VAL0
            DO 10 K = 1, 2*N - 1
               VAL  = VAL + 2*( 1 + MOD( K,2 ) )*PLKF( V( 1 ) + K*DEL )
 10         CONTINUE
            VAL  = DEL / 3.*VAL
            IF( DABS( ( VAL - OLDVAL ) / VAL ).LE.1.E-6 ) GO TO  30
            OLDVAL = VAL
 20      CONTINUE
         CALL WRITE_MESSAGE(ROUTINE_NAME, 'WARNING', 
     $        'PLKAVG--Simpson rule didnt converge')
   30    CONTINUE
         RES = SIGDPI * T**4 * CONC * VAL
         RETURN
      END IF
c                          *** General case ***
      SMALLV = 0
      DO 60 I = 1, 2
         IF( V( I ).LT.VCUT ) THEN
c           ** Use power series
            SMALLV = SMALLV + 1
            VSQ    = V( I )**2
            P( I ) = CONC*VSQ*V( I )*( A1 +
     $           V( I )*( A2 + V( I )*( A3 + VSQ*( A4 + VSQ*( A5 +
     $           VSQ*A6 ) ) ) ) )
         ELSE
c           ** Use exponential series
            MMAX  = 0
c           ** Find upper limit of series
   40       CONTINUE
            MMAX  = MMAX + 1
            IF( V(I) < VCP( MMAX ) ) GO TO  40
            EX     = DEXP( - V(I) )
            EXM    = 1.0
            D( I ) = 0.0
            DO 50 M = 1, MMAX
               MV     = M*V( I )
               EXM    = EX*EXM
               D( I ) = D( I ) + EXM*( 6.+ MV*( 6.+ MV*( 3.+ MV ) ) )
     &                  / M**4
   50       CONTINUE
            D( I ) = CONC*D( I )
         END IF
   60 CONTINUE
c     ** Handle ill-conditioning
      IF( SMALLV.EQ.2 ) THEN
c        ** WNUMLO and WNUMHI both small
         RES = P( 2 ) - P( 1 )
      ELSE IF( SMALLV.EQ.1 ) THEN
c        ** WNUMLO small, WNUMHI large
         RES = 1.- P( 1 ) - D( 2 )
      ELSE
c        ** WNUMLO and WNUMHI both large
         RES = D( 1 ) - D( 2 )
      END IF
      RES = SIGDPI * T**4 * RES

      IF( RES == 0.d0 )
     &     CALL WRITE_MESSAGE(ROUTINE_NAME, 'WARNING',
     $     'PLKAVG--returns zero; possible underflow')

      RETURN
      END SUBROUTINE PLKAVG
