!      All modules of PRO4SAIL original version 
MODULE MOD_ANGLE
  REAL*8,SAVE :: pi,rd
END MODULE MOD_ANGLE
!*************************************************************************************
MODULE MOD_staticvar
  REAL*8,SAVE :: cts,cto,ctscto
  REAL*8,SAVE :: ddb,ddf,dob,dof,dso
  REAL*8,SAVE :: ko,ks,sdb,sdf
  REAL*8,SAVE :: sob,sof,sumint
  REAL*8,ALLOCATABLE,SAVE :: sb(:),sf(:),vb(:),vf(:),w(:)
  REAL*8,ALLOCATABLE,SAVE :: m(:),m2(:),att(:),sigb(:),rinf(:),lidf(:)
END MODULE MOD_staticvar
!**********************************************************************************
MODULE MOD_output_PROSPECT
  REAL*8,ALLOCATABLE,SAVE :: LRT(:,:),rho(:),tau(:)
END MODULE MOD_output_PROSPECT
!**********************************************************************************
MODULE MOD_flag_util
  LOGICAL flag(7),init_completed,init_completed0
  LOGICAL delta_geom,delta_lai,delta_hot,delta_leaf,delta_skyl,delta_soil,delta_lidf
  REAL*8,ALLOCATABLE,SAVE :: lidf_old(:),rsoil_old(:)
  REAL*8,SAVE :: N_old,Cab_old,Car_old,Cbrown_old,Cw_old,Cm_old
  REAL*8,SAVE :: LIDFa_old,LIDFb_old,lai_old,q_old,skyl_old
  REAL*8,SAVE :: tts_old,tto_old,psi_old
END MODULE MOD_flag_util
!***********************************************************************************
MODULE MOD_SAIL
  REAL*8,SAVE :: tss,too,tsstoo
  REAL*8,ALLOCATABLE,SAVE :: tso(:), tsd(:), tdd(:), tdo(:), rsd(:), rdd(:), rso(:), rdo(:)
  REAL*8,ALLOCATABLE,SAVE :: rsos(:), rsod(:), rddt(:), rsdt(:), rdot(:), rsodt(:), &
       rsost(:),rsot(:)
END MODULE MOD_SAIL
!***********************************************************************************
!***********************************************************************************
!***********************************************************************************

SUBROUTINE PROSAIL( N_inc, X_inc, N_ref, X_ref, N_Phi, X1, R_dir, R_dif, &
     *, Nder, dR_dir, dR_dif )
  ! 09 22 2011
  ! This program allows modeling reflectance data from canopy
  ! - modeling leaf optical properties with PROSPECT-5 (feret et al. 2008)
  ! - modeling leaf inclination distribution function with the subroutine campbell
  ! (Ellipsoidal distribution function caracterised by the average leaf 
  ! inclination angle in degree), or dladgen (2 parameters LIDF)
  ! - modeling canopy reflectance with 4SAIL (Verhoef et al., 2007)
	
  ! This version has been implemented by Jean-Baptiste Feret
  ! Jean-Baptiste Feret takes the entire responsibility for this version 
  ! All comments, changes or questions should be sent to:
  ! jbferet@stanford.edu
  
  ! References:
  ! 	Verhoef et al. (2007) Unified Optical-Thermal Four-Stream Radiative
  ! 	Transfer Theory for Homogeneous Vegetation Canopies, IEEE TRANSACTIONS 
  ! 	ON GEOSCIENCE AND REMOTE SENSING, VOL. 45, NO. 6, JUNE 2007
  ! 	Fret et al. (2008), PROSPECT-4 and 5: Advances in the Leaf Optical
  ! 	Properties Model Separating Photosynthetic Pigments, REMOTE SENSING OF 
  ! 	ENVIRONMENT
  ! The specific absorption coefficient corresponding to brown pigment is
  ! provided by Frederic Baret (EMMAH, INRA Avignon, baret@avignon.inra.fr)
  ! and used with his autorization.
  ! the model PRO4SAIL is based on a version provided by
  !	Wout Verhoef
  !	NLR	
  !	April/May 2003

  ! The original 2-parameter LIDF model is developed by and described in:
  ! 	W. Verhoef, 1998, "Theory of radiative transfer models applied in 
  !	optical remote sensing of vegetation canopies", Wageningen Agricultural
  !	University,	The Netherlands, 310 pp. (Ph. D. thesis)
  ! the Ellipsoidal LIDF is taken from:
  !   Campbell (1990), Derivtion of an angle density function for canopies 
  !   with ellipsoidal leaf angle distribution, Agricultural and Forest 
  !   Meteorology, 49 173-176
!*                                                                       *
!*************************************************************************

  USE INFO,      ONLY: RET_INFO
  USE LOG_PARS,  ONLY: EU
  USE PRO_SAIL,  ONLY: TypeLidf, NW_S, NW_E, LIDFa, LIDFb, LAI, hspot, Stc, &
                       Cab, Car, Cbrown, Cw, Cm, Psoil
  USE TYPE_VARS, ONLY: KND
  !  PRO4SAIL modules:
  USE MOD_ANGLE		   ! defines pi & rad conversion
  USE MOD_staticvar	   ! static variables kept in memory for optimiz.
  USE MOD_flag_util	   ! flags for optimization
  USE MOD_output_PROSPECT  ! output variables of PROSPECT
  USE MOD_SAIL		   ! variables of SAIL
  USE MOD_dataSpec_P5B		
  IMPLICIT NONE

  INTEGER,   INTENT(IN) :: N_inc, N_ref, N_phi, Nder
  REAL(KND), INTENT(IN) :: X_inc(N_inc), X_ref(N_ref), X1(N_Phi)
  REAL(KND)  R_dir(N_ref,N_inc,N_phi,NW_E-NW_S+1), & 
       R_dif (N_ref,N_inc,N_Phi,NW_E-NW_S+1)
  REAL(KND), ALLOCATABLE :: dR_dir(:,:,:,:,:), dR_dif(:,:,:,:,:)
  OPTIONAL :: Nder, dR_dir, dR_dif
  ! Structure coefficient - Stc in modules PRO_SAIL:
  REAL(KND)  N
  CHARACTER (LEN=7), PARAMETER :: R_NAME = 'PROSAIL'
  CHARACTER (LEN=10) PAR_NAME
  INTEGER    Ii, Ip, Ir, Iw, K, Nwa, Ik, Ico, I3
  LOGICAL    Dwr_Pso
  REAL(KND) :: tto, tts, psi, D_pert, Del_X
  REAL(KND) :: P0(9), Pp(9)
  REAL(KND), ALLOCATABLE :: inc_arr(:)
  REAL(KND), ALLOCATABLE, SAVE :: rsoil0(:)
! ========================================================================

  CALL ST_TRACK (R_NAME, 1, 'I')

  ! Number of actual wavelengths:
  Nwa = NW_E - NW_S + 1

  ! Save input parameters
  P0(1) = Cbrown ; P0(2) = Cm ; P0(3) = Cab   ; P0(4) = LAI ; P0(5) = Stc
  P0(6) = Car    ; P0(7) = Cw ; P0(8) = LIDFa ; P0(9) = Psoil

  IF( PRESENT(Nder) ) THEN
     ALLOCATE( dR_dir(N_ref,N_inc,N_phi,Nwa,Nder), &
       dR_dif(N_ref,N_inc,N_phi,Nwa,Nder) )
  ENDIF

  Nw = Nwa
  ! Perturbation in [%] 
  D_pert = 0.1 

 ! ANGLE CONVERSION
 pi=ATAN(1.)*4. ;  rd=pi/180.
 
 ! PROSPECT output
 ALLOCATE (LRT(nw,2),rho(nw),tau(nw), sb(nw), sf(nw), vb(nw), vf(nw), w(nw))
 ALLOCATE (m(nw), m2(nw), att(nw), sigb(nw), rinf(nw))
 ALLOCATE (tsd(nw), tdd(nw), tdo(nw), rsd(nw), rdd(nw), rso(nw), rdo(nw))
 ALLOCATE (rddt(nw), rsdt(nw), rdot(nw), rsodt(nw), rsost(nw), rsot(nw), &
      rsos(nw), rsod(nw), lidf(13), inc_arr(N_inc), rsoil_old(nw), rsoil0(nw))
 ! Structure parameter
 N = Stc

 rsoil0 = psoil*Rsoil1(Nw_S:Nw_E) + (1-psoil)*Rsoil2(Nw_S:Nw_E)

 Dwr_Pso = .FALSE.
 IF( PRESENT(Nder) ) THEN
    DO I3 = 1,SIZE( RET_INFO )
       IF( .NOT.RET_INFO(I3)%STAT ) CYCLE
       IF( .NOT.(RET_INFO(I3)%TYPE=='prosail') ) CYCLE
       SELECT CASE( RET_INFO(I3)%NAME )
       CASE('prosail_Cbr') 
          PAR_NAME = 'Cbrown' ; IF( Cbrown == 0.d0 ) GOTO 402 
       CASE('prosail_Cm')  
          PAR_NAME = 'Cm' ; IF( Cm == 0.d0 ) GOTO 402 
       CASE('prosail_Cab') 
          PAR_NAME = 'Cab' ; IF( Cab == 0.d0 ) GOTO 402 
       CASE('prosail_LAI') 
          PAR_NAME = 'LAI' ; IF( LAI == 0.d0 ) GOTO 402 
       CASE('prosail_Stc') 
          PAR_NAME = 'Stc' ; IF( Stc == 0.d0 ) GOTO 402 
       CASE('prosail_Car') 
          PAR_NAME = 'Car' ; IF( Car == 0.d0 ) GOTO 402 
       CASE('prosail_Cw')  
          PAR_NAME = 'Cw' ; IF( Cw == 0.d0 ) GOTO 402 
       CASE('prosail_Ala') 
          PAR_NAME = 'LIDFa' ; IF( LIDFa == 0.d0 ) GOTO 402 
       CASE('prosail_Pso') 
          Dwr_Pso = .TRUE.
          PAR_NAME = 'Psoil' ; IF( Psoil == 0.d0 ) GOTO 402 
       END SELECT
    ENDDO
 ENDIF
 
 init_completed=.false.	! only at first call of PRO4SAIL

 !  Radians to degrees for azimuthal angles
 inc_arr(:) = DACOS( X_inc(:) )/Pi*180.d0

 do Ip = 1, N_phi
    psi = 180.d0 - X1(Ip)/Pi*180.d0
    do Ii = 1, N_inc
       tts = inc_arr(Ii) 
       do ir = 1, N_ref
          tto = DACOS( X_ref(ir) )/Pi*180.d0       
          IF( Dwr_Pso ) &
               rsoil0 = P0(9)*Rsoil1(Nw_S:Nw_E) + (1-P0(9))*Rsoil2(Nw_S:Nw_E)
          CALL PRO4SAIL(P0(5), P0(3), P0(6), P0(1), P0(7), P0(2), P0(8), LIDFb, &
               TypeLIDF, P0(4), hspot, tts, tto, psi, rsoil0)
          R_dir(Ir, Ii, Ip, :) = rsot(:)
          R_dif(Ir, Ii, Ip, :) = rdot(:)
!         ... Derivatives if present
          IF( PRESENT(Nder) ) THEN
             Ik = 1
             DO I3 = 1,SIZE( RET_INFO )
                IF( .NOT.RET_INFO(I3)%STAT ) CYCLE
                IF( .NOT.(RET_INFO(I3)%TYPE=='prosail') ) CYCLE
                SELECT CASE( RET_INFO(I3)%NAME )
                CASE('prosail_Cbr') ; Ico = 1
                CASE('prosail_Cm')  ; Ico = 2
                CASE('prosail_Cab') ; Ico = 3
                CASE('prosail_LAI') ; Ico = 4
                CASE('prosail_Stc') ; Ico = 5
                CASE('prosail_Car') ; Ico = 6
                CASE('prosail_Cw')  ; Ico = 7
                CASE('prosail_Ala') ; Ico = 8
                CASE('prosail_Pso') ; Ico = 9
                END SELECT
                Pp(:) = P0(:)
                Pp(Ico) = (1.d0 + D_pert/100.d0)*P0(Ico)
                IF( Ico==9 ) rsoil0 =  &
                     Pp(Ico)*Rsoil1(Nw_S:Nw_E) + (1-Pp(Ico))*Rsoil2(Nw_S:Nw_E)
                CALL PRO4SAIL(Pp(5), Pp(3), Pp(6), Pp(1), Pp(7), Pp(2), Pp(8), &
                     LIDFb, TypeLIDF, Pp(4), hspot, tts, tto, psi, rsoil0)
                Del_X = (Pp(Ico) - P0(Ico))
                IF( RET_INFO(I3)%WF_DIM=='rel' ) Del_X = Del_X/P0(Ico)
                dR_dir(Ir, Ii, Ip, :, Ik) = &
                     (rsot(:) - R_dir(Ir, Ii, Ip, :))/Del_X 
                dR_dif(Ir, Ii, Ip, :, Ik) = &
                     (rdot(:) - R_dif(Ir, Ii, Ip, :))/Del_X 
                Ik = Ik + 1
             ENDDO
          ENDIF  ! PRESENT(Nder)
       enddo  ! over reflected angles (Ir)
    enddo  ! over incidence angles (Ii)
 enddo  ! over azimuthal angles (Ip)

 DEALLOCATE (LRT,rho,tau, sb, sf, vb, vf, w, m, m2, att, sigb, rinf, &
      tsd, tdd, tdo, rsd, rdd, rso, rdo, rddt, rsdt, rdot, rsodt, rsost, &
      rsot, rsos, rsod, lidf, inc_arr, rsoil_old, rsoil0) 

 CALL ST_TRACK (R_NAME, 1, 'O')
 RETURN

402 CALL WRITE_MESSAGE (R_NAME, 'ERROR', ' ')
 WRITE(EU,*) '   Derivative of intensity with respect to a parameter'
 WRITE(EU,*) ' of vegetation layer cannot be calculated if parameter'
 WRITE(EU,*) ' value is set to 0.'
 WRITE(EU,*) '   Check parameter : ', TRIM(PAR_NAME)
 RETURN 1

END SUBROUTINE PROSAIL

!********************************************************************************
!*                          Campbell.f                            
!*     
!*    Computation of the leaf angle distribution function value (freq) 
!*    Ellipsoidal distribution function caracterised by the average leaf 
!*    inclination angle in degree (ala)                                     
!*    Campbell 1986                                                      
!*                                                                              
!********************************************************************************
SUBROUTINE campbell(n,ala,freq)	
  IMPLICIT NONE  
  INTEGER*4 :: i,n
  REAL*8 :: ala,freq(n),dum
  REAL*8 :: excent,tl1(n),tl2(n),x1,x2,alpha,alpha2
  REAL*8 :: tx1(13),tx2(13)
  REAL*8 :: x12,x22,almx1,almx2,alpx1,alpx2,sum0
  DATA tx2 /0, 10, 20, 30, 40, 50, 60, 70, 80, 82, 84, 86, 88/
  DATA tx1 /10, 20, 30, 40, 50, 60, 70, 80, 82, 84, 86, 88, 90/
  
  DO i=1,n
     tl1(i)=tx1(i)*ATAN(1.)/45.
     tl2(i)=tx2(i)*ATAN(1.)/45.
  ENDDO
  excent=EXP(-1.6184e-5*ala**3+2.1145e-3*ala**2-1.2390e-1*ala+3.2491)
  sum0 = 0.
  DO i=1,n
     x1  = excent/(SQRT(1.+excent**2*TAN(tl1(i))**2))
     x2  = excent/(SQRT(1.+excent**2*TAN(tl2(i))**2))
     IF (excent.eq.1.) THEN
        freq(i) = ABS(COS(tl1(i))-COS(tl2(i)))
     ELSE
        alpha  = excent/SQRT(ABS(1.-excent**2))
        alpha2 = alpha**2
        x12 = x1**2
        x22 = x2**2
        IF (excent.gt.1) THEN
           alpx1 = SQRT(alpha2+x12)
           alpx2 = SQRT(alpha2+x22)
           dum   = x1*alpx1+alpha2*LOG(x1+alpx1)
           freq(i) = ABS(dum-(x2*alpx2+alpha2*LOG(x2+alpx2)))
        ELSE
           almx1 = SQRT(alpha2-x12)
           almx2 = SQRT(alpha2-x22)
           dum   = x1*almx1+alpha2*ASIN(x1/alpha)
           freq(i) = ABS(dum-(x2*almx2+alpha2*ASIN(x2/alpha)))
        ENDIF
     ENDIF
     sum0 = sum0+freq(i)
  ENDDO
  DO i=1,n
     freq(i)=freq(i)/sum0	!*100.
  ENDDO
  
  RETURN
END SUBROUTINE campbell

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

SUBROUTINE calc_LIDF_ellipsoidal(na,alpha,freqvar)
  IMPLICIT NONE

  INTEGER*4 :: na,ia,icou,k
  REAL*8 :: opt,bemu,benu,alphadeg,alpha,freqvar(na),freq(na),pi
  !...............................................................................
  !     Call leaf angle distribution fonction                  
  !...............................................................................	
  alphadeg=alpha  
  CALL campbell(na,alphadeg,freq)
  DO ia=1,na
     freqvar(ia)=freq(ia)
  ENDDO
  
  RETURN
END SUBROUTINE calc_LIDF_ellipsoidal


SUBROUTINE PRO4SAIL(N, Cab, Car, Cbrown, Cw, Cm, LIDFa, LIDFb, TypeLIDF, lai, q, &
     tts, tto, psi, rsoil)
  ! This version has been implemented by Jean-Baptiste Fret
  ! Jean-Baptiste Fret takes the entire responsibility for this version 
  ! All comments, changes or questions should be sent to:
  ! jbferet@stanford.edu

  !	Jean-Baptiste Fret
  !	Institut de Physique du Globe de Paris
  !	Space and Planetary Geophysics
  !	October 2009
	
  !	this model PRO4SAIL is based on a version provided by
  !	Wout Verhoef 
  !	NLR	
  !	April/May 2003,
  !	original version downloadable at http://teledetection.ipgp.jussieu.fr/prosail/
  !	Improved and extended version of SAILH model that avoids numerical singularities
  !	and works more efficiently if only few parameters change.
  ! References:
  ! 	Verhoef et al. (2007) Unified Optical-Thermal Four-Stream Radiative
  ! 	Transfer Theory for Homogeneous Vegetation Canopies, IEEE TRANSACTIONS 
  ! 	ON GEOSCIENCE AND REMOTE SENSING, VOL. 45, NO. 6, JUNE 2007

  !	The module "staticvar" is meant for internal use in order to keep
  !	several variables static between successive calls of the subroutine.
  !	For calculation of vertical flux profiles inside the canopy, the quantities 
  !	m, sb, sf, ks, and rinf are needed by the calling routine, so in this case 
  !	this module must also be declared in the calling routine.

  USE MOD_ANGLE
  USE MOD_dataSpec_P5B
  USE MOD_output_PROSPECT
  USE MOD_SAIL
  USE MOD_flag_util
  USE MOD_staticvar
  IMPLICIT NONE

!!!!!!!!		INPUT / OUTPUT		!!!!!!!!
  REAL*8,INTENT(in) :: N,Cab,Car,Cbrown,Cw,Cm
  REAL*8,INTENT(in) :: LIDFa,LIDFb,lai,q,rsoil(nw)
  REAL*8,INTENT(in) :: tts,tto,psi
  INTEGER*4,INTENT(in) :: TypeLIDF
!!!!!!!!	  END INPUT / OUTPUT	!!!!!!!!

  REAL*8 :: chi_o,chi_s,ctl,sto,sts,tanto,tants,cospsi,koli,ksli,bf,bfli,alf,litab(13)
  REAL*8 :: f1,f2,g1(nw),g2(nw),ttl,sigf(nw),sobli,sofli,x1,x2,y1,y2,z
  REAL*8 :: fhot,fint,frho,ftau,J1ko(nw),J1ks(nw),J2ko(nw),J2ks(nw),Jfunc3
  REAL*8 :: T1(nw),T2(nw),T3(nw),Tv1(nw),Tv2(nw)
  REAL*8 :: e1(nw),e2(nw),rinf2(nw),re(nw),denom(nw)
  REAL*8 :: Ps(nw),Qs(nw),Pv(nw),Qv(nw),dn(nw)
  INTEGER*4 :: na
  data litab/5.,15.,25.,35.,45.,55.,65.,75.,81.,83.,85.,87.,89./

!	Raise all flags if we arrive here for the first time, lower them at other times
  DO i=1,7
     flag(i)=.not.init_completed
  ENDDO

  IF (init_completed) THEN
	!	Detect which inputs have changed (if it is not the first time)
     delta_lai	= (lai.ne.lai_old)
     delta_hot	= (q.ne.q_old)
     delta_geom	= (tts.ne.tts_old).or.(tto.ne.tto_old).or.&
          (psi.ne.psi_old)
     delta_soil	= (SUM((rsoil-rsoil_old)**2).ne.0)
     delta_lidf	= (LIDFa.ne.LIDFa_old).or.(LIDFb.ne.LIDFb_old)
     delta_leaf	= (N.ne.N_old).or.(Cab.ne.Cab_old).or.(Car.ne.Car_old).or.&
					(Cbrown.ne.Cbrown_old).or.(Cw.ne.Cw_old).or.(Cm.ne.Cm_old)

	!	Raise the flags for the modules to be executed
     flag(1) = delta_geom
     flag(2)	= delta_lidf
     flag(3) = delta_geom.or.delta_lidf
     flag(4) = flag(3).or.delta_leaf
     flag(5) = flag(4).or.delta_lai.or.delta_soil
     flag(6) = flag(3).or.delta_lai.or.delta_hot
     flag(7) = delta_leaf
  ENDIF

  !	Make sure that on next occasions init is regarded as being completed
  init_completed=.true.

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!LEAF CHEM & STR PROPERTIES!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
IF(flag(7)) THEN
   CALL prospect_5B(N,Cab,Car,Cbrown,Cw,Cm,LRT)
   rho	=	LRT(:,1)
   tau	=	LRT(:,2)
ENDIF
	
IF(flag(1)) THEN
!	Geometric quantities
	cts		= COS(rd*tts)
	cto		= COS(rd*tto)
	ctscto	= cts*cto
	tants	= TAN(rd*tts)
	tanto	= TAN(rd*tto)
	cospsi	= COS(rd*psi)
	dso		= SQRT(tants*tants+tanto*tanto-2.*tants*tanto*cospsi)
ENDIF

na=13
IF (flag(2)) THEN
!	Generate leaf angle distribution from average leaf angle (ellipsoidal) or (a,b) parameters
	IF(TypeLidf.EQ.1) THEN
		CALL dladgen(LIDFa,LIDFb,lidf)
	ELSEIF(TypeLidf.EQ.2) THEN
		CALL calc_LIDF_ellipsoidal(na,LIDFa,lidf)
	ENDIF
ENDIF

! angular distance, compensation of shadow length
IF (flag(3)) THEN
   !	Calculate geometric factors associated with extinction and scattering 
   !	Initialise sums
   ks	= 0
   ko	= 0
   bf	= 0
   sob	= 0
   sof	= 0

   !	Weighted sums over LIDF
   DO i=1,na
      ttl = litab(i)	! leaf inclination discrete values
      ctl = COS(rd*ttl)
      !	SAIL volume scattering phase function gives interception and portions to be 
      !	multiplied by rho and tau
      CALL volscatt(tts,tto,psi,ttl,chi_s,chi_o,frho,ftau)
      
      !********************************************************************************
      !*                   SUITS SYSTEM COEFFICIENTS 
      !*
      !*	ks  : Extinction coefficient for direct solar flux
      !*	ko  : Extinction coefficient for direct observed flux
      !*	att : Attenuation coefficient for diffuse flux
      !*	sigb : Backscattering coefficient of the diffuse downward flux
      !*	sigf : Forwardscattering coefficient of the diffuse upward flux
      !*	sf  : Scattering coefficient of the direct solar flux for downward diffuse flux
      !*	sb  : Scattering coefficient of the direct solar flux for upward diffuse flux
      !*	vf   : Scattering coefficient of upward diffuse flux in the observed direction
      !*	vb   : Scattering coefficient of downward diffuse flux in the observed direction
      !*	w   : Bidirectional scattering coefficient
      !********************************************************************************
      
      !	Extinction coefficients
      ksli = chi_s/cts
      koli = chi_o/cto

      !	Area scattering coefficient fractions
      sobli	= frho*pi/ctscto
      sofli	= ftau*pi/ctscto
      bfli	= ctl*ctl
      ks	= ks+ksli*lidf(i)
      ko	= ko+koli*lidf(i)
      bf	= bf+bfli*lidf(i)
      sob	= sob+sobli*lidf(i)
      sof	= sof+sofli*lidf(i)
      
   ENDDO
   !	Geometric factors to be used later with rho and tau
   sdb	= 0.5*(ks+bf)
   sdf	= 0.5*(ks-bf)
   dob	= 0.5*(ko+bf)
   dof	= 0.5*(ko-bf)
   ddb	= 0.5*(1.+bf)
   ddf	= 0.5*(1.-bf)
ENDIF

IF(flag(4)) THEN
	!	Here rho and tau come in
	sigb= ddb*rho+ddf*tau
	sigf= ddf*rho+ddb*tau
	att	= 1.-sigf
	m2=(att+sigb)*(att-sigb)
	WHERE (m2.LT.0) 
		m2=0
	ENDWHERE
	m=SQRT(m2)
	sb	= sdb*rho+sdf*tau
	sf	= sdf*rho+sdb*tau
	vb	= dob*rho+dof*tau
	vf	= dof*rho+dob*tau
	w	= sob*rho+sof*tau
ENDIF

IF (flag(5)) THEN
	!	Here the LAI comes in
	!   Outputs for the case LAI = 0
	IF (lai.le.0) THEN
		tss		= 1.;		too		= 1.;		tsstoo	= 1.
		rdd		= 0.;		tdd		= 1.;		rsd		= 0.
		tsd		= 0.;		rdo		= 0.;		tdo		= 0.
		rso		= 0.;		rsos	= 0.;		rsod	= 0.

		rddt	= rsoil;	rsdt	= rsoil;	rdot	= rsoil
		rsodt	= 0.;		rsost	= rsoil;	rsot	= rsoil
		RETURN
	ENDIF

	!	Other cases (LAI > 0)
	e1		= EXP(-m*lai)
	e2		= e1*e1
	rinf	= (att-m)/sigb
	rinf2	= rinf*rinf
	re		= rinf*e1
	denom	= 1.-rinf2*e2

	CALL Jfunc1(ks,m,lai,J1ks)
	CALL Jfunc2(ks,m,lai,J2ks)
	CALL Jfunc1(ko,m,lai,J1ko)
	CALL Jfunc2(ko,m,lai,J2ko)

	Ps = (sf+sb*rinf)*J1ks
	Qs = (sf*rinf+sb)*J2ks
	Pv = (vf+vb*rinf)*J1ko
	Qv = (vf*rinf+vb)*J2ko

	rdd	= rinf*(1.-e2)/denom
	tdd	= (1.-rinf2)*e1/denom
	tsd	= (Ps-re*Qs)/denom
	rsd	= (Qs-re*Ps)/denom
	tdo	= (Pv-re*Qv)/denom
	rdo	= (Qv-re*Pv)/denom

	tss	= EXP(-ks*lai)
	too	= EXP(-ko*lai)
	z	= Jfunc3(ks,ko,lai)
	g1	= (z-J1ks*too)/(ko+m)
	g2	= (z-J1ko*tss)/(ks+m)

	Tv1 = (vf*rinf+vb)*g1
	Tv2 = (vf+vb*rinf)*g2
	T1	= Tv1*(sf+sb*rinf)
	T2	= Tv2*(sf*rinf+sb)
	T3	= (rdo*Qs+tdo*Ps)*rinf

	!	Multiple scattering contribution to bidirectional canopy reflectance
	rsod = (T1+T2-T3)/(1.-rinf2)
ENDIF

IF (flag(6)) THEN
	!	Treatment of the hotspot-effect
	alf=1e6
	!	Apply correction 2/(K+k) suggested by F.-M. Bron
	IF (q.gt.0.) THEN
		alf=(dso/q)*2./(ks+ko)
	ENDIF
	IF (alf.GT.200.) THEN	!inserted H. Bach 1/3/04
		alf=200.
	ENDIF
	IF (alf.eq.0.) THEN
		!	The pure hotspot - no shadow
		tsstoo = tss
		sumint = (1-tss)/(ks*lai)
	ELSE
		!	Outside the hotspot
		fhot=lai*SQRT(ko*ks)
		!	Integrate by exponential Simpson method in 20 steps
		!	the steps are arranged according to equal partitioning
		!	of the slope of the joint probability function
		x1=0.
		y1=0.
		f1=1.
		fint=(1.-EXP(-alf))*.05
		sumint=0.

		DO i=1,20
			IF (i.lt.20) THEN
				x2=-LOG(1.-i*fint)/alf
			ELSE
				x2=1.
			ENDIF
			y2=-(ko+ks)*lai*x2+fhot*(1.-EXP(-alf*x2))/alf 
			f2=EXP(y2)
			sumint=sumint+(f2-f1)*(x2-x1)/(y2-y1)
			x1=x2
			y1=y2
			f1=f2
		ENDDO
		tsstoo=f1
	ENDIF
ENDIF

!	Bidirectional reflectance
!	Single scattering contribution
	rsos = w*lai*sumint

!	Total canopy contribution
	rso=rsos+rsod

!	Interaction with the soil
dn=1.-rsoil*rdd

! rddt: bi-hemispherical reflectance factor
rddt=rdd+tdd*rsoil*tdd/dn
! rsdt: directional-hemispherical reflectance factor for solar incident flux
rsdt=rsd+(tsd+tss)*rsoil*tdd/dn
! rdot: hemispherical-directional reflectance factor in viewing direction    
rdot=rdo+tdd*rsoil*(tdo+too)/dn
! rsot: bi-directional reflectance factor
rsodt = rsod + ( (tss+tsd)*tdo + (tsd+tss*rsoil*rdd)*too )*rsoil/dn
rsost = rsos + tsstoo*rsoil
rsot  = rsost + rsodt

	!	Before returning, save current parameters as old ones 
	N_old	    = N	    ;  Cab_old   = Cab   ; Car_old    = Car
	Cbrown_old = Cbrown ;  Cw_old    = Cw    ; Cm_old     = Cm
	lai_old	   = lai    ;  LIDFa_old = LIDFa ; LIDFb_old  = LIDFb
	tts_old	   = tts    ;  tto_old   = tto   ; psi_old    = psi
	q_old	   = q      ;  rsoil_old	= rsoil
	
RETURN
END

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!****************************************************
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
SUBROUTINE Jfunc1(k,l,t,Jout)

  USE MOD_dataSpec_P5B
  IMPLICIT NONE
!	J1 function with avoidance of singularity problem
  REAL*8,INTENT(in) :: k,l(nw),t
  REAL*8,INTENT(out) :: Jout(nw)
  REAL*8 :: del(nw)

  del=(k-l)*t
  WHERE (ABS(del)>1e-3)
     Jout=(EXP(-l*t)-EXP(-k*t))/(k-l)
  ELSEWHERE
     Jout=0.5*t*(EXP(-k*t)+EXP(-l*t))*(1.-del*del/12.)
  END WHERE
  RETURN  
END SUBROUTINE Jfunc1
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!****************************************************
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

SUBROUTINE Jfunc2(k,l,t,Jout)

  USE MOD_dataSpec_P5B
  IMPLICIT NONE
!	J2 function
  REAL*8,INTENT(in) :: k,l(nw),t
  REAL*8,INTENT(out) :: Jout(nw)
  REAL*8 :: del(nw)
  Jout=(1.-EXP(-(k+l)*t))/(k+l)
  RETURN
END SUBROUTINE Jfunc2
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!****************************************************
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

FUNCTION Jfunc3(k,l,t)

  USE MOD_dataSpec_P5B
  IMPLICIT NONE
!	J2 function
  REAL*8 :: k,l,t
  REAL*8 :: Jfunc3
  Jfunc3=(1.-EXP(-(k+l)*t))/(k+l)
END FUNCTION Jfunc3

! ******************************************************************************** 
! tav_abs.f90 
! ******************************************************************************** 
! computation of the average transmittivity at the leaf surface within a given 
! solid angle. teta is the incidence solid angle (in radian). The average angle 
! that works in most cases is 40deg*pi/180. ref is the refaction index. 
! ******************************************************************************** 
! Stern F. (1964), Transmission of isotropic radiation across an interface between 
! two dielectrics, Applied Optics, 3:111-113. 
! Allen W.A. (1973), Transmission of isotropic light across a dielectric surface in 
! two and three dimensions, Journal of the Optical Society of America, 63:664-666. 
! ******************************************************************************** 
! version 5.02 (25 July 2011) 
! ******************************************************************************** 
 
subroutine tav_abs(theta,refr,res) 

  USE PRO_SAIL, ONLY:  NW_E, NW_S
  USE MOD_dataSpec_P5B 
  IMPLICIT none 
  REAL*8, intent(in) :: theta, refr(nwb) 
  REAL*8, intent(out) :: res(nw) 
 
  REAL*8 :: pi, thetarad 
  REAL*8 :: refr2(nw) 
  REAL*8 :: ax(nw),bx(nw),b0(nw),b1(nw),b2(nw) 
  REAL*8 :: ts(nw),tp(nw),tp1(nw),tp2(nw),tp3(nw),tp4(nw),tp5(nw) 
 
  pi=atan(1.)*4. 
  thetarad=pi*theta/180.
  if (theta.eq.0.) then 
     res = 4.*refr(Nw_S:Nw_E)/(refr(Nw_S:Nw_E) + 1.)**2 
     return 
  endif

  refr2=refr(Nw_S:Nw_E)*refr(Nw_S:Nw_E)
  ax=(refr(Nw_S:Nw_E)+1.)**2/2.
  bx=-(refr2-1.)**2/4.

  if (thetarad.eq.pi/2.) then
     b1=0.
  else
     b1=dsqrt((sin(thetarad)**2-(refr2+1.)/2.)**2+bx)
  endif
  b2=sin(thetarad)**2-(refr2+1.)/2.
  b0=b1-b2
  ts=(bx**2/(6.*b0**3)+bx/b0-b0/2.)-(bx**2/(6.*ax**3)+bx/ax-ax/2.)
  tp1=-2.*refr2*(b0-ax)/(refr2+1.)**2
  tp2=-2.*refr2*(refr2+1.)*dlog(b0/ax)/(refr2-1.)**2
  tp3=refr2*(1./b0-1./ax)/2.
  tp4=16.*refr2**2*(refr2**2+1.)*dlog((2.*(refr2+1.)*b0-(refr2-1.)**2)/ &
       (2.*(refr2+1.)*ax-(refr2-1.)**2))/((refr2+1.)**3*(refr2-1.)**2)
  tp5=16.*refr2**3*(1./(2.*(refr2+1.)*b0-((refr2-1.)**2))-1./(2.*(refr2+1.) & 
       *ax-(refr2-1.)**2))/(refr2+1.)**3
  tp=tp1+tp2+tp3+tp4+tp5
  res=(ts+tp)/(2.*sin(thetarad)**2)
  
  return
end subroutine tav_abs
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!****************************************************
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

SUBROUTINE volscatt(tts,tto,psi,ttl,chi_s,chi_o,frho,ftau)
!********************************************************************************
!*	tts		= solar zenith
!*	tto		= viewing zenith
!*	psi		= azimuth
!*	ttl		= leaf inclination angle
!*	chi_s	= interception functions
!*	chi_o	= interception functions
!*	frho	= function to be multiplied by leaf reflectance rho
!*	ftau	= functions to be multiplied by leaf transmittance tau
!********************************************************************************

!	Compute volume scattering functions and interception coefficients
!	for given solar zenith, viewing zenith, azimuth and leaf inclination angle.

!	chi_s and chi_o are the interception functions.
!	frho and ftau are the functions to be multiplied by leaf reflectance rho and
!	leaf transmittance tau, respectively, in order to obtain the volume scattering
!	function.

!	Wout Verhoef, april 2001, for CROMA

  USE MOD_ANGLE
  IMPLICIT NONE
  REAL*8,INTENT(in) :: tts,tto,psi,ttl
  REAL*8,INTENT(inout) :: chi_s,chi_o,frho,ftau

  REAL*8 costs,costo,sints,sinto,cospsi
  REAL*8 psir
  REAL*8 costl,sintl,cs,co,ss,so,ds
  REAL*8 cosbts,cosbto,bts,bto
  REAL*8 btran1,btran2,bt1,bt2,bt3,t1,t2
  REAL*8 doo,denom

  costs=COS(rd*tts)
  costo=COS(rd*tto)
  sints=SIN(rd*tts)
  sinto=SIN(rd*tto)
  cospsi=COS(rd*psi)

  psir=rd*psi

  costl=COS(rd*ttl)
  sintl=SIN(rd*ttl)
  cs=costl*costs
  co=costl*costo
  ss=sintl*sints
  so=sintl*sinto

!c ..............................................................................
!c     betas -bts- and betao -bto- computation
!c     Transition angles (beta) for solar (betas) and view (betao) directions
!c     if thetav+thetal>pi/2, bottom side of the leaves is observed for leaf azimut 
!c     interval betao+phi<leaf azimut<2pi-betao+phi.
!c     if thetav+thetal<pi/2, top side of the leaves is always observed, betao=pi
!c     same consideration for solar direction to compute betas
!c ..............................................................................


  cosbts=5.
  IF (ABS(ss).gt.1e-6) THEN
     cosbts=-cs/ss
  ENDIF

  cosbto=5.
  IF (ABS(so).gt.1e-6) THEN
     cosbto=-co/so
  ENDIF

  IF (ABS(cosbts).lt.1.d0) THEN
     bts=ACOS(cosbts)
     ds=ss
  ELSE
     bts=pi
     ds=cs
  ENDIF

  chi_s=2./pi*((bts-pi*.5)*cs+sin(bts)*ss)

  IF (ABS(cosbto).lt.1.d0) THEN
     bto=ACOS(cosbto)
     doo=so
  ELSEIF(tto.lt.90.) THEN
     bto=pi
     doo=co
  ELSE
     bto=0
     doo=-co
  ENDIF

  chi_o=2./pi*((bto-pi*.5)*co+SIN(bto)*so)

!c ..............................................................................
!c   Computation of auxiliary azimut angles bt1, bt2, bt3 used          
!c   for the computation of the bidirectional scattering coefficient w              
!c .............................................................................
  btran1=abs(bts-bto)
  btran2=pi-abs(bts+bto-pi)
  
  IF (psir.le.btran1) THEN
     bt1=psir
     bt2=btran1
     bt3=btran2
  ELSE
     bt1=btran1
     IF (psir.le.btran2) THEN
        bt2=psir
        bt3=btran2
     ELSE
        bt2=btran2
        bt3=psir
     ENDIF
  ENDIF

  t1=2.*cs*co+ss*so*cospsi
  t2=0.
  IF (bt2.gt.0.) THEN
     t2=SIN(bt2)*(2.*ds*doo+ss*so*COS(bt1)*COS(bt3))
  ENDIF
  
  denom=2.*pi*pi
  frho=((pi-bt2)*t1+t2)/denom
  ftau=    (-bt2*t1+t2)/denom
  
  IF (frho.lt.0) THEN
     frho=0
  ENDIF
  
  IF (ftau.lt.0) THEN
     ftau=0
  ENDIF

  RETURN
END SUBROUTINE volscatt
! ******************************************************************************** 
! prospect_5B.f90 
! ******************************************************************************** 
! 
! Jean-Baptiste FERET 
!  
! Department of Global Ecology / Carnegie Institution for Sciences 
! 260 Panama Street 
! Stanford, CA 94305, USA 
! E-mail: jbferet@stanford.edu 
! 
! Stphane JACQUEMOUD 
! 
! Universit Paris Diderot / Institut de Physique du Globe de Paris 
! 35 rue Hlne Brion 
! 75013 Paris, France 
! E-mail: jacquemoud@ipgp.fr 
! 
! http://teledetection.ipgp.fr/prosail/ 
! 
! ******************************************************************************** 
! Fret J.B., Franois C., Asner G.P., Gitelson A.A., Martin R.E., Bidel L.P.R., 
! Ustin S.L., le Maire G., Jacquemoud S. (2008), PROSPECT-4 and 5: Advances in the 
! leaf optical properties model separating photosynthetic pigments, Remote Sensing 
! of Environment, 112:3030-3043. 
! Jacquemoud S., Ustin S.L., Verdebout J., Schmuck G., Andreoli G., Hosgood B. 
! (1996), Estimating leaf biochemistry using the PROSPECT leaf optical properties 
! model, Remote Sensing of Environment, 56:194-202. 
! Jacquemoud S., Baret F. (1990), PROSPECT: a model of leaf optical properties 
! spectra, Remote Sensing of Environment, 34:75-91. 
! ******************************************************************************** 
! version 5.02 (25 July 2011) 
! ******************************************************************************** 
 
SUBROUTINE prospect_5B(N,Cab,Car,Cbrown,Cw,Cm,RT) 

  USE PRO_SAIL, ONLY:  NW_E, NW_S
  USE MOD_dataSpec_P5B 
  IMPLICIT none 
  REAL*8, intent(in) :: N,Cab,Car,Cbrown,Cw,Cm 
  REAL*8, intent(out) :: RT(nw,2)
  
  REAL*8 :: k(nw), tau(nw), xx(nw), yy(nw) 
  REAL*8 :: x1(nw),x2(nw),x3(nw),x4(nw),x5(nw),x6(nw) 
  REAL*8 :: theta1, theta2, t1(nw),t2(nw)
  REAL*8 :: r(nw),t(nw),ra(nw),ta(nw) 
  REAL*8 :: delta(nw),beta(nw),va(nw),vb(nw),s1(nw),s2(nw),s3(nw) 
  
  k = (Cab*k_Cab(Nw_S:Nw_E) + Car*k_Car(Nw_S:Nw_E) + &
       Cbrown*k_Brown(Nw_S:Nw_E) + Cw*k_Cw(Nw_S:Nw_E) + Cm*k_Cm(Nw_S:Nw_E))/N 

! ******************************************************************************** 
! reflectance and transmittance of one layer 
! ******************************************************************************** 
! Allen W.A., Gausman H.W., Richardson A.J., Thomas J.R. (1969), Interaction of 
! isotropic ligth with a compact plant leaf, Journal of the Optical Society of 
! American, 59:1376-1379. 
! ******************************************************************************** 
 
! exponential integral: S13AAF routine from the NAG library 
 
  where (k.le.0.0) 
     tau=1 
  end where
  where (k.gt.0.0.and.k.le.4.0) 
     xx=0.5*k-1.0 
     yy=(((((((((((((((-3.60311230482612224d-13 & 
          *xx+3.46348526554087424d-12)*xx-2.99627399604128973d-11) & 
          *xx+2.57747807106988589d-10)*xx-2.09330568435488303d-9) & 
          *xx+1.59501329936987818d-8)*xx-1.13717900285428895d-7) & 
          *xx+7.55292885309152956d-7)*xx-4.64980751480619431d-6) & 
          *xx+2.63830365675408129d-5)*xx-1.37089870978830576d-4) & 
          *xx+6.47686503728103400d-4)*xx-2.76060141343627983d-3) & 
          *xx+1.05306034687449505d-2)*xx-3.57191348753631956d-2) & 
          *xx+1.07774527938978692d-1)*xx-2.96997075145080963d-1 
     yy=(yy*xx+8.64664716763387311d-1)*xx+7.42047691268006429d-1 
     yy=yy-log(k) 
     tau=(1.0-k)*dexp(-k)+k**2*yy 
  end where
  where (k.gt.4.0.and.k.le.85.0) 
     xx=14.5/(k+3.25)-1.0 
     yy=(((((((((((((((-1.62806570868460749d-12 & 
          *xx-8.95400579318284288d-13)*xx-4.08352702838151578d-12) & 
          *xx-1.45132988248537498d-11)*xx-8.35086918940757852d-11) & 
          *xx-2.13638678953766289d-10)*xx-1.10302431467069770d-9) & 
          *xx-3.67128915633455484d-9)*xx-1.66980544304104726d-8) & 
          *xx-6.11774386401295125d-8)*xx-2.70306163610271497d-7) & 
          *xx-1.05565006992891261d-6)*xx-4.72090467203711484d-6) & 
          *xx-1.95076375089955937d-5)*xx-9.16450482931221453d-5) & 
          *xx-4.05892130452128677d-4)*xx-2.14213055000334718d-3 
     yy=((yy*xx-1.06374875116569657d-2)*xx-8.50699154984571871d-2)*xx+9.23755307807784058d-1 
     yy=exp(-k)*yy/k 
     tau=(1.0-k)*dexp(-k)+k**2*yy 
  end where
  where (k.gt.85.0) 
     tau=0 
  end where
 
! transmissivity of the layer 
  theta1=90. 
  call tav_abs(theta1,refractive,t1) 
  theta2=40. 
  call tav_abs(theta2,refractive,t2) 
  x1=1-t1 
  x2 = t1**2*tau**2*( refractive(Nw_S:Nw_E)**2-t1 ) 
  x3 = t1**2*tau*refractive(Nw_S:Nw_E)**2 
  x4 = refractive(Nw_S:Nw_E)**4 - tau**2*(refractive(Nw_S:Nw_E)**2 - t1)**2 
  x5=t2/t1 
  x6=x5*(t1-1)+1-t2 
  r=x1+x2/x4 
  t=x3/x4 
  ra=x5*r+x6 
  ta=x5*t 
 
! ******************************************************************************** 
! reflectance and transmittance of N layers 
! ******************************************************************************** 
! Stokes G.G. (1862), On the intensity of the light reflected from or transmitted 
! through a pile of plates, Proceedings of the Royal Society of London, 11:545-556. 
! ********************************************************************************
  delta=(t**2-r**2-1)**2-4*r**2 
  beta=(1+r**2-t**2-sqrt(delta))/(2*r) 
  va=(1+r**2-t**2+sqrt(delta))/(2*r) 
  vb=sqrt(beta*(va-r)/(va*(beta-r))) 
  s1=ra*(va*vb**(N-1)-va**(-1)*vb**(-(N-1)))+(ta*t-ra*r)*(vb**(N-1)-vb**(-(N-1))) 
  s2=ta*(va-va**(-1)) 
  s3=va*vb**(N-1)-va**(-1)*vb**(-(N-1))-r*(vb**(N-1)-vb**(-(N-1))) 
  RT(:,1)=s1/s3 
  RT(:,2)=s2/s3 
  RETURN
end subroutine prospect_5B
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!****************************************************
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

subroutine dladgen(a,b,freq) 
  INTEGER I, I1, I2
  real*8 a,b,freq(13),t 
  real*8 dcum 
  do i1=1,8 
     t=i1*10.  
     freq(i1)=dcum(a,b,t) 
  end do
  do i2=9,12 
     t=80.+(i2-8)*2. 
     freq(i2)=dcum(a,b,t) 
  end do
  freq(13)=1. 
  do i=13,2,-1 
     freq(i)=freq(i)-freq(i-1) 
  end do
  return 
end subroutine dladgen
! *******************************************************

function dcum(a,b,t) 
  real*8 a,b,t,pi,rd,eps,delx,p,y,x,dx,dcum 
  pi=atan(1.)*4. 
  rd=pi/180. 
  
  if (a.gt.1.) then 
     dcum=1.-cos(rd*t) 
  else      
     eps=1.e-8 
     delx=1. 
     x=2*rd*t 
     p=x 
     do while (delx.gt.eps)  
        y = a*sin(x)+.5*b*sin(2.*x) 
        dx=.5*(y-x+p)  
        x=x+dx 
        delx=abs(dx) 
     end do
     dcum=(2.*y+p)/pi 
  endif
  return 
end function dcum
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!***********  End PRO4SAIL collection  **************
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
