C********************************************************************C
C
C       SCIATRAN - Version 3.4 - Last modification: 30 Apr 2014
C
C       Copyright (C) - Institute of Remote Sensing,
C                       University of Bremen, Germany
C
C********************************************************************C
      SUBROUTINE RAYLEIGH_SCATT (JPOS, POS, DEPOL, SCR, *)

C     Calculates Rayleigh scattering profile SCR in km-1

C     RAY_SCATT_IDX  Index for choice of Rayleigh scattering formula
C     POS            wavelength as input in [nm]
C     NLAYERS	     Number of layers
C     PPP            Pressure profile
C     TTT            Temperature profile
C     AIR_DENSITY    Molecular density [mol/(km*cm**2)]
C     DEPOL          Depolarisation ratio of atmospheric molecules
C     SCR            Volume Rayleigh scattering coefficient profile
C     SCR_W          Volume Rayleigh scattering coefficient profile of 
c                    pure water
      USE AEROSOLS,   ONLY: SCR_WL
      USE GRIDS,      ONLY: NLAYERS, ZZZ
      USE LOG_PARS,   ONLY: N_ESO, LOG_LEVEL, LUNIT
      USE PROFILES,   ONLY: PPP => PRESS, TTT => TEMP, AIR_DENSITY
      USE ST_VARS,    ONLY: FP, SMKM, RAY_SCATT_IDX, PIE, PVC, 
     $                      LL_factor, Refr_index, King_factor
      USE TYPES,      ONLY: KND
      USE WATER,      ONLY: UWT, NL_UW => NLAYERS_UW
c     ==================================================================

c     ----------------------------------------------------------------      
      INTEGER       JPOS
      REAL (KND), INTENT(IN)  :: POS, DEPOL
      REAL (KND)    SCR(NLAYERS)
c     ------------------------
c     ... Local parameters
c     --------------------
      CHARACTER (LEN=*), PARAMETER :: R_NAME = 'RAYLEIGH_SCATT'
c      LOGICAL      ERROR
      REAL (KND)   W_loc, R_sec
      REAL (KND), PARAMETER :: 
c          Avogadro's number [molecules/mol]
     $     N_Avo = 6.0221367d23, 
c          Molar volume [L/mol] at 0 C and 1013.25 mb  
     $     Mol_V = 22.4141d0, 
c          Loschmidt number [molecules/cm^3] at 
c               0 C, 1013.25 mb             15 C, 1013.25 mb   
     $     Ln_0C = N_Avo/Mol_V*1.d-03, Ln_15C = Ln_0C*273.15d0/288.15d0

C     Indices for scattering formula
      INTEGER, PARAMETER :: RAY_Bates_IDX = 1
      INTEGER, PARAMETER :: RAY_Bucholtz_IDX = 2
      INTEGER, PARAMETER :: RAY_Bodhaine_IDX = 3
      INTEGER, PARAMETER :: RAY_User_IDX = 4

C     Constants for the Bucholtz-95 approximation
      REAL (KND), DIMENSION (2):: HF0, HF1, HF2, HF3
      DATA  HF0, HF1, HF2, HF3
     $     /3.01577d-28, 4.01061d-28, 3.55212d0, 3.99668d0, 1.35579, 
     $     1.10298d-03, 0.11563d0, 2.71393d-02 /

C     Constants for the Bodhaine et.al Rayleigh scattering cross-section
      REAL (KND), PARAMETER :: Ba = 1.0455996d0, Bb= -341.29061d0,
     $     Bc = -0.90230850d0, Bd = 0.0027059889d0 , Be = -85.968563d0

      REAL (KND)  VV, Ps, tC, S1, S2, R4(4), Ki(4)
      REAL (KND), EXTERNAL :: Ray_sec,
     $     Refr_N2, Refr_O2, Refr_Ar, 
     $     Refr_CO2, King_N2, King_O2, King_Ar, King_CO2,
     $     Refr_air_E53, Refr_air_E66, Refr_air_PR
c     ------------------------------------------------------------------

      CALL ST_TRACK (R_NAME, 1, 'I')
 
c     The routine-function "Ray_sec" is in collection "st_refr_king.f"
c     ... Rayleigh cross-section [cm2/molecule] : R_sec
      R_sec = Ray_sec(POS, DEPOL)
c     ... Rayleigh volume scattering coefficient [km-1]: SCR
      SCR(:) = R_sec*AIR_DENSITY(:)
      IF(ALLOCATED(SCR_WL)) SCR_WL(JPOS,:) = SCR(:)

cc     ... Test version for JPL with 8 km scale height
c      WRITE(*,*) '   ********************************    '
c      WRITE(*,*) '    Message from "rayleigh_scatt,f"     '
c      WRITE(*,*) '    !!! Test JPL Rayleigh case !!!     '
c      WRITE(*,*) '   ********************************    '
c      DO I = 1, NLAYERS
c         SCR(I) = 1.d0* DEXP(-ZZZ(I)/8.d0)
c      ENDDO
cc     ... Test version with constant Rayleigh coefficient
c      WRITE(*,*) '   ******************************    '
c      WRITE(*,*) '   !!!   Rayleigh constant    !!!    '
c      WRITE(*,*) '   ******************************    '
c      DO I = 1, NLAYERS
c         SCR(I) = 1.d0
c      ENDDO

      IF( LOG_LEVEL(1) >= N_ESO ) THEN
        WRITE(LUNIT,*)
     $        '      Wavelength [nm]               ', POS
         WRITE(LUNIT,*) 
     $        '      Rayleigh cross-section [cm^2] ', R_sec
         WRITE(LUNIT,*) 
     $        '      Rayleigh volume at STP [1/km] ', 
     $        R_sec*1013.25d0*FP/288.15*SMKM
         WRITE(LUNIT,*)
      ENDIF

c      CALL BATES_ORIG

      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN

      CONTAINS
c *************************************************************************
c      DOUBLE PRECISION FUNCTION Refr_air_PR (X)
c      DOUBLE PRECISION   X, DC
c      INTEGER            IDI
cc     ... Number of spectral intervals where coefficients for the 
cc         approximation of air refractive index are given
c      INTEGER, PARAMETER :: NI_PR = 2, Nwl = NI_PR + 1
cc     ... Wavelengths in micrometer for these intervals
c      DOUBLE PRECISION WL_PR(Nwl)
c      DATA WL_PR /0.0d0, 0.23d0, 30d0/
c      DOUBLE PRECISION A0(NI_PR), A1(NI_PR), A2(NI_PR), 
c     $     B1(NI_PR), B2(NI_PR)
c      DATA A0 /8060.51d0, 0.d0/
c      DATA A1 /2480990.0d0, 5791817.0d0/
c      DATA A2 /17455.7d0, 167909.0d0/
c      DATA B1 /132.274d0, 238.0185d0/
c      DATA B2 /39.32957d0, 57.362d0/
c      W_loc = X*1.0d-03         ! Transform to micrometer
c      CALL LIN_SEARCH(W_loc, DC, IDI, Nwl, WL_PR, ERROR) 
cc      write(*,*) '  air PR interval :', IDI
c      W_loc = 1.d0/W_loc/W_loc
c      Refr_air_PR = 1.d0 + ( A0(IDI) + A1(IDI)/(B1(IDI) - W_loc) 
c     $     + A2(IDI)/(B2(IDI) - W_loc) )*1.d-08
c      RETURN
c      END FUNCTION
c
c *************************************************************************
c         The same as Refr_air_PR but using five-parameter approximation
c                       in all spectral range
      DOUBLE PRECISION FUNCTION Refr_air_PR5 (X)
      DOUBLE PRECISION   X
      DOUBLE PRECISION A0, A1, A2, B1, B2
      DATA A0 /8060.51d0/
      DATA A1 /2480990.0d0/
      DATA A2 /17455.7d0/
      DATA B1 /132.274d0/
      DATA B2 /39.32957d0/
      W_loc = X*1.0d-03         ! Transform to micrometer
      W_loc = 1.d0/W_loc/W_loc
      Refr_air_PR5 = 1.d0+( A0 + A1/(B1-W_loc) + A2/(B2-W_loc) )*1.d-08
      RETURN
      END FUNCTION

cc *************************************************************************
cc     ... Refractive index of air for 15C and 1013.25mb
c      DOUBLE PRECISION FUNCTION Refr_air_E53 (X)
c      DOUBLE PRECISION   X
c      DOUBLE PRECISION A0, A1, A2, B1, B2
c      DATA A0, A1, A2, B1, B2 
c     $     /6432.80d0, 2949810.d0, 25540.d0, 146.d0, 41.d0/
c      W_loc = X*1.0d-03         ! Transform to micrometer
c      W_loc = 1.d0/W_loc/W_loc
c      Refr_air_E53 = 
c     $     1.d0 + ( A0 + A1/(B1-W_loc) + A2/(B2-W_loc) )*1.d-08
c      RETURN
c      END FUNCTION
c
cc *************************************************************************
c      DOUBLE PRECISION FUNCTION Refr_air_E66 (X)
c      DOUBLE PRECISION   X
c      DOUBLE PRECISION A0, A1, A2, B1, B2
c      DATA A0, A1, A2, B1, B2 
c     $     /8342.13d0, 2406030.d0, 15997.d0, 130.d0, 38.9d0/
c      W_loc = X*1.0d-03         ! Transform to micrometer
c      W_loc = 1.d0/W_loc/W_loc
c      Refr_air_E66 = 
c     $     1.d0 + ( A0 + A1/(B1-W_loc) + A2/(B2-W_loc) )*1.d-08
c      RETURN
c      END FUNCTION

c***********************************************************************
c     The following function is needed as input to d_func:
c     R_SCAT corresponds to sall as a function of pressure
C***********************************************************************
      DOUBLE PRECISION FUNCTION R_SCAT (X, X_1)
c     ... As input parameters: pressure [mb] and temperature [K]
      DOUBLE PRECISION  X, X_1
      DOUBLE PRECISION, PARAMETER :: PP0 = 0.750246763D0, DR1 = 1.049D0,
     $     DR2 = 0.0157D0, DR3 = 720.883D0, DR4 = 0.003661D0
      
      R_SCAT = ( X * PP0 * 
     $ ( 1.0D0 + X * PP0 * ( DR1 - DR2 * ( X_1 - 273.15D0 ) ) * 1.D-6 )
     $ / ( DR3 * ( 1.0D0 + DR4 * ( X_1 - 273.15D0 ) ) ) ) **2
     $ / ( FP * X ) * X_1 * SMKM

      RETURN
      END FUNCTION

C********************************************************************
c       Calculation of a numerical derivative
c         input : function FUNC, position X, stepsize H
c         output: value of derivative D_FUNC
c********************************************************************
      FUNCTION D_FUNC(X, X_1, H, VARY_FIRST_ARG)
      USE TYPES,   ONLY: KND
c     ------------------------------------------------------------------      
      LOGICAL             VARY_FIRST_ARG
      REAL (KND) D_FUNC 
      REAL (KND) CON, CON2, BIG, SAFE
      INTEGER    NTAB, I, J
      PARAMETER ( CON = 1.4D0, CON2 = CON*CON, BIG = 1.D30,
     &     NTAB = 10, SAFE = 2.D0)
      REAL (KND) A(NTAB,NTAB), X, H, HH, ERR, ERRT, FAC, X_1
c     ==================================================================

      HH = H
      IF(VARY_FIRST_ARG) THEN
         A(1,1) = (R_SCAT(X+HH, X_1) - R_SCAT(X-HH, X_1))/(2.0D0*HH)
      ELSE
         A(1,1) = (R_SCAT(X, X_1+HH) - R_SCAT(X, X_1-HH))/(2.0D0*HH)
      END IF
      ERR    = BIG
      DO I = 2, NTAB            
         HH     = HH/CON
         IF(VARY_FIRST_ARG) THEN
            A(1,I) = (R_SCAT(X+HH, X_1) - R_SCAT(X-HH, X_1))/(2.0D0*HH)
         ELSE
            A(1,I) = (R_SCAT(X, X_1+HH) - R_SCAT(X, X_1-HH))/(2.0D0*HH)
         END IF
         FAC    = CON2
         DO  J = 2, I           
            A(J,I) = (A(J-1,I)*FAC - A(J-1,I-1))/(FAC - 1.0D0)
            FAC    = CON2 * FAC
            ERRT   = DMAX1 ( DABS( A(J,I)-A(J-1,I) ),
     &               DABS( A(J,I)-A(J-1,I-1) ) )
            IF ( ERRT .LE. ERR ) THEN
               ERR = ERRT
               D_FUNC = A(J,I)
            ENDIF
         End Do                 
         IF (DABS( A(I,I) - A(I-1,I-1) ) .GE. SAFE*ERR) EXIT
      End Do                    

      RETURN                    
      END FUNCTION

C**************************************************************************
c     Here we have presented some extracted original data given by Bates 
c          (Table 1) for the validation of the implementation of his  
c          original approach and comparison to the approximations given by 
c          other authors. 
c**************************************************************************
      SUBROUTINE BATES_ORIG
c      INTEGER, PARAMETER :: N_WL = 9, N_T = 46, WUNIT = 51
      INTEGER, PARAMETER :: N_WL = 9, N_T = 52, WUNIT = 51
      INTEGER               I, IDI
      CHARACTER (LEN=80)    PATH_WRITE
      DOUBLE PRECISION, DIMENSION(N_WL):: 
     $                      WL, N_AIR, R_SEC, F_KING
      DOUBLE PRECISION      Lam, Hel, CO2_loc, N_PR, Cor_PT, 
     $                      Cons, V_mkm, C1, C2, PVC_L(4)
      DOUBLE PRECISION, DIMENSION(N_T):: 
     $                      WLT, Ft_wei, Ft_eff, NB, NPR, NE53, NE66,
     $                      RE53a, RE53b, RE53c, RE66, R_Bu, 
     $                      R_Bod, R_Bat, R_Bu_apr, R_Bod_ap
     $                      
      DATA WLT /
     $     200.0d0,  205.0d0,  210.0d0,  215.0d0,  220.0d0,  225.0d0, 
     $     230.0d0,  240.0d0,  250.0d0,  260.0d0,  270.0d0,  280.0d0, 
     $     290.0d0,  300.0d0,  310.0d0,  320.0d0,  330.0d0,  340.0d0, 
     $     350.0d0,  360.0d0,  370.0d0,  380.0d0,  390.0d0,  400.0d0, 
     $     450.0d0,  500.0d0,  550.0d0,  600.0d0,  650.0d0,  700.0d0, 
     $     750.0d0,  800.0d0,  850.0d0,  900.0d0,  950.0d0,  1000.0d0,
     $    1200.0d0, 1400.0d0, 1600.0d0, 1800.0d0, 2000.0d0,  2200.0d0,
c     $    2400.0d0, 2600.0d0, 2800.0d0, 3000.0d0 /
     $    2400.0d0, 2600.0d0, 2800.0d0, 3000.0d0, 5000.d0,  7000.d0,
     $    10000.d0, 15000.d0, 20000.d0, 30000.d0/  
c         Wavelength in [nm]
      DATA WL /210.0, 225.0, 290.0, 320.0, 360.0, 450.0, 600.0, 
     $     850.0, 1000.00/
c          (N_air - 1)*10^4 for 0C and 1013.25 mb
      DATA N_AIR /3.352d0, 3.274d0, 3.095d0, 3.051d0, 3.011d0, 
     $     2.959d0, 2.924d0, 2.902d0, 2.896d0/
c          Rayleigh croos-section [cm^2] 
      DATA R_SEC /2.842e-25, 2.046e-25, 6.553e-26, 4.285e-26,  
     $     2.601e-26, 1.027e-26, 3.167e-27, 7.739e-28, 4.022e-28/
c          King factor
      DATA F_KING /1.074, 1.068, 1.056, 1.054, 1.052, 1.050, 
     $     1.048, 1.047, 1.047/

        PATH_WRITE = '../Users_guide/Rayleigh/'
c         Comparison of refractive index to the Bates original version
        Ps = 101325.0d0           ! Pressure in [Pa]
        tC = 0.0d0
        PVC_L(:) = PVC(:)
        PVC_L(4) = 0.0003
        CO2_loc = 0.03
        Cor_PT = ( Ps*(1.d0 + Ps*(60.1d0 - 0.972d0*tC)*1.d-10) )/
     $       (96095.43d0*(1.d0 + 0.003661d0*tC))
        WRITE(*,*) ' Correction factor :', Cor_PT
c
c       Comparison original and implemented King factor
c       -----------------------------------------------
        WRITE(*,*) 
        WRITE(*,*) ' ---> Original and implemented King factor'
        DO I = 1,N_WL 
           Lam = WL(I)  
           R4(1) = Refr_N2(Lam)-1.d0  ;  R4(2) = Refr_O2(Lam) -1.d0
           R4(3) = Refr_Ar(Lam)-1.d0  ;  R4(4) = Refr_CO2(Lam)-1.d0
           Ki(1) = King_N2(Lam) ; Ki(2) = King_O2(Lam)
           Ki(3) = King_Ar(Lam) ; Ki(4) = King_CO2(Lam)
           S1 = SUM( PVC_L(:)*R4(:)**2*Ki(:) )
           S2 = SUM( PVC_L(:)*R4(:) )
c          ... Comparison of weighted King factors
           C1 = S1/S2**2
           Hel = F_KING(I)
           WRITE(*,'(1x,f7.2,2(1x,f6.3),1x,4(2x,f7.4))') 
     $          WL(I), Hel, C1, (Hel-C1)/Hel*100.
        ENDDO
c
c       Comparison of the effective and weighted King factor
c       ----------------------------------------------------
        OPEN(WUNIT, FILE=TRIM(PATH_WRITE)//'King_comp.dat-20')
        WRITE(WUNIT,'(A)') '; ---> Comparison of '//
     $       ' effective and weighted King factor for 0C'
        WRITE(WUNIT,'(A)') ';   column 1 - wavelength [nm]'
        WRITE(WUNIT,'(A)') ';   column 2 - effective King'//
     $       ' factor for air'
        WRITE(WUNIT,'(A)') ';   column 3 - relative difference [%] '//
     $       '(effective-weighted)/effective'
        DO I = 1,N_T
           Lam = WLT(I) 
           R4(1) = Refr_N2(Lam)-1.d0  ;  R4(2) = Refr_O2(Lam) -1.d0
           R4(3) = Refr_Ar(Lam)-1.d0  ;  R4(4) = Refr_CO2(Lam)-1.d0
           Ki(1) = King_N2(Lam) ; Ki(2) = King_O2(Lam)
           Ki(3) = King_Ar(Lam) ; Ki(4) = King_CO2(Lam)
           Ft_wei(I) = SUM( PVC_L(:)*Ki(:) )/SUM( PVC_L(:) )
           S1 = SUM( PVC_L(:)*R4(:)**2*Ki(:) )
           S2 = SUM( PVC_L(:)*R4(:) )
c          ... Comparison of weighted King factors
           Ft_eff(I) = S1/S2**2
           Hel = Ft_eff(I)
           WRITE(WUNIT,'(1x,f7.2,1(1x,f6.3),1x,4(2x,f7.4))') 
     $          WLT(I), Hel, (Hel - Ft_wei(I))/Hel*100.
        ENDDO
        CLOSE(WUNIT)
c
c       Comparison original and implemented air refractive index
c       --------------------------------------------------------
        WRITE(*,*) 
        WRITE(*,*) ' ---> Original and implemented'//
     $       ' air refractive index 0C, 10013.25 mb'
        DO I = 1,N_WL 
           Lam = WL(I)  
           R4(1) = Refr_N2(Lam)-1.d0  ;  R4(2) = Refr_O2(Lam) -1.d0
           R4(3) = Refr_Ar(Lam)-1.d0  ;  R4(4) = Refr_CO2(Lam)-1.d0
c          ... Bates as implemented in SCIATRAN
           C1 = SUM( PVC_L(:)*R4(:) )*1.d4
           Hel = N_AIR(I)
           WRITE(*,'(1x,f7.2,2(1x,f6.3),1x,4(2x,f7.4))') 
     $          WL(I), Hel, C1, (Hel - C1)/Hel*100.
        ENDDO
c
c       Comparison of air refractive index calculated according 
c         to Edlwn 53, Edlen 66, and Peck and Reeder 74
c       -------------------------------------------------------
        OPEN(WUNIT, FILE=TRIM(PATH_WRITE)//'Refr_comp.dat-20')
        WRITE(WUNIT,'(A)') '; ---> Air refractive index comparison'//
     $       ' at 15C, 10013.25 mb'
        WRITE(WUNIT,'(A)') ';      according to Bates (B), Edlen 53 '//
     $       '(E53), Edlen 66 (E66),'
        WRITE(WUNIT,'(A)') ';      and Peck & Reeder 72 (PR)'
        WRITE(WUNIT,'(A)') ';   column 1 - wavelength [nm]'
        WRITE(WUNIT,'(A)') ';   column 2 - (n-1)*10^4 from Bates'//
     $       ', where n is refractive index '
        WRITE(WUNIT,'(A)') ';   column 3 - relative difference [%] '//
     $       '(B - E53)/B'
        WRITE(WUNIT,'(A)') ';   column 4 - relative difference [%] '//
     $       '(B - E66)/B'
        WRITE(WUNIT,'(A)') ';   column 5 - relative difference [%] '//
     $       '(B - PR)/B'
        DO I = 1,N_T 
           Lam = WLT(I)  
           R4(1) = Refr_N2(Lam)-1.d0  ;  R4(2) = Refr_O2(Lam) -1.d0
           R4(3) = Refr_Ar(Lam)-1.d0  ;  R4(4) = Refr_CO2(Lam)-1.d0
c          ... Bates as implemented in SCIATRAN
           NB(I) = SUM( PVC_L(:)*R4(:) )*1.d4/Cor_PT
c          ... Multiplier 1.d4 is used for comparison with Bates data
           NE53(I) = (Refr_air_E53(Lam) - 1.d0)*1.d4 
           NE66(I) = (Refr_air_E66(Lam) - 1.d0)*1.d4 
           NPR (I) = (Refr_air_PR(Lam)  - 1.d0)*1.d4 
        ENDDO
        DO I = 1,N_T
           Hel = NB(I)
           WRITE(WUNIT,'(1x,f7.2,1(1x,f6.3),1x,4(2x,f7.4))') 
     $          WLT(I), Hel, (Hel - NE53(I))/Hel*100.,
     $          (Hel - NE66(I))/Hel*100., (Hel - NPR (I))/Hel*100.
        ENDDO
        CLOSE(WUNIT)
c
c       Comparison original and implemented scattering cross-section
c       ----------------------------------------------------------------
        WRITE(*,*) 
        WRITE(*,*) ' ---> Rayleigh scattering cross-section'
        WRITE(*,*) '   Wl        Bates        B/B     '
c          ... Comparison of Rayleigh scattering cross-section
c          ... for 15C, and 1013.25mb
        DO I = 1,N_WL
           V_mkm = WL(I)*1.d-03 ; Lam = WL(I)  ;  Hel = R_sec(I)
c          ... according to Bates implemented in SCIATRAN
           R4(1) = Refr_N2(Lam)-1.d0  ;  R4(2) = Refr_O2(Lam) -1.d0
           R4(3) = Refr_Ar(Lam)-1.d0  ;  R4(4) = Refr_CO2(Lam)-1.d0
           Ki(1) = King_N2(Lam) ; Ki(2) = King_O2(Lam)
           Ki(3) = King_Ar(Lam) ; Ki(4) = King_CO2(Lam)
           Cons = 32.d0*Pie**3/3.d0/(Lam*1.d-07)**4/Ln_0C**2
           C1 = Cons*SUM( PVC_L(:)*R4(:)**2*Ki(:) )
           WRITE(*,'(1x,f7.2,2(1x,e12.4),1x,6(2x,f7.4))') 
     $          WL(I), Hel, C1, (Hel - C1)/Hel*100.
        ENDDO
c
c       Comparison of scattering cross-section
c       --------------------------------------
        OPEN(WUNIT, FILE=TRIM(PATH_WRITE)//'Scat_comp.dat-20')
        WRITE(WUNIT,'(A)') '; ---> Rayleigh scattering cross-section'
        WRITE(WUNIT,'(A)') ';      according to Bates (B), Edlen 53 '//
     $       '(E53), Edlen 66 (E66),'
        WRITE(WUNIT,'(A)') 
     $       ';      Bucholtz 95 (Bu), and Bodhaine et.al 99 (Bo)'
        WRITE(WUNIT,'(A)') ';   column 1 - wavelength [nm]'
        WRITE(WUNIT,'(A)') ';   column 2 - cross-section [cm^2]'//
     $       ' from Bates'
        WRITE(WUNIT,'(A)') ';   column 3 - relative difference [%] '//
     $       '(B - E53)/B'
        WRITE(WUNIT,'(A)') ';   column 4 - relative difference [%] '//
     $       '(B - E66)/B'
        WRITE(WUNIT,'(A)') ';   column 5 - relative difference [%] '//
     $       '(B - Bu)/B'
        WRITE(WUNIT,'(A)') ';   column 6 - relative difference [%] '//
     $       '(B - Bo)/B'
c          ... Comparison of Rayleigh scattering cross-section
c          ... for 15C, and 1013.25mb
        DO I = 1,N_T
           V_mkm = WLT(I)*1.d-03 ; Lam = WLT(I) 
           R4(1) = Refr_N2(Lam)-1.d0  ;  R4(2) = Refr_O2(Lam) -1.d0
           R4(3) = Refr_Ar(Lam)-1.d0  ;  R4(4) = Refr_CO2(Lam)-1.d0
           Ki(1) = King_N2(Lam) ; Ki(2) = King_O2(Lam)
           Ki(3) = King_Ar(Lam) ; Ki(4) = King_CO2(Lam)
           C1 = SUM( PVC_L(:)*Ki(:) )/SUM( PVC_L(:) )

           S2 = 24.d0*Pie**3/(Lam*1.d-07)**4/Ln_15C**2
c          ... Edlen-53
           S1 = Refr_air_E53(Lam)
           RE53a(I) = S2*( (S1**2 - 1.d0)/(S1**2 + 2.d0) )**2*C1
              C2 = 32.d0*Pie**3/3.d0/(Lam*1.d-07)**4/Ln_15C**2
              RE53b(I) = C2*(S1 - 1.d0)**2*C1
              C2 = 8.d0*Pie**3/3.d0/(Lam*1.d-07)**4/Ln_15C**2
              RE53c(I) = C2*(S1**2 - 1.d0)**2*C1

c          ... Edlen-66
           S1 = Refr_air_E66(Lam)
           RE66(I) = S2*( (S1**2 - 1.d0)/(S1**2 + 2.d0) )**2*C1

c          ... Bucholtz approximation
           IDI = 1  ;  IF( V_mkm > 0.5d0 ) IDI = 2         
           R_Bu_apr(I) = HF0(IDI)/
     $          V_mkm**(HF1(IDI) + HF2(IDI)*V_mkm + HF3(IDI)/V_mkm)
c          ... Bucholtz direct
           N_PR = Refr_air_PR(Lam)
           R_Bu(I) = S2*(N_PR**2 - 1.d0)**2/(N_PR**2 + 2.d0)**2*C1

c          ... Bodhaine approximation
           VV = V_mkm*V_mkm
           R_Bod_ap(I) = 
     $          (Ba + Bb/VV + Bc*VV)/(1.0d0 + Bd/VV + Be*VV)*1.0d-28
c          ... Bodhaine direct
           N_PR = Refr_air_PR5(Lam)
           R_bod(I) = S2*(N_PR**2 - 1.d0)**2/(N_PR**2 + 2.d0)**2*C1

c          ... according to Bates implemented in SCIATRAN
           Cons = 32.d0*Pie**3/3.d0/(Lam*1.d-07)**4/Ln_15C**2
           R_Bat(I) = Cons*SUM( PVC_L(:)*R4(:)**2*Ki(:) )
           R_Bat(I) = R_Bat(I)/Cor_PT/Cor_PT
        ENDDO
c       ... Print results ...
        DO I = 1,N_T
           Hel = R_Bat(I)
           WRITE(WUNIT,'(1x,f7.2,1(1x,e12.4),1x,6(2x,f7.4))') 
     $          WLT(I), Hel, 
     $          (Hel - RE53a(I) )/Hel*100.,
     $          (Hel - RE66(I) )/Hel*100.,
     $          (Hel - R_Bu(I) )/Hel*100.,
     $          (Hel - R_Bod(I))/Hel*100.
        ENDDO
        CLOSE(WUNIT)
c
c       Comparison of Lorentz-Lorenz factor impct
c       -----------------------------------------
        OPEN(WUNIT, FILE=TRIM(PATH_WRITE)//'Lor_comp.dat-20')
        WRITE(WUNIT,'(A)') '; ---> Rayleigh scattering cross-section'
        WRITE(WUNIT,'(A)') ';      according to Edlen 53 (a), '//
     $       ' Edlen 53 (b),'
        WRITE(WUNIT,'(A)') ';      Edlen 53 (c)'
        WRITE(WUNIT,'(A)') ';   column 1 - wavelength [nm]'
        WRITE(WUNIT,'(A)') ';   column 2 - cross-section [cm^2]'//
     $       ' from Edlen 53 (a)'
        WRITE(WUNIT,'(A)') ';   column 3 - relative difference [%] '//
     $       '(a - b)/a'
        WRITE(WUNIT,'(A)') ';   column 4 - relative difference [%] '//
     $       '(a - c)/a'
        DO I = 1,N_T
           Hel = RE53a(I)
           WRITE(WUNIT,'(1x,f7.2,1(1x,e12.4),1x,6(2x,f7.4))') 
     $          WLT(I), Hel, (Hel - RE53b(I) )/Hel*100.,
     $          (Hel - RE53c(I) )/Hel*100.
        ENDDO
        CLOSE(WUNIT)

c     
c      Comparison of approximated expression for cross-section
c      -------------------------------------------------------
        OPEN(WUNIT, FILE=TRIM(PATH_WRITE)//'Apr_comp.dat-20')
        WRITE(WUNIT,'(A)') '; ---> Comparison approximation of'//
     $       ' Bucholtz and Bodhaine'
        WRITE(WUNIT,'(A)') ';   column 1 - wavelength [nm]'
        WRITE(WUNIT,'(A)') ';   column 2 - cross-section [cm^2]'//
     $       ' from Bucholtz'
        WRITE(WUNIT,'(A)') ';   column 3 - relative difference [%] '//
     $       'for Bucholtz approximation'
        WRITE(WUNIT,'(A)') ';   column 3 - relative difference [%] '//
     $       'for Bodhaine et. al approximation'
        DO I = 1,N_T           
           WRITE(WUNIT,'(1x,f7.2,1(1x,e12.4),2x,f7.4,2x,E12.4)') 
     $          WLT(I), R_Bu(I), 
     $          (R_Bu(I) - R_Bu_apr(I))/R_Bu(I)*100.,
     $          (R_Bod(I) - R_Bod_ap(I))/R_Bod(I)*100.
        ENDDO
        CLOSE(WUNIT)
        STOP '   Stop in test routine "BATES_ORIG"'
      END SUBROUTINE BATES_ORIG
c***********************************************************************

      END
c***********************************************************************
