!***************************************************************************
!
!	SCIATRAN - Version 4.5 - Last modification: 26 Oct 2021
!
!	Copyright (C) - Institute of Remote Sensing,
!			University of Bremen, Germany
!
!***************************************************************************
SUBROUTINE READ_HITRAN_CIA (NSPEC, WL, *)

!     Reads collision induced absorption cross sections fron HITRAN data base
  USE CIA_VARS,  ONLY: CIA_DIR, CIA_INFO
  USE LOG_PARS,  ONLY: LOG_LEVEL, LUNIT, EU, N_SSO
  USE TYPE_VARS, ONLY: KND

  INTEGER, INTENT(IN) :: NSPEC
  REAL (KND), INTENT(IN) :: WL(NSPEC)

! Local variables:
  CHARACTER (LEN = *), PARAMETER :: R_NAME = 'READ_HITRAN_CIA'
  INTEGER   MAX_CIA_TEMPS, JJ, I_P, I_B, I_T, I_S, BAND_N, NN, CU
  INTEGER   CIA_B_IDX(NSPEC), MAX_T(Size(CIA_INFO))
  LOGICAL, ALLOCATABLE :: T_PRESENT(:,:)
  CHARACTER (LEN = :), ALLOCATABLE :: FILE_NAME
  CHARACTER (LEN = 10)  STRING
  CHARACTER (LEN = 50)  TRC_STR
  REAL (KND) WL_S, WL_E, WN_S, WN_E, BAND_S, BAND_E
  REAL (KND) CIA_TMP(NSPEC)
  REAL (KND), ALLOCATABLE :: CIA_CS(:,:), CIA_T(:,:), READ_CS(:), READ_WL(:) 
  REAL (KND) TTT(1), YYY(1)

  CALL ST_TRACK (R_NAME, 1, 'I')
  CALL DEALLOCATE_ARRAYS

  ForAll(I_P = 1:Size(CIA_INFO)) MAX_T(I_P) = MaxVal(CIA_INFO(I_P)%N_CIA_T)
  MAX_CIA_TEMPS = MAXVAL(MAX_T)
  ALLOCATE(CIA_CS(NSPEC,MAX_CIA_TEMPS),  T_PRESENT(NSPEC,MAX_CIA_TEMPS), & !A
       CIA_T(MAXVAL(CIA_INFO%N_CIA_B), MAX_CIA_TEMPS)) !A

  WL_S = MINVAL(WL) ;  WL_E = MAXVAL(WL)
  Pairs: DO I_P = 1, Size(CIA_INFO) ! Collision pairs
     IF(.NOT.CIA_INFO(I_P)%Flg) CYCLE
     T_PRESENT = .False. ; CIA_T = 0.0D0 ;  CIA_CS = 0.0D0 ; CIA_B_IDX = 0
     FILE_NAME = CIA_DIR//'/'//TRIM(CIA_INFO(I_P)%Name)//'.cia'
     IF(LOG_LEVEL(1) >= N_SSO) WRITE(LUNIT,*) ' *** Reading: '//TRIM(FILE_NAME) 
     OPEN(NewUnit=CU, FILE = FILE_NAME, STATUS ='OLD')
     DO I_B = 1, CIA_INFO(I_P)%N_CIA_B ! Spectral bands
        DO I_T = 1, CIA_INFO(I_P)%N_CIA_T(I_B) ! Temperatures
           READ(CU,*) STRING, WN_S, WN_E, BAND_N, CIA_T(I_B,I_T)
           !write(6,*) STRING, WN_S, WN_E, BAND_N, CIA_T(I_B,I_T)
           BAND_S = 1.0D+7/WN_E ;  BAND_E = 1.0D+7/WN_S
           CreateArrays: Block
             If(Allocated(READ_WL)) Then
                IF(BAND_N <= Size(READ_WL)) Exit CreateArrays
                DeAllocate(READ_WL, READ_CS)
             End If
             Allocate(READ_WL(BAND_N), READ_CS(BAND_N))
           End Block CreateArrays
           IF(BAND_E < WL_S .OR. BAND_S > WL_E) THEN
              DO JJ = 1, BAND_N
                 READ(CU,*) 
              END DO
              CYCLE
           END IF
           DO JJ = 1, BAND_N
              READ(CU,*) READ_WL(JJ), READ_CS(JJ)
           END DO
           READ_WL(1:BAND_N) = 1.0D+7/READ_WL(1:BAND_N) !WN to WL
           CALL AL_INTERP (BAND_N, READ_WL(1:BAND_N), READ_CS(1:BAND_N), &
                NSPEC, WL(1:NSPEC), CIA_TMP, .TRUE., *880)
           WHERE(WL(1:NSPEC) >= BAND_S .AND. WL(1:NSPEC) <= BAND_E) 
              CIA_CS(1:NSPEC,I_T) = CIA_TMP(:)
              T_PRESENT(1:NSPEC,I_T) = .TRUE.
              CIA_B_IDX(1:NSPEC) = I_B
           END WHERE
        END DO ! Temperatures
     END DO ! Spectral bands
     IF (ALL(CIA_B_IDX(1:NSPEC) == 0)) Then
        CIA_INFO(I_P)%Flg = .FALSE.
        Cycle
     End If
     Allocate(CIA_INFO(I_P)%DB(NSPEC))
     DO I_S = 1, NSPEC
        Associate(CIA_DB => CIA_INFO(I_P)%DB(I_S))
          NN = COUNT(T_PRESENT(I_S,:))
          IF(NN == 0) CYCLE
          Allocate(CIA_DB%CS(NN), CIA_DB%TEMP(NN)) !A
          CIA_DB%CS = PACK(CIA_CS(I_S,:),T_PRESENT(I_S,:))
          I_B = CIA_B_IDX(I_S)
          CIA_DB%TEMP = PACK(CIA_T(I_B,:),T_PRESENT(I_S,:))
        End Associate
     End DO
  END DO Pairs

  CALL ST_TRACK (R_NAME, 1, 'O')
  RETURN
           
202 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Too many temperatures have been '// &
         'read from the CIA data base')
  RETURN 1

880 WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)
  RETURN 1

CONTAINS
!***********************************************************************
  SUBROUTINE DEALLOCATE_ARRAYS
    INTEGER II, JJ
    
    DO II = 1, Size(CIA_INFO, 1)
       If(.Not.Allocated(CIA_INFO(II)%DB)) Cycle
       DO JJ = 1, Size(CIA_INFO(II)%DB)
          IF(ALLOCATED(CIA_INFO(II)%DB(JJ)%CS)) &
               DEALLOCATE(CIA_INFO(II)%DB(JJ)%CS, &
               CIA_INFO(II)%DB(JJ)%TEMP)
       END DO
    END DO
    
    RETURN
  END SUBROUTINE DEALLOCATE_ARRAYS
!***********************************************************************
END SUBROUTINE READ_HITRAN_CIA
