!**************************************************************************
!
!	SCIATRAN - Version 4.5 - Last modification: 19 Oct 2021
!
!	Copyright (C) - Institute of Remote Sensing,
!			University of Bremen, Germany
!
!****************************************************************************
SUBROUTINE Read_HITRAN_Pars(HITRAN_DIR, *)
  ! Called by "gt_iface.f"
  
  USE INFO,      ONLY: TG_INFO, RET_INFO
  USE LOG_PARS,  ONLY: EU, LUNIT, LOG_LEVEL, N_SSO
  USE LBL,       ONLY: C, MAX_ISO, N_HT_MOLS, LBL_WING_CUTOFF, SEG_WL, &
                       NSEGS_LBL, TDat, StEm_MaxWn, HT_C2
  USE TOOLS,     ONLY: FIND_IDX
  USE TYPES,     ONLY: KND

  use cs_table !mhilker

  CHARACTER (LEN=*), PARAMETER :: R_NAME = 'Read_HITRAN_Pars'
  CHARACTER (LEN=*)   HITRAN_DIR
  Character(Len=:), Allocatable :: FILENAME
  CHARACTER (LEN=200) STRING 
  CHARACTER (LEN=4)   MOL_NAME
  INTEGER   I, L, K, II, LL, I_MOL, JJ, IT, CU, Iso, NLines, N_ISO(N_HT_MOLS) 
  Integer, parameter :: NPars = 7
  REAL (KND) IsoMass(N_HT_MOLS,MAX_ISO), WN_MIN(NSEGS_LBL), WN_MAX(NSEGS_LBL)
  Real (Knd), Target :: Pars(NPars)
  REAL (KND) POS_OLD, XXX, XXX_1, XXX_2, SQRT_2KNa_GtoKG_divbyC
  REAL (KND), POINTER :: POS
  Type LineParsList
     Integer Iso
     Real (Knd), Allocatable:: Pars(:)
     TYPE (LineParsList), Pointer :: NextLine
  End Type LineParsList
  Type (LineParsList), Pointer :: FirstLine => Null(), NextLine, LineList => Null()
  Type ErrList
     Integer Iso
     TYPE (ErrList), Pointer :: NextElem
  End Type ErrList
  Type (ErrList), Pointer :: FirstErr => Null(), NextErr, ElemErr => Null()
  !==================================================================
  
  CALL ST_TRACK (R_NAME, 1, 'I')
  
  ! Initialization for BD_TIPS
  I = 0
  DO IT = 1,20
     I = I+1
     TDAT(I) = IT*1.
  ENDDO
  DO IT = 22,9000,2
     I = I+1
     TDAT(I) = IT*1.
  ENDDO

  IsoMass = 0.0D0
  
  IF(NSEGS_LBL == 0) THEN
     CALL WRITE_MESSAGE(R_NAME, 'WARNING', 'No line absorber '&
          //'windows in the selected selected spectral intervals')
     RETURN 1
  END IF

  ! Minimum and maximum wavenumbers for spectral segments
  DO I = 1, NSEGS_LBL
     WN_MIN(I) = 1.0D7/SEG_WL(2,I) - LBL_WING_CUTOFF
     WN_MAX(I) = 1.0D7/SEG_WL(1,I) + LBL_WING_CUTOFF
  END DO

  ! Read isotopic parameters file
  ! =============================
  ! Contents: Molecular weights [gramm/mole], e.g. O2(16,16): 32.0
  FILENAME = TRIM(HITRAN_DIR)//'/molparam.txt'
  IF(LOG_LEVEL(1) >= N_SSO) WRITE(LUNIT,*) ' *** Reading: '//FILENAME
  OPEN(NewUnit=CU, FILE=FILENAME, STATUS='OLD')

  JJ = 0
  READ(CU,'(A)') STRING
  DO
     READ(CU,'(A)',END=300) STRING
     STRING = ADJUSTL(STRING)
     IF(IACHAR(STRING(1:1)) < 48 .OR. IACHAR(STRING(1:1)) > 57) THEN
        IF(JJ == 1) GOTO 132
        LL = LEN_TRIM(STRING)
        DO L = 1, LL
           IF(IACHAR(STRING(L:L)) == 40) EXIT
        END DO
        READ(STRING(L + 1:LL - 1),*) I_MOL
        JJ = 1
     ELSE
        IF(JJ > MAX_ISO) GOTO 133
        IF(I_MOL > N_HT_MOLS) GOTO 134
        IF(JJ == 0) GOTO 134
        READ(STRING,*) II, XXX, XXX_1, XXX_2, IsoMass(I_MOL,JJ)
        N_ISO(I_MOL) = JJ
        JJ = JJ + 1
     END IF
  END DO
300 CONTINUE
  CLOSE(CU)

  ! Read spectroscopic parameter file
  ! =================================
  Molecules: DO II = 1, SIZE(TG_INFO)

     IF(.NOT.TG_INFO(II)%FM_FLG)      CYCLE
     IF(TG_INFO(II)%M_TYPE /= 'line'.AND. &
          TG_INFO(II)%M_TYPE /= 'mixed') CYCLE
     TG_INFO(II)%NIso = N_ISO(TG_INFO(II)%IdxHT)
     
     !----------------------------------------------------mhilker
     if(associated(table_lists(II)%table)) cycle
     !----------------------------------------------------mhilker

     FILENAME = TRIM(HITRAN_DIR)//'/linepars_'//TRIM(TG_INFO(II)%NAME)//'.dat'
     IF(LOG_LEVEL(1) >= N_SSO) WRITE(LUNIT,*) ' *** Reading: '//FILENAME
     OPEN(NewUnit=CU, FILE=FILENAME, STATUS='OLD')
     !Skip header
     STRING(1:1) = '#'
     DO WHILE (STRING(1:1) == ';' .or. STRING(1:1) == '#' )
        READ(CU,*) STRING
     END DO
     BACKSPACE (CU)

     !Read line parameters
     !Assumption: line parameters ordered by wavenumber (incr.)
     !Contents:
     !    - MOL_NAME: Molecule name (e.g. 'O2  ')
     !    - ISO:  Hitran isotope index (1 (most abundant) .. 7 (less abundant))
     !    - Pars: temporary array of line parameters
     !         1) Position [cm-1]
     !         2) Intensity [cm2/molec*cm-1] @ 296 K
     !            (already istotope abundance weighted!)
     !         3) Air-broadened halfwidth (HWHM) in cm-1/atm @ 296 K
     !         4) Lower state energy [cm-1]
     !         5) Coeff. of T-depend. of air-broadened halfwidth [1]
     !         6) Self-broadened halfwidth (HWHM) in cm-1/atm @ 296 K (not used)
     !         7) Line shift
     !    - The element Pars(6) is replaced by
     !      the Doppler width (1/e-HM [cm-1]) divided by the square root of
     !      the temperature:
     !pos/c sqrt(2*k*T/m)/sqrt(T) = pos/c sqrt(2*k/m) = pos/c sqrt(2*k*Na*1E+3/M)
     !      (c: speed of light, m: molecular mass in kg = M*1E-3/Na,
     !       M : isotop mass in g/mol,
     !       Na = 6.022045D+23: Avogadro's constant,
     !       k = 1.380662D-23: Boltzmann's constant)
     !---------------------------------------------------------
     SQRT_2KNa_GtoKG_divbyC = SQRT(2.0D+3*1.380662D0*6.022045D0)/&
          299.792458D+6 ! C-1_SQRT_2KNa_GtoKG = sqrt(2*k*Na*1E+3)/c
     POS_OLD = 0.0D0
     NLines = 0
     Lines: Do
        Read(CU,*, END=200) MOL_NAME, Iso, Pars(:)
!        Read(CU,*, END=200, ERR=120) MOL_NAME, Iso, Pars(:)
!        If(.False.) Then
!120        Continue
!           backspace(CU)
!           Read(CU,"(A200)") String
!           Write(6,*) String
!           Goto 121
        !        End If
        If(Iso < TG_INFO(II)%IsoMin .or. Iso > TG_INFO(II)%IsoMax) Cycle
        POS => Pars(1)
        IF(POS < POS_OLD) GOTO 124
        POS_OLD = POS
        IF(POS < MINVAL(WN_MIN(:))) CYCLE
        IF(POS > MAXVAL(WN_MAX(:))) EXIT
        Segments: DO I = 1, NSEGS_LBL
           IF(POS < WN_MIN(I).OR.POS > WN_MAX(I)) CYCLE
           IF (Iso < 1) GOTO 125
           IF (Iso > TG_INFO(II)%NIso) Call WrongIso
           
           ! Self-broadened half-width is not used in the current program
           ! version. The corresponding sub-array is used to store the Doppler
           ! width Doppler width is divided by the square root of the
           ! temperature!
           Pars(6) = POS*SQRT_2KNa_GtoKG_divbyC/Sqrt(IsoMass(TG_INFO(II)%IdxHT,Iso))
           If (.Not.Associated(LineList)) Then
              Allocate(FirstLine)
              LineList => FirstLine
              NLines = 1
           Else
              Allocate(LineList%NextLine)
              LineList => LineList%NextLine
              NLines = NLines + 1
           End If
           
           Allocate(LineList%Pars(Npars))
           LineList%Pars = Pars
           LineList%Iso = Iso
           EXIT
        End Do Segments
        
        Cycle
200     CALL WRITE_MESSAGE(R_NAME, 'WARNING', 'Line parameter'//' file "'//FILENAME&
             //'" does not cover '//' the selected spectral range')
        WRITE(EU,*) '    Selected minimum wavelength:',&
             1.0D7/MAXVAL(WN_MAX(:))
        WRITE(EU,*) '    Start of data file: ', 1.0D7/POS
        WRITE(EU,*)
        EXIT
     End Do Lines
     CLOSE(CU)
     LinesPresent: If (NLines /= 0) Then
        ! Strore line parameters and clean the reading list
        If(Allocated(TG_INFO(II)%LinePars)) DeAllocate(TG_INFO(II)%LinePars)
        Allocate (TG_INFO(II)%LinePars(NLines))
        Nullify(LineList%NextLine)
        LineList => FirstLine
        JJ = 0
        Do While (Associated(LineList))
           JJ = JJ + 1
           If(LineList%Pars(4) < 0.0D0) LineList%Pars(4) = 0.0D0
           If(Any(LineList%Pars((/1,2,3,6/)) < 0.0D0)) GOTO 126
           Associate(LP => TG_INFO(II)%LinePars(JJ))
             LP%Iso   = LineList%Iso
             LP%Pos   = LineList%Pars(1)
             LP%Int   = LineList%Pars(2)
             ! Accounting for the stimulated emission at small wavenumbers
             If (LP%Pos > StEm_MaxWn) LP%Int = LP%Int/&
                  (1.0D0 - Exp(-HT_C2*LP%Pos/296.0D0))
             LP%LorW  = LineList%Pars(3)
             LP%LSE   = LineList%Pars(4)
             LP%TExp  = LineList%Pars(5)
             LP%DopW  = LineList%Pars(6)
             LP%Shift = LineList%Pars(7)
           End Associate
           NextLine => LineList%NextLine
           DEALLOCATE (LineList%Pars)
           DEALLOCATE (LineList) ; NULLIFY (LineList)
           LineList => NextLine
        End Do
    Else
        ! The molecules having no absorption in the selected spectral
        ! range are swithed off
        CALL WRITE_MESSAGE(R_NAME, 'WARNING', 'No lines of '//&
             MOL_NAME// ' in selected spectral intervals:')
        WRITE(EU,'(500(5X,F6.1,A,F6.1),$)') &
             (1.0D7/WN_MAX(I), ' - ', 1.0D7/WN_MIN(I), I = 1, NSEGS_LBL)
        WRITE(EU,*)

        If(TG_INFO(II)%WF_FLG) I = FIND_IDX(TG_INFO(II)%NAME,RET_INFO%NAME) 
        SELECT CASE (TG_INFO(II)%M_TYPE)
        CASE('line') 
           TG_INFO(II)%FM_FLG = .FALSE.
           If(TG_INFO(II)%WF_FLG)  Then
              TG_INFO(II)%WF_FLG = .FALSE.; RET_INFO(I)%STAT   = .FALSE.
           End If
        CASE('mixed')
           IF(TG_INFO(II)%MAX_WL_CS < 1.0D7/MAXVAL(WN_MAX(:))) THEN
              TG_INFO(II)%FM_FLG = .FALSE.
              If(TG_INFO(II)%WF_FLG)  Then
                 TG_INFO(II)%WF_FLG = .FALSE. ; RET_INFO(I)%STAT   = .FALSE.
              End If
           ENDIF
        END SELECT

     ENDIF LinesPresent

     ! Clean error handling list
     ElemErr => FirstErr
     If(Associated(FirstErr)) Nullify(FirstErr)
     Do While (Associated(ElemErr))
        NextErr => ElemErr%NextElem
        Deallocate (ElemErr)
        Nullify (ElemErr)
        ElemErr => NextErr
     End Do
     
  END DO Molecules

  ! Show number of lines read
  ! -------------------------
  IF(LOG_LEVEL(1) >= N_SSO)  THEN
     WRITE(LUNIT,*)
     WRITE(LUNIT,'(A)') '  *** Number of lines '//&
          'in wavelength intervals (including wing cut-off [nm])'
     DO I = 1, NSEGS_LBL
        WRITE(LUNIT,'(2X, F6.1,A,F6.1,A,F6.1,A,F6.1,A)') &
             SEG_WL(1,I), ' - ', SEG_WL(2,I), ' (',&
             1.0D7/WN_MAX(I), ' - ', 1.0D7/WN_MIN(I), '):'
        DO II = 1, SIZE(TG_INFO)
           If(.Not.Allocated(TG_INFO(II)%LinePars)) Cycle
           NLines = Count(TG_INFO(II)%LinePars(:)%Pos >= WN_MIN(I) &
                .And.TG_INFO(II)%LinePars(:)%Pos <= WN_MAX(I))
           WRITE(LUNIT,'(A,I6)')'    '//TG_INFO(II)%NameUC//': ',  NLines
        ENDDO
     END DO
     WRITE(LUNIT,*)
  ENDIF

  ! Needed for voigt2 function in line_shapes
  ! ******************************************
  K = -16
  DO JJ = 1, 31
     K = K + 1
     C (JJ) = 0.0897935610625833D0 * EXP (-DBLE(K * K) / 9.D0)
  END DO
  
  CALL ST_TRACK (R_NAME, 1, 'O')
  RETURN
  
121 CONTINUE
  CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Reading error '//FILENAME)
  RETURN 1

124 CONTINUE
  CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Line parameters not in increasing order')
  WRITE(EU,*) '     Line position [cm-1]: ', POS
  RETURN 1

125 CONTINUE
  CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Line parameter file: wrong isotope index')
  WRITE(EU,*) '     Position: ', POS
  WRITE(EU,*) '     Isotope index: ', Iso
  RETURN 1

126 CONTINUE
  CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Line parameter file: negative line parameters')
  RETURN 1
  
132 CONTINUE
  Do II = 1, SIZE(TG_INFO)
     if(TG_INFO(II)%IdxHT == I_MOL) Exit
  End Do
  CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Error in the HITRAN '//&
       'isotop parameters (no isotopes found for '//TG_INFO(II)%NameUC//')')
  RETURN 1

133 CONTINUE
  Do II = 1, SIZE(TG_INFO)
     if(TG_INFO(II)%IdxHT == I_MOL) Exit
  End Do
  CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Error in HITRAN '//&
       'isotop parameters (the number of isotopes for '//TG_INFO(II)%NameUC&
       //'exeeds maximum)')
  RETURN 1

134 CONTINUE
  CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Wrong format of the '//&
       'HITRAN isotop parameter file')
  RETURN 1

Contains
!********************************************************************
  Subroutine WrongIso
    ! Check if this error message is already present
    If (.Not.Associated(ElemErr)) THEN
       Allocate(FirstErr)
       ElemErr => FirstErr
    Else
       ElemErr => FirstErr
       ErrMsg: Do
          If(ElemErr%Iso == Iso) Return
          If(.Not.Associated(ElemErr%NextElem)) Then
             Allocate(ElemErr%NextElem)
             ElemErr => ElemErr%NextElem
             Exit ErrMsg
          End If
          ElemErr => ElemErr%NextElem
       End Do ErrMsg
    End IF

    ElemErr%Iso = Iso
    Nullify(ElemErr%NextElem)
    CALL WRITE_MESSAGE(R_NAME, 'WARNING', 'No auxilary '//&
         'information is available, isotop skipped: ')
    Write(EU,*) "Molecule name: "//TRIM(TG_INFO(II)%NAME)
    Write(EU,*) "Isotope number:", Iso
      
    Return
  End Subroutine WrongIso

!********************************************************************
END SUBROUTINE Read_HITRAN_Pars
!********************************************************************
