!***************************************************************************
!
!	SCIATRAN - Version 4.7 - Last modification: 18 Aug 2023
!
!	Copyright (C) - Institute of Remote Sensing,
!			University of Bremen, Germany
!
!***************************************************************************
SUBROUTINE READ_RTM_WF (*)
! ... called by "gt_fileread.f90"

  USE ABSORBERS,  ONLY: N_ST_MOL, AMF_IDX
  USE AEROSOLS,   ONLY: Use_der, N_Dwr, Plus_mi
  USE CLOUDS,     ONLY: DO_CLOUD
  USE CLD_INP,    ONLY: Deriv_Phc, Phm_const, MLCS
  USE FLU_LAND,   ONLY: FLU_L
  USE INFO,       ONLY: TG_INFO, AER_INFO, PAR_INFO, CLD_INFO
  USE INTERN_DIMS,ONLY: Nvs
  USE LOG_PARS,   ONLY: Work_Dir, EU
  USE RTM_OUTPUT, ONLY: IND_NUM_WF, WF_UNIT, MIN_WF_Z, MAX_WF_Z, OWF_FLG_ARR
  USE ST_INPUT,   ONLY: BAND_NUM
  USE ST_VARS,    ONLY: RTM_MODE, RTM_CORE, OWF_Chl, OWF_Ks, OWF_Kl, OWF_Ray, &
                        OWF_Use, Gauss_Field, Lo_der, Vec_Sca, NS_wf, Nc_der, &
                        RTM_TYPE, Ainc, DO_AEROSOL, OWF_FLAG, SURF_TYPE, Ray_to_WF
  USE TOOLS,      ONLY: FIND_MATCHES, BLANK_OR_COMMENT, READ_INPUT, INQ_WF, &
                        SPLIT_STRING, UPPER_LOWER_CASE, CHK_STAT, CUT_STR, &
                        FIND_IDX
  USE TYPES,      ONLY: KND, DYN_CHAR
  USE WATER,      ONLY: UWT
  USE WF_NUM,     ONLY: NUM_WF, NUM_ABS, NUM_RAY, NUM_AER, NUM_PRE, NUM_CLD,&
                        NO_SOL, NO_LS, SOL_AND_LS, NUM_TEM, &
                        NUM_ORAY, NUM_Chl, NUM_OWF, SCA_or_ABS, Wat_or_Ice

  ! Local parameters
  CHARACTER (LEN = *), PARAMETER :: R_NAME = 'READ_RTM_WF'
  CHARACTER (LEN=:), ALLOCATABLE :: STR_INP, MSG
  INTEGER  II, KK, III, K
  INTEGER, PARAMETER :: N_OWF_GEN = 5
  LOGICAL  FLG, WRT_FLG
  TYPE (DYN_CHAR), ALLOCATABLE :: READ_ARR(:)
  TYPE (DYN_CHAR), ALLOCATABLE :: STR_ARR(:), STR_ARR_1(:) 
  !==================================================================
  
  CALL ST_TRACK (R_NAME, 1, 'I')

  CALL DEALLOCATE_ARRAYS ! All arrays allocated below must be 
  ! conditionally deallocated here

  WF_UNIT = 'standard'
  SELECT CASE (RTM_CORE)
  CASE ('DOM') 
     SELECT CASE (RTM_MODE)
     CASE ('wf','num_pert','block_amf')
        STR_INP = 'WF normalization'
        CALL Read_Input (STR_INP, READ_ARR, *880)
        WF_UNIT = CUT_STR(READ_ARR(1)%S)

        SELECT CASE (TRIM(WF_UNIT))
        CASE('standard', 'unified')
        CASE DEFAULT
           CALL NOT_FOUND (R_NAME, 'WF normalization', WF_UNIT, *880)
        END SELECT
     END SELECT
  END SELECT    

  SELECT CASE (RTM_MODE)
  CASE('num_pert', 'wf', 'ret')
     CALL Read_WFs ('Weighting functions: trace gases', TG_INFO%NAME, &
          TG_INFO%WF_FLG, TG_INFO%WF_DIM, *880, TG_INFO%FM_FLG)

     IF (DO_AEROSOL) THEN
        CALL Read_WFs ('Weighting functions: aerosol parameters', &
          AER_INFO%NAME, AER_INFO%WF_FLG, AER_INFO%WF_DIM, *880)
             
        Use_der     = .FALSE.
        IF( CHK_STAT('aer-modr', AER_INFO%NAME,AER_INFO%WF_FLG) ) &
             Use_der(1) = .TRUE.
        IF( CHK_STAT('aer-modr_f', AER_INFO%NAME,AER_INFO%WF_FLG) ) &
             Use_der(1) = .TRUE.
        IF( CHK_STAT('aer-var',   AER_INFO%NAME, AER_INFO%WF_FLG) ) &
             Use_der(2) = .TRUE.
        IF( ANY(Use_der).OR. &
             CHK_STAT('aer-cm_frac',AER_INFO%NAME,AER_INFO%WF_FLG) ) THEN
           N_dwr = COUNT( AER_INFO%WF_FLG ) 
           ! Number of derivatives with respect to PSD function 
           ! parameters (excluding number density)
           IF(CHK_STAT('aer-des',AER_INFO%NAME,AER_INFO%WF_FLG) ) &
           N_dwr = N_dwr - 1
        ENDIF
     END IF  ! DO_AEROSOL

     CALL Read_WFs ('Weighting functions: other atmospheric parameters', &
          PAR_INFO%NAME, PAR_INFO%WF_FLG, PAR_INFO%WF_DIM, *880)
     !    ... Switch off WFs for parameters which were not included 
     !        in the forward calculations:
     DO II = 1, SIZE(PAR_INFO)
        IF(INDEX( PAR_INFO(II)%NAME,'prosail')>0.AND.PAR_INFO(II)%WF_FLG) THEN
           IF( TRIM(SURF_TYPE)/='prosail' ) PAR_INFO(II)%WF_FLG=.FALSE.
        ENDIF
        IF(INDEX( PAR_INFO(II)%NAME,'flu')>0.AND.PAR_INFO(II)%WF_FLG) THEN
           IF( .NOT.FLU_L ) PAR_INFO(II)%WF_FLG=.FALSE.
        ENDIF
     ENDDO

     !     pause '  in read_rtm_wf  '
     IF (DO_CLOUD) THEN
        CALL Read_WFs ('Weighting functions: cloud parameters', &
             CLD_INFO%NAME, CLD_INFO%WF_FLG, CLD_INFO%WF_DIM, *880)
        ! If Phase function depends on the altitude (on R_eff)
!        IF( .NOT.Phm_const ) THEN
        IF( .NOT.ANY(MLCS(:)%Phm_co) ) THEN
           DO II = 1, UBOUND(CLD_INFO,1)
              IF( CLD_INFO(II)%WF_FLG.AND. &
                   CLD_INFO(II)%NAME(5:7)=='ref' ) THEN 
                 Deriv_Phc = .TRUE.
                 EXIT
              END IF
           END DO
        ENDIF
     END IF
  END SELECT

  ! Numerical weighting function for atmosphere parameters
  NUM_WF  = .FALSE. ; NUM_ABS = .FALSE. ; NUM_RAY = .FALSE.
  NUM_PRE = .FALSE. ; NUM_AER = .FALSE. ; NO_SOL  = .FALSE.
  NUM_CLD = .FALSE. ; NO_LS   = .FALSE. ; SOL_AND_LS = .FALSE.
  NUM_TEM = .FALSE.

  ! Numerical weighting function for ocean parameters
  NUM_OWF = .FALSE. ; NUM_ORAY = .FALSE. ; NUM_Chl = .FALSE.

  SELECT CASE (RTM_MODE)
  CASE('num_pert')
     IF(SUM(BAND_NUM(:)) > 1) GOTO 201
     STR_INP = 'Lower and upper boundary for WF calculation'
     CALL Read_Input (STR_INP, READ_ARR, *880)
     READ(READ_ARR(1)%S,*,ERR=101) MIN_WF_Z, MAX_WF_Z
     
     III = 0
     DO II = 1, N_ST_MOL
        IF(TG_INFO(II)%WF_FLG) THEN
           IND_NUM_WF = II  ; III = 1 ; EXIT
        END IF
     END DO
     
     NUM_WF  = .TRUE.
     IF( ANY(TG_INFO%WF_FLG)  ) NUM_ABS = .TRUE.
     IF( ANY(AER_INFO%WF_FLG) ) NUM_AER = .TRUE.
     IF( ANY(CLD_INFO%WF_FLG) ) NUM_CLD = .TRUE.
     DO II = 1, UBOUND(PAR_INFO,1)
        IF(.NOT.PAR_INFO(II)%WF_FLG) CYCLE
        IF(PAR_INFO(II)%NAME == 'ray')   NUM_RAY = .TRUE.
        IF(PAR_INFO(II)%NAME == 'press') NUM_PRE = .TRUE.
        IF(PAR_INFO(II)%NAME == 'temp')  NUM_TEM = .TRUE.
        III = III + 1
     END DO

!     IF(RTM_MODE == 'num_pert') GOTO 303
     
     NO_SOL = .FALSE. ; NO_LS = .FALSE. ; SOL_AND_LS = .TRUE.
!     NO_SOL = .FALSE. ; NO_LS = .FALSE. ; SOL_AND_LS = .FALSE.
     
     III = COUNT(AER_INFO(:)%WF_FLG) + COUNT(CLD_INFO(:)%WF_FLG) + &
          COUNT(TG_INFO(:)%WF_FLG) + COUNT(PAR_INFO(:)%WF_FLG)
     
     IF( UWT ) THEN
        STR_INP = 'Ocean parameters - weighting functions'
        CALL Read_Input (STR_INP, READ_ARR, *880)
        READ(READ_ARR(2)%S,*,ERR=101) &
             (OWF_FLG_ARR(II), II = 1, SIZE(OWF_FLG_ARR))
        IF( ANY(OWF_FLG_ARR(:)) ) NUM_OWF = .TRUE.
        III = III + COUNT( OWF_FLG_ARR )
        IF( COUNT(OWF_FLG_ARR) /= 0 ) NUM_ABS = .FALSE.
        IF( OWF_Ray ) NUM_ORAY = .TRUE.
        IF( OWF_Chl ) NUM_Chl  = .TRUE.
     ENDIF

     IF( III /= 1 ) GOTO 104

     IF( NUM_AER .OR. NUM_CLD ) THEN
        STR_INP = 'Numerical perturbation with respect to'
        CALL Read_Input (STR_INP, READ_ARR, *880)
        SCA_or_ABS = CUT_STR(READ_ARR(1)%S)
        
        SELECT CASE( SCA_or_ABS )
        CASE('aer_modr')
        CASE('scattering','absorption','aer_dens')
        CASE('eff_radius','lw_content')
        CASE('soot__conc','par_number')
        CASE('aerosol extinction')
           If(RTM_CORE /= "CDI" .or. RTM_TYPE /= "transmission") GOTO 105
        CASE DEFAULT
           GOTO 105
        END SELECT
     Else
        SCA_or_ABS = ''        
     ENDIF
     IF( NUM_CLD ) THEN
        STR_INP = 'Numerical perturbation of water/ice'
        CALL Read_Input (STR_INP, READ_ARR, *880)
        Wat_or_Ice = CUT_STR(READ_ARR(1)%S)
        SELECT CASE( TRIM(Wat_or_Ice) )
        CASE('water','ice','nomix')
        CASE DEFAULT
           GOTO 105
        END SELECT
     ENDIF
     
  CASE('wf','ret')
     Gauss_Field = .TRUE.
     ALLOCATE( Lo_der(Nvs) )
     
     IF( Vec_Sca=='vect_rte' ) THEN
        STR_INP = 'Weighting functions for Stokes component'
        CALL Read_Input (STR_INP, READ_ARR, *880)
        READ(READ_ARR(1)%S,*,ERR=101) (Lo_der(II), II=1,4)
        ! Number of Stokes components for which weighting
        ! function should be calculated
        NS_wf = COUNT( Lo_der(:) )
        ALLOCATE( Nc_der(NS_wf), Ainc(NS_wf,4) )
        Ainc = 0.d0
        K = 1
        ! Elements of Nc_der(:) show for which Stokes component
        ! derivative should be calculated
        DO II = 1,4
           IF( Lo_der(II) ) THEN
              Nc_der(K) = II ; Ainc(K,II) = 1.d0 
              K = K + 1
           ENDIF
        ENDDO
        IF( NS_wf == 0 ) GOTO 106
        
     ENDIF

     IF( CHK_STAT('ray', PAR_INFO%NAME, PAR_INFO%WF_FLG) ) Ray_to_WF = .TRUE.
!     II = FIND_IDX('Ray', PAR_INFO%NAME)
     IF( CHK_STAT('temp', PAR_INFO%NAME, PAR_INFO%WF_FLG) .OR. &
          CHK_STAT('press', PAR_INFO%NAME, PAR_INFO%WF_FLG) ) &
          Ray_to_WF = .TRUE.
!          PAR_INFO(II)%WF_FLG = .TRUE.
     
     IF( UWT ) THEN
        STR_INP = 'Ocean parameters - weighting functions'
        CALL Read_Input (STR_INP, READ_ARR, *880)
        READ(READ_ARR(2)%S,*,ERR=101) &
             (OWF_FLG_ARR(II), II = 1, SIZE(OWF_FLG_ARR))
        IF( ANY(OWF_FLG_ARR(:)) ) OWF_FLAG = .TRUE.
     ENDIF
     
     IF( RTM_TYPE == 'transmission') THEN
        DO II = 1, Size(AER_INFO)
           If(AER_INFO(II)%Name == 'aer-ext') Exit
        End DO
        FLG = AER_INFO(II)%WF_FLG
        IF(ANY(AER_INFO(:)%WF_FLG).OR.ANY(CLD_INFO(:)%WF_FLG).OR. &
             ANY(PAR_INFO(:)%WF_FLG)) THEN
           CALL WRITE_MESSAGE(R_NAME, 'WARNING', &
                'Some of the weighting functions are not ' &
                //'supported in "transmission" mode, they were' &
                //'automatically switched off !')
           AER_INFO(:)%WF_FLG = .FALSE.
           PAR_INFO(:)%WF_FLG = .FALSE.
           CLD_INFO(:)%WF_FLG = .FALSE.
        END IF
        AER_INFO(II)%WF_FLG = FLG
     ELSE
        SELECT CASE (RTM_CORE)
        CASE ('CDI')
           MSG = 'Some of the weighting functions are not supported '&
                //'in the "CDI" mode, they were automatically switched off !'
           WRT_FLG = .TRUE.
           IF (ANY(AER_INFO(:)%WF_FLG)) THEN
              IF(WRT_FLG) CALL WRITE_MESSAGE(R_NAME, 'WARNING', MSG)
              AER_INFO(:)%WF_FLG = .FALSE.
              WRT_FLG = .FALSE.
           END IF
           IF (ANY(CLD_INFO(:)%WF_FLG)) THEN
              IF(WRT_FLG) CALL WRITE_MESSAGE(R_NAME, 'WARNING', MSG)
              CLD_INFO(:)%WF_FLG = .FALSE.
              WRT_FLG = .FALSE.
           END IF
           DO II = 1, UBOUND(PAR_INFO, 1) 
              SELECT CASE (PAR_INFO(II)%NAME) 
              CASE ('albedo', 'th', 'elev')
              CASE DEFAULT
                 IF(WRT_FLG) CALL WRITE_MESSAGE(R_NAME, 'WARNING', MSG)
                 PAR_INFO(II)%WF_FLG = .FALSE.
                 WRT_FLG = .FALSE.
              END SELECT
           END DO
        CASE DEFAULT
           FLG = CHK_STAT('th', PAR_INFO%NAME, PAR_INFO%WF_FLG)
           IF(FLG) THEN
              CALL WRITE_MESSAGE(R_NAME, 'WARNING', 'Tangent height weighting '&
                   //'functions are supported only in "CDI" mode !')
              II = FIND_IDX('th', PAR_INFO(:)%NAME)
              PAR_INFO(II)%WF_FLG = .FALSE.
           END IF
        END SELECT
     END IF
            
  CASE('block_amf')
     TG_INFO(:)%WF_FLG = .FALSE.
     TG_INFO(AMF_IDX)%WF_FLG = .TRUE.
     TG_INFO(AMF_IDX)%WF_DIM  = 'abs'
     Gauss_Field = .TRUE.
     Deriv_Phc = .FALSE.
     ALLOCATE( Lo_der(Nvs) )
     NS_wf = 1
     Lo_der(:) = .FALSE. ; Lo_der(1) = .TRUE. 
     IF( Vec_Sca=='vect_rte' ) THEN
        ALLOCATE( Nc_der(NS_wf), Ainc(NS_wf,Nvs) )
        Nc_der(1) = 1 ;  Ainc = 0.d0 ;  Ainc(1,1) = 1.d0
     ENDIF
     
  CASE DEFAULT
     Deriv_Phc = .FALSE.
     
  END SELECT

  CALL ST_TRACK (R_NAME, 1, 'O')
  RETURN
  
101 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Reading error')
  WRITE(EU,*) '    Parameter to read: ', TRIM(STR_INP)
  GOTO 881
  
102 CALL WRITE_MESSAGE(R_NAME,'ERROR','Unrecognized setting to select' &
         // ' absolute or relative weighting functions ')
  GOTO 881
  
103 CALL WRITE_MESSAGE(R_NAME,'ERROR', 'Numerical perturbations can be '//&
         ' done only for relative weighting functions.')
  GOTO 881

104 CALL WRITE_MESSAGE(R_NAME,'ERROR','')
  WRITE(EU,*) '     Numerical perturbation can be performed'
  WRITE(EU,*) '     for a single parameter only. '
  WRITE(EU,*) '     Check "control_wf.inp" '
  GOTO 881

105 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Unrecognized parameter value')
  WRITE(EU,*) '    Parameter to read: ', TRIM(STR_INP)
  GOTO 881
  
106 CALL WRITE_MESSAGE(R_NAME, 'ERROR', ' ')
  WRITE(EU,'(a)') '     For which Stokes component should be calculated'
  WRITE(EU,'(a)') '     the weighting function?'
  WRITE(EU,'(a,2x,I3)') '     Check "Weighting functions for Stokes component"'
  WRITE(EU,'(a,2x,I3)') '     control parameter in "control_wf.inp" file.'
  GOTO 881

201 CALL WRITE_MESSAGE(R_NAME, 'ERROR', ' ')
  WRITE(EU,*) '      Numerical perturbation can be performed '//&
       'for a single spectral point only.'
  WRITE(EU,*) '      Check "Wavelength segment info" in the file '//&
       '"control.inp".'
  GOTO 881

880 WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)
  
881 CONTINUE
  RETURN 1

CONTAINS
!***************************************************************************
  SUBROUTINE DEALLOCATE_ARRAYS
    IF(ALLOCATED(OWF_FLG_ARR)) DEALLOCATE(OWF_FLG_ARR)
    IF(ALLOCATED(Lo_der)) DEALLOCATE(Lo_der)
  END SUBROUTINE DEALLOCATE_ARRAYS

!***************************************************************************
  SUBROUTINE Read_WFs (STR_INP, NAMES, WF_FLG, WF_DIM, *, ADD_FLG)
    LOGICAL  WF_FLG(:), ADD_FLG(:)
    CHARACTER (LEN=*)  STR_INP
    CHARACTER (LEN = 20) NAMES(:)
    CHARACTER (LEN = 3) WF_DIM(:)
    OPTIONAL :: ADD_FLG
  ! Local parameters
    CHARACTER (LEN = *), PARAMETER :: R_NAME = 'READ_WFs'
    INTEGER II, KK
    LOGICAL FLG_SEL(SIZE(NAMES)), FLG_ADD(SIZE(NAMES))
    TYPE (DYN_CHAR), ALLOCATABLE :: READ_ARR(:), STR_ARR(:), STR_ARR_1(:)
  !==================================================================
    IF(PRESENT(ADD_FLG)) THEN
       FLG_ADD = ADD_FLG
    ELSE
       FLG_ADD = .TRUE.
    END IF
    
    CALL Read_Input (STR_INP, READ_ARR, *883)
    DO II = 1, SIZE(READ_ARR)
       IF(BLANK_OR_COMMENT(READ_ARR(II)%S)) EXIT
       CALL SPLIT_STRING(READ_ARR(II)%S, STR_ARR, ';')
       DO KK = 1, SIZE(STR_ARR)
           CALL SPLIT_STRING(STR_ARR(KK)%S, STR_ARR_1, ':')
           IF(SIZE(STR_ARR_1) /= 2) GOTO 301
           CALL UPPER_LOWER_CASE (STR_ARR_1(1)%S,'L2U')
           CALL FIND_MATCHES (*883, STR_ARR_1(2)%S, NAMES, FLG_SEL)
           WHERE(FLG_SEL.AND.FLG_ADD) WF_FLG = .TRUE.
           SELECT CASE (STR_ARR_1(1)%S)
           CASE ('A')
              WHERE(FLG_SEL.AND.WF_FLG) WF_DIM = 'abs'
!              IF(RTM_MODE == 'num_pert') GOTO 303
           CASE ('R')
              WHERE(FLG_SEL.AND.WF_FLG) WF_DIM = 'rel'
           CASE DEFAULT
              GOTO 302
           END SELECT
        END DO
     END DO

     RETURN
     
301  CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Reading error')
     WRITE(EU,*) '    Parameter to read: ', STR_INP
     GOTO 882

302  CALL WRITE_MESSAGE(R_NAME,'ERROR','Unrecognized setting to select' &
          // ' absolute or relative weighting functions ')
     GOTO 882
     
!303 CALL WRITE_MESSAGE(R_NAME,'ERROR', 'Numerical perturbations can be '//&
!          ' done only for relative weighting functions.')
!     GOTO 882
     
883 WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)
  
882  CONTINUE
     RETURN 1

  END SUBROUTINE READ_WFS
!***************************************************************************
END SUBROUTINE READ_RTM_WF
!***************************************************************************
