C********************************************************************
C
C       SCIATRAN - Version 4.6 - Last modification: 08 Apr 2022
C
C       Copyright (C) - Institute of Remote Sensing,
C                       University of Bremen, Germany
C *********************************************************************
c       
C     Subroutine to calculate the Stokes vector of the reflected 
c       from the surface direct radiation accounting for BRDF
c                in the spherical geometry.
C      !!! It is assumed that incident radiation is unpolarized !!!
C ----------------------------------------------------------------------
      SUBROUTINE REFL_DIR_SPH(NLSA, NSZA, NPHI, MuV, Mue, Phi, POS, 
     $     ALBEDO, FLU_int, SOL_EXP, PLANCK_SURF, BC_S1, LBC, REF_SUR)
c     ... Called by "st_ss_stokes.f" single scattering in spherical mode
c              "sfi_spher.f" multiple scattering (approximate spherical)
c              "integrate_init.f90" multiple scattering (full spherical)

      USE BRDF,        ONLY: BRDF_PARAM, DO_OC_COMP, Ker_cof => IVG_cof,
     $                       Nwl_veg, WL_veg, Wgr_svk, Alb_svk, N_svk, 
     $                       Nwl_BRF
      USE BRDF_SUBRS,  ONLY: BRDF_ACT_DOM
      USE INTERN_DIMS, ONLY: Nvs
      USE ST_VARS,     ONLY: SURF_TYPE, BRDF_FLAG
      USE TYPE_VARS,   ONLY: KND
      USE WATER,       ONLY: SS_PDF
c     ------------------------------------------------------------------
c     Subroutine arguments:
c     ---------------------
      INTEGER,    INTENT(IN) :: NLSA, NSZA, NPHI
      REAL (KND), INTENT(IN) :: ALBEDO, FLU_int, POS, SOL_EXP, 
     $     PLANCK_SURF, MuV(NLSA), Mue(NSZA), Phi(NPHI)
      REAL (KND) BC_S1(Nvs), LBC(Nvs)
      REAL(KND), ALLOCATABLE :: REF_SUR(:,:)
      OPTIONAL :: REF_SUR
c     ------------------------------------------------------------------
c     ... Local variables
      CHARACTER (LEN=20) TXT
      INTEGER  J, Jc, Ws, Ws1, IN1, N4
      REAL (KND)  WL1(1), Area, DC, Uc, Aveg,
     $     BRF_OUT(Nvs*NLSA,Nvs*NSZA,NPHI,1)
      REAL (KND), ALLOCATABLE:: R1_OUT(:,:,:,:)
c     ==================================================================

      IF( .NOT.BRDF_FLAG ) THEN
         BC_S1(1) = ALBEDO*ABS(Mue(1))*SOL_EXP + PLANCK_SURF + FLU_int   
c         write(*,*)  ALBEDO,  Mue(1), SOL_EXP
         LBC(1)   = ALBEDO  ;  RETURN
      ENDIF

      WL1(1) = POS ; Area = 0.d0 ; BRF_OUT = 0.d0

      N4 = 1
      SELECT CASE( TRIM(SURF_TYPE) )
         CASE('Ross-Li')                ;  N4 = N4 + 1
         CASE('Ross-Li-snow','snow_fasmar') ;  N4 = N4 + 2
      END SELECT

      ALLOCATE( R1_OUT(Nvs*NLSA,Nvs*NSZA,NPHI,N4) )

c     There is no reflection from ocean surface
c     -----------------------------------------
      IF( .NOT.ANY(DO_OC_COMP) ) THEN

         SELECT CASE( TRIM(SURF_TYPE) )
         CASE('Ross-Li','Ross-Li-snow')
c           ... Calculation should be performed for two kernel
            CALL BRDF_ACT_DOM( BRDF_PARAM, NLSA, MuV, NPHI, Phi,
     $           NSZA, Mue, 1, WL1, R1_OUT, 'none' )
            CALL LIN_SEARCH_NEW(POS, DC, Ws, Ws1, Nwl_veg, WL_veg)
            Uc = (1.d0 - Dc)
            BRF_OUT(1:Nvs*NLSA:Nvs,1:Nvs*NSZA:Nvs,:,1) = 
     $           (Uc*Ker_cof(1,Ws) + Dc*Ker_cof(1,Ws1))
            DO J = 1,Nwl_BRF
               BRF_OUT(:,:,:,1) = BRF_OUT(:,:,:,1) + 
     $              (Uc*Ker_cof(J+1,Ws) + Dc*Ker_cof(J+1,Ws1))
     $              *R1_OUT(:,:,:,J)
            ENDDO

         CASE('svk')
            IF( POS >= Wgr_svk ) THEN
               TXT = 'second' ; IN1 = 2
            ELSE
               TXT = 'first'  ; IN1 = 1
            ENDIF
            CALL BRDF_ACT_DOM( BRDF_PARAM, NLSA, MuV, NPHI, Phi,
     $           NSZA, Mue, 1, WL1, R1_OUT, TXT )
            CALL LIN_SEARCH_NEW(POS, DC, Ws, Ws1, Nwl_veg, WL_veg)
            Aveg = (1.d0 - DC)*Alb_svk(Ws) + DC*Alb_svk(Ws1)
            BRF_OUT = Aveg*N_svk(IN1)*R1_OUT 

         CASE('mRPV_facet')
            CALL BRDF_ACT_DOM( BRDF_PARAM, NLSA, MuV, NPHI, Phi,
     $           NSZA, Mue, 1, WL1, R1_OUT, 'Gauss' )
            BRF_OUT = R1_OUT 

         CASE('snow_artm','white_si')
            CALL BRDF_ACT_DOM( BRDF_PARAM, NLSA, MuV, NPHI, Phi,
     $           NSZA, Mue, 1, WL1, R1_OUT, 'Gauss' )
            BRF_OUT = R1_OUT 

         CASE('melt_pond')
            CALL POND_BRF( NLSA, MuV, NPhi, Phi, NSZA, Mue,
     $           1, WL1, R1_OUT )
            BRF_OUT = R1_OUT

         CASE('rff')
            CALL RFF_BRDF_INT(NLSA, MuV, NSZA, Mue, NPHI, Phi,
     $           POS, R1_OUT)
            BRF_OUT = R1_OUT 

         CASE DEFAULT
            CALL BRDF_SETUP
            CALL BRDF_ACT_DOM( BRDF_PARAM, NLSA, MuV, NPHI, Phi,
     $           NSZA, Mue, 1, WL1, R1_OUT, 'Gauss' )
            BRF_OUT = R1_OUT 
         END SELECT
      ENDIF

c     There is reflection from ocean surface
c     --------------------------------------
c     ... Whitecaps
      IF( DO_OC_COMP(1) ) THEN 
         CALL USER_WHITECAP( Nvs, NLSA, MuV, NPHI, Phi, 
     $        NSZA, Mue, 1, WL1, R1_OUT, Area )
         BRF_OUT = BRF_OUT + R1_OUT
      ENDIF
c     ... Glint
      IF( DO_OC_COMP(2) ) THEN 
         CALL BRDF_SETUP
         CALL BRDF_ACT_DOM( BRDF_PARAM, NLSA, MuV, NPHI, Phi,
     $        NSZA, Mue, 1, WL1, R1_OUT, SS_PDF )
         BRF_OUT = BRF_OUT + (1.d0 - Area)*R1_OUT 
      ENDIF
c     ... Water leaving
      IF( DO_OC_COMP(3) ) THEN 
         CALL USER_WATER( Nvs, NLSA, MuV, NPHI, Phi, 
     $        NSZA, Mue, 1, WL1, R1_OUT )
         BRF_OUT = BRF_OUT + (1.d0 - Area)*R1_OUT
      ENDIF

      IF( PRESENT(REF_SUR) ) THEN 
         IF(.NOT.ALLOCATED(REF_SUR)) ALLOCATE( REF_SUR(Nvs,Nvs) ) 
         REF_SUR = 0.d0
         DO Jc = 1,Nvs
            REF_SUR(Jc,:) = BRF_OUT(Jc,1:Nvs,1,1)
         ENDDO
      ELSE
         DO Jc = 1,Nvs
            BC_S1(Jc) = ABS(Mue(1))*SOL_EXP*BRF_OUT(Jc,1,1,1)
            LBC(Jc)   = BRF_OUT(Jc,1,1,1)
         ENDDO
      ENDIF

      RETURN
      END
      
c **********************************************************************
c              For scalar case only!
c ********************************************************************** 
      SUBROUTINE RFF_BRDF_INT(NLSA, MuV, NSZA, Mue, NPHI, Phi,
     $     POS, R1_OUT)
c     Called by  "refl_dir_sph"
      
      USE BRDF,        ONLY: WL_BRF, Nwl_BRF, X_brf, BRF_GG_WL
      USE GT_POLYN,    ONLY: XM
      USE ST_VARS,     ONLY: NN => NSTRMS
      USE INTERN_DIMS, ONLY: Ng1_brf, Ng2_brf, Ns_brf, Nh_brf, Nvs
      USE TYPE_VARS,   ONLY: KND

      INTEGER, INTENT(IN):: NLSA, NSZA, NPHI
      REAL (KND), INTENT(IN):: POS, Phi(NPHI), MuV(NLSA), Mue(NSZA)
      REAL (KND)  R1_OUT(Nvs*NLSA,Nvs*NSZA,NPHI,1)
c     ... Local parameters:
c     ------------------------------------------------------------------
      CHARACTER (LEN=*), PARAMETER :: R_NAME = 'RFF_BRDF_INT'
      INTEGER I, Iv, Iso, J, K, Iaz
      INTEGER  Igg, Mh, Ws, Ws1, Wss(2)
      REAL (KND) B_int(2), BRF_GG(Ng1_brf,Ng2_brf,Nh_brf)
      REAL (KND) DM, Dc, Dcs, S_res(2,Ng1_brf)
c     ==================================================================

      CALL ST_TRACK (R_NAME, 1, 'I')

      CALL LIN_SEARCH_NEW(POS, Dc, Ws, Ws1, Nwl_BRF, WL_BRF)
      BRF_GG(:,:,:) = (1.d0 - Dc)*BRF_GG_WL(:,:,:,Ws) 
     $     + Dc*BRF_GG_WL(:,:,:,Ws1)
      DO Iso = 1, NSZA
         CALL LIN_SEARCH_NEW(Mue(Iso), Dcs, Wss(1), Wss(2), Ng2_brf, XM)
         DO Iaz = 1, NPHI
            DO I = 1,2 ; Igg = Wss(I)
               S_res(I,:) = BRF_GG(:,Igg,1)
               DO Mh = 2, Nh_brf 
                  DM = DFLOAT(Mh-1)
                  S_res(I,:) = S_res(I,:) + 2.d0*BRF_GG(:,Igg,Mh)
     $                 *DCOS( DM*Phi(Iaz) )
               ENDDO  ! over Fourier harmonics (Mh)
            ENDDO  ! over selected grid angles (incidence) (I)
            DO Iv = 1, NLSA
               CALL LIN_SEARCH_NEW(Muv(Iv), Dc, Ws, Ws1, Ng1_brf, X_brf)
c              ... B_int is reflectance at actual direction at two
c                  incidence grid angles
               B_int(:) = (1.d0 - Dc)*S_res(:,Ws) + Dc*S_res(:,Ws1)
               R1_OUT(Iv,Iso,Iaz,1) =
     $              (1.d0 - Dcs)*B_int(1) + Dcs*B_int(2)
            ENDDO  ! over viewing zenith angles (Iv)
         ENDDO  ! over user azimuthal angles (Iaz)
      ENDDO  ! over solar zenith angles (Iso)

      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN
      END SUBROUTINE RFF_BRDF_INT
c **********************************************************************
      
