!***************************************************************************
!
!	SCIATRAN - Version 4.6 - Last modification: 29 Sep 2022
!
!	Copyright (C) - Institute of Remote Sensing,
!			University of Bremen, Germany
!
!***************************************************************************
SUBROUTINE REFR_INTERFACE (N_LINE, Z_LINE, C_PSI_LINE, TH_LINE, REFR_FLAG, *)
  USE GRIDS,        ONLY: ZZZ, NLAYERS
  USE LOG_PARS,     ONLY: EU
  USE TYPE_VARS,    ONLY: KND
  USE REFRACT_VARS, ONLY: H_NOREF_SOL, COS_NOREF_SOL
  
  CHARACTER (LEN = *), PARAMETER :: R_NAME = 'REFR_INTERFACE'
  
  INTEGER           N_LINE, I, J, N_START, N_END, N_STEP
  LOGICAL           REFR_FLAG(N_LINE), SHADOW_FLG(N_LINE)
  REAL (KND), DIMENSION (N_LINE) :: Z_LINE, C_PSI_LINE, TH_LINE, PSI_LINE
  REAL (KND)        PSI_TMP
  !==================================================================
  CALL ST_TRACK (R_Name, 1, 'I')

  REFR_FLAG = .TRUE. ;  SHADOW_FLG = .FALSE.
  PSI_LINE(:) = DACOS(C_PSI_LINE(:))
  
  ! Refraction is turned off if SZA < 70 deg, or  TH > 35 km,
  ! or both H > 35 km and  SZA < 90 deg
  WHERE(C_PSI_LINE >= COS_NOREF_SOL.OR.TH_LINE >= H_NOREF_SOL.OR. &
       (C_PSI_LINE >= 0.0D0.AND.Z_LINE >= H_NOREF_SOL)) REFR_FLAG = .FALSE. 
      
  IF(REFR_FLAG(1)) CALL REFRACTION (Z_LINE(1), C_PSI_LINE(1), PSI_LINE(1), &
       TH_LINE(1), SHADOW_FLG(1), *880)
  
  N_START = 0; N_END = 0
  IF(.NOT.SHADOW_FLG(1)) THEN
     ! The first point of the LOS is illuminated
     N_START = 2; N_END = N_LINE; N_STEP = 1
  ELSE
     IF(REFR_FLAG(N_LINE).AND.N_LINE /= 1) THEN 
        CALL REFRACTION (Z_LINE(N_LINE), C_PSI_LINE(N_LINE), &
             PSI_LINE(N_LINE), TH_LINE(N_LINE), SHADOW_FLG(N_LINE), *880)
     END IF
     IF(.NOT.SHADOW_FLG(N_LINE)) THEN
        ! The last point of the LOS is illuminated
        N_START = N_LINE - 1; N_END = 2; N_STEP = - 1
     END IF
  END IF

  IF(N_START == 0) THEN
     ! If neigher first nor last point of the LOS is illuminated we do 
     ! not need to calculate solar light paths
     TH_LINE = - 7777.
     SHADOW_FLG = .TRUE.
     C_PSI_LINE = - 1.0D0
     RETURN
  END IF

  J = 0
  DO I = N_START, N_END, N_STEP
     IF(.NOT.REFR_FLAG(I)) THEN
        J = 0 ;  CYCLE
     END IF
     
     J = J + 1
     IF(J >= 3) THEN
        IF(ALL(TH_LINE(I - 2*N_STEP:I:N_STEP) > - 7000.)) THEN
           
           IF(DABS(Z_LINE(I - N_STEP) - Z_LINE(I - 2*N_STEP)) > 0.1D0) THEN
              ! If the altitudes of the neighboring LOS points are not too  
              ! close to each other we calculate the first guess TH for the 
              ! next point by the extrapolation 
              PSI_TMP = PSI_LINE(I - N_STEP) + &
                   (PSI_LINE(I - N_STEP) - PSI_LINE(I - 2*N_STEP))*&
                   (Z_LINE(I) - Z_LINE(I - N_STEP))&
                   /(Z_LINE(I - N_STEP) - Z_LINE(I - 2*N_STEP))
              PSI_LINE(I) = PSI_TMP
           ELSE
              ! Otherwise, we use the minimum of the refractive TH from the 
              ! previous step and the geometrical TH from the current step. 
              IF(TH_LINE(I - N_STEP) > - 7000.) PSI_LINE(I) = &
                   MAX(PSI_LINE(I),PSI_LINE(I - N_STEP))
           END IF
        END IF
     END IF

     ! if( Z_LINE(I) /= 10.5D0 ) cycle
     CALL REFRACTION (Z_LINE(I), C_PSI_LINE(I), PSI_LINE(I), TH_LINE(I), &
          SHADOW_FLG(I), *880)
 
     ! If 3 points at the LOS are not illuminated, the remaining points 
     ! are not illuminated as well
     IF(J >= 3) THEN
        IF(ALL(SHADOW_FLG(I - 2*N_STEP:I:N_STEP))) THEN
           TH_LINE(I + N_STEP:N_END:N_STEP) = - 7777.
           SHADOW_FLG(I + N_STEP:N_END:N_STEP) = .TRUE.
           C_PSI_LINE(I + N_STEP:N_END:N_STEP) = - 1.0D0
           EXIT
        END IF
     END IF
  END DO

  CALL ST_TRACK (R_Name, 1, 'O')
  RETURN

880 WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)
  RETURN 1
END SUBROUTINE REFR_INTERFACE

!***************************************************************************
!                                                                   
!       Subroutine ref_fine for calculation  1/(1+n'*r/n)           
!       (n - refraction index ,n' - differential of n)              
!                      on a fine grid                               
!                                                                   
!***************************************************************************
SUBROUTINE REFR_FINE
  USE GRIDS,    ONLY: NLAYERS, ZZZ, NL_FINE, Z_FINE
  USE LOG_PARS, ONLY: LOG_LEVEL, N_TSO, LUNIT
  USE PROFILES, ONLY: PRESS, TEMP
  USE RefrVar,  ONLY: NL_REFR, Z_REFR, RI_Z, INT_TERM, D_RI_Z, &
                      REFR_IND, D_RI, P_UP, IT_FINE, RefI, RHG_STEP, &
                      G_SZA_T, IDX_REFR, NLR_FINE, WL_REF
  USE ST_VARS,  ONLY: EARTH_RADIUS, GROUND_HEIGHT, PIE, R_GH
  USE TYPES,    ONLY: KND

  CHARACTER (LEN = *), PARAMETER :: R_NAME = 'REFR_FINE'
  INTEGER      J, K, I, JJ, NL_THSMP
  LOGICAL      SHADOW_FLG
  LOGICAL, ALLOCATABLE :: FLAG_ZR(:)
  REAL (KND), ALLOCATABLE :: T_REFR(:), P_REFR(:), R_REFR(:), RI_TMP(:)
  REAL (KND)   DLP(NLAYERS), DELTA, S, S_ADD, DD, ZZ, WN2, HPa_Torr, &
       TH_SAMP, FL_THSAMP
  REAL (KND), PARAMETER :: CONST = 0.8153601250877D-04, & 
       D_REFR_LIM = 1.0D-6, D_RI_LIM = 1.0D-1
  !==================================================================
  CALL ST_TRACK (R_Name, 1, 'I')
  
  RHG_STEP = 1.0D-4 ; TH_SAMP = 1.0D-6
  NLR_FINE = (ZZZ(1) - GROUND_HEIGHT)/RHG_STEP + 1
  
  IF(ALLOCATED(Z_REFR)) DEALLOCATE(Z_REFR, REFR_IND, RI_Z, INT_TERM,&
       D_RI_Z, D_RI, RefI, IT_FINE)
  ALLOCATE(Z_REFR(NLR_FINE), REFR_IND(0:NLR_FINE), RI_Z(NL_FINE), &!A
       INT_TERM(0:NLR_FINE), FLAG_ZR(NLR_FINE), D_RI_Z(NL_FINE), &!A
       T_REFR(NLR_FINE), P_REFR(NLR_FINE), R_REFR(NLR_FINE), &!A
       RI_TMP(NLR_FINE), D_RI(0:NLR_FINE), IT_FINE(NLR_FINE),  &!A
       RefI(NL_FINE)) !A

  ! Refraction grid
  Z_REFR(1) =  ZZZ(1)
  Z_REFR(NLR_FINE) =  GROUND_HEIGHT
  DO K = NLR_FINE - 1, 2, -1 
     Z_REFR(K) = GROUND_HEIGHT + RHG_STEP*(NLR_FINE - K)
  END DO
  R_REFR(:) = Z_REFR(:) + EARTH_RADIUS
  
  ! Interpolation of pressure and temperature profiles to the refraction grid
  T_REFR(1) = TEMP(1) ;   T_REFR(NLR_FINE) = TEMP(NLAYERS)
  P_REFR(1) = PRESS(1) ;  P_REFR(NLR_FINE) = PRESS(NLAYERS)
  DLP = DLOG(PRESS)
  J = NLAYERS
  DO K = NLR_FINE - 1, 2, -1 
     IF(Z_REFR(K) > ZZZ(J - 1)) J = J - 1
     DELTA = (Z_REFR(K) - ZZZ(J))/(ZZZ(J - 1) - ZZZ(J))
     T_REFR(K) = TEMP(J - 1)*DELTA + TEMP(J)*(1.0D0 - DELTA)
     P_REFR(K) = DLP(J - 1)*DELTA + DLP(J)*(1.0D0 - DELTA)
  END DO
  P_REFR(2:NLR_FINE - 1) = DEXP(P_REFR(2:NLR_FINE - 1))
  
  ! Refractive index in accordance to Edlen, 1966
  WN2 = (1.0D3/WL_REF)**2 !(wavenumber in mkm-1))
  RI_TMP(:) = (8342.13D0 + 2406030.0D0/(130.0D0 - WN2) +&
       15997.0D0/(38.9D0 - WN2))*1.0D-8
  ! Dependence on pressure and temperature
  ! 1 hPa = 0.750061673821 torr
  Hpa_Torr = 0.750061673821D0
  RI_TMP(:) = 1.0D0 + RI_TMP(:)*Hpa_Torr*P_REFR(:)/720.775D0*&
       (1.0D0 + Hpa_Torr*P_REFR(:)*(0.817 - 0.0133*&
       (T_REFR(:) - 273.15D0))*1D-6)/&
       (1.0D0 + 0.0036620*(T_REFR(:) - 273.15D0))

  ! This is to ensure a minimum positive gradient in the
  ! product (z + R)*n. If not fulfilled, the solution for
  ! the refractive tangent height is non-unique.
  ! RI_TMP(:) = 1.0D0 + CONST*P_REFR(:)/T_REFR(:)
  REFR_IND(1) = RI_TMP(1)
  DO I = 2, NLR_FINE
     DD = (R_REFR(I - 1)*RI_TMP(I - 1) - R_REFR(I)*RI_TMP(I))/RHG_STEP     
     IF(DD < D_RI_LIM) THEN
        REFR_IND(I) = (R_REFR(I - 1)*REFR_IND(I - 1) -&
             D_RI_LIM*RHG_STEP)/R_REFR(I)
     ELSE
        REFR_IND(I) = RI_TMP(I)
     END IF
  END DO
  
  P_UP = 1.0D0/(EARTH_RADIUS + ZZZ(1))/REFR_IND(1)

  ! Altitude derivative of the corrected refraction coefficient
  ! Calculation via P and T derivative is no longer possible because
  ! of the correction
  D_RI(1) = (REFR_IND(1) - REFR_IND(2))/RHG_STEP
  DO J = 2, NLR_FINE - 1
     D_RI(J) = (REFR_IND(J - 1) - REFR_IND(J + 1))/RHG_STEP/2.0D0 
  END DO
  D_RI(NLR_FINE) = (REFR_IND(NLR_FINE - 1) - REFR_IND(NLR_FINE))/RHG_STEP

  ! Here, we select sub-layers from the fine grid to assure 
  ! a non-negligible gradient of the refractive index. 
  ! The gradient at a new sub-grid must be larger then D_REFR_LIM
  ! for all altitude levels.
  S = DABS(D_RI(1)*(Z_REFR(1) - Z_REFR(2)))
  FLAG_ZR(1) = .TRUE.
  FLAG_ZR(2:NLR_FINE - 1) = .FALSE.
  do i = 2, NLR_FINE - 1
     S_ADD = DABS(D_RI(i)*(Z_REFR(i - 1) - Z_REFR(i + 1))/2.)
     S = S + S_ADD
     IF(S > D_REFR_LIM) THEN
        FLAG_ZR(I - 1) = .TRUE.
        S = S_ADD
     END IF
  end do
  IF(S > D_REFR_LIM) FLAG_ZR(NLR_FINE - 1) = .TRUE.
  FLAG_ZR(NLR_FINE) = .TRUE.
  
  NL_REFR = COUNT(FLAG_ZR)
  IF(ALLOCATED(IDX_REFR)) DEALLOCATE(IDX_REFR)
  ALLOCATE(IDX_REFR(NL_REFR))
  IDX_REFR = PACK((/(I, I = 1, NLR_FINE)/),FLAG_ZR)
      
  INT_TERM(1:NLR_FINE) = 1.0D0/(1.0D0 + R_REFR(:)/&
       REFR_IND(1:NLR_FINE)*D_RI(1:NLR_FINE))

  ! This is needed to get the interpolation after GET_IDX_BELOW work
  ! if Z is slightly above the TOA (IDX = 1)
  D_RI(0) = D_RI(1)
  REFR_IND(0) = REFR_IND(1) ;  INT_TERM(0) = INT_TERM(1)

  DO J = 1, NL_FINE
     CALL GET_IDX_BELOW (Z_FINE(J), JJ, DD)
     RefI(J) = (REFR_IND(JJ - 1)*DD + REFR_IND(JJ)*(1.0D0 - DD))
     RI_Z(J) = (Z_FINE(J) + EARTH_RADIUS)*RefI(J)
     D_RI_Z(J) = (Z_FINE(J) + EARTH_RADIUS)*&
          (D_RI(JJ - 1)*DD + D_RI(JJ)*(1.0D0 - DD))
     IT_FINE(J) = (INT_TERM(JJ - 1)*DD + INT_TERM(JJ)*(1.0D0 - DD))
  END DO
  
  CALL ST_TRACK (R_Name, 1, 'O')
  
  RETURN
END SUBROUTINE REFR_FINE

!***************************************************************************
SUBROUTINE GEOM_ANGLE_V3 (Z_I, SOL_INP, FI_I, H_0, SHADOW_FLG, TIR_FLG, *)
  
  USE LOG_PARS,     ONLY: EU
  USE GRIDS,        ONLY: ZZZ
  USE REFRACT_VARS, ONLY: P_UP, Z_REFR, REFR_IND, INT_TERM, &
                          NLR_FINE, IDX_REFR, RHG_STEP
  USE TYPES,        ONLY: KND, DYN_CHAR
  USE ST_VARS,      ONLY: EARTH_RADIUS, GROUND_HEIGHT, PIE
  Use Tools,        Only: ErrMsg

  CHARACTER (LEN = *), PARAMETER :: R_NAME = 'GEOM_ANGLE_V3'

  INTEGER       II_NEXT, JJ, COS_SIGN
  LOGICAL       TH_FLAG, LAST_LAYER, REFR_FLAG, RESET_FLG
  LOGICAL       SHADOW_FLG, TIR_FLG
  REAL (KND)    REFR_I, Z_I, FI_I, Z_1, Z_A, R_A, R_NEXT, REFR_NEXT
  REAL (KND)    ANG_A1, ANG_A2, SIN_A, COS_A, SOL_INP, FI_1, Z_TMP
  REAL (KND)    DD, IT_1, IT_2, Z_NEXT, H_0, AUX
  CHARACTER (LEN=200) StrLoc
  TYPE (DYN_CHAR), ALLOCATABLE :: Msg(:)
  !==================================================================
  CALL ST_TRACK (R_NAME, 1, 'I')

  Z_1 = ZZZ(1)
  RESET_FLG = .FALSE.
777 CONTINUE
  SHADOW_FLG = .FALSE. ; TIR_FLG = .FALSE.
  Z_A = Z_I
  ANG_A1 = SOL_INP
  R_A = Z_A + EARTH_RADIUS

  COS_A = DCOS(ANG_A1) ;  SIN_A = DSIN(ANG_A1)
  CALL GET_IDX_BELOW (Z_A, JJ, DD)
  REFR_I = REFR_IND(JJ - 1)*DD + REFR_IND(JJ)*(1.0D0 - DD)
  IT_1 = INT_TERM(JJ - 1)*DD + INT_TERM(JJ)*(1.0D0 - DD)
  
  FI_I = DASIN(REFR_I*R_A*SIN_A*P_UP)
  FI_1 = 0
  LAST_LAYER = .FALSE.
  COS_SIGN = IDINT(sign(1.0D0, COS_A))
  REFR_FLAG = .TRUE.
  IF(COS_SIGN >= 0) THEN
     II_NEXT = COUNT(Z_REFR(IDX_REFR(:)) > Z_A)
     If(II_NEXT < 1 ) GoTo 201
  ELSE
     II_NEXT = COUNT(.NOT.Z_REFR(IDX_REFR(:)) < Z_A) + 1
     IF(Z_A <= GROUND_HEIGHT) THEN
        FI_1 = FI_1 + PIE - 2.0D0*ANG_A1 
        FI_I = FI_I - PIE + 2.0D0*ANG_A1
        ! write(6,*) '***** FI_1, FI_I ****', FI_1, FI_I
        ! write(6,*) '***** ANG_1 ****', ANG_A1/PIE*180.
        COS_SIGN = 1
        ANG_A1 = PIE - ANG_A1
        ! write(6,*) '***** ANG_1 new ****', ANG_A1/PIE*180.
        II_NEXT = II_NEXT - 1
        If(II_NEXT > Size(IDX_REFR) ) GoTo 201
        IF(GROUND_HEIGHT - Z_A > 1.0D-4) THEN
           CALL WRITE_MESSAGE (R_NAME, 'ERROR', &
                'Error in the refraction routine !')
           RETURN 1
        END IF
     END IF
  END IF
  IF(.NOT.RESET_FLG) THEN
     H_0 = R_A*SIN_A - EARTH_RADIUS 
     CALL TH_REFR_V2(Z_A, SIN_A, H_0, *880)
  END IF
  DO
     
     Z_NEXT = Z_REFR(IDX_REFR(II_NEXT))

     IF(COS_SIGN <= 0.AND.H_0 >= Z_NEXT) THEN
        ! write(6,*) '*** Z_NEXT is below the tangent height ! ***'
        CALL TH_REFR_V2(Z_A, SIN_A, H_0, *880)
        
        ! write(6,*) '***** H_0 ****', H_0
        ! pause
        IF(H_0 > Z_A .OR. H_0 < Z_NEXT) THEN
           ! First estimation of the TH was wrong --> RESET
           !           write(6,*) SIN_A, Z_A, Z_NEXT, H_0
           !           write(6,*) '********** RESET ! *************'
           !           pause
           RESET_FLG = .TRUE.
           GOTO 777
        END IF
        TH_FLAG = .TRUE.
        Z_NEXT = H_0 ;  COS_SIGN = 1
        CALL GET_IDX_BELOW (Z_NEXT, JJ, DD)
        
        REFR_NEXT = REFR_IND(JJ - 1)*DD + REFR_IND(JJ)*(1.0D0 - DD)
        IT_2 = INT_TERM(JJ - 1)*DD + INT_TERM(JJ)*(1.0D0 - DD)
     ELSE
        TH_FLAG = .FALSE.
        IF(IDX_REFR(II_NEXT) == NLR_FINE) THEN
           ! The refracted beam crosses the Earth's surface --> shadow
           REFR_FLAG = .FALSE.
           SHADOW_FLG = .TRUE.  
           Z_NEXT = GROUND_HEIGHT
           JJ = NLR_FINE
        ELSE
           Z_TMP = DDIM(Z_NEXT, GROUND_HEIGHT)
           JJ = NLR_FINE - IDNINT(Z_TMP/RHG_STEP)
        END IF
        IT_2 = INT_TERM(JJ)
        REFR_NEXT = REFR_IND(JJ)
     END IF
     
     IF(Z_NEXT >= Z_1) THEN
        LAST_LAYER = .TRUE.
        Z_NEXT = Z_1 
        IT_2 = INT_TERM(1)
        REFR_NEXT = REFR_IND(1)
     END IF
     
     R_NEXT = Z_NEXT + EARTH_RADIUS
     
     IF(TH_FLAG) THEN
        ANG_A2 = PIE/2.0D0
     ELSE
        DD = R_A/R_NEXT*REFR_I/REFR_NEXT*SIN_A
        IF(DD > 1.0D0) THEN
!           IF(DD - 1.0D+1**(-PRECISION(DD)) > 1.0D0) Then
              FI_I = 0.0D0
              TIR_FLG = .TRUE.
              ! This beam is internally reflected and cannot leave the atmosphere
              Return
!           END IF
           !CALL WRITE_MESSAGE &
           !     (R_NAME, 'E', 'Sinus argument is larger than 1')
!           ANG_A2 = PIE/2.0D0
        ELSE
           ANG_A2 = DASIN(DD)
        END IF
        IF(COS_SIGN < 0) ANG_A2 = PIE - ANG_A2
     END IF
     
     AUX = (IT_2 + IT_1)*(ANG_A2 - ANG_A1)/2.0D0
     FI_1 = FI_1 + AUX ;  FI_I = FI_I - AUX
     
     IF(.NOT.REFR_FLAG) THEN
        AUX = PIE - 2.0D0*ANG_A2 
        FI_1 = FI_1 + AUX ;  FI_I = FI_I - AUX
        COS_SIGN = 1
        ANG_A2 = PIE - ANG_A2
        REFR_FLAG = .TRUE.
     END IF

     IF(LAST_LAYER) EXIT

     ANG_A1 = ANG_A2 ;  SIN_A = DSIN(ANG_A2)
     ! COS_A = DCOS(ANG_A2)

     R_A = R_NEXT ;  Z_A = R_A - EARTH_RADIUS ; REFR_I = REFR_NEXT
     IT_1 = IT_2 ;  II_NEXT = II_NEXT - COS_SIGN
  END DO

  CALL ST_TRACK (R_NAME, 1, 'O')
  RETURN
201 Allocate(Msg(2))
  Msg(1)%S = 'Error in the refraction routine:'
  Write(StrLoc,*) II_NEXT
  Msg(2)%S = 'Non-existing element of IDX_REFR array is requested:'//Trim(StrLoc)
  call ErrMsg(R_Name, 'E', Msg)

  RETURN 1
880 WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)
  RETURN 1
END SUBROUTINE GEOM_ANGLE_V3

!***************************************************************************
FUNCTION REFR_INTERP_V2(Z_OUT)
  USE REFRACT_VARS, ONLY: REFR_IND
  USE TYPE_VARS,    ONLY: KND
  
  INTEGER       JJ
  REAL (KND)    DD, Z_OUT, REFR_INTERP_V2

  CALL GET_IDX_BELOW (Z_OUT, JJ, DD)
  REFR_INTERP_V2 = REFR_IND(JJ - 1)*DD + REFR_IND(JJ)*(1.0D0 - DD)
  
  RETURN
END FUNCTION REFR_INTERP_V2
!***************************************************************************
SUBROUTINE GET_IDX_BELOW (Z_OUT, IDX, DD)
  USE REFRACT_VARS, ONLY: Z_REFR, NLR_FINE, RHG_STEP
  USE ST_VARS,      ONLY: GROUND_HEIGHT
  USE TYPE_VARS,    ONLY: KND
  
  INTEGER       IDX
  REAL (KND)    DD, Z_TMP, Z_OUT
  
  Z_TMP = DDIM(Z_OUT,GROUND_HEIGHT)
  IDX = NLR_FINE - FLOOR(Z_TMP/RHG_STEP)
  DD = (Z_TMP + GROUND_HEIGHT - Z_REFR(IDX))/RHG_STEP
  
  RETURN
END SUBROUTINE GET_IDX_BELOW
!***************************************************************************
!SUBROUTINE TH_REFR_V2(Z, SIN_Z, H_0)
!
!  USE ST_VARS,      ONLY: EARTH_RADIUS
!  USE TYPE_VARS,    ONLY: KND
!  
!  REAL (KND)    H_0, H_1, Z, SIN_Z, REFR_IDX_TH, REFR_IDX_TH_1, REFR_IDX, ZZ
!  REAL (KND)    REFR_INTERP_V2
!  REAL (KND), PARAMETER :: CRIT = 1.0D-5
!
!  H_1 = -7777 
!  REFR_IDX = (Z + EARTH_RADIUS)*REFR_INTERP_V2(Z) 
!  DO WHILE(DABS(H_0 - H_1) > CRIT)
!     H_1 = H_0
!     REFR_IDX_TH = REFR_INTERP_V2(H_0)
!     H_0 = REFR_IDX*SIN_Z/REFR_IDX_TH - EARTH_RADIUS
!  END DO
!
!  IF(DABS(H_0 - Z) < CRIT) H_0 = Z
!  
!  RETURN
!END SUBROUTINE TH_REFR_V2
!***************************************************************************
SUBROUTINE TH_REFR_V2(Z, SIN_Z, H_0, *)

  USE ST_VARS,      ONLY: EARTH_RADIUS
  USE TYPE_VARS,    ONLY: KND

  CHARACTER (LEN = *), PARAMETER :: R_NAME = 'TH_REFR_V2'
  INTEGER     IT, II, IMIN, IMAX
  INTEGER, PARAMETER :: MAX_IT = 200
  REAL (KND)  TH(MAX_IT), RI_TH(0:MAX_IT)
  REAL (KND)  H_0, H_1, Z, SIN_Z, REFR_IDX_TH, REFR_IDX_TH_1, REFR_IDX, ZZ
  REAL (KND)  REFR_INTERP_V2, DH, DD, RI_1, R_TH, TH_MIN, TH_MAX, R_MIN, R_MAX
  REAL (KND)  D_1, D_2, T_1, T_2, MM
  REAL (KND), PARAMETER :: CRIT = 1.0D-7
  ! If CRIT is decreased, the parameter A_CRIT in refraction subroutine
  ! has to be decreased as well

  IT = 1 ; H_1 = -7777 ;  RI_TH(0) = 99999.
  REFR_IDX = (Z + EARTH_RADIUS)*REFR_INTERP_V2(Z)*SIN_Z
  DO WHILE(ABS(H_0 - H_1) > CRIT .AND. IT <= MAX_IT)
     H_1 = H_0
     REFR_IDX_TH = REFR_INTERP_V2(H_0) ;  TH(IT) = H_0
     RI_TH(IT) = REFR_IDX_TH*(TH(IT) + EARTH_RADIUS)
     !write(6,*) RI_TH(IT), H_0
     IF(ABS(RI_TH(IT) - REFR_IDX) > ABS(RI_TH(IT - 1) - REFR_IDX)) EXIT
     H_0 = REFR_IDX/REFR_IDX_TH - EARTH_RADIUS
     IT = IT + 1
  END DO
  IT = IT - 1
  
  ! If no convergence was reached within 10 iterations
  IF(ABS(H_0 - H_1) > CRIT) Then
     IMIN = MINLOC(ABS(REFR_IDX - RI_TH(1:IT)), 1, RI_TH(1:IT) < REFR_IDX)
     IMAX = MINLOC(ABS(REFR_IDX - RI_TH(1:IT)), 1, RI_TH(1:IT) > REFR_IDX)
     D_1 = RI_TH(IMIN) - REFR_IDX ; D_2 = RI_TH(IMAX) - REFR_IDX
     T_1 = TH(IMIN) ; T_2 = TH(IMAX)
    ! write(6,*) RI_TH(IMIN), RI_TH(IMAX), REFR_IDX
     IF(RI_TH(IMIN) < REFR_IDX.AND.RI_TH(IMAX) > REFR_IDX) THEN
        DO
           IT = IT + 1 
           ! Jumping around the minimum:
           ! use false position method (Anderson & Bjrk's algorithm)
           H_0 = T_2 - D_2*(T_2 - T_1)/(D_2 - D_1)
           ! write(6,*) 'H0', H_0
           R_TH = REFR_INTERP_V2(H_0)*(H_0 + EARTH_RADIUS)
           ! write(6,*) 'HH', REFR_IDX, R_TH, REFR_IDX - R_TH
           If((R_TH - REFR_IDX)*D_2 <= 0) Then
              D_1 = D_2 ;  D_2 = R_TH - REFR_IDX
              T_1 = T_2 ; T_2 = H_0
           Else
              MM = 1 - (R_TH - REFR_IDX)/D_2
              IF(MM < 0.0D0) MM = 0.5D0 
              D_1 = D_1*MM ;  D_2 = R_TH - REFR_IDX
              ! D_1 = D_1/2.0D0 ;  D_2 = R_TH - REFR_IDX
              T_1 = T_1 ;  T_2 = H_0
           End If
           IF(ABS(T_1 - T_2) < CRIT) Exit
        END DO
        ! write(6,*) IT, H_0, REFR_IDX - REFR_INTERP_V2(H_0)*(H_0 + EARTH_RADIUS)
        ! PAUSE
     ELSE
        CALL WRITE_MESSAGE(R_NAME, 'E', &
             'Refractive tangent height cannot be found !')
        RETURN 1
     END IF
     
  End If
  
  IF(DABS(H_0 - Z) < CRIT) H_0 = Z
  
  RETURN
END SUBROUTINE TH_REFR_V2
!***************************************************************************
SUBROUTINE REFRACTION(Z_I, SOL_COS, SOL_1, TH_OUT, SHADOW_FLG, *)
  
  USE LOG_PARS, ONLY: EU, LUNIT
  Use Tools,    only: ErrMsg
  USE TYPES,    ONLY: KND, DYN_CHAR
  USE ST_VARS,  ONLY: EARTH_RADIUS, PIE, GROUND_HEIGHT, R_GH
  
  LOGICAL       SHADOW_FLG
  REAL (KND)    SOL_COS, Z_I, SOL_1, TH_OUT

  ! Local parameters
  CHARACTER (LEN = *), PARAMETER :: R_NAME = 'REFRACTION'
  CHARACTER (LEN=200) StrLoc
  
  INTEGER     N_ITER, II
  INTEGER, PARAMETER :: MAX_ITER = 200
  LOGICAL     FLAG_M, FLAG_P, TIR_FLG, WR_FLG, CheckLow, CheckHigh
  
  REAL (KND)  TH_2, DIFF, REFR_1, REFR_2, SOL_2, SOL_G, SOL_T, DI, FI_1,&
       FI_2, DIFF_OLD, SOL_NEW, REFR_INTERP_V2, S_1, S_2, D_1, D_2, FI_3, &
       SGN, MM, SOL_1a, Sol_1b, FI_1a, FI_1b, A_STEP, StepF
  REAL (KND), PARAMETER :: B_STEP = 1.0D-4, A_CRIT = 1.0D-6
  TYPE (DYN_CHAR), ALLOCATABLE :: Msg(:)
  ! Parameter A_CRIT is coupled with the accuracy of the tangent height
  ! determination in th_refr_v2
  
  ! ******************** Attention !!!! ************************
  ! SOL_COL is the cosine of the geometrical solar zenith angle
  ! SOL_1 is the first guess for the solar zenith angle. 
  ! This is not necessary the same angle!
  !==================================================================
     
  SOL_G = DACOS(SOL_COS)    ! Must be the geometrical angle
      
  REFR_1 = REFR_INTERP_V2(GROUND_HEIGHT)
  REFR_2 = REFR_INTERP_V2(Z_I)
  ! SOL_T = PIE - DASIN(REFR_1*R_GH/REFR_2/R_I)
  SOL_T = PIE - ASIN(REFR_1/REFR_2*R_GH/(Z_I + EARTH_RADIUS))
  CALL GEOM_ANGLE_V3 (Z_I, SOL_T, FI_1, TH_OUT, SHADOW_FLG, TIR_FLG, *880)
  IF (TIR_FLG) CALL WRITE_MESSAGE(R_NAME, 'E', 'Problem in the '&
             //'refraction routine: angle is unreachable !')

  IF(SOL_G > FI_1) THEN
     TH_OUT = - 7777.
     SOL_1 = Pie ;  SOL_COS = - 1.0D0
     SHADOW_FLG  = .TRUE. ! Darkness
     RETURN
  END IF
  SOL_1 = MIN(SOL_1, SOL_T)

!  write(6,*) 'SOL_G, Z_I ======> ', SOL_G, Z_I
  N_ITER = 1 ; DIFF_OLD = 999. ;  FLAG_P = .FALSE. ;  FLAG_M = .FALSE.
  WR_FLG = .FALSE.
  A_STEP = 1.0D-3
  DO
     CALL GEOM_ANGLE_V3 (Z_I, SOL_1, FI_1, TH_OUT, SHADOW_FLG, TIR_FLG, *880)
     If (TIR_FLG) Then
        WR_FLG = .TRUE.
        SOL_1a = SOL_1 ;  SOL_1b = SOL_1
        Do While (TIR_FLG)
           SOL_1a = SOL_1a + B_STEP
           CALL GEOM_ANGLE_V3 (Z_I, SOL_1a, FI_1, TH_OUT, SHADOW_FLG, TIR_FLG, *880)
           FI_1a = FI_1
        End Do
        TIR_FLG = .TRUE.
        Do While (TIR_FLG)
           SOL_1b = SOL_1b - B_STEP
           CALL GEOM_ANGLE_V3 (Z_I, SOL_1b, FI_1, TH_OUT, SHADOW_FLG, TIR_FLG, *880)
           FI_1b = FI_1
        End Do
        IF(ABS(SOL_G - FI_1a) < ABS(SOL_G - FI_1b)) THEN
           SOL_1 = SOL_1a ; FI_1 = FI_1a
        ELSE
           SOL_1 = SOL_1b ; FI_1 = FI_1b
        END IF
!        pause
     End If
     IF(SHADOW_FLG) THEN
        SOL_T = MIN(SOL_T,SOL_1)
     END IF
           
     DIFF = SOL_G - FI_1
     !write(6,*) "DIFF >>>>", SOL_G/3.14*180., FI_1/3.14*180., FI_1
     IF(DABS(DIFF) < A_CRIT) EXIT

     ! Saving interval bounaries to initialise the false position method
     IF(DIFF > 0.0D0) THEN  ! The angle is above SOL_G
        IF(.NOT.FLAG_P) THEN
           D_1 = DIFF ;  S_1 = SOL_1 ;  FLAG_P = .TRUE.
        END IF
     ELSE                   ! The angle is below SOL_G
        IF(.NOT.FLAG_M) THEN
           D_2 = DIFF
           S_2 = SOL_1
           FLAG_M = .TRUE.
        END IF
     END IF

     IF(FLAG_P.AND.FLAG_M) THEN
        ! False position method (Anderson & Bjrk's algorithm)
        If(Diff*D_2 <= 0) Then
           D_1 = D_2 ;  D_2 = Diff
           S_1 = S_2 ; S_2 = SOL_1
        Else
           MM = 1 - Diff/D_2
           IF(MM < 0.0D0) MM = 0.5D0 
           D_1 = D_1*MM ;  D_2 = Diff
           S_1 = S_1 ;  S_2 = SOL_1
        End If
        SOL_NEW = S_2 - D_2*(S_2 - S_1)/(D_2 - D_1)        
     ELSE
        ! Numerical derivative 
        Do
           ! If the stepsize is too small, i.e. FI_2 - FW_1 << DIFF there is a 
           ! high risk of the non-linearity. We repeat the procedure with an 
           ! increased sepsize. Same if the stepsize is too large. This loop 
           ! optimizes the stepsize
           StepF = 4.0E+0 ; CheckLow = .False. ;  CheckHigh = .False.
           Do 
              SOL_2 = SOL_1 - A_STEP
              Call Geom_Angle_V3(Z_I, SOL_2, FI_2, TH_2, SHADOW_FLG, &
                   TIR_FLG, *880)
              If(Abs(FI_2 - FI_1) < Abs(Diff)/5.0E0) Then
                 CheckLow = .True.
                 If (CheckHigh) Then
                    StepF = StepF/2.0D0
                    CheckHigh = .False.
                 End If
                 A_Step = A_Step*(1.0D0 + StepF)
                 Cycle
              End If
              If(Abs(FI_2 - FI_1) > Abs(Diff)*5.0E0) Then
                 CheckHigh = .True.
                 If (CheckLow) Then
                    StepF = StepF/2.0D0
                    CheckLow = .False.
                 End If
                 A_Step = A_Step/(1.0D0 + StepF)
                 Cycle
              End If
              Exit
           End Do

           !write(6,*) "Sol_2a: ", (SOL_1 - A_Step)/3.14*180, FI_2/3.14*180, (FI_2 - FI_1), A_Step/(FI_2 - FI_1)*DIFF, &
           !     DIM(SOL_1 + (SOL_2 - SOL_1)/(FI_2 - FI_1)*DIFF,0.0D0)/3.14*180.
          If (.NOT.TIR_FLG) EXIT
          SOL_2 = SOL_2 - A_STEP
        End Do

        IF(DIFF*(SOL_G - FI_2) < 0) Then
           SOL_NEW = SOL_2
        ELSE
           IF(FI_2 > FI_1) THEN
              ! SOL_1 in the local minimum
              SGN = SIGN(1.0D0, DIFF) ;  FI_3 = FI_1 ;  SOL_NEW = SOL_1
              Do 
                 SOL_NEW = SOL_NEW + SGN*A_STEP
                 CALL GEOM_ANGLE_V3(Z_I, SOL_NEW, FI_3, TH_2, &
                      SHADOW_FLG, TIR_FLG, *880)
                 If (TIR_FLG) Cycle
                 If ((SOL_G - FI_3)*Diff < 0.0D0) Exit
              End Do
           ELSE
              ! We do not allow negative values 
              SOL_NEW = DIM(SOL_1 + (SOL_2 - SOL_1)/(FI_2 - FI_1)*DIFF,0.0D0)
           END IF
        END IF
     END IF
     
     !write(6,*) "SOL_NEW: ", SOL_NEW/3.14*180., SOL_NEW
     
     ! We do not allow SOL_NEW to be in the darkness
     IF(SOL_NEW >= SOL_T) SOL_NEW = SOL_T
     
     ! write(6,*) 'SSSSSS', SOL_NEW, SOL_1
     ! write(6,*) 'SSSSSS_2', DIFF_OLD, DIFF
     IF(SOL_NEW == SOL_1) THEN
        ! No more change in the local angle: 
        IF(DIFF_OLD/DIFF < 0.0) THEN
           ! The zero is inbetween -> insufficien calculation accuracy
           ! We keep the value of the angle write warning and continue
           Allocate(Msg(3))
           Msg(1)%S = 'The geometrical angle can not be reached !'
           Write(StrLoc,*) A_CRIT
           Msg(2)%S = 'Required accuracy:'//Trim(StrLoc)
           Write(StrLoc,*) DIFF_OLD, DIFF
           Msg(3)%S = 'Residuals:'//Trim(StrLoc)
           call ErrMsg(R_Name, 'W', Msg)
           EXIT
        END IF
        ! Here something is really wrong, debugging is needed if occurs
        CALL WRITE_MESSAGE(R_NAME, 'E', &
             'No convergence in the refraction routine !')
        RETURN 1
     END IF
     DIFF_OLD = DIFF
     SOL_1 = SOL_NEW
     IF(WR_FLG) Then
        write(LUNIT,*) '***** SOL_G ****', SOL_1, FI_1, SOL_G !, SOL_G/PIE*180. 
        write(LUNIT,*) '***** DIFF *****', Z_I, N_ITER, DIFF
     End If
     
     N_ITER = N_ITER + 1
     
     IF(N_ITER > MAX_ITER) THEN
        CALL WRITE_MESSAGE(R_NAME, 'E', 'Problem in the '&
             //'refraction routine, iteration limit is reached !')
        RETURN 1
     END IF
     
  END DO
  
!     IF(Z_I > 15.271 .and. Z_I < 15.273 .and. SOL_G > 1.61781 .and. SOL_G < 1.61782) then
  IF(WR_FLG) Then
     write(LUNIT,*) '***** Final DIFF *****', Z_I, N_ITER, SOL_1, DIFF
!     PAUSE
  End If
!  End if
  
  ! write(6,*) '***** SOL_1 ****', SOL_1
  ! write(6,*) '***** FI_1 ****', FI_1
  ! write(6,*) '***** SOL_G ****', SOL_G, SOL_G/PIE*180. 
  ! write(6,*) '***** DIFF, DELTA *****', DIFF

  SOL_COS = COS(SOL_1)
  IF(TH_OUT < GROUND_HEIGHT.AND.SOL_COS < 0.0D0) THEN
     SHADOW_FLG  = .TRUE.
  ELSE
     SHADOW_FLG  = .FALSE.
  END IF

  RETURN
  
880 WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)
  RETURN 1
END SUBROUTINE REFRACTION
!***************************************************************************
 

