!***************************************************************************
!
!	SCIATRAN - Version 3.8 - Last modification: 01 Aug 2017
!
!	Copyright (C) - Institute of Remote Sensing,
!			University of Bremen, Germany
!
!***************************************************************************
SUBROUTINE SC_AER_SETUP( START_WAV, END_WAV,* )
  ! Called by "rtm_init.f"

  USE AER_MIE,     ONLY: DENS_PRF
  USE AEROSOLS,    ONLY: AER_SHIFT, PATH_TO_SCM, Plus_mi, Nga_db, OUT_A_SCM, &
                         LAY_A, WEL_A, NSCA_A, GE_A, APH_VS, NAW => N_AER_WAVS, &
                         AER_WAVS, AER_COEFF, NG_aer, NWLS, NWLE, LA_N, LA_S, &
                         LA_E, L_TOP, LA_Ndb, SCI => GT_SCAT_IDX, Greek_Aer, &
                         ASI => GT_ASYM_IDX, Scal_aer, AOT_out_VAL, EXI => GT_EXTC_IDX, &
                         Rtau_aer, Rwl_aer, AER_SCALING, IDX_AER, IDX_AER_mi, IDX_AER_db, &
                         L_scal, ZS1, ZS2, AER_FACT, V_PAP
  USE CLOUDS_SS,   ONLY: MUM => MAXMOM, DO_SSRCA
  USE DELTA_M,     ONLY: FDM_AER, AER_DM_FLG, AER_TRUNC_IDX
  USE GRIDS,       ONLY: NLAYERS, ZHOLD => ZZZ, DZ
  USE GT_POLYN,    ONLY: NLEGEN
  USE INFO,        ONLY: AER_INFO
  USE INTERN_DIMS, ONLY: Nm_vs
  USE LOG_PARS,    ONLY: LUNIT, EU, DIR_OUT, LOG_LEVEL, N_SSO, N_ESO, N_DSO, Work_Dir
  USE RTM_PARS,    ONLY: DATA_ITER
  USE ST_INPUT,    ONLY: ADB => PATH_AER_DB, APH_TYPE
  USE ST_VARS,     ONLY: RTM_TYPE, GROUND_HEIGHT
  USE TOOLS,       ONLY: CHK_STAT, SET_PTR, CUT_STR, SPLIT_STRING, Read_Input
  USE TYPES,       ONLY: KND, DYN_CHAR
  
  !  (a) Input variables
  !  ===================
  !      APH_TYPE          : type of phase function type
  !                          'mie' = Mie, 'h-g' = Henyey-Greenstein 
  !      AER_DM_FLG        : delta-M approximation yes/no 
  !                          <---- From Module DELTA_M
  !      AER_TRUNC_IDX     : truncation index for delta-M approximation 
  !                          <---- From Module DELTA_M
  !      ZHOLD             : forward model altitude grid
  !      NLAYERS           : number of layers in altitude grid
  !      NLEGEN            : number of Legendre moments
  !      LUNIT             : LOG file unit number
  !      ADB               : path to data base
  
  !  (b) Output variables
  !  ====================
  !      AER_COEFF         : scattering/extinction coefficients and 
  !                          asymmetry parameters 
  !      AER_PHASMOMS      : Legendre moments
  !      NAW               : number of wavelengths of chosen aerosol 
  !                          parameterization
  !      AER_WAVS          : wavelengths of aerosol data base (depends on
  !                          parameterization chosen)
  
  !  (c) Local variables for SCIATRAN parameterization
  !  =================================================
  !      LAN              : number of aerosol layers
  !      L_TOP             : upper boundary [km] of aerosol layer
  !      RELHUM            : relative humidity [%] of aerosol layer
  !      AER_TYPE          : aerosol type 
  !      NAC               : number of aerosol components of each aerosol 
  !                          type 
  !      ACI               : id-numbers of aerosol components (1-12)
  !      RD                : relative particle number densities of the
  !                          various aerosol components of an aerosol type
  !      PRF_TYPE          : aerosol profile type
  !                          'exti' - extinction coefficient [1/km]  
  !                          'part' - particle number density [1/cm^3]
  !      PRF_LAMBDA        : wavelength at which the extinction profile is
  !                          given
  !      N_AER_LEV         : number of aerosol profile levels     
  !      AER_LEV           : aerosol profile levels
  !      AER_PRF           : aerosol profile (extinction coefficient or
  !                          particle number density)
  !      NL_DB             : number of moments in the data basis 
  !      M_H               : number of humidities 
  !      M_CH              : number of aerosol components 
  !                          depending on humidity
  !      M_CI              : number of aerosol components
  !                          independent on humidity
  
  ! Local parameters
  LOGICAL               ERROR
  CHARACTER (LEN = *), PARAMETER :: R_NAME = 'SC_AER_SETUP'

  INTEGER    NL_DB, M_H, M_CH, M_CI, IUNIT, N_DIM, L_REF
  PARAMETER  (M_H = 8, M_CH = 4, M_CI = 8)
  INTEGER    I, I1, J, K, K1, Ko, L, Itr, N_AER_LEV, LAN
  CHARACTER(LEN=:),  ALLOCATABLE :: PRF_TYPE, STR_INP
  CHARACTER(LEN=30), ALLOCATABLE :: AER_TYPE(:)
  CHARACTER(LEN=3)   SNA(M_CH+M_CI)
  REAL (KND)  DB_HUMS(M_H), TMP1(2*M_H)
  DATA DB_HUMS  /0., 50., 70., 80., 90., 95., 98., 99./
  DATA SNA     /'-01','-02','-03','-04','-05','-06','-07', '-08','-09','-10','-11','-12'/
  INTEGER  NWL_DB, M_CO, N_SKIP, Ig
  REAL (KND) START_WAV, END_WAV, PRF_LAMBDA, F_DM, DUM, Scoe, DD1, DD2, H_PRF(NLAYERS)
  INTEGER, ALLOCATABLE :: LAS(:), LAE(:), IDA(:), NAC(:), ACI(:,:), BACI(:,:), &
       NH(:), N_MAX(:)
  REAL (KND), ALLOCATABLE :: ECO(:,:), SCO(:,:), USER_MOM(:,:,:,:), AER_LEV(:), &
       AER_PRF(:), REL_HUM(:), RD(:,:), ASYM_PAR(:,:), D1(:), TAU(:), ECO_REF(:), &
       DB_EXT_REF(:,:), DB_WAVS(:), DB_EXT(:,:,:), DB_SCA(:,:,:), DB_MOMS(:,:,:,:)
  REAL (KND), POINTER :: AER_PHASMOMS(:,:,:) 
  TYPE (DATA_ITER), POINTER :: RTM_PTR
  TYPE (DYN_CHAR), ALLOCATABLE :: STR_ARR(:), READ_ARR(:), STR_ARR_1(:)
  !=========================================================================

  ! ===================================
  !  SCIATRAN AEROSOL PARAMETERIZATION
  ! ===================================
  CALL ST_TRACK (R_NAME, 1, 'I')
      
  CALL DEALLOCATE_ARRAYS ! All arrays allocated below must be 
                         ! conditionally deallocated here
  IF(CHK_STAT('aer-dens',AER_INFO%NAME,AER_INFO%WF_FLG) ) THEN
     ALLOCATE( DENS_PRF(NLAYERS) )
     DENS_PRF(:) = 1.d0
     CALL SET_PTR( RTM_PTR, 'aer-dens' )
     RTM_PTR%APR(:) = DENS_PRF(:)
     RTM_PTR%CRN(:) = DENS_PRF(:)    
  ENDIF

  ERROR = .FALSE.

  ! Reading of wavelength grid
  ! ---------------------------
  OPEN(NewUnit=IUNIT,FILE = TRIM(ADB)//'DB_SC_WL.dat',STATUS='OLD')
  DO I = 1,5
     READ(IUNIT,*)
  ENDDO
  READ(IUNIT,*) NWL_DB
  ALLOCATE( DB_WAVS(NWL_DB), N_MAX(NWL_DB) )
  DO I = 1,NWL_DB
     READ(IUNIT,*) DB_WAVS(I), N_MAX(I)
  ENDDO
  CLOSE(IUNIT)
  IF( DB_WAVS(NWL_DB) < END_WAV ) GOTO 128
  
  ! Find position of start- and end-wavelengths of current net in
  ! wavelength array of aerosol data
  ! -------------------------------------------------------------
  IF( START_WAV < DB_WAVS(1) ) THEN
     NWLS = 1 
     CALL WRITE_MESSAGE(R_NAME,'WARNING',' ')
     WRITE(EU,'(A)') '    Required wavelength range is outside of the'//&
          ' aerosol data base.'
     WRITE(EU,'(A,2x,F9.2)') '    First wavelength in data base:', DB_WAVS(1)
     WRITE(EU,'(A,2x,F9.2)') '    First required wavelength    :', START_WAV
     WRITE(EU,*) '  !!! The extrapolation with constant is used !!!'
  ELSE
     CALL LIN_SEARCH (START_WAV, DUM, NWLS, NWL_DB, DB_WAVS, ERROR)
     IF (ERROR) GOTO 127
  ENDIF

  CALL LIN_SEARCH (END_WAV, DUM, NWLE, NWL_DB, DB_WAVS, ERROR)
  IF (ERROR) GOTO 127
  NWLE = NWLE + 1
  IF( DB_WAVS(NWLE) .EQ. END_WAV ) NWLE = NWLE + 1
  
  NAW = NWLE - NWLS + 1 

  !========================================================
  !    Read scenario input parameters file "scai_aer.inp" 
  !========================================================
  CALL READ_SCIA_AER_INP(*880, *122)
  NL_DB = N_MAX(NWLS)
  N_DIM = MAX0(NL_DB,NLEGEN)

  ALLOCATE( APH_VS(NLEGEN,LAN,Nm_vs,NAW), AER_WAVS(NAW), & ! A
       AER_COEFF(3,NLAYERS,NAW), USER_MOM(N_DIM,LAN,Nm_vs,NAW), D1(NAW), & ! A
       ECO(LAN,NAW), SCO(LAN,NAW), TAU(NAW), ECO_REF(LAN), LAS(LAN), & ! A
       LAE(LAN), ASYM_PAR(LAN,NAW), IDA(NLAYERS) )            ! A

  USER_MOM = 0.d0  ;  APH_VS = 0.d0
  
  AER_WAVS(:) = DB_WAVS(NWLS:NWLE)
  AER_PHASMOMS => APH_VS( :,:,1,: )
  AER_COEFF = 0.d0 
  
  !======================================================================
  !Conversion from input component indices to data base component indices
  !======================================================================
  NH = 0
  DO I = 1,LAN
     DO J = 1,M_H
        IF( REL_HUM(I) == DB_HUMS(J) ) NH(I) = J
     END DO
     IF(NH(I) == 0) GOTO 123
  ENDDO
  
  DO I = 1,LAN
     DO J = 1,NAC(I)  ;  K1 = ACI(I,J)
        IF( K1 <= 4 ) THEN
           BACI(I,J) = M_H*(K1 - 1) + NH(I)
        ELSE
           BACI(I,J) = (M_H - 1)*M_CH + K1
        ENDIF
     ENDDO
  ENDDO

  !=====================================================
  !   Interpolate input profile AER_PRF to "NLAYERS" layers
  !=====================================================
  !  AER_PRF( 1:N_AER_LEV ) contains the aerosol input profile
  !  ( AER_PRF(1) corresponds to the earth surface ).
  !  H_PRF(1:NLAYERS) contains the aerosol profile 
  !  interpolated to the forward model altitude grid 
  !  ( H_PRF(NLAYERS) corresponds to the earth surface ).
  CALL LIN_INT(N_AER_LEV, AER_LEV, AER_PRF, NLAYERS, ZHOLD, H_PRF, 'Z')

  IF( L_Top(LAN-1) > ZHOLD(1) ) GOTO 125
  IF( L_Top(1) <= ZHOLD(NLAYERS) ) GOTO 126

  ! Setup layer boundaries:
  ! -------------------------
  DO I = 1,LAN
     DUM = L_Top(LAN-I+1)
     LAS(I) = MINLOC( DABS(DUM-ZHOLD(:)),1 )
  ENDDO
  DO I = 1,LAN-1
     LAE(I) = LAS(I+1)-1
  ENDDO
  LAE(LAN) = NLAYERS

  ! Indexes of altitude grid levels which are above of first
  ! aerosol layers
  IDA(:) = 1
  ! Indexes of altitude grid levels which are within aerosol layers
  IF( LAN > 1 ) THEN
     DO I = 1,LAN - 1
        DD1 = ZHOLD(LAS(I)) ;  DD2 = ZHOLD(LAS(I+1))
        DO J = 1,NLAYERS
           IF( ZHOLD(J)<=DD1 .AND. ZHOLD(J)>DD2 ) IDA(J)=I
        ENDDO
     ENDDO
  ENDIF
  DD1 = ZHOLD(LAS(LAN))
  ! Indexes of altitude grid levels which are below of last aerosol layer
  DO J = 1,NLAYERS
     IF( ZHOLD(J) .LE. DD1 ) IDA(J) = LAN
     !         WRITE(*,'(2x,F7.3,2x,I2)') ZHOLD(J), IDA(J)
  ENDDO

  IF(LOG_LEVEL(1) >= N_DSO) THEN
     WRITE(LUNIT,*) ' ------ Test of aerosol layering  -------- '
     DO I = 1,LAN
        WRITE(LUNIT,'(2x,I3,2(1x,f7.3))') I, ZHOLD(LAS(I)) ,ZHOLD(LAE(I)) 
     ENDDO
     WRITE(LUNIT,*) ' ----------------------------------------- '
  ENDIF

  ! Calculate extinction and scattering coefficients 
  ! (norm. to 1 particle /cm^3) of component mixture --> ECO, SCO
  ! =====================================================
  ! Maximal number of aerosol components
  M_CO = MAXVAL( NAC(:) )

  ALLOCATE( DB_EXT(LAN,M_CO,NAW), DB_SCA(LAN,M_CO,NAW), DB_EXT_REF(LAN,M_CO), &!A
       DB_MOMS(LAN,M_CO,NAW,N_MAX(NWLS)) ) ! A

  DB_MOMS = 0.d0

  ! Find position of the reference wavelength PRF_LAMBDA 
  ! within database wavelength grid
  ! --------------------------------------------------
  L_REF = -1
  IF( PRF_TYPE == 'exti' ) THEN             
     DO I = 1,NWL_DB
        IF( REAL(PRF_LAMBDA) == REAL(DB_WAVS(I)) ) THEN
           L_REF = I ; EXIT
        ENDIF
     END DO
     IF( L_REF < 1 .OR. L_REF > NWL_DB ) GOTO 124
  ENDIF

  ! Reading aerosol parameters from data basis:
  ! -------------------------------------------
  CALL READ_DB_SC_PAR

  ! Extinction and scattering coefficients of a given aerosol component mixture:
  ! ----------------------------------------------------------------------------
  ! WRITE(*,*) '    ECO and SCO      '
  ECO = 0.d0 ; SCO = 0.d0
  DO I = 1,LAN
     DO J = 1,NAC(I)
        ECO(I,:) = ECO(I,:) + DB_EXT(I,J,:)*RD(I,J)
        SCO(I,:) = SCO(I,:) + DB_SCA(I,J,:)*RD(I,J)
     ENDDO
     !     WRITE(*,'(2x,I3,2(2x,E12.4))')  I, ECO(I,1), SCO(I,1)
  ENDDO
  
  ! Convert extinction profile to particle density profile:
  ! ---------------------------------------------------------
  IF( PRF_TYPE == 'exti' ) THEN             
     ! Calculate particle number densities:
     ! ------------------------------------
     ECO_REF = 0.d0
     DO I = 1,LAN
        DO J = 1,NAC(I)
           ECO_REF(I) = ECO_REF(I) + DB_EXT_REF(I,J)*RD(I,J)
        ENDDO
        DO J = LAS(I),LAE(I)
           H_PRF(J) = H_PRF(J)/ECO_REF(I)
        ENDDO
     ENDDO
  ENDIF

  DO I = NLAYERS,1,-1
     WRITE(99,'(1x,F7.2,2x,E12.4)') ZHOLD(I), H_PRF(I)
  ENDDO


  ! Multiply ECO and SCO with particle number density profile:
  ! ----------------------------------------------------------  
  DO I = 1,LAN
     ! WRITE(*,*)  'Layer - ', I, ECO(I,1)
     DO J = LAS(I),LAE(I)
        AER_COEFF(EXI,J,:) = ECO(I,:)*H_PRF(J)
        AER_COEFF(SCI,J,:) = SCO(I,:)*H_PRF(J)
     ENDDO
  ENDDO
  
  IF( L_scal ) THEN
     ! Scaling of number density between ZS1 - ZS2
     DO I = 1, NAW
        WHERE( ZHOLD(:) >= ZS1.AND.ZHOLD(:) <= ZS2 ) 
           AER_COEFF(SCI,:,I) = AER_FACT*AER_COEFF(SCI,:,I) 
           AER_COEFF(EXI,:,I) = AER_FACT*AER_COEFF(EXI,:,I) 
        END WHERE
     ENDDO
     ! Scaling of the scattering coefficient between ZS1 - ZS2
     ! in order to change the single scattering albedo as follows:
     !    SSA' = 1 - V_PAP*(1 - SSA) 
     ! Extinction coefficient remains the same
     DO I = 1, NAW
        WHERE( ZHOLD(:) >= ZS1.AND.ZHOLD(:) <= ZS2 ) 
           AER_COEFF(SCI,:,I) = AER_COEFF(SCI,:,I) - (V_PAP - 1.d0) &
                *(AER_COEFF(EXI,:,I ) - AER_COEFF(SCI,:,I ))
        END WHERE
     ENDDO
  ENDIF

  !  WRITE(*,*) '  Before delta-M  '
  !  WRITE(*,*) '  Scat ',  AER_COEFF(SCI,1:2,1)
  !  WRITE(*,*) '   Ext ',  AER_COEFF(EXI,1:2,1)
  
  ! ===============================================
  !    Calculate profiles of Mie Legendre moments 
  !    and of asymmetry parameter
  ! ===============================================
  IF ( APH_TYPE == 'mie' ) THEN            
     DO I = 1,LAN 
        DO L = 1,NL_DB  ;  D1(:) = 0.d0
           DO J = 1,NAC(I) 
              D1(:) = D1(:) + DB_MOMS(I,J,:,L)*DB_SCA(I,J,:)*RD(I,J)
           ENDDO
           USER_MOM(L,I,1,:) = D1(:)*DBLE(2*L-1)/SCO(I,:)
        ENDDO
        ASYM_PAR(I,:) = USER_MOM(2,I,1,:)/3.d0
     ENDDO                  ! End loop over aerosol layers
  ENDIF

  !============================================================
  !   Calculate profiles of Henyey-Greenstein Legendre moments
  !   and of asymmetry parameter
  !============================================================
  IF ( APH_TYPE == 'h-g' ) THEN
     DO I = 1,LAN  ;  D1(:) = 0.D0  
        DO J = 1,NAC(I) 
           D1(:) = D1(:) + DB_MOMS(I,J,:,2)*DB_SCA(I,J,:)*RD(I,J)
        ENDDO
        D1(:) = D1(:)/SCO(I,:)  ;  ASYM_PAR(I,:) = D1(:)
        DO L = 1,NL_DB
           USER_MOM(L,I,1,:) = D1(:)**(L-1)*DBLE(2*L-1)
        ENDDO
     ENDDO                  ! End loop over aerosol layers           
  ENDIF

  ! Single scattering correction algorithm -> save all expansion coeff.
  ! --------------------------------------    ------------------------
  IF( DO_SSRCA .OR. TRIM(RTM_TYPE) == 'spher_scat' ) THEN
     Nga_db = NL_DB
     IF (LOG_LEVEL(1) >= N_ESO) WRITE(LUNIT,*) '  Allocate Greek_aer :', Nga_db
     ALLOCATE( Greek_aer( Nga_db,LAN,Nm_vs,NAW ) )
     Greek_aer = 0.d0
     DO I = 1,NAW 
        Greek_aer(:,:,:,I) = USER_MOM(1:Nga_db,:,:,I)
     ENDDO
     IF( .NOT.Plus_mi ) NG_aer = Nga_db
  ENDIF

  IF( OUT_A_SCM ) THEN 
     IF( LAY_A > LAN ) GOTO 130
     IF( WEL_A > NAW )  GOTO 131
     CALL COEF_TO_SCATMATR(NL_DB, Nm_vs, USER_MOM(1:NL_DB,LAY_A,1:Nm_vs,WEL_A),&
          NSCA_A, GE_A, PATH_TO_SCM)
  ENDIF

  DO I = 1,LAN 
     DO J = LAS(I),LAE(I)
        AER_COEFF(ASI,J,:) = ASYM_PAR(I,:)
     ENDDO
  ENDDO

  DO K = 1,NAW
     TAU(K) = SUM( AER_COEFF(EXI,:,K)*DZ(:) )/2.0d0
  ENDDO

  IF( Scal_aer ) THEN
     CALL LIN_SEARCH(Rwl_aer, DUM, K, NAW, AER_WAVS, ERROR)
     IF (ERROR) GOTO 129
     Scoe = Rtau_aer/( (1.D0 - DUM)*TAU(K) + DUM*TAU(K+1) )
     AER_COEFF(EXI,:,:) = Scoe*AER_COEFF(EXI,:,:)
     AER_COEFF(SCI,:,:) = Scoe*AER_COEFF(SCI,:,:)
     TAU(:) = Scoe*TAU(:)
  ENDIF
  AOT_out_VAL = TAU(1)

  !======================================
  !   Write aerosol scenario output file
  !======================================
  CALL WRITE_SCEN_SA(*880)

  L_TOP(:) = L_TOP(LAN:1:-1)

  ! Allocate array for MODULE DELTA_M
  ! =================================
  ALLOCATE ( FDM_AER(NLAYERS,NAW) ) ! A
  FDM_AER = 0.d0
  IF( AER_DM_FLG  ) THEN 
     Itr = AER_TRUNC_IDX
     CALL AER_DM
  ELSE
     APH_VS(:,:,:,:) = USER_MOM(1:NLEGEN,:,:,:)
  ENDIF
      
  LA_Ndb = LAN
  IF( .NOT.Plus_mi ) THEN
     LA_N = LAN
     ALLOCATE( LA_S(LAN), LA_E(LAN), IDX_AER(NLAYERS) )
     LA_S = LAS ;  LA_E = LAE  ;  IDX_AER = IDA
  ELSE
     ALLOCATE( IDX_AER_db(NLAYERS) )
     IDX_AER_db = IDA
  ENDIF

  CALL ST_TRACK (R_NAME, 1, 'O')

  RETURN

122 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Particle number densities not equal 1')
  WRITE(EU,*) '    for layer: ', I
  WRITE ( EU, * ) '    Check aerosol input file!'
  RETURN 1
  
123 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Selected humidity value not allowed (aerosols)')
  WRITE(EU,*) '    ', REL_HUM (LAN-I+1), ' % not allowed'
  WRITE ( EU, * ) '    Check aerosol input file!'
  RETURN 1

124 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Reference wavelength (see "scia_aer.inp") equal to ')
  WRITE(EU,'(a,F7.2,a)') '    ', PRF_LAMBDA, &
       ' does not coincide with the standard wavelength grid.'
  RETURN 1
      
125 CALL WRITE_MESSAGE(R_NAME, 'ERROR', '')
  WRITE(EU,*) '    Upper aerosol layer outside the atmosphere.'
  WRITE(EU,'(A,2x,F9.2,A,F9.2)') '    Upper aerosol layer:', L_TOP(LAN-1),' - ',L_TOP(LAN)
  WRITE(EU,'(A,2x,F9.2,A)') '      Top of atmosphere:', ZHOLD(1),' km'
  WRITE(EU,*) '    Check "scia_aer.inp" and altitude grid '
  WRITE(EU,*) '    as defined in the file "control.inp" '
  RETURN 1

126 CALL WRITE_MESSAGE(R_NAME, 'ERROR', '')
  WRITE(EU,*) '    Lower aerosol layer outside the atmospherea.'
  WRITE(EU,'(A,2x,F6.2,A)') '       Top of aerosol layer:', L_TOP(1),' km'
  WRITE(EU,'(A,2x,F6.2,A)') '       Surface elevation   :', ZHOLD(NLAYERS),' km'
  WRITE(EU,*) '    Check "scia_aer.inp" and "Height above '
  WRITE(EU,*) '    sea level" in the file "control.inp" '
  RETURN 1

127 CALL WRITE_MESSAGE (R_NAME, 'ERROR', 'Error in LIN_SEARCH')
  RETURN 1

128 CALL WRITE_MESSAGE(R_NAME, 'ERROR', &
         'Required wavelength range is outside of the data base.')
  WRITE(EU,'(A,2x,F9.2)') '    Last wavelength in data base:', DB_WAVS(NWL_DB)
  WRITE(EU,'(A,2x,F9.2)') '    Last required wavelength    :', END_WAV
  WRITE(EU,*) '    Use another aerosol data base. '
  WRITE(EU,*) '    Sea the file "control_aer.inp" '
  RETURN 1

129 CALL WRITE_MESSAGE (R_NAME,'ERROR',&
         'Reference wavelength for AOT is out of range')
  WRITE(EU,'(A,F7.2)') '    Reference wavelength: ', Rwl_aer
  WRITE(EU,'(A,2(2x,F7.2))') '    Database wavelengths: ', AER_WAVS(1), AER_WAVS(NAW)
  WRITE(EU,'(A)') '    Set reference wavelength between database wavelengths.'
  RETURN 1
  
130 CALL WRITE_MESSAGE (R_NAME,'ERROR', 'Required number of aerosol layer is out of range')
  WRITE(EU,'(A,I3)') '    Number of aerosol layers: ', LAN
  WRITE(EU,'(A,I3)') '    Required layer number   : ', LAY_A
  RETURN 1

131 CALL WRITE_MESSAGE (R_NAME,'ERROR', 'Required number of wavelength is out of range')
  WRITE(EU,'(A,I3)') '    Number of wavelengths for aerosol: ',NAW
  WRITE(EU,'(A,I3)') '    Required wavelength number       : ',WEL_A 
  RETURN 1
      
880 WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)
  RETURN 1
  
CONTAINS
  !***********************************************************************
  SUBROUTINE READ_SCIA_AER_INP(*,*)
    CHARACTER (LEN = *), PARAMETER :: R_NAME = 'READ_SCIA_AER_INP'
    INTEGER            K1, II
    DOUBLE PRECISION   S_UM
      
  STR_INP  = 'Aerosol layers (SCIA_AER)'
  CALL Read_Input (STR_INP, READ_ARR, *880)
  CALL SPLIT_STRING (READ_ARR(1)%S, STR_ARR)
  LAN = SIZE(STR_ARR)

  ALLOCATE( L_TOP(LAN), REL_HUM(LAN), NAC(LAN), ACI(LAN,M_CH+M_CI), &
       BACI(LAN,M_H*M_CH+M_CI), RD(LAN,M_CH+M_CI), AER_TYPE(LAN), NH(LAN) ) ! A

  READ(READ_ARR(1)%S,*,ERR = 101) ( L_TOP(I), I= 1,LAN )
  READ(READ_ARR(2)%S,*,ERR = 101) ( REL_HUM(LAN-I+1), I=1,LAN )

  IF( AER_SHIFT ) L_TOP(:) = L_TOP(:) + GROUND_HEIGHT
        
  STR_INP  = 'Aerosol type definition (SCIA_AER)'
  CALL Read_Input (STR_INP, READ_ARR, *880)
  II = 1
  DO I = 1, LAN
     K1 = LAN - I + 1
     AER_TYPE(I) = CUT_STR(READ_ARR(II)%S)
     READ(READ_ARR(II + 1)%S,*,ERR = 101) NAC(K1)
     S_UM = 0.D0
     II = II + 2
     DO J = 1,NAC(K1)
        READ(READ_ARR(II)%S,*,ERR = 101) ACI(K1,J), RD(K1,J)
        S_UM = S_UM + RD(K1,J)
        II = II + 1
     END DO
     IF( S_UM < 0.99D0 .OR. S_UM > 1.01D0) RETURN 2
  END DO
        
  STR_INP  = 'Profile type (SCIA_AER)'
  CALL Read_Input (STR_INP, READ_ARR, *880)
  PRF_TYPE = CUT_STR(READ_ARR(1)%S)

  SELECT CASE( PRF_TYPE )
  CASE('exti')
     STR_INP  = 'Reference wavelength (SCIA_AER)'
     CALL Read_Input (STR_INP, READ_ARR, *880)
     READ(READ_ARR(1)%S,*,ERR = 101) PRF_LAMBDA
  CASE('part')
  CASE DEFAULT
     CALL WRITE_MESSAGE(R_NAME, 'ERROR', &
          'Input value of "Profile type" is not recognized')
     WRITE(EU,*) '   Check file "scia_aer.inp".'
     RETURN 1
  END SELECT

  STR_INP  = 'Extiction coefficient profile (SCIA_AER)'
  CALL Read_Input (STR_INP, READ_ARR, *880)
  N_AER_LEV = SIZE(READ_ARR)
  IF(ALLOCATED(AER_LEV)) DEALLOCATE(AER_LEV, AER_PRF)
  ALLOCATE( AER_LEV(N_AER_LEV), AER_PRF(N_AER_LEV) ) ! A
  DO I = 1,N_AER_LEV
     READ(READ_ARR(I)%S,*,ERR = 101) AER_LEV(I), AER_PRF(I)
  END DO
  IF( AER_SHIFT ) AER_LEV(:) = AER_LEV(:) + GROUND_HEIGHT
  AER_PRF(:) = AER_SCALING*AER_PRF(:) 
  RETURN

101 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Reading error')
  WRITE(EU,*) '    Parameter to read: ', TRIM(STR_INP) 
  RETURN 1
  
880 WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)
  RETURN 1
END SUBROUTINE READ_SCIA_AER_INP

!***************************************************************************
SUBROUTINE READ_DB_SC_PAR
  CHARACTER(LEN=:), ALLOCATABLE ::  FN_EXT, FN_MOM
  INTEGER    IUB1, IUB2
  PARAMETER  (IUB1=60, IUB2=61)

  IF( PRF_TYPE == 'exti' ) THEN             
     ! From the top of atmosphere to the bottom
     DO I = 1,LAN
        DO J = 1,NAC(I)  ;  Ko = ACI(I,J)
           FN_EXT = TRIM(ADB)//'DB_SC_EXT'//SNA(Ko)//'.dat'
           OPEN(IUB1, FILE=FN_EXT, STATUS='OLD')
           DO K = 1,L_REF-1
              READ(IUB1,*)
           ENDDO
           ! Reading of aerosol components depending on humidity
           ! ====================================================
           IF( Ko <= 4 ) THEN
              READ(IUB1,*) ( TMP1(Ig), Ig=1,2*M_H )
              DB_EXT_REF(I,J) = TMP1(NH(I))
              ! Reading For aerosol components independent of humidity
              ! ======================================================
           ELSE
              READ(IUB1,*) DB_EXT_REF(I,J)
           ENDIF
           CLOSE(IUB1)  
        ENDDO
     ENDDO
  ENDIF

  N_SKIP = SUM( N_MAX(1:NWLS-1) )

  ! From the top of atmosphere to the bottom
  DO I = 1,LAN
     DO J = 1,NAC(I)  
        Ko = ACI(I,J)
        FN_EXT = TRIM(ADB)//'DB_SC_EXT'//SNA(Ko)//'.dat'
        FN_MOM = TRIM(ADB)//'DB_SC_MOM'//SNA(Ko)//'.dat'

        OPEN(IUB1, FILE=FN_EXT, STATUS='OLD')
        DO K = 1,NWLS-1
           READ(IUB1,*)
        ENDDO
        OPEN(IUB2, FILE=FN_MOM, STATUS='OLD')
        DO K = 1,N_SKIP
           READ(IUB2,*)
        ENDDO
        ! Reading For aerosol components depending on humidity
        ! ====================================================
        IF( Ko <= 4 ) THEN
           ! Read extinction and scattering coefficients:
           ! --------------------------------------------
           DO K = 1,NAW
              READ(IUB1,*) (TMP1(Ig), Ig=1,2*M_H)
              DB_EXT(I,J,K) = TMP1(NH(I))
              DB_SCA(I,J,K) = TMP1(M_H+NH(I))
           ENDDO
           ! Read expansion coefficients:
           ! ----------------------------
           K1 = NWLS
           DO K = 1,NAW
              DO L = 1,N_MAX(K1)
                 READ(IUB2,*) (TMP1(Ig), Ig=1,M_H)
                 DB_MOMS(I,J,K,L) = TMP1(NH(I))
              ENDDO
              K1 = K1 + 1
           ENDDO
           ! Reading For aerosol components independent of humidity
           ! ======================================================
        ELSE
           ! Read extinction and scattering coefficients:
           ! --------------------------------------------
           DO K = 1,NAW
              READ(IUB1,*) DB_EXT(I,J,K), DB_SCA(I,J,K)
           ENDDO
           ! Read expansion coefficients:
           ! ----------------------------
           K1 = NWLS
           DO K = 1,NAW
              DO L = 1, N_MAX(K1)
                 READ(IUB2,*) DB_MOMS(I,J,K,L) 
              ENDDO
              K1 = K1 + 1
           ENDDO
        ENDIF
        CLOSE(IUB1)  
        CLOSE(IUB2)  
            
        ! WRITE(*,'(1x,2(1x,I3),5(1x,E12.4))') I, J, (DB_EXT(I,J,K), K=1,NAW)
        
     ENDDO
  ENDDO
END SUBROUTINE READ_DB_SC_PAR

!***************************************************************************

SUBROUTINE AER_DM
  IF (LOG_LEVEL(1) >= N_ESO) THEN
     WRITE(LUNIT,*) '   ***********************  '
     WRITE(LUNIT,*) '   Aerosol D-M scalar case  '
     WRITE(LUNIT,*) '   ***********************  '
  ENDIF
  DO I1 = 1,LAN
     DO K = 1,NAW
        F_DM = USER_MOM(Itr+1,I1,1,K)/DBLE(2*Itr + 1)
        
        ! IF( K==1 ) THEN
        !    WRITE(*,*) I1, Itr, USER_MOM(Itr+1,I1,1,K)
        ! ENDIF

        FDM_AER(LAS(I1):LAE(I1),K) = F_DM
        DO L = 1,Itr
           AER_PHASMOMS(L,I1,K) = (USER_MOM(L,I1,1,K) - F_DM*DBLE(2*L-1))/(1.D0 - F_DM)
        END DO
     ENDDO
  ENDDO
  AER_PHASMOMS(Itr+1:NLEGEN,:,:) = 0.0D0

  ! "Renormalize" scattering and extinction coefficients
  ! ====================================================
  DO I = 1, NLAYERS
     DO K = 1,NAW
        F_DM = FDM_AER(I,K)
        DUM = AER_COEFF(EXI,I,K) - AER_COEFF(SCI,I,K)
        AER_COEFF(SCI,I,K) = AER_COEFF(SCI,I,K)*(1.0D0 - F_DM)
        AER_COEFF(EXI,I,K) = AER_COEFF(SCI,I,K) + DUM
     ENDDO
  ENDDO

  ! WRITE(*,*) '  After delta-M  '
  ! WRITE(*,*) '  Scat ',  AER_COEFF(SCI,1:2,1)
  ! WRITE(*,*) '   Ext ',  AER_COEFF(EXI,1:2,1)

END SUBROUTINE AER_DM
!***************************************************************************

SUBROUTINE  WRITE_SCEN_SA(*)
  USE  AEROSOLS, ONLY: OWL_AER, NOWL_AER
  USE TOOLS,     ONLY: OpenOut, WriteOut, CloseOut 

  Integer Unt
  DOUBLE PRECISION    SS_A, Ext
  CHARACTER(LEN=3)    PHT
  CHARACTER(LEN=17)   DB_COMP_TXT(M_H*M_CH+M_CI)
  Character (Len=500)  OutStr
  DATA DB_COMP_TXT / 'water sol.  0%   ', 'water sol. 50%   ', 'water sol. 70%   ',&
       'water sol. 80%   ', 'water sol. 90%   ', 'water sol. 95%   ', &
       'water sol. 98%   ', 'water sol. 99%   ', 'sea salt acc.  0%', &
       'sea salt acc. 50%', 'sea salt acc. 70%', 'sea salt acc. 80%', &
       'sea salt acc. 90%', 'sea salt acc. 95%', 'sea salt acc. 98%', &
       'sea salt acc. 99%', 'sea salt coa.  0%', 'sea salt coa. 50%', &
       'sea salt coa. 70%', 'sea salt coa. 80%', 'sea salt coa. 90%', &
       'sea salt coa. 95%', 'sea salt coa. 98%', 'sea salt coa. 99%', &
       'sulfate  0%      ', 'sulfate 50%      ', 'sulfate 70%      ', &
       'sulfate 80%      ', 'sulfate 90%      ', 'sulfate 95%      ', &
       'sulfate 98%      ', 'sulfate 99%      ', 'insoluble        ', &
       'soot             ', 'mineral nuc.     ', 'mineral acc.     ', &
       'mineral coarse   ', 'mineral transp.  ', 'fresh volcanic   ', &
       'meteoric         ' /
  REAL (KND), ALLOCATABLE :: T_OUT(:)
      
  IF (LOG_LEVEL(1) >= N_SSO) WRITE (lunit,*) ' *** writing: SCE_AEROSOL.OUT'

  ! Optical thickness at user defined wavelength grid
  IF( NOWL_AER > 0 ) ALLOCATE( T_OUT(NOWL_AER) )
  DO I = 1, NOWL_AER
     CALL LIN_SEARCH(OWL_AER(I), DUM, K, NAW, AER_WAVS, ERROR)
     IF (ERROR) GOTO 130
     T_OUT(I) =  (1.d0 - DUM)*TAU(K) + DUM*TAU(K+1)
  ENDDO

  Call OpenOut ('SCE_AEROSOL.OUT', Unt)
  WRITE(OutStr,*) '     ==================================='
  Call WriteOut (Unt, OutStr)
  WRITE(OutStr,*) '     SETTINGS FOR SCIATRAN AEROSOL MODEL'
  Call WriteOut (Unt, OutStr)
  WRITE(OutStr,*) '     ==================================='
  Call WriteOut (Unt, OutStr)
  WRITE(OutStr,*) ' '
  Call WriteOut (Unt, OutStr)

  IF( Scal_aer ) THEN
     WRITE(OutStr,'(A,F7.3,A,F7.2,A)') '   !!! ATTENTION !!! AOT is set to ',&
          Rtau_aer,' at', Rwl_aer,' nm wavelength.'
     Call WriteOut (Unt, OutStr)
     WRITE(OutStr,'(19x,A)') '  See setting in "control_aer.inp" '
     Call WriteOut (Unt, OutStr)
     WRITE(OutStr,'(6x,A)')  '----------------------------------------------------'
     Call WriteOut (Unt, OutStr)
     WRITE(OutStr,*) 
     Call WriteOut (Unt, OutStr)
  ENDIF

  WRITE(OutStr,'(A27,2X,I2)') '  Number of aerosol layers:', LAN
  Call WriteOut (Unt, OutStr)
  WRITE ( OutStr, * )
  Call WriteOut (Unt, OutStr)
  DO I = 1, LAN
     K1 = LAN-I+1
     WRITE(OutStr,'(A27,2X,I2)') '  Layer :                 :', I
     Call WriteOut (Unt, OutStr)
     WRITE(OutStr,'(A27,2X,F6.2)') '  Upper boundary [km]     :', L_TOP(I)
     Call WriteOut (Unt, OutStr)
     WRITE(OutStr,'(A27,2X,F3.0)') '  Rel. humidity [%]       :', REL_HUM(K1)
     Call WriteOut (Unt, OutStr)
     WRITE(OutStr,'(A27,2X,A30)')  '  Aerosol type            :', AER_TYPE (I)
     Call WriteOut (Unt, OutStr)
     WRITE(OutStr,'(29X,A20,A15)') '  Component           ', 'Mix.ratio'
     Call WriteOut (Unt, OutStr)
     DO J = 1,NAC(K1) 
        WRITE(OutStr,'(29X,A20,E15.5)') DB_COMP_TXT( BACI(K1,J) ), RD(K1,J)
        Call WriteOut (Unt, OutStr)
    ENDDO
     WRITE(OutStr,*)
     Call WriteOut (Unt, OutStr)
  ENDDO

  IF( NOWL_AER > 0 ) THEN
     WRITE ( OutStr, * ) ' AOT at user-defined wavelengths [nm]:'
     Call WriteOut (Unt, OutStr)
     DO I = 1, NOWL_AER
        WRITE(OutStr,'(12x,F7.2,4x,F8.4)') OWL_AER(I),T_OUT(I)
        Call WriteOut (Unt, OutStr)
     ENDDO
  ENDIF
  WRITE ( OutStr, * ) ' '
  Call WriteOut (Unt, OutStr)
          
  SELECT CASE( PRF_TYPE )
  CASE ('exti')
     WRITE(OutStr,'(A43,F7.2,A4)') 'Profile type: extinction profile [1/km] at ', &
          PRF_LAMBDA, ' nm.'
     Call WriteOut (Unt, OutStr)
  CASE ('part')
     WRITE(OutStr,'(A47)') 'Profile type: particle number density [1/cm^3].'
     Call WriteOut (Unt, OutStr)
  END SELECT

  WRITE(OutStr,*)
  Call WriteOut (Unt, OutStr)
  WRITE(OutStr,*) ' Vertical profiles of aerosol'
  Call WriteOut (Unt, OutStr)
  WRITE(OutStr,*) '         extinction coefficient (Ext coeff) and'
  Call WriteOut (Unt, OutStr)
  WRITE(OutStr,*) '         single scattering albedo (Ss albedo):'
  Call WriteOut (Unt, OutStr)
  IF(APH_TYPE == 'mie') THEN
     PHT = 'Mie'
  ELSE
     PHT = 'H-G'
  ENDIF
  DO K = 1,NAW
     WRITE(OutStr,*) '   '
     Call WriteOut (Unt, OutStr)
     WRITE(OutStr,'(1x,a11,F7.2,a6,a6,F8.4)') &
          'Wavelength ',AER_WAVS(K),' [nm];',' AOT =',TAU(K)
     Call WriteOut (Unt, OutStr)
     Call WriteOut (Unt, ' ----------------------------------------------')
     Call WriteOut (Unt, '     Number, Alt [km], Ext coeff [km-1], Ss albedo')
     DO I = 1, NLAYERS
        Ext = AER_COEFF(EXI,I,K)
        IF( Ext == 0.0d0 ) CYCLE
        SS_A = AER_COEFF(SCI,I,K)/Ext
        WRITE(OutStr,'(6x,I3,4x,F6.2,6x,ES9.2,6x,E12.4)') I, ZHOLD(I), Ext, SS_A
        Call WriteOut (Unt, OutStr)
     END DO
     WRITE(OutStr,*) ' '
     Call WriteOut (Unt, OutStr)
     WRITE(OutStr,*) '    Asymmetry factor used in the aerosol layers :'
     Call WriteOut (Unt, OutStr)
     WRITE(OutStr,*) '    ------------------------------------------- :'
     Call WriteOut (Unt, OutStr)
     WRITE(OutStr,*) '    Layer  Top [km]  Bottom [km]   Asymmetry    Phase'
     Call WriteOut (Unt, OutStr)
     WRITE(OutStr,*) '                                    factor     function'
     Call WriteOut (Unt, OutStr)
     DO I = 1,LAN
        WRITE(OutStr,'(6x,I3,3x,F6.2,4x,F6.2,9x,F6.3,7x,a3)') &
             I, ZHOLD(LAS(I)) ,ZHOLD(LAE(I)), Asym_Par(I,K), PHT
        Call WriteOut (Unt, OutStr)
     ENDDO
  ENDDO                   ! Over wavelengths
          
  Call CloseOut (Unt)
  RETURN
  
130 CALL WRITE_MESSAGE (R_NAME,'ERROR', 'Output wavelength for AOT is out of range')
  WRITE(EU,'(A,F7.2)') '    Output wavelength: ', OWL_AER(I)
  WRITE(EU,'(A,2(2x,F7.2))') '    Database wavelengths: ', AER_WAVS(1), AER_WAVS(NAW)
  WRITE(EU,'(A)') '    Set output wavelength between database wavelengths'
  WRITE(EU,'(A)') '    in the "control_aer.inp" file.'
  RETURN 1

END SUBROUTINE  WRITE_SCEN_SA

!***************************************************************************
SUBROUTINE DEALLOCATE_ARRAYS
  
  IF(ALLOCATED(DENS_PRF)) DEALLOCATE(DENS_PRF)
  IF(ALLOCATED(APH_VS)) DEALLOCATE(APH_VS, AER_WAVS, AER_COEFF)
  IF(ALLOCATED(Greek_aer)) DEALLOCATE(Greek_aer)
  IF(ALLOCATED(FDM_AER)) DEALLOCATE(FDM_AER)
  IF(ALLOCATED(LA_S)) DEALLOCATE(LA_S, LA_E, IDX_AER)
  IF(ALLOCATED(IDX_AER_db)) DEALLOCATE(IDX_AER_db)
  IF(ALLOCATED(L_TOP)) DEALLOCATE(L_TOP)
  
  RETURN
END SUBROUTINE DEALLOCATE_ARRAYS
!***************************************************************************
END SUBROUTINE SC_AER_SETUP
!***************************************************************************
