C********************************************************************C
C
C	SCIATRAN - Version 3.8 - Last modification: 01 Jun 2017
C
C	Copyright (C) 1995-2013, Institute of Remote Sensing,
C			         University of Bremen, Germany
C
C********************************************************************C

c **********************************************************************
c  THE PROGRAM COMPUTES LIGHT SCATTERING BY POLYDISPERSE SPHERICAL    
c              PARTICLES USING THE LORENZ-MIE THEORY     
c **********************************************************************
c  THE PROGRAM IS BASED ON THE PAPERS                                  
c                                                                      
c    1. W. A. De Rooij and C. C. A. H. van der Stap, expansion of Mie  
c       scattering matrices in generalized spherical functions,        
c       Astronomy and Astrophysics, vol. 131, PP. 237-248, 1984        
c                                                                      
c    2. M. I. Mishchenko, J. M. Dlugach, E. G. Yanovitskij,            
c       and N. T. Zakharova, Bidirectional reflectance of flat,        
c       optically thick particulate laters:  an efficient radiative    
c       transfer solution and applications to snow and soil surfaces,  
c       J. Quant. Spectrosc. Radiat. Transfer, VOL. 63, 409-432 (1999).
c
c  The second paper is available upon reuest from Michael Mishchenko
c  (send a mesage to crmim@giss.nasa.gov) and is also available
c  in the pdf and ps formats at 
c  http://www.giss.nasa.gov/~crmim/publications/index.html 
                                                                       
c  Note that the F11 element of the scattering matrix (i.e. the phase  
c  function) is normalized to unity rather than to the single          
c  scattering albedo [see remark after equation (32) in ref. 1         
c  and equation (7) in ref. 2].                                        
c  the paper by de ROOIJ and van der STAP describes numerical          
c  aspects of mie computations.                                        
                                                                       
c  The code has been developed by Michael Mishchenko at the NASA       
c  GODDARD INSTITUTE FOR SPACE STUDIES, NEW YORK.  the development     
c  of the code was supported by the nasa radiation science program.    
                                                                       
c  The code can be used without limitations in any not-for-            
c  profit scientific research.  we only request that in any            
c  publication using the code the source of the code be acknowledged   
c  and proper references be made.                                      
                                                                       
                                                                       
c  INPUT PARAMETERS:                                                   
c     NDISTR specifies the type of particle size distribution         
c     NDISTR = 1 - Modified gamma distribution                         
c          [Eq. (20) of Ref. 2]                                        
c              AA=alpha                                                
c              BB=r_c                                                  
c              GAM=gamma                                               
c
c     NDISTR = 2 - Log normal distribution [Eq. (21) of Ref. 2]
c              AA = r_g  ;  BB = ln^2(sigma_g)
c
c     NDISTR = 3 - POWER LAW DISTRIBUTION                              
c          [Eq. (22) of Ref. 2]                                        
c               AA=r_eff (EFFECTIVE RADIUS)                            
c               BB=v_eff (EFFECTIVE VARIANCE)                          
c               PARAMETERS R1 AND R2 (SEE BELOW) ARE CALCULATED        
c               AUTOMATICALLY FOR GIVEN AA AND BB                      
c     NDISTR = 4 - GAMMA DISTRIBUTION                                  
c          [Eq. (23) of Ref. 2]                                        
c               AA=a                                                   
c               BB=b                                                   
c     NDISTR = 5 - MODIFIED POWER LAW DISTRIBUTION                     
c          [Eq. (24) of Ref. 2]                                        
c               BB=alpha                                               
c     NDISTR = 6 - BIMODAL VOLUME LOG NORMAL DISTRIBUTION              
c              [Eq. (11) of Mishchenko et al, Appl. Opt. 38,             
c              7325-7341 (1999); available at                                
c              http://www.giss.nasa.gov/~crmim/publications/index.html] 
c              AA1=r_g1                                                
c              BB1=[ln(sigma_g1)]**2                                   
c              AA2=r_g2                                                
c              BB2=[ln(sigma_g2)]**2                                   
c              GAM=GAMMA                                               
c   - R1 AND R2 - Minimum and maximum                                  
c         radii in the size distribution for ndistr=1-4 AND 6.         
c         R1 and R2 are calculated automatically                       
c         for the power law distribution with given r_eff and v_eff    
c         but must be specified for other distributions.               
c             -----------------
c         FOR THE MODIFIED POWER LAW DISTRIBUTION (NDISTR=5), THE      
c         MINIMUM RADIUS IS 0, R2 IS THE MAXIMUM RADIUS,               
c         AND R1 IS THE INTERMEDIATE RADIUS AT WHICH THE               
c         n(r)=const DEPENDENCE IS REPLACED BY THE POWER LAW           
c         DEPENDENCE.                                                  
c   - LAM - wavelength of the incident light in the surrounding medium
c   - MRR AND MRI - real and imaginary parts of the relative refractive
c                   index (mri must be non-negative)                        
c   - IMPORTANT:  r_c, r_g, r_eff, a, LAM, R1, AND R2                  
c                 must be in the same units of length (e.g., microns)  
c   - N - number of integration subintervals on the interval (r1,r2)   
c         of particle radii                                            
c   - NP - number of integration subintervals on the interval (0,R1)   
c         for the modified power law distribution                      
c   - NK - Number of gaussian division points on each of the           
c          integration subintervals                                    
c   - NPNA - number of scattering angles at which the scattering       
c        matrix is computed                                            
c        (SEE THE PARAMETER STATEMENT IN SUBROUTINE MATR).             
c        THE CORRESPONDING SCATTERING ANGLES ARE GIVEN BY              
c        180*(I-1)/(NPNA-1) (DEGREES), WHERE I NUMBERS THE ANGLES.     
c        THIS WAY OF SELECTING SCATTERING ANGLES CAN BE EASILY CHANGED 
c        AT THE BEGINNING OF SUBROUTINE MATR BY PROPERLY MODIFYING     
c        THE FOLLOWING LINES:                                          
c                                                                      
c          N=NPNA                                                       
c          DN=1D0/DFLOAT(N-1)                                          
c          DA=DACOS(-1D0)*DN                                           
c          DB=180D0*DN                                                 
c          TB=-DB                                                      
c          TAA=-DA                                                     
c          DO 500 I1=1,N                                               
c             TAA=TAA+DA                                               
c             TB=TB+DB                                                 
c                                                                      
c        and leaving the rest of the subroutine intact.                
c        This flexibility is provided by the fact that                 
c        after the expansion coefficients AL1,...,BET2 (see below)     
c        are computed by subroutine SPHER, the scattering matrix       
c        can be computed for any set of scattering angles without      
c        repeating Lorenz-Mie calculations.                                   
c        The expansion coefficients:
c        AL1, AL2, AL3, AL4, BET1, BET2 are put into array
c        Grs(1:6,:).
c   - Grs_Dr(1:6,:) - derivatives of the expansion coefficients with
c        respect to the model radius of the log normal distribution.
c   - Grs_Ds(1:6,:) - derivatives of the expansion coefficients with
c        respect to the RMS of the log normal distribution.
C   - DDELT - Desired numerical accuracy of computing the scattering   
c        matrix elements	                                       
                                                                       
                                                                       
c  OUTPUT INFORMATION:                                                 
c                                                                      
c   - REFF AND VEFF - effective radius and effective variance          
c                     of the size distribution                         
c          [Hansen and Travis, Space Sci. Rev., vol. 16, 527-610,      
c          1974, Eqs. (2.53) and (2.54)]                               
c   - CEXT AND CSCA - average extinction and scattering cross sections per
c                     particle                                         
c   - <COS> - ASYMMETRY PARAMETER OF THE PHASE FUNCTION                
c   - ALBEDO - SINGLE-SCATTERING ALBEDO                                
c   - <G> - Average projected area per particle                        
c   - <V> - AVERAGE VOLUME PER PARTICLE                                
c   - Rvw - VOLUME WEIGHTED AVERAGE RADIUS                             
c   - <R> - MEAN RADIUS                                                
c   - F11, F33, F12, AND F34 - elements of the scattering matrix       
c          [Eq. (39) of Ref. 1].                                       
c          Note that for spheres, F22=F11, F44=F33, F21=F12,           
c          and F43=-F34.  All other elements are equal to zero.        
c          Nonzero elements of the scattering matrix are put into array
c          Phm(1:4,:) 
c   - Phm_Dr(1:4,:)- derivatives of nonzero elements of the scattering matrix
c          with respect to modal radius of the log-normal distribution
c   - Phm_Ds(1:4,:)- derivatives of nonzero elements of the scattering matrix
c          with respect to RMS of the log-normal distribution
c   - ALPHA1,...,BETA2 - Coefficients appearing in the expansions
c          of the elements of the scattering matrix in
c          generalized spherical functions                   
c          [EQS. (41)-(46) OF REF. 1;  EQ. (6) OF REF. 2].             
c
c   The first line of file 10 provides the single-scattering
c   albedo and the number of numerically significant 
c   expansion coeffieients ALPHA1,...,BETA2, followed
c   by the values of the expansion coefficients.
c                                                                      
c   Optical efficiency factors QEXT and QSCA are computed as           
c   QEXT=CEXT/<G> AND QSCA=CSCA/<G>.  the absorption cross section     
c   is given by CABS=CEXT-CSCA.  the absorption efficiency factor      
c   is equal to QABS=QEXT-QSCA.                                        
                                                                       
c   To calculate scattering and absorption by a monodisperse 
c   particle with radius R = AA, use the following options:          
c                                                                      
c        BB=1D-1                                                       
c        NDISTR=4                                                      
c        NK=1                                                          
c        N=1                                                           
c        R1=AA*0.9999999 D0                                            
c        R2=AA*1.0000001 D0                                            
                                                                       
c    Although the mathematical value of the parameter r2 for            
c   modified gamma, log normal, and gamma distributions is infinity,   
c   in practice a finite value must be chosen.  
c    Two interpretations of a truncated size distributions can be given.  
c     First, R2 can     
c   be increased until the scattering cross sections and the           
c   scattering matrix elements converge within some numerical          
c   accuracy.  In this case the converged truncated                    
c   size distribution is numerically equivalent to the distribution    
c   with R2=infinity. 
c     Second, a truncated distribution can be         
c   considered a specific distribution with optical cross sections     
c   and scattering matrix elements different from those for the        
c   distribution with R2=infinity. In this latter case the effective  
c   radius and effective variance of the truncated distribution        
c   will also be different from those computed analytically for the    
c   respective distribution with R2=infinity.              
c   similar considerations apply to parameter R1 whose theoretical     
c   value for modified gamma, log normal, and gamma distributions      
c   is zero, but in practice can be any number smaller than R2.        
                                                                       
c   IMPORTANT:  For given size distribution parameters NDISTR, AA,     
c   BB, GAM, R1, and R2, the parameters N, NP, and/or NK 
c   should be increased until convergent results are obtained  
c   for the extinction and scattering cross sections and, especially,
c   the elements of the scattering matrix.                 
 
c   I would highly appreciate information on any problems encountered
c   with this code.  Please send your message to Michael Mishchenko
c   at crmim@giss.nasa.gov.

c   While this computer program has been tested for a variety of cases,
c   it is not inconceivable that it contains undetected errors. also,
c   input parameters can be used which are outside the envelope of
c   values for which results are computed accurately. for this reason,
c   the author and his organization disclaim all liability for
c   any damages that may result from the use of the program.  
c
c************************************************************************
      SUBROUTINE SC_MIE(AA, BB, GAM, Nps, LAM, MRR, MRII, R1, R2, N, NP, 
     $     NDISTR, NK, Lout, Grs, CEXT, CSCA, Grs_Dr, Grs_Ds, 
     $     CEXT_Dr, CSCA_Dr, CEXT_Ds, CSCA_Ds, AREA, VOLUME, RVW, 
     $     RMEAN, AA1, BB1, AA2, BB2, DDELT, NOWR, L1_max, PM_2i5,
     $     PM_2i5_MOD) 
c     ------------------------------------------------------------------
      USE AER_MIE,  ONLY: COEF1, COEF2, COEF3, COEF4, COEF5, COEF6, 
     $                    COEF7, COEF8
      USE LOG_PARS, ONLY: DIR_OUT

      INTEGER, INTENT(IN) :: NDISTR, L1_max, Nps
      LOGICAL, INTENT(IN) :: NOWR, PM_2i5, PM_2i5_MOD
      INTEGER  N, NP, NK
      INTEGER, PARAMETER :: KND = kind(0.0d0)
      INTEGER  Lout(Nps)
      INTEGER  I, I1, J, K, L, J1, IJ, NNK, NNPK, M, NG, L1, M1, 
     $     M2, M3, M4, J2, L1MAX
      REAL (KND) AA(Nps), BB(Nps), GAM, LAM, R1, R2
      REAL (KND) AREA(Nps), VOLUME(Nps), RVW, RMEAN, AA1, BB1, AA2, BB2,
     $     DDELT, RX, Z1, Z2, DD, DD1, DD2, CM, RMR, RMI, ZJ, REFF, 
     $     VEFF, CEXT(Nps), CSCA(Nps), CEXT_Dr(Nps), CSCA_Dr(Nps), 
     $     CEXT_Ds(Nps), CSCA_Ds(Nps), Y, Dp, Dm,
     $     ZW(Nps), ZW_Dr(Nps), ZW_Ds(Nps), 
     $     RXR, RXI, CDD, CD, DC, DS, CX, CXR, CXI,
     $     QMAX, D4, DJ, FR, FI, OR, OI, ORI, PSI1, DR1, DI1, HI1,
     $     OR1, OI1, OR2, OI2, OAB, PSI2, HI2, YAR, YAI, YBI, CE,
     $     CS, CJ, ARJ, AIJ, BRJ, BIJ, CDA, CDB, SPR, SPI, SMR,
     $     SMI, PJ, TJ, PP, PM, D1, D2, VOL, ALB, ALB_DR, ALB_DS,
     $     D6, WI, FF11, FF33, FF12, FF34, FP, FM, CL,
     $     A2, A3, YBR
      REAL (KND) MRR, MRI, MRII, PI, WN, X(L1_max), W(L1_max), 
     $     Grs(Nps,6,L1_max), Grs_Dr(Nps,6,L1_max), 
     $     Grs_Ds(Nps,6,L1_max) 

      REAL*8, ALLOCATABLE :: FD(:,:), FD_Dr(:,:), FD_Ds(:,:), PINa(:,:),
     $     TAUNa(:,:), Phm(:,:,:), Phm_Dr(:,:,:), Phm_Ds(:,:,:),
     $     COEFF1(:), COEFF2(:), COEFF3(:), 
     $     P1(:), P2(:), P3(:), P4(:), PSI(:), RPSI(:), DR(:), DI(:),
     $     HI(:), AR(:), AI(:), BR(:), BI(:), XX(:), YY(:), WY(:)
c     ==================================================================

      MRI = -MRII
c     ... Power law distribution
      IF( NDISTR == 3 ) CALL POWER(AA(1), BB(1), R1, R2)

c     ...  NK is the number of gaussian division points on each 
c          of the integration subintervals
      IF( NK > L1_max ) THEN 
         WRITE(*,*) 'NK is greater than L1_max. execution terminated'
         WRITE(*,*) '      NK = ', NK
         WRITE(*,*) '  L1_max = ', L1_max
         STOP                                                      
      ENDIF

c     ... Calculation of gaussian division points and weights
      CALL GAUSS(NK, 0, 0, X, W) 

c     N  - number of integration subintervals on the interval (r1,r2)   
c          of particle radii;
      NNK  = N*NK 
      NNPK = 0
c     ... Modified power law distribution:
C         NP is the number of integration subintervals on the
c         interval (0,r1) for the modified power law distribution 
      IF( NDISTR == 5 ) NNPK = NP*NK
      NNK = NNK + NNPK

c     ... Allocate some arrays at the angle grid
      ALLOCATE( FD(Nps,NNK), FD_Dr(Nps,NNK), FD_Ds(Nps,NNK), YY(NNK), 
     $     WY(NNK) )

      PI = 3.14159 26535 89793 2 D0
      WN = 2d0*PI/LAM 

      RX = R2*WN
      M = IDINT(RX + 4.05d0*RX**0.33333d0 + 8d0)

      ALLOCATE( COEFF1(M), COEFF2(M), COEFF3(M) )

      DO I = 1,M
         DD = DFLOAT(I)
         DD1 = DD + 1.d0 
         COEFF1(I) = DD1/DD 
         DD2 = (2.d0*DD + 1.d0)
         COEFF2(I) = DD2
         COEFF3(I) = 0.5D0*DD2/(DD*DD1)
      ENDDO                                                                
      
      NG    = 2*M-1 
      L1MAX = 2*M 
     
      ALLOCATE( P1(L1MAX), P2(L1MAX), P3(L1MAX), P4(L1MAX), XX(NK) )

      CM  = 1.d0/(MRR*MRR + MRI*MRI)
      RMR = MRR*CM 
      RMI =-MRI*CM 

      IF( NDISTR.NE.5 ) GO TO 30

      Z1 = R1/DFLOAT(NP) 
      Z2 = Z1*0.5D0 
      XX(1:NK) = Z2*X(1:NK) + Z2

      DO J = 1,NP 
         J1 = J-1
         ZJ = Z1*DFLOAT(J1)
         IJ = J1*NK
         DO I = 1,NK
            I1 = I + IJ 
            YY(I1) = XX(I) + ZJ
            WY(I1) = W(I)*Z2
         ENDDO 
      ENDDO

   30 Z1 = (R2 - R1)/DFLOAT(N)
      Z2 = Z1*0.5d0
      DO I = 1,NK 
         XX(I) = Z2*X(I) + Z2 
      ENDDO
      DO J = 1,N
         J1 = J-1
         ZJ = Z1*DFLOAT(J1) + R1 
         IJ = J1*NK + NNPK 
         DO I=1,NK 
            I1 = I+IJ 
            YY(I1) = XX(I) + ZJ 
            WY(I1) = W(I)*Z2
         ENDDO
      ENDDO

      CALL DISTRB( NNK, YY, WY, FD, FD_Dr, FD_Ds, NDISTR, AA, BB, GAM,
     &     R1, R2, REFF, VEFF, AREA, VOLUME, RVW, RMEAN, PI, AA1, BB1, 
     $     AA2, BB2, NOWR, Nps, PM_2i5, PM_2i5_MOD)

      CEXT    = 0.d0 ; CSCA    = 0.d0
      CEXT_Dr = 0.d0 ; CSCA_Dr = 0.d0
      CEXT_Ds = 0.d0 ; CSCA_Ds = 0.d0

      IF( PM_2i5 ) RETURN
      
      CALL GAUSS(NG, 0, 0, X, W)
      ALLOCATE( Phm(Nps,4,NG), Phm_Dr(Nps,4,NG), Phm_Ds(Nps,4,NG) )

      Phm = 0.d0 ; Phm_Dr = 0.d0 ; Phm_ds = 0.d0

cc      OPEN( 85,file=DIR_OUT//'/Pin-Tau.dat' )
        RX = YY(NNK)*WN 
        M1 = IDINT(RX + 4.05d0*RX**0.33333d0 + 8d0)
        M2 = M1 + 2 + IDINT(1.2d0*DSQRT(RX)) + 5 

        RXR = MRR*RX  ;  RXI = MRI*RX 
        CDD = RXR*RXR + RXI*RXI
        CD = DSQRT(CDD)
        QMAX = DMAX1(DFLOAT(M1),CD)
        M4 = IDINT(6.4d0*QMAX**0.33333d0 + QMAX) + 8 

        ALLOCATE( PINa(M1,NG), TAUNa(M1,NG), PSI(M1), AR(M1), AI(M1),
     $       BR(M1), BI(M1), RPSI(M2), DR(M4), DI(M4), HI(M4) )

c     c        WRITE(85,*) M1
        DO K = 1,NG
           CALL ANGL(M1, X(K), PINa(1:M1,K), TAUNa(1:M1,K))
cc           WRITE(85,'(200(1x,E15.7))') (PINa(K,J),  J=1,M1)
cc           WRITE(85,'(200(1x,E15.7))') (TAUNa(K,J), J=1,M1)
        ENDDO
cc      CLOSE(85)

c      OPEN( 85,file=DIR_OUT//'/Pin-Tau.dat' )
c      READ(85,*) M1
c      DO K = 1,NG
c         READ(85,*) (PINa(K,J),  J=1,M1)
c         READ(85,*) (TAUNa(K,J), J=1,M1)
c      ENDDO
c      CLOSE(85)

c     ... Averaging over sizes
c     ------------------------
      DO 100 I = 1,NNK          ! over monodisperse grid
         Y     = YY(I)
         RX = Y*WN  ;  RXR = MRR*RX  ;  RXI = MRI*RX 
         CDD = RXR*RXR + RXI*RXI
         CD = DSQRT(CDD) ; DC = DCOS(RX) ; DS = DSIN(RX)
         CX = 1d0/RX     ; CXR = RXR/CDD ; CXI = -RXI/CDD 
         
C         ... Calculation of the Mie coefficients  
         M1 = IDINT(RX + 4.05D0*RX**0.33333D0 + 8D0)
         M2 = M1 + 2 + IDINT(1.2D0*DSQRT(RX)) + 5 
         M3 = M2-1 

         QMAX = DMAX1(DFLOAT(M1),CD)
         M4 = IDINT(6.4D0*QMAX**0.33333D0 + QMAX) + 8 

         D4 = DFLOAT(M4+1)  ;  DR(M4) = D4*CXR ; DI(M4) = D4*CXI 
         HI(1) = DS + DC*CX 
         HI(2) = 3.0D0*HI(1)*CX - DC 
         PSI(1) = CX*DS - DC 

         RPSI(M2) = RX/DFLOAT(2*M2+1)
         DO J1 = M2-1, M2-M3+1, -1
            RPSI(J1) = 1.d0/(DFLOAT(2*J1+1)*CX - RPSI(J1+1)) 
         ENDDO

         DO J1 = M4, 2, -1
            DJ = DFLOAT(J1)
            FR = DJ*CXR      ; FI = DJ*CXI 
            OR = DR(J1) + FR ; OI = DI(J1) + FI 
            ORI = 1D0/(OR*OR + OI*OI)
            DR(J1-1) = FR - OR*ORI 
            DI(J1-1) = FI + OI*ORI
         ENDDO

         M2 = M1-1 
         DO J = 2,M2 
            HI(J+1) = DFLOAT(2*J+1)*HI(J)*CX - HI(J-1) 
         ENDDO
         DO J = 2,M1 
            PSI(J) = RPSI(J)*PSI(J-1)
         ENDDO

         PSI1 = PSI(1)
         DR1 = DR(1) ; DI1 = DI(1) ; HI1 = HI(1)
         OR = DR1*RMR - DI1*RMI + CX
         OR1 = OR*PSI1 - DS 
         OI = DR1*RMI + DI1*RMR 
         OI1 = OI*PSI1 
         OR2 = OR*PSI1 - OI*HI1-DS 
         OI2 = OR*HI1 + OI*PSI1-DC
         OAB = 1D0/(OR2*OR2 + OI2*OI2)
c        ...
         AR(1) = (OR1*OR2 + OI1*OI2)*OAB
         AI(1) = (OR2*OI1 - OR1*OI2)*OAB
         OR = MRR*DR1 - MRI*DI1 + CX 
         OI = MRR*DI1 + MRI*DR1 
         OR1 = OR*PSI1 - DS  
         OR2 = OR*PSI1 - OI*HI1 - DS 
         OI1 = OI*PSI1
         OI2 = OR*HI1 + OI*PSI1 - DC 
         OAB = 1D0/(OR2*OR2 + OI2*OI2)
c        ... 
         BR(1) = (OR1*OR2 + OI1*OI2)*OAB 
         BI(1) = (OR2*OI1 - OR1*OI2)*OAB 
         DO J = 2,M1
            J1 = J-1
            DJ = DFLOAT(J)*CX
            PSI1 = PSI(J) ; PSI2 = PSI(J1)
            HI1  = HI(J)  ;  HI2 = HI(J1)
            DR1  = DR(J)  ;  DI1 = DI(J)
c           -----------------------------
            OR = DR1*RMR - DI1*RMI + DJ ; OI = DR1*RMI + DI1*RMR
            OR1 = OR*PSI1 - PSI2  
            OI1 = OI*PSI1 
c            OR2 = OR*PSI1 - OI*HI1 - PSI2
            OR2 = OR1 - OI*HI1
            OI2 = OR*HI1 + OI*PSI1 - HI2
            OAB = (OR2*OR2 + OI2*OI2)
            YAR = (OR1*OR2 + OI1*OI2)/OAB
            YAI = (OR2*OI1 - OR1*OI2)/OAB
            AR(J) = YAR   ;   AI(J) = YAI
c           -----------------------------
            OR = MRR*DR1 - MRI*DI1 + DJ ; OI = MRR*DI1 + MRI*DR1
            OR1 = OR*PSI1 - PSI2
c            OR2 = OR*PSI1 - OI*HI1 - PSI2
            OR2 = OR1 - OI*HI1 
            OI1 = OI*PSI1
            OI2 = OR*HI1 + OI*PSI1 - HI2
            OAB = (OR2*OR2 + OI2*OI2)
            YBR = (OR1*OR2 + OI1*OI2)/OAB
            YBI = (OR2*OI1 - OR1*OI2)/OAB
            BR(J) = YBR    ;    BI(J) = YBI 
c           -------------------------------
c            YAR = YAR*YAR + YAI*YAI + YBR*YBR + YBI*YBI 
         ENDDO
c        *********** End of computing the Mie coefficients  **********

         CE = 0.d0 ; CS = 0.d0
         DO J = 1,M1
            CJ  = COEFF2(J)
            ARJ = AR(J)
            AIJ = AI(J)
            BRJ = BR(J)
            BIJ = BI(J)
            CDA = ARJ*ARJ + AIJ*AIJ
            CDB = BRJ*BRJ + BIJ*BIJ
            CE  = CE + CJ*(ARJ + BRJ) 
            CS  = CS + CJ*(CDA + CDB)
            CJ  = COEFF3(J)
            AR(J) = CJ*(ARJ + BRJ)
            AI(J) = CJ*(AIJ + BIJ)
            BR(J) = CJ*(ARJ - BRJ)
            BI(J) = CJ*(AIJ - BIJ)
         ENDDO

         ZW(:)    = WY(I)*FD(:,I)
         ZW_Dr(:) = WY(I)*FD_Dr(:,I)
         ZW_Ds(:) = WY(I)*FD_Ds(:,I)

         CEXT(:) = CEXT(:) + ZW(:)*CE
         CSCA(:) = CSCA(:) + ZW(:)*CS
c         ... wrt r_mode
         CEXT_Dr(:) = CEXT_Dr(:) + ZW_Dr(:)*CE
         CSCA_Dr(:) = CSCA_Dr(:) + ZW_Dr(:)*CS
c         ... wrt sigma
         CEXT_Ds(:) = CEXT_Ds(:) + ZW_Ds(:)*CE
         CSCA_Ds(:) = CSCA_Ds(:) + ZW_Ds(:)*CS

c         WRITE(*,*) '  Number of NG =  ', NG, M1

         DO K = 1,NG

c            CALL ANGL(M1, X(K), PIN, TAUN)
            SPR = 0.d0 ; SPI = 0.d0 ; SMR = 0.d0 ; SMI = 0.d0
            DO J = 1,M1
c               PJ = PIN(J)          ; TJ = TAUN(J) 
               PJ = PINa(J,K)       ; TJ = TAUNa(J,K) 
               PP = TJ + PJ         ; PM = TJ - PJ
               SPR = SPR + AR(J)*PP ; SPI = SPI + AI(J)*PP 
               SMR = SMR + BR(J)*PM ; SMI = SMI + BI(J)*PM 
            ENDDO  
c            SPR = SUM( AR(1:M1)*(TAUNa(K,1:M1) + PINa(K,1:M1)) ) 
c            SPI = SUM( AI(1:M1)*(TAUNa(K,1:M1) + PINa(K,1:M1)) ) 
c            SMR = SUM( BR(1:M1)*(TAUNa(K,1:M1) - PINa(K,1:M1)) ) 
c            SMI = SUM( BI(1:M1)*(TAUNa(K,1:M1) - PINa(K,1:M1)) ) 

            D1 = (SPR*SPR + SPI*SPI) ; D2 = (SMR*SMR + SMI*SMI)
            Dp = (SPR*SMR + SPI*SMI) ; Dm = (SPR*SMI - SPI*SMR)
            Phm(:,1,K) = Phm(:,1,K) + (D1 + D2)*ZW(:)
            Phm(:,2,K) = Phm(:,2,K) + (D1 - D2)*ZW(:) 
            Phm(:,3,K) = Phm(:,3,K) +        Dp*ZW(:)*2d0 
            Phm(:,4,K) = Phm(:,4,K) +        Dm*ZW(:)*2d0 

c            ... Derivative of the scattering matrix elements
c                with respect to the mode radius
            Phm_Dr(:,1,K) = Phm_Dr(:,1,K) + (D1 + D2)*ZW_Dr(:)
            Phm_Dr(:,2,K) = Phm_Dr(:,2,K) + (D1 - D2)*ZW_Dr(:)
            Phm_Dr(:,3,K) = Phm_Dr(:,3,K) +        Dp*ZW_Dr(:)*2d0 
            Phm_Dr(:,4,K) = Phm_Dr(:,4,K) +        Dm*ZW_Dr(:)*2d0 

c            ... Derivative of the scattering matrix elements
c                with respect to the RMS
            Phm_Ds(:,1,K) = Phm_Ds(:,1,K) + (D1 + D2)*ZW_Ds(:)
            Phm_Ds(:,2,K) = Phm_Ds(:,2,K) + (D1 - D2)*ZW_Ds(:)
            Phm_Ds(:,3,K) = Phm_Ds(:,3,K) +        Dp*ZW_Ds(:)*2d0 
            Phm_Ds(:,4,K) = Phm_Ds(:,4,K) +        Dm*ZW_Ds(:)*2d0 
         ENDDO                  ! over Gaussian angle grid

c         WRITE(*,*) '  End coeff and phase '
 100  CONTINUE                  ! end over monodisperse grid (I)
      
c      write(*,*) '  end mono  '

C  ******  End of averaging over sizes  ************************

C     ... Elements of the scattering matrix  *******************
      DO I = 1,Nps
         DD = 2.d0/CSCA(I)  ;  Phm(I,:,:) = Phm(I,:,:)*DD
         Phm_Dr(I,:,:) =Phm_Dr(I,:,:)*DD - Phm(I,:,:)/CSCA(I)*CSCA_Dr(I)
         Phm_Ds(I,:,:) =Phm_Ds(I,:,:)*DD - Phm(I,:,:)/CSCA(I)*CSCA_Ds(I)
      ENDDO

c      write(*,*) '   Cext before VOL ', NNK
c      write(*,'(2x,e15.7)') SUM( FD(1,1:NNK) )
c      write(*,'(2x,e15.7)')  CEXT(1)

C     ... Cross sections and single scattering albedo 
      VOL  = 2d0*PI/(WN*WN) 
      CEXT(:)    = CEXT(:)*VOL    ; CSCA(:)    = CSCA(:)*VOL 
      CEXT_Dr(:) = CEXT_Dr(:)*VOL ; CSCA_Dr(:) = CSCA_Dr(:)*VOL 
      CEXT_Ds(:) = CEXT_Ds(:)*VOL ; CSCA_Ds(:) = CSCA_Ds(:)*VOL 

      ALB     = CSCA(1)/CEXT(1)
      ALB_DR  = CSCA_Dr(1)/CEXT_Dr(1)
      ALB_DS  = CSCA_Ds(1)/CEXT_Ds(1)
     
      Grs = 0.d0 ;  Grs_Dr = 0.d0 ; Grs_Ds = 0.d0
      D6 = 0.25d0*DSQRT(6.d0)  
      DO I = 1,NG 
          CALL GENER(X(I))
          WI = W(I)
          DO K = 1,Nps
             FF11 = Phm(K,1,I)*WI ; FF33 = Phm(K,2,I)*WI 
             FF12 = Phm(K,3,I)*WI ; FF34 = Phm(K,4,I)*WI 
             FP   = FF11 + FF33 ; FM   = FF11 - FF33 
             Grs(K,1,1:L1MAX) = Grs(K,1,1:L1MAX) + FF11*P1(1:L1MAX) 
             Grs(K,2,1:L1MAX) = Grs(K,2,1:L1MAX) + FP  *P2(1:L1MAX)
             Grs(K,3,1:L1MAX) = Grs(K,3,1:L1MAX) + FM  *P3(1:L1MAX) 
             Grs(K,4,1:L1MAX) = Grs(K,4,1:L1MAX) + FF33*P1(1:L1MAX) 
             Grs(K,5,1:L1MAX) = Grs(K,5,1:L1MAX) + FF12*P4(1:L1MAX) 
             Grs(K,6,1:L1MAX) = Grs(K,6,1:L1MAX) + FF34*P4(1:L1MAX) 
          ENDDO
c         ... Derivatives wrt mod. radius
c         -------------------------------
          CALL GRS_A( Phm_Dr,Grs_Dr,Nps )
c         ... Derivatives wrt RMS
c         -------------------------------
          CALL GRS_A( Phm_Ds,Grs_Ds,Nps )

       ENDDO                                                              

       DO K = 1,Nps
          DO L1 = 1,L1MAX 
             CL = DFLOAT(L1-1) + 0.5D0
             L = L1
             Grs(K,1,L1) = Grs(K,1,L1)*CL

             A2 = Grs(K,2,L1)*CL*0.5d0
             A3 = Grs(K,3,L1)*CL*0.5d0
             Grs(K,2,L1) = A2 + A3
             Grs(K,3,L1) = A2 - A3
             Grs(K,4,L1) = Grs(K,4,L1)*CL
             Grs(K,5,L1) = Grs(K,5,L1)*CL
             Grs(K,6,L1) =-Grs(K,6,L1)*CL

c            ... Derivatives wrt mod. radius
c            -------------------------------
             CALL GRS_B( Grs_Dr )
c            ... Derivatives wrt RMS
c            -------------------------------
             CALL GRS_B( Grs_Ds )
             IF( DABS(Grs(K,1,L1)).LE.DDELT ) EXIT 
          ENDDO
          Lout(K) = L
      ENDDO  

      L1MAX = L 

c      WRITE(*,*) '   end "Gener" '
c      WRITE(*,*) '  2 - L1MAX =   ', L1MAX
c      WRITE(*,*) '        2*M =   ', 2*M

C     ****************  OUTPUT   **************************
c     ... Elements of the scattering matrix: F11, F33, F12, and F34
      IF( .NOT.NOWR ) THEN
         OPEN(21, file=DIR_OUT//'/Phm_out.dat')
         DO L = 1,NG
            WRITE(21,'(I6,1x,E12.4,4(1x,E15.7))') 
     $           L, DACOS(X(L))/Pi*180., (Phm(1,K,L),K=1,4)
         ENDDO
         CLOSE(21)

c        ... Derivatives the elements of the scattering matrix: 
c            F11, F33, F12, and F34 with respect to the modal radius
         OPEN(21, file=DIR_OUT//'/Phm_Dr_out.dat')
         DO L = 1,NG
            WRITE(21,'(I6,1x,E12.4,4(1x,E15.7))') 
     $           L, DACOS(X(L))/Pi*180., (Phm_Dr(1,K,L),K=1,4)
         ENDDO
         CLOSE(21)

c        ... Derivatives the elements of the scattering matrix: 
c            F11, F33, F12, and F34 with respect to RMS
         OPEN(21, file=DIR_OUT//'/Phm_Ds_out.dat')
         DO L = 1,NG
            WRITE(21,'(I6,1x,E12.4,4(1x,E15.7))') 
     $           L, DACOS(X(L))/Pi*180., (Phm_Ds(1,K,L),K=1,4)
         ENDDO
         CLOSE(21)
 
         WRITE(*,'(2(A,E12.4))') '# R1 =',R1,'   R2 =',R2
         WRITE(*,'(2(A,E12.4))') '# REFF =',REFF,'   VEFF =',VEFF
         WRITE(*,'(3(A,I4))') '# NK =',NK,'   N =',N,'   NP =',NP   
         WRITE(*,'(A,E12.4)') '# <COS> =', Grs(1,1,2)/3d0 
         WRITE(*,'(3(A,D13.6))') 
     $        '# CEXT = ',CEXT(1),'   CSCA = ',CSCA(1), 
     $        '   ALBEDO = ',ALB
c         WRITE(*,'(A)') '#        ALPHA 1    ALPHA 2    ALPHA 3',
c     $        '    ALPHA 4     BETA 1     BETA 2'
c     ..........................................
c         DO L = 1,L1MAX 
c            WRITE(*,'(I4,1X,6F11.5)') L-1, (Grs(1,I,L), I=1,6)
c         ENDDO 
      ENDIF

      RETURN

      CONTAINS

C********************************************************************** 
C     Calculation of the angular functions PI(N) and TAU(N) for 
C             given argument U=COS(THETA) N .LE. NMAX 
C**********************************************************************  
      SUBROUTINE ANGL( NMAX, U, PIN, TAUN ) 

      INTEGER  NMAX, N
      REAL*8   PIN(NMAX), TAUN(NMAX)
      REAL*8   P1, P2, S, U, T
c     ------------------------------------------------------------------
      P1 = 0.d0 ; P2 = 1.d0
      N = 1
    5 S = U*P2 
      T = S-P1
      TAUN(N) = DFLOAT(N)*T - P1 
      PIN(N) = P2
      P1 = P2
      P2 = S + COEFF1(N)*T 
      N = N + 1 
      IF( N.LE.NMAX ) GO TO 5 
      RETURN   
      END SUBROUTINE ANGL

c     *****************************************************************
      SUBROUTINE GRS_A( Phi,Gri,Nps )
c     ... Input parameters
      INTEGER, INTENT(IN):: Nps
      REAL (KND)  Phi(Nps,4,NG), Gri(Nps,6,L1_max)
c     ... Local
      INTEGER  K
      DO K = 1, Nps
         FF11 = Phi(K,1,I)*WI ; FF33 = Phi(K,2,I)*WI 
         FF12 = Phi(K,3,I)*WI ; FF34 = Phi(K,4,I)*WI 
         FP   = FF11 + FF33 ; FM   = FF11 - FF33 
         Gri(K,1,1:L1MAX) = Gri(K,1,1:L1MAX) + FF11*P1(1:L1MAX) 
         Gri(K,2,1:L1MAX) = Gri(K,2,1:L1MAX) + FP  *P2(1:L1MAX)
         Gri(K,3,1:L1MAX) = Gri(K,3,1:L1MAX) + FM  *P3(1:L1MAX) 
         Gri(K,4,1:L1MAX) = Gri(K,4,1:L1MAX) + FF33*P1(1:L1MAX) 
         Gri(K,5,1:L1MAX) = Gri(K,5,1:L1MAX) + FF12*P4(1:L1MAX) 
         Gri(K,6,1:L1MAX) = Gri(K,6,1:L1MAX) + FF34*P4(1:L1MAX) 
      ENDDO
      
      END SUBROUTINE GRS_A 
c     ******************************************************************

      SUBROUTINE GRS_B( Gri )
c     ... Input parameters
c      INTEGER, INTENT(IN) :: Nps
      REAL (KND)  Gri(Nps,6,L1_max)

      Gri(K,1,L1) = Gri(K,1,L1)*CL
      A2 = Gri(K,2,L1)*CL*0.5D0 
      A3 = Gri(K,3,L1)*CL*0.5D0
      Gri(K,2,L1) = A2 + A3 
      Gri(K,3,L1) = A2 - A3 
      Gri(K,4,L1) = Gri(K,4,L1)*CL
      Gri(K,5,L1) = Gri(K,5,L1)*CL
      Gri(K,6,L1) =-Gri(K,6,L1)*CL
      END SUBROUTINE GRS_B

C***********************************************************************
C        Calculation of the generalized spherical functions 
C***********************************************************************
      SUBROUTINE GENER(U) 

      INTEGER  L1, LMAX, L2, L3
      REAL *8 DUP, U, DUM, DU, C1, C2, C3, C4, C5, C6, C7, C8,
     $     CU1, CU2, DL

      DUP = 1.d0 + U ; DUM = 1.d0 - U 
      DU = U*U 
      P1(1) = 1d0 ; P1(2) = U   ; P1(3) = 0.5d0*(3d0*DU - 1.d0)
      P2(1:2) = 0d0 ;  P2(3) = 0.25d0*DUP*DUP 
      P3(1:2) = 0d0 ;  P3(3) = 0.25d0*DUM*DUM  
      P4(1:2) = 0d0 ;  P4(3) = D6*(DU - 1.d0)
      LMAX = L1MAX-1
      DO L1 = 3,LMAX  
         C1 = COEF1(L1)
         C2 = COEF2(L1)
         C3 = COEF3(L1)
         C4 = COEF4(L1)
         C5 = COEF5(L1)
         C6 = COEF6(L1)
         C7 = COEF7(L1)
         C8 = COEF8(L1)
         CU1 = C2*U  ; CU2 = C6*U 
         L2 = L1+1   ; L3 = L1-1 
         DL = DFLOAT(L3)
         P1(L2) = C1*( CU1*P1(L1)      - DL*P1(L3) ) 
         P2(L2) = C5*( (CU2-C7)*P2(L1) - C8*P2(L3) ) 
         P3(L2) = C5*( (CU2+C7)*P3(L1) - C8*P3(L3) )
         P4(L2) = C3*( CU1*P4(L1)      - C4*P4(L3) )
      ENDDO
      RETURN 
      END SUBROUTINE GENER

      END                       ! SPHER

C*********************************************************************

      SUBROUTINE DISTRB( NNK, YY, WY, FD, FD_Dr, FD_Ds, NDISTR, AA, BB,
     &     GAM, R1, R2, REFF, VEFF, AREA, VOLUME, RVW, RMEAN, PI, AA1,
     $     BB1, AA2, BB2, NOWR, Nps, PM_2i5, PM_2i5_MOD)
c     ------------------------------------------------------------------
      USE AER_MIE,  ONLY:  Rm_log

      INTEGER NNK, NDISTR, Nps
      LOGICAL NOWR, PM_2i5, PM_2i5_MOD
      REAL*8 AA(Nps), BB(Nps), GAM, R1, R2, REFF, VEFF, AREA(Nps), 
     $     VOLUME(Nps), RVW, RMEAN, AA1, BB1, AA2, BB2, PI, V1
      REAL*8 YY(NNK), WY(NNK), FD(Nps,NNK), FD_Dr(Nps,NNK), 
     $     FD_Ds(Nps,NNK), Prod(NNK)

      INTEGER I, Jst
      REAL*8  A2, B2, DA, DA1, DA2, DB, DAB, X, Y, Y1, Y2, Snor(Nps),
     $      Snor1, G, XI, B1, PDr, PDs, DUM
c     ==================================================================

      SELECT CASE(NDISTR)
      CASE(1)
         IF( .NOT.NOWR ) THEN
            WRITE(56,'(A)') '# Modified gamma distribution:'
            WRITE(56,'(3(A,F7.4))') 
     $           '# ALPHA =',AA(1),'  r_c =',BB(1),'  GAMMA =',GAM
         ENDIF
         A2 = AA(1)/GAM 
         DB = 1d0/BB(1)
         DO I = 1,NNK 
            X = YY(I)
            Y = X**AA(1) 
            X = X*DB
            Y = Y*DEXP(-A2*(X**GAM))
            WY(I) = WY(I)*Y 
         ENDDO
c     ------------------------------------------------------------------
      CASE(2)
         IF( .NOT.NOWR ) THEN
            WRITE(56,'(A)') '# log normal distribution:'
            DO I = 1, Nps
               WRITE(56,'(2(A,F8.4))') 
     $              '# r_g =', AA(I), '  [ln(sigma_g)]**2 =',BB(I)
            ENDDO
         ENDIF
         PDs = 0.d0    ; PDr = 1.d0
         DO Jst = 1,Nps
            DA = 1.d0/AA(Jst) ; B1 = BB(Jst)
            IF( .NOT.Rm_log ) THEN
               PDs = 2.d0*B1 ; PDr = DEXP(B1)
            ENDIF
            DO I = 1,NNK
               X  = YY(I)
               Y  = DLOG(X*DA)
               Y1 = DEXP( -Y*Y*0.5d0/B1 )/X
               FD   (Jst,I) = Y1
c              ... Derivative w.r.t. mode radius and variance
               FD_Dr(Jst,I) = Y*DA/B1*PDr * Y1
               FD_Ds(Jst,I) = ( (Y + PDs)*Y*0.5d0/B1/B1 )* Y1
            ENDDO
         ENDDO
c     ------------------------------------------------------------------
      CASE(3)
         IF( .NOT.NOWR ) THEN
            WRITE(56,'(A)')
     $           '# Power law distribution of Hansen & Travis 1974:'         
         ENDIF
         DO I = 1,NNK
            X = YY(I) 
            WY(I) = WY(I)/(X*X*X)
         ENDDO                                                            
c     ------------------------------------------------------------------
      CASE(4)
         IF( .NOT.NOWR ) THEN
            WRITE(56,'(A)') '# Gamma distribution:'
            WRITE(56,'(2(A,F8.4))') '# a =', AA(1), '  b =',BB(1)
         ENDIF
         B2=(1D0-3D0*BB(1))/BB(1)
         DAB=1D0/(AA(1)*BB(1))
         DO I = 1,NNK 
            X = YY(I)
            X = (X**B2)*DEXP(-X*DAB)
            WY(I) = WY(I)*X
         ENDDO 
c     ------------------------------------------------------------------
      CASE(5)
         IF( .NOT.NOWR ) THEN
            WRITE(56,'(A)') '# Modified power law distribution:'
            WRITE(56,'(A,D11.4)') '#   ALPHA =', BB(1)
         ENDIF
         DO I = 1,NNK
            X = YY(I)
            IF (X.LE.R1) WY(I) = WY(I)
            IF (X.GT.R1) WY(I) = WY(I)*(X/R1)**BB(1)
         ENDDO 
c     ------------------------------------------------------------------
      CASE(6)
         IF( .NOT.NOWR ) THEN
            WRITE(56,'(A)') '# Bimodal volume log normal distribution:'
            WRITE(56,'(2(A,F8.4))') 
     $           '# r_g1 =', AA1, '  [ln(sigma_g1)]**2 =',BB1
            WRITE(56,'(3(A,F8.4))') 
     $           '# r_g2 =', AA2, '  [ln(sigma_g2)]**2 =',BB2,
     $           ' gamma =', GAM
         ENDIF
         DA1 = 1.d0/AA1       
         DA2 = 1.d0/AA2                  
         DO I = 1,NNK
            X = YY(I)
            Y1 = DLOG(X*DA1)
            Y2 = DLOG(X*DA2)                                           
            Y1 = DEXP(-Y1*Y1*0.5d0/BB1)
            Y2 = DEXP(-Y2*Y2*0.5d0/BB2)
            WY(I) = WY(I)*(Y1 + GAM*Y2)/(X*X*X*X)                        
         ENDDO
         
      END SELECT

      DO Jst = 1,Nps
c        ... PSD normalization factor
         IF( PM_2i5_MOD ) THEN
            Snor(Jst) = DSQRT( 2.d0*Pi*BB(Jst) ) ! Analytic form
         ELSE
            Snor(Jst) = SUM( WY(1:NNK)*FD(Jst,1:NNK) )
         ENDIF
         FD(Jst,1:NNK) = FD(Jst,1:NNK)/Snor(Jst)
c              WRITE(*,'(2x,A,1x,I2,2(2x,E12.4))') '   Snor = ',
c     $            Jst, Snor(Jst), DSQRT(2.d0*Pi*BB(Jst) )         
         Snor1 = SUM( WY(1:NNK)*FD_Dr(Jst,1:NNK) )
         FD_Dr(Jst,1:NNK) = FD_Dr(Jst,1:NNK)/Snor(Jst)
c               correction from normalization factor
     $        - FD(Jst,1:NNK)/Snor(Jst)*Snor1

         Snor1 = SUM( WY(1:NNK)*FD_Ds(Jst,1:NNK) )
         FD_Ds(Jst,1:NNK) = FD_Ds(Jst,1:NNK)/Snor(Jst) 
     $        - FD(Jst,1:NNK)/Snor(Jst)*Snor1
      ENDDO

      DO Jst = 1,Nps 
         Prod(1:NNK) = WY(1:NNK)*FD(Jst,1:NNK)
c        ... Parameters of the selected distribution
         G = 0.d0
         DO I = 1,NNK
            X = YY(I)
            G = G + X*X*Prod(I)
         ENDDO
c        ... effective radius of the size distribution
         REFF = 0.d0
         DO I = 1,NNK
            X = YY(I)
            REFF = REFF + X*X*X*Prod(I)
         ENDDO
         REFF = REFF/G

         VEFF = 0.d0 ; V1 = 0.d0 ; RVW = 0.d0 ; RMEAN  = 0.d0
         DO I = 1,NNK
            X = YY(I)
            XI     = X - REFF
            VEFF   = VEFF   + XI*XI*X*X*Prod(I)
            V1     = V1     + X*X*X    *Prod(I)
            RVW    = RVW    + X*X*X*X  *Prod(I)
            RMEAN  = RMEAN  + X        *Prod(I)
         ENDDO
c         ... Effective variance of the size distribution
         VEFF = VEFF/(G*REFF*REFF)
c        ... Volume weighted average radius
         RVW = RVW/V1
c        ... Average projected area per particle
         AREA(Jst) = G*PI  
c        ... Average volume per particle 
         VOLUME(Jst) = V1*4d0*PI/3d0
         IF( PM_2i5 ) THEN
            DUM = Snor(Jst)/DSQRT(2.d0*Pi*BB(Jst) )
            AREA(Jst)   = AREA(Jst)*DUM**2
            VOLUME(Jst) = VOLUME(Jst)*DUM**2
         ENDIF
      ENDDO  ! over number of PSD (Nps)

      RETURN 
      END  

C**********************************************************     
      SUBROUTINE GAUSS(N, IND1, IND2, Z, W)

      INTEGER N, IND1, IND2
      REAL*8  Z(N), W(N)
      REAL*8  A, B, C
      DATA    A, B, C /1D0,2D0,3D0/
      INTEGER  I, J, K, M, NITER, IND
      REAL*8  F, X, CHECK, PA, PB, PC, DJ, ZZ
c     ------------------------------------------------------------------
      IND = MOD(N,2)
      K=N/2+IND
      F=DFLOAT(N)
      DO 100 I=1,K
          M=N+1-I
          IF(I.EQ.1) X=A-B/((F+A)*F)
          IF(I.EQ.2) X=(Z(N)-A)*4D0+Z(N)
          IF(I.EQ.3) X=(Z(N-1)-Z(N))*1.6D0+Z(N-1)
          IF(I.GT.3) X=(Z(M+1)-Z(M+2))*C+Z(M+3)
          IF(I.EQ.K.AND.IND.EQ.1) X=0D0
          NITER=0
          CHECK=1D-16
   10     PB=1D0
          NITER=NITER+1
          IF (NITER.LE.100) GO TO 15
          CHECK=CHECK*10D0
   15     PC=X
          DJ=A
          DO 20 J=2,N
              DJ=DJ+A
              PA=PB
              PB=PC
   20         PC=X*PB+(X*PB-PA)*(DJ-A)/DJ
          PA=A/((PB-X*PC)*F)
          PB=PA*PC*(A-X*X)
          X=X-PB
          IF(DABS(PB).GT.check*DABS(X)) GO TO 10
          Z(M)=X
          W(M)=PA*PA*(A-X*X)
          IF(IND1.EQ.0) W(M)=B*W(M)
          IF(I.EQ.K.AND.IND.EQ.1) GO TO 100
          Z(I)=-Z(M)
          W(I)=W(M)
  100 CONTINUE

      IF( IND2.NE.1 ) GOTO 115
      WRITE(*,'(A,I4,A)') 
     $     ' ***  Points and weights of Gaussian quadrature formula',
     *     ' of ',N,'-th order'
      DO I=1,K
         ZZ = -Z(I)
         WRITE(*,'(1x,I4,2x,2(1x,F17.14))') I, ZZ, W(I)
      ENDDO

  115 CONTINUE

      IF( IND1 == 0 ) RETURN
      Z(:) = (A + Z(:))/B

      RETURN
      END
 
C***********************************************************************
c
C   Computation of R1 and R2 for the power law size distribution with
C           effective radius A and effective variance B
c 
c***********************************************************************
      SUBROUTINE POWER(A, B, R1, R2)
      REAL*8  A, B, R1, R2
      REAL*8, EXTERNAL :: F, ZEROIN
      REAL*8  AA, BB, AX, BX
      COMMON   AA, BB
      AA = A
      BB = B
      AX = 1D-12
      BX = A  
      R1 = ZEROIN (AX,BX,F,0D0)
      R2 = (1D0+B)*2D0*A-R1
      RETURN
      END
 
C***********************************************************************
 
      DOUBLE PRECISION FUNCTION F(R1)
      REAL*8 A, B, R1, R2
      COMMON A, B
      R2 = (1d0 + B)*2d0*A - R1
      F  = (R2 - R1)/DLOG(R2/R1) - A
      RETURN
      END
 
C***********************************************************************
 
      DOUBLE PRECISION FUNCTION ZEROIN (AX,BX,F,TOL)
      REAL*8 AX, BX, TOL, EPS, TOL1, A, B, FA, FB, C, FC, D, E,
     $     XM, S, P, Q, R 
      REAL*8, EXTERNAL :: F

      EPS=1D0
   10 EPS=0.5D0*EPS
      TOL1=1D0+EPS
      IF (TOL1.GT.1D0) GO TO 10
   15 A=AX
      B=BX
      FA=F(A)
      FB=F(B)
   20 C=A
      FC=FA
      D=B-A
      E=D
   30 IF (DABS(FC).GE.DABS(FB)) GO TO 40
   35 A=B
      B=C
      C=A
      FA=FB
      FB=FC
      FC=FA
   40 TOL1=2D0*EPS*DABS(B)+0.5D0*TOL
      XM=0.5D0*(C-B)
      IF (DABS(XM).LE.TOL1) GO TO 90
   44 IF (FB.EQ.0D0) GO TO 90
   45 IF (DABS(E).LT.TOL1) GO TO 70
   46 IF (DABS(FA).LE.DABS(FB)) GO TO 70
   47 IF (A.NE.C) GO TO 50
   48 S=FB/FA
      P=2D0*XM*S
      Q=1D0-S
      GO TO 60
   50 Q=FA/FC
      R=FB/FC
      S=FB/FA
      P=S*(2D0*XM*Q*(Q-R)-(B-A)*(R-1D0))
      Q=(Q-1D0)*(R-1D0)*(S-1D0)
   60 IF (P.GT.0D0) Q=-Q
      P=DABS(P)
      IF ((2D0*P).GE.(3D0*XM*Q-DABS(TOL1*Q))) GO TO 70
   64 IF (P.GE.DABS(0.5D0*E*Q)) GO TO 70
   65 E=D
      D=P/Q
      GO TO 80
   70 D=XM
      E=D
   80 A=B
      FA=FB
      IF (DABS(D).GT.TOL1) B=B+D
      IF (DABS(D).LE.TOL1) B=B+DSIGN(TOL1,XM)
      FB=F(B)
      IF ((FB*(FC/DABS(FC))).GT.0D0) GO TO 20
   85 GO TO 30
   90 ZEROIN=B
      RETURN
      END

C **********************************************************************
