!***********************************************************************
!
!       SCIATRAN - Version 4.7 - Last modification: 08 Aug 2023
!
!       Institute of Remote Sensing, University of Bremen, Germany
!
!***********************************************************************
!           Calculate phase function of ice layer using stochastic 
!        mixture model of ice particles and air gaps (vector version)
!     ******************************************************************
SUBROUTINE SM_MODEL_V(Ncl, Nam_DB, N_1, *)
  USE CLD_INP,     ONLY: ADD_ABS, MLCS
  USE CLOUDS,      ONLY: CLD_LAYER_COEFF, CLDPROFILE, CLD_NETWAVS, &
                         NCW => N_CLD_NETWAVS, Ref_ip, RefI_r, RefI_i
  USE INTERN_DIMS, ONLY: Nvs, Nm_vs
  USE LOG_PARS,    ONLY: DIR_OUT, EU
  USE ST_VARS,     ONLY: RTD => RAD_TO_DEG, Pie
  USE TOOLS,       ONLY: READ_FILE
  USE TYPE_VARS,   ONLY: KND
! -----------------------------------------------------------------------
  
  INTEGER    Ncl, N_1
  CHARACTER (LEN=200) NAM_DB(5)      
!    Local variables:
  CHARACTER (LEN=*), PARAMETER :: R_NAME='SM_MODEL_V'
  INTEGER I, J, K, L, II, Ik, N_ang, Ltc, N_Ice, Max_sm
  REAL (KND) STEP, R_ri(2),  Rout_int, One_out, &
       Rr, U1, D1, T2, T3, Td, x, omeg, Lal, Det
  REAL (KND), ALLOCATABLE::  ANG_i(:), As_tout(:), Tout(:,:,:),     &
       Rout(:,:,:), Rin(:,:,:), Ft(:,:,:), Ft0(:,:,:), Cas_tout(:), &
       Ca_i(:), Tet_t(:), Ctet_t(:), H_ang(:)
  INTEGER, PARAMETER :: N_gau = 500, Mum = 500
  INTEGER  Na_st, Nc_fin(NCW)
  LOGICAL  Error
  REAL (KND)  Poly(MUM,N_gau), Omega(NCW,1), Alph(NCW,1), Yel_conc(1), &
       A_VEC( (Nvs + 1)*(Nvs + 2)/2 )
  REAL (KND)  Abs_yel, Alpha, Norm_fin, XS, XE, WL_loc, Yel_390, Norm
  REAL (KND), DIMENSION (N_gau) :: X_gau, AG_gau, X_rad, &
       Mult, Mult2, Pht, Del, H_gau, Ph_t2, Ph_fin
  REAL (KND), ALLOCATABLE :: Sa_inp(:), Sa_rad(:), REF_WVL(:),  &
       to_cof(:), ro_cof(:), f_cof(:), Jac(:), &
       DATA_R(:,:), Rex_gau(:,:), Rin_gau(:,:),  Ph_ret(:,:),  &
       T6_gau(:,:), T_gs(:,:), Rin_gs(:,:), Rin_gs_mat(:,:), T_gs_mat(:,:),&
       Gl_gs(:,:), Gl_gs_mat(:,:), Gl_gau(:,:), &
       F_gau(:,:),  F_gs(:,:),  &
       Rin_gs_inv(:,:), Hel(:,:), Tes_inv(:,:), &
       Rex_gs(:,:), D_02g(:,:), D_P2g(:,:), D_M2g(:,:)
!     =================================================================
      
!     ... Refractive index of ice (WL input [micron])
  XS = CLD_NETWAVS(1)  /1.d03     !  [nm] ==> [microm] 
  XE = CLD_NETWAVS(NCW) /1.d03
  CALL READ_FILE(3, NAM_DB(5), DATA_R, *880, XS, XE, NPT = 1, Skip_line = 1)
  N_ice = Size(DATA_R,1)
  ALLOCATE ( REF_WVL(N_Ice),RefI_r(NCW),RefI_i(NCW)  )
  REF_WVL(:) = DATA_R(:,1)         ! wavelength in [micron] at this step
  REF_WVL(:) = REF_WVL(:)*1000.0d0 ! Transform from [mkm] => [nm]      
  Call LIN_INT (N_ice, REF_WVL, DATA_R(:,2), NCW, CLD_NETWAVS, RefI_r, 'Z')
  Call LIN_INT (N_ice, REF_WVL, DATA_R(:,3), NCW, CLD_NETWAVS, RefI_i, 'Z')
  DEALLOCATE( DATA_R )
      
!     ... Real and imaginary part of ice refractive index
  WRITE(*,*) '  ------> "sm_model_v.f"'
  DO I = 1, NCW
     WRITE(*,'(2x,F8.2,*(2x,e12.4))') CLD_NETWAVS(I), RefI_r(I), RefI_i(I)
  ENDDO
!     ... Pause '   Pause after reading of refractive index of ice'
! -----------------------------------------------------------------------

! ... Gauss grid and weights for integration (-1,1)
  CALL GAULEG ( -1.d0, 1.d0, X_gau, AG_gau, N_gau )
! ... Legendre polynomials for all Gauss angles
  CALL WIGNER_PLG( X_gau, N_gau, 0, MuM, Poly )
!     ... Arrays used in the vector case
  ALLOCATE( D_02g(MuM,N_gau), D_P2g(MuM,N_gau), & 
       D_M2g(MuM,N_gau) )
  CALL WIGNER_PLG     ( X_gau, N_gau, 2, MuM, D_02g )
  CALL WIGNER_FUNCTION( X_gau, N_gau, 2, MuM, D_P2g, D_M2g )

  X_rad(:) = DACOS( X_gau(N_gau:1:-1) )

  N_ang = 181
  Na_st = 181
  ALLOCATE( Sa_inp(Na_st), Sa_rad(Na_st), &
       to_Cof(Mum), ro_Cof(Mum), f_Cof(Mum),  &
       Tet_t(Na_st), Ctet_t(Na_st), H_ang(Na_st) ) ! A
  to_Cof = 0.d0 ;  ro_Cof = 0.d0
      
! ... Equidistant grid of scattering angles [0,180]
  STEP = 180.d0/DBLE( Na_st-1 ) ; Sa_inp(1) = 0.d0
  DO I = 2,Na_st
     Sa_inp(I) = Sa_inp(I-1) + STEP
  ENDDO
  Sa_rad(:) = Sa_inp(:)/RTD
      
!     ... Auxiliary functions for Legendre series
  DO I = 1, MuM
     Mult2(I) = (2.d0*DBLE(I) - 1.d0)/2.d0
     Mult(I)  = (2.d0*DBLE(I) - 1.d0)
  ENDDO
!     ... Vertically homogeneous polydisperse snow layer (stochastic model)
  WRITE(*,*)  '  Mean chord at the top ', CLDPROFILE(Ncl,1,7),' [micron]'
  Ref_ip = CLDPROFILE(Ncl,1,7) *1.d3 ! [micron] in [nm]
      
!     ... Incident angles in the case of internal reflection corresponding
!         to the scattering angles [0,180]
  Tet_t(:)  = (180.d0 - Sa_inp(:))/2.d0
  Ctet_t(:) = DCOS( Tet_t(:)/180.d0*Pie )

  ALLOCATE( ANG_i(N_ANG), Tout(N_ANG,Nvs,Nvs),      &
       As_tout(N_ANG), Cas_tout(N_ANG), Ca_i(N_ANG), &
       Rout(N_ANG,Nvs,Nvs), Ft(Na_st,Nvs,Nvs), Ft0(N_ANG,Nvs,Nvs), &
       T6_gau(N_gau,Nm_vs), T_gs(MuM,Nm_vs), T_gs_mat(Nvs,Nvs), &
       Gl_gs(MuM,Nm_vs), Gl_gs_mat(Nvs,Nvs),  Ph_ret(N_gau,Nm_vs), &
       Jac(N_ang), Rex_gau(N_gau,Nm_vs), Rin_gau(N_gau,Nm_vs), &
       Rin_gs(MuM,Nm_vs), Rin_gs_mat(Nvs,Nvs), Rex_gs(MuM,Nm_vs),  &
       F_gs(N_gau,Nm_vs), &
       Rin_gs_inv(Nvs,Nvs), Hel(Nvs,Nvs), Tes_inv(Nvs,Nvs), &
       Rin(Na_st,Nvs,Nvs), Gl_gau(N_gau,Nm_vs), F_gau(N_gau,Nm_vs) )
      
!     ... Equidistant grid of incident angles air-ice interface [0,90]
  STEP = 90.d0/DBLE( N_ANG-1 ) ; ANG_i(1) = 0.d0         
  DO I = 2,N_ANG
     ANG_i(I) = ANG_i(I-1) + STEP
  ENDDO
  Ca_i(:) = DCOS( ANG_I(:)/180.*Pie )

!     ... Assuming constant over snow layer yellow matter absorption coefficient
  Yel_390 = 0.d0
  IF( ADD_ABS(Ncl) ) Yel_390 = MLCS(Ncl)%C_SOOT(1) *1.d-9 ! m^-1 ==> nm^-1
      
  Yel_conc(1) = Yel_390
  CALL OMEGA_SM( NCW, CLD_NETWAVS, RefI_r, RefI_i, NCL, 1, &
       CLDPROFILE(Ncl,1,7), Omega, Alph, Yel_conc )
      
  DO II = 1, NCW
     Gl_gs = 0.d0
!    ... Setting of refractive index
     R_ri(1) = RefI_r(II)  ;  R_ri(2) = RefI_i(II)         
!    ... Scattering angles corresponding to the Fresnel refraction
!        air-ice interface at incident angles [0,Pi/2]
     DO I = 1,N_ANG
        As_tout(I) = ANG_i(I)   &
             - DASIN( DSIN(ANG_i(I)/180.d0*Pie)/R_ri(1) )/Pie*180.d0
     ENDDO
     Cas_tout(:) = DCOS( As_tout(:)/180.d0*Pie )
         
!     ... Fresnel transmission air-ice at incident angles [0,Pi/2]
!         Tout[N_ANG,Nvs,Nvs]
     CALL Fre_Taw_v(N_ANG, Tout, Ca_I, R_ri)
!     ... Fresnel reflection air-ice at incident angles [0,Pi/2]
!         Rout[N_ANG,Nvs,Nvs]
     CALL Fre_Ref_v(N_ANG, Rout, Ca_I, R_ri)
!     ... Fresnel internal reflection at incident angles [0,Pi/2]
!         with values equal to 1 for total internal reflection
!         Rin[Na_st,Nvs,Nvs]
     CALL Fre_tot(Na_st, Rin, Ctet_t, R_ri)

!     ----------------- Transmission matrix --------------------
!    ... Jacobian and other multipliers of transmission matrix
     Jac(:) = 4.d0*R_ri(1)**3*(R_ri(1) - Cas_tout(:))**2   &
          /( R_ri(1)**2 - 2.d0*R_ri(1)*Cas_tout(:) + 1.d0 )**2
     WRITE(*,*) '    =====> vector '
     DO I = 1, N_ang
        Ft0(I,:,:) = Tout(I,:,:) !*Jac(I)
!        write(*,*) I, Ft0(I,1,1)
     ENDDO
!    ... Interpolate matrix Ft for scattering angles between
!        [0, 180] with zero values for scattering angles > crit.
     Ft = 0.d0
     CALL LIN_INT(N_ang, As_tout, Ft0(:,1,1), Na_st, Sa_inp, Ft(:,1,1), 'Z')
     Ft(:,2,2) = Ft(:,1,1)
     CALL LIN_INT(N_ang, As_tout, Ft0(:,3,3), Na_st, Sa_inp, Ft(:,3,3), 'Z')
     Ft(:,4,4) = Ft(:,3,3)
     CALL LIN_INT(N_ang, As_tout, Ft0(:,1,2), Na_st, Sa_inp, Ft(:,1,2), 'Z')
     Ft(:,2,1) = Ft(:,1,2)     

!     DO I = 1, Na_st
!        write(*,*) I, Ft(I,1,1)
!     ENDDO
     
!     ... Interpolate scattering function Ft on the Gauss grid
     T6_gau = 0.d0
     DO Ik = 1, 4, 2 
        CALL AKIMA_INTERP( Na_st, Sa_rad, Ft(:,Ik,Ik), &
             N_gau,  X_rad, H_gau, Error)  ;  IF( Error ) GOTO 201
!     ... Invert to start from negative values
        T6_gau(:,Ik)   = H_gau(N_gau:1:-1)
        T6_gau(:,Ik+1) = T6_gau(:,Ik)
     ENDDO
     CALL AKIMA_INTERP( Na_st, Sa_rad, Ft(:,1,2), &
          N_gau,  X_rad, H_gau, Error)  ;   IF( Error ) GOTO 201
!     ... Invert to start from negative values
     T6_gau(:,5) = H_gau(N_gau:1:-1)
!     Norm = SUM( T6_gau(:,1)*AG_gau(:) )*0.5d0
!     WRITE(*,*) '   Norm of T matrix: ', NORM
     CALL SCM_TO_GSM( N_gau, MuM, T6_gau, T_gs)
!
     OPEN(45, FILE='./DATA_OUT/T_gau-vec.dat' )
     DO I = 1, N_gau
        WRITE(45,*) DACOS( X_gau(I) )/Pie*180.,  &
             T6_gau(I,1), T6_gau(I,3), T6_gau(I,5)
     ENDDO
     CLOSE( 45 )
!
     CALL GSM_TO_SCM( N_gau, MuM, Ph_ret, T_gs )
     OPEN(45, FILE='./DATA_OUT/T_ret-vec.dat' )
     DO I = 1, N_gau
        WRITE(45,*) DACOS( X_gau(I) )/Pie*180.,  &
             Ph_ret(I,1), Ph_ret(I,3), Ph_ret(I,5)
     ENDDO
     CLOSE( 45 )
!     STOP  '    Stop in "sm_model_f.f90" after transmission matrix'
!    ----------------------------------------------------------------------
     
!    ------------------ Internal reflection matrix ------------------------
     Rin_gau = 0.d0
!    ... Interpolate internal reflection matrix elements on the Gauss grid 
     DO Ik = 1, 4, 2 
        H_ang(:) = Rin(:,Ik,Ik) !   from scalar case /4.d0/Pie
        CALL AKIMA_INTERP(Na_st, Sa_rad, H_ang, N_gau, X_rad, H_gau, Error)
        IF( Error ) GOTO 201
!       ... Invert to start from negative value
        Rin_gau(:,Ik)   = H_gau(N_gau:1:-1)
        Rin_gau(:,Ik+1) = H_gau(N_gau:1:-1)
     ENDDO
     H_ang(:) = Rin(:,1,2)
     CALL AKIMA_INTERP(Na_st, Sa_rad, H_ang, N_gau, X_rad, H_gau, Error)
     Rin_gau(:,5) = H_gau(N_gau:1:-1)

     WRITE(*,*) '     SCM_TO_GSM  ==> Rin_gs  '
     CALL SCM_TO_GSM( N_gau, MuM, Rin_gau, Rin_gs)
     !pause
     
     CALL SET_COEFF  ! ---> Alpha
     Lal = 1.d0/( 1.d0 + Alpha )
     DO L = 1, MuM
        Rin_gs_mat = 0.d0
        DO I = 1, Nvs
           Rin_gs_mat(I,I) = 1.d0 - Rin_gs(L,I)* Lal
        ENDDO
        Rin_gs_mat(1,2) = - Rin_gs(L,5)* Lal
        Rin_gs_mat(2,1) = - Rin_gs(L,5)* Lal
!        FORALL (I = 1:Nvs)   &
!             A_VEC(I*(I - 1)/2 + 1:I*(I + 1)/2) = Rin_gs_mat(I,1:I)
!        CALL CHINV3( Nvs*(Nvs+1)/2, Nvs, A_VEC )
!        FORALL (I = 1:Nvs)  &
!             Rin_gs_inv(I,1:I) = A_VEC(I*(I - 1)/2 +1:I*(I + 1)/2)
!        FORALL (I = 1:Nvs, J = 1:Nvs, J > I)  &
!             Rin_gs_inv(I,J) = Rin_gs_inv(J,I)
!        Hel(:,:) = MATMUL( Rin_gs_mat(:,:),Rin_gs_inv(:,:) )
!        IF( L==3.OR.L==4.OR.L==5) THEN
        Det = Rin_gs_mat(1,1)*Rin_gs_mat(2,2) - Rin_gs_mat(1,2)*Rin_gs_mat(2,1)
!           write(*,*) '   L = ', L
!           DO I = 1, Nvs
!              write(*,'(2x,*(2x,e12.4))') (Rin_gs_mat(I,J), J = 1, Nvs)
!           ENDDO
!           write(*,*) '   Rin_gs_inv '
!           DO I = 1, Nvs
!              write(*,'(2x,*(2x,e12.4))') (Rin_gs_inv(I,J), J = 1, Nvs)
!           ENDDO
!           write(*,*) '   Tes_inv '
        Rin_gs_inv = 0.d0
!          ... Inverse upper-left 2X2 sub-matrix
        Rin_gs_inv(1,1) =  Rin_gs_mat(2,2) ; Rin_gs_inv(2,2) =  Rin_gs_mat(1,1)
        Rin_gs_inv(1,2) = -Rin_gs_mat(1,2) ; Rin_gs_inv(2,1) = -Rin_gs_mat(2,1)
        Rin_gs_inv      = Rin_gs_inv/Det
!          ... Inverse lower-right 2X2 sub-matrix
        Rin_gs_inv(3,3) = 1.d0/Rin_gs_mat(3,3)
        Rin_gs_inv(4,4) = 1.d0/Rin_gs_mat(4,4)

        WRITE(*,*) L, Rin_gs_mat(1,1), Rin_gs(L,1) !,  Rin_gs_inv(1,1) 

        T_gs_mat = 0.d0
        DO I = 1, Nvs
           T_gs_mat(I,I) = T_gs(L,I)
        ENDDO
        T_gs_mat(1,2) = T_gs(L,5)  ;  T_gs_mat(2,1) = T_gs(L,5)

        Hel(:,:) = MATMUL( Rin_gs_inv(:,:),T_gs_mat(:,:) )
        Gl_gs_mat(:,:) = MATMUL( T_gs_mat(:,:),Hel(:,:) )
        Gl_gs_mat = Gl_gs_mat(:,:)*Lal/R_ri(1)**2
        DO I = 1, Nvs
           Gl_gs(L,I) = Gl_gs_mat(I,I)
        ENDDO
        Gl_gs(L,5) = Gl_gs_mat(1,2)
     ENDDO  ! over expansion coefficients (L)
     CALL GSM_TO_SCM( N_gau, MuM, Gl_gau, Gl_gs )
!
     OPEN(45, FILE='./DATA_OUT/Rin_gau.dat' )
     DO I = 1, N_gau
        WRITE(45,*) DACOS( X_gau(I) )/Pie*180.,  &
             Rin_gau(I,1), Rin_gau(I,3), Rin_gau(I,5)
     ENDDO
     CLOSE( 45 )
     OPEN(45, FILE='./DATA_OUT/Gl_gau.dat' )
     DO I = 1, N_gau
        WRITE(45,*) DACOS( X_gau(I) )/Pie*180.,  &
             Gl_gau(I,1), Gl_gau(I,3), Gl_gau(I,5)
     ENDDO
     CLOSE( 45 )

     CALL GSM_TO_SCM( N_gau, MuM, Ph_ret, Rin_gs )
     OPEN(45, FILE='./DATA_OUT/Rin_ret.dat' )
     DO I = 1, N_gau
        WRITE(45,*) DACOS( X_gau(I) )/Pie*180.,  &
             Ph_ret(I,1), Ph_ret(I,3), Ph_ret(I,5)
     ENDDO
     CLOSE( 45 )
!     STOP  ' Stop in "sm_model_f.f90" after internal reflection matrix'
!    ------------------------------------------------------------------

!    ------------------ External reflection matrix --------------------
     Rex_gau = 0.d0
     DO Ik = 1, 4, 2 
!       ... Start from 1 to -1 
        H_ang(:) =  Rout(Na_st:1:-1,Ik,Ik)
!       ... Interpolate scattering function Ft on the Gauss grid
        CALL AKIMA_INTERP( Na_st, Sa_rad, H_ang, N_gau, X_rad,  H_gau, Error)
        IF( Error ) GOTO 201
!       ... Invert to start from negative values
        Rex_gau(:,Ik)   =  H_gau(N_gau:1:-1)
        Rex_gau(:,Ik+1) =  H_gau(N_gau:1:-1)
     ENDDO  ! over diagonal elements of external reflection matrix (Ik)
!    ... Matrix element 1,2
     H_ang(:) =  Rout(Na_st:1:-1,1,2)
!    ... Interpolate scattering function Ft on the Gauss grid
     CALL AKIMA_INTERP( Na_st, Sa_rad, H_ang, N_gau, X_rad,  H_gau, Error)
     IF( Error ) GOTO 201
!    ... Invert to start from negative values
     Rex_gau(:,5) = H_gau(N_gau:1:-1)
     CALL SCM_TO_GSM( N_gau, MuM, Rex_gau, Rex_gs)
!    ----------------------------------------------------------------
     DO I = 1, N_gau
        WRITE(*,'(2x,f7.3,*(2x,e12.4))')  &
             DACOS( X_gau(I) )/Pie*180., Rex_gau(I,1), Gl_gau(I,1)
     ENDDO
     
     F_gau(:,:) = Rex_gau(:,:) + Gl_gau(:,:)
     
     Norm_fin = SUM( F_gau(:,1)*AG_gau(:) )*0.5d0
     F_gau(:,:) = F_gau(:,:)/Norm_fin
!   ... Expansion coefficients of the final scattering matrix
     CALL SCM_TO_GSM( N_gau, MuM, F_gau, F_gs)
     OPEN(45, FILE='./DATA_OUT/F_gau.dat' )
     DO I = 1, N_gau
        WRITE(45,*) DACOS( X_gau(I) )/Pie*180.,  &
             F_gau(I,1), F_gau(I,3), F_gau(I,5)
     ENDDO
     CLOSE( 45 )

     CALL GSM_TO_SCM( N_gau, MuM, Ph_ret, F_gs )
     OPEN(45, FILE='./DATA_OUT/F_ret.dat' )
     DO I = 1, N_gau
        WRITE(45,*) DACOS( X_gau(I) )/Pie*180.,  &
             Ph_ret(I,1), Ph_ret(I,3), Ph_ret(I,5)
     ENDDO
     CLOSE( 45 )

!     STOP  ' Stop in "sm_model_f.f90" after final matrix'
      
!     Pht(:) = 0.d0
!     DO L = 1,MUM
!        f_cof(L) = Mult2(L)*SUM( Ph_fin(:)*Poly(L,:)*AG_gau(:) )
!        DO J = 1,N_gau
!           Pht(J) = Pht(J) + Poly(L,J)*f_cof(L)
!        ENDDO
!        Del(:) = DABS((Ph_fin(:) - Pht(:)))/Ph_fin(:)*100.d0
!        IF( MAXVAL( Del(:) ) <= 0.1d0 ) THEN
!           MOM_DB(II,1:L,Ncl,1,P_1,1) = f_cof(1:L)
!           EXIT
!        ENDIF
!     ENDDO  ! over expansion coefficients (L)
!     IF( L==MuM ) MOM_DB(II,1:L,Ncl,1,P_1,1) = f_cof(1:L)

     Nc_fin(II) = L
  ENDDO  ! over cloud wavelength grid (II)

  Max_sm = MAXVAL( Nc_fin )
!      WRITE(*,*) '   Max number of moments: ', Max_sm
!      stop '   Stop in "general_cloud_init" '

  RETURN
      
201 Call WRITE_MESSAGE(R_NAME,'ERROR', 'Interpolation error')
  WRITE(EU,'(A,1x,2(2x,I3))') '   Subroutine "akima_interp" '
  RETURN 1     
      
880 WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)
  RETURN 1
  
CONTAINS
      
!   *******************************************************************
  !  Elements of scattering matrix into Greek-symbol matrix 
  SUBROUTINE SCM_TO_GSM( N_gau, MuM, Inp_mat, Out_greek)
    INTEGER N_gau, MuM
    REAL (KND) Clk, Inp_mat(N_gau,Nm_vs), Out_greek(MuM,Nm_vs)
    REAL (KND) Cofm, Cofp, Cf(Nm_vs)
!   ----------------------------------------------------------------------
    DO L = 1, MuM
       Clk = (DBLE(L) - 0.5d0) !/Norm
       Cf(1) =  Clk*SUM( Inp_mat(:,1)*Poly(L,:)*AG_gau(:) )
       WRITE(*,*) L, Cf(1)
       Cf(4) =  SUM( Inp_mat(:,4)*Poly(L,:)*AG_gau(:) )
       Cf(5) = -SUM( Inp_mat(:,5)*D_02g(L,:)*AG_gau(:) )
       Cf(6) = -SUM( Inp_mat(:,6)*D_02g(L,:)*AG_gau(:) )
       Cofm =   SUM( (Inp_mat(:,2) - Inp_mat(:,3))*D_M2g(L,:)*AG_gau(:) )
       Cofp =   SUM( (Inp_mat(:,2) + Inp_mat(:,3))*D_P2g(L,:)*AG_gau(:) )
       Cf(2) = (Cofp + Cofm)/2.d0  ;  Cf(3) = (Cofp - Cofm)/2.d0
       Cf(2:Nm_vs) = Clk*Cf(2:Nm_vs)
       Cf(5:6) = -Cf(5:6)
       Out_greek(L,:) = Cf(:)
    ENDDO  
  END SUBROUTINE SCM_TO_GSM
  
!   *******************************************************************
  SUBROUTINE GSM_TO_SCM( N_gau, MuM, Out_gau, Inp_greek )
    INTEGER N_gau, MuM
    REAL (KND) Inp_greek(MuM,Nm_vs), Out_gau(N_gau,Nm_vs)
    REAL (KND) Cofm, Cofp, Cf(Nm_vs)
!   ----------------------------------------------------------------------
    Out_gau = 0.d0
    DO L = 1, MuM
       Cf(:) = Inp_greek(L,:)
       Cf(5:6) = -Cf(5:6)
       Out_gau(:,1) = Out_gau(:,1) + Poly(L,:) *Cf(1)
       Out_gau(:,4) = Out_gau(:,4) + Poly(L,:) *Cf(4)
       Out_gau(:,5) = Out_gau(:,5) - D_02g(L,:) *Cf(5)
       Out_gau(:,6) = Out_gau(:,6) - D_02g(L,:) *Cf(6)
       Cofp = Cf(2) + Cf(3)  ;  Cofm = Cf(2) - Cf(3)
       Out_gau(:,2) = Out_gau(:,2) &
            + (D_P2g(L,:)*Cofp + D_M2g(L,:)*Cofm)/2.d0
       Out_gau(:,3) = Out_gau(:,3) &
            + (D_P2g(L,:)*Cofp - D_M2g(L,:)*Cofm)/2.d0
    ENDDO
    RETURN
  END SUBROUTINE GSM_TO_SCM
  
!   *******************************************************************
  SUBROUTINE SET_COEFF
    WL_loc = CLD_NETWAVS(II) ;  Abs_yel = 0.d0 
    IF( ADD_ABS(Ncl) ) THEN
       IF( WL_loc <= 500.d0 ) THEN
          Abs_yel = Yel_390*DEXP( -1.5d-2*(WL_loc - 390.d0) ) 
       ELSE
          Abs_yel = Yel_390   &
               *DEXP(-1.5d-2*110.d0 - 1.1d-2*(WL_loc- 500.d0))
       ENDIF
    ENDIF
!        ... Set to zero in the case of non-absorbing media         
!         Alpha = 0.d0
     Alpha = (4.d0*Pie*R_ri(2)/WL_loc + Abs_yel) *Ref_ip
         
     Rr = R_ri(1)   ! real part of refraction index
     U1 = 2.0*(5.0*Rr**6 + 8.0*Rr**5 + 6.0*Rr**4 - 5.0*Rr**3 -Rr-1)
     D1 = 3.0*( Rr**3 +  Rr**2 + Rr + 1.0)*(Rr**4 - 1.0)
     T2 = Rr**2*(Rr**2 - 1.0)**2/( Rr**2 + 1.0)**3   &
          *DLOG( (Rr+1.0)/(Rr-1.0) )
     T3 = 8.0*Rr**4*(Rr**4 + 1.0)/(Rr**4-1.0)**2/(Rr**2 + 1.0)  &
          *DLOG( Rr )
     Td = U1/D1 + T2 - T3
     x = Rr**2* Alpha
     Omeg = 1.0 - x*Td/(x + Td)         
!         write(*,'(A,I3,*(2x,E15.7))') ' two 1-SSA: ', II, 
!     $        1.0 - Omeg, 1. - Omega(II,1)
!         write(*,'(A,*(2x,E15.7))') ' two Alph: ', Alpha, Alph(II,1)
!        ... Save extinction and scattering coefficients corresponding
!            to the unit volume concentration
     CLD_LAYER_COEFF(NCL,II,N_1,1) = 1.d0/CLDPROFILE(Ncl,1,7)*1.d9 ! [km-1] 
     CLD_LAYER_COEFF(NCL,II,N_1+1,1) =  &
          1.d0/CLDPROFILE(Ncl,1,7)*Omeg*1.d9 

  END SUBROUTINE SET_COEFF
      
END SUBROUTINE SM_MODEL_V
! **********************************************************************
