!***************************************************************************
!
!	SCIATRAN - Version 4.3 - Last modification: 12 Feb 2021
!
!	Copyright (C) - Institute of Remote Sensing,
!			University of Bremen, Germany
!
!***************************************************************************
Module SRF_Mod
  Use OutInf, Only: EU
  Use ST_Vars,Only: NSegs
  Use Tools,  Only: Read_Input, Cut_Str, Split_String, Set_Path, Read_File, &
                    ErrMsg, Upper_Lower_Case
  Use Types,  Only : Knd, Dyn_Char
  Character (Len=:), Allocatable :: Str_Inp
  Type (Dyn_Char), ALLOCATABLE :: Read_Arr(:), Str_Arr(:)

  type :: SRF_Container 
     Character (Len=:), Allocatable :: Name, Units
     Real (Knd) Width, Par_A, Delta(2)
     Real (Knd), Allocatable :: Wl(:), Wgt(:)
     Procedure(SRF_WLB), pointer, pass(me) :: SWB, LWB
  end type SRF_Container

  abstract interface
     function SRF_WLB (me,Wl)
       Import :: Knd, SRF_Container
       real (Knd) SRF_WLB
       real (Knd), intent(IN) :: Wl
       class(SRF_Container) :: me
     end function SRF_WLB
  end interface

  Type(SRF_Container), Allocatable :: SRF(:)
  Type(Dyn_Char), Allocatable :: Msg(:)
  
  Private
  Public :: SRF, SRF_Init, SRF_WGT
  
Contains
!***************************************************************************
  Subroutine SRF_Init (ConvFlg, *)
    Logical ConvFlg
    ! Local parameters
    Character (Len=*), Parameter :: R_Name = 'SFR_Init'
    Integer ISeg
    Real (Knd) CutOff

    If(Allocated(SRF)) DeAllocate(SRF)
    Allocate(SRF(NSegs))
    If(.Not.ConvFlg) Then
       Do Iseg = 1, NSegs
          SRF(Iseg)%SWB => DoNothing
          SRF(Iseg)%LWB => DoNothing
       End Do
       Return
    End If
    
    Str_Inp = 'Spectral response function parameters'
    Call Read_Input (Str_Inp, Read_Arr, *880, Nlines = NSegs)
    Do Iseg = 1, NSegs
       Call Split_String(Read_Arr(ISeg)%S, Str_Arr)
       SRF(Iseg)%Name = Cut_Str(Str_Arr(1)%S)
       Call Upper_Lower_Case (SRF(Iseg)%Name,'U2L')
       Select Case (SRF(Iseg)%Name)
       Case ("gaussian")
          READ(Str_Arr(2)%S, *, Err = 101) SRF(Iseg)%Width
          SRF(Iseg)%Width = SRF(Iseg)%Width/2.0D0 ! FWHM --> HWHM
          SRF(Iseg)%Units = Cut_Str(Str_Arr(3)%S)
          Call Upper_Lower_Case (SRF(Iseg)%Units,'U2L')
          READ(Str_Arr(4)%S, *, Err = 101) CutOff
          SRF(Iseg)%Delta(:) = CutOff*SRF(Iseg)%Width
       Case ("sinc","hamming")
          READ(Str_Arr(2)%S, *, Err = 101) SRF(Iseg)%Width
          READ(Str_Arr(3)%S, *, Err = 101) CutOff
          SRF(Iseg)%Delta(:) = CutOff
          SRF(Iseg)%Units = "cm-1"
          If (SRF(ISeg)%Name == "hamming") &
               READ(Str_Arr(4)%S, *, Err = 101) SRF(Iseg)%Par_A
       Case ("boxcar")
          READ(Str_Arr(2)%S, *, Err = 101) SRF(Iseg)%Width
          SRF(Iseg)%Delta(:) = SRF(Iseg)%Width/2.0D0
          SRF(Iseg)%Units = Cut_Str(Str_Arr(3)%S)
          Call Upper_Lower_Case (SRF(Iseg)%Units,'U2L')
       Case ("user")
          CALL Read_SRF (Iseg, *880)
       Case Default
          Call Not_Found (R_Name, Str_Inp, SRF(Iseg)%Name, *880)
       End Select
       
       Select Case (SRF(Iseg)%Units)
       case("nm")
          SRF(Iseg)%SWB => SWB_NM
          SRF(Iseg)%LWB => LWB_NM
       case("cm-1")
          SRF(Iseg)%SWB => SWB_WN
          SRF(Iseg)%LWB => LWB_WN
!          SRF(Iseg)%Width = SRF(Iseg)%Width*1.0E-7 !cm-1 to nm-1
          SRF(Iseg)%Delta = SRF(Iseg)%Delta*1.0E-7 !cm-1 to nm-1
       Case Default
          Call Not_Found (R_Name, Str_Inp, SRF(Iseg)%Units, *880)
       End Select
    End Do

    Return
    
101 Allocate(Msg(2))
    Msg(1)%S = 'Reading error'
    Msg(2)%S = 'Parameter to read: '//STR_INP
    call ErrMsg(R_Name, 'e', Msg)
    Return 1
    
880 Write(EU, *) ' ---> Called by '//R_Name
    Return 1

  End Subroutine SRF_INIT
!***************************************************************************
  Function SWB_NM (me,Wl)
    real (Knd) SWB_NM
    real (Knd), intent(IN) :: Wl
    Class(SRF_Container) :: me
    SWB_NM = Wl - me%Delta(1)
    Return
  End Function SWB_NM
!***************************************************************************
  Function LWB_NM (me,Wl)
    real (Knd) LWB_NM
    real (Knd), intent(IN) :: Wl
    Class(SRF_Container) :: me
    LWB_NM = Wl + me%Delta(2)
    Return
  End Function LWB_NM
!***************************************************************************
  Function SWB_WN (me,Wl)
    ! Short wave boundary
    real (Knd) SWB_WN
    real (Knd), intent(IN) :: Wl
    Class(SRF_Container) :: me
!    write(6,*) "S", me%Delta(1), Wl, 1.0D0/Wl, 1.0D0/(1.0D0/Wl + me%Delta(1)) 
    SWB_WN = 1.0D0/(1.0D0/Wl + me%Delta(1)) 
    Return
  End Function SWB_WN
!***************************************************************************
  Function LWB_WN (me,Wl)
    ! Long wave boundary
    real (Knd) LWB_WN
    real (Knd), intent(IN) :: Wl
    Class(SRF_Container) :: me
!    write(6,*) "L", me%Delta(2), Wl, 1.0D0/Wl, 1.0D0/(1.0D0/Wl - me%Delta(2)) 
    LWB_WN = 1.0D0/(1.0D0/Wl - me%Delta(2)) 
    Return
  End Function LWB_WN
!***************************************************************************
  Function DoNothing (me,Wl)
    ! Just returns the input value
    real (Knd) DoNothing
    real (Knd), intent(IN) :: Wl
    Class(SRF_Container) :: me
    DoNothing = Wl
    Return
  End Function DoNothing
!***************************************************************************
  Function SRF_WGT (Wl,ISeg,IS,IE)
    Use ST_Vars,Only: Wl_RTM, Pie
    Integer, intent(IN) :: ISeg,IS,IE
    Real (Knd), intent(IN) :: Wl
    Real (Knd) SRF_WGT(IE - IS + 1)
    ! Local variables
    Integer II, NN
    Real (Knd) Width, WLL(IE - IS + 1), DWL(IE - IS + 1)
    Real (Knd), Pointer :: WLG(:)
    !==================================================================

    WLG => WL_RTM(IS:IE)
    
    Select Case (SRF(ISeg)%Name)
    Case ("gaussian")
       If(SRF(ISeg)%Units == "nm") Then
          SRF_WGT = Exp( - Log(2.d0)*((Wl - WLG)/SRF(ISeg)%Width)**2)
       Else
          Width = SRF(Iseg)%Width*1.0E-7 !cm-1 to nm-1
          SRF_WGT = Exp( - Log(2.d0)*((1.0D0/Wl - 1.0D0/WLG)/Width)**2)
       End If
    Case ("boxcar")
       SRF_WGT = 1.0D0
    Case ("sinc", "hamming")
       Associate(TMPD => 2.0D0*SRF(ISeg)%Width*1.0E7, &
            DWN => (1.0D0/Wl - 1.0D0/WLG), AA => 2.0D0*SRF(ISeg)%Par_A)
         Where(DWN /= 0.0D0)
            SRF_WGT = Sin(TMPD*Pie*DWN)/(Pie*DWN)
         ElseWhere
            SRF_WGT = TMPD
         End Where
         If (SRF(ISeg)%Name == "hamming") SRF_WGT = SRF_WGT*((1.0D0 - AA) + &
              AA*(TMPD*DWN)**2/(1.0D0 - (TMPD*DWN)**2))
       End Associate
    Case("user")
       If(SRF(ISeg)%Units == "nm") Then
          WLL = WL - WLG
       Else
          WLL = 1.0d0/WL - 1.0d0/WLG
       End If
       CALL LIN_INT(Size(SRF(ISeg)%WL), SRF(ISeg)%WL, SRF(ISeg)%Wgt, &
            Size(WLL), WLL, SRF_WGT, 'Z')
    End Select

    If (SRF(ISeg)%Name /= "boxcar") Then
       If(SRF(ISeg)%Units == "nm") Then
          WLL = WLG
       Else
          WLL = 1.0d0/WLG
       End If
       DWL(1) = WLL(2) - WLL(1)
       NN = Size(WLL)
       DO II = 2, NN - 1
          DWL(II) = WLL(II + 1) - WLL(II - 1)
       ENDDO
       DWL(NN) = WLL(NN) - WLL(NN - 1)
       SRF_WGT = SRF_WGT*DWL
    End If

    SRF_WGT = SRF_WGT/SUM(SRF_WGT)
     
    Return
  End Function SRF_WGT
!***************************************************************************
  Subroutine Read_SRF (Iseg, *)
    Character (Len = *), Parameter :: R_Name = 'Read_SRF'
    Character (Len = 4) Seg_Str
    Character (Len = :), Allocatable :: FName, WL_Str
    Integer  LU, ISeg
    Real (Knd) CWL
    Real (Knd), Allocatable :: Arr(:,:)
    !==================================================================

    FName = Set_Path(Str_Arr(2)%S)
    WL_Str = Cut_Str(Str_Arr(3)%S)
    Call Upper_Lower_Case (WL_Str,'U2L')
    SRF(Iseg)%Units = Cut_Str(Str_Arr(4)%S)
    Call Upper_Lower_Case (SRF(Iseg)%Units,'U2L')
    Call Read_File (2, FName, Arr, *880)
       
    If(Allocated(SRF(ISeg)%Wl)) DeAllocate(SRF(ISeg)%Wl, SRF(ISeg)%Wgt)
    Allocate(SRF(ISeg)%WL, Source = Arr(:,1))
    Allocate(SRF(ISeg)%Wgt, Source = Arr(:,2))
    Associate(WL => SRF(ISeg)%WL)
      If(WL_Str == 'auto') Then
         CWL = (MinVal(WL) + MaxVal(WL))/2.0D0
      Else
         Read(WL_Str,*,Err=101) CWL
         Write(Seg_Str,'(I0)') Iseg
         If(Abs(2.0*CWL - MinVal(WL) - MaxVal(WL)) > 1.0D-8) &
              Call Write_Message(R_Name, 'WARNING',  &
              'Specified central wavelength is not in the center of '&
              //'the spectral range where the slit function is defined '&
              //'(spectral segment '//TRIM(SEG_STR)//')')
      End If
      WL(:) = WL(:) - CWL
      If(MinVal(Abs(WL)) > 1.0D-5) GOTO 402
      If(MinVal(Wl)*MaxVal(Wl) > 0) GoTo 401

      Select Case (SRF(Iseg)%Units)
      case("nm")
         SRF(Iseg)%Delta(1) = Abs(MinVal(Wl)) 
         SRF(Iseg)%Delta(2) = MaxVal(Wl) 
      case("cm-1")
         SRF(Iseg)%Delta(1) = MaxVal(Wl) 
         SRF(Iseg)%Delta(2) = Abs(MinVal(Wl)) 
      End Select
    End Associate

    Return

401 CONTINUE
    CALL WRITE_MESSAGE(R_NAME, 'ERROR',&
         'Specified central wavelength is outside the spectral '&
         //'range where the slit function is defined '&
         //'(spectral segment '//TRIM(SEG_STR)//')')
    RETURN 1

402 CONTINUE
    CALL WRITE_MESSAGE(R_NAME, 'ERROR', &
         'Specified central wavelength must coincide with one of '&
         //'the wavelength grid points in the slit function file '&
         //'(spectral segment '//TRIM(SEG_STR)//')')
    RETURN 1


101 Allocate(Msg(2))
    Msg(1)%S = 'Reading error'
    Msg(2)%S = 'Parameter to read: '//STR_INP
    call ErrMsg(R_Name, 'e', Msg)
    Return 1
    
880 WRITE(EU, *) ' ---> Called by '//R_NAME
    RETURN 1
  END  SUBROUTINE READ_SRF
!***************************************************************************
End Module SRF_Mod
!***************************************************************************
