!****************************************************************************
!
!	SCIATRAN - Version 4.3 - Last modification: 09 Feb 2021
!
!	Copyright (C) - Institute of Remote Sensing,
!			University of Bremen, Germany
!
!****************************************************************************
SUBROUTINE ST_CONVOL(NSPEC_OUT, WL_OUT, NSEGS, SPECSEGS, *)
  ! Called by "rtm_main.f"

  USE AEROSOLS,    ONLY: N_AOT
  USE DATA_RET,    ONLY: RET_FLAG
  USE GRIDS,       ONLY: NLAYERS
  USE INFO,        ONLY: WF_DB
  USE INTERN_DIMS, ONLY: Nvs
  USE IRRAD,       ONLY: IRRAD_FLAG, SOL_FLUX, SOLSPEC_OUT, ACT_WL_FLAG
  USE LOG_PARS,    ONLY: EU, DIR_OUT
  USE RAMAN,       ONLY: N_WL_C
  USE RTM_OUTPUT,  ONLY: RTM_OUT_ARR, NL_OUT, N_LOS, N_RTM_ARR, NU_VEC_PARS, &
                         OWF_OUT_VEC
  Use SRF_Mod,     Only: SRF, SRF_WGT
  USE ST_VARS,     ONLY: CONVOL_FLAG, NSPEC_RTM, WL_RTM, &
                         LOW_RES_FLAG, SF_DELTA, LA_FLAG, OWF_FLAG
  USE SPHER_PARS,  ONLY: N_LOS_OUT
  USE TYPE_VARS,   ONLY: KND
  USE WATER,       ONLY: NLW => NLAYERS_UW

  INTEGER, INTENT(IN) :: NSPEC_OUT, NSEGS, SPECSEGS(2,NSEGS)
  REAL (KND) WL_OUT(NSPEC_OUT)
  ! Local variables
  CHARACTER (LEN = 9), PARAMETER :: R_NAME = 'ST_CONVOL'
  INTEGER  I, J, J1, K, L, Jv, KK, J_S, J_E, JSEG, MAX_POINTS, NN, NP
  INTEGER, DIMENSION (NSPEC_OUT) :: I_S, I_E, N_P
  LOGICAL  ONE_POINT(NSPEC_OUT), TMP_FLAG(NSPEC_RTM)
  REAL (KND), POINTER :: ARRAY(:)
  REAL (KND), ALLOCATABLE :: WEIGHTS(:,:), I_RRAD(:), WL_TMP(:), D_WL(:)
  !==================================================================
  
  CALL ST_TRACK (R_NAME, 1, 'I')
  
  ONE_POINT(:) = .FALSE.
  
  IF( IRRAD_FLAG ) THEN
     ALLOCATE(I_RRAD(NSPEC_RTM))
     I_RRAD(:) = PACK(SOL_FLUX(:),ACT_WL_FLAG(:))
  END IF
  
  DO JSEG = 1, NSEGS       
     J_S = SPECSEGS(1,JSEG) + 1
     J_E = SPECSEGS(1,JSEG) + SPECSEGS(2,JSEG)
     
     DO I = J_S, J_E
        IF( LOW_RES_FLAG(I) ) THEN
           ! Convolution for "Line absorber windows"
           KK = MINLOC(DABS(WL_RTM(:) - WL_OUT(I)),1,.NOT.LA_FLAG)
           I_S(I) = KK ;  I_E(I) = KK  ;  WL_OUT(I) = WL_RTM(KK)            
           ONE_POINT(I) = .TRUE.
           CYCLE
        END IF
        !************************************************************
        IF(.NOT.CONVOL_FLAG) THEN
           ! No convolution
           KK = MINLOC(DABS(WL_RTM(:) - WL_OUT(I)),1)
           I_S(I) = KK ;  I_E(I) = KK ;  ONE_POINT(I) = .TRUE.
           WL_OUT(I) = WL_RTM(KK)
           CYCLE
        END IF
        ! Standard convolution for all spectral points
        TMP_FLAG(:) = .FALSE.
        WHERE( WL_RTM(:) < SRF(JSEG)%SWB(WL_OUT(I)) - SF_DELTA) TMP_FLAG(:) = .TRUE.
        I_S(I) = COUNT(TMP_FLAG(:)) + 1
        
        TMP_FLAG(:) = .FALSE.
        WHERE(WL_RTM(:) <= SRF(JSEG)%LWB(WL_OUT(I)) + SF_DELTA) TMP_FLAG(:) = .TRUE.
        I_E(I) = COUNT(TMP_FLAG(:))
     END DO   ! over spectral points in segment (I)

     N_P(J_S:J_E) = I_E(J_S:J_E) - I_S(J_S:J_E) + 1

     IF( ANY(.NOT.ONE_POINT(J_S:J_E)) ) THEN
        IF(MAXVAL(N_P(J_S:J_E),.NOT.ONE_POINT(J_S:J_E)) /= &
             MINVAL(N_P(J_S:J_E),.NOT.ONE_POINT(J_S:J_E))) THEN
           CALL WRITE_MESSAGE(R_NAME, 'WARNING', &
                'The number of convolution points is supposed '//&
                'to be constant inside a spectral segment')
           WRITE(EU,*) ' *** To get rid of this message, set the full '&
                //'length of the convolution'
           WRITE(EU,*) ' ***  interval to a whole number of ' &
                //'internal wavelength steps, ' 
           WRITE(EU,*) ' *** i.e., 2*SF_HWHM*SF_CUTOFF = N*STEP'
        END IF
     END IF
  END DO   ! over spectral segments (JSEG)


  IF( MAXVAL(I_E(:)) > NSPEC_RTM.OR.MINVAL(I_S(:)) < 1 ) GOTO 201
  
  MAX_POINTS = MAXVAL(N_P(:))
  ALLOCATE( WEIGHTS(NSPEC_OUT,MAX_POINTS), WL_TMP(MAX_POINTS), D_WL(MAX_POINTS) )
      
  DO JSEG = 1, NSEGS       
     J_S = SPECSEGS(1,JSEG) + 1
     J_E = SPECSEGS(1,JSEG) + SPECSEGS(2,JSEG)
     DO I = J_S, J_E

        IF(ONE_POINT(I)) THEN
           ! No convolution
           WEIGHTS(I,1) = 1.d0
           CYCLE
        END IF
        
        WEIGHTS(I,1:N_P(I)) = SRF_WGT(WL_OUT(I),JSEG,I_S(I),I_E(I))
     END DO   ! over spectral points in segment (I)
  END DO   ! over spectral segments (JSEG)
         

  ! Convolution of radiation field 
  DO I = 1, N_AOT+1
     DO J = 1, NL_OUT
        DO L = 1, N_LOS
           DO K = 1, N_RTM_ARR
              ARRAY => RTM_OUT_ARR(:,I,J,L,K)
              CALL CONVOLUTION (IRRAD_FLAG)
           END DO
        END DO
     END DO
  ENDDO

  ! Weighting functions convolution (atmospheric parameters)
  DO K = 1, UBOUND(WF_DB,1)
     DO J = 1, N_LOS_OUT
        DO L = 1, UBOUND(WF_DB(K)%WF,3)
           DO Jv = 1, Nvs
              ARRAY => WF_DB(K)%WF(:,J,L,Jv)
              CALL CONVOLUTION (IRRAD_FLAG)
           ENDDO
        END DO
     END DO
  END DO

  ! Weighting functions convolution (oceanic parameters)
  IF( OWF_FLAG ) THEN
     DO J = 1, N_LOS_OUT
        DO L = 1, NLW
           DO K = 1, NU_VEC_PARS
              DO Jv = 1, Nvs
                 ARRAY => OWF_OUT_VEC(:,J,L,K,Jv)
                 CALL CONVOLUTION (IRRAD_FLAG)
              ENDDO
           END DO
        END DO
     END DO
  ENDIF

  ! Convolution of solar spectrum
  IF(IRRAD_FLAG) THEN
     IF(ALLOCATED(SOLSPEC_OUT)) DEALLOCATE(SOLSPEC_OUT)
     ALLOCATE(SOLSPEC_OUT(MAX(NSPEC_RTM,NSPEC_OUT)))
     SOLSPEC_OUT(:) = 1.d0
     ARRAY => SOLSPEC_OUT(:)
     CALL CONVOLUTION (IRRAD_FLAG)
  END IF
  
  CALL SET_POINTERS (NSPEC_OUT, *880)

  ! For the retrieval mode intensities and weighting functions need
  ! to be renormalized back to the unity solar flux
  ! Attention: weighting function for the ocean are currently excluded
  IF(RET_FLAG .AND. IRRAD_FLAG) THEN
     ! Intensities
     FORALL(I = 1:N_AOT+1, J = 1:NL_OUT, L = 1:N_LOS, &
          K = 1:N_RTM_ARR) RTM_OUT_ARR(:,I,J,L,K) = &
          RTM_OUT_ARR(:,I,J,L,K)/SOLSPEC_OUT(:)

     ! Weighting functions convolution (atmospheric parameters)
     DO K = 1, SIZE(WF_DB)
        FORALL(J = 1:N_LOS_OUT, L = 1:SIZE(WF_DB(K)%WF,3), &
             Jv = 1:Nvs) WF_DB(K)%WF(:,J,L,Jv) = &
             WF_DB(K)%WF(:,J,L,Jv)/SOLSPEC_OUT(:)
     END DO
  END IF
  
  CALL ST_TRACK (R_NAME, 1, 'O')
  RETURN
  
201 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Wrong indexes for convolution points')
  WRITE(EU,*) '    Start and end indexes: ', MINVAL(I_S(:)), MAXVAL(I_E(:))
  RETURN 1
  
880 WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)
  RETURN 1
  
CONTAINS
!****************************************************************
  SUBROUTINE CONVOLUTION (ABS_UNITS)
    INTEGER    I, I1, K1
    LOGICAL    ABS_UNITS
    REAL (KND) ARR_TMP(NSPEC_OUT)
    !==================================================================
    
    IF(ABS_UNITS) THEN
       FORALL(I = 1:NSPEC_OUT, .NOT.ONE_POINT(I)) &
            ARR_TMP(I) =  SUM( WEIGHTS(I,1:N_P(I))* &
            ARRAY(I_S(I):I_E(I))*I_RRAD(I_S(I):I_E(I)),&
            LA_FLAG(I_S(I):I_E(I)) )
       I = 1
       FORALL(I = 1:NSPEC_OUT, ONE_POINT(I)) &
            ARR_TMP(I) =  ARRAY(I_S(I))*I_RRAD(I_S(I))
       ARRAY(1:NSPEC_OUT) = ARR_TMP(1:NSPEC_OUT)
    ELSE

       FORALL(I = 1:NSPEC_OUT, .NOT.ONE_POINT(I)) &
            ARR_TMP(I) = SUM( WEIGHTS(I,1:N_P(I))* &
            ARRAY(I_S(I):I_E(I)), LA_FLAG(I_S(I):I_E(I)) )

       FORALL(I = 1:NSPEC_OUT, ONE_POINT(I)) ARR_TMP(I) = ARRAY(I_S(I))
       ARRAY(1:NSPEC_OUT) = ARR_TMP(1:NSPEC_OUT)
    END IF

      END SUBROUTINE CONVOLUTION
!***************************************************************************
      END
!***************************************************************************
