C********************************************************************
C
C       SCIATRAN - Version 4.6 - Last modification: 08 Apr 2022
C
C       Copyright (C) - Institute of Remote Sensing,
C                       University of Bremen, Germany
C *********************************************************************
       
C         Subroutine to calculate the Stokes vector ignoring
c             all scattering processes in the atmosphere
C         It is assumed that incident radiation is unpolarized.
C      ------------------------------------------------------- 
      SUBROUTINE ST_DIR_REFL(JPOS, POS, ALBEDO, E_W,
     $     LOA, SSB, SRR, SRR_Jact, PWF, TAC, Flu_int, *) 
c     ... Call in "dom_spherical.f" 

      USE ABSORBERS,   ONLY: N_GAS
      USE BRDF,        ONLY: BRDF_PARAM, DO_OC_COMP, Wgr_svk, Nwl_veg, 
     $                       WL_veg, Alb_svk, N_svk
      USE BRDF_SUBRS,  ONLY: BRDF_ACT_DOM
      USE GRIDS,       ONLY: NL => NLAYERS, DZR, DZ, SOL_GRD
      USE INFO,        ONLY: WF_DB
      USE INTERN_DIMS, ONLY: Nvs, Nm_vs
      USE LOG_PARS,    ONLY: ERROR_TXT, N_DSO, LOG_LEVEL
      USE RTM_OUTPUT,  ONLY: STOK_OUT
      USE SPHER_PARS,  ONLY: NSZA => N_SOL_ANG, N_OBS_ALT, 
     $                       NLO => N_LOS_OUT,  
     $                       JV_S, JV_E, N_AZ_VAR, NLSA => N_LOS_ANG, 
     $                       ANG_OUT, N_AZ_ANG, ANG_SEL_FLAG
      USE ST_VARS,     ONLY: BRDF_FLAG, SURF_TYPE, WF_FLAG, DO_PLANCK
      USE TOOLS,       ONLY: INQ_WF
      USE TYPE_VARS,   ONLY: KND, DCMP
      USE TYPES,       ONLY: LOA_TYPE, SSB_TYPE, RRS_TYPE, PWF_TYPE,
     $                       LWF_TYPE
      USE WATER,       ONLY: SS_PDF
c     ------------------------------------------------------------------

      CHARACTER (LEN = *), PARAMETER :: R_NAME = 'ST_DIR_REFL'
c     Subroutine arguments:
c     ---------------------
      INTEGER,    INTENT(IN) :: JPOS, SRR_Jact
      REAL (KND), INTENT(IN) :: POS, E_W, ALBEDO, Flu_int, TAC(NL,N_GAS)
c     ------------------------------------------------------
      TYPE (LOA_type) :: LOA    ! Local optical parameters of atmosphere
      TYPE (LWF_type) :: LWF    ! Local weighting functions
      TYPE (PWF_type) :: PWF    ! General weighting function 
      TYPE (RRS_type) :: SRR    ! RRS related parameters
      TYPE (SSB_type) :: SSB    ! Spherical solar beam
c     ------------------------------------------------------
c     ... Local variables & arrays
c     ----------------------------
      INTEGER  I, J, Js, Jv, Ja, KK
      REAL (KND)  ECV, EXS, Mus, Mls
      REAL (KND)  T_AU(NL), Ex_J(NLO), MuV(NLSA), Phi(N_AZ_ANG), 
     $     Tls(NLO,Nvs)
      REAL (KND), POINTER ::  Mue(:,:)
c     ==================================================================

      CALL ST_TRACK (R_NAME, 1, 'I')

      ERROR_TXT = 'Error in '//R_NAME

      Mue => SSB%Mue
      Phi(:) = ANG_OUT(3,1:N_AZ_ANG)

      SELECT CASE( ANG_SEL_FLAG )
      CASE( 'one' )
         MuV(:) = DABS( ANG_OUT(2,1:NLSA) )
      CASE( 'all' )
         MuV(:) = DABS( ANG_OUT(2,1:NLSA*N_AZ_ANG:N_AZ_ANG) )
      END SELECT

      T_AU(1) = 0.d0
      DO J = 1, NL-1
         T_AU(J+1) = T_AU(J) + (LOA%AL(J) + LOA%AL(J+1))*0.5d0*DZR(J)
      ENDDO
      
      Tls = 0.d0 
c     ... Calculate reflection of direct solar beam from surface
      IF( BRDF_FLAG ) THEN
c        ... BRDF reflection from surface
         CALL REFLECTED_DIR
      ELSE
c        ... Lambertian reflection from surface
         J = 0
         DO Js = 1,NSZA
            EXS = SSB%EXS(NL,Js)
            DO Jv = JV_S(Js),JV_E(Js) 
               DO Ja = 1,N_AZ_VAR  ;  J = J + 1
                  Tls(J,1) = ALBEDO*Sol_grd(Js)*EXS
c                  Tls(J,1) = ALBEDO*Mue(Js,NL-1)*EXS
              ENDDO
            ENDDO
         ENDDO
      ENDIF

c     ... Calculation of reflected radiation at the top of atmosphere
      J = 0
      DO Js = 1,NSZA
         DO Jv = JV_S(Js),JV_E(Js)  
            ECV = DEXP(-T_AU(NL)/Muv(Jv))
            DO Ja = 1,N_AZ_VAR  ;  J = J + 1
               Ex_j(J) = ECV
            ENDDO
         ENDDO
      ENDDO

      FORALL( I=1:Nvs ) STOK_OUT(SRR_Jact,1,1,:,I) = 
     $     STOK_OUT(SRR_Jact,1,1,:,I) + (Tls(:,I) + Flu_int)*Ex_j(:)*E_W

c     ...  ! scalar case only !
      IF( WF_FLAG ) THEN
         J = 0
         DO Js = 1,NSZA                ;  Mus = 1.d0/Mue(Js,NL-1)
            DO Jv = JV_S(Js),JV_E(Js)  ;  Mls = 1.d0/Muv(Jv)
               DO Ja = 1,N_AZ_VAR      ;  J = J + 1
                  PWF%WF_esp(:,J,1) = -Ex_j(J)*DZ(:)/2.d0
     $                *( Tls(J,1)*(Mus + Mls) + Flu_int*Mls )
               ENDDO
            ENDDO
         ENDDO

         LWF%WF_ext => PWF%WF_esp

c        ... WF for the gaseous absorber profiles
         IF( INQ_WF('tg', 'WFT') )
     $        CALL SET_OUTPUT_CWF(JPOS, LWF, DO_PLANCK, TAC, E_W)

c        ... WF for the temperature profile 
         IF(INQ_WF('temp', 'WFN')) CALL SET_OUTPUT_TWF(JPOS, LWF,  
     $        LOA%SCR, TAC, DO_PLANCK, PWF%T_WF_ABSTERM, E_W)

c        ... WF for fluorescence scaling factor
         IF(INQ_WF('flu_sfl', 'WFN', KK)) THEN
            IF( WF_DB(KK)%DIM == 'rel' ) THEN
               WF_DB(KK)%WF(JPOS,:,1,1) = WF_DB(KK)%WF(JPOS,:,1,1) 
     $              + Flu_int*Ex_j(:)*E_W
            ELSE
               WF_DB(KK)%WF(JPOS,:,1,1) = WF_DB(KK)%WF(JPOS,:,1,1) 
     $              + Ex_j(:)*E_W
            ENDIF
         ENDIF

c        ... WF for Lambertian albedo profile
         IF(INQ_WF('albedo', 'WFN',KK)) THEN
            SELECT CASE ( WF_DB(KK)%DIM )
            CASE( 'rel' )
               WF_DB(KK)%WF(JPOS,:,1,1) = WF_DB(KK)%WF(JPOS,:,1,1) + 
     $              Tls(:,1)*Ex_j(:)*E_W
            CASE( 'abs' )
               J = 0
               DO Js = 1,NSZA  ;  EXS = SSB%EXS(NL,Js)
                  DO Jv = JV_S(Js),JV_E(Js) 
                     DO Ja = 1,N_AZ_VAR  ;  J = J + 1
                        Tls(J,1) = Sol_grd(Js)*EXS
c                        Tls(J,1) = Mue(Js,NL-1)*EXS
                     ENDDO
                  ENDDO
               ENDDO
               WF_DB(KK)%WF(JPOS,:,1,1) = WF_DB(KK)%WF(JPOS,:,1,1) + 
     $              Tls(:,1)*Ex_j(:)*E_W
            END SELECT
         ENDIF
      ENDIF

      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN

      CONTAINS
c **********************************************************************

      SUBROUTINE REFLECTED_DIR
      
      USE BRDF, ONLY:  Ker_cof => IVG_cof, Nwl_BRF
c     ------------------------------------------------------------------

      CHARACTER (LEN=20) TXT
      INTEGER   J, Ws, Ws1, IN1
      REAL (KND)  WL1(1), Area, DC, Uc, Aveg,
     $     BRF_OUT(Nvs*NLSA,Nvs*NSZA,N_AZ_ANG,1) 
      REAL (KND), ALLOCATABLE:: R1_OUT(:,:,:,:)
c     ==================================================================

      WL1(1) = POS ; Area = 0.d0 ; BRF_OUT = 0.d0

      IF( .NOT.ANY(DO_OC_COMP) ) THEN

         IF(LOG_LEVEL(1) >= N_DSO) 
     $        WRITE(*,*) ' ====> Call BRDF_ACT_DOM in SS_STOKES'

         SELECT CASE( TRIM(SURF_TYPE) )
         CASE('Ross-Li','Ross-Li-snow','snow_fasmar')
c        ... Calculation should be performed for two or three kernels
            ALLOCATE( R1_OUT(Nvs*NLSA,Nvs*NSZA,N_AZ_ANG,Nwl_BRF) )
            CALL BRDF_ACT_DOM( BRDF_PARAM, NLSA, MuV, N_AZ_ANG, Phi,
     $           NSZA, Sol_grd, 1, WL1, R1_OUT, 'none' )
c     $           NSZA, Mue(:,NL-1), 1, WL1, R1_OUT, 'none' )
            CALL LIN_SEARCH_NEW(POS, DC, Ws, Ws1, Nwl_veg, WL_veg)
            Uc = (1.d0 - Dc)
            BRF_OUT(1:Nvs*NLSA:Nvs,1:Nvs*NSZA:Nvs,:,1) = 
     $           (Uc*Ker_cof(1,Ws) + Dc*Ker_cof(1,Ws1))
            DO J = 1,Nwl_BRF
               BRF_OUT(:,:,:,1) = BRF_OUT(:,:,:,1) + 
     $              (Uc*Ker_cof(J+1,Ws) + Dc*Ker_cof(J+1,Ws1))
     $              *R1_OUT(:,:,:,J)
            ENDDO
         CASE('svk')
            ALLOCATE( R1_OUT(Nvs*NLSA,Nvs*NSZA,N_AZ_ANG,1) )
            IF( POS >= Wgr_svk ) THEN
               TXT = 'second' ; IN1 = 2
            ELSE
               TXT = 'first'  ; IN1 = 1
            ENDIF
            CALL BRDF_ACT_DOM( BRDF_PARAM, NLSA, MuV, N_AZ_ANG, Phi,
     $           NSZA, Sol_grd, 1, WL1, R1_OUT, TXT )
c     $           NSZA, Mue(:,NL-1), 1, WL1, R1_OUT, TXT )
            CALL LIN_SEARCH_NEW(POS, DC, Ws, Ws1, Nwl_veg, WL_veg)
            Aveg = (1.d0 - DC)*Alb_svk(Ws) + DC*Alb_svk(Ws1)
            BRF_OUT = Aveg*N_svk(IN1)*R1_OUT 

         CASE('mRPV_facet','snow_artm','white_si')
            ALLOCATE( R1_OUT(Nvs*NLSA,Nvs*NSZA,N_AZ_ANG,1) )
            CALL BRDF_ACT_DOM( BRDF_PARAM, NLSA, MuV, N_AZ_ANG, Phi,
     $           NSZA, Sol_grd, 1, WL1, R1_OUT, 'Gauss' )
c     $           NSZA, Mue(:,NL-1), 1, WL1, R1_OUT, 'Gauss' )
            BRF_OUT = R1_OUT 

         CASE('melt_pond')
            ALLOCATE( R1_OUT(Nvs*NLSA,Nvs*NSZA,N_AZ_ANG,1) )
c            CALL POND_BRF( NLSA, MuV, N_AZ_ANG, Phi, NSZA, Mue(:,NL-1),
            CALL POND_BRF( NLSA, MuV, N_AZ_ANG, Phi, NSZA, Sol_grd,
     $           1, WL1, R1_OUT )
            BRF_OUT = R1_OUT
            

         CASE DEFAULT
            ALLOCATE( R1_OUT(Nvs*NLSA,Nvs*NSZA,N_AZ_ANG,1) )
            CALL BRDF_SETUP
            CALL BRDF_ACT_DOM( BRDF_PARAM, NLSA, MuV, N_AZ_ANG, Phi,
     $           NSZA, Sol_grd, 1, WL1, R1_OUT, 'Gauss' )
c     $           NSZA, Mue(:,NL-1), 1, WL1, R1_OUT, 'Gauss' )
            BRF_OUT = R1_OUT 
         END SELECT
      ELSE
         ALLOCATE( R1_OUT(Nvs*NLSA,Nvs*NSZA,N_AZ_ANG,1) )
      ENDIF

      IF( DO_OC_COMP(1) ) THEN  ! Ocean reflection ==> Whitecaps
         IF(LOG_LEVEL(1) >= N_DSO) 
     $        WRITE(*,*) '******* CALL "USER_WHITECAP" in SS_STOKES'
         CALL USER_WHITECAP( Nvs, NLSA, MuV, N_AZ_ANG, Phi, 
     $        NSZA, Sol_grd, 1, WL1, R1_OUT, Area )
c     $        NSZA, Mue(:,NL-1), 1, WL1, R1_OUT, Area )
         BRF_OUT = BRF_OUT + R1_OUT
      ENDIF
      IF( DO_OC_COMP(2) ) THEN  ! Ocean reflection ==> Glint
         CALL BRDF_SETUP
         IF(LOG_LEVEL(1) >= N_DSO) 
     $        WRITE(*,*) ' ====> Call BRDF_ACT_DOM in SS_STOKES'
c         WRITE(*,*) ' ====> Call BRDF_ACT_DOM in SS_STOKES'
         CALL BRDF_ACT_DOM( BRDF_PARAM, NLSA, MuV, N_AZ_ANG, Phi,
     $        NSZA, Sol_grd, 1, WL1, R1_OUT, SS_PDF )
c     $        NSZA, Mue(:,NL-1), 1, WL1, R1_OUT, SS_PDF )
         BRF_OUT = BRF_OUT + (1.d0 - Area)*R1_OUT 
      ENDIF
      IF( DO_OC_COMP(3) ) THEN  ! Ocean reflection ==> Water leaving
         IF(LOG_LEVEL(1) >= N_DSO) 
     $        WRITE(*,*) '******* CALL "USER_WATER" in SS_STOKES'
         CALL USER_WATER( Nvs, NLSA, MuV, N_AZ_ANG, Phi, 
     $        NSZA, Sol_grd, 1, WL1, R1_OUT )
c     $        NSZA, Mue(:,NL-1), 1, WL1, R1_OUT )
         BRF_OUT = BRF_OUT + (1.d0 - Area)*R1_OUT
      ENDIF

      SELECT CASE( ANG_SEL_FLAG )
      CASE( 'one' )
         DO Js = 1,NSZA
            EXS = SSB%EXS(NL,Js)
            DO I = 1,Nvs
c               Tls(Js,I) = Mue(Js,NL-1)*EXS*
               Tls(Js,I) = Sol_grd(Js)*EXS*
     $              BRF_OUT((Js-1)*Nvs+I,(Js-1)*Nvs+1,Js,1)
            ENDDO
         ENDDO
      CASE( 'all' )
         J = 0
         DO Js = 1,NSZA
            EXS = SSB%EXS(NL,Js)
            DO Jv = JV_S(Js),JV_E(Js) 
               DO Ja = 1,N_AZ_VAR  ;  J = J + 1
                  DO I = 1,Nvs
c                     Tls(J,I) = Mue(Js,NL-1)*EXS*
                     Tls(J,I) = Sol_grd(Js)*EXS*
     $                    BRF_OUT((Jv-1)*Nvs+I,(Js-1)*Nvs+1,Ja,1)
                  ENDDO
               ENDDO
            ENDDO
         ENDDO
      END SELECT
      END SUBROUTINE REFLECTED_DIR

      END    
C********************************************************************C
