C********************************************************************C
C
C       SCIATRAN - Version 3.0 - Last modification: 23 Mar 2009
C
C       Copyright (C) - Institute of Remote Sensing,
C                       University of Bremen, Germany
C
C********************************************************************C
C
C ORIGINAL DESCRIPTION: 
C
! The ECSTRA model is a climatological model of the extinction coefficient
! of stratospheric aerosols in the UV-visible range based on SAGE II data.
! The model is a function of wavelength, latitude, relative altitude with
! respect to the tropopause and a volcanism parameter defined as
! V=ln(1+deltamean/10.E-4), where deltamean is the optical thickness 
! averaged over the whole latitude range and over a 2 months time window.
! 
! The model has been calculated by means of the aerosol extinction profiles 
! recorded between Oct 1984 and Dec 1995 by the SAGE II experiment.
!       
! The ECSTRA model is described in 'A volcanism dependent model for the
! extinction profile of stratospheric aerosols in the UV-visible range',
! by Didier Fussen and Christine Bingen, GRL, 26:703-706, 1999. The
! performances of the model are further discussed in 'Structure and
! spectral features of the stratospheric aerosol extinction profiles in the
! UV-visible range derived from SAGE-II data', by Christine Bingen and
! Didier Fussen, JGR, 105: 4767-4776 (2000).
!
! NOTE: Use of this model is on the pre-condition that the authors are
!       properly cited and that they are notified of any publication
!       resulting from its use.
!
! Any comments or questions on the ECSTRA model are welcome and can be
! sent to the authors at :
!
!  E-mail:      Christine.Bingen@oma.be, Didier.Fussen@oma.be
!  address:     IASB-BIRA
!               3, Avenue Circulaire
!               B-1180 Brussels
!               Belgium
!  Tel:         +32-2-3730410, +32-2-3730371
!  Fax:         +32-2-3748423
!
! ***********************************************************************
!                               version 1.0 /  april 16th 1997
!                               version 1.1 /  april 23th 1997
!                               version 3.0 /  april 30th 1998
!                               version 3.1 /  july 8th 1998
! -> introduction secutity on volcanism: must be > vrayleigh
!                               version 3.2 / september 22th 1998
!                               Christine BINGEN
!                               IASB-BIRA
!                               Belgian Institute for Space Aeronomy
!                               3, avenue Circulaire
!                               B-1180 Brussels (BELGIUM)
! ***********************************************************************
! VARIABLE DESCRIPTION :
!
!   INPUT :
!       DELTAMEAN (real) :
!               mean optical thickness over the whole latitude range
!               dimensionless
!               range : ~1.5E-3 -> ~1.E-1 or more;
!                       the value of deltamean is necessary >= its
!                       Rayleigh contribution which is ~ 2.5E-4 
!
!               Rem: typical ranges are :
!                    [~1.5E-3 -> ~3.E-3] : low volcanism
!                                 (typ. before Pinatubo eruption)
!                    [~ 3.E-3 -> ~1.E-2] : moderate volcanism
!                                 (typ. around Ruiz eruption)
!                    [~ 1.E-2 -> ~1.E-1] : high volcanism
!                                 (eruption decay just after Pinatubo)
!       Z_ECSTRA(NL_ECSTRA) (real array) :
!               vector of profile reduced altitudes in km
!               (origine taken at z_tropopause+2km)
!               range : [0.0 km -> 50.0 km]
!       NL_ECSTRA (integer) :
!               size of vector Z; 1 <= nz <= nzmax
!       WAVLEN(NWAV) (real array) :
!               vector of profile wavelengthes in microns
!               range : [~0.3 microns -> ~1.0 microns]
!       NWAV (integer) :
!               size of vector WAVLEN; 1 <= nwav <= nwavmax
!   OUTPUT :
!       BETA(NZMAX) (real array) :
!               vertical/spectral extinction profile
!               typical range : [~1e-7 -> ~1e-1]
!----------------------------------------------------------------
      SUBROUTINE ECSTRA_INIT (WL_START, WL_END, *)
c     ... Call in "rtm_init.f"

      USE AEROSOLS, ONLY: ECSTRA_DM, Z_ECSTRA, ZTROPO, NL_ECSTRA, 
     $                    BETA_VERT, NCOEFF_ECSTRA, ACOEFF_ECSTRA
      USE GRIDS,    ONLY: ZZZ, NLAYERS
      USE LOG_PARS, ONLY: EU
      USE ST_VARS,  ONLY: LATITUDE, PIE
c     ------------------------------------------------------------------

      CHARACTER (LEN=*), PARAMETER :: R_NAME = 'ECSTRA_INIT'
      INTEGER             ILEG
      INTEGER, PARAMETER :: NLEG = 3  
      DOUBLE PRECISION    VRAYCOEFF(NLEG), PLEG(NLEG), VRAYLEIGH
      DOUBLE PRECISION    COLAT, POLLEG, VOLC, VOLCMIN, VOLCMAX
      DOUBLE PRECISION    WL_START, WL_END
      DOUBLE PRECISION, PARAMETER :: DELTA0 = 1.0D-4, DELTAMIN = 1.5D-3, 
     $     DELTAMAX = 1.0D-1
      DATA VRAYCOEFF / .129727E+01, -.129960E+00,  .238904E-01/
      EXTERNAL POLLEG

      CALL ST_TRACK (R_NAME, 1, 'I')

      IF(WL_END > 1.10D+3) GOTO 202
      IF(WL_START < 0.3D+3.OR.WL_END > 1.020D+3) THEN
         CALL WRITE_MESSAGE(R_NAME, 'WARNING', 'A spectral '
     $        //'extrapolation is performed by the ECSTRA aerosol'
     $        //' model')
      END IF
      IF (ECSTRA_DM.GE.1.E+17) GOTO 201

      VOLC = DLOG(1 + ECSTRA_DM/DELTA0)
      VOLCMIN = DLOG(1 + DELTAMIN/DELTA0) 
      VOLCMAX = DLOG(1 + DELTAMAX/DELTA0) 

      CALL GET_TPHGT(ZTROPO)

      NL_ECSTRA = COUNT(ZZZ(:) > ZTROPO)
      IF(ALLOCATED(Z_ECSTRA)) 
     $     DEALLOCATE(Z_ECSTRA, BETA_VERT, ACOEFF_ECSTRA)
      ALLOCATE(Z_ECSTRA(NL_ECSTRA), BETA_VERT(NL_ECSTRA), ! A
     $     ACOEFF_ECSTRA(NCOEFF_ECSTRA,NL_ECSTRA)) ! A

      Z_ECSTRA(:) = ZZZ(1:NL_ECSTRA) - (ZTROPO + 2.0D0)

      COLAT = PIE*(90.D0 - LATITUDE)/180.D0
      DO ILEG = 1, NLEG
         PLEG(ILEG) = POLLEG(ILEG, COLAT)
      END DO

C     Check minimum value for volcanism
C     ---------------------------------
      VRAYLEIGH = DOT_PRODUCT(VRAYCOEFF(:), PLEG(:))
      IF (VOLC < VRAYLEIGH) GOTO 203

      CALL ECSTRA_VERT (VOLC, DELTA0, PLEG, *880)
      CALL ECTRA_SPEC_INIT (VOLC, VOLCMIN, VOLCMAX, PLEG)

      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN

 201  CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Invalid '
     $     // 'volcanism parameter')
      RETURN 1

 202  CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'ECSTRA aerosol'
     $     // 'model should nod be used at wavelenghts > 1100 nm')
      RETURN 1

 203  CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Unable to obtain the'
     $     // 'spectral dependance of the aerosol extinction '//
     $     'coefficient from the ECSTRA model')
      RETURN 1

 880  WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)
      RETURN 1

      END

C**************************************************************************
      SUBROUTINE ECSTRA_VERT (VOLC, DELTA0, PLEG, *)

!       This routine computes the vertical extinction profile at
!       lambda=1.020nm for the entered altitude and latitude sets.
!       The extinction profile is analytically calculated from the
!       fit function :
!
!       beta(lambda=1.020nm, altitude, latitude) 
!               =  exp(x1+x2*z) + exp(x3-((z-x4)/x5)**2)
!
!       where x1, ..., x9 are described by adequate functions of
!       the latitude and the mean optical thickness.
!----------------------------------------------------------------
      USE AEROSOLS,  ONLY: NL_ECSTRA, Z_ECSTRA, BETA => BETA_VERT
      USE ST_VARS, ONLY : PIE

      CHARACTER (LEN=*), PARAMETER :: R_NAME = 'ECSTRA_VERT'

      INTEGER              IZ, IFITC
      INTEGER, PARAMETER :: NFITCOEFF = 11, NCOEFF = 9, NLEG = 3 

      DOUBLE PRECISION     FITCOEFF(NLEG, NFITCOEFF), PLEG(NLEG)
      DOUBLE PRECISION     FITC(NFITCOEFF), X(NCOEFF), ARGLOG(5)
      DOUBLE PRECISION     VMIN, FITINV, FITRATIO, DELTA0, VOLC, ERF

      DATA FITCOEFF / .129671E+01, -.609814E-01,  .103020E+00,
     &                .937752E+01, -.134148E+01,  .220127E+01,
     &               -.120237E+02,  .469101E+01,  .196397E+01,
     &               -.256263E+01,  .220248E+01, -.762632E+00,
     &               -.330161E+00,  .191957E+00, -.214081E+00,
     &               -.994609E-02,  .702232E-02, -.115140E-01,
     &                .803981E+00,  .229471E+00, -.353349E+00,
     &               -.229158E+00, -.675369E-01,  .867668E-02,
     &                .440249E-01,  .122706E-01,  .298406E-02,
     &                .149391E+00, -.856363E-02, -.248412E-01,
     &                .222069E-02, -.129227E-03,  .182297E-02 /

      EXTERNAL ERF

      VMIN = DOT_PRODUCT(FITCOEFF(:, 1),PLEG(:))

C     Test on optical thickness range
C     -------------------------------
      IF (VOLC < VMIN) GOTO 202
 
C     DETERMINATION OF THE FIT FUNCTIONS
C     ----------------------------------
      DO IFITC = 2, NFITCOEFF
         FITC(IFITC) = DOT_PRODUCT(FITCOEFF(:,IFITC),PLEG(:))
      END DO

      FITRATIO = FITC(7) + FITC(8)*(VOLC - VMIN)**2 + 
     $     FITC(9)*(VOLC - VMIN)**3
      FITINV = FITC(10) + FITC(11)*(VOLC - VMIN)**2

C     DETERMINATION OF XI=X(ICOEFF)
C     -----------------------------
      IF(FITINV <= 1.E-36.OR.FITC(2) <= 1.E-36) GOTO 202

      X(2) = -1.0D0/FITC(2)
      X(5) = 1.0D0/FITINV
      X(4) = FITRATIO*X(5)

      ARGLOG(1) = EXP(VOLC) - EXP(VMIN)
      ARGLOG(2) = DELTA0*2/SQRT(PIE)
      ARGLOG(3) = FITINV
      ARGLOG(4) = 1 + ERF(FITRATIO)
      ARGLOG(5) = -X(2)*DELTA0*(EXP(VMIN) - 1.)

      IF (ANY(ARGLOG(:) <= 1.E-36)) GOTO 202
      ARGLOG = DLOG(ARGLOG)
      
      X(1) = ARGLOG(5)
      X(3) = SUM(ARGLOG(1:3)) - ARGLOG(4)

      FORALL(IZ = 1:NL_ECSTRA) BETA(IZ) = EXP(X(1) + X(2)*Z_ECSTRA(IZ))
     $     + EXP(X(3) - ((Z_ECSTRA(IZ) - X(4))/X(5))**2)

c      X(6:9) = FITC(3:6)
c      WHERE(Z_ECSTRA(:) < 0.) BETA(:) = BETA(:) + 
c     $     EXP(X(6) + X(7)*Z_ECSTRA(:) + X(8)*Z_ECSTRA(:)**2 + 
c     $     X(9)*Z_ECSTRA(:)**3)

      RETURN

 202  CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Unable to obtain the'
     $     // 'vertical distribution of the aerosol extinction '//
     $     'coefficient from the ECSTRA model')
      RETURN 1

      END

C**************************************************************************
      SUBROUTINE ECTRA_SPEC_INIT (VOLC, VOLCMIN, VOLCMAX, PLEG)
!----------------------------------------------------------------------
!       Initialization of the spectral fit coefficients used in 
!       ECSTRA_SPEC subroutine
!----------------------------------------------------------------------
      USE AEROSOLS, ONLY: ZRED => Z_ECSTRA, NCOEFF => NCOEFF_ECSTRA, 
     $     ACOEFF => ACOEFF_ECSTRA, NL_ECSTRA

      CHARACTER (LEN=*), PARAMETER :: R_NAME = 'ECTRA_SPEC_INIT'
      INTEGER              IZ, NPTS, ICOEFF, IPTS, IFCTL, IFCTV, IFCTZ
      INTEGER, PARAMETER :: NFCTV = 3, NFCTZ = 4, NFCTL = 3

      DOUBLE PRECISION     ACOEFFIT(4), ACMIN(NCOEFF), ACMAX(NCOEFF)
      DOUBLE PRECISION     SPECTRFCT(NFCTZ, NFCTV, NFCTL, NCOEFF)
      DOUBLE PRECISION     FCTALT(4, NFCTZ), FCTVOLC(NFCTV)
      DOUBLE PRECISION     VOLCFIT(4), ZREDFIT(4), PLEG(NFCTL)
      DOUBLE PRECISION     VOLC, VOLCMIN, VOLCMAX, VOLCRATIO, ZREDRATIO
      DOUBLE PRECISION, PARAMETER :: ZREDMIN = 0.0D0, ZREDMAX = 20.0D0, 
     $     VOLCINCR = 0.01D0, ZREDINCR = 0.1D0

      DATA ACMIN     / -15., -7., -9. /
      DATA ACMAX     /   0.,  5.,  6. /
      DATA SPECTRFCT / 
     &   -.163864E+02 , .145911E+01 ,-.560585E-01 ,-.947258E-03 
     &  , .309712E+01 ,-.575902E+00 , .585504E-02 , .988860E-03 
     &  ,-.198146E+00 , .599068E-01 ,-.104100E-02 ,-.100105E-03 
     &  ,-.175729E+01 , .145077E+01 ,-.209425E+00 , .837081E-02 
     &  , .860714E+00 ,-.671152E+00 , .101509E+00 ,-.410844E-02 
     &  ,-.933807E-01 , .731506E-01 ,-.116890E-01 , .484972E-03 
     &  , .121401E+01 ,-.904276E+00 , .174133E+00 ,-.735668E-02 
     &  ,-.572574E+00 , .427309E+00 ,-.794334E-01 , .333495E-02 
     &  , .765811E-01 ,-.500318E-01 , .923801E-02 ,-.388923E-03 
     &  ,-.123077E+02 , .454431E+01 ,-.499497E+00 , .150635E-01 
     &  , .406562E+01 ,-.217922E+01 , .239394E+00 ,-.717201E-02 
     &  ,-.354643E+00 , .227687E+00 ,-.257313E-01 , .773485E-03 
     &  ,-.913366E+01 , .793075E+00 , .877072E-01 ,-.645608E-02 
     &  , .376365E+01 ,-.282042E+00 ,-.406565E-01 , .295053E-02 
     &  ,-.404704E+00 , .302832E-01 , .429118E-02 ,-.312167E-03 
     &  ,-.393186E+00 , .300998E+00 ,-.421858E-01 , .139071E-02 
     &  , .794096E-01 ,-.168140E+00 , .224465E-01 ,-.692275E-03 
     &  , .431211E-01 , .337068E-02 ,-.115206E-02 , .439995E-04 
     &  ,-.495586E+01 , .424411E+01 ,-.521985E+00 , .173384E-01 
     &  , .157374E+01 ,-.214916E+01 , .272684E+00 ,-.898750E-02 
     &  ,-.156689E+00 , .222205E+00 ,-.288772E-01 , .959340E-03 
     &  ,-.154460E+02 , .641306E+00 , .257765E+00 ,-.156254E-01 
     &  , .657520E+01 ,-.130901E+00 ,-.127941E+00 , .759604E-02 
     &  ,-.697566E+00 , .568709E-02 , .147478E-01 ,-.850682E-03 
     &  ,-.485792E+00 , .691055E-01 ,-.815791E-01 , .481410E-02 
     &  ,-.115263E+00 ,-.958257E-01 , .452021E-01 ,-.242390E-02 
     &  , .888769E-01 ,-.795187E-02 ,-.347795E-02 , .234072E-03/

      DO IZ = 1, NL_ECSTRA

C     FOR 1 CHOICE OF IZ, SELECTION OF THE PROCESSING FOR A0, A1, A2
C     (FIT, BILINEAR EXTRAPOLATION, NO PROCESSING)
C     ---------------------------------------------------------------
         IF ((VOLC >= VOLCMIN).AND.VOLC <= VOLCMAX) THEN
            IF (ZRED(IZ) >= ZREDMIN.AND.ZRED(IZ) <= ZREDMAX) THEN
C     fit valid on the current point
C     ------------------------------
               NPTS = 1 ;  VOLCFIT(1) = VOLC ;  ZREDFIT(1) = ZRED(IZ)

            ELSE IF (ZRED(IZ) > ZREDMAX) THEN
C     fit with the fixed value zredmax of zred
C     ------------------------------------------------
               NPTS = 1 ;  VOLCFIT(1) = VOLC ;  ZREDFIT(1) = ZREDMAX
               
            ELSE                ! (ZRED(IZ) < ZREDMAX)
C     fit not valid on the current point; 4 points bilinear extrapolation
C     -------------------------------------------------------------------
               NPTS = 4
               VOLCFIT(:) = (/VOLC - VOLCINCR, VOLC + VOLCINCR,
     $              VOLC - VOLCINCR, VOLC + VOLCINCR/)
               ZREDFIT(:) = (/ZREDMIN, ZREDMIN + ZREDINCR, 
     $              ZREDMIN + ZREDINCR, ZREDMIN/)
            END IF              ! (ZRED)
            
         ELSE
            
            NPTS = 4
            IF (VOLC > VOLCMAX) THEN
               VOLCFIT(:) = (/VOLCMAX, VOLCMAX - VOLCINCR, VOLCMAX, 
     $              VOLCMAX - VOLCINCR/)
            ELSE
               VOLCFIT(:) = (/VOLCMIN, VOLCMIN + VOLCINCR, VOLCMIN,
     $              VOLCMIN + VOLCINCR/)
            END IF
            
            IF (ZRED(IZ) >= ZREDMIN.AND.ZRED(IZ) <= ZREDMAX) THEN
               ZREDFIT(:) = (/ZRED(IZ) + ZREDINCR, ZRED(IZ) - ZREDINCR
     $              , ZRED(IZ) - ZREDINCR, ZRED(IZ) + ZREDINCR/)
            ELSE IF (ZRED(IZ) < ZREDMIN) THEN
               ZREDFIT(:)= (/ZREDMIN, ZREDMIN + ZREDINCR, 
     $              ZREDMIN + ZREDINCR, ZREDMIN/)
            ELSE
C     fit not valid on the current point; 
C     linear extrapolation in volcanism; zred(iz) = zredmax
C     ------------------------------------------------------------
               ZREDFIT(:) = ZREDMAX
            END IF
         END IF
     
C     correction for values exceeding the limit with less than
C     the increment value
C     --------------------------------------------------------
         WHERE(ZREDFIT(:) > ZREDMAX) ZREDFIT(:) = ZREDMAX
         WHERE(ZREDFIT(:) < ZREDMIN)  ZREDFIT(:) = ZREDMIN
         WHERE(VOLCFIT(:) > VOLCMAX)  VOLCFIT(:) = VOLCMAX
         WHERE(VOLCFIT(:) < VOLCMIN)  VOLCFIT(:) = VOLCMIN
              
         DO ICOEFF = 1, NCOEFF
            
C     FOR THE CURRENT CHOICE OF IWAV, IZ, FIT OF AI(ICOEFF)
C     -------------------------------------------------------
            DO IPTS = 1, NPTS
               
C     Calculation altitude related fit coefficients
C     ---------------------------------------------
               FCTALT(IPTS, 1) = 1.0D0
               FORALL(IFCTZ = 2:NFCTZ)
     $              FCTALT(IPTS, IFCTZ)= ZREDFIT(IPTS)**(IFCTZ - 1)
               
C     Calculation volcanism related fit coefficients
C     ----------------------------------------------
               FCTVOLC(1) = 1.0D0
               FORALL(IFCTV = 2:NFCTV) 
     $              FCTVOLC(IFCTV) = VOLCFIT(IPTS)**(IFCTV - 1)
               
C     Calculation spectral fit coefficients
C     -------------------------------------
               ACOEFFIT(IPTS) = 0.0D0
               DO IFCTL=1, NFCTL
                  DO IFCTV=1, NFCTV
                     ACOEFFIT(IPTS) = ACOEFFIT(IPTS) + SUM(
     &                    SPECTRFCT(:, IFCTV, IFCTL, ICOEFF)*
     $                    FCTALT(IPTS,:))*FCTVOLC(IFCTV)*PLEG(IFCTL)
                  END DO 
               END DO
               
            END DO
            
C     DEDUCTION OF ACOEFF(ICOEFF) FOR THE CURRENT CHOICE OF IWAV, IZ
C     ----------------------------------------------------------------
            IF (NPTS == 1) THEN
C     direct use of the fit result
C     ----------------------------
               ACOEFF(ICOEFF, IZ) = ACOEFFIT(1)
               
            ELSE                ! (npts == 4)
!     
!     * * bilinear approximation
!     ----------------------
               VOLCRATIO = (VOLC - VOLCFIT(2))/(VOLCFIT(3) - VOLCFIT(2))
               IF (ZREDFIT(2) /= ZREDFIT(4)) THEN
                  ZREDRATIO = (ZRED(IZ) - ZREDFIT(2))/
     $                 (ZREDFIT(4) - ZREDFIT(2))
               ELSE
                  ZREDRATIO = 0.0D0
               END IF
               ACOEFF(ICOEFF, IZ) = VOLCRATIO*ZREDRATIO*ACOEFFIT(1) +
     &              (1 - VOLCRATIO)*(1 - ZREDRATIO)*ACOEFFIT(2) +
     &              VOLCRATIO*(1 - ZREDRATIO)*ACOEFFIT(3) +
     &              ZREDRATIO*(1 - VOLCRATIO)*ACOEFFIT(4)
               
            END IF
            
C     Ultimate security on acoeff(icoeff)
C     -----------------------------------
            ACOEFF(ICOEFF, IZ) = 
     &           MAX(ACOEFF(ICOEFF,IZ), ACMIN(ICOEFF))
            ACOEFF(ICOEFF, IZ)= 
     &           MIN(ACOEFF(ICOEFF,IZ), ACMAX(ICOEFF))
         END DO

      END DO
               
      RETURN
      END

C**************************************************************************
      SUBROUTINE ECSTRA_SPEC(WL_INP, BETA, *)

!       This routine computes the spectral extinction profile for 
!       the entered latitude, altitude and wavelength sets.
!       The extinction profile is analytically calculated from an adequate
!       development of ln(beta) in a power serie up to order 2.
!
!----------------------------------------------------------------------
      USE AEROSOLS, ONLY: NL_ECSTRA, ACOEFF => ACOEFF_ECSTRA
      USE ST_VARS, ONLY : PIE

      CHARACTER (LEN=*), PARAMETER :: R_NAME = 'ECTRA_SPEC'
      INTEGER              IZ, NPTSWL, IPTSWL
      DOUBLE PRECISION     BETA(NL_ECSTRA), BETAFIT(NL_ECSTRA, 2)
      DOUBLE PRECISION     WAVFIT(2), WL_INP, WAVLEN, WAVLEN0, WAVRATIO
      DOUBLE PRECISION, PARAMETER :: WLINCR=0.05D0, WAVLMIN = 0.3D0, 
     $     WAVLMAX = 1.02D0

      DATA WAVLEN0   /1.020/

      WAVLEN = WL_INP*1.0D-3

C     SELECTION OF THE PROCESSING FOR BETA
C     ------------------------------------
      IF(WAVLEN >= WAVLMIN.AND.
     $     WAVLEN <= WAVLMAX) THEN ! fit valid on current point
         NPTSWL = 1
         WAVFIT(1) = WAVLEN
            
      ELSE ! fit not valid on the current point, linear extrapolation
         NPTSWL = 2
         IF(WAVLEN < WAVLMIN) THEN
            WAVFIT(1) = WAVLMIN
            WAVFIT(2) = WAVLMIN + WLINCR
         ELSE
            WAVFIT(1) = WAVLMAX
            WAVFIT(2) = WAVLMAX - WLINCR
         END IF
         WAVRATIO = (WAVLEN - WAVFIT(2))/(WAVFIT(1) - WAVFIT(2))
      END IF
      
      DO IZ = 1, NL_ECSTRA
            
         FORALL(IPTSWL = 1:NPTSWL) BETAFIT(IZ, IPTSWL) = 
     $        EXP(ACOEFF(2,IZ)*(WAVFIT(IPTSWL) - WAVLEN0) + 
     $        ACOEFF(3,IZ)*(WAVFIT(IPTSWL) - WAVLEN0)**2)
         
         IF (NPTSWL.EQ.1) THEN  ! direct use of the fit result
            BETA(IZ) = BETAFIT(IZ, 1)
         ELSE                   ! linear approximation
            BETA(IZ) = BETAFIT(IZ, 2)
     $           + WAVRATIO*(BETAFIT(IZ, 1) - BETAFIT(IZ, 2))
         END IF
         
C     Ultimate securities on beta
C     ----------------------------
         IF (WAVLEN > WAVLMAX) BETA(IZ) = 
     $        MIN(BETA(IZ), 2.0D0*EXP(-ACOEFF(1,IZ)))
         IF (BETA(IZ) < 0.0D0) BETA(IZ) = 0.
         
      END DO
               
      RETURN

      END

C**************************************************************************
      SUBROUTINE GET_TPHGT (TROPHEIGHT)

      USE LOG_CONTROL,   ONLY : C_UNIT
      USE ST_VARS, ONLY : LATITUDE
      USE ST_INPUT,      ONLY : DAY_OF_YEAR, YEAR, TPH_DB_PATH
      
      INTEGER              monInt, idxLat, DAY 
      INTEGER, PARAMETER :: numLat = 45    ! number of latitude bins
      CHARACTER (LEN = 2)  monChar
      CHARACTER (LEN=200)  FILENAME
      DOUBLE PRECISION     lat(numLat), tpHgt(numLat), TROPHEIGHT

      CALL CALDAY(DAY_OF_YEAR, YEAR, monInt, DAY)
      write(monChar,'(I2.2)') monInt

      FILENAME = TRIM(TPH_DB_PATH)//'/tphgt_mon'
     $     //monChar//'.ncep'
      OPEN (UNIT=C_UNIT, FILE = FILENAME, STATUS='OLD')
      CALL SKIP_HEADER (C_UNIT)
      DO IDXLAT = 1, NUMLAT
         READ (C_UNIT,*) LAT(IDXLAT), TPHGT(IDXLAT) 
      ENDDO
      CLOSE (C_UNIT)

      idxLat = MINLOC(DABS(LAT(:) - LATITUDE),1)
      tropHeight = tpHgt(idxLat)
 
      RETURN
      END

C**************************************************************************
      DOUBLE PRECISION FUNCTION PolLeg (IFCTCOLAT, COLAT)

!
! * Calculation of the Legendre polynom at order 2*ifctcolat, for the
! * variable value colat.
!
!                               version 1.0 /  April 16th, 1997
!                               version 3.0 /  April 27th, 1998 (unchanged)
!                               Christine BINGEN
!                               IASB-BIRA
!                               Belgian Institute for Space Aeronomy
!                               3, avenue Circulaire
!                               B-1180 Brussels (BELGIUM)
!
      INTEGER              ifctcolat  
      DOUBLE PRECISION     colat


      IF (ifctcolat.EQ.1) THEN
         PolLeg=1.
         RETURN
      ELSE IF (ifctcolat.EQ.2) THEN
         PolLeg=(1.+3.*cos(2*colat))/4.
         RETURN
      ELSE IF (ifctcolat.EQ.3) THEN
         PolLeg=(9.+20.*cos(2*colat)+35.*cos(4*colat))/64.
         RETURN
      ELSE IF (ifctcolat.GE.4) THEN
         PolLeg=0.
         WRITE(*,*)'Function PolLeg.: order of the polynom too high;'
         WRITE(*,*)'Value assigned to zero'
         WRITE(*,*)
         RETURN
      END IF
      RETURN

      END

C**************************************************************************
      DOUBLE PRECISION FUNCTION erf(x)

! * Computes the error function for the value x, following a
! * rational approximation furnished by Abramovitz:
! * Rem: the estimated maximal error on the approximation
! * is <= 2.5E-5 .
!
!                             version 1.0 / April 30th, 1998
!                             Christine BINGEN
!                             IASB-BIRA
!                             Belgian Insitute for Space Aeronomy
!                             3, Avenue Circulaire
!                             B-1180 Brussels (BELGIUM)
!

      DOUBLE PRECISION     A1, A2, A3, A4, A5, P, T, ERFABS, XABS, X
      DATA a1 /  .254829592/
      DATA a2 / -.284496736/
      DATA a3 / 1.421413741/
      DATA a4 /-1.453152027/
      DATA a5 / 1.061405429/
      DATA p / .3275911/

      xabs=abs(x)
      t=1./(1.+p*xabs)
      erfabs=1.D0-(a1*t+a2*t**2+a3*t**3+a4*t**4+a5*t**5)
     &      *dexp(-1.D0*xabs*xabs)
      erf=sign(sngl(erfabs), sngl(x))

      RETURN
      END
C**************************************************************************



