C***********************************************************************
C                                                                      *
C       SCIATRAN - Version 4.3 - Last modification: 02 Feb 2021        *
C                                                                      *
C       Copyright (C) Institute of Remote Sensing,                     *
C                      University of Bremen, Germany                   *
C                                                                      *
C**********************************************************************
      SUBROUTINE ST_PLANCK_INIT(JPOS, POS, NPLANCK, PL_SURF, *)
c     ... Called by "dom_spherical.f"

      USE GRIDS,       ONLY: NLAYERS
      USE INTERN_DIMS, ONLY: NL_pla
      USE IRRAD,       ONLY: SOLSPEC => SOL_FLUX, WL_IRRAD_DIM
      USE LOG_PARS,    ONLY: EU, LUNIT, LOG_LEVEL, N_ESO
      USE PROFILES,    ONLY: TTT => TEMP
      USE ST_VARS,     ONLY: M_PLA
      USE THERMAL_VAR, ONLY: BTEM, WNUMLO, WNUMHI, INTEGR_PLANCK, HC_K,
     $                       Nwl_emis, Emi_FILE, Sur_Emis, Wl_Emis,
     $                       Emi_akima
      USE TYPE_VARS,   ONLY: KND
c     ------------------------------------------------------------------
      INTEGER,    INTENT(IN) :: JPOS
      REAL (KND), INTENT(IN) :: POS
      REAL (KND), INTENT(OUT):: PL_SURF
      REAL (KND) NPLANCK(NL_pla)
c     Local parameters
      CHARACTER (LEN=*), PARAMETER :: R_NAME = 'ST_PLANCK_INIT'
      INTEGER     I
      REAL (KND)  RENORM, Emis
c    ===================================================================
    
      CALL ST_TRACK (R_NAME, 1, 'I')

      RENORM = 1.d0
      IF( INTEGR_PLANCK ) THEN
         RENORM = 1.d0/WNUMLO - 1.d0/WNUMHI
         SELECT CASE( TRIM(WL_IRRAD_DIM(1)) )
         CASE('nm')       ;  RENORM = RENORM*1.D+07
         CASE('micron')   ;  RENORM = RENORM*1.D+04
         CASE('angstrom') ;  RENORM = RENORM*1.d+08
         END SELECT
         CALL PLKAVG( WNUMLO, WNUMHI, BTEM, PL_SURF, *880 )
c         write(*,*) 
c     $        ' Integrated Planck function at the surface temperature '
c         write(*,'(10x,2(2x,e12.4))')  PL_SURF
         DO I = 1,NLAYERS
            CALL PLKAVG( WNUMLO, WNUMHI, TTT(I), NPLANCK(I), *880 )
         ENDDO
         NPLANCK(:)     = NPLANCK(:)/SOLSPEC(JPOS)*3.1415926D0
         PL_SURF = PL_SURF/SOLSPEC(JPOS)*3.1415926D0
      ELSE
         CALL ST_PLANCK
      ENDIF
      NPLANCK(:) = NPLANCK(:)/RENORM
      PL_SURF = PL_SURF/RENORM
      
      IF( Emi_FILE ) THEN
c     ... Emissivity depends on the wavelength
         CALL AL_INTERP(Nwl_emis, Wl_emis, Sur_emis,
     $        1, POS, Emis, Emi_akima, *880)  
      ELSE
c     ... Emissivity is wavelength independent
         Emis  = Sur_emis(1)
      ENDIF
      IF( LOG_LEVEL(1) >= N_ESO ) THEN
         write(*,*) 
     $        ' Planck function at Ts normalized by Pi/Solspec '
         write(*,'(10x,3(2x,e12.4))')  PL_SURF, Emis, SOLSPEC(JPOS)
      ENDIF
      PL_SURF  = PL_SURF*Emis

      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN

 880  WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)
      RETURN 1

      CONTAINS

C*************************************************************************
C     Calculate Planck function  in absolute physical units corresponding 
C     to the selected solar irradiance units
c*************************************************************************
      SUBROUTINE ST_PLANCK
      REAL (KND)    WLM
      REAL (KND), PARAMETER :: 
     $              C = 299792458.0D0, ! Light speed [m/s]
     $              hP = 6.626071D-34  ! Planck's constant [J s]

c     ... Convert the internal wavelength [nm] into [m]
      WLM = POS*1.0D-9

c     ... phot/s/m^2/m/sr ; 
c         HC_K = [Planck_const*Light_speed/Boltzmann_constant] = [m K]
      NPLANCK(:) = 2.d0*C/WLM**4/(DEXP(HC_K/WLM/TTT(:)) - 1.d0)
c     ... W/m^2/m/sr
      IF( TRIM(WL_IRRAD_DIM(2))=='W' ) NPLANCK(:) = NPLANCK(:)*hP*C/WLM

c      WRITE(*,*) '  Test Planck surface M_PLA = ', M_PLA
c      WRITE(*,'(5x,f7.2,2x,e12.4)') TTT(NLAYERS), M_PLA*NPLANCK(NLAYERS)
c      WRITE(*,*) '  Extra terrestial solar flux'
c      WRITE(*,'(14x,e12.4)') SOLSPEC(JPOS)

c      DO I = 1,NLAYERS
c         write(*,'(5x,I3,2x,F7.3,2x,e13.5)') I, TTT(I), M_PLA*NPLANCK(I)
c      ENDDO

      NPLANCK(:) = M_PLA*NPLANCK(:)/SOLSPEC(JPOS)*3.1415926d0

c     Planck function for surface temperature
      PL_SURF = 2.D0*C/WLM**4/(DEXP(HC_K/WLM/BTEM) - 1.d0)
c     ... W/m^2/m/sr
      IF( TRIM(WL_IRRAD_DIM(2))=='W' ) 
     $     PL_SURF = PL_SURF*hP*C/WLM

c      write(*,*) ' Monochromatic Planck function '//
c     $     'at the surface temperature '
c      write(*,'(10x,2(2x,e12.4))')  M_PLA*PL_SURF

      PL_SURF = M_PLA*PL_SURF/SOLSPEC(JPOS)*3.1415926D0

      END SUBROUTINE ST_PLANCK
C**********************************************************************
      END
C**********************************************************************
