C********************************************************************C
C
C    This is an example of the user-defined subroutine for the 
C               calculation of a surface BRDF.
C    The subroutine calculates the BRDF of "leaf forest" scenario 
C                                          -------------
C    according to the RPV BRDF model (see for details:
C    Rahman H, Pinty B, Verstraete MM. "Coupled surface-atmosphere 
C    reflectance (CSAR) model." J.Geophys.Res., 1993:98;20791-20801. 
C
C    In the vector case this subroutine calculates elements of the 
C    reflection matrix corresponding to the first Stokes component only.
C    All other elements of the reflection matrix will be set to zero.
C
C********************************************************************C
      SUBROUTINE USER_BRDF(Nvs, N_ZEN_REFL, COS_REFL, N_AZM_REFL, 
     $     PHI_REFL, N_ZEN_INC, COS_INC, Nwl, WL, SURF_REFL)
c     Input parameters:
c     ================
c     Nvs                     equals to 1 or to 4 for the scalar or 
c                             vector case, respectively
c          Reflected radiation 
c     N_ZEN_REFL              Number of zenith angles
c     COS_REFL(1:N_ZEN_REFL)  Cosines of zenith angles
c     N_AZM_REFL              Number of azimuthal angles
c     PHI_REFL(1:N_AZM_REFL)  Relative azimuthal angles [Rad]

c          Incident radiation
c     N_ZEN_INC               Number of zenith angles
c     COS_INC(1:N_ZEN_INC)    Cosines of zenith angles
c               Azimuthal angle for the incident radiation is set to zero

c     Nwl                     Number of wavelength for which surface BRDF 
c                             will be calculated
c          !! It is assumed that RPV approximation contains only single 
c               wavelength dependent parameter i.e., rho !!
c     Output parameters:
c     =================
c     WL(1:Nwl)               Output wavelengths [nm]
c     SURF_REFL(1:Nvs*N_ZEN_REFL,1:Nvs*N_ZEN_INC,1:N_AZM_REFL,1:Nwl) 
c                             Output surface BRDF

      INTEGER            N_ZEN_REFL, N_AZM_REFL, N_ZEN_INC, Nwl, Nvs
      DOUBLE PRECISION   COS_REFL(N_ZEN_REFL), PHI_REFL(N_AZM_REFL)
      DOUBLE PRECISION   COS_INC(N_ZEN_INC)

c     Output parameters:
c     ==================
      DOUBLE PRECISION   WL(Nwl)
      DOUBLE PRECISION   
     $     SURF_REFL(Nvs*N_ZEN_REFL,Nvs*N_ZEN_INC,N_AZM_REFL,Nwl)

c     Local parameters:
c     =================
      INTEGER  I, J, K, L, I1, K1
      DOUBLE PRECISION, PARAMETER :: Pie = 3.1415927d0, 
     $     Pa = 1.246d0, Pb = 1.186d0, Pc = 5.157d0
      DOUBLE PRECISION   COS_PHI, COS_G, G, R, H, DCR, DCI, Pga, ANG_G
      DOUBLE PRECISION, DIMENSION (N_ZEN_REFL) :: SIN_REFL
      DOUBLE PRECISION, DIMENSION (N_ZEN_INC)  :: SIN_INC

      WL(1) = 500.d0
c      The following setting is for test purpose only !
      IF( Nwl > 1 ) THEN
         WL(2) = 600.d0
      ENDIF

      SURF_REFL = 0.d0

      SIN_REFL = DSQRT(1.D0 - COS_REFL**2)
      SIN_INC  = DSQRT(1.D0 - COS_INC**2)

      DO I = 1, N_ZEN_REFL
         I1 = (I-1)*Nvs+1  ;  DCR = DABS(COS_REFL(I))
         DO J = 1, N_AZM_REFL
            COS_PHI = DCOS(PHI_REFL(J))
            DO K = 1, N_ZEN_INC
               K1 = (K-1)*Nvs+1 ; DCI = DABS(COS_INC(K))
               COS_G = -DCR*DCI + SIN_REFL(I)*SIN_INC(K)*COS_PHI
               ANG_G = DACOS( COS_G )/Pie*180.d0
               Pga = 11.1*DEXP(-0.087*ANG_G) + 1.1*DEXP(-0.014*ANG_G)
               DO L = 1, Nwl
                  SURF_REFL(I1,K1,J,L) = 
     $                 (Pa + Pb*(DCR + DCI) + Pc*DCR*DCI + Pga)
     $                 /4.d0/(DCR + DCI)
               ENDDO
            ENDDO
         ENDDO
      ENDDO
      
      RETURN
      END
