C***********************************************************************C
C
C    This is an user-defined subroutine for the calculation 
C               of water-leaving reflectance.
C    In the vector case this subroutine calculates elements of the 
C    reflection matrix corresponding to the first Stokes component only.
C    All other elements of the reflection matrix will be set to zero.
C
C***********************************************************************C
      SUBROUTINE USER_WATER(Nvs, N_ZEN_REFL, COS_REFL, N_AZM_REFL, 
     $     PHI_REFL, N_ZEN_INC, COS_INC_INP, Nwl, WL, SURF_REFL )
c     Input parameters:
c     ================
c     Nvs                     equals to 1 or to 4 for the scalar or 
c                             vector case, respectively
c          Reflected radiation 
c     N_ZEN_REFL              Number of zenith angles
c     COS_REFL(1:N_ZEN_REFL)  Cosines of zenith angles
c     N_AZM_REFL              Number of azimuthal angles
c     PHI_REFL(1:N_AZM_REFL)  Relative azimuthal angles [Rad]

c          Incident radiation
c     N_ZEN_INC               Number of zenith angles
c     COS_INC_INP(1:N_ZEN_INC)    Cosines of zenith angles
c               Azimuthal angle for the incident radiation is set to zero

c     Nwl                     Number of wavelength for which surface BRDF 
c                             will be calculated
c     WL(1:Nwl)               Wavelengths [nm]
c     Output parameters:
c     =================
c     SURF_REFL(1:Nvs*N_ZEN_REFL,1:Nvs*N_ZEN_INC,1:N_AZM_REFL,1:Nwl) 
c                             Output surface BRDF
      USE DEPOLARIZATION_ALBEDO_AEROSOL, 
     $                     ONLY : Nwl_BRF, WL_BRF, N_beta, Beta_ang,
     $                            Beta_val, P_yell, AlfW, Alf0, WL0
      USE LOG_PARS,        ONLY : LUNIT, LOG_LEVEL, N_ESO, DIR_OUT 
      USE ST_VARS,   ONLY : Pie, RTD => RAD_TO_DEG
c      USE GT_VARS,         ONLY : NN => NSTRMS
      USE WATER,           ONLY : Ts => T_ocean, Salin, Chlorophyll, 
     $                            Vs => V_small, Vl => V_large,
     $                            VSF_pure_model, N_ang_wa, Rho_wa_ang, 
     $                            Rho_wa, N_ang_aw, Rho_aw_ang, Rho_aw

      CHARACTER (LEN = *), PARAMETER :: R_NAME='USER_WATER'
      INTEGER            N_ZEN_REFL, N_AZM_REFL, N_ZEN_INC, Nwl, Nvs
      DOUBLE PRECISION   COS_REFL(N_ZEN_REFL), PHI_REFL(N_AZM_REFL)
      DOUBLE PRECISION   COS_INC_INP(N_ZEN_INC), COS_INC(N_ZEN_INC)
      DOUBLE PRECISION   WL(Nwl)

c     Output parameters:
c     ==================
      DOUBLE PRECISION   
     $     SURF_REFL(Nvs*N_ZEN_REFL,Nvs*N_ZEN_INC,N_AZM_REFL,Nwl)

c     Local parameters:
c     =================
      INTEGER            I, J, K, L, I1, K1
      DOUBLE PRECISION   SCAT, B_int, G, Chl_dep, TMP
      DOUBLE PRECISION   SINI(N_ZEN_INC), SINR(N_ZEN_REFL), 
     $                   COAZ(N_AZM_REFL), Rho_loc_wa(N_ZEN_REFL), 
     $                   Rho_loc_aw(N_ZEN_INC)
      DOUBLE PRECISION   Cap(Nwl), RR(Nwl), Abs_co(Nwl), Sca_pure(Nwl), 
     $                   Sca_small(Nwl), Sca_larger(Nwl), Omega(Nwl), 
     $                   Ext_all(Nwl), S1wl(Nwl), S2wl(Nwl)
      DOUBLE PRECISION, TARGET  :: Rezr(1), Rezi(1) 
      DOUBLE PRECISION, POINTER :: P_abs(:), P_chl(:)
c      Parameters related to the Kopelevich model
      DOUBLE PRECISION   ANGL(8), Ph_s(8), Ph_l(8)
      DATA ANGL /60.d0, 75.d0, 90.d0, 105.d0, 120.d0, 135.d0, 150.d0,
     $           180.d0 /
      DATA Ph_s /4.1d-02, 2.0d-02, 1.2d-02, 8.6d-03, 7.4d-03, 7.4d-03, 
     $           7.5d-03, 8.1d-03 /
      DATA Ph_l /3.8d-04, 2.0d-04, 6.3d-05, 4.4d-05, 2.9d-05, 2.0d-05, 
     $           2.0d-05, 7.0d-05 /
      DOUBLE PRECISION    S_ANG, Phsi, Phli
      CHARACTER (LEN=20)  Petzold_Kopelevich
c      Parameters related to the Buiteveld et al. model
      DOUBLE PRECISION    B_T, Nref(Nwl), Ndl(Nwl), NdT, NdlT, p_90, Sf,
     $                    B_90(Nwl), Fac(Nwl)
      DOUBLE PRECISION, PARAMETER ::    
     $                    Ts0=20.d0, Tk=293.15d0, Kb=1.38054d0, 
     $                    DepB=0.051d0, DepSM=0.09d0, Sal0=37.d0
c                         Fluorescent parameter [nm]
c     $                    S_FLU=10.64d0

      CALL ST_TRACK (R_NAME, 1, 'I')

      COS_INC  = DABS( COS_INC_INP )

c     It can be used the wavelength independent volume scattering 
c     function (experimental data of Petzold) or model given by 
c     Kopelevich
c     ------------------------------------------------------------------
      Petzold_Kopelevich = 'Kopelevich'

c     Expressions which are used below to calculate volume scattering 
c     function (VSF) correspondent to a pure water (salinity 0%).
c     The factor to take into account the effect of salinity is 
c     introduced as follows:
c     ------------------------------------------------------------------
      Sf = 1.d0 + 0.3d0*Sal0/Salin

      SELECT CASE( TRIM(VSF_pure_model) )
      CASE('Morel')
         B_90(:) = (450.d0/WL(:))**4.32*Sf
         p_90    = (1.d0 - DepSM)/(1.d0 + DepSM)
         Sca_pure(:) = 3.5d-03 *B_90(:)
         B_90(:)     = 2.18d-04*B_90(:)
      CASE('Shifrin')
         B_90(:) = (546.d0/WL(:))**4.17*Sf
         p_90    = (1.d0 - DepSM)/(1.d0 + DepSM)
         Sca_pure(:) = 1.49d-03*B_90(:)
         B_90(:)     = 0.93d-04*B_90(:)
      CASE('Buiteveld')
         B_T = (5.062271d0 - 0.03179d0*Ts + 0.000407d0*Ts*Ts) ! *1.d-10
         Nref(:) = 1.3247d0 + 3.3d03/WL(:)**2 - 3.2d07/WL(:)**4 
     $        - 2.5d-06*Ts*Ts
         Ndl(:) = (-0.000156d0*WL(:) + 1.5989d0) ! *1.d-10
         NdT    = ( 1.61857d0 - 0.005785*Ts )    ! *1.d-10
         NdlT   = ( 1.61857d0 - 0.005785*Ts0 )   ! *1.d-10
         B_90(:) = 2.d0*Pie**2/WL(:)**4/B_T*Kb*Tk*Nref(:)**2*1.d03
     $        *Ndl(:)*NdT/NdlT*(6.d0 + 6.d0*DepB)/(6.d0 - 7.d0*DepB)*Sf
         p_90 = (1.d0 - DepB)/(1.d0 + DepB)
         Sca_pure(:) = 16.d0*Pie/3.d0*B_90(:)
     $        *(2.d0 + DepB)/(1.d0 + DepB)/2.d0 ! Scattering coeff [m-1]
      END SELECT

c     Interpolation on all Gauss or all line-of-sight angles
c     depending on the "Entry"
c     ------------------------------------------------------
      DO I = 1,N_ZEN_REFL
         TMP = DACOS(COS_REFL(I))
         G = DASIN( DSIN( TMP )/1.34d0 )*RTD
         CALL LIN_INT(N_ang_wa,Rho_wa_ang,Rho_wa, 1,G,Rho_loc_wa(I),'Z')
      ENDDO

      DO I = 1,N_ZEN_INC
         G = COS_INC(I)
         CALL LIN_INT(N_ang_aw,Rho_aw_ang,Rho_aw, 1,G,Rho_loc_aw(I),'C')
      ENDDO

c     Set BRDF parameters at the needed wavelength grid
c     =================================================
      IF( Nwl == 1 .AND. Nwl_BRF > 1) THEN
c        Interpolation on the current wl grid using SS_STOKES entry
         CALL LIN_SEARCH_NEW(WL(1), G, I, K, Nwl_BRF, WL_BRF)
         Rezr(1) = (1.D0 - G)*AlfW(I) + G*AlfW(K)
         P_abs => Rezr(1:1)
         Rezi(1) = (1.D0 - G)*P_yell(I) + G*P_yell(K)
         P_chl => Rezi(1:1)
      ELSE
c        Using previously interpolated or constant values
         P_abs => AlfW(:)  ;  P_chl => P_yell(:)
      ENDIF

c     ... Wl dependence of the small particles angular scattering coefficient
      S1wl(:) = (550.d0/WL(:))**1.7
c     ... Wl dependence of the larger particles angular scattering coefficient
      S2wl(:) = (550.d0/WL(:))**0.3

c     ... Chlorophyll related absorption coefficient is obtained 
c         multiplying Chl_dep by P_chl(:) (see below)
      Chl_dep = 0.06d0*Chlorophyll**0.65d0

c     ... Dissolved organic matter or yellow matter absorption coefficient
      IF( REAL(Chl_dep) == 0.0 ) THEN
         Cap(:) = 0.d0
      ELSE
         Cap(:) = 0.2d0*(Alf0 + Chl_dep)*DEXP( -0.014d0*(WL(:)-WL0) )
c         Cap(:) = 0.2d0*(Chl_dep)*DEXP( -0.014d0*(WL(:)-WL0) )
      ENDIF

c     ... Resulting absorption coefficient including pure water 
c         absorption given by term P_abs(:)
      Abs_co(:) = Cap(:) + Chl_dep*P_chl(:) + P_abs(:)

      Sca_small(:)  = 1.34d0 *Vs*S1wl(:)
      Sca_larger(:) = 0.312d0*Vl*S2wl(:)

      Ext_all(:) = 
     $     Sca_pure(:) + Sca_small(:) + Sca_larger(:) + Abs_co(:)
      Omega(:) = (Sca_pure(:) + Sca_small(:) + Sca_larger(:))/Ext_all(:)

c      WRITE(*,*) '   Single scattering albedo of small fraction:'
c      WRITE(*,*) '   ------>  ', Omega(:)
c      WRITE(*,*) 
      Fac(:) = 1.d0 - (0.50d0*Sca_pure(:) + 0.39d-01*Sca_small(:) 
     $     + 0.64d-03*Sca_larger(:))/
     $     (Sca_pure(:) + Sca_small(:) + Sca_larger(:))

c      WRITE(*,*) '   Faktor F:'
c      WRITE(*,*) Fac(:)

      IF( Nvs > 1 ) SURF_REFL = 0.0d0

      WHERE( P_abs(:) > 0.0d0 ) 
c         RR(:) = Pie/( Cap(:) + Chl_dep*P_chl(:) + P_abs(:) ) /1.34d0**2
c            RR(:) = Pie/Ext_all(:)/( 1.d0 - Omega(:)*Fac(:) ) /1.34d0**2
         RR(:) = Pie/Ext_all(:)/( 1.d0 - Omega(:) ) /1.34d0**2
      ELSEWHERE
         RR(:) = 0.0d0
      END WHERE
 
      SINI     = DSQRT( 1.0d0 - COS_INC*COS_INC )
      COS_REFL = DABS( COS_REFL )
      SINR     = DSQRT( 1.0d0 - COS_REFL*COS_REFL )
      COAZ(:)  = DCOS( PHI_REFL(:) )

      SELECT CASE ( TRIM(Petzold_Kopelevich) )
      CASE('Kopelevich')
         DO I = 1, N_ZEN_REFL
            I1 = (I-1)*Nvs+1
            DO J = 1, N_AZM_REFL
               DO K = 1, N_ZEN_INC
                  K1 = (K-1)*Nvs+1
                  SCAT = -COS_REFL(I)*COS_INC(K)+SINR(I)*SINI(K)*COAZ(J)
                  S_ANG = DACOS(SCAT)/Pie*180.d0
                  CALL LIN_INT(8, ANGL, Ph_s, 1, S_ANG, Phsi, 'L')
                  CALL LIN_INT(8, ANGL, Ph_l, 1, S_ANG, Phli, 'L')
                  DO L = 1, Nwl
                     SURF_REFL(I1,K1,J,L) = RR(L)
     $                    *(1.d0 - Rho_loc_wa(I))
     $                    *(1.d0 - Rho_loc_aw(K))
     $                    *( B_90(L)*(1.d0 + p_90*SCAT**2)
     $                    + Vs*Phsi*S1wl(L) + Vl*Phli*S2wl(L) )
     $                    /(COS_REFL(I) + COS_INC(K))
                  ENDDO
               ENDDO
            ENDDO
         ENDDO
      CASE('Petzold')
         DO I = 1, N_ZEN_REFL
            I1 = (I-1)*Nvs+1
            DO J = 1, N_AZM_REFL
               DO K = 1, N_ZEN_INC
                  K1 = (K-1)*Nvs+1
                  SCAT = -COS_REFL(I)*COS_INC(K)+SINR(I)*SINI(K)*COAZ(J)
                  S_ANG = DACOS(SCAT)/Pie*180.d0
                  CALL LIN_INT(N_beta, Beta_ang, Beta_val, 
     $                 1, S_ANG, B_int, 'L') 
                  DO L = 1, Nwl
                     SURF_REFL(I1,K1,J,L) = RR(L)*B_int
     $                    /(COS_REFL(I) + COS_INC(K))
                  ENDDO
               ENDDO
            ENDDO
         ENDDO

      END SELECT

      IF (LOG_LEVEL(1) >= N_ESO) THEN
         WRITE(LUNIT,*) '   Water-leaving BRF in nadir '//
     $        'direction (at all solar angles)'
         I1 = (N_ZEN_REFL-1)*Nvs + 1 ! In nadir direction
         K = MIN0(3,N_ZEN_INC)
         DO L = 1, Nwl
            WRITE(LUNIT,'(12x,F7.3,10(2x,E12.4))') 
     $           WL(L), (SURF_REFL(I1,K1,1,L), K1 = 1,K*Nvs,Nvs)
         ENDDO
         WRITE(LUNIT,*) '  '
      ENDIF
c       K1 = NN*Nvs+1
c       open(63,file=DIR_OUT//'/refl_wl.dat')
c          DO L = 1,Nwl
c             WRITE(63,*) WL(L), SURF_REFL(I1,K1,1,L)
c          ENDDO
c       close(63)

c       stop
c      CALL INTEGRATE_PHF

      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN
c      CONTAINS

cc *************************************************************************
c      SUBROUTINE INTEGRATE_PHF
c      INTEGER, PARAMETER:: MUM=1000, C_UNIT=55, N_gau = 600
c      INTEGER              I, J
c      INTEGER              Na_st
c      LOGICAL              ERROR
c      CHARACTER (Len=80)   Dir, Ph_name
c      DOUBLE PRECISION     RAD_TO_DEG, Bs, Bsf, Bsb
c      DOUBLE PRECISION     TMP1(MUM), TMP2(MUM,2)
c      DOUBLE PRECISION, DIMENSION (N_gau) :: X_gau, AG_gau, Phs_gau, 
c     $                                       X_rad
c      DOUBLE PRECISION, DIMENSION (N_gau/2) :: X_gau1, AG_gau1,  
c     $                                       Phs_gau1, X_rad1
c      DOUBLE PRECISION, ALLOCATABLE :: Sa_inp(:), Sa_rad(:), 
c     $                     Phs_inp(:), Sa_cos(:), Phs_ln(:)
c
c      Dir = '/home/home/rozanov/SCIATRAN/data/surface/'
c      Ph_name = 'Phf_Kopelevich_sl.dat'
c
c      OPEN(C_UNIT, FILE = TRIM(Dir)//TRIM(Ph_name), STATUS='OLD') 
c        CALL SKIP_HEADER (C_UNIT)           
c        DO I = 1,MUM
c           READ(C_UNIT,*,END=28,ERR=29) tmp1(I), (tmp2(I,J), J=1,2)
c        END DO
c 29     continue
cc 29     CALL WRITE_MESSAGE (ROUTINE_NAME, 'ERROR', 
cc     $       'Error reading data file !')
cc        WRITE(ERRORUNIT,*) '    File name : ', TRIM(Names(I1,K))
c        STOP 'Error reading'
cc        RETURN 1             
c 28     CONTINUE
c        WRITE(*,*) ' End of reading I = ', I
c        WRITE(*,*) tmp1(1), (tmp2(1,J), J=1,2)
c        WRITE(*,*) tmp1(I-1), (tmp2(I-1,J), J=1,2)
c        Na_st = I-1
c      CLOSE(C_UNIT)
c
c      ALLOCATE( Sa_inp(Na_st),Sa_rad(Na_st),Phs_inp(Na_st),
c     $     Sa_cos(Na_st), Phs_ln(Na_st) )
c      Sa_inp(:)  = TMP1(1:Na_st)
c      RAD_TO_DEG = 180.0D0/DACOS(-1.0D0)
c      Sa_rad(:)  = Sa_inp(:)/RAD_TO_DEG
c
c      Sa_cos(:)  = DCOS( Sa_rad(:) )
c      WRITE(*,*) '   Sa_rad: [deg]' 
c      WRITE(*,*) Sa_rad*RAD_TO_DEG
c      WRITE(*,*) '                 ------------- '
c
c      Phs_inp(:) = TMP2(1:Na_st,2)*2.d0*Pie
c      WRITE(*,*) 'Phs_inp first angle '
c      WRITE(*,*) Phs_inp(1)
c
cc     ... Gauss grid and weights
c      CALL GAULEG ( -1.D0, 1.D0, X_gau, AG_gau, N_gau )
cc        ... Invert Gauss angles to start from forward direction
c      X_rad(:)   = DACOS( X_gau(N_gau:1:-1) )
c      WRITE(*,*) '   X_rad: [deg]' 
c      WRITE(*,*) X_rad*RAD_TO_DEG
c      WRITE(*,*) '                 ------------- '
c
cc     ... Gauss grid and weights in [0,1]
c      CALL GAULEG ( 0.D0, 1.D0, X_gau1, AG_gau1, N_gau/2 )
cc        ... Invert Gauss angles to start from forward direction
c      X_rad1(:)   = DACOS( X_gau1(N_gau/2:1:-1) )
c      WRITE(*,*) '   X_rad1: [deg]' 
c      WRITE(*,*) X_rad1*RAD_TO_DEG
c      WRITE(*,*) '                 ------------- '
c      
cc        ... Interpolate phase function on the Gauss grid [-1,1]
c      Phs_ln(:) = DLOG( Phs_inp(:) )
c      CALL AKIMA_INTERP (Na_st, Sa_rad, Phs_ln,
c     $     N_gau, X_rad, Phs_gau, ERROR)  
c      Phs_gau(:) = DEXP( Phs_gau(:) )
c      IF ( ERROR ) STOP 'Interpolation error in "AKIMA_INTERP"'
c      WRITE(*,*) X_rad(1)*RAD_TO_DEG, Phs_gau(1)
c
c      Bs = SUM( Phs_gau(:)*AG_gau(:) )
c      WRITE(*,*) '  Small fraction: integrated scattering function'
c      WRITE(*,*) Bs
c
cc      Interpolation for the forward integration
c      CALL AKIMA_INTERP (Na_st, Sa_rad, Phs_ln,
c     $     N_gau/2, X_rad1, Phs_gau1, ERROR)  
c      Phs_gau1(:) = DEXP( Phs_gau1(:) )
c      IF ( ERROR ) STOP 'Interpolation error in "AKIMA_INTERP"'
c      WRITE(*,*) X_rad1(1)*RAD_TO_DEG, Phs_gau1(1)
c
c      Bsf = SUM( Phs_gau1(:)*AG_gau1(:) )
c
cc      Interpolation for the backward integration
c      X_rad1(:) = X_rad1(:) + 90.d0/RAD_TO_DEG
c      CALL AKIMA_INTERP (Na_st, Sa_rad, Phs_ln,
c     $     N_gau/2, X_rad1, Phs_gau1, ERROR)  
c      Phs_gau1(:) = DEXP( Phs_gau1(:) )
c      IF ( ERROR ) STOP 'Interpolation error in "AKIMA_INTERP"'
c      WRITE(*,*) X_rad1(1)*RAD_TO_DEG, Phs_gau1(1)
c
c      Bsb = SUM( Phs_gau1(:)*AG_gau1(:) )
c      WRITE(*,*) 
c     $     '  Small fraction: integrated scattering function forw:'
c      WRITE(*,*) Bsf
c      WRITE(*,*) Bsb
c      WRITE(*,*) Bsf + Bsb, Bs
c      WRITE(*,*) (Bsf + Bsb - Bs)/Bs*100.
c      WRITE(*,*) Bsb/Bs
c      WRITE(*,*) '   ' 
c
cc      Direct simple integration
c      Bs = 0.d0
c      DO I = 1, Na_st-1
c         Bs = Bs + (Phs_inp(I) + Phs_inp(I+1))/2.
c     $        *DABS(Sa_cos(I) - Sa_cos(I+1))
c      ENDDO
c      WRITE(*,*) '  Direct integration'
c      WRITE(*,*) Bs
c      WRITE(*,*) '   ' 
c
c      STOP
c
c      END SUBROUTINE INTEGRATE_PHF
      END
