C***********************************************************************C
C
C    This is an user-defined subroutine for the calculation 
C               of reflectance of whitecap.
C    In the vector case this subroutine calculates elements of the 
C    reflection matrix corresponding to the first Stokes component only.
C    All other elements of the reflection matrix will be set to zero.
C
C***********************************************************************C
      SUBROUTINE USER_WHITECAP(Nvs, N_ZEN_REFL, COS_REFL, N_AZM_REFL, 
     $     PHI_REFL, N_ZEN_INC, COS_INC, Nwl, WL, SURF_REFL, Area )
c     Input parameters:
c     ================
c     Nvs                     equals to 1 or to 4 for the scalar or 
c                             vector case, respectively
c          Reflected radiation 
c     N_ZEN_REFL              Number of zenith angles
c     COS_REFL(1:N_ZEN_REFL)  Cosines of zenith angles
c     N_AZM_REFL              Number of azimuthal angles
c     PHI_REFL(1:N_AZM_REFL)  Relative azimuthal angles [Rad]

c          Incident radiation
c     N_ZEN_INC               Number of zenith angles
c     COS_INC(1:N_ZEN_INC)    Cosines of zenith angles
c               Azimuthal angle for the incident radiation is set to zero

c     Nwl                     Number of wavelength for which surface BRDF 
c                             will be calculated
c     WL(1:Nwl)               Wavelengths [nm]

c     Output parameters:
c     =================
c     SURF_REFL(1:Nvs*N_ZEN_REFL,1:Nvs*N_ZEN_INC,1:N_AZM_REFL,1:Nwl) 
c                             Output surface BRDF
      USE DEPOLARIZATION_ALBEDO_AEROSOL, 
     $                     ONLY : Nwl_BRF, WL_BRF, RefF_i,
     $                            P_yell, Bubble_Dim, DO_OC_COMP,
     $                            LWC_Foam, AlfW, Alf0, WL0
      USE LOG_CONTROL,     ONLY : LUNIT, LOG_LEVEL, N_ESO
      USE ST_VARS,   ONLY : Pie
      USE WATER,           ONLY : Chlorophyll, W_speed

      CHARACTER (LEN = 13), PARAMETER :: ROUTINE_NAME='USER_WHITECAP'
      INTEGER            N_ZEN_REFL, N_AZM_REFL, N_ZEN_INC, Nwl, Nvs
      LOGICAL            SET_AS_WATER
      DOUBLE PRECISION   Chl_dep
      DOUBLE PRECISION   COS_REFL(N_ZEN_REFL), PHI_REFL(N_AZM_REFL)
      DOUBLE PRECISION   COS_INC(N_ZEN_INC)

c     Output parameters:
c     ==================
      DOUBLE PRECISION   WL(Nwl), Area
      DOUBLE PRECISION   
     $     SURF_REFL(Nvs*N_ZEN_REFL,Nvs*N_ZEN_INC,N_AZM_REFL,Nwl)

c     Local parameters:
c     =================
      INTEGER            I, J, K, L, I1, K1
      DOUBLE PRECISION   De, R_inf, G
      DOUBLE PRECISION   Cap(Nwl), Alpha(Nwl), Ref(Nwl)
      DOUBLE PRECISION, PARAMETER:: Be=2.3d0
      DOUBLE PRECISION, TARGET   :: Rez(1)
      DOUBLE PRECISION, POINTER  :: P_refi(:)
      DOUBLE PRECISION, TARGET  :: Rezr(1), Rezi(1) 
      DOUBLE PRECISION, POINTER :: P_abs(:), P_chl(:)

      IF( DO_OC_COMP(3) ) THEN
         SET_AS_WATER = .TRUE.
      ELSE
         SET_AS_WATER = .FALSE.
      ENDIF

c     Set BRDF parameters at the needed wavelength grid
c     =================================================
      IF( SET_AS_WATER ) THEN
         IF (LOG_LEVEL(1) >= N_ESO) THEN
            WRITE(LUNIT,*) '  ****************************************'
            WRITE(LUNIT,*) '    Foam absorption coefficient is used   '
            WRITE(LUNIT,*) '        the same as for ocean water       '
            WRITE(LUNIT,*) '  ****************************************'
         ENDIF
         IF( Nwl == 1 .AND. Nwl_BRF > 1) THEN
c           Interpolation on the current wl grid using SS_STOKES entry
            CALL LIN_SEARCH_NEW(WL(1), G, I, K, Nwl_BRF, WL_BRF)
            Rezr(1) = (1.D0-G)*AlfW(I) + G*AlfW(K)
            P_abs => Rezr(1:1)
            Rezi(1) = (1.D0-G)*P_yell(I) + G*P_yell(K)
            P_chl => Rezi(1:1)
         ELSE
c           Using previously interpolated or constant values
            P_abs => AlfW(:)  ;  P_chl => P_yell(:)
         ENDIF
      ELSE
         IF( Nwl == 1 .AND. Nwl_BRF > 1) THEN
c           Interpolation on the current wl grid using SS_STOKES entry
            CALL LIN_SEARCH_NEW(WL(1), G, I, K, Nwl_BRF, WL_BRF)
            Rez(1) = (1.D0-G)*RefF_i(I) + G*RefF_i(K)
            P_refi => Rez(1:1)
         ELSE
c           Using previously interpolated or constant values 
            P_refi => RefF_i(:)
         ENDIF
      ENDIF

      Area = 2.95D-06*W_speed**3.52d0

c     Reflection of optically semi-infinite medium is set to 1 i.e., 
c     the foam reflectance is considered as Lambertian preliminary 
      R_inf = 1.0d0
      IF( SET_AS_WATER ) THEN
         De = Bubble_Dim*1.D-03 ! Convert [mm] to [m]
         Chl_dep = 0.06d0*Chlorophyll**0.65d0
         Cap(:) = 0.2d0*(Alf0 + Chl_dep)*DEXP( -0.014d0*(WL(:) - WL0) ) 
c                This is a mistake
c     $        + 0.06d0*P_chl(:) + P_abs(:)
     $        + Chl_dep*P_chl(:) + P_abs(:)

         Alpha(:) = DSQRT( Cap(:)*De )

         WHERE( P_abs(:) > 0.0d0 ) 
            Ref(:) = Area*R_inf*Dexp( -DSQRT( Cap(:)*17.1D-03 ) )
c            Ref(:) = Area*R_inf*Dexp( -Be*LWC_Foam**0.25d0*Alpha(:) )
         ELSEWHERE
            Ref(:) = 0.0d0
         END WHERE
      ELSE
         De = Bubble_Dim*1.D6   ! Convert [mm] to [nm]
         Alpha(:) = 2.0d0*DSQRT( Pie*P_refi(:)/WL(:)*De )
         WHERE( Alpha(:) > 0.0d0 ) 
            Ref(:) = Area*R_inf*Dexp( -Be*LWC_Foam**0.25d0*Alpha(:) )
         ELSEWHERE
            Ref(:) = 0.0d0
         END WHERE
      ENDIF

      IF (LOG_LEVEL(1) >= N_ESO) THEN
         WRITE(LUNIT,*) '   Wind speed [m/s]         whitecap area '
         WRITE(LUNIT,'(6x,f6.2,10x,E12.4)') W_speed, Area
      ENDIF

      IF( Nvs > 1 ) SURF_REFL = 0.0d0

      DO I = 1, N_ZEN_REFL
         I1 = (I-1)*Nvs+1
         DO J = 1, N_AZM_REFL
            DO K = 1, N_ZEN_INC
               K1 = (K-1)*Nvs+1
               DO L = 1, Nwl
                  SURF_REFL(I1,K1,J,L) = Ref(L)
               ENDDO
            ENDDO
         ENDDO
      ENDDO

      IF (LOG_LEVEL(1) >= N_ESO) THEN
         WRITE(LUNIT,*) '   Whitecap BBRF in nadir '//
     $        'direction (at all solar angles)'
         I1 = (N_ZEN_REFL-1)*Nvs + 1 ! In nadir direction
         K = MIN0(3,N_ZEN_INC)
         DO L = 1, Nwl
            WRITE(LUNIT,'(12x,F7.3,10(2x,E12.4))') 
     $           WL(L), (SURF_REFL(I1,K1,1,L), K1 = 1,K*Nvs,Nvs)
         ENDDO
      ENDIF

      RETURN
      END
