!***************************************************************************
!
!	SCIATRAN - Version 4.0 - Last modification: 20 Aug 2018
!
!	Copyright (C) - Institute of Remote Sensing,
!			University of Bremen, Germany
!
!***************************************************************************
SUBROUTINE WMO_AER_SETUP(START_WAV, END_WAV, *)
  ! Called by "rtm_init.f"
                              
  USE AER_MIE,   ONLY: DENS_PRF
  USE AEROSOLS,  ONLY: AER_SHIFT, PATH_TO_SCM, OUT_A_SCM, LAY_A, WEL_A, NSCA_A, &
       GE_A, APH_VS, AER_WAVS, NAW => N_AER_WAVS, Greek_Aer, AER_COEFF, Scal_aer, &
       SCI => GT_SCAT_IDX, NG_aer, ASY => GT_ASYM_IDX, Rtau_aer, L_scal, ZS1, &
       EXI => GT_EXTC_IDX, Rwl_aer, ZS2, Plus_mi, NWLS, NWLE, AER_PARAM, &
       AER_SCALING, Nga_db, LA_N, LA_S, LA_E, L_TOP, IDX_AER, LA_Ndb, AOT_out_VAL, &
       AER_FACT, V_PAP, IDX_AER_db
  USE CLOUDS_SS, ONLY: DO_SSRCA
  USE DELTA_M,   ONLY: FDM_AER, AER_DM_FLG, D_MF_AER, AER_TRUNC_IDX
  USE GRIDS,     ONLY: NLAYERS, ZHOLD => ZZZ, DZ
  USE GT_POLYN,  ONLY: NLEGEN
  USE INFO,      ONLY: AER_INFO
  USE I_DIMS,    ONLY: Nm_vs
  USE LOG_PARS,  ONLY: Work_Dir, LOG_LEVEL, LUNIT, EU, DIR_OUT, N_SSO, N_ESO
  USE RTM_PARS,  ONLY: DATA_ITER
  USE ST_VARS,   ONLY: RAD_TO_DEG, Vec_Sca, RTM_TYPE, GROUND_HEIGHT, SCATT_MODE
  USE ST_INPUT,  ONLY: APH_TYPE, ADB => PATH_AER_DB
  USE TOOLS,     ONLY: READ_INPUT, CHK_STAT, SET_PTR, CUT_STR 
  USE TYPES,     ONLY: KND, DYN_CHAR
  
  REAL (KND), INTENT(IN) :: START_WAV, END_WAV
  
  ! Local parameters
  LOGICAL            ERROR
  CHARACTER (LEN = 13), PARAMETER :: R_NAME = 'WMO_AER_SETUP'

  ! CONVENTION: If AER_TRUNC_IDX is specified as the delta-M truncation
  !             index in 'control.dat', then the sum will be taken
  !             ranging from 1 to AER_TRUNC_IDX. 
  !             The moment with which the delta-M truncation coefficient
  !             F_DM will be calculated, corresponds to AER_TRUNC_IDX+1.
  !             After the renormalization all moments from AER_TRUNC_IDX+1 
  !             to NLEGEN will be set to zero.
      
  !  (a) Input variables

  !      AER_DM_FLG        : delta-M approximation yes/no 
  !                          <---- From Module DELTA_M
  !      AER_TRUNC_IDX     : truncation index for delta-M approximation 
  !                          <---- From Module DELTA_M
  !      ZHOLD             : forward model altitude grid
  !      NLAYERS           : number of layers in altitude grid
  !      NLEGEN            : number of Legendre moments
  !      LUNIT             : LOG file unit number
  
  !  (b) Output variables
  !      AER_COEFF         : scattering / extinction coefficients and 
  !                          asymmetry parameters 
  !      AER_PHASMOMS      : Legendre moments
  !      NAW               : number of wavelengths of chosen aerosol 
  !                          parameterization 
  !      AER_WAVS          : wavelengths of aerosol data base (depends on
  ! parameterization chosen)
  INTEGER  N_Z_EXT, N_SKIP, K_NUM
  REAL (KND)  DUM, WL_REF, Scoe, DD1, DD2, WL_S, WL_E
  INTEGER, ALLOCATABLE :: N_MAX(:), IDX_W(:)
  REAL (KND), ALLOCATABLE :: PHAS_LOC (:), PL_RES(:,:), Z_EXT(:), TMP_MOM(:,:,:), &
       EXT_COEFF(:), USER_MOM(:,:,:,:), R_COF(:), X_ang(:), PROF(:), ECO(:,:), &
       SCO(:,:), WEI(:,:), CEXT(:,:), CSCA(:,:), PSCA(:), DB_WAVS(:), ASYM_PAR(:,:), &
       D1(:), TAU(:)
  CHARACTER (LEN=15), ALLOCATABLE :: Type_NAME(:), Type_INP(:)
  CHARACTER (LEN=:), ALLOCATABLE :: STR_INP
  INTEGER, ALLOCATABLE :: LAS(:), LAE(:), IDA(:)
  REAL (KND), POINTER :: AER_PHASMOMS(:,:,:)
  INTEGER  I, I1, J, K, L, NWL_DB, N_DIM, I_REF, Jc, MAX_MOM, KK, Jm, Jk, Nwe, NLEG_LOC, &
       DUNIT, K1, Itr, N_COMP, N_SCE, LAN
  TYPE (DATA_ITER), POINTER :: RTM_PTR
  TYPE (DYN_CHAR), ALLOCATABLE :: READ_ARR(:)
  ! Variable for Delta-M
  ! ====================
  REAL (KND)  F_DM, AER_ABS_COEFF
  ! Variable for Delta-fit
  ! ======================
  INTEGER     N_ANG, Nleg_df, N_KAT
  REAL (KND)  F_Dfit_loc
  !=========================================================================
  CALL ST_TRACK (R_NAME, 1, 'I')
      
  CALL DEALLOCATE_ARRAYS ! All arrays allocated below must be 
                         ! conditionally deallocated here
  IF(CHK_STAT('aer-dens',AER_INFO%NAME,AER_INFO%WF_FLG) ) THEN
     ALLOCATE( DENS_PRF(NLAYERS) )
     DENS_PRF(:) = 1.d0
     CALL SET_PTR( RTM_PTR, 'aer-dens' )
     RTM_PTR%APR(:) = DENS_PRF(:)
     RTM_PTR%CRN(:) = DENS_PRF(:)    
  ENDIF

  N_ANG = 180
  ALLOCATE( X_ang(N_ANG) )
  ERROR = .FALSE.

!  WRITE(*,*) '  wmo_setup:   AER_DM_FLG  ', AER_DM_FLG
  
  ! ==============================================================
  ! Reading of WMO-model general input parameters

  STR_INP  = 'Number of wavelengths (WMO)'
  CALL Read_Input (STR_INP, READ_ARR, *880)
  READ(READ_ARR(1)%S,*,ERR = 101) NWL_DB

  STR_INP  = 'Number of aerosol components'
  CALL Read_Input (STR_INP, READ_ARR, *880)
  READ(READ_ARR(1)%S,*,ERR = 101) N_COMP

  STR_INP  = 'Names aerosol types'
  CALL Read_Input (STR_INP, READ_ARR, *880, ReArrange = .TRUE.)
  N_SCE = SIZE(READ_ARR)

  ! Allocation of local arrays 
  ALLOCATE( DB_WAVS(NWL_DB), N_MAX(NWL_DB), Type_NAME(N_SCE), & ! A
       WEI(N_SCE,N_COMP), ECO(NWL_DB,N_COMP), SCO(NWL_DB,N_COMP) ) ! A
  ! Reading the names of the basic aerosol types
  DO I = 1, N_SCE
     Type_NAME(I) = CUT_STR(READ_ARR(I)%S)
  ENDDO 
 
  STR_INP  = 'Composition of aerosol types'
  CALL Read_Input (STR_INP, READ_ARR, *880)
  DO I = 1, N_SCE
     READ(READ_ARR(I + 1)%S,*,ERR = 101) (WEI(I,J), J=1,N_COMP)
  ENDDO
  
  IF (LOG_LEVEL(1) >= N_ESO) THEN
     WRITE(LUNIT,*) '   Type_NAME and weights:'
     DO I = 1,N_SCE
        WRITE(LUNIT,'(4x,I2,2x,a)') I, TRIM(Type_NAME(I))
        WRITE(LUNIT,'(6(2x,E12.4))') (WEI(I,J), J=1,N_COMP)
     ENDDO
  ENDIF
!===============================================================
! ------>   End of reading  WMO-model general input parameters
!===============================================================
  Select Case(AER_PARAM)
  Case('advanced')
     STR_INP  = 'Reference wavelength (WMO)'
     Call Read_Input (STR_INP, READ_ARR, *880)
     Read(READ_ARR(1)%S,*,ERR=101) WL_REF
  Case('default')
     WL_REF = 550.d0
  End Select

  !>>>**********************************************************
  !   Reading: 
  !               wavelength, 
  !               maximal number of phase momens for each component, 
  !               extinction and scattering coefficients for each
  !               optical aerosol components.
  !   Aerosol optical components are in the following order:
  !           1    water-soluble
  !           2    dust-like
  !           3    oceanic
  !           4    soot
  !           5    stratospheric
  !           6    vulkanic
  OPEN(NewUnit = DUNIT, FILE=TRIM(ADB)//'WMO_DB_WL_UV-TIR.dat',STATUS='OLD')
  DO I = 1, NWL_DB 
     READ(DUNIT,*) DB_WAVS(I), N_MAX(I), (ECO(I,Jc), SCO(I,Jc), Jc = 1,N_COMP)
  ENDDO
  CLOSE(DUNIT)

  WL_S = Min(START_WAV,WL_REF,Rwl_aer) 
  WL_E = Max(END_WAV,WL_REF,Rwl_aer) 
  !========= End of reading the file "WMO_DB_WL_UV-TIR.dat" ========
  IF( DB_WAVS(NWL_DB) < WL_E ) GOTO 130
     
!------------------------------------------------------------------
!   Find position of start- and end-wavelengths of current net in
!   wavelength array of aerosol data
!------------------------------------------------------------------
  IF( WL_S < DB_WAVS(1) ) THEN
     NWLS = 1 
     CALL WRITE_MESSAGE(R_NAME,'WARNING',' ')
     WRITE(EU,'(A)')  '    Required wavelength range is outside of the'//&
          ' aerosol data base.'
     WRITE(EU,'(A,2x,F9.2)') '    First wavelength in data base:', DB_WAVS(1)
     WRITE(EU,'(A,2x,F9.2)') '    First required wavelength    :', WL_S
     WRITE(EU,*) '  !!! The extrapolation with constant is used !!!'
  ELSE
     CALL LIN_SEARCH(WL_S, DUM, NWLS, NWL_DB, DB_WAVS, ERROR)
     IF (ERROR) GOTO 126
  ENDIF

  CALL LIN_SEARCH(WL_E, DUM, NWLE, NWL_DB, DB_WAVS, ERROR)
  IF (ERROR) GOTO 126
  NWLE = NWLE + 1
  IF ( DB_WAVS(NWLE) == WL_E ) NWLE = NWLE + 1

  ! Number of wavelength in the selected part of aerosol data bank
  NAW = NWLE - NWLS + 1
  ALLOCATE( AER_WAVS(NAW), FDM_AER(NLAYERS,NAW), IDA(NLAYERS), & !A
       AER_COEFF(3,NLAYERS,NAW), PSCA(NLAYERS), TAU(NAW) ) !A

  AER_WAVS(:) = DB_WAVS(NWLS:NWLE)
  FDM_AER = 0.d0 ; AER_COEFF = 0.d0 

  !=========================================================
  !   Read user-defined aerosol scenario input parameters
  !=========================================================
  SELECT CASE( TRIM(AER_PARAM) )
  CASE('advanced')
     STR_INP  = 'WMO aerosol layers'
     CALL Read_Input (STR_INP, READ_ARR, *880, ReArrange = .TRUE.)
     LAN = SIZE(READ_ARR)

     ALLOCATE( LAS(LAN), LAE(LAN), L_TOP(LAN), D1(NAW), Type_INP(LAN), & !A
          IDX_W(LAN), ASYM_PAR(LAN,NAW), APH_VS( NLEGEN,LAN,Nm_vs,NAW ), & !A
          CEXT(LAN,NWL_DB), CSCA(LAN,NWL_DB) )          !A
     APH_VS = 0.0d0  
     AER_PHASMOMS => APH_VS( :,:,1,: )

     DO J = 1, LAN
        READ(READ_ARR(J)%S,*,ERR=101) L_TOP(J)
     END DO

     IF( AER_SHIFT ) L_TOP(:) = L_TOP(:) + GROUND_HEIGHT

     IF( LAN > 1 ) THEN
        IF( L_Top(LAN-1) > ZHOLD(1) ) GOTO 131
     ELSE
        IF( L_Top(LAN) > ZHOLD(1) ) GOTO 133
     ENDIF
     IF( L_Top(1) <= ZHOLD(NLAYERS) ) GOTO 132

     STR_INP  = 'WMO aerosol type'
     CALL Read_Input (STR_INP, READ_ARR, *880, ReArrange = .TRUE.)
     ! Type_INP(:) bottom --> top
     DO J = 1, LAN
        Type_INP(J) = CUT_STR(READ_ARR(J)%S)
!        WRITE(*,*) '  layer ', J, Type_INP(J)
     END DO

     STR_INP  = 'Profile of extinction coefficient (WMO)'
     CALL Read_Input (STR_INP, READ_ARR, *880)
     N_Z_EXT = SIZE(READ_ARR)

     ALLOCATE(EXT_COEFF(N_Z_EXT), Z_EXT(N_Z_EXT), PROF(NLAYERS) )
     DO I = 1,N_Z_EXT
        READ(READ_ARR(I)%S,*) Z_EXT(I), EXT_COEFF(I)
     END DO
     
     ! Test version 
     ! WRITE(*,*) '   **************************************    '
     ! WRITE(*,*) '      Message from "wmo_aer_setup.f"         '
     ! WRITE(*,*) '    !!! Density is increased by 0.1% !!!     '
     ! WRITE(*,*) '   **************************************    '
     ! EXT_COEFF(:) = EXT_COEFF(:)*(1.d0 + 5.5d0)
     ! PAUSE

  CASE('default')
     LAN = 4
     ALLOCATE( LAS(LAN), LAE(LAN), L_TOP(LAN), D1(NAW), Type_INP(LAN), & !A
          IDX_W(LAN), ASYM_PAR(LAN,NAW), APH_VS( NLEGEN,LAN,Nm_vs,NAW ), & !A
          CEXT(LAN,NWL_DB), CSCA(LAN,NWL_DB) ) !A
     APH_VS = 0.d0  
     AER_PHASMOMS => APH_VS( :,:,1,: )

     ! Default tops of aerosol layers
     L_TOP(1) = 2.d0  ; L_TOP(2) = 10.d0
     L_TOP(3) = 30.d0 ; L_TOP(4) = 60.d0
     IF( LAN > 1 ) THEN
        IF( L_Top(LAN-1) > ZHOLD(1) ) GOTO 131
     ELSE
        IF( L_Top(LAN) > ZHOLD(1) ) GOTO 133
     ENDIF
     IF( L_Top(1) <= ZHOLD(NLAYERS) ) GOTO 132

     ! Default optical aerosol models
     Type_INP(1) = 'maritime'   ; Type_INP(2) = 'continental'
     Type_INP(3) = 'background' ; Type_INP(4) = 'background'

     N_Z_EXT = 12
     ALLOCATE(EXT_COEFF(N_Z_EXT), Z_EXT(N_Z_EXT), PROF(NLAYERS) )
     Z_EXT(1)  =  0.0  ;  EXT_COEFF(1)  = 0.025   
     Z_EXT(2)  =  2.0  ;  EXT_COEFF(2)  = 0.025   
     Z_EXT(3)  =  3.0  ;  EXT_COEFF(3)  = 0.0025  
     Z_EXT(4)  =  12.0 ;  EXT_COEFF(4)  = 0.0025  
     Z_EXT(5)  =  13.0 ;  EXT_COEFF(5)  = 2.18d-04
     Z_EXT(6)  =  20.0 ;  EXT_COEFF(6)  = 2.18d-04
     Z_EXT(7)  =  30.0 ;  EXT_COEFF(7)  = 3.32d-05
     Z_EXT(8)  =  35.0 ;  EXT_COEFF(8)  = 2.45d-05
     Z_EXT(9)  =  40.0 ;  EXT_COEFF(9)  = 8.00d-06
     Z_EXT(10) =  45.0 ;  EXT_COEFF(10) = 4.02d-06
     Z_EXT(11) =  50.0 ;  EXT_COEFF(11) = 2.10d-06
     Z_EXT(12) =  60.0 ;  EXT_COEFF(12) = 1.60d-07
  END SELECT
  
  IF( AER_SHIFT ) Z_EXT(:) = Z_EXT(:) + GROUND_HEIGHT

  !====================================================================
  !   End of reading the user-defined aerosol scenario input parameters
  !====================================================================

  ! Find position of reference wavelength in wavelength array of aerosol data
  I_REF = MINLOC( DABS(DB_WAVS(:)-WL_REF),1 )
  EXT_COEFF(:) = AER_SCALING*EXT_COEFF(:)
  ! Interpolate ext. coeff. at the standard altitude grid
  CALL LIN_INT(N_Z_EXT, Z_EXT, EXT_COEFF, NLAYERS, ZHOLD, PROF, 'Z')
  
  DO I = 1,LAN
     ! Ordered from the bottom to the top 
     LAS(I) = MINLOC( DABS(L_Top(I)-ZHOLD(:)),1 )
  ENDDO
  LAE(1) = NLAYERS
  DO I = 2,LAN
     LAE(I) = LAS(I-1)-1
  ENDDO

  ! Indexes of altitude grid levels which are within aerosol layers
  IDA(:) = LAN
  DO I = 1,LAN - 1
     DD1 = ZHOLD(LAS(I)) ;  DD2 = ZHOLD(LAE(I))
     DO J = 1,NLAYERS
        IF( ZHOLD(J) <= DD1.AND.ZHOLD(J) >= DD2 )  IDA(J) = I 
     ENDDO
  ENDDO

  DD1 = ZHOLD(MAXVAL(LAE))
  ! Indexes of altitude grid levels which are below of lowest aerosol layer
  DO J = 1,NLAYERS
     IF( ZHOLD(J).LE.DD1 ) IDA(J) = 1
  ENDDO
  
  IF (LOG_LEVEL(1) >= N_ESO) THEN
     WRITE(LUNIT,*) ' ------ Test of aerosol layering WMO mode --- '
     DO I = 1,LAN
        WRITE(LUNIT,'(2x,I3,2(1x,f7.3),2x,a)') &
             I, ZHOLD(LAS(I)), ZHOLD(LAE(I)), TRIM( Type_INP(I))
     ENDDO
  ENDIF

  IDX_W = 0
  DO J = 1,LAN
     DO I = 1, N_SCE
        IF( TRIM(Type_INP(J)) == TRIM(Type_NAME(I)) ) THEN 
           IDX_W(J) = I
        ENDIF
     ENDDO
     IF( IDX_W(J) == 0 ) GOTO 127
     ! Mixing of ext. and scat. coeff. of basic aerosol optical 
     ! components in all layers and all wavelengths
     DO I = 1, NWL_DB
        CEXT(J,I) = SUM( WEI(IDX_W(J),:)*ECO(I,:) )
        CSCA(J,I) = SUM( WEI(IDX_W(J),:)*SCO(I,:) )
     ENDDO
  ENDDO  ! over aerosol layers (J)

  ! Find scaling profile for the aerosol extinction coefficients
  DO J = 1,LAN
     DO K1 = LAS(J), LAE(J)
        PSCA(K1) = PROF(K1)/CEXT(J,I_REF)
     ENDDO
  ENDDO

  ! Scaling of aerosol number density profile
  IF( L_scal ) THEN
     WHERE( ZHOLD(:) >= ZS1.AND.ZHOLD(:) <= ZS2 )  PSCA(:) = AER_FACT*PSCA(:)
  ENDIF

  DO J = 1,LAN
     DO K1 = LAS(J), LAE(J)
        I1 = 1
        DO K = NWLS, NWLE
           AER_COEFF(EXI,K1,I1) = CEXT(J,K)*PSCA(K1)
           AER_COEFF(SCI,K1,I1) = CSCA(J,K)*PSCA(K1)
           I1 = I1 + 1
        ENDDO
     ENDDO
  ENDDO

  ! Scaling of the scattering coefficient between ZS1 - ZS2
  ! in order to change the single scattering albedo as follows:
  !    SSA' = 1 - V_PAP*(1 - SSA) 
  ! Extinction coefficient remains the same
  IF( L_scal ) THEN
     DO I = 1, NAW
        WHERE( ZHOLD(:) >= ZS1.AND.ZHOLD(:) <= ZS2 ) &
             AER_COEFF(SCI,:,I) = AER_COEFF(SCI,:,I) - (V_PAP - 1.d0) &
             *(AER_COEFF(EXI,:,I ) - AER_COEFF(SCI,:,I ))
     ENDDO
  ENDIF

!  WRITE(*,*) '   AER_COEFF(SCI,:,2 ) before DM NAW = ', NAW
!  DO I = 1, NLAYERS
!     WRITE(*,'(2x,I5,F8.4,2x,I3,2x,ES15.7)') I,ZHOLD(I),IDA(I), AER_COEFF(SCI,I,2)
!  ENDDO

  !***************************************************************
  !      Reading of the scattering matrixes of WMO data bank          
  !***************************************************************
  MAX_MOM = N_MAX(NWLS)

  IF (LOG_LEVEL(1) >= N_ESO) THEN
     WRITE(LUNIT,*) '   Max number of expansion coeff. :', MAX_MOM
     WRITE(LUNIT,*) '  ----------------------------------'
  ENDIF
  ! The maximal number of expansion coefficients can be smaller than NLEGEN. 
  N_DIM = MAX0(MAX_MOM,NLEGEN)

  ! Temporary arrays for the expansion coefficients
  ALLOCATE( TMP_MOM(NAW,N_DIM,6*N_COMP), USER_MOM(NAW,N_DIM,LAN,Nm_vs) ) ! A
  USER_MOM   = 0.d0 ; TMP_MOM = 0.d0

  N_SKIP = SUM( N_MAX(1:NWLS-1) )
  OPEN(NewUnit=DUNIT,FILE=TRIM(ADB)//'WMO_DB_MOM_UV-TIR.dat',STATUS='OLD')
  DO I = 1, N_SKIP  ;   READ(DUNIT,*)  ;   ENDDO
  I1 = 1
  DO I = NWLS, NWLE
     READ(DUNIT,*) K_NUM, (TMP_MOM(I1,1,J), J=1,6*N_COMP)
     IF( K_NUM .NE. 0 ) THEN
        WRITE(LUNIT,*) I, I1, K_NUM ; GOTO 128
     ENDIF
     DO K = 2, N_MAX(I)
        READ(DUNIT,*) K_NUM, (TMP_MOM(I1,K,J), J=1,6*N_COMP)
     ENDDO
     I1 = I1 + 1
  ENDDO
  CLOSE(DUNIT)
! Calculate the weighted expansion coefficients according to the basic aerosol type
  IF( APH_TYPE == 'mie' ) THEN            
     DO J = 1,LAN
        I1 = 1 ; Nwe = IDX_W(J)
        DO I = NWLS, NWLE
           DO Jm = 1, MAX_MOM
              DO Jk = 1, Nm_vs
                 USER_MOM(I1,Jm,J,Jk) = SUM( TMP_MOM(I1,Jm,Jk:6*N_COMP:6) &
                      *WEI(Nwe,:)*SCO(I,:) )/CSCA(J,I)
              ENDDO         ! End for 
           ENDDO            ! End for expansion coefficients
           ASYM_PAR(J,I1) = USER_MOM(I1,2,J,1)/3.d0
           I1 = I1 + 1
        ENDDO  ! over wavelength
     ENDDO  ! over aerosol layers
     IF( Nm_vs > 1 ) THEN
        USER_MOM(:,:,:,5) = -USER_MOM(:,:,:,5)
        USER_MOM(:,:,:,6) = -USER_MOM(:,:,:,6)
     ENDIF
  ENDIF

  IF( APH_TYPE == 'h-g' ) THEN            
     DO J = 1,LAN
        I1 = 1 ; Nwe = IDX_W(J)
        DO I = NWLS, NWLE
           D1(I1) = SUM( TMP_MOM(I1,2,1:6*N_COMP:6)*WEI(Nwe,:)*SCO(I,:) )/CSCA(J,I)
           I1 = I1 + 1
        ENDDO
        D1(:) = D1(:)/3.d0  ;  ASYM_PAR(J,:) = D1(:)
        DO Jm = 1, MAX_MOM
           USER_MOM(:,Jm,J,1) = D1(:)**(Jm-1)*DBLE(2*Jm-1)
        ENDDO
     ENDDO
  ENDIF

!  WRITE(*,*) '  DO_SSRCA and  TRIM(RTM_TYPE)   ', DO_SSRCA, TRIM(RTM_TYPE)

  ! Single scattering correction algorithm 
  ! --------------------------------------
  IF( DO_SSRCA.OR.TRIM(RTM_TYPE) == 'spher_scat'.OR.SCATT_MODE =='ss') THEN
     Nga_db = MAX_MOM 
     ALLOCATE( Greek_aer( Nga_db,LAN,Nm_vs,NAW ) )
     Greek_aer = 0.d0
     DO I = 1,NAW 
        DO K = 1,Nm_vs
           DO L = 1, LAN
              Greek_aer(:,L,K,I) = USER_MOM(I,:,L,K)
           ENDDO
        ENDDO
     ENDDO
     IF( .NOT.Plus_mi ) NG_aer = Nga_db
  ENDIF

  IF( OUT_A_SCM ) THEN 
     IF( LAY_A > LAN ) GOTO 135
     IF( WEL_A > NAW )  GOTO 136
     CALL COEF_TO_SCATMATR(MAX_MOM, Nm_vs, &
          USER_MOM(WEL_A,1:MAX_MOM,LAY_A,1:Nm_vs), NSCA_A, GE_A, PATH_TO_SCM)
!     OPEN(45,FILE='../data/aerosol/Phase_coef_L3.out')
!       WRITE(45,'(a)')  &
!            '#  Scattering function of given expansion coefficients '
!       WRITE(45,'(a)') '#  Column :'
!       WRITE(45,'(a)') '#  1 - Number'
!       WRITE(45,'(a)') '#  2 - Expamsion coefficients'
!       DO I = 1, MAX_MOM
!          WRITE(45,'(2x,I4,2x,E15.7)') I, USER_MOM(WEL_A,I,LAY_A,1)
!       ENDDO
!     CLOSE(45)
!    STOP '    Stop in "wmo_aer_setup.f90" '
  ENDIF

  DO I = 1,LAN 
     DO J = LAS(I),LAE(I)
        AER_COEFF(ASY,J,:) = ASYM_PAR(I,:)
     ENDDO
  ENDDO

  ! Aerosol total optical thickness
  DO K = 1,NAW
     TAU(K) = SUM( AER_COEFF(EXI,:,K)*DZ(:) )/2.d0
  ENDDO

  IF( Scal_aer ) THEN
     CALL LIN_SEARCH(Rwl_aer, DUM, K, NAW, AER_WAVS, ERROR)
     IF (ERROR) GOTO 129
     DD1 = (1.d0 - DUM)*TAU(K) + DUM*TAU(K+1)
     IF( DD1 > 0.d0 ) THEN
        Scoe = Rtau_aer/DD1
     ELSE
        Scoe = 0.d0
     ENDIF
     AER_COEFF(EXI,:,:) = Scoe*AER_COEFF(EXI,:,:)
     AER_COEFF(SCI,:,:) = Scoe*AER_COEFF(SCI,:,:)
     TAU(:) = Scoe*TAU(:)
  ENDIF
  AOT_out_VAL = TAU(1)

  ! WRITE(*,*) '   =====> Inp AOT at reference wl '
  ! DO K = 1,NAW
  !    WRITE(*,'(2x,I2,2x,F8.3,2x,E12.4)') K, AER_WAVS(K),&
  !        SUM( AER_COEFF(EXI,:,K)*DZ(:) )/2.d0
  ! ENDDO
       
  !======================================
  !   Write aerosol scenario output file
  !======================================
  CALL WRITE_SCEN_WMO(*880)

!  WRITE(*,*) '      ------ WMO_AER_SETUP ----- '
!  DO I = 1, 20
!     WRITE(*,'(2x,I3,*(2x,e12.4))')  &
!          I, USER_MOM(2,I,1,1), USER_MOM(2,I,1,5), USER_MOM(2,I,1,6)
!  ENDDO

  !Allocate array for MODULE DELTA_M
  !=================================
!  WRITE(*,*) '     AER_DM_FLG  ', AER_DM_FLG, AER_TRUNC_IDX
!  write(*,*) '  a: AER_COEFF(EXI,NZ,:) ', AER_COEFF(EXI,NLAYERS,:)
!  write(*,*) '  a: AER_COEFF(SCI,NZ,:) ', AER_COEFF(SCI,NLAYERS,:)

  IF ( AER_DM_FLG ) THEN 
     Itr = AER_TRUNC_IDX
     SELECT CASE( Vec_Sca )
     CASE('vect_rte')
        CALL AER_DM_VEC
     CASE('scal_rte')
        SELECT CASE ( TRIM(D_MF_AER) )
        CASE( 'D-FA' )
           Nleg_df = MAX_MOM 
           ALLOCATE(PHAS_LOC( Nleg_df), PL_RES( Nleg_df,N_ANG )) ! A 
           Nleg_Loc = AER_TRUNC_IDX + 1
           ALLOCATE( R_COF( Nleg_Loc ))
           CALL Start_Delta_Fit( N_ANG, Nleg_df )
           DO I1 = 1,LAN
              ! WRITE(*,*) '  I1 = ',I1
              PHAS_LOC(1:Nleg_df) = USER_MOM(1,1:Nleg_df,I1,1)
              CALL Delta_Fit( N_ANG, Nleg_df, PHAS_LOC )
              DO K1 = LAS(I1), LAE(I1)
                 DO K = 1, NAW
                    FDM_AER(K1,K) = F_Dfit_loc
                    AER_PHASMOMS(1:Nleg_loc,I1,K) = & ! Proverit' 
                         R_cof(1:Nleg_loc)/R_cof(1)
                 ENDDO
              ENDDO
           ENDDO
        CASE( 'D-MA' )
           CALL AER_DM
        END SELECT
     END SELECT
     !"Renormalize" scattering and extinction coefficients
     !====================================================
     DO I = 1, NLAYERS
        DO K = 1,NAW
           ! test for consistence between "mi" and "wmo_ni" modes
           IF( AER_COEFF(EXI,I,K) == 0.d0 ) FDM_AER(I,K) = 0.d0
           F_DM = FDM_AER(I,K)
           AER_ABS_COEFF = AER_COEFF(EXI,I,K ) - AER_COEFF(SCI,I,K)
           AER_COEFF(SCI,I,K) = AER_COEFF(SCI,I,K)*(1.d0-F_DM)
           AER_COEFF(EXI,I,K) = AER_COEFF(SCI,I,K) + AER_ABS_COEFF 
        ENDDO
     ENDDO
  ELSE
     DO I1 = 1,LAN
        DO K = 1, NAW
           APH_VS(:,I1,:,K) = USER_MOM(K,1:NLEGEN,I1,:)
        ENDDO
     ENDDO
     ! DO I1 = 1,LAN
     !    DO K1 = LAS(I1), LAE(I1)
     !       DO K = 1, NAW
     !          APH_VS(:,K1,:,K) = USER_MOM(K,1:NLEGEN,I1,:)
     !       ENDDO
     !    ENDDO
     ! ENDDO
  ENDIF
!  write(*,*) '     After D-M option '
!  write(*,*) '  b: AER_COEFF(EXI,NZ,:) ', AER_COEFF(EXI,NLAYERS,:)
!  write(*,*) '  b: AER_COEFF(SCI,NZ,:) ', AER_COEFF(SCI,NLAYERS,:)
!  pause
  
  LA_Ndb = LAN
  IF( .NOT.Plus_mi ) THEN
     LA_N = LAN
     ALLOCATE( LA_S(LAN), LA_E(LAN), IDX_AER(NLAYERS) )
     LA_S = LAS ;  LA_E = LAE  ; IDX_AER = IDA
  ELSE
     ALLOCATE( IDX_AER_db(NLAYERS) )
     IDX_AER_db = IDA
  ENDIF
  
  CALL ST_TRACK (R_NAME, 1, 'O')
  RETURN

101 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Reading error')
  WRITE(EU,*) '    Parameter to read: ', STR_INP 
  RETURN 1

126 CALL WRITE_MESSAGE (R_NAME, 'ERROR', 'Error in LIN_SEARCH')
  RETURN 1
  
127 CALL WRITE_MESSAGE (R_NAME,'ERROR', 'Aerosol models name not found')
  RETURN 1
  
128 CALL WRITE_MESSAGE (R_NAME,'ERROR', 'Error reading of "WMO_DB_MOM_UV-TIR.dat"')
  RETURN 1

129 CALL WRITE_MESSAGE (R_NAME,'ERROR', 'Reference wavelength for AOT is out of range')
  WRITE(EU,'(A,F8.3)') '    Reference wavelength: ', Rwl_aer
  WRITE(EU,'(A,2(2x,F8.3))') '    Database wavelengths: ', AER_WAVS(1), AER_WAVS(NAW)
  WRITE(EU,'(A)') '    Set reference wavelength between database wavelengths.'
  RETURN 1

130 CALL WRITE_MESSAGE(R_NAME, 'ERROR', &
         'Required wavelength range is outside of the data base.')
  WRITE(EU,'(A,2x,F9.2)') '    Last wavelength in data base:', DB_WAVS(NWL_DB)
  WRITE(EU,'(A,2x,F9.2)') '    Last required wavelength    :', WL_E
  WRITE(EU,*) '    Aerosol data base does not exist '
  WRITE(EU,*) '    for wavelengths > 40 micron.'
  RETURN 1

131 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Upper aerosol layer outside the atmosphere')
  WRITE(EU,'(A,2x,F9.2,A,F9.2)') & 
       '    Upper aerosol layer:', L_TOP(LAN-1),' - ',L_TOP(LAN)
  WRITE(EU,'(A,2x,F9.2,A)') '      Top of atmosphere:', ZHOLD(1),' km'
  WRITE(EU,*) '    Check "wmo_aer.inp" and altitude grid '
  WRITE(EU,*) '    as defined in the file "control.inp" '
  RETURN 1

132 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Lower aerosol layer outside the atmosphere')
  WRITE(EU,'(A,2x,F6.2,A)') '       Top of aerosol layer:', L_TOP(1),' km'
  WRITE(EU,'(A,2x,F6.2,A)') '       Surface elevation   :', ZHOLD(NLAYERS),' km'
  WRITE(EU,*) '    Check "wmo_aer.inp" and "Height above '
  WRITE(EU,*) '    sea level" in the file "control.inp" '
  RETURN 1

133 CALL WRITE_MESSAGE(R_NAME, 'ERROR', ' Aerosol layer outside the atmosphere')
  WRITE(EU,'(A,2x,F9.2,A,F9.2)') '    Aerosol layer:', ZHOLD(NLAYERS),' - ',L_TOP(LAN)
  WRITE(EU,'(A,2x,F9.2,A)') '      Top of atmosphere:', ZHOLD(1),' km'
  WRITE(EU,*) '    Check "wmo_aer.inp" and altitude grid '
  WRITE(EU,*) '    as defined in the file "control.inp" '
  RETURN 1

135 CALL WRITE_MESSAGE (R_NAME,'ERROR', &
         'Required number of aerosol layer is out of range')
  WRITE(EU,'(A,I3)') '    Number of aerosol layers: ', LAN
  WRITE(EU,'(A,I3)') '    Required layer number   : ', LAY_A
  RETURN 1

136 CALL WRITE_MESSAGE (R_NAME,'ERROR',&
         'Required number of wavelength is out of range')
  WRITE(EU,'(A,I3)') '    Number of wavelengths for aerosol: ',NAW
  WRITE(EU,'(A,I3)') '    Required wavelength number       : ',WEL_A
  RETURN 1
  
880 WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)
  RETURN 1
  
CONTAINS
!***********************************************************************
  SUBROUTINE Start_Delta_Fit( N_ANG, Nleg_df )
    INTEGER           N_ang, Nleg_df
    ! Local variables
    REAL (KND)  Kat_ang
    REAL (KND), DIMENSION (N_ANG) ::  X_gau, AG_ang
    ! Cut-angle ]deg]
    Kat_ang = 2.0d0
    Kat_ang = DCOS( Kat_ang/RAD_TO_DEG )

    CALL GAULEG ( -1.D0, 1.D0, X_gau, AG_ang, N_ANG )

    ! Invert Gauss grid to start from zero scattering angle 
    X_ang(:) = X_gau(N_ang:1:-1)

    ! Legendre polynomials for all Gauss angles and all Max_mom
    CALL WIGNER_PLG( X_ang, N_ang, 0, Nleg_df, PL_RES )

    ! Calculate phase function for all Gauss nodes
    DO I = 1,N_ang
       IF( X_ang(I) < Kat_ang ) Then
          N_KAT = I
          EXIT
       ENDIF
    ENDDO
    IF (LOG_LEVEL(1) >= N_ESO) THEN
       WRITE(LUNIT,*) 
       WRITE(LUNIT,*) ' ***** Cut-angle for Delta-fit approximation **** '
       WRITE(LUNIT,'(2x,i3,2x,e12.4)') N_KAT,DACOS(X_ang(N_KAT))*RAD_TO_DEG
       WRITE(LUNIT,*) 
    END IF

  END SUBROUTINE Start_Delta_Fit

!***********************************************************************
  SUBROUTINE Delta_Fit( N_ANG, Nleg_df, Phas_moms )
    INTEGER           N_ang, Nleg_df
    REAL (KND)  Phas_moms(Nleg_df)
    ! Local variables
    INTEGER           I, K
    REAL (KND), DIMENSION (N_ANG) :: Phm1, Phm1_RD
    REAL (KND)  A_VEC( (Nleg_loc + 1)*(Nleg_loc + 2)/2 ), PL_Nor( Nleg_loc,N_ANG ),&
         PL_Nor1( Nleg_loc,N_ANG ), A_M( Nleg_loc,Nleg_loc )
    
    DO I = 1, N_ANG
       Phm1(I) = SUM( PL_RES(:,I)*Phas_moms(:) )
    ENDDO
    
    PL_NOR1 = 0.0d0
    DO K = 1,N_ANG
       PL_NOR(:,K) = PL_RES(1:Nleg_loc,K)/Phm1(K)
    ENDDO
    PL_NOR1(:,N_KAT:N_ang) = PL_NOR(:,N_KAT:N_ang)
    A_M(:,:) = MATMUL( PL_NOR(:,:),TRANSPOSE(PL_NOR1(:,:)) )
    
    FORALL (I = 1:Nleg_loc) A_VEC(I*(I - 1)/2 + 1:I*(I + 1)/2) = A_M(I,1:I)
    CALL CHINV3 (Nleg_loc*(Nleg_loc+1)/2, Nleg_loc, A_VEC)
    
    FORALL (I = 1:Nleg_loc) A_M(I,1:I) = A_VEC(I*(I - 1)/2 +1:I*(I + 1)/2)
    FORALL (I = 1:Nleg_loc, J = 1:Nleg_loc, J > I) A_M(I,J) = A_M(J,I)
    R_COF(:) = MATMUL( A_M,SUM( PL_NOR1(:,:),2 ) )
    F_Dfit_loc   = 1.0d0 - R_COF(1)
    !      write( *,* ) ' ---> R_COF(1) = ', R_COF(1)

    DO I = 1, N_ANG
       Phm1_RD(I) = SUM( PL_RES(1:Nleg_loc,I)*R_COF(1:Nleg_loc) )
    ENDDO

    OPEN( NewUnit = DUNIT, file = DIR_OUT//'phase.test' )
    DO I = 1, N_ANG
       WRITE( DUNIT,'(5(2x,e15.7))') X_ANG(I), DACOS(X_ANG(I))/3.14159*180., & 
            Phm1(I), Phm1_RD(I)
    ENDDO
    CLOSE(DUNIT)

  END SUBROUTINE Delta_Fit

!***********************************************************************
  SUBROUTINE AER_DM
    DO I1 = 1,LAN
       DO K = 1, NAW
          F_DM = USER_MOM(K,Itr+1,I1,1)/DBLE(2*Itr + 1)
          FDM_AER(LAS(I1):LAE(I1),K) = F_DM
          DO L = 1, Itr
             AER_PHASMOMS(L,I1,K) = ( USER_MOM(K,L,I1,1) - &
                  F_DM*DBLE(2*L-1) )/(1.d0 - F_DM)
          ENDDO
       ENDDO
    ENDDO
    AER_PHASMOMS(Itr+1:NLEGEN,:,:) = 0.0D0
  END SUBROUTINE AER_DM
  
!***********************************************************************
  SUBROUTINE AER_DM_VEC
    DO I1 = 1,LAN
       DO K = 1, NAW
          F_DM = USER_MOM(K,Itr+1,I1,1)/DBLE(2*Itr + 1)
          FDM_AER(LAS(I1):LAE(I1),K) = F_DM
          DO L = 1, Itr
             APH_VS(L,I1,1:4,K) = ( USER_MOM(K,L,I1,1:4) - &
                  F_DM*DBLE(2*L-1) )/(1.0D0 - F_DM)
             APH_VS(L,I1,5:6,K) = USER_MOM(K,L,I1,5:6)/(1.0D0 - F_DM)
          ENDDO
       ENDDO
    ENDDO
    APH_VS(Itr+1:NLEGEN,:,:,:) = 0.0D0
    
  END SUBROUTINE AER_DM_VEC

!***********************************************************************
  SUBROUTINE  WRITE_SCEN_WMO(*)
    USE  AEROSOLS, ONLY: OWL_AER, NOWL_AER, Alph, C_a
    USE TOOLS,     ONLY: OpenOut, WriteOut, CloseOut 
 
    ! M_H               : number of humidities 
    ! M_CH              : number of aerosol components depending on humidity
    ! M_CI              : number of aerosol components independent on humidity
    INTEGER             K1, K2, J, M_CI, M_H, M_CH, Unt
    PARAMETER           (M_CI = 6, M_H = 0, M_CH = 0)
    CHARACTER(LEN=3)    PHT
    CHARACTER(LEN=4)    PRF_TYPE
    Character (Len=500)  OutStr
    PARAMETER           (PRF_TYPE = 'exti')
    CHARACTER(LEN=17)   DB_COMP_TXT(M_H*M_CH+M_CI)
    DATA DB_COMP_TXT / 'water-soluble    ', 'dust-like        ', 'oceanic          ', &
         'soot             ',  'stratospheric    ', 'vulkanic         '/
    REAL (KND)  SS_A, Ext, S1
    REAL (KND), ALLOCATABLE :: T_OUT(:), E_OUT(:), S_OUT(:), T_BL(:), T_UP(:), P_EXT(:,:)
!***********************************************************************
    
    IF (LOG_LEVEL(1) >= N_SSO) WRITE (lunit,*) ' *** writing: AEROSOL.OUT'

    ! Optical thickness at user defined wavelength grid
    IF( NOWL_AER > 0 ) THEN 
       K1 = LAS(1) ; K2 = LAE(1)

       ALLOCATE( T_OUT(NOWL_AER),E_OUT(NOWL_AER),S_OUT(NOWL_AER),&
            T_BL(NOWL_AER), T_UP(NOWL_AER), P_EXT(NLAYERS,2))
       DO I = 1, NOWL_AER
          CALL LIN_SEARCH(OWL_AER(I), DUM, K, NAW, AER_WAVS, ERROR)
          IF (ERROR) GOTO 134
          T_OUT(I) = (1.d0 - DUM)*TAU(K) + DUM*TAU(K+1)
          
          ! Single scattering albedo at user-defined wavelengths
          S_OUT(I) = (1.d0 - DUM)*AER_COEFF(SCI,NLAYERS-1,K) &
               + DUM*AER_COEFF(SCI,NLAYERS-1,K+1)

          ! Extinction coefficient at user-defined wavelengths
          P_EXT(:,1:2) = AER_COEFF(EXI,:,K:K+1)
          E_OUT(I) = (1.d0 - DUM)*P_EXT(NLAYERS-1,1) + DUM*P_EXT(NLAYERS-1,2)

          ! Optical thickness of the boundary layer
          S1 = ( (1.d0-DUM)*P_EXT(K1,1) + DUM*P_EXT(K1,2) ) &
               *(ZHOLD(K1) - ZHOLD(K1+1)) + ( (1.d0-DUM)*P_EXT(K2,1) &
               + DUM*P_EXT(K2,2) )*(ZHOLD(K2-1) - ZHOLD(K2))
          DO J = K1+1,K2-1
             S1 = S1 + ( (1.d0-DUM)*P_EXT(J,1) + DUM*P_EXT(J,2) ) &
                  *(ZHOLD(J-1) - ZHOLD(J+1))
          ENDDO
          T_BL(I) = S1/2.d0

          ! Optical thickness of the atmosphere above boundary layer
          S1 = ( (1.d0-DUM)*P_EXT(1,1) + DUM*P_EXT(1,2) )*(ZHOLD(1) - ZHOLD(2)) &
               + ( (1.d0-DUM)*P_EXT(K1,1) + DUM*P_EXT(K1,2) ) &
               *(ZHOLD(K1-1) - ZHOLD(K1))
          DO J = 2,K1-1
             S1 = S1 + ( (1.d0-DUM)*P_EXT(J,1) + DUM*P_EXT(J,2) ) &
                  *(ZHOLD(J-1) - ZHOLD(J+1))
          ENDDO
          T_UP(I) = S1/2.d0
       ENDDO

       OPEN(NewUnit=DUNIT,FILE = DIR_OUT//'AOT.OUT')            
       WRITE(DUNIT,*) '# AOT and single scattering albedo (boundary layer) '
       WRITE(DUNIT,*) '#        at user-defined wavelengths [nm]:'
       DO I = 1, NOWL_AER
          WRITE(DUNIT,'(2x,F8.3,4(2x,E15.7))') OWL_AER(I),&
               T_BL(I), T_UP(I), T_OUT(I), S_OUT(I)/E_OUT(I)
       ENDDO
       CLOSE(DUNIT)

       ! Angstrom parameters for this aerosol model
       ! Alph = DLOG( T_OUT(1)/T_OUT(NOWL_AER) )&
       !      /DLOG( OWL_AER(1)/OWL_AER(NOWL_AER) )
       ! C_a = T_OUT(1)/OWL_AER(1)**Alph
       Alph = DLOG( T_BL(1)/T_BL(NOWL_AER) )/DLOG( OWL_AER(1)/OWL_AER(NOWL_AER) )
       C_a = T_BL(1)/OWL_AER(1)**Alph

!         WRITE(*,*) '  Angstrom parameters '
!         WRITE(*,*) '  C_a  = ', C_a
!         WRITE(*,*) '  Alph = ', Alph
    ENDIF

    Call OpenOut ('SCE_AEROSOL.OUT', Unt)
    WRITE(OutStr,*) '        SETTINGS FOR WMO AEROSOL MODEL  '
    Call WriteOut (Unt, OutStr)
    WRITE(OutStr,*) '     ==================================='
    Call WriteOut (Unt, OutStr)
    WRITE(OutStr,*) ' '
    Call WriteOut (Unt, OutStr)
  
    IF( Scal_aer ) THEN
       WRITE(OutStr,'(A,F7.3,A,F9.2,A)') &
       '   !!! ATTENTION !!! AOT is set to ',Rtau_aer,' at', Rwl_aer,' nm wavelength.'
       Call WriteOut (Unt, OutStr)
       WRITE(OutStr,'(19x,A)') '  See setting in "control_aer.inp" '
       Call WriteOut (Unt, OutStr)
       WRITE(OutStr,'(6x,A)')  '----------------------------------------------------'
       Call WriteOut (Unt, OutStr)
       WRITE(OutStr,*) 
       Call WriteOut (Unt, OutStr)
    ENDIF

    IF( NOWL_AER > 0 ) THEN
       WRITE ( OutStr, * ) ' AOT at user-defined wavelengths [nm]:'
       Call WriteOut (Unt, OutStr)
       DO I = 1, NOWL_AER
          WRITE(OutStr,'(12x,F7.2,4x,F8.4)') OWL_AER(I),T_OUT(I)
          Call WriteOut (Unt, OutStr)
       ENDDO
    ENDIF
    Call WriteOut (Unt, ' ')

    WRITE(OutStr,'(A27,2X,I2)') '  Number of aerosol layers:', LAN
    Call WriteOut (Unt, OutStr)
    Call WriteOut (Unt, '  ')
    DO I = 1, LAN
       WRITE(OutStr,'(A27,2X,I2)') '  Layer :                 :', I
       Call WriteOut (Unt, OutStr)
       WRITE(OutStr,'(A27,2X,F6.2)') '  Upper boundary [km]     :', L_TOP(I)
       Call WriteOut (Unt, OutStr)
       WRITE(OutStr,'(A27,2X,A30)') '  Aerosol type            :', TRIM( Type_INP(I))
       Call WriteOut (Unt, OutStr)
       WRITE(OutStr,'(29X,A20,A15)') '  Component           ', 'Mix.ratio'
       Call WriteOut (Unt, OutStr)
       DO J = 1,M_CI
          IF( WEI(IDX_W(I),J) == 0.0d0 ) CYCLE
          WRITE(OutStr,'(29X,A20,E15.5)') DB_COMP_TXT(J), WEI(IDX_W(I),J)
          Call WriteOut (Unt, OutStr)
       ENDDO
       Call WriteOut (Unt, '')
    ENDDO
          
    SELECT CASE( PRF_TYPE )
    CASE ('exti')
       WRITE(OutStr,'(A43,F9.2,A4)') &
            'Profile type: extinction coeff.  [km-1] at ', WL_REF, ' nm.'
       Call WriteOut (Unt, OutStr)
    CASE ('part')
       WRITE(OutStr,'(A47)') 'Profile type: particle number density [1/cm^3].'
       Call WriteOut (Unt, OutStr)
    END SELECT
    
    Call WriteOut (Unt, ' ')
    Call WriteOut (Unt,' Vertical profiles of aerosol')
    Call WriteOut (Unt,'         extinction coefficient (Ext coeff) and')
    Call WriteOut (Unt,'         single scattering albedo (Ss albedo):')
    IF(APH_TYPE == 'mie') THEN
       PHT = 'Mie'
    ELSE
       PHT = 'H-G'
    ENDIF

    DO K = 1,NAW
       Call WriteOut (Unt,'   ')
       WRITE(OutStr,'(1x,a11,F9.2,a6,a6,F8.4)') &
            'Wavelength ',AER_WAVS(K),' [nm];',' AOT =',TAU(K)
       Call WriteOut (Unt, OutStr)
       Call WriteOut (Unt,' ----------------------------------------------')
       Call WriteOut (Unt, '     Number, Alt [km], Ext coeff [km-1], Ss albedo')
       DO I = 1, NLAYERS
          Ext = AER_COEFF(EXI,I,K)
          IF( Ext == 0.0d0 ) CYCLE
          SS_A = AER_COEFF(SCI,I,K)/Ext
          WRITE(OutStr,'(6x,I3,4x,F6.2,5x,ES9.2,6x,E12.4)') I, ZHOLD(I), Ext, SS_A
          Call WriteOut (Unt, OutStr)
       END DO
       Call WriteOut (Unt, ' ')
       Call WriteOut (Unt,'    Asymmetry factor used in the aerosol layers :')
       Call WriteOut (Unt,'    ------------------------------------------- :')
       Call WriteOut (Unt,'    Layer  Top [km]  Bottom [km]   Asymmetry    Phase')
       Call WriteOut (Unt,'                                    factor     function')
       DO I = 1,LAN
          WRITE(OutStr,'(6x,I3,3x,F6.2,4x,F6.2,9x,F6.3,7x,a3)') &
               I, ZHOLD(LAS(I)) ,ZHOLD(LAE(I)), Asym_Par(I,K), PHT
          Call WriteOut (Unt, OutStr)
       ENDDO
    ENDDO                   ! Over wavelengths
          
    Call CloseOut (Unt)
    RETURN

134 CALL WRITE_MESSAGE (R_NAME,'ERROR', 'Output wavelength for AOT is out of range')
    WRITE(EU,'(A,F7.2)') '    Output wavelength: ', OWL_AER(I)
    WRITE(EU,'(A,2(2x,F7.2))') '    Database wavelengths: ', AER_WAVS(1), AER_WAVS(NAW)
    WRITE(EU,'(A)') '    Set output wavelength between database wavelengths'
    WRITE(EU,'(A)') '    in the "control_aer.inp" file.'
    RETURN 1

  END SUBROUTINE  WRITE_SCEN_WMO

!***********************************************************************
  SUBROUTINE DEALLOCATE_ARRAYS
    
    IF(ALLOCATED(DENS_PRF)) DEALLOCATE(DENS_PRF)
    IF(ALLOCATED( AER_WAVS)) DEALLOCATE(AER_WAVS, AER_COEFF, FDM_AER)
    IF(ALLOCATED(L_TOP)) DEALLOCATE(L_TOP, APH_VS)
    IF(ALLOCATED(Greek_aer)) DEALLOCATE(Greek_aer)
    IF(ALLOCATED(LA_S)) DEALLOCATE(LA_S, LA_E, IDX_AER)
    IF(ALLOCATED(IDX_AER_db)) DEALLOCATE(IDX_AER_db)
    
    RETURN
  END SUBROUTINE DEALLOCATE_ARRAYS
!***********************************************************************
END SUBROUTINE WMO_AER_SETUP
!***********************************************************************

